//=========================================================
//
// AI
//
//=========================================================

#ifndef _AI_
#define _AI_

#include <map>
#include <list>

#include "Rule.h"
#include "Card.h"
#include "Move.h"
#include "Position.h"
#include "../Utils/Allocator.h"
#include "NotXML.h"

#include <pthread.h>
#include <thread>


#include <Windows.h>

using namespace std;

namespace AI_Modules {

	class InnerExpression;

	struct MemoryPool {
	
		char *memoryPool;
		char *currentIt;

		MemoryPool() {
			memoryPool = NULL;
			currentIt = NULL;
		}
	};

	//=========================================================
	// 実用のため、思考の進行状況をお伝えするインターフェイスを持ちますよ。
	//=========================================================
	struct ProgressData {
		
		float          progressRate;    // 0〜100 %（負の数は開始前を表すとする）
		vector<string> messageLogs;     // 出来事を随時、追加したりしなかったり。
		Move           answerMove;      // 回答（進捗 100% のとき、意味をなす）
		string         evaluationSumUp; // 外部向けの評価値  EVALUATION::SUM_UP

		bool           isStop;          // これが true になっているのを確認すると、そこで思考を止める。
		
		ProgressData () {
			progressRate = -1.0f;
			isStop = false;
			evaluationSumUp = EVALUATION::SUM_UP::EVEN;
		}
	};

		

	//=========================================================
	// 指し手リスト
	//=========================================================
	/*
	struct MoveList {

		Move *moveList;
		int size;

		MoveList (int arg_size) {
			size = arg_size;
			moveList = new Move[size];
		}
	};
	*/
	//==========================================================
	// 駒の価値を評価するための素子
	//==========================================================
	struct PieceEvaluateElement {

		int x, y;          // 相対位置（ルートは 0, 0）
		int throuableInfo; // 直下のパネル  THROUABLE_INFO::EX 
		int point;         // 加点値
		vector<PieceEvaluateElement> relativeList; // 周辺パネルリスト
	};

	//==========================================================
	// 地形探索パラメータ
	//==========================================================
	struct TerrainSearchUsage {
	
		int depth;

		int bias_TerrainEvaluate;
		int bias_PeeEvaluate;

		TerrainSearchUsage() {
			depth = 0;
			bias_TerrainEvaluate = 100;
			bias_PeeEvaluate     = 0;
		}
	};

	//==========================================================
	// 傘と柱での評価設定
	//==========================================================
	struct PC_Evaluate_Settings {

		bool isEnabled;

		int attackBias;
		int guardBias;

		int applyBias_forTerrain;
		int applyBias_forPlain;

		vector<int> goalValueList;

		int reqiredHeightForPillar;

		int pieceHeightWeight;

		// PILLAR_COVER::PIECE_POSITION に対応。パーセント。
		int pieceWeight_NOT_INVOLVED;
		int pieceWeight_IN_THE_BOX;
		int pieceWeight_ABOVE_COVER;
		int pieceWeight_ABOVE_COVER_DIAGONALLY;
		int pieceWeight_UNDER_COVER_SIDE;
		int pieceWeight_UNDER_COVER;

		// PILLAR_COVER::ALGORISM_FOR_RISK_DEGREE_ADDITION_TO_A_PILLAR
		int addingAlgorism;
		
		
		// 駒をぶつける手に限定して、一手だけ先読み
		bool oneStepFutureTelling;

		PC_Evaluate_Settings () {
			isEnabled      = false;
			applyBias_forTerrain = 0;
			applyBias_forPlain   = 100;
			reqiredHeightForPillar = 3;
			pieceHeightWeight = 0;
			attackBias = 100;
			guardBias = 100;
			addingAlgorism = PILLAR_COVER::ALGORISM_FOR_RISK_DEGREE_ADDITION_TO_A_PILLAR::MAX_ONLY;
			pieceWeight_NOT_INVOLVED           = 0;
			pieceWeight_IN_THE_BOX             = 0;
			pieceWeight_ABOVE_COVER            = 0;
			pieceWeight_ABOVE_COVER_DIAGONALLY = 0;
			pieceWeight_UNDER_COVER_SIDE       = 0;
			pieceWeight_UNDER_COVER            = 0;
			oneStepFutureTelling = false;
		}
	};

	//==========================================================
	// 柱構造体
	//==========================================================
	struct Pillar {
	
		int bottomX, bottomY; // 根本の位置
		int topX, topY;       // 先端の位置
	
		int coverTopHeight;      // 傘（一番高い位置の）の高さ。根本からの。
		vector<int> contentList; // vector<PILLAR_COVER::THROUABLE_INFO>
		int riskDegree;          // 危険度（パネル成分で危険度最大値を割り振ったもの）
		int maxRiskDegree;       // 危険度最大値（先端がどこまでの高さに達しているか）
	
		Pillar() {
			coverTopHeight = -1; // 傘がない＝地面に埋まっている、という考え方。
			riskDegree     = 0;
			maxRiskDegree  = 0;
		}
		void Clear() {
			bottomX = 0;
			bottomY = 0;
			topX = 0;
			topY = 0;
	
			coverTopHeight = -1;
			contentList.clear();
			riskDegree = 0;
			maxRiskDegree = 0;
		}
	};

	//==========================================================
	// 手札評価パラメータ
	//==========================================================
	struct HandCardEvaluate {

		bool isEnabled;
		int point_Card;
		int point_Throughable;
		int point_MustGoSomeway;

		HandCardEvaluate () {
			isEnabled             = false;
			point_Card          = 0;
			point_Throughable   = 0;
			point_MustGoSomeway = 0;
		}
	};

	//==========================================================
	// カード補充手評価パラメータ
	//==========================================================
	struct TakeCardMoveEvaluate {

		bool isEnabled;
		int  handCardsLessThanNum;
		int  addRate;
		int  nextCheckMateValue;

		TakeCardMoveEvaluate () {
			isEnabled            = false;
			handCardsLessThanNum = 0;
			addRate              = 0;
			nextCheckMateValue   = 0;
		}
	};

	//==========================================================
	// 詰めろ探索設定
	//==========================================================
	struct PreMateSearchSetting {

		bool isEnabled;
		int  depth;
		int  subjectNum;
		int  value;
		bool isIncludeDirectCheck; // その手自体が王手である手を「詰めろ」の一種とするか？ trueで「一種とする」デフォルトは「しない」

		PreMateSearchSetting () {
			isEnabled  = false;
			depth      = 0;
			subjectNum = 0;
			value      = 0;
			isIncludeDirectCheck = false;
		}
	};
	
	
	//==========================================================
	// 手筋で位置の評価判定を考慮するためのパラメータ構造体
	//==========================================================
	struct HandyProc_PositionBias {
		
		int x;
		int y;
		int riskBias;
		int advantageBias;
		
		HandyProc_PositionBias (){
			x = 0;
			y = 0;
			riskBias = 0;
			advantageBias = 0;
		}
	};
	
	//==========================================================
	// 手筋構造体用のマス目の状態構造体
	//==========================================================
	struct HandyProc_GridStatus {
		
		int x;
		int y;
		int THROUGHABLE_INFO_EX;
		
		HandyProc_GridStatus () {
			x = 0;
			y = 0;
			THROUGHABLE_INFO_EX = 0;
		}
		
	};
	
	//==========================================================
	// 手筋構造体
	//==========================================================
	struct HandyProc {
		
		int gridFromX;
		int gridFromY;
		int gridToX;
		int gridToY;
		
		int gameProgressFrom;
		int gameProgressTo;
		
		int action;
		int actionAffectsX;
		int actionAffectsY;
		int actionAffectsStatus; // THROUGHABLE_INFO_EX を使用
		string actionCardId;    // カード設置の場合のみ参照される。空文字で未指定の意味
		int actionCardDirection; // カード設置の場合のみ参照される。NONEで未指定の意味
		
		int point;
		
		vector<HandyProc_GridStatus> gridStatusList;
		
		vector<HandyProc_PositionBias> positionBiasList;
		
		HandyProc () {
			gameProgressFrom = 0;
			gameProgressTo   = -1;  // -1 で 「無条件」とする
			
			action = 0;
			actionAffectsX = 0;
			actionAffectsY = 0;
			actionAffectsStatus = 0;
			actionCardId = "";
			actionCardDirection = DIRECTION::NONE;
			
			point = 0;
			
			gridFromX = 0;
			gridFromY = 0;
			gridToX = 8;
			gridToY = 8;
		}
		
	};
	
	//=========================================================
	// 位置評価パラメータ＿方向別
	//=========================================================
	struct Parameters_PositionEvaluation_Sub {
		
		int THROUABLE_Risk;
		int NOT_THROUABLE_Risk;
		int MUST_TO_UP_Risk;
		int MUST_TO_DOWN_Risk;
		int MUST_TO_HORIZONTAL_Risk;
		
		int THROUABLE_Advantage;
		int NOT_THROUABLE_Advantage;
		int MUST_TO_UP_Advantage;
		int MUST_TO_DOWN_Advantage;
		int MUST_TO_HORIZONTAL_Advantage;
		
		int myPiece_AdvantageStart;
		int myPiece_THROUABLE;
		int myPiece_NOT_THROUABLE;
		int myPiece_MUST_TO_UP;
		int myPiece_MUST_TO_DOWN;
		int myPiece_MUST_TO_HORIZONTAL;
		int myPiece_againstMyPiece;
		int myPiece_againstOpPiece;
		
		int opPiece_RiskStart;
		int opPiece_THROUABLE;
		int opPiece_NOT_THROUABLE;
		int opPiece_MUST_TO_UP;
		int opPiece_MUST_TO_DOWN;
		int opPiece_MUST_TO_HORIZONTAL;
		int opPiece_againstMyPiece;
		int opPiece_againstOpPiece;
		
		int myPieceReliefPower;
		
		Parameters_PositionEvaluation_Sub () {
		
			THROUABLE_Risk = 0;
			NOT_THROUABLE_Risk = 0;
			MUST_TO_UP_Risk = 0;
			MUST_TO_DOWN_Risk = 0;
			MUST_TO_HORIZONTAL_Risk = 0;
			
			THROUABLE_Advantage = 0;
			NOT_THROUABLE_Advantage = 0;
			MUST_TO_UP_Advantage = 0;
			MUST_TO_DOWN_Advantage = 0;
			MUST_TO_HORIZONTAL_Advantage = 0;
			
			myPiece_AdvantageStart = 0;
			myPiece_THROUABLE = 0;
			myPiece_NOT_THROUABLE = 0;
			myPiece_MUST_TO_UP = 0;
			myPiece_MUST_TO_DOWN = 0;
			myPiece_MUST_TO_HORIZONTAL = 0;
			myPiece_againstMyPiece = 0;
			myPiece_againstOpPiece = 0;
			
			opPiece_RiskStart = 0;
			opPiece_THROUABLE = 0;
			opPiece_NOT_THROUABLE = 0;
			opPiece_MUST_TO_UP = 0;
			opPiece_MUST_TO_DOWN = 0;
			opPiece_MUST_TO_HORIZONTAL = 0;
			opPiece_againstMyPiece = 0;
			opPiece_againstOpPiece = 0;
			
			myPieceReliefPower = 0; // 置いておくだけの安心度
		}
	};

	//=========================================================
	// 位置評価パラメータ
	//=========================================================
	struct Parameters_PositionEvaluation {
		
		bool isEnabled;
		
		int attackBias;
		int guardBias;

		int applyBias_forTerrain;
		int applyBias_forPlain;
		
		Parameters_PositionEvaluation_Sub upperFront;
		Parameters_PositionEvaluation_Sub upperLeft;
		Parameters_PositionEvaluation_Sub upperRight;
		Parameters_PositionEvaluation_Sub underFront;
		Parameters_PositionEvaluation_Sub underLeft;
		Parameters_PositionEvaluation_Sub underRight;
		
		Parameters_PositionEvaluation () {
			isEnabled = false;
			attackBias = 100;
			guardBias = 100;

			applyBias_forTerrain = 0;
			applyBias_forPlain = 100;
		}
		
	};
	
	//=========================================================
	// 位置の評価関数で、特定位置の評価を加味するためのもの
	//=========================================================
	struct PositionCare {
		
		int x;
		int y;
		int advantageBias;
		int riskBias;
		
		PositionCare () {
			x = 0;
			y = 0;
			advantageBias = 0;
			riskBias = 0;
		}
	};
	struct PositionCareSettings {
		
		bool isEnabled;
		int applyBias_forTerrain;
		int applyBias_forPlain;
		
		vector<PositionCare> positionList;
		
		PositionCareSettings() {
			isEnabled = false;
			applyBias_forTerrain = 0;
			applyBias_forPlain   = 0;
		}
	};
	
	//==========================================================
	// 圧力バランス評価用パラメータ群
	//==========================================================
	struct Parameters_PressureEvaluation {
		
		bool isEnabled;
		
		int type; // タイプ：COLUMN 同縦軸での対抗 / MIRROR ミラー
		
		// 実行するターン
		int enabledTurn_from; // 以降。この数字を含む
		int enabledTurn_to;   // 以前。この数字を含む
		vector<int> enabledTurn_skip; // 飛ばす
		vector<int> enabledTurn_forced; // 強制実行。トップの isEnabled の方が優先
		
		int pointForIncreasePressuringNum; // 自分が圧力をかけている    軸 の本数が増える指し手に加算するポイント
		int pointForDecreasePressuredNum;  // 相手に圧力をかけられている軸 の本数が増える指し手に加算するポイント
		int pointForMirrorMove;            // 鏡打ちになる指し手に加算するポイント
		
		// 特定の指し手に対するバイアス(%)
		int bias_iinogare_h;
		int bias_iinogare_d;
		int bias_keno_h;
		int bias_keno_d;
		int bias_take_card;
		int bias_others;
		
		int TYPE_COLUMN;
		int TYPE_MIRROR;
		
		Parameters_PressureEvaluation () {
			isEnabled = false;
			enabledTurn_from = 0;
			enabledTurn_to   = 9999;
			pointForIncreasePressuringNum = 0;
			pointForDecreasePressuredNum  = 0;
			pointForMirrorMove            = 0;
			
			bias_iinogare_h = 100;
			bias_iinogare_d = 100;
			bias_keno_h     = 100;
			bias_keno_d     = 100;
			bias_take_card  = 100;
			bias_others     = 100;
			
			TYPE_COLUMN = 1;
			TYPE_MIRROR = 2;
			
			type = TYPE_COLUMN;
		}
	};
	
	//=========================================================
	// 論理距離での絞り込み戦略のパラメータ
	//=========================================================
	struct DistanceDiffScreening {
		
		bool isEnabled;

		// 実行するターン
		int enabledTurn_from; // 以降。この数字を含む
		int enabledTurn_to;   // 以前。この数字を含む
		
		// この件数に絞り込む
		int screeningLength;
		
		// 地形探索で使用する際のバイアス(%)：0ならば地形探索時に使用しない
		int bias_TerrainEvaluate;
		
		// 駒台の駒を考慮する際の設置コスト
		int putPieceCost;
		
		// 駒台の駒を考慮する方式
		int handPieceConsideringMethod;
		bool handPieceConsideringMethod_onlyWhenNoPieceOnBoard;
		int handPieceConsideringMethod_sweepFrom; // この数字含む。
		int handPieceConsideringMethod_sweepTo;   // この数字含む。
		int handPieceConsideringMethod_than; // この数字を含む。maxOnly は -1 
		int handPieceConsideringMethod_NONE;
		int handPieceConsideringMethod_SWEEP;
		int handPieceConsideringMethod_ALL_BOTTOM_GRIDS;
		int handPieceConsideringMethod_MORE_THROUABLE_COLUMNS;
		
		
		int achievePoint_Ahead_forMe;
		int achievePoint_Ahead_againstOp;
		int achievePoint_Backward_forMe;
		int achievePoint_Backward_againstOp;
		int achievePoint_Backward_turnBack;
		
		bool oneStepFutureTelling_Ahead_forMe;
		bool oneStepFutureTelling_Ahead_againstOp;
		bool oneStepFutureTelling_Backward_forMe;
		bool oneStepFutureTelling_Backward_againstOp;
		
		int distanceOffset_forMe;
		int distanceOffset_forOp;
		
		DistanceDiffScreening () {
			
			isEnabled        = false;
			enabledTurn_from = 0;
			enabledTurn_to   = -1;
			screeningLength  = 20;
			putPieceCost     = 1;
			achievePoint_Ahead_forMe        = 0;
			achievePoint_Ahead_againstOp    = 0;
			achievePoint_Backward_forMe     = 0;
			achievePoint_Backward_againstOp = 0;
			achievePoint_Backward_turnBack  = 0;
			distanceOffset_forMe = 0;
			distanceOffset_forOp = 0;
			bias_TerrainEvaluate = 0;
			
			oneStepFutureTelling_Ahead_forMe        = false;
			oneStepFutureTelling_Ahead_againstOp    = false;
			oneStepFutureTelling_Backward_forMe     = false;
			oneStepFutureTelling_Backward_againstOp = false;
			
			handPieceConsideringMethod_NONE                   = 0;
			handPieceConsideringMethod_SWEEP                  = 1;
			handPieceConsideringMethod_ALL_BOTTOM_GRIDS       = 2;
			handPieceConsideringMethod_MORE_THROUABLE_COLUMNS = 3;
			
			handPieceConsideringMethod                        = handPieceConsideringMethod_NONE;
			handPieceConsideringMethod_onlyWhenNoPieceOnBoard = true;
			handPieceConsideringMethod_sweepFrom              = 0;
			handPieceConsideringMethod_sweepTo                = 0;
			handPieceConsideringMethod_than                   = -1;

		}
	};
	
	//=========================================================
	// 相手陣への論理距離での判定のパラメータ
	//=========================================================
	struct Parameters_DistanceToGoal {
		
		bool isEnabled;
		
		int guardBias;
		int attackBias;
		
		// 移動コスト各種
		int moveCostDirectionChange;
		int moveCostArrow;
		int moveCostFromBlank;
		int moveCostToBlank;
		int moveCostMyPiece;
		int moveCostMyPieceClose;
		int moveCostOpPiece;
		int moveCostOpPieceClose;
		
		int handPieceBias_exist;   // 持ち駒があるときの考慮割合(%)
		int handPieceBias_absent;  // 持ち駒がないときの考慮割合(%)
		
		int goalHeight; // 相手陣の端から何段目を目標とするか。0で一番端。
		
		vector<int> distanceRateList; // 目標地点までの論理距離ごとの評価ポイント。ハミ出る分は 0 扱い。
		
		int distanceLimit; // 調査時、この距離「を越えたら」打ち切る。
		
		
		///// 絞込み戦略パラメータ
		DistanceDiffScreening distanceDiffScreening;
		
		Parameters_DistanceToGoal () {
			
			isEnabled = false;
			
			guardBias  = 100;
			attackBias = 100;
			distanceLimit = 20;

			goalHeight = 0;

			moveCostDirectionChange = 1;
			moveCostArrow           = 10;
			moveCostFromBlank       = 2;
			moveCostToBlank         = 2;
			moveCostMyPiece         = 1;
			moveCostMyPieceClose    = 1;
			moveCostOpPiece         = 5;
			moveCostOpPieceClose    = 10;
			
			handPieceBias_exist  = 30;
			handPieceBias_absent = 10;
			
			
		}
		
	};
	
	//=========================================================
	// 結果を取得する際にランダム要素を入れる設定
	//=========================================================
	struct PickResultFrom {
		
		bool isEnabled;
		int enabledTurn_from ;
		int enabledTurn_to;
		int topLength; // トップから何件を候補とするか
		
		PickResultFrom () {
			
			isEnabled = false;
			enabledTurn_from = 0;
			enabledTurn_to = 0;
			topLength = 1;
		}
	};
	
	
	//==========================================================
	// 駒評価用パラメータ群
	//==========================================================
	struct Parameters_PieceEvaluation {

		// 駒評価用素子リスト
		vector<PieceEvaluateElement> pieceEvaluateElementList;

		// 高さバイアス
		int heightValue_Bias;

		// 地形探索パラメータ
		TerrainSearchUsage terrainSearchUsage;
		
		// 詰み探索深度
		int mateSearchDepth;

		// 詰まされ警戒探索深度
		int matedSearchDepth;
		int matedSearchSubjectNumber; // 候補手の上位何件に対してチェックを行うか：ハミ出たら諦めて、最長手数で詰まされる手を選択。

		// 詰めろ探索設定
		PreMateSearchSetting preMate;

		// 傘と柱での評価設定
		PC_Evaluate_Settings pcEvalInfo;

		// 手札評価パラメータ
		HandCardEvaluate handCardEvaluate;

		// 手札補充手評価パラメータ
		TakeCardMoveEvaluate takeCardMoveEvaluate;
		
		// 手筋リスト：これは「駒」の評価とは違う。けどもう面倒なので一緒にしてしまいますよ。
		vector<HandyProc> handyProcList;
		
		// 位置評価パラメータ：これも「駒」の評価とは違うけれど以下同文。
		Parameters_PositionEvaluation parameters_PositionEvaluation;
		
		// 圧力バランス判定パラメータ：こ以下同文。
		Parameters_PressureEvaluation parameters_PressureEvaluation;
		
		// 特定位置の評価を加味するためのもの
		PositionCareSettings positionCareSettings;
		
		Parameters_DistanceToGoal parameters_DistanceToGoal;
		
		PickResultFrom pickResultFrom;
		
		Parameters_PieceEvaluation() {
			mateSearchDepth  = 0;
			matedSearchDepth = 0;
			matedSearchSubjectNumber = 0;
		}

	};

	namespace Learning {

		//==========================================================
		// 課題局面と正解手
		//==========================================================
		struct Problem {

			string problemFile;                   // 課題局面
			InnerExpression *ie;                  // 内部表現；
			vector< pair<Move, int> > answerList; // 正解手（点数マイナスでNG手）

			Problem() {
				problemFile = "";
				ie          = NULL;
			}
		};

		//==========================================================
		// 学習用手順データ
		//==========================================================
		struct LearningProgram {

			string AI_File_Input;  // 初期AI設定ファイル
			string AI_File_Output; // 学習完了後AI保存ファイル

			vector<Problem> problemList; // 課題局面リスト
			int passBorderPoint;         // 合格点（100点満点）

			int examinCount;     // 試行回数
			int copyCount;       // コピー件数

			LearningProgram() {
				AI_File_Input  = "";
				AI_File_Output = "";
				passBorderPoint = 0;
				examinCount     = 0;
				copyCount       = 0;
			}
		};

		//==========================================================
		// 自己対戦学習データ
		//==========================================================
		struct SelfMatch {

			string AI_File_Input;  // 初期AI設定ファイル
			string AI_File_Output; // 学習完了後AI保存ファイル
			string matchInitState; // 対戦開始時の局面。ルールも必要？
			int copyCount; // コピー生成件数：これに、前回のトップを加える

			SelfMatch() {
				AI_File_Input  = "";
				AI_File_Output = "";
				matchInitState = "";

				copyCount = 0;

			}
		};

	};
	class AI {

	public:

		///// 駒評価用パラメータ群
		Parameters_PieceEvaluation _parameters_PieceEvaluation;
		
		///// 駒価値評価素子リスト：大元として、直下パネルごとに持つ
		//vector<PieceEvaluateElement> _pieceEvaluateElementList;

		///// 高さ評価値
		int _heightValue_Goal; // 相手陣に達している駒の価値（絶対値。位置関係判定は行われない）
		//int _heightValue_Bias; // 高さを百分率した値に掛けて、駒評価値にプラスする。

		///// 駒評価素子をコピー。ゆらぎ値を指定 ( 0 で純正コピー )
		static PieceEvaluateElement PEE_Copy( const PieceEvaluateElement &original, int fluctuation, bool isPointOnly = false );
		static Parameters_PieceEvaluation Parameters_PEE_Copy( const Parameters_PieceEvaluation &original, int fluctuation, bool isPointOnly = false );
		static int FluctuatedCopy( int originalValue, int fluctuation, int fluctuateRange );

		///// コピーの際、素子が消滅するかどうか
		static bool IsVanish ( const PieceEvaluateElement &PEE, int fluctuation );

		///// コピーの際、素子が発生するかどうか
		static void SpringElement( vector<PieceEvaluateElement> &result, const PieceEvaluateElement &PEE, int fluctuation );

		///// 盤面上のグリッドとして、この距離のものをください。ランダムで。
		static pair<int, int> GetPOINT_From_2DBoard_Distance ( int x, int y, int distance );

		///// THROUABLE_INFO::EX をゆらぎコピーする。
		static int GetThrouableInfo_AS_FCopy ( const int &original, int fluctuation );

		///// 素子の子要素の数
		static int HowManySubElement ( const PieceEvaluateElement &PEE);

		///// 素子の子要素以下の加点値の合計
		static int HowMuchPointDoSubElementsHave ( const PieceEvaluateElement &PEE);

		///// 駒評価素子リストをファイルに保存
		static void PEE_Output_File ( const Parameters_PieceEvaluation &parameters_PieceEvaluation, string fileName);

		static string PEE_to_String ( const PieceEvaluateElement &pee, int indent);

		///// ファイルから、駒評価素子リストを読み込む
		static Parameters_PieceEvaluation PEE_Read_File( string fileName );
		static PieceEvaluateElement PEE_Read_PEE_Tag ( const NotXML::Tag &peeTag, const map < string, PieceEvaluateElement > &variablePEEList );

		///// 地形探索パラメータタグ と 構造体とを相互変換
		static TerrainSearchUsage Read_TerrainSearchTag ( const NotXML::Tag &terrainTag );
		static string TerrainSearchUsage_to_String_forXMLSave ( const TerrainSearchUsage &terrainSearchUsage );

		///// 柱と傘評価設定タグ と 構造体とを相互変換
		static PC_Evaluate_Settings Read_PC_Evaluate_SettingsTag ( const NotXML::Tag &pcEvalTag, PC_Evaluate_Settings inheritBase = PC_Evaluate_Settings() );
		static string PC_Evaluate_Setting_to_String_forXMLSave ( const PC_Evaluate_Settings &pcEvalSetting );

		///// 手札評価パラメータ タグ と 構造体とを相互変換
		static HandCardEvaluate Read_HandCardEvaluateTag ( const NotXML::Tag &handCardEvalTag );
		static string HandCardEvaluate_to_String_forXMLSave ( const HandCardEvaluate &handCardEval );

		///// 手札補充手 タグ と 構造体とを相互変換
		static TakeCardMoveEvaluate Read_TakeCardMoveEvaluateTag ( const NotXML::Tag &takeCardMoveEvalTag );
		static string TakeCardMoveEvaluate_to_String_forXMLSave ( const TakeCardMoveEvaluate &takeCardMoveEval );

		///// 詰めろ探索 タグ と 構造体を 相互変換
		static PreMateSearchSetting Read_PreMateSearchTag ( const NotXML::Tag &preMateTag );
		static string PreMateSearchSetting_to_String_forXMLSave ( const PreMateSearchSetting &preMate );

		///// 駒評価素子関連の値を初期設定
		void PEE_Init();
		void PEE_Init( Parameters_PieceEvaluation parameters_PieceEvaluation );
		
		///// 指し手をタグ化
		static NotXML::Tag Move_Into_Tag ( const Move &move );

		///// MoveタグからMove構造体を再現
		static Move Move_From_Tag ( const NotXML::Tag &moveTag );

		///// Move が一致しているかどうか
		static bool MoveEquals ( const Move &moveA, const Move &moveB );

		///// 学習用手順構造体をファイルから読み込む
		static Learning::LearningProgram ReadLearningProgram( string fileName ); 

		///// 素子同士を掛け合わせる
		static Parameters_PieceEvaluation GeneticCrossover ( Parameters_PieceEvaluation PEE1, Parameters_PieceEvaluation PEE2, int fluctuation = 0 );
		static Parameters_PieceEvaluation GeneticCrossover ( Parameters_PieceEvaluation PEE1, Parameters_PieceEvaluation PEE2, Parameters_PieceEvaluation PEE3, int fluctuation = 0 );
		static Parameters_PieceEvaluation GeneticCrossover ( vector<Parameters_PieceEvaluation> PEEList, int fluctuation = 0 );

		///// 素子２個掛け合わせ；全組み合わせ
		static vector<Parameters_PieceEvaluation> GeneticCrossover_ALL ( Parameters_PieceEvaluation PEE1, Parameters_PieceEvaluation PEE2, int fluctuation = 0 );

		//static void Test();
		//static AI* GetInstance();
		
		static vector<int> Get_ThrouableInfoEXList_ALL();
		static vector<int> Get_ThrouableInfoEXList_Root();

		//=========================================================
		// 指し手(Move)をコンソールに表示、または文字列として取得
		//=========================================================
		static string ShowMoveToConsole( const AI_Modules::Move &move, bool isOnlyReturnValue = false );

		//=========================================================
		// ルールはアプリケーション内で共通ですよね？ というつもり。
		//=========================================================
		static Rule _rule;
		static map<int, Card> _cardList;
		static map<int, string> _cardNameList;

		AI();
		~AI();

		//=========================================================
		// ルールをセット
		//=========================================================
		static void SetRule( Rule rule, map<int, Card> cardList);
		static void SetCardNameList( map<int, string> cardNameList);


		//=========================================================
		// カード情報をください
		//=========================================================
		static Card GetCardInfo( int cardId );

		//=========================================================
		// 次の一手を教えて
		//=========================================================
		Move GetNextMove( InnerExpression *innerExpression );
		Move GetNextMove_Random( InnerExpression *innerExpression );
		Move GetNextMove_Monte( InnerExpression *innerExpression );
		Move GetNextMove_Tree( InnerExpression *innerExpression );
		Move GetNextMove_Composite ( InnerExpression *innerExpression );
		Move GetNextMove_Composite2( InnerExpression *innerExpression );
		
		// 「この局面」で「この指し手」を選んだ場合の、モンテカルロでの勝率を取得
		float GetWinRateByMonte ( InnerExpression *currentInnerExpression, Move thisMove );

		Move GetNextMove_For_AI_SelfMatchLearning( InnerExpression *innerExpression );

		// 学習者用
		Move GetNextMove_For_AI_SelfMatchLearning_ForCandidate( InnerExpression *innerExpression );
		// 対戦相手用
		Move GetNextMove_For_AI_SelfMatchLearning_ForOpponent( InnerExpression *innerExpression );

		// PEEファイルの実験用：手作業でいじりながら効果を確かめるための。
		//Move GetNextMove_PEE_Test ( InnerExpression *innerExpression);
		void GetNextMove_PEE_Test ( InnerExpression *innerExpression, ProgressData &progressData );
		vector<Move> GetNextMove_PEE_Test_PreSearch ( vector<Move> &allMoveList, InnerExpression *innerExpression);  // 事前の絞込み処理
		vector<Move> GetNextMove_PEE_Test_PreSearch ( vector<Move> &allMoveList, InnerExpression *innerExpression, ProgressData &progressData );  // 事前の絞込み処理
		bool         GetNextMove_PEE_Test_AvoidMated ( vector< pair<Move, int>> &allMoveList, InnerExpression *innerExpression, ProgressData &progressData ); // 詰まされる手を除外する
		void         GetNextMove_PEE_Test_PreMate    ( vector< pair<Move, int>> &allMoveList, InnerExpression *innerExpression, ProgressData &progressData ); // 詰めろを評価

		// 相手からの詰め手順があれば妨害する手を探す
		vector<Move> GetNextMove_PEE_Test_DisturbMated ( InnerExpression *innerExpression);

		//=========================================================
		// 詰み手順を教えて
		//=========================================================
		vector<Move> GetMateProcedure( InnerExpression *currentState, int level, bool isCheckAlreadyMate = false );
		void GetMateProcedure_Thread( vector<Move> &result, vector<Move> &allMoveList, InnerExpression *currentState, int level, bool isCheckAlreadyMate = false );
		void GetMateProcedure_Thread( vector<Move> &result, InnerExpression *currentState, int level, bool isCheckAlreadyMate = false );

		// 詰みがあればtrue 指し手は最初の攻め手の一手しか記録しない。
		bool GetMateProcedure_Revise( Move &mateMove, int &mateDepth, InnerExpression *currentState, int level);
		bool DigInto_Mate_Revise( MemoryPool &memoryPool, Move &mateMove, int &mateDepth, InnerExpression *currentState, int level);


		//=========================================================
		// 評価関数
		//【引数】局面、どっちの立場で評価しますか？
		//=========================================================
		int Evaluate( InnerExpression* innerExpression, int forThisTurn, int rutinFlgs = -1);
		
		bool Evaluate_IsWin( int &result, InnerExpression* innerExpression, int &forThisTurn);

		int _count_Evaluate_Piece;
		int Evaluate_Piece( const InnerExpression* innerExpression, const int forThisTurn);
		int Evaluate_Card( const InnerExpression* innerExpression, const int forThisTurn);
		int Evaluate_Check( InnerExpression *innerExpression, const int forThisTurn);

		int Evaluate_Grid ( const int gridList_ThrouableInfo[15][15], int x, int y, int currentMax, int restStep, const int forThisTurn);
		int Evaluate_BlackholeExaminaion ( const int gridList_ThrouableInfo[15][15], int x, int y, int stepCount = 0 );
		
		
		struct PositionValueData {
			int risk;
			int advantage;
			PositionValueData () {
				risk = 0;
				advantage = 0;
			}
		};
		PositionValueData Evaluate_Position ( const InnerExpression *innerExpression, Position pos );
		PositionValueData Evaluate_Position ( const InnerExpression *innerExpression, Position pos, const int forThisTurn );
		
		int Evaluate_PositionForWholeBoard ( const InnerExpression *innerExpression, const int forThisTurn );
		int Evaluate_PositionForSpecificPositions ( const InnerExpression *innerExpression, const int forThisTurn );
		
		
		int Evaluate_Move_By_PressureBalance ( const InnerExpression *currentIe, Move move, const InnerExpression *nextIe, int forThisTurn );
		int GetPressureBalanceAtThisColumn ( const InnerExpression *innerExpression, int x );
		pair<int, int> GetNumberOfPressuringColumns (  const InnerExpression *innerExpression );
		
		// この状態から一定手数指し進めた場合の評価値を求める
		int Evaluate_SemiMonte ( InnerExpression *innerExpression, int moveNum, int forThisTurn );
		int Evaluate_SemiMonte_EvalThisBoard ( const InnerExpression *innerExpression, int forThisTurn, bool isUsingPEE, bool isUsingPC);
		
		int GetDistanceToGoal ( InnerExpression* innerExpression, int forThisTurn, int x, int y, int entryAngle, int stepsSoFar, int momentumSoFar = 0 );
		int GetDistanceToGoal ( int currentMinSteps, const int &forThisTurn, const int &x, const int &y, int entryAngle, int stepsSoFar, int momentumSoFar = 0 );

		//int GetDistanceToGoal ( int currentMinSteps, set<int> &passedGridList_GetDistanceToGoal, const int &forThisTurn, const int &x, const int &y, int entryAngle, int stepsSoFar, int momentumSoFar = 0 );
		
		int Evaluate_DistanceToGoal ( InnerExpression* innerExpression, int forThisTurn );
		
		
		//==========================================================
		// 駒評価素子を使った評価
		// 大元では、駒があった位置を指定してこれを呼ぶ。
		// 以降の再帰時には、子要素すべてを呼ぶ。
		//【引数】
		// int x, y 座標。この位置を中心とする
		//==========================================================
		int Evaluate_PEE ( 
			const InnerExpression* innerExpression, 
			const int &myPieceColor, // GRID:: の値
			int x, int y, const vector<PieceEvaluateElement> &PEE_List );

		//==========================================================
		// 柱と傘での評価
		//==========================================================
		int Evaluate_PC ( const InnerExpression* innerExpression, int forThisTurn );
		int Evaluate_PC_S ( const InnerExpression* innerExpression, int forThisTurn );

		//==========================================================
		// 手札を評価
		//==========================================================
		int Evaluate_HandCards ( const InnerExpression* innerExpression, int forThisTurn );

		//==========================================================
		// 手札を補充する手を評価
		//==========================================================
		int Evaluate_TakeCardMove ( InnerExpression* innerExpression, Move &takeCardMove, int forThisTurn);
		
		//=========================================================
		// 手筋を評価
		//=========================================================
		int AI::Evaluate_HandyProc ( const InnerExpression* currentIe, Move move, const InnerExpression* nextIe );


		//==========================================================
		// 評価関数：セット
		//==========================================================
		///// 地形判定の探索ツリーの末端で使うもの
		int Evaluate_Set_For_TerrainSearch ( InnerExpression* innerExpression, int forThisTurn);

		///// 中盤用：探索深度は浅めにして、評価判定には地形判定を使う
		int Evaluate_Set_For_MiddleStage ( InnerExpression* innerExpression, int forThisTurn );

		///// ツリー探索内で、選択肢が多いときの絞込みに使う評価
		int Evaluate_Set_For_PickUp ( InnerExpression* innerExpression, int forThisTurn );


		//==========================================================
		// グリッドの価値判定
		//==========================================================
		int Evaluate_Terrain 
		(
			const InnerExpression *currentState  // この局面における評価
			,int x, int y                        // 探索の現在地
			,int currentMoveDirection            // 現在、この方向へ移動中（直線移動なら低コスト）
			,int energy                          // 残り移動エネルギー
			,int myColor                         // 自分の色
			,vector<vector<bool>> walkedGrid     // すでに通った位置へ印をつける（サイズは大元の呼び出し元で定義されているとする）
			
			,int level = 0
		);

		//==========================================================
		// グリッドの価値を、周辺にあるパネルの密集度で判断
		//==========================================================
		int Evaluate_DensityOfPanel(
			const InnerExpression *currentState  // この局面における評価
			,int x, int y                        // 探索の現在地
			,int myPiece                         // 自分の色
		);

		//==========================================================
		// 地形の価値を見定める：地形を変えない指し手だけで探索してみる
		//==========================================================
		bool _recursionBlock_Evaluate_Terrain;
		int Evaluate_Terrain ( InnerExpression *currentState, int forThisTurnsView, int depth );


		//=========================================================
		// 勝敗判定：純粋に、現時点で、ルール上
		//【引数】この手番として、勝ってますか？
		//【戻値】true 勝ってる false まだ勝ってない
		//=========================================================
		bool IsWin( InnerExpression *innerExpression, int turn );

		//=========================================================
		// 勝つ指し手
		//【引数】この指し手は、この手番として、勝ってますか？
		//【戻値】true 勝ってる false まだ勝ってない
		//=========================================================
		//bool IsWinMove( Move &move, int turn );
		bool IsWinMove( Move &move);

		//=========================================================
		// 次の手番を取得
		//=========================================================
		static int SwitchTurn( int turn );



		//==========================================================
		// Evaluate_PC 子処理群
		//==========================================================
		int Evaluate_PC_Translate_THROUGHABLE_into_OneSidePath ( int throuableInfo, int forThisTurn, bool isDiagonal );
		int Evaluate_PC_DevideSubstansialNumber ( int value, int divider ); // 盤面上のマス目数を想定した割り算。
		bool Evaluate_PC_IsOnTheBoard ( int x, int y); // _rule に照らして、盤の上なら true

		void Evaluate_PC_MakePillarList_BlackToWhite ( const InnerExpression *innerExpression, vector<Pillar> &pillarList );
		void Evaluate_PC_MakePillarList_WhiteToBlack ( const InnerExpression *innerExpression, vector<Pillar> &pillarList );
		void Evaluate_PC_RiskDegreeUpdate ( vector<Pillar> &pillarList, int attackerTurn );
		void Evaluate_PC_MakePiecePositionList ( const InnerExpression *innerExpression, vector< pair<int, int>> & piecePisitionList, int gridColor);

		int Evaluate_PC_GetPiecePositionTowardThePillar ( const Pillar &pillar, int pieceX, int pieceY, int &result_PieceHeight);
		int Evaluate_PC_HowFarPieceFromPillar ( const Pillar &pillar, int pieceX, int pieceY, const InnerExpression* innerExpression);

		int Evaluate_PC_CalicurateApparedRiskDegree ( const vector<Pillar> &pillarList, const vector<pair<int, int>> &piecePositionList, const InnerExpression *innerExpression);
		
		int Evaluate_PC_GetRisk_forThisTurn (  const InnerExpression *innerExpression, int turn );
		int Evaluate_PC_GetRisk_forThisTurn_OneStepFutureTelling ( InnerExpression *innerExpression, const vector<Move> &nextMoveList, int forThisTurn, bool isGetMaxOrMin );
	

		///// 柱とする枠・型を生成：現時点のルールに基づいて
		void Evaluate_PC_Prepare_PillarMold();
		void Evaluate_PC_Prepare_PillarMold_Black();
		void Evaluate_PC_Prepare_PillarMold_White();
		vector< vector<Position> > _pillarMold_Black;
		vector< vector<Position> > _pillarMold_White;

		// 用意済みの型を使って、柱を作成
		void Evaluate_PC_MakePillarList_withPreparedMold ( 
			const vector< vector<Position> > &pillarMoldList, int attacker,
			const InnerExpression *innerExpression, vector<Pillar> &pillarList );

		// 用意済みの型を使って、柱を作成：ついでに潜在的危険度も計算
		void Evaluate_PC_MakePillarList_withPreparedMold_and_CaliculatePotentialRiskDegree ( 
			const vector< vector<Position> > &pillarMoldList, int attacker,
			const InnerExpression *innerExpression, vector<Pillar> &pillarList );


		//==========================================================
		// Evaluate_PC 子処理群：高速化実験
		//==========================================================

		Pillar *pillarList_Black_to_White_S;
		Pillar *pillarList_White_to_Black_S;

		Position *piecePositionList_Black;
		Position *piecePositionList_White;

		int Evaluate_PC_S_Translate_THROUGHABLE_into_OneSidePath ( int throuableInfo, int forThisTurn, bool isDiagonal );
		int Evaluate_PC_S_DevideSubstansialNumber ( int value, int divider ); // 盤面上のマス目数を想定した割り算。
		bool Evaluate_PC_S_IsOnTheBoard ( int x, int y); // _rule に照らして、盤の上なら true

		void Evaluate_PC_S_MakePillarList_BlackToWhite ( const InnerExpression *innerExpression, vector<Pillar> &pillarList );
		void Evaluate_PC_S_MakePillarList_WhiteToBlack ( const InnerExpression *innerExpression, vector<Pillar> &pillarList );
		void Evaluate_PC_S_RiskDegreeUpdate ( vector<Pillar> &pillarList, int attackerTurn );
		int  Evaluate_PC_S_MakePiecePositionList ( const InnerExpression *innerExpression, Position* &piecePisitionList, int gridColor);

		int Evaluate_PC_S_GetPiecePositionTowardThePillar ( const Pillar &pillar, int pieceX, int pieceY, int &result_PieceHeight);
		int Evaluate_PC_S_HowFarPieceFromPillar ( const Pillar &pillar, int pieceX, int pieceY, const InnerExpression* innerExpression);

		int Evaluate_PC_S_CalicurateApparedRiskDegree ( 
			Pillar* &pillarList, Position* &piecePositionList, const InnerExpression *innerExpression
			, int pillarListSize, int pieceListSize
			);


		///// 柱とする枠・型を生成：現時点のルールに基づいて
		void Evaluate_PC_S_Prepare_PillarMold();
		void Evaluate_PC_S_Prepare_PillarMold_Clear();
		void Evaluate_PC_S_Prepare_PillarMold_Black();
		void Evaluate_PC_S_Prepare_PillarMold_White();
		
		Position** _pillarMold_Black_S;
		Position** _pillarMold_White_S;

		int* _pillarMold_Black_SizeList;
		int* _pillarMold_White_SizeList;

		// 柱の本数、柱内の要素数：黒用
		int pillarListSizeMax_Black;     // 柱型、の件数の最大値
		int pillarListSize_Black;        // 柱型、の件数
		int pillarGridListSizeMax_Black; // 柱内の要素数の最大値（盤サイズの縦幅に等しい）

		// 柱の本数、柱内の要素数：白用
		int pillarListSizeMax_White;     // 柱型、の件数の最大値
		int pillarListSize_White;        // 柱型、の件数
		int pillarGridListSizeMax_White; // 柱内の要素数の最大値（盤サイズの縦幅に等しい）
		/*
			実際には黒白とも同じ値になるはず
		*/


		// 用意済みの型を使って、柱を作成
		void Evaluate_PC_S_MakePillarList_withPreparedMold ( 
			Position** &pillarMoldList, int attacker,
			const InnerExpression *innerExpression, Pillar* &pillarList
			, int &pillarMoldListSize   // 柱型リストの件数
			, int* &pillarMold_SizeList // 各柱のグリッド数
		);

		// 用意済みの型を使って、柱を作成：ついでに潜在的危険度も計算
		void Evaluate_PC_S_MakePillarList_withPreparedMold_and_CaliculatePotentialRiskDegree ( 
			Position** &pillarMoldList, int attacker,
			const InnerExpression *innerExpression, Pillar* &pillarList
			, int &pillarMoldListSize   // 柱型リストの件数
			, int* &pillarMold_SizeList // 各柱のグリッド数
			);
		int _PC_InvokeCount;
		
		// これまでの局面リスト：（千日手回避用に）（AIが自分で覚えておくものではない。「次の一手」の際の参考情報として事前に入れておくもの）
		vector<InnerExpression*> _recordList;
		void SetRecordList ( vector<InnerExpression*> arg ) { _recordList = arg; }
		
		bool _isResign; // 投了フラグ 
		
		// 最新の選択肢リストのメモ。通常は使わないが、選択肢バトルの際に選択肢を作成するための便宜としてアクセスできるようにしておく。
		vector< pair<Move, int> > _evaluatedMoveList_theLatest;
		
		
	private:

		// static AI *_self;

		Allocator<InnerExpression> *_allocator;


		//char *_memoryPool_ForTreeSearch;
		//char *_memoryPool_ForTreeSearch_currentIt;

		MemoryPool _memoryPool_ForTreeSearch;
		
		int _terrainEvalTable_Throuable[5][5];
		int _terrainEvalTable_GoUP     [5][5];
		int _terrainEvalTable_GoDOWN   [5][5];
		int _terrainEvalTable_GoRIGHT  [5][5];
		int _terrainEvalTable_GoLEFT   [5][5];
		int _terrainEvalTable_NotThrouable[5][5];

		float _terrainEval_hightBias; // 高さ重み値

		
		MemoryPool _memoryPool_ForTerrainSearch;
		
		
		//=========================================================
		// 木探索用メモリプールを操作する関数群
		//=========================================================
		/*
		void MemoryPool_ForTreeSearch_Setup    ( int numberOfInnerExpressions ); // プールを作る
		void MemoryPool_ForTreeSearch_Allocate ( InnerExpression* &memory );     // プールからメモリをください
		void MemoryPool_ForTreeSearch_Free     ( InnerExpression* &memory );     // プールにメモリをお返しします。
		void MemoryPool_ForTreeSearch_End      ();                               // プールを片付ける
		*/

		void MemoryPool_ForTreeSearch_Setup    ( MemoryPool &pool, int numberOfInnerExpressions ); // プールを作る
		void MemoryPool_ForTreeSearch_Allocate ( MemoryPool &pool, InnerExpression* &memory );     // プールからメモリをください
		void MemoryPool_ForTreeSearch_Free     ( MemoryPool &pool, InnerExpression* &memory );     // プールにメモリをお返しします。
		void MemoryPool_ForTreeSearch_End      ( MemoryPool &pool);                                // プールを片付ける


		//=========================================================
		// この局面から終局まで指す
		//【引数】InnerExpression
		//【戻値】0.どっちが勝ったか？ 1.かかった手数
		//=========================================================
		vector<int> GoToEnd(InnerExpression *innerExpression);

		//=========================================================
		// 掘り進む
		//【戻値】指し手と、評価値
		//=========================================================
		int _allReadState;
		pair<Move, int> DigInto( InnerExpression *currentState, int level, int fromThisTurnsViewPoint);
		
		///// オプション引数構造体
		struct DigInto_Window_OptionArgs {

			int  allReadState;           // 読んだ手数
			bool isDoShallowSearchPrune; // 最初に手を大雑把に絞り込む：デフォルト true
			bool isCurrentTerrainOnly;   // 取得する手を地形を変えないものだけに限定する；デフォルト false

			int evaluateFunctionFlgs;    // 評価関数として何を使うかを指定するフラグ

			DigInto_Window_OptionArgs() {
				allReadState           = 0;
				isDoShallowSearchPrune = true;
				isCurrentTerrainOnly   = false;

				evaluateFunctionFlgs = 
					EVALUATION::RUTIN::GAME_OVER
					//| EVALUATION::RUTIN::CONSIDER_ONES_TURN
					//| EVALUATION::RUTIN::VALUE_OF_CARDS
					| EVALUATION::RUTIN::VALUE_OF_PIECES
					| EVALUATION::RUTIN::VALUE_OF_CHECK
					//| EVALUATION::RUTIN::VALUE_OF_TERRAIN
					;
			}
		};
		pair<Move, int> DigInto_Window( MemoryPool &memoryPool, InnerExpression *currentState, int level, int fromThisTurnsViewPoint, int windowUpperLimit, int windowLowerLimit, DigInto_Window_OptionArgs &optionArgs);
		pair<Move, int> DigInto_Window( MemoryPool &memoryPool, vector<Move> &allMoveList, InnerExpression *currentState, int level, int fromThisTurnsViewPoint, int windowUpperLimit, int windowLowerLimit, DigInto_Window_OptionArgs &optionArgs);
		//pair<Move, int> DigInto_Window( InnerExpression *currentState, int level, int fromThisTurnsViewPoint, int windowUpperLimit, int windowLowerLimit, bool isDoShallowSearchPrune = true);
		//pair<Move, int> DigInto_Window( vector<Move> &allMoveList, InnerExpression *currentState, int level, int fromThisTurnsViewPoint, int windowUpperLimit, int windowLowerLimit, bool isDoShallowSearchPrune = true);

		//=========================================================
		// 掘り進む：詰み探索用
		//=========================================================
		//pair<int, vector<Move>> DigInto_Mate( InnerExpression *currentState, int level, int fromThisTurnsViewPoint, vector<Move> &route, int windowUpperLimit, int windowLowerLimit);
		//void DigInto_Mate( pair<int, vector<Move>> &result, InnerExpression *currentState, int level, int fromThisTurnsViewPoint, vector<Move> &route, int windowUpperLimit, int windowLowerLimit);
		pair<int, vector<Move>> DigInto_Mate( MemoryPool &memoryPool, InnerExpression *currentState, int level, int fromThisTurnsViewPoint, vector<Move> &route, int windowUpperLimit, int windowLowerLimit);
		void DigInto_Mate( MemoryPool &memoryPool, pair<int, vector<Move>> &result, InnerExpression *currentState, int level, int fromThisTurnsViewPoint, vector<Move> &route, int windowUpperLimit, int windowLowerLimit);

		//=========================================================
		// 詰み探索掘り進みスレッドに渡す引数用の構造体
		//=========================================================
		struct DigInto_Mate_Thread_Arg {

			AI *ai;
			InnerExpression *innerExpression;
			Move previousMove;
			int level; // もともとの深さ？ 一段降りた後の深さ？ じゃあ、一段降りた後の深さ。
			//bool isEnd;
		};
		Allocator<DigInto_Mate_Thread_Arg> *_allocator_DigInto_Mate_Thread_Arg;

		//=========================================================
		// 詰み探索掘り進みスレッド
		//=========================================================
		static unsigned int DigInto_Mate_Thread(void *obj);

		// スレッドハンドル
		//vector<HANDLE> _threadHandleList;
		static int _numberOfThread; // うーん、これもstatic？

#if defined(_WIN32)

		CRITICAL_SECTION _criticalSection_For_DigInto_Mate_Thread_Result; // Win API

#elif defined(__linux__)
		
		pthread_mutex_t _criticalSection_For_DigInto_Mate_Thread_Result; // Linux

#endif

		// スレッドに号令をかけるための変数：static でいいの？ 
		static bool _DigInto_Mate_Thread_Signal_isDiscoverMateProcedure;   // これがtrueになったら全スレッド中止（どこかで詰みが発見されたときに使う）

		// スレッド版掘り進みメソッド用の戻り値となる値：privateだけどAIクラス同士だからアクセスできるんでしたっけ？
		vector<Move> _DigInto_Mate_Thread_Result;


		//=========================================================
		// 次の一手を教えて：基本子処理：事前下見
		//=========================================================
		void GetNextMove_SubProcedure_Pre( vector<Move> &allMoveList, InnerExpression *innerExpression );

		void GetNextMove_SubProcedure_Pre_shallowSearch( vector<Move> &allMoveList, InnerExpression *innerExpression );
		void GetNextMove_SubProcedure_Pre_shallowSearch_SortCandidate( vector< pair<Move, int> > &point_CandidateMove);

		void GetNextMove_SubProcedure_Pre_avoid_Mated( vector<Move> &allMoveList, InnerExpression *innerExpression, int allMoveListSize );


		//=========================================================
		// 事前下見＿浅く読んで手を絞る＿ツリー探索内から呼ぶ前提のもの
		//=========================================================
		void ShallowSearch_Prune( vector<Move> &allMoveList, InnerExpression *innerExpression);
		

		// 駒の個数。「次の一手」を問い合わせられるたびに、現局面を見て確認する。
		int _pieceCount_black;
		int _pieceCount_white;
		
		
		void HandyProc_KakuninForDebug();
		
		static string Convert_throuableInfoEx_to_string ( int throughableInfoEx );
		static int    Convert_string_to_throuableInfoEx ( string str );
		static int    GetGameProgressDegree ( const InnerExpression *innerExpression );
		
		
		//=========================================================
		// 位置評価、方向別の共通処理
		//=========================================================
		void AI::Evaluate_Position_SubForEachDirection ( 
			AI::PositionValueData &result,
			const InnerExpression *innerExpression, const Position &pos,
			const int &myPiece, const int &opPiece, const int &mustToUpForThisTurn, const int &mustToDownForThisTurn,
			const vector<Position> &path, const Parameters_PositionEvaluation_Sub &paramSub, bool isUpperSide
		);
		
		
		//=========================================================
		// 圧力バランス判定関連
		//=========================================================
		void Evaluate_Move_By_PressureBalance_Init ( InnerExpression *innerExpression );
		int _securedColumnsNumForBlack;
		int _securedColumnsNumForWhite;
		int _gameProgress;
		bool Evaluate_Move_By_PressureBalance_IsWithinTheGameProgress ();
		void Evaluate_Move_By_PressureBalance_ShowSettingsForDebug();
		int  Evaluate_Move_By_PressureBalance_GetMoveBias ( const Move &move );
		
		int Evaluate_Move_By_PressureBalance_TypeColumn ( const InnerExpression *currentIe, const Move &move, const InnerExpression *nextIe, int forThisTurn);
		int Evaluate_Move_By_PressureBalance_TypeMirror ( const InnerExpression *currentIe, const InnerExpression *nextIe, int forThisTurn);
		
		bool IsMirrorState ( const InnerExpression *ie );
		
		void ShowPositionCareSettingsForDebug();
		
		
		//=========================================================
		// 距離判定関連
		//=========================================================
		int  Evaluate_DistanceToGoal_DistanceRate ( int distance );
		void Evaluate_DistanceToGoal_ShowSettingsForDebug();
		
		int _Evaluate_Move_by_Distance_currentMyDistance;
		int _Evaluate_Move_by_Distance_currentOpDistance;
		void Evaluate_Move_by_Distance_Init ( InnerExpression *currentIe );
		
		int GetMinDistanceToGoal ( InnerExpression *currentIe, int forThisTurn );
		
		InnerExpression *_ie_GetDistanceToGoal;
		int _passReminderYX_GetDistanceToGoal_Init[9][9];
		int _passReminderYX_GetDistanceToGoal[9][9];
		//int _passReminderY_GetDistanceToGoal[9];
		//set<int> _passedGridList_GetDistanceToGoal;
		//void GetDistanceToGoal_AddPass     ( const int &x, const int &y );
		//void GetDistanceToGoal_RemovePass  ( const int &x, const int &y );
		//bool GetDistanceToGoal_IsExistPass ( const int &x, const int &y );
		
		//int Evaluate_Move_by_Distance ( InnerExpression *currentIe, const Move &move, InnerExpression *nextIe, int forThisTurn );
		int Evaluate_DistanceDiff ( InnerExpression *nextIe, int forThisTurn );
		bool Evaluate_Move_by_Distance_isScreeningEnabled ( InnerExpression *currentIe );
		
		int Evaluate_Move_by_Distance_OneStepFutureTelling ( InnerExpression *nextIe, const vector<Move> &nextMoveList, int forThisTurn, bool isGetMaxOrMin );
		
		bool isEqual ( const Move &m1, const Move &m2 );
		string moveToUSTRING ( const Move &m );
		
		
		//=========================================================
		// 結果取得の際にランダム要素を入れる処理関連
		//=========================================================
		void PickResultFrom_ShowSettingsForDebug();
		int PickResultFrom_GetPickedIndex ( const vector< pair<Move, int> > &evaluatedMoveList, InnerExpression *innerExpression );
		
	};
};
#endif
