//=========================================================
//
// AI_実装ファイル  ツリーウィンドウ探索
//
//=========================================================

#include "pch.h"
#include <stdio.h>
#include "AI.h"
#include "InnerExpression.h"
#include "../Utils/Utils_Cal.h"
#include "../Utils/Allocator.h"
#include "../Utils/Utils_Others.h"


using namespace AI_Modules;

//=========================================================
// 掘り進む
//=========================================================
//pair<Move, int> AI::DigInto_Window( InnerExpression *currentState, int level, int fromThisTurnsViewPoint, int windowUpperLimit, int windowLowerLimit, bool isDoShallowSearchPrune) {
pair<Move, int> AI::DigInto_Window( MemoryPool &memoryPool, InnerExpression *currentState, int level, int fromThisTurnsViewPoint, int windowUpperLimit, int windowLowerLimit, DigInto_Window_OptionArgs &optionArgs) {

	///// 可能な指し手リストを出して
	vector<Move> allMoveList;
	//currentState->GetLegalNextMoveList( allMoveList );
	
	if (optionArgs.isCurrentTerrainOnly) {
		currentState->GetLegalNextMoveList( allMoveList, MOVE::ACTION::CURRENT_TERRAIN_ONLY );
	} else {
		currentState->GetLegalNextMoveList( allMoveList );
	}
	///// リレー
	return DigInto_Window( memoryPool, allMoveList, currentState, level, fromThisTurnsViewPoint, windowUpperLimit, windowLowerLimit, optionArgs); //, isDoShallowSearchPrune);
}
//pair<Move, int> AI::DigInto_Window( vector<Move> &allMoveList, InnerExpression *currentState, int level, int fromThisTurnsViewPoint, int windowUpperLimit, int windowLowerLimit, bool isDoShallowSearchPrune) {
pair<Move, int> AI::DigInto_Window( MemoryPool &memoryPool, vector<Move> &allMoveList, InnerExpression *currentState, int level, int fromThisTurnsViewPoint, int windowUpperLimit, int windowLowerLimit, DigInto_Window_OptionArgs &optionArgs) {

	/*
		現局面で、可能な指し手のうち、どの手が最善で・何点なのか？
		を出すためのルーチン。

		深さ 0 で呼んだ場合、
		「現局面で」可能な指し手のうち、どの手が最善で・何点なのか？
		を出すことになる。再帰はしない。
	*/

	pair<Move, int> result;
	result.second = 0;

	///// 可能な指し手リストを出して
	//vector<Move> allMoveList;
	//currentState->GetLegalNextMoveList( allMoveList );

	int bestEval = 0;

	
	// bestEvalの初期値：今からMAX読みをする
	if ( currentState->_currentTurn == fromThisTurnsViewPoint ) {

		bestEval = windowLowerLimit;

	// bestEvalの初期値：今からMIN読みをする
	} else {

		bestEval = windowUpperLimit;
	}

	///// 大雑把に絞り込む（まだ掘る必要がある場合のみ）
	//if (isDoShallowSearchPrune && level > 0) {
	if ( optionArgs.isDoShallowSearchPrune && level > 0) {
		ShallowSearch_Prune ( allMoveList, currentState );
	}

	///// 可能な指し手でループ
	for (unsigned int i = 0; i < allMoveList.size(); ++i) {

		//++_allReadState;
		optionArgs.allReadState += 1;

		// こう指すと、こうなる。
		Move move = allMoveList[i];
		//InnerExpression *nextState = currentState->GetNext( move );
		//InnerExpression *nextState = currentState->GetNext( move, _allocator);
		
		//InnerExpression *nextState = (InnerExpression*)_memoryPool_ForTreeSearch_currentIt;
		InnerExpression *nextState = (InnerExpression*)memoryPool.currentIt;
		currentState->GetCopy( nextState );
		nextState->GoNext( move );
		//_memoryPool_ForTreeSearch_currentIt += sizeof( InnerExpression );
		memoryPool.currentIt += sizeof( InnerExpression );

		// 評価値は…
		int eval = 0;

		// 勝負あり？
		bool isEnd = IsWinMove( move );

		// まだ掘る？
		pair<Move, int> digResult;
		if ( level > 0 && !isEnd) {

			//digResult =	DigInto( nextState, level - 1, fromThisTurnsViewPoint);
			

			// 今、MAX読みをしている → 次はMIN読みになりますよ
			if ( currentState->_currentTurn == fromThisTurnsViewPoint ) {

				digResult =	DigInto_Window( memoryPool, nextState, level - 1, fromThisTurnsViewPoint, windowUpperLimit, bestEval, optionArgs);

			// 今、MIN読みをしている → 次はMAX読みになりますよ
			} else {

				digResult =	DigInto_Window( memoryPool, nextState, level - 1, fromThisTurnsViewPoint, bestEval, windowLowerLimit, optionArgs );
			}

			eval = digResult.second;

		// もう掘らない → 評価：ルートプレイヤーの立場で
		} else {

			///// 引数で指定されている評価関数を呼ぶ：中盤用
			if ( optionArgs.evaluateFunctionFlgs == EVALUATION::RUTIN_SET::FOR_MIDDLE_STAGE ) {

				eval = Evaluate_Set_For_MiddleStage ( nextState, fromThisTurnsViewPoint );

			///// 引数で指定されている評価関数を呼ぶ：地形評価用
			} else if ( optionArgs.evaluateFunctionFlgs == EVALUATION::RUTIN_SET::FOR_TERRAIN_SEARCH ) {

				eval = Evaluate_Set_For_TerrainSearch ( nextState, fromThisTurnsViewPoint );
			}


			//if ( _allReadState % 10000 == 0 ) {	printf ( "%d 手目 評価値:%d\n", _allReadState, eval );}
			if ( optionArgs.allReadState % 10000 == 0 ) {	printf ( "%d 手目 評価値:%d\n", optionArgs.allReadState, eval );}
		}

		bool isPrune = false;
		// 枝狩り判定：今、MAX読みをしている
		if ( currentState->_currentTurn == fromThisTurnsViewPoint ) {

			if (eval >= windowUpperLimit) {
				isPrune = true;
			}

		// 枝狩り判定：今、MIN読みをしている
		} else {

			if (eval <= windowLowerLimit) {
				isPrune = true;
			}
		}
		// 枝狩りで打ち切り
		if ( isPrune) {

			result.first  = move;
			result.second = eval;

			// deleteすると重い。とりあえずなしで作業を進める。
			//delete nextState;
			// 最終的に _memoryPool_ForTerrainSearch ごと解放しているので、個別のdeleteは不要というつもり。本当かな？ くわばらくわばら。
			
			nextState->~InnerExpression();
			//InnerExpression::operator delete(nextState, _memoryPool_ForTreeSearch_currentIt);
			//_memoryPool_ForTreeSearch_currentIt -= sizeof( InnerExpression );
			InnerExpression::operator delete(nextState, memoryPool.currentIt);
			memoryPool.currentIt -= sizeof( InnerExpression );
			
			break;
		}
		// ベスト？
		bool isBest = false;

		// 自分の手番
		if ( currentState->_currentTurn == fromThisTurnsViewPoint ) {
		
			// MAXを求める
			isBest = ( eval > bestEval || i == 0 );

		// 相手の手番
		} else {

			// MINを求める
			isBest = ( eval < bestEval || i == 0 );
		}

		// ベスト値を更新？
		if ( isBest ) {

			bestEval = eval;
			result.first  = move;
			result.second = eval;
		}

		//delete nextState;

		nextState->~InnerExpression();
		//InnerExpression::operator delete(nextState, _memoryPool_ForTreeSearch_currentIt);
		//_memoryPool_ForTreeSearch_currentIt -= sizeof( InnerExpression );
		InnerExpression::operator delete(nextState, memoryPool.currentIt);
		memoryPool.currentIt -= sizeof( InnerExpression );
	}

	/*
	if (InnerExpression::_instanceList.size() > 1000) {
		InnerExpression::LeakRelease();
	}
	*/

	return result;
}

//=========================================================
// 事前下見＿浅く読んで手を絞る＿ツリー探索内から呼ぶ前提のもの
//=========================================================
void AI::ShallowSearch_Prune( vector<Move> &allMoveList, InnerExpression *innerExpression ) {

	///// 浅く読んで、見込みのある手を残し、見込みのない手を省く

	// 上位いくつを残す？
	int topN = 10;
	
	// 読み深度
	int depth = 0;
	//if      ( allMoveList.size() > 250 ) { depth = 0; topN = 100;}
	//else if ( allMoveList.size() >  50 ) { depth = 1; topN = 50;}
	//else if ( allMoveList.size() >   0 ) { depth = 2; topN = allMoveList.size();} // 悪い手ああれば省くだけ。
	
	///// すでに候補手が少なかったら、絞り込まない。
	if (allMoveList.size() <= topN) {
		return;
	}

	///// ここでシャッフルしてもいいはず。
	// 上位を残す処理のとき、同率順位で並び順が固定しないように。
	//UTILS_OTHERS::RandomShuffle<Move>( allMoveList );

	Allocator<InnerExpression> *allocator = new Allocator<InnerExpression>();
	allocator->Alloc(1500);

	// 可能な手・評価値(浅)
	vector< pair<Move, int> > point_CandidateMove;

	vector<Move> nextMoveList;
	for (unsigned int i = 0; i < allMoveList.size(); ++i) 
	{

		InnerExpression *nextState = innerExpression->GetNext( allMoveList[i], allocator );

		//_allReadState = 0;

		nextMoveList.clear();
		nextState->GetLegalNextMoveList(nextMoveList);

		pair<Move, int> searchResult; // = DigInto_Window ( nextMoveList, nextState, depth, nextState->_currentTurn, 99999, -99999);
		//printf("評価値：%d 全 %d 手\n", searchResult.second, _allReadState);

		// 評価する。
		//searchResult.second = Evaluate( nextState, innerExpression->_currentTurn );
		searchResult.second = Evaluate_Set_For_PickUp ( nextState, innerExpression->_currentTurn );

		// この手を指した場合、ということでしたよね、もともと。
		searchResult.first = allMoveList[i];

		// 相手の番での評価値になっているから、符号を入れ替える
		//searchResult.second *= -1;

		//printf("%d / %d  評価値 : %d\n", i, allMoveList.size(), searchResult.second);

		// 一時リストに入れる：ソートは後回しで。
		point_CandidateMove.push_back( searchResult );

	}
	// ソート
	GetNextMove_SubProcedure_Pre_shallowSearch_SortCandidate(point_CandidateMove);

	delete allocator;

	///// 上位を残す。
	vector< pair<Move, int> > point_CandidateMove_topN;
	for (unsigned int i = 0; i < point_CandidateMove.size() && i < topN; ++i) {

		point_CandidateMove_topN.push_back ( point_CandidateMove[i] );
	}
	//printf("残り %d 件\n", point_CandidateMove_topN.size());


	///// あまりにも悪い手は省く（たとえ上位に残っていたとしても）
	//printf("極端に悪い手を省く\n");
	int uncertificatablePoint = -5000;


	vector< pair<Move, int> > point_CandidateMove_topN_noBad;

	for (unsigned int i = 0; i < point_CandidateMove_topN.size(); ++i) {

		// 耐える点数を下回ったら、ループ終了
		if ( point_CandidateMove_topN[i].second < uncertificatablePoint ) {
			break;
		}

		// 候補に残す
		point_CandidateMove_topN_noBad.push_back ( point_CandidateMove_topN[i] );
	}
		
	// 候補リストを置き換える：悪い手しか残ってない場合は、置き換えない。
	if ( point_CandidateMove_topN_noBad.size() > 0 ) {
		point_CandidateMove_topN = point_CandidateMove_topN_noBad;
	} else {
		//printf("悪い手しか残ってなかったので置き換えない\n");
	}

	//printf("残り %d 件\n", point_CandidateMove_topN.size());
	

	///// 絞り込み結果を反映させる
	allMoveList.clear();
	for ( unsigned int i = 0; i < point_CandidateMove_topN.size(); ++i ) {

		allMoveList.push_back( point_CandidateMove_topN[i].first );
	}

	//printf("Shallow探索終了\n");


}


