//=========================================================
//
// AI_実装ファイル_評価関数関係
//
//=========================================================
#include "pch.h"

#include <stdio.h>
#include "AI.h"
#include "InnerExpression.h"
#include "../Utils/Utils_Cal.h"
#include "../Utils/Allocator.h"


using namespace AI_Modules;


//==========================================================
// 手筋の評価
// move そもそも合法な手が来ている前提
//==========================================================
int AI::Evaluate_HandyProc ( const InnerExpression* currentIe, Move move, const InnerExpression* nextIe ) {
	
	int result = 0;
	
	//string m = ShowMoveToConsole ( move, true);
	//printf ( "handyProc (%s) eval \n", m.c_str() );
	
	// 現在の手番
	int currentTurn = currentIe->_currentTurn;
	
	// 現在の進行度合い
	int gameProgressDegree = GetGameProgressDegree ( currentIe );
	
	//printf ( " gameProgressDegree[%d]\n ", gameProgressDegree );
	
	// 手筋でループ
	for ( unsigned int i = 0; i < _parameters_PieceEvaluation.handyProcList.size(); ++i ) {
		
		HandyProc handyProc = _parameters_PieceEvaluation.handyProcList[i];
		
		// 進行度が範囲内かどうか
		if ( handyProc.gameProgressFrom > gameProgressDegree ) { continue;  }
		if ( handyProc.gameProgressTo >= 0 &&  handyProc.gameProgressTo < gameProgressDegree ) { continue; }
		
		//printf ( "handyProc.gameProgressFrom[%d], handyProc.gameProgressTo[%d]\n", handyProc.gameProgressFrom, handyProc.gameProgressTo );
		
		
		// 対象となる操作かどうか
		if ( move.action != handyProc.action ) {
			continue;
		}
		
		// カード設置で、カードIDが指定されている場合
		if ( move.action == MOVE::ACTION::PUT_CARD && handyProc.actionCardId != "" ) {
			
			if (_cardNameList [ move.putCardId ] != handyProc.actionCardId ) {
				continue;
			}
		}
		// カード設置で、方向が指定されている場合
		if ( move.action == MOVE::ACTION::PUT_CARD && handyProc.actionCardDirection != DIRECTION::NONE ) {
			
			int putCardDirection = move.putCardDirection;
			
			if ( currentTurn == TURN::WHITE ) {
				if      ( putCardDirection == DIRECTION::UP   ) { putCardDirection = DIRECTION::DOWN; }
				else if ( putCardDirection == DIRECTION::DOWN ) { putCardDirection = DIRECTION::UP; }
			}
			
			if ( putCardDirection != handyProc.actionCardDirection ) {
				continue;
			}
		}
		
		//printf ( "move.action[%d] == handyProc.action[%d]\n", move.action, handyProc.action );
		
		// 手番に合わせて上下を入れ替えておく必要がある場合のもの
		if ( currentTurn == TURN::WHITE ) {
			
			// 矢印の上下を逆にする
			if      ( handyProc.actionAffectsStatus == THROUGHABLE_INFO::EX::MUST_TO_UP   ) { handyProc.actionAffectsStatus = THROUGHABLE_INFO::EX::MUST_TO_DOWN; } 
			else if ( handyProc.actionAffectsStatus == THROUGHABLE_INFO::EX::MUST_TO_DOWN ) { handyProc.actionAffectsStatus = THROUGHABLE_INFO::EX::MUST_TO_UP; } 
			
			// 相対位置の Y を逆にする
			handyProc.actionAffectsY *= -1;
		}
		else if ( currentTurn == TURN::BLACK ) {
			
			// 盤面サーチ範囲の Y を逆にする
			handyProc.gridFromY = _rule.boardHeight - 1 - handyProc.gridFromY;
			handyProc.gridToY   = _rule.boardHeight - 1 - handyProc.gridToY;
			
		}
		
		// 盤面全体をサーチする際の指定範囲
		/*
			Y に関して、自分の側の最も手前を 0 と換算する
			その上で、実際にサーチする範囲を求める
		*/
		int boardFromX = handyProc.gridFromX;
		int boardFromY = handyProc.gridFromY;
		int boardToX   = handyProc.gridToX;
		int boardToY   = handyProc.gridToY;
		
		// from < to であるようにする
		if ( boardFromX > boardToX ) { int t = boardFromX; boardFromX = boardToX; boardToX = t;}
		if ( boardFromY > boardToY ) { int t = boardFromY; boardFromY = boardToY; boardToY = t;}
		
		bool isConditionMet = false;
		
		// 盤面全体をサーチ（指定された範囲で）
		for ( unsigned int y = boardFromY; y <= boardToY; ++y ) { if ( isConditionMet ) { break; }
			
			for ( unsigned int x = boardFromX; x <= boardToX; ++x ) {
				
				//printf ( "board ( %d, %d )  range(%d,%d)-(%d,%d)\n", x, y, boardFromX, boardFromY, boardToX, boardToY);
				
				
				// マス目の条件が全部合っていたかどうか
				bool isAllGridValid = true;
				
				
				// マス目の条件でループ
				for ( unsigned int n = 0; n < handyProc.gridStatusList.size(); ++n ) {
					
					HandyProc_GridStatus gridStatus = handyProc.gridStatusList[n];
					
					//printf ( "HandyProc_GridStatus (%d, %d) : [%s]\n" , gridStatus.x, gridStatus.y, Convert_throuableInfoEx_to_string(gridStatus.THROUGHABLE_INFO_EX).c_str());
					
					// 条件に合っているかどうか
					bool isValid = false;
					
					// 手番に合わせて上下を入れ替えておく必要がある場合のもの
					if ( currentTurn == TURN::WHITE ) {
						
						// 矢印の上下を逆にする
						if      ( gridStatus.THROUGHABLE_INFO_EX == THROUGHABLE_INFO::EX::MUST_TO_UP   ) { gridStatus.THROUGHABLE_INFO_EX = THROUGHABLE_INFO::EX::MUST_TO_DOWN; } 
						else if ( gridStatus.THROUGHABLE_INFO_EX == THROUGHABLE_INFO::EX::MUST_TO_DOWN ) { gridStatus.THROUGHABLE_INFO_EX = THROUGHABLE_INFO::EX::MUST_TO_UP; } 
						
						// 相対位置の Y を逆にする
						gridStatus.y *= -1;
					}
					
					int targetX = x + gridStatus.x;
					int targetY = y + gridStatus.y;
					
					
					// 盤面の外側
					if ( targetX < 0 || targetX >= _rule.boardWidth || targetY < 0 || targetY >= _rule.boardHeight ) {
						
						if ( gridStatus.THROUGHABLE_INFO_EX == THROUGHABLE_INFO::EX::OUT_OF_BOARD ) {
							isValid = true;
						}
						//printf ( "out of board targetXY(%d, %d) isValid [%s]\n", targetX, targetY,isValid?"true":"false" );
						
					// 盤面の内側
					} else {
						
						// 条件が通行条件に関するものである場合
						if ( gridStatus.THROUGHABLE_INFO_EX >= THROUGHABLE_INFO::EX::NOT_THROUGHABLE && gridStatus.THROUGHABLE_INFO_EX <= THROUGHABLE_INFO::EX::MUST_TO_DOWN ) {
							
							//printf ( "a\n" );
							
							if ( currentIe->_gridList_ThrouableInfo[targetY][targetX] == gridStatus.THROUGHABLE_INFO_EX) {
								isValid = true;
							}
							
						// 条件が駒に関するものである場合
						} else if ( gridStatus.THROUGHABLE_INFO_EX >= THROUGHABLE_INFO::EX::PIECE_EMPTY && gridStatus.THROUGHABLE_INFO_EX <= THROUGHABLE_INFO::EX::PIECE_OPPONENT ) {
							
							//printf ( "b\n" );
							// GRID { ENPTY  BLACK   WHITE } 
							int pieceInfo = currentIe->_gridList_Piece[targetY][targetX];
							
							
							// 駒がないこと、という条件の場合
							if ( gridStatus.THROUGHABLE_INFO_EX == THROUGHABLE_INFO::EX::PIECE_EMPTY ) { 
								
								if ( pieceInfo == GRID::ENPTY ) {
									isValid = true; 
								}
							}
							// 黒番の場合
							else if ( currentTurn == TURN::BLACK ) {
								
								if ( pieceInfo == GRID::BLACK  ) {
									if ( gridStatus.THROUGHABLE_INFO_EX == THROUGHABLE_INFO::EX::PIECE_MY ) { isValid = true;  }
								}
								else if ( pieceInfo == GRID::WHITE  ) {
									if ( gridStatus.THROUGHABLE_INFO_EX == THROUGHABLE_INFO::EX::PIECE_OPPONENT ) { isValid = true;  }
								}
								
							// 白番の場合
							} else if ( currentTurn == TURN::WHITE ) {
								
								if ( pieceInfo == GRID::WHITE  ) {
									if ( gridStatus.THROUGHABLE_INFO_EX == THROUGHABLE_INFO::EX::PIECE_MY ) { isValid = true;  }
								}
								else if ( pieceInfo == GRID::BLACK  ) {
									if ( gridStatus.THROUGHABLE_INFO_EX == THROUGHABLE_INFO::EX::PIECE_OPPONENT ) { isValid = true;  }
								}
							}
						}
						
						// 条件：相手陣
						else if ( gridStatus.THROUGHABLE_INFO_EX == THROUGHABLE_INFO::EX::OPPONENT_AREA ) {
							
							//printf ( "c\n" );
							// 黒番の場合
							if ( currentTurn == TURN::BLACK ) {
								
								if ( targetY == 0 ) {
									isValid = true;
								}
							}
							// 白番の場合
							else if ( currentTurn == TURN::WHITE ) {
								
								if ( targetY == _rule.boardHeight - 1 ) {
									isValid = true;
								}
							}
							
							//printf ( "gridStatus.THROUGHABLE_INFO_EX == THROUGHABLE_INFO::EX::OPPONENT_AREA isValid[%s]\n", (isValid)?"true":"false" );
						}
						// 条件：カード取得可能
						else if ( gridStatus.THROUGHABLE_INFO_EX == THROUGHABLE_INFO::EX::CARD_TAKEABLE ) {
							
							//printf ( "d\n" );
							int                         pieceInfo = currentIe->_gridList_Piece[targetY][targetX];
							InnerExpression::CardOnBoard cardInfo  = currentIe->_gridList_Card[targetY][targetX];
							
							// 黒番の場合
							if ( currentTurn == TURN::BLACK ) {
								
								if ( pieceInfo == GRID::BLACK && cardInfo.cardId != -1 ) {
									isValid = true;
								}
							}
							// 白番の場合
							else if ( currentTurn == TURN::WHITE ) {
								
								if ( pieceInfo == GRID::WHITE && cardInfo.cardId != -1 ) {
									isValid = true;
								}
							}
						
						// どれでもなかった。何かがおかしい
						} else {
							//printf ( " sonotano gridStatus.THROUGHABLE_INFO_EX [%s] \n", Convert_throuableInfoEx_to_string ( gridStatus.THROUGHABLE_INFO_EX ).c_str() );
						}
						
						
						// １つでも条件に合わないマス目があったら、そこで打ち切る
						if ( !isValid ) {
							
							//printf ( "end by not met \n" );
							
							isAllGridValid = false;
							break;
						}
						
					}
					
					
				
				} // マス目の条件
				
				// 全部のマス目の条件が合っていた？
				if ( !isAllGridValid ) {
					
					// 全部は合ってなかった
					continue;
				}
				
				//printf ( "gridListStatus all conditions met\n" );
				
				////////////////////
				// 指し手の条件に合うかどうか
				bool isMoveConditionMet = false;
				do {
					
					int targetX = x + handyProc.actionAffectsX;
					int targetY = y + handyProc.actionAffectsY;
					
					// 盤面の範囲内かどうかを確認
					if ( targetX < 0 || targetX >= _rule.boardWidth || targetY < 0 || targetY >= _rule.boardHeight ) {
						break;
					}
					
					// Action：駒設置の場合
					if ( move.action == MOVE::ACTION::PUT_PIECE ) {
						
						// 指定位置への設置じゃないなら対象外
						if ( move.putPieceX != targetX || move.putPieceY != targetY ) {
							break;
						}
					}
					// Action：駒移動の場合
					else if ( move.action == MOVE::ACTION::MOVE_PIECE ) {
						
						// 指定位置の駒の状態が合っているかどうか
						
						// 空白指定
						if ( handyProc.actionAffectsStatus == THROUGHABLE_INFO::EX::PIECE_EMPTY ) {
						
							if ( nextIe->_gridList_Piece[targetY][targetX] != GRID::ENPTY ) {
								break;
							}
							
						}
						// 自駒指定
						else if ( handyProc.actionAffectsStatus == THROUGHABLE_INFO::EX::PIECE_MY ) {
							
							if ( currentTurn == TURN::BLACK ) {
								
								if ( nextIe->_gridList_Piece[targetY][targetX] != GRID::BLACK ) {
									break;
								}
							}
							else if ( currentTurn == TURN::WHITE ) {
								
								if ( nextIe->_gridList_Piece[targetY][targetX] != GRID::WHITE ) {
									break;
								}
								
							}
						}
						// 相手駒指定
						else if ( handyProc.actionAffectsStatus == THROUGHABLE_INFO::EX::PIECE_OPPONENT ) {
							
							if ( currentTurn == TURN::BLACK ) {
								
								if ( nextIe->_gridList_Piece[targetY][targetX] != GRID::WHITE ) {
									break;
								}
							}
							else if ( currentTurn == TURN::WHITE ) {
								
								if ( nextIe->_gridList_Piece[targetY][targetX] != GRID::BLACK ) {
									break;
								}
								
							}
						}
						
					}
					// Action：カード設置の場合
					else if ( move.action == MOVE::ACTION::PUT_CARD ) {
						
						// カード取得可能、という指定の場合
						if ( handyProc.actionAffectsStatus == THROUGHABLE_INFO::EX::CARD_TAKEABLE ) {
							
							// 指定位置にカードと自分の駒があるかどうか
							
							InnerExpression::CardOnBoard cardInfo = nextIe->_gridList_Card[targetY][targetX];
							int                         pieceInfo = nextIe->_gridList_Piece[targetY][targetX];
							
							if ( pieceInfo == GRID::BLACK && cardInfo.cardId != -1 ) {
								
								if ( currentTurn != TURN::BLACK ) { break; }
							}
							else if ( pieceInfo == GRID::WHITE && cardInfo.cardId != -1 ) {
								
								if ( currentTurn != TURN::WHITE ) { break; }
							}
							
						// 通行可能状況についての指定の場合
						} else {
						
							// 指定位置の通行可能状態が合っているかどうか
							if ( nextIe->_gridList_ThrouableInfo[targetY][targetX] != handyProc.actionAffectsStatus) {
								break;
							}
							
						}
					}
					// Action：カード取得の場合
					else if ( move.action == MOVE::ACTION::TAKE_CARD ) {
						
						// 取得する位置、ではなく、取得の結果の、指定位置の通行可能状況、を見る
						if ( nextIe->_gridList_ThrouableInfo[targetY][targetX] != handyProc.actionAffectsStatus) {
							break;
						}
						
					// それ以外の動作
					} else {
						break;
					}
					
					isMoveConditionMet = true;
				
				} while (false);
				
				
				if ( isMoveConditionMet ) {
				
					printf ("move condition met (move : %s) POINT [%d]\n", ShowMoveToConsole ( move, true).c_str(), handyProc.point);
					
					int point = handyProc.point;
					
					// 条件が合った、ここで位置評価値を計算：指す前？指した後？ 指す前で
					for ( unsigned int j = 0; j < handyProc.positionBiasList.size(); ++j ) {
						
						HandyProc_PositionBias positionBias = handyProc.positionBiasList[j];
						
						AI::PositionValueData positionValue = Evaluate_Position ( currentIe, Position( x + positionBias.x, y + positionBias.y), currentTurn );
						
						printf ("positionBias(%d, %d) advantage[%d]*%d%, risk[%d]%d%\n", x + positionBias.x, y + positionBias.y, positionValue.advantage, positionBias.advantageBias, positionValue.risk, positionBias.riskBias);
						
						point += positionValue.advantage * positionBias.advantageBias / 100;
						point += positionValue.risk      * positionBias.riskBias      / 100;
						
					}
					
					result += point;
					
					// １つでも条件に合うのがあれば、もう終わっていいハズ
					// いや、一手で条件を２つ以上同時に満たすという場合もある。したがって、加算していくこととする
					//return result;
					
					// 同じ形が盤面の別の箇所にもあるかもしれない。
					// が、そちらを今回の指し手で満たせる可能性はないハズ。
					// ゆえに、この段階で盤面のxyサーチは break ということで問題ないハズ。
					
					isConditionMet = true;
					break;
				}
				
			} // x
		} // y
		
		
		
		
	}
	
	
	return result;
}









//=========================================================
// ゲームの進行度合いを数える
//
// 何かが入っているマス目の数、で表す。ターン数ではない。
// 駒、または通り道（カード自体ではなく） の数。→ 効果範囲つきのカードを置く方が「進行速度が早い」という判定になる
//
//=========================================================
int AI::GetGameProgressDegree ( const InnerExpression *innerExpression ) {
	
	int result = 0;
	
	for ( unsigned int y = 0; y < _rule.boardHeight; ++y ) {
		for ( unsigned int x = 0; x < _rule.boardWidth; ++x ) {
			
			if ( innerExpression->_gridList_Piece[y][x] != GRID::ENPTY ) {
				++result;
			}
			if ( innerExpression->_gridList_ThrouableInfo[y][x] != THROUGHABLE_INFO::EX::NOT_THROUGHABLE ) {
				++result;
			}
		}
		
	}
	
	
	return result;
}









//==========================================================
// 手筋設定を読み込めているかどうかの確認
//==========================================================
void AI::HandyProc_KakuninForDebug() {
	
	printf ( "\nAI::HandyProc_KakuninForDebug() start \n\n" );
	
	for ( unsigned int i = 0; i < _parameters_PieceEvaluation.handyProcList.size(); ++i ) {
		
		printf ("<HANDY_PROC>\n");
		
		HandyProc handyProc = _parameters_PieceEvaluation.handyProcList[i];
		
		printf ( "\t<GAME_PROGRESS from=\"%d\" to=\"%d\" />\n", handyProc.gameProgressFrom, handyProc.gameProgressTo );
		
		string action = "";
		if       ( handyProc.action == MOVE::ACTION::PUT_PIECE  ) { action = "PUT_PIECE"; }
		else if ( handyProc.action == MOVE::ACTION::MOVE_PIECE ) { action = "MOVE_PIECE"; }
		else if ( handyProc.action == MOVE::ACTION::PUT_CARD   ) { action = "PUT_CARD"; }
		else if ( handyProc.action == MOVE::ACTION::TAKE_CARD  ) { action = "TAKE_CARD"; }

		string status = Convert_throuableInfoEx_to_string ( handyProc.actionAffectsStatus );
		
		string cardDirection = "NONE";
		if      ( handyProc.actionCardDirection == DIRECTION::LEFT  ) { cardDirection = "LEFT"; }
		else if ( handyProc.actionCardDirection == DIRECTION::RIGHT ) { cardDirection = "RIGHT"; }
		else if ( handyProc.actionCardDirection == DIRECTION::UP    ) { cardDirection = "UP"; }
		else if ( handyProc.actionCardDirection == DIRECTION::DOWN  ) { cardDirection = "DOWN"; }
		
		printf ( "\t<MOVE action=\"%s\" x=\"%d\" y=\"%d\" status=\"%s\" cardId=\"%s\" cardDirection=\"%s\" />\n", action.c_str(), handyProc.actionAffectsX, handyProc.actionAffectsY, status.c_str(), handyProc.actionCardId.c_str(), cardDirection.c_str());
		
		printf ( "\t<POINT>%d</POINT>\n", handyProc.point );
		
		printf ( "\t<GRID_STATUS_LIST>\n" );
		
		for ( unsigned int n = 0;n < handyProc.gridStatusList.size(); ++n ) {
			
			HandyProc_GridStatus gridStatus = handyProc.gridStatusList[n];
			
			string throuableInfo = Convert_throuableInfoEx_to_string ( gridStatus.THROUGHABLE_INFO_EX );
			
			printf ( "\t\t<GRID x=\"%d\"  y=\"%d\" THROUGHABLE_INFO_EX=\"%s\" />\n", gridStatus.x, gridStatus.y, throuableInfo.c_str());
			
		}
		
		printf ( "\t</GRID_STATUS_LIST>\n" );
		
		for ( unsigned int n = 0; n < handyProc.positionBiasList.size(); ++n ) {
			
			HandyProc_PositionBias positionBias = handyProc.positionBiasList[n];
			
			printf ( "\t<POSITION_BIAS x=\"%d\"  y=\"%d\" advantage=\"%d\" risk=\"%d\" />\n", positionBias.x, positionBias.y, positionBias.advantageBias, positionBias.riskBias);
			
		}
		
		printf ("</HANDY_PROC>\n");
	}
	
	printf ( "\nAI::HandyProc_KakuninForDebug() end \n\n" );
	
}



//=========================================================
// 変換：THROUGHABLE_INFO::EX -> string
//=========================================================
string AI::Convert_throuableInfoEx_to_string ( int throughableInfoEx ) {
	
	string result = "";
	
	if      ( throughableInfoEx == THROUGHABLE_INFO::EX::NOT_THROUGHABLE  ) { result = "NOT_THROUGHABLE"; }
	else if ( throughableInfoEx == THROUGHABLE_INFO::EX::THROUGHABLE      ) { result = "THROUGHABLE"    ; }
	else if ( throughableInfoEx == THROUGHABLE_INFO::EX::MUST_TO_RIGHT    ) { result = "MUST_TO_RIGHT"  ; }
	else if ( throughableInfoEx == THROUGHABLE_INFO::EX::MUST_TO_LEFT     ) { result = "MUST_TO_LEFT"   ; }
	else if ( throughableInfoEx == THROUGHABLE_INFO::EX::MUST_TO_UP       ) { result = "MUST_TO_UP"     ; }
	else if ( throughableInfoEx == THROUGHABLE_INFO::EX::MUST_TO_DOWN     ) { result = "MUST_TO_DOWN"   ; }
	else if ( throughableInfoEx == THROUGHABLE_INFO::EX::PIECE_EMPTY      ) { result = "PIECE_EMPTY"    ; }
	else if ( throughableInfoEx == THROUGHABLE_INFO::EX::PIECE_MY         ) { result = "PIECE_MY"       ; }
	else if ( throughableInfoEx == THROUGHABLE_INFO::EX::PIECE_OPPONENT   ) { result = "PIECE_OPPONENT" ; }
	else if ( throughableInfoEx == THROUGHABLE_INFO::EX::OUT_OF_BOARD     ) { result = "OUT_OF_BOARD"   ; }
	else if ( throughableInfoEx == THROUGHABLE_INFO::EX::OPPONENT_AREA    ) { result = "OPPONENT_AREA"  ; }
	else if ( throughableInfoEx == THROUGHABLE_INFO::EX::CARD_TAKEABLE    ) { result = "CARD_TAKEABLE"  ; }
	
	return result;
}

//=========================================================
// 変換：string -> THROUGHABLE_INFO::EX
//=========================================================
int    AI::Convert_string_to_throuableInfoEx ( string str ) {
	
	int result = 0;
	
	if       ( str == "NOT_THROUGHABLE"  ) { result = THROUGHABLE_INFO::EX::NOT_THROUGHABLE  ; }
	else if ( str == "THROUGHABLE"      ) { result = THROUGHABLE_INFO::EX::THROUGHABLE      ; }
	else if ( str == "MUST_TO_RIGHT"    ) { result = THROUGHABLE_INFO::EX::MUST_TO_RIGHT    ; }
	else if ( str == "MUST_TO_LEFT"     ) { result = THROUGHABLE_INFO::EX::MUST_TO_LEFT     ; }
	else if ( str == "MUST_TO_UP"       ) { result = THROUGHABLE_INFO::EX::MUST_TO_UP       ; }
	else if ( str == "MUST_TO_DOWN"     ) { result = THROUGHABLE_INFO::EX::MUST_TO_DOWN     ; }
	else if ( str == "PIECE_EMPTY"      ) { result = THROUGHABLE_INFO::EX::PIECE_EMPTY      ; }
	else if ( str == "PIECE_MY"         ) { result = THROUGHABLE_INFO::EX::PIECE_MY         ; }
	else if ( str == "PIECE_OPPONENT"   ) { result = THROUGHABLE_INFO::EX::PIECE_OPPONENT   ; }
	else if ( str == "OUT_OF_BOARD"     ) { result = THROUGHABLE_INFO::EX::OUT_OF_BOARD     ; }
	else if ( str == "OPPONENT_AREA"    ) { result = THROUGHABLE_INFO::EX::OPPONENT_AREA    ; }
	else if ( str == "CARD_TAKEABLE"    ) { result = THROUGHABLE_INFO::EX::CARD_TAKEABLE    ; }

	return result;
}






