//=========================================================
//
// AI_実装ファイル_評価関数関係
//
//=========================================================
#include "pch.h"

#include <stdio.h>
#include "AI.h"
#include "InnerExpression.h"
#include "../Utils/Utils_Cal.h"
#include "../Utils/Allocator.h"


using namespace AI_Modules;


//==========================================================
// 位置の危険度と有利さを判定 
//==========================================================
AI::PositionValueData AI::Evaluate_Position ( const InnerExpression *innerExpression, Position pos ) {
	
	int forThisTurn = innerExpression->_currentTurn;
	return Evaluate_Position ( innerExpression, pos, forThisTurn);
	
}
//==========================================================
// 位置の危険度と有利さを判定
//==========================================================
AI::PositionValueData AI::Evaluate_Position ( const InnerExpression *innerExpression, Position pos, const int forThisTurn ) {
	
	//printf ( "AI::Evaluate_Position Start (%d, %d), Turn[%d]\n", pos.x, pos.y, forThisTurn );
	
	const Parameters_PositionEvaluation &param = _parameters_PieceEvaluation.parameters_PositionEvaluation;
	
	AI::PositionValueData result;
	
	///// 自分にとって「上」側の経路
	vector<Position> upperLeftPath;
	vector<Position> upperFrontPath;
	vector<Position> upperRightPath;
	
	///// 自分にとって「下」側の経路
	vector<Position> underLeftPath;
	vector<Position> underFrontPath;
	vector<Position> underRightPath;
	
	
	///// 自分にとっての「上矢印」と「下矢印」
	int mustToUpForThisTurn   = THROUGHABLE_INFO::EX::MUST_TO_UP;
	int mustToDownForThisTurn = THROUGHABLE_INFO::EX::MUST_TO_DOWN;
	
	if ( forThisTurn == TURN::WHITE ) {
		mustToUpForThisTurn   = THROUGHABLE_INFO::EX::MUST_TO_DOWN;
		mustToDownForThisTurn = THROUGHABLE_INFO::EX::MUST_TO_UP;
	}
	
	///// 自分の駒、相手の駒
	int myPiece = GRID::BLACK;
	int opPiece = GRID::WHITE;
	if ( forThisTurn == TURN::WHITE ) {
		myPiece = GRID::WHITE;
		opPiece = GRID::BLACK;
	}
	
	///// 対象とする手番に合わせて各方向の経路を求める
	if ( forThisTurn == TURN::BLACK ) {
	
		int c = 0; // カウント用メモ 
		
		///// 上側の経路を作成
		for ( int y = pos.y; y >= 0; --y) {
			
			// 根本はどれも指定位置自身となる。
			upperFrontPath.push_back ( Position ( pos.x, y ) );
			if ( pos.x - c >= 0                ) { upperLeftPath.push_back ( Position ( pos.x - c, y ) ); }
			if ( pos.x + c < _rule.boardWidth ) { upperRightPath.push_back ( Position ( pos.x + c, y ) ); }
			
			++c;
		}
		
		///// 下側の経路を作成
		c = 0; // カウント用メモ 
		for ( int y = pos.y; y < _rule.boardHeight; ++y) {
			
			// 根本はどれも指定位置自身となる。
			underFrontPath.push_back ( Position ( pos.x, y ) );
			if ( pos.x - c >= 0                ) { underLeftPath.push_back ( Position ( pos.x - c, y ) ); }
			if ( pos.x + c < _rule.boardWidth ) { underRightPath.push_back ( Position ( pos.x + c, y ) ); }
			
			++c;
		}
	}
	else if ( forThisTurn == TURN::WHITE ) {
		
		int c = 0; // カウント用メモ 
		
		///// 下側の経路を作成
		for ( int y = pos.y; y >= 0; --y) {
			
			// 根本はどれも指定位置自身となる。
			underFrontPath.push_back ( Position ( pos.x, y ) );
			if ( pos.x - c >= 0 ) { underLeftPath.push_back ( Position ( pos.x - c, y ) ); }
			if ( pos.x + c < _rule.boardWidth ) { underRightPath.push_back ( Position ( pos.x + c, y ) ); }
			
			++c;
		}
		
		///// 上側の経路を作成
		c = 0; // カウント用メモ 
		for ( int y = pos.y; y < _rule.boardHeight; ++y) {
			
			// 根本はどれも指定位置自身となる。
			upperFrontPath.push_back ( Position ( pos.x, y ) );
			if ( pos.x - c >= 0                ) { upperLeftPath.push_back ( Position ( pos.x - c, y ) ); }
			if ( pos.x + c < _rule.boardWidth ) { upperRightPath.push_back ( Position ( pos.x + c, y ) ); }
			
			++c;
		}
	
	} else {
		// おかしいですね
		return result;
	}
	
	bool UPPER = true;
	bool UNDER = false;
	
	///// 経路ごとに計算
	Evaluate_Position_SubForEachDirection (result, innerExpression, pos, myPiece, opPiece, mustToUpForThisTurn, mustToDownForThisTurn, upperFrontPath, param.upperFront, UPPER );
	Evaluate_Position_SubForEachDirection (result, innerExpression, pos, myPiece, opPiece, mustToUpForThisTurn, mustToDownForThisTurn, underFrontPath, param.underFront, UNDER );
	Evaluate_Position_SubForEachDirection (result, innerExpression, pos, myPiece, opPiece, mustToUpForThisTurn, mustToDownForThisTurn, upperLeftPath, param.upperLeft, UPPER );
	Evaluate_Position_SubForEachDirection (result, innerExpression, pos, myPiece, opPiece, mustToUpForThisTurn, mustToDownForThisTurn, underLeftPath, param.underLeft, UNDER );
	Evaluate_Position_SubForEachDirection (result, innerExpression, pos, myPiece, opPiece, mustToUpForThisTurn, mustToDownForThisTurn, upperRightPath, param.upperRight, UPPER );
	Evaluate_Position_SubForEachDirection (result, innerExpression, pos, myPiece, opPiece, mustToUpForThisTurn, mustToDownForThisTurn, underRightPath, param.underRight, UNDER );
	
	
	// バイアスをかける
	result.advantage = result.advantage * param.attackBias / 100;
	result.risk      = result.risk * param.guardBias / 100;
	
	
	return result;
}



//=========================================================
// 位置評価、方向別の共通処理
//=========================================================
void AI::Evaluate_Position_SubForEachDirection ( 
	AI::PositionValueData &result,
	const InnerExpression *innerExpression, const Position &pos,
	const int &myPiece, const int &opPiece, const int &mustToUpForThisTurn, const int &mustToDownForThisTurn,
	const vector<Position> &path, const Parameters_PositionEvaluation_Sub &paramSub, bool isUpperSide
) 
{
	
	for ( int i = 0; i < path.size(); ++i ) {
		
		int x = path[i].x;
		int y = path[i].y;
		
		// 根本
		if ( (isUpperSide && i == 0 ) || ( !isUpperSide && i == path.size() - 1)  ) 
		{
			if ( innerExpression->_gridList_ThrouableInfo[y][x] == THROUGHABLE_INFO::THROUGHABLE ) {
				result.risk      += paramSub.THROUABLE_Risk;//10;
				result.advantage += paramSub.THROUABLE_Advantage;//10;
			}
			else if ( innerExpression->_gridList_ThrouableInfo[y][x] == THROUGHABLE_INFO::NOT_THROUGHABLE ) {
				result.risk      += paramSub.NOT_THROUABLE_Risk;//0;
				result.advantage += paramSub.NOT_THROUABLE_Advantage;//0;
			}
			else if ( innerExpression->_gridList_ThrouableInfo[y][x] == mustToUpForThisTurn ) {
				result.risk      += paramSub.THROUABLE_Risk;//10; // 根本の「上向き」の危険度の扱いは単に「通行可能」と同じ
				result.advantage += paramSub.MUST_TO_UP_Advantage;//20;
			}
			else if ( innerExpression->_gridList_ThrouableInfo[y][x] == mustToDownForThisTurn ) {
				result.risk      += paramSub.THROUABLE_Risk;//10; // 根本の「下向き」の危険度の扱いは単に「通行可能」と同じ
				result.advantage += paramSub.MUST_TO_DOWN_Advantage;// -20;
			}
			else if ( innerExpression->_gridList_ThrouableInfo[y][x] == THROUGHABLE_INFO::MUST_TO_LEFT || innerExpression->_gridList_ThrouableInfo[y][x] == THROUGHABLE_INFO::MUST_TO_RIGHT ) {
				result.risk      += paramSub.THROUABLE_Risk;//10; // 根元の「横向き」の危険度の扱いは単に「通行可能」と同じ
				result.advantage += paramSub.MUST_TO_HORIZONTAL_Advantage;//-10;
			}
			
			// 自分の駒を置いておいた場合の安心度
			if ( innerExpression->_gridList_Piece[y][x] == myPiece ) {
				result.risk -= paramSub.myPieceReliefPower;
			}
		}
		// 先端
		else if ( (!isUpperSide && i == 0 ) || ( isUpperSide && i == path.size() - 1)  ) 
		{
			if ( innerExpression->_gridList_ThrouableInfo[y][x] == THROUGHABLE_INFO::THROUGHABLE ) {
				result.risk      += paramSub.THROUABLE_Risk;//10;
				result.advantage += paramSub.THROUABLE_Advantage;//10;
			}
			else if ( innerExpression->_gridList_ThrouableInfo[y][x] == THROUGHABLE_INFO::NOT_THROUGHABLE ) {
				result.risk      += paramSub.NOT_THROUABLE_Risk;//0;
				result.advantage += paramSub.NOT_THROUABLE_Advantage;//0;
			}
			else if ( innerExpression->_gridList_ThrouableInfo[y][x] == mustToUpForThisTurn ) {
				result.risk      += paramSub.MUST_TO_UP_Risk;//-20;
				result.advantage += paramSub.THROUABLE_Advantage;//10; // 先端の「上向き」の有利さの扱いは単に「通行可能」と同じ
			}
			else if ( innerExpression->_gridList_ThrouableInfo[y][x] == mustToDownForThisTurn ) {
				result.risk      += paramSub.MUST_TO_DOWN_Risk;//20;
				result.advantage += paramSub.THROUABLE_Advantage;//10; // 先端の「下向き」の有利さの扱いは単に「通行可能」と同じ
			}
			else if ( innerExpression->_gridList_ThrouableInfo[y][x] == THROUGHABLE_INFO::MUST_TO_LEFT || innerExpression->_gridList_ThrouableInfo[y][x] == THROUGHABLE_INFO::MUST_TO_RIGHT ) {
				result.risk      += paramSub.MUST_TO_HORIZONTAL_Risk;//-10;
				result.advantage += paramSub.THROUABLE_Advantage;//10; // 先端の「横向き」の有利さの扱いは単に「通行可能」と同じ
			}
			
			// 先端では自分の駒を置いておくことによる安心度は考慮しない
		}
		// 間の経路上
		else 
		{
			if ( innerExpression->_gridList_ThrouableInfo[y][x] == THROUGHABLE_INFO::THROUGHABLE ) {
				result.risk      += paramSub.THROUABLE_Risk;//10;
				result.advantage += paramSub.THROUABLE_Advantage;//10;
			}
			else if ( innerExpression->_gridList_ThrouableInfo[y][x] == THROUGHABLE_INFO::NOT_THROUGHABLE ) {
				result.risk      += paramSub.NOT_THROUABLE_Risk;//0;
				result.advantage += paramSub.NOT_THROUABLE_Advantage;//0;
			}
			else if ( innerExpression->_gridList_ThrouableInfo[y][x] == mustToUpForThisTurn ) {
				result.risk      += paramSub.MUST_TO_UP_Risk;//-20;
				result.advantage += paramSub.MUST_TO_UP_Advantage;//20;
			}
			else if ( innerExpression->_gridList_ThrouableInfo[y][x] == mustToDownForThisTurn ) {
				result.risk      += paramSub.MUST_TO_DOWN_Risk;//20;
				result.advantage += paramSub.MUST_TO_DOWN_Advantage;// -20;
			}
			else if ( innerExpression->_gridList_ThrouableInfo[y][x] == THROUGHABLE_INFO::MUST_TO_LEFT || innerExpression->_gridList_ThrouableInfo[y][x] == THROUGHABLE_INFO::MUST_TO_RIGHT ) {
				result.risk      += paramSub.MUST_TO_HORIZONTAL_Risk;//-10;
				result.advantage += paramSub.MUST_TO_HORIZONTAL_Advantage;//-10;
			}
			// 自分の駒を置いておいた場合の安心度
			if ( innerExpression->_gridList_Piece[y][x] == myPiece ) {
				result.risk -= paramSub.myPieceReliefPower;
			}

		}
		
		///// 自分の駒があった
		if ( innerExpression->_gridList_Piece[y][x] == myPiece ) {
			
			// 加算する有利さの初期値。ここから加減していく。主に減らし。
			int advantage = paramSub.myPiece_AdvantageStart;//30;
			
			// 現在位置から、経路の先端へ。有利さへの加算
			for ( int n = i; n < path.size(); ++n ) {
			
				int piecePathX = path[n].x;
				int piecePathY = path[n].y;
				
				// 通行可能状況による加減。先端へ到達できるのであれば、先端が下矢印だろうが考慮しない
				if      ( innerExpression->_gridList_ThrouableInfo[piecePathY][piecePathX] == THROUGHABLE_INFO::THROUGHABLE     ) { advantage += paramSub.myPiece_THROUABLE;}//0;  }
				else if ( innerExpression->_gridList_ThrouableInfo[piecePathY][piecePathX] == THROUGHABLE_INFO::NOT_THROUGHABLE ) { advantage += paramSub.myPiece_NOT_THROUABLE;}//-5;  }
				else {
					if ( n < path.size()-1 )
					{
						if      ( innerExpression->_gridList_ThrouableInfo[piecePathY][piecePathX] == mustToDownForThisTurn             ) { advantage += paramSub.myPiece_MUST_TO_DOWN;}//-15; }
						else if ( innerExpression->_gridList_ThrouableInfo[piecePathY][piecePathX] == mustToUpForThisTurn               ) { advantage += paramSub.myPiece_MUST_TO_UP;}// 0;  }
						else if ( innerExpression->_gridList_ThrouableInfo[piecePathY][piecePathX] == THROUGHABLE_INFO::MUST_TO_LEFT    ) { advantage += paramSub.myPiece_MUST_TO_HORIZONTAL;}//-10; }
						else if ( innerExpression->_gridList_ThrouableInfo[piecePathY][piecePathX] == THROUGHABLE_INFO::MUST_TO_RIGHT   ) { advantage += paramSub.myPiece_MUST_TO_HORIZONTAL;}//-10; }
						
					// 先端に到達しているのなら、矢印があっても関係ない（通行可能と同じ扱いで）
					} else {
						advantage += paramSub.myPiece_THROUABLE;//0;
					}
				}
				// 駒の存在状況による加減
				if ( n != i ) { // 自分自身とぶつからないように
					if      ( innerExpression->_gridList_Piece[piecePathY][piecePathX] == myPiece ) { advantage += paramSub.myPiece_againstMyPiece;}// -5;  }
					else if ( innerExpression->_gridList_Piece[piecePathY][piecePathX] == opPiece ) { advantage += paramSub.myPiece_againstOpPiece;}// -15; }
				}
				// マイナスにはしないこととする
				if ( advantage < 0 ) {
					advantage = 0;
					break;
				}
			}
			
			result.advantage += advantage;
		}
			
		///// 相手の駒があった
		if ( innerExpression->_gridList_Piece[y][x] == opPiece ) {
			
			// 加算する危険度の初期値。ここから加減していく。主に減らし。
			int risk = paramSub.opPiece_RiskStart;// 30;
			
			// 現在位置から、経路の根本へ。危険度への加算
			for ( int n = i; n >=0; --n ) {
				
				int piecePathX = path[n].x;
				int piecePathY = path[n].y;
				
				// 通行可能状況による加減。根本に到達できてしまう分に関しては、上矢印だろうが考慮しない
				if      ( innerExpression->_gridList_ThrouableInfo[piecePathY][piecePathX] == THROUGHABLE_INFO::THROUGHABLE     ) { risk += paramSub.opPiece_THROUABLE;}// 0;  }
				else if ( innerExpression->_gridList_ThrouableInfo[piecePathY][piecePathX] == THROUGHABLE_INFO::NOT_THROUGHABLE ) { risk += paramSub.opPiece_NOT_THROUABLE;}//-5;  }
				else {
					if ( n > 0 )
					{
						if      ( innerExpression->_gridList_ThrouableInfo[piecePathY][piecePathX] == mustToDownForThisTurn             ) { risk += paramSub.opPiece_MUST_TO_DOWN;}//0; }
						else if ( innerExpression->_gridList_ThrouableInfo[piecePathY][piecePathX] == mustToUpForThisTurn               ) { risk += paramSub.opPiece_MUST_TO_UP;}//-15; }
						else if ( innerExpression->_gridList_ThrouableInfo[piecePathY][piecePathX] == THROUGHABLE_INFO::MUST_TO_LEFT    ) { risk += paramSub.opPiece_MUST_TO_HORIZONTAL;}//-10; }
						else if ( innerExpression->_gridList_ThrouableInfo[piecePathY][piecePathX] == THROUGHABLE_INFO::MUST_TO_RIGHT   ) { risk += paramSub.opPiece_MUST_TO_HORIZONTAL;}//-10; }
						
					// 根本に到達しているのなら、矢印があっても関係ない（通行可能と同じ扱いで）
					} else {
						risk += paramSub.opPiece_THROUABLE;//0;
					}
				}
				
				// 駒の存在状況による加減
				if ( n != i ) { // 自分自身とぶつからないように
					if      ( innerExpression->_gridList_Piece[piecePathY][piecePathX] == myPiece ) { risk += paramSub.opPiece_againstMyPiece;}// -15;  }
					else if ( innerExpression->_gridList_Piece[piecePathY][piecePathX] == opPiece ) { risk += paramSub.opPiece_againstOpPiece;}// -5; }
				}
				// マイナスにはしないこととする
				if ( risk < 0 ) {
					risk = 0;
					break;
				}
				
			}
			
			result.risk += risk;
		}
		
	}

}


//=========================================================
// 位置評価＿盤面全体に対して。
//=========================================================
int AI::Evaluate_PositionForWholeBoard ( const InnerExpression *innerExpression, const int forThisTurn ) {
	
	int result = 0;
	
	for ( unsigned int y = 0; y < _rule.boardHeight; ++y ) {
		for ( unsigned int x = 0; x < _rule.boardWidth; ++x ) {
			
			AI::PositionValueData positionValue = Evaluate_Position ( innerExpression, Position (x, y), forThisTurn );
			
			result += positionValue.advantage;
			result -= positionValue.risk;
			
		}
	}
	
	
	return result;
}

//=========================================================
// 位置評価＿設定されている特定の位置（リスト）に対して
//=========================================================
int AI::Evaluate_PositionForSpecificPositions ( const InnerExpression *innerExpression, const int forThisTurn ) {
	
	int result = 0;
	
	for ( unsigned int j = 0; j < _parameters_PieceEvaluation.positionCareSettings.positionList.size(); ++j ) {
		
		PositionCare positionCare = _parameters_PieceEvaluation.positionCareSettings.positionList[j];
		
		int myY = 0;
		if ( forThisTurn == TURN::BLACK ) {
			myY = (_rule.boardHeight -1) - positionCare.y;
		}
		else if ( forThisTurn == TURN::WHITE ) {
			myY = positionCare.y;
		}
		
		AI::PositionValueData positionEval = Evaluate_Position ( innerExpression, Position( positionCare.x, myY )  ,forThisTurn );
		
		int advantage =  (int)((float)positionEval.advantage * (float)positionCare.advantageBias / 100.0f);
		int risk      =  (int)((float)positionEval.risk      * (float)positionCare.riskBias      / 100.0f);
		
		result += advantage;
		result -= risk;
		
	}
	
	
	return result;
}


//=========================================================
// デバッグ用。設定内容を表示
//=========================================================
void AI::ShowPositionCareSettingsForDebug() {
	
	PositionCareSettings params = _parameters_PieceEvaluation.positionCareSettings;
	
	printf ( "<POSITION_CARE>\n");
	printf ( "\t<ENABLED>%s</ENABLED>\n", params.isEnabled ? "on" : "off");
	printf ( "\t<APPLY_FOR_TERRAIN_SEARCH>%d</APPLY_FOR_TERRAIN_SEARCH>\n", params.applyBias_forTerrain);
	printf ( "\t<APPLY_FOR_PLAIN>%d</APPLY_FOR_PLAIN>\n", params.applyBias_forPlain);
	
	for ( unsigned int i = 0; i < params.positionList.size(); ++i ) {
		
		if ( i == 0 ) {
			printf ( "\t<POSITIONS>\n" );
		}
		
		PositionCare posParam = params.positionList[i];
		
		printf ( "\t\t<POSITION x=\"%d\" y=\"%d\" advantage=\"%d\" risk=\"%d\" />\n"
			,posParam.x, posParam.y, posParam.advantageBias, posParam.riskBias 
		);
		
		if ( i == params.positionList.size() - 1 ) {
			printf ( "\t</POSITIONS>\n" );
		}
	}
	printf ( "</POSITION_CARE>\n");
	
	
	
	
	
	
	
	
}





















