//=========================================================
//
// AI_実装ファイル_評価関数関係
//
//=========================================================
#include "pch.h"

#include <stdio.h>
#include "AI.h"
#include "InnerExpression.h"
#include "../Utils/Utils_Cal.h"
#include "../Utils/Utils_Convert.h"
#include "../Utils/Allocator.h"


using namespace AI_Modules;


//=========================================================
// 圧力バランス判定・準備しておく値
//=========================================================
void AI::Evaluate_Move_By_PressureBalance_Init ( InnerExpression *innerExpression ) {
	
	int securedColumnsNumForBlack = 0;
	int securedColumnsNumForWhite = 0;
	
	pair<int, int> data = GetNumberOfPressuringColumns ( innerExpression );
	_securedColumnsNumForBlack = data.first;
	_securedColumnsNumForWhite = data.second;
	
	_gameProgress = GetGameProgressDegree ( innerExpression );
	
}

//=========================================================
// 圧力バランス判定・実行するかどうかをゲーム進行度で判定
//=========================================================
bool AI::Evaluate_Move_By_PressureBalance_IsWithinTheGameProgress () {
	
	bool result = false;
	
	int from = _parameters_PieceEvaluation.parameters_PressureEvaluation.enabledTurn_from;
	int to   = _parameters_PieceEvaluation.parameters_PressureEvaluation.enabledTurn_to;
	vector<int> skipList   = _parameters_PieceEvaluation.parameters_PressureEvaluation.enabledTurn_skip;
	vector<int> forcedList = _parameters_PieceEvaluation.parameters_PressureEvaluation.enabledTurn_forced;
	
	// _gameProgress に値が入っている前提。Evaluate_Move_By_PressureBalance_Init () が実行されていること
	
	// 範囲内の場合
	if ( from <= _gameProgress && _gameProgress <= to ) {
		
		result = true;
		
		// スキップ回かどうか：正確に言うと、実際の手数ではなく、「進行度」なのだけれど、もうどうでもいいや。どうでもいいのかよ。
		for ( unsigned int i = 0; i < skipList.size(); ++i ) {
			if ( _gameProgress == skipList[i] ) {
				result = false;
				break;
			}
		}
	
	// 範囲内じゃなかった
	} else {
		
		result = false;
		
		// 強制回かどうか：正確に言うと以下略。
		for ( unsigned int i = 0; i < forcedList.size(); ++i ) {
			if ( _gameProgress == forcedList[i] ) {
				result = true;
				break;
			}
		}
	}
	
	
	return result;
}
		
//==========================================================
// 圧力バランスでの評価
// move そもそも合法な手が来ている前提
//==========================================================
int AI::Evaluate_Move_By_PressureBalance ( const InnerExpression *currentIe, Move move, const InnerExpression *nextIe, int forThisTurn) {
	
	int result = 0;
	
	/*
	// ゲームの進行度に応じて、実行したりしなかったり
	if ( !Evaluate_Move_By_PressureBalance_IsWithinTheGameProgress() ) {
		return result;
	}
	*/
	
	Parameters_PressureEvaluation params = _parameters_PieceEvaluation.parameters_PressureEvaluation;
	
	///// 種別ごとの判定処理
	if ( params.type == params.TYPE_COLUMN ) {
		
		result = Evaluate_Move_By_PressureBalance_TypeColumn ( currentIe, move, nextIe, forThisTurn );
		
	}
	else if ( params.type == params.TYPE_MIRROR ) {
	
		result = Evaluate_Move_By_PressureBalance_TypeMirror( currentIe, nextIe, forThisTurn );
		
	} else {
		// おかしいですね。
	}
	
	return result;
}

//=========================================================
// 圧力判定・タイプ：同縦軸で
//=========================================================
int AI::Evaluate_Move_By_PressureBalance_TypeColumn ( const InnerExpression *currentIe, const Move &move, const InnerExpression *nextIe, int forThisTurn) {
	
	int result = 0;
	
	// 指した後の局面での圧力確保軸数を求める
	pair<int, int> data = GetNumberOfPressuringColumns ( nextIe );
	
	int nextSecuredColumnsNumForBlack = data.first;
	int nextSecuredColumnsNumForWhite = data.second;
	
	
	// 手番を抽象化
	int securedColumnsNumForMy;
	int securedColumnsNumForOp;
	
	int nextSecuredColumnsNumForMy;
	int nextSecuredColumnsNumForOp;
	
	///// 黒の立場で判定
	if ( forThisTurn == TURN::BLACK ) 
	{
		securedColumnsNumForMy = _securedColumnsNumForBlack;
		securedColumnsNumForOp = _securedColumnsNumForWhite;
		
		nextSecuredColumnsNumForMy = nextSecuredColumnsNumForBlack;
		nextSecuredColumnsNumForOp = nextSecuredColumnsNumForWhite;
	}
	///// 白の立場で判定
	else if ( forThisTurn == TURN::WHITE ) 
	{
		securedColumnsNumForMy = _securedColumnsNumForWhite;
		securedColumnsNumForOp = _securedColumnsNumForBlack;
		
		nextSecuredColumnsNumForMy = nextSecuredColumnsNumForWhite;
		nextSecuredColumnsNumForOp = nextSecuredColumnsNumForBlack;
		
		//printf ( "securedColumnsNumForMy[%d], securedColumnsNumForOp[%d], nextSecuredColumnsNumForMy[%d], nextSecuredColumnsNumForOp[%d]\n", securedColumnsNumForMy, securedColumnsNumForOp, nextSecuredColumnsNumForMy, nextSecuredColumnsNumForOp );
	}
	
	
	
	// 圧力をかけられている軸が減った
	if ( securedColumnsNumForOp > nextSecuredColumnsNumForOp ) {
		
		int point    = _parameters_PieceEvaluation.parameters_PressureEvaluation.pointForDecreasePressuredNum;
		int moveBias = Evaluate_Move_By_PressureBalance_GetMoveBias ( move );
		
		result += point * moveBias / 100;
		
	}
	
	// 圧力をかけている軸が増えた
	if ( securedColumnsNumForMy < nextSecuredColumnsNumForMy ) {
		
		int point    = _parameters_PieceEvaluation.parameters_PressureEvaluation.pointForIncreasePressuringNum;
		int moveBias = Evaluate_Move_By_PressureBalance_GetMoveBias ( move );
		
		result += point * moveBias / 100;
		
	}
	
	return result;
}

//=========================================================
// 圧力判定・タイプ：鏡打ち
//=========================================================
int AI::Evaluate_Move_By_PressureBalance_TypeMirror ( const InnerExpression *currentIe, const InnerExpression *nextIe, int forThisTurn) {
	
	int result = 0;
	
	bool isMirror = false;
	
	isMirror = IsMirrorState ( nextIe );
	
	// 鏡打ちになっているならばポイント追加
	if ( isMirror ) {
		
		int point    = _parameters_PieceEvaluation.parameters_PressureEvaluation.pointForMirrorMove;
		
		result += point;
		
	}
	
	
	return result;
}

//=========================================================
// 局面が上下の鏡像状態になっているかどうか
//=========================================================
bool AI::IsMirrorState ( const InnerExpression *ie ) {
	
	bool result = true;
	
	// 奇数なので、真ん中を除外
	int maxY = ( _rule.boardHeight - (_rule.boardHeight%2) ) / 2;
	
	for ( unsigned int y = 0; y < maxY; ++y ) {
		
		if ( !result ) { break;}
		
		for ( unsigned int x = 0; x < _rule.boardWidth; ++x ) {
			
			int mirrorY = (_rule.boardHeight-1) - y;
			
			///// 通行可能状況
			if ( ie->_gridList_ThrouableInfo[y][x] == THROUGHABLE_INFO::NOT_THROUGHABLE ) {
				if ( ie->_gridList_ThrouableInfo[mirrorY][x] != THROUGHABLE_INFO::NOT_THROUGHABLE ) { result = false; break; }
			}
			else if ( ie->_gridList_ThrouableInfo[y][x] == THROUGHABLE_INFO::THROUGHABLE ) {
				if ( ie->_gridList_ThrouableInfo[mirrorY][x] != THROUGHABLE_INFO::THROUGHABLE ) { result = false; break; }
			}
			else if ( ie->_gridList_ThrouableInfo[y][x] == THROUGHABLE_INFO::MUST_TO_DOWN ) {
				if ( ie->_gridList_ThrouableInfo[mirrorY][x] != THROUGHABLE_INFO::MUST_TO_UP ) { result = false; break; }
			}
			else if ( ie->_gridList_ThrouableInfo[y][x] == THROUGHABLE_INFO::MUST_TO_UP ) {
				if ( ie->_gridList_ThrouableInfo[mirrorY][x] != THROUGHABLE_INFO::MUST_TO_DOWN ) { result = false; break; }
			}
			else if ( ie->_gridList_ThrouableInfo[y][x] == THROUGHABLE_INFO::MUST_TO_LEFT ) {
				if ( ie->_gridList_ThrouableInfo[mirrorY][x] != THROUGHABLE_INFO::MUST_TO_LEFT ) { result = false; break; }
			}
			else if ( ie->_gridList_ThrouableInfo[y][x] == THROUGHABLE_INFO::MUST_TO_RIGHT ) {
				if ( ie->_gridList_ThrouableInfo[mirrorY][x] != THROUGHABLE_INFO::MUST_TO_RIGHT ) { result = false; break; }
			}
			
			///// 駒
			if ( ie->_gridList_Piece[y][x] == GRID::ENPTY ) {
				if ( ie->_gridList_Piece[mirrorY][x] != GRID::ENPTY ) { result = false; break; }
			}
			else if ( ie->_gridList_Piece[y][x] == GRID::BLACK ) {
				if ( ie->_gridList_Piece[mirrorY][x] != GRID::WHITE ) { result = false; break; }
			}
			else if ( ie->_gridList_Piece[y][x] == GRID::WHITE ) {
				if ( ie->_gridList_Piece[mirrorY][x] != GRID::BLACK ) { result = false; break; }
			}

		}
	}
	
	///// 中央の段
	if ( _rule.boardHeight%2 == 1 ) {
		
		int middleY = maxY;
		
		for ( unsigned int x = 0; x < _rule.boardWidth; ++x ) {
			
			///// 通行可能状況
			if (   ie->_gridList_ThrouableInfo[middleY][x] == THROUGHABLE_INFO::MUST_TO_UP 
				|| ie->_gridList_ThrouableInfo[middleY][x] == THROUGHABLE_INFO::MUST_TO_DOWN
			) { 
				result = false; break;
			}
			
			///// 駒
			if ( ie->_gridList_Piece[middleY][x] != GRID::ENPTY ) { result = false; break; }
		}
	}
	
	
	
	return result;
}

//=========================================================
// 圧力判定・特定の指し手に対して設定されているバイアスを取得
//=========================================================
int AI::Evaluate_Move_By_PressureBalance_GetMoveBias ( const Move &move ) {
	
	int result = 100;
	
	if ( move.action == MOVE::ACTION::PUT_CARD ) {
		
		/*
			カード名は ./data/cardConfig.xml で設定されている name 
			
			指し手としては嫌悪の場合は「UP」で矢印が下向きになる。
			が、ここでいう「下向き」というのはつまり矢印の向きが、という意味なので、
			「指し手としては」という点は隠蔽しておくこととする
		*/
		
		if ( _cardNameList [ move.putCardId ] == "iinogare" ) {
			
			if ( move.putCardDirection == AI_Modules::DIRECTION::DOWN ) {
				
				result = _parameters_PieceEvaluation.parameters_PressureEvaluation.bias_iinogare_d;
			}
			else if ( move.putCardDirection == AI_Modules::DIRECTION::LEFT || move.putCardDirection == AI_Modules::DIRECTION::RIGHT ) {
				
				result = _parameters_PieceEvaluation.parameters_PressureEvaluation.bias_iinogare_h;
			}
			
		}
		else if ( _cardNameList [ move.putCardId ] == "keno" ) {
			
			if ( move.putCardDirection == AI_Modules::DIRECTION::UP ) {
				
				result = _parameters_PieceEvaluation.parameters_PressureEvaluation.bias_keno_d;
			}
			else if ( move.putCardDirection == AI_Modules::DIRECTION::LEFT || move.putCardDirection == AI_Modules::DIRECTION::RIGHT ) {
				
				result = _parameters_PieceEvaluation.parameters_PressureEvaluation.bias_keno_h;
			}
		
		}
		
	} else if ( move.action == MOVE::ACTION::TAKE_CARD ) {
		
		result = _parameters_PieceEvaluation.parameters_PressureEvaluation.bias_take_card;
		
	} else {
	
		result = _parameters_PieceEvaluation.parameters_PressureEvaluation.bias_others;
	}
	
	
	return result;
}

//=========================================================
// 指定の縦軸での圧力判定
//【戻り値】
// 黒有利、白有利、なし
//=========================================================
int AI::GetPressureBalanceAtThisColumn ( const InnerExpression *innerExpression, int x ) {
	
	int result = TURN::NONE;
	
	bool isUpExist         = false;
	bool isDownExist       = false;
	bool isHorizontalWallExist_forBlack = false; // 一番上にあっても壁にならないので、それを除外する目的
	bool isHorizontalWallExist_forWhite = false; // 一番下にあっても壁にならないので、それを除外する目的
	
	for ( unsigned int y = 0; y < _rule.boardHeight; ++y ) {
		
		if ( innerExpression->_gridList_ThrouableInfo[y][x] == THROUGHABLE_INFO::MUST_TO_UP ) {
			
			isUpExist = true;
			
		} else if ( innerExpression->_gridList_ThrouableInfo[y][x] == THROUGHABLE_INFO::MUST_TO_DOWN ) {
			
			isDownExist = true;
			
		} else if ( innerExpression->_gridList_ThrouableInfo[y][x] == THROUGHABLE_INFO::MUST_TO_LEFT ) {
			
			if ( y > 0 ) {
				isHorizontalWallExist_forBlack = true;
			}
			
			if ( y < _rule.boardHeight - 1 ) {
				isHorizontalWallExist_forWhite = true;
			}
			
		} else if ( innerExpression->_gridList_ThrouableInfo[y][x] == THROUGHABLE_INFO::MUST_TO_RIGHT ) {
			
			if ( y > 0 ) {
				isHorizontalWallExist_forBlack = true;
			}
			
			if ( y < _rule.boardHeight - 1 ) {
				isHorizontalWallExist_forWhite = true;
			}
			
		}
		
		
		// 黒駒がありました
		if ( innerExpression->_gridList_Piece[y][x] == GRID::BLACK ) {
			// さしあたり無視
		}
		// 白駒がありました
		else if ( innerExpression->_gridList_Piece[y][x] == GRID::WHITE ) {
			// さしあたり無視
		}
		
	}
	
	
	// 判定
	if ( !isHorizontalWallExist_forBlack && isUpExist && !isDownExist ) {
		result = TURN::BLACK;
	}
	else if ( !isHorizontalWallExist_forWhite && !isUpExist && isDownExist ) {
		result = TURN::WHITE;
	}
	
	
	return result;
}


//=========================================================
// 各手番の圧力を確保している縦軸の本数を求める
//【戻り値】
// result.first  : 黒の本数
// result.second : 白の本数 
//=========================================================
pair<int, int> AI::GetNumberOfPressuringColumns ( const InnerExpression *innerExpression ) {
	
	pair<int, int> result;
	result.first  = 0;
	result.second = 0;
	
	for ( unsigned int x = 0; x < _rule.boardWidth; ++x ) {
		
		int turn = GetPressureBalanceAtThisColumn ( innerExpression, x );
		
		if ( turn == TURN::BLACK ) {
			result.first += 1;
		}
		else if ( turn == TURN::WHITE ) {
			result.second += 1;
		}
	}
	
	return result;
}




//=========================================================
// 圧力バランス判定の設定内容を確認・デバッグ用
//=========================================================
void AI::Evaluate_Move_By_PressureBalance_ShowSettingsForDebug() {
	
	printf ( "<PRESSURE_BALANCE_SETTING>\n");
	
	printf ( "\t<ENABLED>%s</ENABLED>\n", (_parameters_PieceEvaluation.parameters_PressureEvaluation.isEnabled) ? "on":"off" );
	
	int from = _parameters_PieceEvaluation.parameters_PressureEvaluation.enabledTurn_from;
	int to   = _parameters_PieceEvaluation.parameters_PressureEvaluation.enabledTurn_to;
	vector<int> skipList   = _parameters_PieceEvaluation.parameters_PressureEvaluation.enabledTurn_skip;
	vector<int> forcedList = _parameters_PieceEvaluation.parameters_PressureEvaluation.enabledTurn_forced;
	
	string skip = "";
	for ( unsigned int i = 0; i < skipList.size(); ++i ) {
		if ( i > 0 ) { skip += ","; }
		skip += Utils_Convert::Int_to_String (skipList[i]);
	}
	string forced = "";
	for ( unsigned int i = 0; i < forcedList.size(); ++i ) {
		if ( i > 0 ) { forced += ","; }
		forced += Utils_Convert::Int_to_String (forcedList[i]);
	}

	string type = "";
	if ( _parameters_PieceEvaluation.parameters_PressureEvaluation.type == _parameters_PieceEvaluation.parameters_PressureEvaluation.TYPE_COLUMN ) {
		type = "COLUMN";
	}
	else if ( _parameters_PieceEvaluation.parameters_PressureEvaluation.type == _parameters_PieceEvaluation.parameters_PressureEvaluation.TYPE_MIRROR ) {
		type = "MIRROR";
	}
	printf ( "\t<TYPE>%s</TYPE>\n", type.c_str() );
	
	printf ( "\t<GAME_PROGRESS from=\"%d\" to=\"%d\" skip=\"%s\" forced=\"%s\" />\n", from, to, skip.c_str(), forced.c_str());
		
	printf ( "\t<POINT_INCREASE_PRESSURING_COLUMNS>%d</POINT_INCREASE_PRESSURING_COLUMNS>\n", _parameters_PieceEvaluation.parameters_PressureEvaluation.pointForIncreasePressuringNum);
	printf ( "\t<POINT_DECREASE_PRESSURED_COLUMNS>%d</POINT_DECREASE_PRESSURED_COLUMNS>\n", _parameters_PieceEvaluation.parameters_PressureEvaluation.pointForDecreasePressuredNum);
	printf ( "\t<POINT_MIRROR_MOVE>%d</POINT_MIRROR_MOVE>\n", _parameters_PieceEvaluation.parameters_PressureEvaluation.pointForMirrorMove);

	printf ( "</PRESSURE_BALANCE_SETTING>\n");
}





