//=========================================================
//
// AI
//
//=========================================================
#include "pch.h"

#include <stdio.h>
#include <fstream>
#include "AI.h"
#include "InnerExpression.h"
#include "../Utils/Utils_Cal.h"
#include "../Utils/Allocator.h"
#include "../Utils/Utils_Convert.h"
#include "../Utils/Utils_String.h"
#include "NotXML.h"

using namespace AI_Modules;


//==========================================================
// 駒評価素子リストをファイルに保存
//==========================================================
void AI::PEE_Output_File( const Parameters_PieceEvaluation &parameters_PieceEvaluation, string fileName ) {

	string str = "";

	// 高さバイアス
	str += "<HEIGHT_VALUE_BIAS>" + Utils_Convert::Int_to_String( parameters_PieceEvaluation.heightValue_Bias ) + "</HEIGHT_VALUE_BIAS>\n";

	// 地形探索パラメータ
	str += TerrainSearchUsage_to_String_forXMLSave( parameters_PieceEvaluation.terrainSearchUsage );

	// 詰み判定
	str += "<MATE_SEARCH>\n";
	str += "\t<DEPTH>" + Utils_Convert::Int_to_String(parameters_PieceEvaluation.mateSearchDepth) + "</DEPTH>\n";
	str += "</MATE_SEARCH>\n";

	// 詰まされ判定
	str += "<MATED_SEARCH>\n";
	str += "\t<DEPTH>" + Utils_Convert::Int_to_String(parameters_PieceEvaluation.matedSearchDepth) + "</DEPTH>\n";
	str += "\t<SUBJECT_NUMBER>" + Utils_Convert::Int_to_String(parameters_PieceEvaluation.matedSearchSubjectNumber) + "</SUBJECT_NUMBER>\n";
	str += "</MATED_SEARCH>\n";

	// 詰めろ設定
	str += PreMateSearchSetting_to_String_forXMLSave( parameters_PieceEvaluation.preMate );

	// 柱と傘での評価設定
	str += PC_Evaluate_Setting_to_String_forXMLSave( parameters_PieceEvaluation.pcEvalInfo );

	// 手札評価パラメータ
	str += HandCardEvaluate_to_String_forXMLSave ( parameters_PieceEvaluation.handCardEvaluate );

	// 手札補充手評価パラメータ
	str += TakeCardMoveEvaluate_to_String_forXMLSave ( parameters_PieceEvaluation.takeCardMoveEvaluate );

	// 駒評価素子
	for (unsigned int i = 0; i < parameters_PieceEvaluation.pieceEvaluateElementList.size(); ++i) {

		if (i > 0) {
			str += "\n";
		}
		str += PEE_to_String ( parameters_PieceEvaluation.pieceEvaluateElementList[i], 0);
	}

	std::ofstream ofs( fileName.c_str());
	ofs << str << std::endl;

}

//==========================================================
// TerrainSearchUsage を文字列化：XML風に保存するための
//==========================================================
string AI::TerrainSearchUsage_to_String_forXMLSave ( const TerrainSearchUsage &terrainSearchUsage ) {

	string result = "";

	result += "<TERRAIN_SEARCH>\n";
	result += "\t<DEPTH>"                 + Utils_Convert::Int_to_String( terrainSearchUsage.depth )                + "</DEPTH>\n";
	result += "\t<TERRAIN_EVALUATE_BIAS>" + Utils_Convert::Int_to_String( terrainSearchUsage.bias_TerrainEvaluate ) + "</TERRAIN_EVALUATE_BIAS>\n";
	result += "\t<PEE_BIAS>"              + Utils_Convert::Int_to_String( terrainSearchUsage.bias_PeeEvaluate )     + "</PEE_BIAS>\n";

	/*
		<SPECIAL_CONDITION> は 未実装
	*/

	result += "</TERRAIN_SEARCH>\n";

	return result;
}
//==========================================================
// 手札評価設定 を文字列化：XML風に保存するための
//==========================================================
string AI::HandCardEvaluate_to_String_forXMLSave ( const HandCardEvaluate &handCardEval ) {

	string result = "";

	///// 開始タグ
	result += "<HAND_CARDS_VALUE>\n";

	///// 有効・無効
	string enabled = ( handCardEval.isEnabled ) ? "on" : "off";
	result += "\t<ENABLED>" + enabled + "</ENABLED>\n";

	///// ポイント：カード
	result += "\t<CARD>" + Utils_Convert::Int_to_String( handCardEval.point_Card ) + "</CARD>\n";

	///// ポイント：TUROUHABLE
	result += "\t<THROUGHABLE>" + Utils_Convert::Int_to_String( handCardEval.point_Throughable ) + "</THROUGHABLE>\n";

	///// ポイント：MUST_GO_SOMEWAY
	result += "\t<MUST_GO_SOMEWAY>" + Utils_Convert::Int_to_String( handCardEval.point_MustGoSomeway ) + "</MUST_GO_SOMEWAY>\n";

	///// 閉じタグ
	result += "</HAND_CARDS_VALUE>\n";

	return result;
}

//==========================================================
// 手札補充手 タグ と 構造体とを相互変換
//==========================================================
string AI::TakeCardMoveEvaluate_to_String_forXMLSave ( const TakeCardMoveEvaluate &takeCardMoveEval ) {

	string result = "";

	///// 開始タグ
	result += "<TAKE_CARD_MOVE_EVALUATE>\n";

	///// 有効・無効
	string enabled = ( takeCardMoveEval.isEnabled ) ? "on" : "off";
	result += "\t<ENABLED>" + enabled + "</ENABLED>\n";

	///// 発動条件：手札がこの枚数以下のとき
	result += "\t<HAND_CARDS_LESS_THAN>" + Utils_Convert::Int_to_String( takeCardMoveEval.handCardsLessThanNum ) + "</HAND_CARDS_LESS_THAN>\n";

	///// ポイント上乗せレート(%)
	result += "\t<ADD_RATE>" + Utils_Convert::Int_to_String( takeCardMoveEval.addRate ) + "</ADD_RATE>\n";

	///// 詰めろの評価値
	result += "\t<NEXT_CHECK_MATE_VALUE>" + Utils_Convert::Int_to_String( takeCardMoveEval.nextCheckMateValue ) + "</NEXT_CHECK_MATE_VALUE>\n";

	///// 閉じタグ
	result += "</TAKE_CARD_MOVE_EVALUATE>\n";

	return result;
}

//==========================================================
// 詰めろ探索 タグ と 構造体を 相互変換
//==========================================================
string AI::PreMateSearchSetting_to_String_forXMLSave ( const PreMateSearchSetting &preMate ) {

	string result = "";

	///// 開始タグ
	result += "<PRE_MATE_SEARCH>\n";

	///// 有効・無効
	string enabled = ( preMate.isEnabled ) ? "on" : "off";
	result += "\t<ENABLED>" + enabled + "</ENABLED>\n";

	///// 探索深度
	result += "\t<DEPTH>" + Utils_Convert::Int_to_String( preMate.depth ) + "</DEPTH>\n";

	///// 適用件数
	result += "\t<SUBJECT_NUMBER>" + Utils_Convert::Int_to_String( preMate.subjectNum ) + "</SUBJECT_NUMBER>\n";

	///// 評価値
	result += "\t<VALUE>" + Utils_Convert::Int_to_String( preMate.value ) + "</VALUE>\n";

	///// 直接王手を含むか？
	result += "\t<INCLUDE_DIRECT_CHECK>" + Utils_Convert::Bool_to_string ( preMate.isIncludeDirectCheck ) + "</INCLUDE_DIRECT_CHECK>\n";

	///// 閉じタグ
	result += "</PRE_MATE_SEARCH>\n";

	return result;
}

//==========================================================
// 柱と傘評価設定 を文字列化：XML風に保存するための
//==========================================================
string AI::PC_Evaluate_Setting_to_String_forXMLSave ( const PC_Evaluate_Settings &pcEvalSetting ) {

	string result = "";

	///// 開始タグ
	result += "<PC_SETTING>\n";

	///// 有効・無効
	string enabled = ( pcEvalSetting.isEnabled ) ? "on" : "off";
	result += "\t<ENABLED>" + enabled + "</ENABLED>\n";

	///// 柱として必要な高さ
	result += "\t<REQUIRED_HEIGHT>" + Utils_Convert::Int_to_String( pcEvalSetting.reqiredHeightForPillar ) + "</REQUIRED_HEIGHT>\n";

	///// 攻めバイアス
	result += "\t<ATTACK_BIAS>" + Utils_Convert::Int_to_String( pcEvalSetting.attackBias ) + "</ATTACK_BIAS>\n";

	///// 守りバイアス
	result += "\t<GUARD_BIAS>" + Utils_Convert::Int_to_String( pcEvalSetting.guardBias ) + "</GUARD_BIAS>\n";

	///// 適用度合い：地形
	result += "\t<APPLY_FOR_TERRAIN_SEARCH>" + Utils_Convert::Int_to_String( pcEvalSetting.applyBias_forTerrain ) + "</APPLY_FOR_TERRAIN_SEARCH>\n";

	///// 適用度合い：平地
	result += "\t<APPLY_FOR_PLAIN>" + Utils_Convert::Int_to_String( pcEvalSetting.applyBias_forPlain ) + "</APPLY_FOR_PLAIN>\n";

	///// 危険度の上限
	for ( unsigned int i = 0; i < pcEvalSetting.goalValueList.size(); ++i ) {
		result += "\t<GOAL_VALUE>" + Utils_Convert::Int_to_String( pcEvalSetting.goalValueList[i] ) + "</GOAL_VALUE>\n";
	}

	///// 駒の高さバイアス
	result += "\t<PIECE_HEIGHT_WEIGHT>" + Utils_Convert::Int_to_String( pcEvalSetting.pieceHeightWeight ) + "</PIECE_HEIGHT_WEIGHT>\n";
	
	///// 駒の位置による重み
	result += "\t<PIECE_WEIGHT state=\"IN_THE_BOX\"            >" + Utils_Convert::Int_to_String( pcEvalSetting.pieceWeight_IN_THE_BOX)              + "</PIECE_WEIGHT>\n";
	result += "\t<PIECE_WEIGHT state=\"ABOVE_COVER\"           >" + Utils_Convert::Int_to_String( pcEvalSetting.pieceWeight_ABOVE_COVER)             + "</PIECE_WEIGHT>\n";
	result += "\t<PIECE_WEIGHT state=\"ABOVE_COVER_DIAGONALLY\">" + Utils_Convert::Int_to_String( pcEvalSetting.pieceWeight_ABOVE_COVER_DIAGONALLY)  + "</PIECE_WEIGHT>\n";
	result += "\t<PIECE_WEIGHT state=\"UNDER_COVER_SIDE\"      >" + Utils_Convert::Int_to_String( pcEvalSetting.pieceWeight_UNDER_COVER_SIDE)        + "</PIECE_WEIGHT>\n";
	result += "\t<PIECE_WEIGHT state=\"UNDER_COVER\"           >" + Utils_Convert::Int_to_String( pcEvalSetting.pieceWeight_UNDER_COVER)             + "</PIECE_WEIGHT>\n";
	result += "\t<PIECE_WEIGHT state=\"NOT_INVOLVED\"          >" + Utils_Convert::Int_to_String( pcEvalSetting.pieceWeight_NOT_INVOLVED)            + "</PIECE_WEIGHT>\n";

	///// 加算アルゴリズム
	string addingAlgorism = "";
	if      ( pcEvalSetting.addingAlgorism == PILLAR_COVER::ALGORISM_FOR_RISK_DEGREE_ADDITION_TO_A_PILLAR::MAX_ONLY ) { addingAlgorism = "MAX_ONLY";}
	else if ( pcEvalSetting.addingAlgorism == PILLAR_COVER::ALGORISM_FOR_RISK_DEGREE_ADDITION_TO_A_PILLAR::ADD      ) { addingAlgorism = "ADD";}
	result += "\t<ALGORISM_FOR_RISK_DEGREE_ADDITION_TO_A_PILLAR>" + addingAlgorism + "</ALGORISM_FOR_RISK_DEGREE_ADDITION_TO_A_PILLAR>\n";

	///// 閉じタグ
	result += "</PC_SETTING>\n";

	return result;


}


//==========================================================
// PEEを文字列化：XML風に保存するための
//==========================================================
string AI::PEE_to_String ( const PieceEvaluateElement &pee, int indent) {

	string indentMark_0 = "";
	for (int i = 0; i < indent; ++i) {
		indentMark_0 += "\t";
	}
	string indentMark_1 = "";
	for (int i = 0; i < indent + 1; ++i) {
		indentMark_1 += "\t";
	}

	string result = indentMark_0 + "<PEE>\n";

	///// パネル条件
	result += indentMark_1 + "<THROUGHABLE_INFO::EX>";
	if ( pee.throuableInfo == THROUGHABLE_INFO::EX::THROUGHABLE     ) { result += "THROUGHABLE";     }
	if ( pee.throuableInfo == THROUGHABLE_INFO::EX::NOT_THROUGHABLE ) { result += "NOT_THROUGHABLE"; }
	if ( pee.throuableInfo == THROUGHABLE_INFO::EX::MUST_TO_UP      ) { result += "MUST_TO_UP";      }
	if ( pee.throuableInfo == THROUGHABLE_INFO::EX::MUST_TO_DOWN    ) { result += "MUST_TO_DOWN";    }
	if ( pee.throuableInfo == THROUGHABLE_INFO::EX::MUST_TO_LEFT    ) { result += "MUST_TO_LEFT";    }
	if ( pee.throuableInfo == THROUGHABLE_INFO::EX::MUST_TO_RIGHT   ) { result += "MUST_TO_RIGHT";   }
	if ( pee.throuableInfo == THROUGHABLE_INFO::EX::PIECE_EMPTY     ) { result += "PIECE_EMPTY";     }
	if ( pee.throuableInfo == THROUGHABLE_INFO::EX::PIECE_MY        ) { result += "PIECE_MY";        }
	if ( pee.throuableInfo == THROUGHABLE_INFO::EX::PIECE_OPPONENT  ) { result += "PIECE_OPPONENT";  }
	if ( pee.throuableInfo == THROUGHABLE_INFO::EX::OUT_OF_BOARD    ) { result += "OUT_OF_BOARD";    }
	if ( pee.throuableInfo == THROUGHABLE_INFO::EX::OPPONENT_AREA   ) { result += "OPPONENT_AREA";   }
	result += "</THROUGHABLE_INFO::EX>";


	///// 相対位置
	result += "\n" + indentMark_1 + "<X>" + Utils_Convert::Int_to_String ( pee.x ) + "</X>";
	result += "\n" + indentMark_1 + "<Y>" + Utils_Convert::Int_to_String ( pee.y ) + "</Y>";

	///// 加点値
	result += "\n" + indentMark_1 + "<POINT>" + Utils_Convert::Int_to_String ( pee.point ) + "</POINT>";

	
	///// 子要素
	for ( unsigned int i = 0; i < pee.relativeList.size(); ++i ) {

		result += "\n" + PEE_to_String ( pee.relativeList[i], indent + 1 );
	}

	result += "\n" + indentMark_0 + "</PEE>";

	return result;
}

//==========================================================
// ファイルから、駒評価素子リストを読み込む
//==========================================================
Parameters_PieceEvaluation AI::PEE_Read_File( string fileName ) {

	//std::ifstream ifs( "savePEE.txt" );
	//std::string str;
	//ifs >> str;

	Parameters_PieceEvaluation result;
	result.heightValue_Bias = 1;
	result.pieceEvaluateElementList.clear();
	
	vector<NotXML::Tag> tagList = NotXML().Read( fileName );
	
	///// ここで先に継承を確認
	/*
		継承は一個だけ。
		循環参照の確認はしませんよ。
	*/
	for ( unsigned int i = 0; i < tagList.size(); ++i) {
		
		if ( tagList[i].name == "INHERIT") {
			
			result = PEE_Read_File ( tagList[i].value );
			
			// 継承は一個だけ。
			break;
		}
	}
	
	
	
	// 変数PEEリスト
	map < string, PieceEvaluateElement > variablePEEList;

	//vector<NotXML::Tag> tagList = NotXML().Read( fileName );
	for ( unsigned int i = 0; i < tagList.size(); ++i) {

		if ( tagList[i].name == "HEIGHT_VALUE_BIAS") {

			result.heightValue_Bias = Utils_Convert::String_to_Int( tagList[i].value );

		} else if ( tagList[i].name == "TERRAIN_SEARCH" ) {

			result.terrainSearchUsage = Read_TerrainSearchTag ( tagList[i] );

		} else if ( tagList[i].name == "MATE_SEARCH" ) {

			for (unsigned int n = 0; n < tagList[i].childList.size(); ++n) {

				if (tagList[i].childList[n].name == "DEPTH") {
					result.mateSearchDepth = Utils_Convert::String_to_Int(tagList[i].childList[n].value);
				}
			}

		} else if ( tagList[i].name == "MATED_SEARCH" ) {

			for (unsigned int n = 0; n < tagList[i].childList.size(); ++n) {

				if        (tagList[i].childList[n].name == "DEPTH") {
					result.matedSearchDepth = Utils_Convert::String_to_Int(tagList[i].childList[n].value);
				} else if (tagList[i].childList[n].name == "SUBJECT_NUMBER") {
					result.matedSearchSubjectNumber = Utils_Convert::String_to_Int(tagList[i].childList[n].value);
				}

			}

		} else if ( tagList[i].name == "PRE_MATE_SEARCH" ) {

			result.preMate = Read_PreMateSearchTag ( tagList[i] );

		} else if ( tagList[i].name == "PC_SETTING" ) {

			result.pcEvalInfo = Read_PC_Evaluate_SettingsTag ( tagList[i], result.pcEvalInfo );

		} else if ( tagList[i].name == "HAND_CARDS_VALUE" ) {

			result.handCardEvaluate = Read_HandCardEvaluateTag ( tagList[i] );

		} else if ( tagList[i].name == "TAKE_CARD_MOVE_EVALUATE" ) {

			result.takeCardMoveEvaluate = Read_TakeCardMoveEvaluateTag ( tagList[i] );

		} else if ( tagList[i].name == "PEE" ) {

			// 変数から使いまわす？：一応、ルート要素にも、変数を使えますよ、ということ。
			string fromVariableName = NotXML::GetAttribute( tagList[i], "FROM_VARIABLE" );
			if ( fromVariableName != "" ) {

				printf("変数が指定された %s\n", fromVariableName.c_str());

				// 変数に登録されている
				if (variablePEEList.find( fromVariableName ) != variablePEEList.end()) {

					PieceEvaluateElement pee = variablePEEList.find( fromVariableName )->second;
					result.pieceEvaluateElementList.push_back( pee );

				// ない
				} else {
					printf("指定の変数名の素子がありません。%s\n", fromVariableName.c_str());
				}

			} else {

				PieceEvaluateElement pee = PEE_Read_PEE_Tag( tagList[i], variablePEEList );

				// 変数にする？
				string variableName = NotXML::GetAttribute( tagList[i], "VARIABLE_NAME" );
				if ( variableName != "" ) {

					// 名前重複
					if ( variablePEEList.find( variableName ) != variablePEEList.end() ) {
						printf("PEE変数名が重複しました。%s\n", variableName.c_str());
					}

					variablePEEList[variableName] = pee;

				// 変数じゃないなら、リストに追加
				} else {
					result.pieceEvaluateElementList.push_back( pee );
				}
			}
		
		} else if ( tagList[i].name == "CLEAR_ALL_PEES" || tagList[i].name == "CLEAR_ALL_PEEs" ) {
			
			result.pieceEvaluateElementList.clear();
		
		} else if ( tagList[i].name == "CLEAR_ALL_HANDY_PROCS" || tagList[i].name == "CLEAR_ALL_HANDY_PROCs" ) {
			
			result.handyProcList.clear();
			
		} else if ( tagList[i].name == "HANDY_PROC" ) {
			
			HandyProc handyProc;
			
			string fromX = NotXML::GetAttribute ( tagList[i], "gridFromX", "0" );
			string fromY = NotXML::GetAttribute ( tagList[i], "gridFromY", "0" );
			string toX   = NotXML::GetAttribute ( tagList[i], "gridToX"  , "8" );
			string toY   = NotXML::GetAttribute ( tagList[i], "gridToY"  , "8" );
			
			handyProc.gridFromX = Utils_Convert::String_to_Int ( fromX );
			handyProc.gridFromY = Utils_Convert::String_to_Int ( fromY );
			handyProc.gridToX   = Utils_Convert::String_to_Int ( toX   );
			handyProc.gridToY   = Utils_Convert::String_to_Int ( toY   );
			
			for (unsigned int n = 0; n < tagList[i].childList.size(); ++n) {
				
				if (tagList[i].childList[n].name == "GAME_PROGRESS") {
					
					string from = NotXML::GetAttribute ( tagList[i].childList[n], "from", "-1" );
					string to   = NotXML::GetAttribute ( tagList[i].childList[n], "to",   "-1" );
					
					handyProc.gameProgressFrom = Utils_Convert::String_to_Int ( from, -1 );
					handyProc.gameProgressTo   = Utils_Convert::String_to_Int ( to, -1 );
					
				} else if (tagList[i].childList[n].name == "MOVE") {
					
					string action        = NotXML::GetAttribute ( tagList[i].childList[n], "action", "0" );
					string x             = NotXML::GetAttribute ( tagList[i].childList[n], "x", "0" );
					string y             = NotXML::GetAttribute ( tagList[i].childList[n], "y", "0" );
					string status        = NotXML::GetAttribute ( tagList[i].childList[n], "status", "" );
					string cardId        = NotXML::GetAttribute ( tagList[i].childList[n], "cardId", "" );
					string cardDirection = NotXML::GetAttribute ( tagList[i].childList[n], "cardDirection", "NONE" );
					
					if (false) {}
					else if ( action == "PUT_PIECE"  ) { handyProc.action = MOVE::ACTION::PUT_PIECE ; }
					else if ( action == "PUT_CARD"   ) { handyProc.action = MOVE::ACTION::PUT_CARD  ; }
					else if ( action == "MOVE_PIECE" ) { handyProc.action = MOVE::ACTION::MOVE_PIECE; }
					else if ( action == "TAKE_CARD"  ) { handyProc.action = MOVE::ACTION::TAKE_CARD ; }
					
					int cardDirectionValue = DIRECTION::NONE;
					if (false) {}
					else if ( cardDirection == "NONE"  ) { handyProc.actionCardDirection = DIRECTION::NONE  ; }
					else if ( cardDirection == "LEFT"  ) { handyProc.actionCardDirection = DIRECTION::LEFT  ; }
					else if ( cardDirection == "RIGHT" ) { handyProc.actionCardDirection = DIRECTION::RIGHT ; }
					else if ( cardDirection == "UP"    ) { handyProc.actionCardDirection = DIRECTION::UP    ; }
					else if ( cardDirection == "DOWN"  ) { handyProc.actionCardDirection = DIRECTION::DOWN  ; }
					
					handyProc.actionAffectsX = Utils_Convert::String_to_Int ( x, 0 );
					handyProc.actionAffectsY = Utils_Convert::String_to_Int ( y, 0 );
					handyProc.actionAffectsStatus = Convert_string_to_throuableInfoEx ( status );
					handyProc.actionCardId        = cardId;
					
				} else if (tagList[i].childList[n].name == "POINT") {
					
					string point = tagList[i].childList[n].value;
					handyProc.point = Utils_Convert::String_to_Int ( point, 0 );
					
				} else if (tagList[i].childList[n].name == "POSITION_BIAS") {
					
					string x          = NotXML::GetAttribute ( tagList[i].childList[n], "x", "0" );
					string y          = NotXML::GetAttribute ( tagList[i].childList[n], "y", "0" );
					string advantage  = NotXML::GetAttribute ( tagList[i].childList[n], "advantage", "0" );
					string risk       = NotXML::GetAttribute ( tagList[i].childList[n], "risk", "0" );
					
					HandyProc_PositionBias positionBias;
					positionBias.x             = Utils_Convert::String_to_Int ( x );
					positionBias.y             = Utils_Convert::String_to_Int ( y );
					positionBias.advantageBias = Utils_Convert::String_to_Int ( advantage );
					positionBias.riskBias      = Utils_Convert::String_to_Int ( risk );
					
					handyProc.positionBiasList.push_back ( positionBias );
				
				} else if (tagList[i].childList[n].name == "GRID_STATUS_LIST") {
					
					for (unsigned int nn = 0; nn < tagList[i].childList[n].childList.size(); ++nn) {
						
						if ( tagList[i].childList[n].childList[nn].name == "GRID") {
							
							HandyProc_GridStatus gridStatus;
							
							string x                 = NotXML::GetAttribute ( tagList[i].childList[n].childList[nn], "x", "0" );
							string y                 = NotXML::GetAttribute ( tagList[i].childList[n].childList[nn], "y", "0" );
							string throughableInfoEx = NotXML::GetAttribute ( tagList[i].childList[n].childList[nn], "THROUGHABLE_INFO_EX", "0" );
							
								gridStatus.x                   = Utils_Convert::String_to_Int ( x, 0 );
								gridStatus.y                   = Utils_Convert::String_to_Int ( y, 0 );
								gridStatus.THROUGHABLE_INFO_EX = Convert_string_to_throuableInfoEx ( throughableInfoEx );
							
							handyProc.gridStatusList.push_back ( gridStatus );
						
						}
						
					}
					
				}
			}
			
			
			result.handyProcList.push_back ( handyProc );
			
			
		
		} else if ( tagList[i].name == "POSITION_SETTING" ) {
			
			for (unsigned int n = 0; n < tagList[i].childList.size(); ++n) {
				
				NotXML::Tag pTag = tagList[i].childList[n];
				
				if        (pTag.name == "ENABLED") {
				
					result.parameters_PositionEvaluation.isEnabled = Utils_Convert::String_to_bool ( pTag.value);
					
				} else if (pTag.name == "APPLY_FOR_TERRAIN_SEARCH") {
					
					result.parameters_PositionEvaluation.applyBias_forTerrain = Utils_Convert::String_to_Int ( pTag.value );
					
				} else if (pTag.name == "APPLY_FOR_PLAIN") {
				
					result.parameters_PositionEvaluation.applyBias_forPlain = Utils_Convert::String_to_Int ( pTag.value );
					
				} else if (pTag.name == "ATTACK_BIAS") {
				
					result.parameters_PositionEvaluation.attackBias = Utils_Convert::String_to_Int ( pTag.value );
					
				} else if (pTag.name == "GUARD_BIAS") {
				
					result.parameters_PositionEvaluation.guardBias = Utils_Convert::String_to_Int ( pTag.value );
					
				} else if (pTag.name == "SUB") {
					
					Parameters_PositionEvaluation_Sub sub;
					
					for (unsigned int j = 0; j < pTag.childList.size(); ++j) {
						
						NotXML::Tag subTag = pTag.childList[j];
						
						
						if ( subTag.name == "THROUABLE" ) {
							
							sub.THROUABLE_Risk      = Utils_Convert::String_to_Int ( NotXML::GetAttribute ( subTag, "risk", "0" ) );
							sub.THROUABLE_Advantage = Utils_Convert::String_to_Int ( NotXML::GetAttribute ( subTag, "advantage", "0" ) );
							
						} else if ( subTag.name == "NOT_THROUABLE" ) {
						
							sub.NOT_THROUABLE_Risk      = Utils_Convert::String_to_Int ( NotXML::GetAttribute ( subTag, "risk", "0" ) );
							sub.NOT_THROUABLE_Advantage = Utils_Convert::String_to_Int ( NotXML::GetAttribute ( subTag, "advantage", "0" ) );
						
						} else if ( subTag.name == "MUST_TO_UP" ) {
						
							sub.MUST_TO_UP_Risk      = Utils_Convert::String_to_Int ( NotXML::GetAttribute ( subTag, "risk", "0" ) );
							sub.MUST_TO_UP_Advantage = Utils_Convert::String_to_Int ( NotXML::GetAttribute ( subTag, "advantage", "0" ) );
						
						} else if ( subTag.name == "MUST_TO_DOWN" ) {
						
							sub.MUST_TO_DOWN_Risk      = Utils_Convert::String_to_Int ( NotXML::GetAttribute ( subTag, "risk", "0" ) );
							sub.MUST_TO_DOWN_Advantage = Utils_Convert::String_to_Int ( NotXML::GetAttribute ( subTag, "advantage", "0" ) );
						
						} else if ( subTag.name == "MUST_TO_HORIZONTAL" ) {
						
							sub.MUST_TO_HORIZONTAL_Risk      = Utils_Convert::String_to_Int ( NotXML::GetAttribute ( subTag, "risk", "0" ) );
							sub.MUST_TO_HORIZONTAL_Advantage = Utils_Convert::String_to_Int ( NotXML::GetAttribute ( subTag, "advantage", "0" ) );
						
						} else if ( subTag.name == "MyPIECE_ADVANTAGE_START"    ) { sub.myPiece_AdvantageStart     = Utils_Convert::String_to_Int ( subTag.value, 0 );
						} else if ( subTag.name == "MyPIECE_THROUABLE"          ) { sub.myPiece_THROUABLE          = Utils_Convert::String_to_Int ( subTag.value, 0 );
						} else if ( subTag.name == "MyPIECE_NOT_THROUABLE"      ) { sub.myPiece_NOT_THROUABLE      = Utils_Convert::String_to_Int ( subTag.value, 0 );
						} else if ( subTag.name == "MyPIECE_MUST_TO_UP"         ) { sub.myPiece_MUST_TO_UP         = Utils_Convert::String_to_Int ( subTag.value, 0 );
						} else if ( subTag.name == "MyPIECE_MUST_TO_DOWN"       ) { sub.myPiece_MUST_TO_DOWN       = Utils_Convert::String_to_Int ( subTag.value, 0 );
						} else if ( subTag.name == "MyPIECE_MUST_TO_HORIZONTAL" ) { sub.myPiece_MUST_TO_HORIZONTAL = Utils_Convert::String_to_Int ( subTag.value, 0 );
						} else if ( subTag.name == "MyPIECE_AGAINST_MyPIECE"    ) { sub.myPiece_againstMyPiece     = Utils_Convert::String_to_Int ( subTag.value, 0 );
						} else if ( subTag.name == "MyPIECE_AGAINST_OpPIECE"    ) { sub.myPiece_againstOpPiece     = Utils_Convert::String_to_Int ( subTag.value, 0 );
						
						} else if ( subTag.name == "OpPIECE_RISK_START"         ) { sub.opPiece_RiskStart          = Utils_Convert::String_to_Int ( subTag.value, 0 );
						} else if ( subTag.name == "OpPIECE_THROUABLE"          ) { sub.opPiece_THROUABLE          = Utils_Convert::String_to_Int ( subTag.value, 0 );
						} else if ( subTag.name == "OpPIECE_NOT_THROUABLE"      ) { sub.opPiece_NOT_THROUABLE      = Utils_Convert::String_to_Int ( subTag.value, 0 );
						} else if ( subTag.name == "OpPIECE_MUST_TO_UP"         ) { sub.opPiece_MUST_TO_UP         = Utils_Convert::String_to_Int ( subTag.value, 0 );
						} else if ( subTag.name == "OpPIECE_MUST_TO_DOWN"       ) { sub.opPiece_MUST_TO_DOWN       = Utils_Convert::String_to_Int ( subTag.value, 0 );
						} else if ( subTag.name == "OpPIECE_MUST_TO_HORIZONTAL" ) { sub.opPiece_MUST_TO_HORIZONTAL = Utils_Convert::String_to_Int ( subTag.value, 0 );
						} else if ( subTag.name == "OpPIECE_AGAINST_MyPIECE"    ) { sub.opPiece_againstMyPiece     = Utils_Convert::String_to_Int ( subTag.value, 0 );
						} else if ( subTag.name == "OpPIECE_AGAINST_OpPIECE"    ) { sub.opPiece_againstOpPiece     = Utils_Convert::String_to_Int ( subTag.value, 0 );

						} else if ( subTag.name == "MyPIECE_RELIEF_POWER"    ) { sub.myPieceReliefPower = Utils_Convert::String_to_Int ( subTag.value, 0 );
						}
						
					}
					
					
					string direction = NotXML::GetAttribute ( pTag, "direction", "" );
					
					if      ( direction == "UPPER_FRONT" ) { result.parameters_PositionEvaluation.upperFront = sub; }
					else if ( direction == "UPPER_LEFT"  ) { result.parameters_PositionEvaluation.upperLeft  = sub; }
					else if ( direction == "UPPER_RIGHT" ) { result.parameters_PositionEvaluation.upperRight = sub; }
					else if ( direction == "UNDER_FRONT" ) { result.parameters_PositionEvaluation.underFront = sub; }
					else if ( direction == "UNDER_LEFT"  ) { result.parameters_PositionEvaluation.underLeft  = sub; }
					else if ( direction == "UNDER_RIGHT" ) { result.parameters_PositionEvaluation.underRight = sub; }
					else {
						// おかしいですね
						printf ( "Unknown direction in POSITION_SETTING SUB direction=\"%s\" \n", direction.c_str());
					}
					
				}

			}
			
		} else if ( tagList[i].name == "POSITION_CARE" ) {
			
			
			for (unsigned int n = 0; n < tagList[i].childList.size(); ++n) {
				
				NotXML::Tag pcTag = tagList[i].childList[n];
				
				if ( pcTag.name == "ENABLED" ) {
					result.positionCareSettings.isEnabled = Utils_Convert::String_to_bool ( pcTag.value );
				}
				else if ( pcTag.name == "APPLY_FOR_TERRAIN_SEARCH" ) {
					
					result.positionCareSettings.applyBias_forTerrain = Utils_Convert::String_to_Int ( pcTag.value );
					
				}
				else if ( pcTag.name == "APPLY_FOR_PLAIN" ) {
					
					result.positionCareSettings.applyBias_forPlain = Utils_Convert::String_to_Int ( pcTag.value );
					
				}
				else if ( pcTag.name == "POSITIONS" ) {
					
					result.positionCareSettings.positionList.clear();
					
					for ( unsigned int j = 0; j < pcTag.childList.size(); ++j ) {
						
						if ( pcTag.childList[j].name == "POSITION" ) {
						
							NotXML::Tag pointTag = pcTag.childList[j];
							
							string x         = NotXML::GetAttribute ( pointTag, "x", "0" );
							string y         = NotXML::GetAttribute ( pointTag, "y", "0" );
							string advantage = NotXML::GetAttribute ( pointTag, "advantage", "0" );
							string risk      = NotXML::GetAttribute ( pointTag, "risk", "0" );
							
							PositionCare positionCare;
							positionCare.x             = Utils_Convert::String_to_Int ( x );
							positionCare.y             = Utils_Convert::String_to_Int ( y );
							positionCare.advantageBias = Utils_Convert::String_to_Int ( advantage );
							positionCare.riskBias      = Utils_Convert::String_to_Int ( risk );
							
							result.positionCareSettings.positionList.push_back ( positionCare );
						}
					}
				}
				
			}
			
			
		} else if ( tagList[i].name == "PRESSURE_BALANCE_SETTING" ) {
			
			for (unsigned int n = 0; n < tagList[i].childList.size(); ++n) {
				
				NotXML::Tag pbTag = tagList[i].childList[n];
				
				if ( pbTag.name == "ENABLED" ) {
					result.parameters_PressureEvaluation.isEnabled = Utils_Convert::String_to_bool ( pbTag.value );
				}
				else if ( pbTag.name == "TYPE" ) {
					
					if ( pbTag.value == "COLUMN" ) {
						result.parameters_PressureEvaluation.type = result.parameters_PressureEvaluation.TYPE_COLUMN;
					}
					else if ( pbTag.value == "MIRROR" ) {
						result.parameters_PressureEvaluation.type = result.parameters_PressureEvaluation.TYPE_MIRROR;
					}
					
				}
				else if ( pbTag.name == "GAME_PROGRESS" ) {
					
					string from   = NotXML::GetAttribute ( pbTag, "from", "0" );
					string to     = NotXML::GetAttribute ( pbTag, "to", "0" );
					string skip   = NotXML::GetAttribute ( pbTag, "skip", "" );
					string forced = NotXML::GetAttribute ( pbTag, "forced", "" );
					
					result.parameters_PressureEvaluation.enabledTurn_from = Utils_Convert::String_to_Int ( from );
					result.parameters_PressureEvaluation.enabledTurn_to   = Utils_Convert::String_to_Int ( to );
					
					if ( skip   != "" ) { result.parameters_PressureEvaluation.enabledTurn_skip.clear(); }
					if ( forced != "" ) { result.parameters_PressureEvaluation.enabledTurn_forced.clear(); }
					
					vector<string> skipList   = Utils_String::Split ( skip, "," );
					vector<string> forcedList = Utils_String::Split ( forced, "," );
					
					for ( unsigned int j = 0; j < skipList.size(); ++j ) {
						string skipTurnStr = Utils_String::Trim ( skipList[j] );
						if ( skipTurnStr == "" ) { continue; }
						
						int     skipTurn    = Utils_Convert::String_to_Int ( skipTurnStr );
						result.parameters_PressureEvaluation.enabledTurn_skip.push_back ( skipTurn );
					}
					for ( unsigned int j = 0; j < forcedList.size(); ++j ) {
						
						string forcedTurnStr = Utils_String::Trim ( forcedList[j] );
						if ( forcedTurnStr == "" ) { continue; }
						
						int     forcedTurn    = Utils_Convert::String_to_Int ( forcedTurnStr );
						result.parameters_PressureEvaluation.enabledTurn_forced.push_back ( forcedTurn );
					}
					
				}
				else if ( pbTag.name == "POINT_INCREASE_PRESSURING_COLUMNS" ) {
					result.parameters_PressureEvaluation.pointForIncreasePressuringNum = Utils_Convert::String_to_Int (pbTag.value );
				}
				else if ( pbTag.name == "POINT_DECREASE_PRESSURED_COLUMNS" ) {
					result.parameters_PressureEvaluation.pointForDecreasePressuredNum = Utils_Convert::String_to_Int (pbTag.value );
				
				}
				else if ( pbTag.name == "POINT_MIRROR_MOVE" ) {
					result.parameters_PressureEvaluation.pointForMirrorMove = Utils_Convert::String_to_Int (pbTag.value );
				
				}
				else if ( pbTag.name == "BIAS" ) {
					
					string m = NotXML::GetAttribute ( pbTag, "move", "" );
					string v = NotXML::GetAttribute ( pbTag, "value", "100" );
					
					int value = Utils_Convert::String_to_Int ( v );
					
					if      ( m == "iinogare_d" ) { result.parameters_PressureEvaluation.bias_iinogare_d = value; }
					else if ( m == "iinogare_h" ) { result.parameters_PressureEvaluation.bias_iinogare_h = value; }
					else if ( m == "keno_d"     ) { result.parameters_PressureEvaluation.bias_keno_d     = value; }
					else if ( m == "keno_h"     ) { result.parameters_PressureEvaluation.bias_keno_h     = value; }
					else if ( m == "take_card"  ) { result.parameters_PressureEvaluation.bias_take_card  = value; }
					else if ( m == "others"     ) { result.parameters_PressureEvaluation.bias_others     = value; }
					
				}
				
				
			}
		} else if ( tagList[i].name == "PICK_RESULT_FROM" ) {
			
			
			for (unsigned int n = 0; n < tagList[i].childList.size(); ++n) {
				
				NotXML::Tag pTag = tagList[i].childList[n];
				
				if ( pTag.name == "ENABLED" ) {
					result.pickResultFrom.isEnabled = Utils_Convert::String_to_bool ( pTag.value );
				}
				else if ( pTag.name == "GAME_PROGRESS" ) {
					
					string from = NotXML::GetAttribute ( pTag, "from", "" );
					string to   = NotXML::GetAttribute ( pTag, "to", "" );
					
					if ( from == "" ) { result.pickResultFrom.enabledTurn_from = 0; }
					else              { result.pickResultFrom.enabledTurn_from = Utils_Convert::String_to_Int (from); }
					
					if ( to   == "" ) { result.pickResultFrom.enabledTurn_to = -1; }
					else              { result.pickResultFrom.enabledTurn_to = Utils_Convert::String_to_Int (to); }
					
				}
				else if ( pTag.name == "TOP" ) {
					
					result.pickResultFrom.topLength = Utils_Convert::String_to_Int (pTag.value);
					
				}
			}
			
		
		
		} else if ( tagList[i].name == "DISTANCE_TO_GOAL" ) {
			
			for (unsigned int n = 0; n < tagList[i].childList.size(); ++n) {
				
				NotXML::Tag dTag = tagList[i].childList[n];
				
				if ( dTag.name == "ENABLED" ) {
					result.parameters_DistanceToGoal.isEnabled = Utils_Convert::String_to_bool ( dTag.value );
				}
				else if ( dTag.name == "GOAL_HEIGHT" ) {
					result.parameters_DistanceToGoal.goalHeight = Utils_Convert::String_to_Int( dTag.value );
				}
				else if ( dTag.name == "ATTACK_BIAS" ) {
					result.parameters_DistanceToGoal.attackBias = Utils_Convert::String_to_Int( dTag.value );
				}
				else if ( dTag.name == "GUARD_BIAS" ) {
					result.parameters_DistanceToGoal.guardBias = Utils_Convert::String_to_Int( dTag.value );
				}
				else if ( dTag.name == "DISTANCE_LIMIT" ) {
					result.parameters_DistanceToGoal.distanceLimit = Utils_Convert::String_to_Int( dTag.value );
				}
				else if ( dTag.name == "MOVE_COST" ) {
					
					string type     = NotXML::GetAttribute ( dTag, "type", "" );
					string valueStr = NotXML::GetAttribute ( dTag, "value", "0" );
					int     value = Utils_Convert::String_to_Int( valueStr );
					
					if      ( type == "directionChange" ) { result.parameters_DistanceToGoal.moveCostDirectionChange = value; }
					else if ( type == "arrow"           ) { result.parameters_DistanceToGoal.moveCostArrow           = value; }
					else if ( type == "fromBlank"       ) { result.parameters_DistanceToGoal.moveCostFromBlank       = value; }
					else if ( type == "toBlank"         ) { result.parameters_DistanceToGoal.moveCostToBlank         = value; }
					else if ( type == "myPiece"         ) { result.parameters_DistanceToGoal.moveCostMyPiece         = value; }
					else if ( type == "myPieceClose"    ) { result.parameters_DistanceToGoal.moveCostMyPieceClose    = value; }
					else if ( type == "opPiece"         ) { result.parameters_DistanceToGoal.moveCostOpPiece         = value; }
					else if ( type == "opPieceClose"    ) { result.parameters_DistanceToGoal.moveCostOpPieceClose    = value; }
					
				}
				else if ( dTag.name == "HAND_PIECE_BIAS" ) {
				
					string type     = NotXML::GetAttribute ( dTag, "type", "0" );
					string valueStr = NotXML::GetAttribute ( dTag, "value", "0" );
					int     value    = Utils_Convert::String_to_Int( valueStr );
					
					if      ( type == "exist"  ) { result.parameters_DistanceToGoal.handPieceBias_exist  = value; }
					else if ( type == "absent" ) { result.parameters_DistanceToGoal.handPieceBias_absent = value; }
				}
				else if ( dTag.name == "DISTANCE_POINTS" ) {
					
					result.parameters_DistanceToGoal.distanceRateList.clear();
					
					for (unsigned int j = 0; j < dTag.childList.size(); ++j) {
						
						NotXML::Tag pTag = dTag.childList[j];
						
						if ( pTag.name == "POINT" ) {
							
							int point = Utils_Convert::String_to_Int( pTag.value );
							
							result.parameters_DistanceToGoal.distanceRateList.push_back ( point );
						}
					}
				}
				
				///// 絞込み戦略
				else if ( dTag.name == "DIFF_SCREENING" ) {
					
					for (unsigned int j = 0; j < dTag.childList.size(); ++j) {
						
						NotXML::Tag dsTag = dTag.childList[j];
						
						if ( dsTag.name == "ENABLED" ) {
							result.parameters_DistanceToGoal.distanceDiffScreening.isEnabled = Utils_Convert::String_to_bool ( dsTag.value );
						}
						else if ( dsTag.name == "GAME_PROGRESS" ) {
					
							string from   = NotXML::GetAttribute ( dsTag, "from", "0" );
							string to     = NotXML::GetAttribute ( dsTag, "to", "" );
							
							result.parameters_DistanceToGoal.distanceDiffScreening.enabledTurn_from = Utils_Convert::String_to_Int ( from );
							
							if ( to != "" ) {
								result.parameters_DistanceToGoal.distanceDiffScreening.enabledTurn_to = Utils_Convert::String_to_Int ( to );
							} else {
								result.parameters_DistanceToGoal.distanceDiffScreening.enabledTurn_to = -1; // 制限なしの意味
							}
							
						}
						else if ( dsTag.name == "SCREENING_LENGTH" ) {
							result.parameters_DistanceToGoal.distanceDiffScreening.screeningLength = Utils_Convert::String_to_Int( dsTag.value );
						}
						else if ( dsTag.name == "PUT_PIECE_COST" ) {
							result.parameters_DistanceToGoal.distanceDiffScreening.putPieceCost = Utils_Convert::String_to_Int( dsTag.value );
						}
						else if ( dsTag.name == "APPLY_FOR_TERRAIN_SEARCH" ) {
							result.parameters_DistanceToGoal.distanceDiffScreening.bias_TerrainEvaluate = Utils_Convert::String_to_Int( dsTag.value );
						}
						else if ( dsTag.name == "ACHIEVE_POINT" ) {
							
							string status = NotXML::GetAttribute ( dsTag, "status", "" );
							string type   = NotXML::GetAttribute ( dsTag, "type",  "" );
							string value  = NotXML::GetAttribute ( dsTag, "value", "0" );
							
							if ( status == "ahead" ) {
								
								if ( type == "forMe" ) {
									result.parameters_DistanceToGoal.distanceDiffScreening.achievePoint_Ahead_forMe = Utils_Convert::String_to_Int( value );
								}
								else if ( type == "againstOp" ) {
									result.parameters_DistanceToGoal.distanceDiffScreening.achievePoint_Ahead_againstOp = Utils_Convert::String_to_Int( value );
								}
							}
							else if ( status == "backward" ) {
								
								if ( type == "forMe" ) {
									result.parameters_DistanceToGoal.distanceDiffScreening.achievePoint_Backward_forMe = Utils_Convert::String_to_Int( value );
								}
								else if ( type == "againstOp" ) {
									result.parameters_DistanceToGoal.distanceDiffScreening.achievePoint_Backward_againstOp = Utils_Convert::String_to_Int( value );
								}
								else if ( type == "turnBack" ) {
									result.parameters_DistanceToGoal.distanceDiffScreening.achievePoint_Backward_turnBack = Utils_Convert::String_to_Int( value );
								}
							}
							
						}
						else if ( dsTag.name == "OneStepFutureTelling" ) {
							
							string status   = NotXML::GetAttribute ( dsTag, "status", "" );
							string type     = NotXML::GetAttribute ( dsTag, "type",  "" );
							string enabled  = NotXML::GetAttribute ( dsTag, "enabled", "off" );
							
							if ( status == "ahead" ) {
								
								if ( type == "forMe" ) {
									result.parameters_DistanceToGoal.distanceDiffScreening.oneStepFutureTelling_Ahead_forMe = Utils_Convert::String_to_bool( enabled );
								}
								else if ( type == "againstOp" ) {
									result.parameters_DistanceToGoal.distanceDiffScreening.oneStepFutureTelling_Ahead_againstOp = Utils_Convert::String_to_bool( enabled );
								}
							}
							else if ( status == "backward" ) {
								
								if ( type == "forMe" ) {
									result.parameters_DistanceToGoal.distanceDiffScreening.oneStepFutureTelling_Backward_forMe = Utils_Convert::String_to_bool( enabled );
								}
								else if ( type == "againstOp" ) {
									result.parameters_DistanceToGoal.distanceDiffScreening.oneStepFutureTelling_Backward_againstOp = Utils_Convert::String_to_bool( enabled );
								}
							}
							
						}
						else if ( dsTag.name == "DISTANCE_OFFSET" ) {
							
							string type   = NotXML::GetAttribute ( dsTag, "type",  "" );
							string value  = NotXML::GetAttribute ( dsTag, "value", "0" );
							
							if  ( type == "forMe" ) {
								result.parameters_DistanceToGoal.distanceDiffScreening.distanceOffset_forMe = Utils_Convert::String_to_Int( value );
							}
							else if  ( type == "forOp" ) {
								result.parameters_DistanceToGoal.distanceDiffScreening.distanceOffset_forOp = Utils_Convert::String_to_Int( value );
							}
						}
						else if ( dsTag.name == "HAND_PIECE_CONSIDERING_METHOD" ) {
							
							string onlyWhenNoPieceOnBoard = NotXML::GetAttribute ( dsTag, "onlyWhenNoPieceOnBoard",  "false" );
							string method                 = NotXML::GetAttribute ( dsTag, "method", "" );
							string from                   = NotXML::GetAttribute ( dsTag, "from", "0" );
							string to                     = NotXML::GetAttribute ( dsTag, "to", "0" );
							string than                   = NotXML::GetAttribute ( dsTag, "than", "-1" );
							
							result.parameters_DistanceToGoal.distanceDiffScreening.handPieceConsideringMethod_onlyWhenNoPieceOnBoard = Utils_Convert::String_to_bool( onlyWhenNoPieceOnBoard );
							result.parameters_DistanceToGoal.distanceDiffScreening.handPieceConsideringMethod_sweepFrom              = Utils_Convert::String_to_Int( from );
							result.parameters_DistanceToGoal.distanceDiffScreening.handPieceConsideringMethod_sweepTo                = Utils_Convert::String_to_Int( to   );
							result.parameters_DistanceToGoal.distanceDiffScreening.handPieceConsideringMethod_than                   = Utils_Convert::String_to_Int( than );
							
							if ( method == "" || method == "NONE" ) {
								result.parameters_DistanceToGoal.distanceDiffScreening.handPieceConsideringMethod = result.parameters_DistanceToGoal.distanceDiffScreening.handPieceConsideringMethod_NONE;
							}
							else if ( method == "SWEEP" ) {
								result.parameters_DistanceToGoal.distanceDiffScreening.handPieceConsideringMethod = result.parameters_DistanceToGoal.distanceDiffScreening.handPieceConsideringMethod_SWEEP;
							}
							else if ( method == "ALL_BOTTOM_GRIDS" ) {
								result.parameters_DistanceToGoal.distanceDiffScreening.handPieceConsideringMethod = result.parameters_DistanceToGoal.distanceDiffScreening.handPieceConsideringMethod_ALL_BOTTOM_GRIDS;
							}
							else if ( method == "MORE_THROUABLE_COLUMNS" ) {
								result.parameters_DistanceToGoal.distanceDiffScreening.handPieceConsideringMethod = result.parameters_DistanceToGoal.distanceDiffScreening.handPieceConsideringMethod_MORE_THROUABLE_COLUMNS;
							}
							
						}
						
					}
					
					
				}
				
				
			}
			
		}
	}
	return result;
}

//==========================================================
// 地形探索パラメータタグ を 構造体にする
//==========================================================
TerrainSearchUsage AI::Read_TerrainSearchTag ( const NotXML::Tag &terrainTag ) {

	TerrainSearchUsage result;

	///// 別のタグだったら？ 中身が一致しているなら、無碍に門前払いする必要はない
	if ( terrainTag.name != "TERRAIN_SEARCH" ) {
		printf("TERRAIN_SEARCH タグじゃない？ %s\n", terrainTag.name.c_str());
	}

	///// 中身をみる
	for ( unsigned int i = 0; i < terrainTag.childList.size(); ++i ) {

		// 深度
		if ( terrainTag.childList[i].name == "DEPTH") {

			result.depth = Utils_Convert::String_to_Int( terrainTag.childList[i].value );

		// バイアス：地形探索の評価を重く見る程度
		} else if ( terrainTag.childList[i].name == "TERRAIN_EVALUATE_BIAS") {

			result.bias_TerrainEvaluate = Utils_Convert::String_to_Int ( terrainTag.childList[i].value );

		// バイアス：PEE単体での評価を重く見る程度
		} else if ( terrainTag.childList[i].name == "PEE_BIAS") {

			result.bias_PeeEvaluate = Utils_Convert::String_to_Int ( terrainTag.childList[i].value );

		// 特殊条件：未実装
		} else if ( terrainTag.childList[i].name == "SPECIAL_CONDITION") {

		}

	}
	return result;
}

//==========================================================
// 柱と傘評価設定タグ を 構造体にする
//==========================================================
PC_Evaluate_Settings AI::Read_PC_Evaluate_SettingsTag ( const NotXML::Tag &pcEvalTag, PC_Evaluate_Settings inheritBase) {

	PC_Evaluate_Settings result = inheritBase;

	///// 別のタグだったら？ 中身が一致しているなら、無碍に門前払いする必要はない
	if ( pcEvalTag.name != "PC_SETTING" ) {
		printf("PC_SETTING タグじゃない？ %s\n", pcEvalTag.name.c_str());
	}

	///// 中身をみる
	for ( unsigned int i = 0; i < pcEvalTag.childList.size(); ++i ) {

		// 有効・無効
		if ( pcEvalTag.childList[i].name == "ENABLED") {

			result.isEnabled = Utils_Convert::String_to_bool( pcEvalTag.childList[i].value );
		}
		// 柱の危険性・先端の、ゴールからの距離。タグが登場した順に距離0から
		/*
		else if ( pcEvalTag.childList[i].name == "GOAL_VALUE") {

			int value = Utils_Convert::String_to_Int( pcEvalTag.childList[i].value );
			result.goalValueList.push_back ( value );
		}
		*/
		// 駒をぶつける手に限定して一歩だけ先読み
		else if ( pcEvalTag.childList[i].name == "OneStepFutureTelling") {

			result.oneStepFutureTelling = Utils_Convert::String_to_bool( pcEvalTag.childList[i].value );
		}
		else if ( pcEvalTag.childList[i].name == "GOAL_VALUES") {
			
			result.goalValueList.clear();
			
			for ( unsigned int n = 0; n < pcEvalTag.childList[i].childList.size(); ++n ) {
				
				if ( pcEvalTag.childList[i].childList[n].name == "GOAL_VALUE" ) {
					int value = Utils_Convert::String_to_Int( pcEvalTag.childList[i].childList[n].value );
					result.goalValueList.push_back ( value );
				}
				
			}
			
		}
		// 駒の重み：柱の危険性を引き出す割合
		else if ( pcEvalTag.childList[i].name == "PIECE_WEIGHT") {

			string state = NotXML::GetAttribute( pcEvalTag.childList[i], "state" );
			int value = Utils_Convert::String_to_Int( pcEvalTag.childList[i].value );

			if        ( state == "IN_THE_BOX"             ) { result.pieceWeight_IN_THE_BOX             = value;
			} else if ( state == "ABOVE_COVER"            ) { result.pieceWeight_ABOVE_COVER            = value;
			} else if ( state == "ABOVE_COVER_DIAGONALLY" ) { result.pieceWeight_ABOVE_COVER_DIAGONALLY = value;
			} else if ( state == "UNDER_COVER_SIDE"       ) { result.pieceWeight_UNDER_COVER_SIDE       = value;
			} else if ( state == "UNDER_COVER"            ) { result.pieceWeight_UNDER_COVER            = value;
			} else if ( state == "NOT_INVOLVED"           ) { result.pieceWeight_NOT_INVOLVED           = value;
			}

		// １つの柱に対する危険度加算アルゴリズム
		} else if ( pcEvalTag.childList[i].name == "ALGORISM_FOR_RISK_DEGREE_ADDITION_TO_A_PILLAR") {

			if      ( pcEvalTag.childList[i].value == "MAX_ONLY" ) { result.addingAlgorism = PILLAR_COVER::ALGORISM_FOR_RISK_DEGREE_ADDITION_TO_A_PILLAR::MAX_ONLY; }
			else if ( pcEvalTag.childList[i].value == "ADD"      ) { result.addingAlgorism = PILLAR_COVER::ALGORISM_FOR_RISK_DEGREE_ADDITION_TO_A_PILLAR::ADD;      }

		// 柱として必要な高さ
		} else if ( pcEvalTag.childList[i].name == "REQUIRED_HEIGHT") {
			result.reqiredHeightForPillar = Utils_Convert::String_to_Int( pcEvalTag.childList[i].value );
		// 攻めバイアス
		} else if ( pcEvalTag.childList[i].name == "ATTACK_BIAS") {
			result.attackBias = Utils_Convert::String_to_Int( pcEvalTag.childList[i].value );
		// 守りバイアス
		} else if ( pcEvalTag.childList[i].name == "GUARD_BIAS") {
			result.guardBias = Utils_Convert::String_to_Int( pcEvalTag.childList[i].value );
		// 駒の高さバイアス
		} else if ( pcEvalTag.childList[i].name == "PIECE_HEIGHT_WEIGHT") {
			result.pieceHeightWeight = Utils_Convert::String_to_Int( pcEvalTag.childList[i].value );
		// 適用度合い：地形探索の末端で
		} else if ( pcEvalTag.childList[i].name == "APPLY_FOR_TERRAIN_SEARCH") {
			result.applyBias_forTerrain = Utils_Convert::String_to_Int( pcEvalTag.childList[i].value );
		// 適用度合い：平地で
		} else if ( pcEvalTag.childList[i].name == "APPLY_FOR_PLAIN") {
			result.applyBias_forPlain = Utils_Convert::String_to_Int( pcEvalTag.childList[i].value );
		}

	}
	return result;
}

//==========================================================
// 手札評価パラメータタグ を 構造体にする
//==========================================================
HandCardEvaluate AI::Read_HandCardEvaluateTag ( const NotXML::Tag &handCardEvalTag ) {

	HandCardEvaluate result;

	///// 別のタグだったら？ 中身が一致しているなら、無碍に門前払いする必要はない
	if ( handCardEvalTag.name != "HAND_CARDS_VALUE" ) {
		printf("HAND_CARDS_VALUE タグじゃない？ %s\n", handCardEvalTag.name.c_str());
	}

	///// 中身をみる
	for ( unsigned int i = 0; i < handCardEvalTag.childList.size(); ++i ) {

		// 有効・無効
		if ( handCardEvalTag.childList[i].name == "ENABLED") {

			result.isEnabled = Utils_Convert::String_to_bool( handCardEvalTag.childList[i].value );

		// ポイント：カード
		} else if ( handCardEvalTag.childList[i].name == "CARD") {
			result.point_Card = Utils_Convert::String_to_Int( handCardEvalTag.childList[i].value );
		// ポイント：THROUGHABLE
		} else if ( handCardEvalTag.childList[i].name == "THROUGHABLE") {
			result.point_Throughable = Utils_Convert::String_to_Int( handCardEvalTag.childList[i].value );
		// ポイント：MUST_GO_SOMEWAY
		} else if ( handCardEvalTag.childList[i].name == "MUST_GO_SOMEWAY") {
			result.point_MustGoSomeway = Utils_Convert::String_to_Int( handCardEvalTag.childList[i].value );
		}

	}
	return result;
}

//==========================================================
// 手札評価パラメータタグ を 構造体にする
//==========================================================
TakeCardMoveEvaluate AI::Read_TakeCardMoveEvaluateTag ( const NotXML::Tag &takeCardMoveEvalTag ) {

	TakeCardMoveEvaluate result;

	///// 別のタグだったら？ 中身が一致しているなら、無碍に門前払いする必要はない
	if ( takeCardMoveEvalTag.name != "TAKE_CARD_MOVE_EVALUATE" ) {
		printf("TAKE_CARD_MOVE_EVALUATE タグじゃない？ %s\n", takeCardMoveEvalTag.name.c_str());
	}

	///// 中身をみる
	for ( unsigned int i = 0; i < takeCardMoveEvalTag.childList.size(); ++i ) {

		// 有効・無効
		if ( takeCardMoveEvalTag.childList[i].name == "ENABLED") {

			result.isEnabled = Utils_Convert::String_to_bool( takeCardMoveEvalTag.childList[i].value );

		// 発動条件：手札がこの枚数以下のとき発動
		} else if ( takeCardMoveEvalTag.childList[i].name == "HAND_CARDS_LESS_THAN") {
			result.handCardsLessThanNum = Utils_Convert::String_to_Int( takeCardMoveEvalTag.childList[i].value );

		// ポイント上乗せレート(%)
		} else if ( takeCardMoveEvalTag.childList[i].name == "ADD_RATE") {
			result.addRate = Utils_Convert::String_to_Int( takeCardMoveEvalTag.childList[i].value );

		// 詰めろの評価値
		} else if ( takeCardMoveEvalTag.childList[i].name == "NEXT_CHECK_MATE_VALUE") {
			result.nextCheckMateValue = Utils_Convert::String_to_Int( takeCardMoveEvalTag.childList[i].value );
		}

	}
	return result;
}

//==========================================================
// 詰めろ探索 タグ と 構造体を 相互変換
//==========================================================
PreMateSearchSetting AI::Read_PreMateSearchTag ( const NotXML::Tag &preMateTag ) {

	PreMateSearchSetting result;

	///// 別のタグだったら？ 中身が一致しているなら、無碍に門前払いする必要はない
	if ( preMateTag.name != "PRE_MATE_SEARCH" ) {
		printf("PRE_MATE_SEARCH タグじゃない？ %s\n", preMateTag.name.c_str());
	}

	///// 中身をみる
	for ( unsigned int i = 0; i < preMateTag.childList.size(); ++i ) {

		// 有効・無効
		if ( preMateTag.childList[i].name == "ENABLED") {

			result.isEnabled = Utils_Convert::String_to_bool( preMateTag.childList[i].value );

		// 探索深度
		} else if ( preMateTag.childList[i].name == "DEPTH") {

			result.depth = Utils_Convert::String_to_Int( preMateTag.childList[i].value );

		// 上位ｎ件に適用
		} else if ( preMateTag.childList[i].name == "SUBJECT_NUMBER") {
			result.subjectNum = Utils_Convert::String_to_Int( preMateTag.childList[i].value );

		// 詰めろの評価値
		} else if ( preMateTag.childList[i].name == "VALUE") {
			result.value = Utils_Convert::String_to_Int( preMateTag.childList[i].value );

		// 直接王手を含めるか？
		} else if ( preMateTag.childList[i].name == "INCLUDE_DIRECT_CHECK") {
			result.isIncludeDirectCheck = Utils_Convert::String_to_bool( preMateTag.childList[i].value );
		}

	}
	return result;
}


//==========================================================
// PEEタグを再生
//==========================================================
PieceEvaluateElement AI::PEE_Read_PEE_Tag ( const NotXML::Tag &peeTag, const map < string, PieceEvaluateElement > &variablePEEList ) {

	PieceEvaluateElement result;

	for (unsigned int i = 0; i < peeTag.childList.size(); ++i) {

		NotXML::Tag subTug = peeTag.childList[i];

		if (subTug.name == "THROUGHABLE_INFO::EX") {

			//result.throuableInfo = Utils_Convert::String_to_Int ( subTug.value );

			if ( subTug.value == "THROUGHABLE"     ) { result.throuableInfo = THROUGHABLE_INFO::EX::THROUGHABLE;     }
			if ( subTug.value == "NOT_THROUGHABLE" ) { result.throuableInfo = THROUGHABLE_INFO::EX::NOT_THROUGHABLE; }
			if ( subTug.value == "MUST_TO_UP"    ) { result.throuableInfo = THROUGHABLE_INFO::EX::MUST_TO_UP;    }
			if ( subTug.value == "MUST_TO_DOWN"  ) { result.throuableInfo = THROUGHABLE_INFO::EX::MUST_TO_DOWN;  }
			if ( subTug.value == "MUST_TO_LEFT"  ) { result.throuableInfo = THROUGHABLE_INFO::EX::MUST_TO_LEFT;  }
			if ( subTug.value == "MUST_TO_RIGHT" ) { result.throuableInfo = THROUGHABLE_INFO::EX::MUST_TO_RIGHT; }
			if ( subTug.value == "PIECE_EMPTY"   ) { result.throuableInfo = THROUGHABLE_INFO::EX::PIECE_EMPTY;   }
			if ( subTug.value == "PIECE_MY"      ) { result.throuableInfo = THROUGHABLE_INFO::EX::PIECE_MY;      }
			if ( subTug.value == "PIECE_OPPONENT") { result.throuableInfo = THROUGHABLE_INFO::EX::PIECE_OPPONENT;}
			if ( subTug.value == "OUT_OF_BOARD"  ) { result.throuableInfo = THROUGHABLE_INFO::EX::OUT_OF_BOARD;  }
			if ( subTug.value == "OPPONENT_AREA" ) { result.throuableInfo = THROUGHABLE_INFO::EX::OPPONENT_AREA; }
			if ( subTug.value == "CARD_TAKEABLE" ) { result.throuableInfo = THROUGHABLE_INFO::EX::CARD_TAKEABLE; }
		}
		else if (subTug.name == "X") {
			result.x = Utils_Convert::String_to_Int ( subTug.value );
		}
		else if (subTug.name == "Y") {
			result.y = Utils_Convert::String_to_Int ( subTug.value );
		}
		else if (subTug.name == "POINT") {
			result.point = Utils_Convert::String_to_Int ( subTug.value );
		}
		else if (subTug.name == "PEE") {

			PieceEvaluateElement child;

			// 変数から取り込むかどうか？
			string fromVariable = NotXML::GetAttribute( subTug, "FROM_VARIABLE");
			
			if ( fromVariable != "" ) {


				if (variablePEEList.find( fromVariable ) != variablePEEList.end() ) {

					child = variablePEEList.find( fromVariable )->second;

				} else {
					printf("指定の変数名で登録された素子がありません。%s\n", fromVariable.c_str());
				}

			} else {

				child = PEE_Read_PEE_Tag( subTug, variablePEEList);
			}


			result.relativeList.push_back( child );
		}

	}
	return result;
}

//==========================================================
// 駒評価用パラメータ群構造体をコピー
//==========================================================
Parameters_PieceEvaluation AI::Parameters_PEE_Copy( const Parameters_PieceEvaluation &original, int fluctuation, bool isPointOnly ) {

	/*
		ルートの兄弟素子は、消滅も発生もさせない、
		という手もある。
	*/

	Parameters_PieceEvaluation result;

	///// 高さバイアス
	result.heightValue_Bias = FluctuatedCopy ( original.heightValue_Bias, fluctuation, 20 );

	///// 位置関係素子
	for ( unsigned int i = 0; i < original.pieceEvaluateElementList.size(); ++i ) {

		// 消滅判定
		bool isVanish = false; //IsVanish ( original.pieceEvaluateElementList[i], fluctuation );

		// 消滅しないなら、コピー
		if (!isVanish) {

			PieceEvaluateElement copyPEE = PEE_Copy( original.pieceEvaluateElementList[i], fluctuation, isPointOnly);

			// ルート要素の特例として、位置は 0, 0 で固定
			copyPEE.x = 0;
			copyPEE.y = 0;

			result.pieceEvaluateElementList.push_back ( copyPEE );
		}
	}
	/*
	///// 新要素が発生するかどうか：ルート要素を一つの素子として考える
	PieceEvaluateElement forSpring;
	forSpring.throuableInfo = THROUGHABLE_INFO::EX::PIECE_MY; // パネルはこれでいい？？
	forSpring.x = 0;
	forSpring.y = 0;
	forSpring.relativeList = original.pieceEvaluateElementList;
	forSpring.point = forSpring.point += HowMuchPointDoSubElementsHave ( forSpring ); 

	///// 新要素発生判定
	vector<PieceEvaluateElement> newElements;
	SpringElement ( newElements, forSpring, fluctuation );

	///// 発生した新要素をルート要素に加える
	for ( unsigned int i = 0; i < newElements.size(); ++i ) {
		result.pieceEvaluateElementList.push_back( newElements[i] );
	}
	*/

	return result;
}
//==========================================================
// 駒評価素子をコピー。ゆらぎ値を指定 ( 0 で純正コピー )
//==========================================================
PieceEvaluateElement AI::PEE_Copy( const PieceEvaluateElement &original, int fluctuation, bool isPointOnly ) {

	PieceEvaluateElement result;

	// パネル条件：これは さしあたり、ゆらがない
	result.throuableInfo = original.throuableInfo;

	// ポイントのみ？
	if (isPointOnly) {

		// 加点値
		result.point = FluctuatedCopy ( original.point, fluctuation, 10);

		// 位置は変えない
		result.x     = original.x;
		result.y     = original.y;

	// ポイント以外も
	} else {

		// 位置、加点値
		result.x     = FluctuatedCopy ( original.x, fluctuation, 2);
		result.y     = FluctuatedCopy ( original.y, fluctuation, 2);
		result.point = FluctuatedCopy ( original.point, fluctuation, 10);

		// 位置を補正
		result.x = ( result.x >  2 ) ?  2 : result.x;
		result.x = ( result.x < -2 ) ? -2 : result.x;
		result.y = ( result.y >  2 ) ?  2 : result.y;
		result.y = ( result.y < -2 ) ? -2 : result.y;
	}

	// 子要素
	for (unsigned int i = 0; i < original.relativeList.size(); ++i) {

		// 消滅判定
		bool isVanish = false;
		if ( isPointOnly ) {
			isVanish = false;
		} else {
			isVanish = IsVanish ( original.relativeList[i], fluctuation );
		}

		// 消滅しないなら、コピー
		if ( !isVanish ) {
			PieceEvaluateElement copy = PEE_Copy( original.relativeList[i], fluctuation , isPointOnly );
			result.relativeList.push_back( copy );
		}
	}

	///// 新たな子要素が発生するかどうか
	if (!isPointOnly) {
		vector<PieceEvaluateElement> newElements;
		SpringElement ( newElements, original, fluctuation );

		///// 発生した新要素をルート要素に加える
		for ( unsigned int i = 0; i < newElements.size(); ++i ) {
			result.relativeList.push_back( newElements[i] );
		}
	}
	return result;
}

//==========================================================
// コピーの際、素子が発生するかどうか
//==========================================================
void AI::SpringElement( vector<PieceEvaluateElement> &result, const PieceEvaluateElement &PEE, int fluctuation ) {

	/*
		関係する要素は？

		子要素、なのですよね。発生するのは。
	*/

	// 施行回数：素子が発生し得る最大数
	int bombCount = Utils_Cal::GerRandom ( fluctuation / 5 );
	for ( int i = 0; i < bombCount; ++i ) {

		// 発生させる？
		bool areYouSure = false;
		for ( int n =0; n < fluctuation; ++n ) {

			// ここで元の加点値が、絡んでくる。
			if (Utils_Cal::MultipleCoinToss ( PEE.point, 50 )) {
				areYouSure = true;
				break;
			}
		}
		// やっぱり、やめておきます。
		if ( !areYouSure ) {
			continue;
		}

		// 距離：ゆらぎ値が大きいほど、距離の可能性の範囲が大きい。
		//int distanceMAX = (fluctuation / 10) + 1;
		//int distance = Utils_Cal::GetRandom( distanceMAX );
		int distance = FluctuatedCopy ( 0, fluctuation, 2 );

		// 距離ゼロの位置には子要素を発生させませんよ。
		if (distance == 0) {
			continue;
		}

		// 発生させるなら、この位置に発生させることになりました。
		pair<int, int> position = GetPOINT_From_2DBoard_Distance( PEE.x, PEE.y, distance );
		int positionX = position.first;
		int positionY = position.second;

		// パネル条件は何にします？
		int throuableInfoEX = GetThrouableInfo_AS_FCopy ( PEE.throuableInfo, fluctuation );

		// 加点値
		int point = 0;

		// パネル条件が同じなら、加点値は、こんなもん
		if ( throuableInfoEX ==  PEE.throuableInfo ) {

			// 元の値の半分、が出発点。
			point = PEE.point / 2;

			// 符号、変える？
			bool isChangeSign = false;
			for ( int n = 0; n < fluctuation; ++n ) {

				// 100回やって、何回ぐらいの割合で？
				if ( Utils_Cal::MultipleCoinToss ( 10, 100  ) ) {
					isChangeSign = true;
					break;
				}
			}
			if (isChangeSign) {
				point *= -1;
			}

			// ゆらぎの範囲
			int f_Max = (float)Utils_Cal::Abs( PEE.point) / 2.0f * 1.5f / 100.0f * (float)fluctuation;
			int f_Min = -1 * f_Max;

			point += Utils_Cal::GerRandom( f_Max - f_Min ) + f_Min;


		// パネル条件が違うなら、加点値は、こんな感じで
		} else {

			// 元の値、が出発点。
			point = PEE.point / 2;

			// 符号、変える？
			bool isChangeSign = Utils_Cal::MultipleCoinToss( 50, 100 );
			if (isChangeSign) {
				point *= -1;
			}

			// ゆらぎの範囲
			int f_Max = (float)Utils_Cal::Abs( PEE.point) * 1.2f / 100.0f * (float)fluctuation;
			int f_Min = -1 * f_Max;

			point += Utils_Cal::GerRandom( f_Max - f_Min ) + f_Min;
		}


		// 素子を発生させる
		PieceEvaluateElement newPEE;
		newPEE.throuableInfo = throuableInfoEX;
		newPEE.x = positionX;
		newPEE.y = positionY;
		newPEE.point = point;

		// 発生早々、子要素は、持たないでいいですよね？
		//newPEE.relativeList; 

		// 結果に追加
		result.push_back ( newPEE );
	}
}

//==========================================================
// THROUGHABLE_INFO::EX をゆらぎコピーする。
//==========================================================
int AI::GetThrouableInfo_AS_FCopy ( const int &original, int fluctuation ) {

	int result = original;

	// 同じの / ちがうの / 発生させない
	int entryBall_SamePanel      = 1;
	int entryBall_DifferentPanel = 2;

	vector<int> entryLots;
	for ( int n = 0; n < fluctuation; ++n ) {

		if ( Utils_Cal::MultipleCoinToss ( 1, 2 ) ) { entryLots.push_back ( entryBall_SamePanel);}
		if ( Utils_Cal::MultipleCoinToss ( 1, 6 ) ) { entryLots.push_back ( entryBall_DifferentPanel);}
	}


	if (entryLots.size() > 0) {

		int lotNum = Utils_Cal::GerRandom( entryLots.size());
		int lot = entryLots[lotNum];

		// 同じパネル条件の素子を発生させる
		if (lot == entryBall_SamePanel) {

			result = original;

		// 違うパネル条件の素子を発生させる
		} else if (lot == entryBall_DifferentPanel) {

			// 違う値になるまでくじびき。
			while ( result == original ) {

				int lotNum = Utils_Cal::GerRandom ( 11 );
				if      (lotNum == 0) { result = THROUGHABLE_INFO::EX::NOT_THROUGHABLE; }
				else if (lotNum == 1) { result = THROUGHABLE_INFO::EX::THROUGHABLE;     }
				else if (lotNum == 2) { result = THROUGHABLE_INFO::EX::MUST_TO_RIGHT; }
				else if (lotNum == 3) { result = THROUGHABLE_INFO::EX::MUST_TO_LEFT;  }
				else if (lotNum == 4) { result = THROUGHABLE_INFO::EX::MUST_TO_UP;    }
				else if (lotNum == 5) { result = THROUGHABLE_INFO::EX::MUST_TO_DOWN;  }
				else if (lotNum == 6) { result = THROUGHABLE_INFO::EX::PIECE_EMPTY;   }
				else if (lotNum == 7) { result = THROUGHABLE_INFO::EX::PIECE_MY;      }
				else if (lotNum == 8) { result = THROUGHABLE_INFO::EX::PIECE_OPPONENT;}
				else if (lotNum == 9) { result = THROUGHABLE_INFO::EX::OUT_OF_BOARD;  }
				else if (lotNum == 10){ result = THROUGHABLE_INFO::EX::CARD_TAKEABLE; }
			}
		}
	}


	return result;
}


//==========================================================
// 盤面上のグリッドとして、この距離のものをください。ランダムで。
//==========================================================
pair<int, int> AI::GetPOINT_From_2DBoard_Distance ( int x, int y, int distance ) {

	/*
		distance は正の数でもらえる前提。
	*/

	pair<int, int> result;
	int resultX = 0;
	int resultY = 0;

	///// 指定距離なのは、XかYか。
	if ( Utils_Cal::MultipleCoinToss( 50, 100 ) ) {

		// 距離って、プラスかマイナスか
		if ( Utils_Cal::MultipleCoinToss( 50, 100 ) ) {
			resultX = x + distance;
		} else {
			resultX = x - distance;
		}

		// もう片方は、距離以内ならいくつでもいい。
		int rangeMin = distance * -1;
		int rangeMax = distance;

		resultY = y + Utils_Cal::GerRandom( rangeMax - rangeMin ) + rangeMin;

	} else {
		// 距離って、プラスかマイナスか
		if ( Utils_Cal::MultipleCoinToss( 50, 100 ) ) {
			resultY = y + distance;
		} else {
			resultY = y - distance;
		}

		// もう片方は、距離以内ならいくつでもいい。
		int rangeMin = distance * -1;
		int rangeMax = distance;

		resultX = x + Utils_Cal::GerRandom( rangeMax - rangeMin ) + rangeMin;
	}

	result.first = resultX;
	result.second = resultY;

	return result;
}

//==========================================================
// ゆらぎつき値コピー
//==========================================================
int AI::FluctuatedCopy( int originalValue, int fluctuation, int fluctuateRange ) {

	/*
		fluctuation は 0〜100
	*/

	int result = originalValue;

	/*
	if ( fluctuation > 0 ) {

		// ゆらぎ最大値・最小値
		int fluctuation_MAX = (float)Utils_Cal::Abs(fluctuateRange) / 100.0f * (float)fluctuation;
		int fluctuation_MIN = (float)Utils_Cal::Abs(fluctuateRange) / 100.0f * (float)fluctuation * -1.0f;

		result = originalValue + Utils_Cal::GerRandom( fluctuation_MAX - fluctuation_MIN ) + fluctuation_MIN;

	}
	*/
	if ( fluctuation > 0 && fluctuateRange > 0) {

		// 何段階？
		int pace = 100.0f / (float)fluctuateRange;

		// 初期エントリー
		vector<int> entries;
		for ( int i = 0; i <= fluctuateRange; ++i ) {

			// i=0 のとき 100、i=max のとき 0 となるように。
			int entry = 100 - pace * i;

			entries.push_back( entry );
		}

		// ゆらぎ値でのエントリー
		for ( int i = 0; i < entries.size(); ++i ) {

			// ゆらぎ値=100 のとき、全エントリーが100となるように。
			entries[i] += fluctuation;
			if ( entries[i] > 100 ) {
				entries[i] = 100;
			}
		}

		// くじびきエントリー
		vector<int> lot;
		for ( int i = 0; i < entries.size(); ++i ) {

			for ( int n = 0; n < entries[i]; ++n ) {
				lot.push_back( i );
			}
		}

		// くじびき
		int lotNum = Utils_Cal::GerRandom( lot.size() );
		int prizeWinner = lot[ lotNum ];

		
		// 符号変化
		if ( Utils_Cal::MultipleCoinToss( 50, 100 ) ) {
			result = originalValue - prizeWinner;
		} else {
			result = originalValue + prizeWinner;
		}

	}
	return result;
}

//==========================================================
// コピーの際、素子が消滅するかどうか
//==========================================================
bool AI::IsVanish ( const PieceEvaluateElement &PEE, int fluctuation ) {

	// true で消滅
	bool result = false;

	///// 関係する値
	int pointABS = Utils_Cal::Abs( PEE.point);
	int subCount = HowManySubElement( PEE );
	int subPoint = HowMuchPointDoSubElementsHave( PEE );

	///// 爆撃回数：ゆらぎ 10以下だと、ほぼ消滅なし
	int bombCount = Utils_Cal::GerRandom ( fluctuation / 10 );

	///// 爆撃開始
	for ( int i = 0; i < bombCount; ++i ) {

		///// どの基準で裁く？
		int judgeCoinToss = Utils_Cal::GerRandom( 10 );

		///// 加点値で裁く
		if ( 0 <= judgeCoinToss && judgeCoinToss <= 2 ) {

			// ポイントが高いほど消滅しにくい（100点以上で、この理由では消滅しない）
			if ( Utils_Cal::MultipleCoinToss ( fluctuation * 1, pointABS ) ) {
				result = true;
				break;
			}

		///// 子要素の数で裁く
		} else if ( 3 <= judgeCoinToss && judgeCoinToss <= 6 ) {

			if ( Utils_Cal::MultipleCoinToss ( fluctuation / 30, subCount ) ) {
				result = true;
				break;
			}

		///// 子要素以下の加点値合計で裁く
		} else if ( 7 <= judgeCoinToss && judgeCoinToss <= 9 ) {
			
			if ( Utils_Cal::MultipleCoinToss ( fluctuation / 50, subPoint ) ) {
				result = true;
				break;
			}		
		}
	}

	return result;
}

//==========================================================
// 素子の子要素の数
//==========================================================
int AI::HowManySubElement ( const PieceEvaluateElement &PEE) {

	int result = 0;

	///// 子要素の数をプラス
	result += PEE.relativeList.size();

	///// 孫要素の数を再帰してプラス
	for ( unsigned int i = 0; i < PEE.relativeList.size(); ++i) {

		result += HowManySubElement ( PEE.relativeList[i] );
	}

	return result;
}
//==========================================================
// 子要素以下の、加点値、の合計
//==========================================================
int AI::HowMuchPointDoSubElementsHave ( const PieceEvaluateElement &PEE) {

	/*
		自分自身の加点値は含まない

		加算するのは絶対値。
	*/

	int result = 0;

	for ( unsigned int i = 0; i < PEE.relativeList.size(); ++i) {

		result += Utils_Cal::Abs( PEE.relativeList[i].point);

		result += HowMuchPointDoSubElementsHave ( PEE.relativeList[i] );
	}

	return result;
}

//==========================================================
// 指し手をタグ化
//==========================================================
NotXML::Tag AI::Move_Into_Tag ( const Move &move ) {

	NotXML::Tag result;
	result.name = "<Move>";

	///// TURN
	NotXML::Tag turn;
	turn.name = "TURN";
	if        (move.turn == TURN::BLACK) { turn.value = "BLACK";
	} else if (move.turn == TURN::WHITE) { turn.value = "WHITE";
	} else                               { turn.value = "NONE"; }
	result.childList.push_back( turn );

	///// MOVE::ACTION::PUT_PIECE
	if ( move.action == MOVE::ACTION::PUT_PIECE ) {

		NotXML::Tag action;
		action.name = "ACTION";
		action.value = "PUT_PIECE";
		result.childList.push_back( action );

		NotXML::Tag putPieceX;
		putPieceX.name = "putPieceX";
		putPieceX.value = Utils_Convert::Int_to_String ( move.putPieceX );
		result.childList.push_back( putPieceX );

		NotXML::Tag putPieceY;
		putPieceY.name = "putPieceY";
		putPieceY.value = Utils_Convert::Int_to_String ( move.putPieceY );
		result.childList.push_back( putPieceY );
	}

	///// MOVE::ACTION::PUT_CARD
	if ( move.action == MOVE::ACTION::PUT_CARD ) {

		NotXML::Tag action;
		action.name = "ACTION";
		action.value = "PUT_CARD";
		result.childList.push_back( action );

		NotXML::Tag putCardId;
		putCardId.name = "putCardId";
		putCardId.value = Utils_Convert::Int_to_String ( move.putCardId );
		result.childList.push_back( putCardId );

		NotXML::Tag putCardX;
		putCardX.name = "putCardX";
		putCardX.value = Utils_Convert::Int_to_String ( move.putCardX );
		result.childList.push_back( putCardX );

		NotXML::Tag putCardY;
		putCardY.name = "putCardY";
		putCardY.value = Utils_Convert::Int_to_String ( move.putCardY );
		result.childList.push_back( putCardY );

		NotXML::Tag putCardDirection;
		putCardDirection.name = "putCardDirection";
		putCardDirection.value = Utils_Convert::Int_to_String ( move.putCardDirection );
		result.childList.push_back( putCardDirection );
	}

	///// MOVE::ACTION::TAKE_CARD
	if ( move.action == MOVE::ACTION::TAKE_CARD ) {

		NotXML::Tag action;
		action.name = "ACTION";
		action.value = "TAKE_CARD";
		result.childList.push_back( action );

		NotXML::Tag takeCardX;
		takeCardX.name = "takeCardX";
		takeCardX.value = Utils_Convert::Int_to_String ( move.takeCardX );
		result.childList.push_back( takeCardX );

		NotXML::Tag takeCardY;
		takeCardY.name = "takeCardY";
		takeCardY.value = Utils_Convert::Int_to_String ( move.takeCardY );
		result.childList.push_back( takeCardY );
	}

	///// MOVE::ACTION::MOVE_PIECE
	if ( move.action == MOVE::ACTION::MOVE_PIECE ) {

		NotXML::Tag action;
		action.name = "ACTION";
		action.value = "MOVE_PIECE";
		result.childList.push_back( action );

		NotXML::Tag movePieceFromX;
		movePieceFromX.name = "movePieceFromX";
		movePieceFromX.value = Utils_Convert::Int_to_String ( move.movePieceFromX );
		result.childList.push_back( movePieceFromX );

		NotXML::Tag movePieceFromY;
		movePieceFromY.name = "movePieceFromY";
		movePieceFromY.value = Utils_Convert::Int_to_String ( move.movePieceFromY );
		result.childList.push_back( movePieceFromY );

		NotXML::Tag movePieceToX;
		movePieceToX.name = "movePieceToX";
		movePieceToX.value = Utils_Convert::Int_to_String ( move.movePieceToX );
		result.childList.push_back( movePieceToX );

		NotXML::Tag movePieceToY;
		movePieceToY.name = "movePieceToY";
		movePieceToY.value = Utils_Convert::Int_to_String ( move.movePieceToY );
		result.childList.push_back( movePieceToY );

		NotXML::Tag movePieceAndIsCllide;
		movePieceAndIsCllide.name = "movePieceAndIsCllide";
		movePieceAndIsCllide.value = (move.movePieceAndIsCllide) ? "true" : "false"; 
		result.childList.push_back( movePieceAndIsCllide );
	}

	///// MOVE::ACTION::PASS
	if ( move.action == MOVE::ACTION::PASS ) {

		NotXML::Tag action;
		action.name = "ACTION";
		action.value = "PASS";
		result.childList.push_back( action );
	}


	return result;
}
//==========================================================
// MoveタグからMove構造体を再現
//==========================================================
Move AI::Move_From_Tag ( const NotXML::Tag &moveTag ) {

	Move result;

	///// 本当にMoveタグですか？
	if (moveTag.name != "Move") {
		result.action = MOVE::ACTION::PASS;
		return result;
	}

	///// 中身のタグでループ
	for ( unsigned int i = 0; i < moveTag.childList.size(); ++i ) {

		NotXML::Tag subTag = moveTag.childList[i];

		if ( subTag.name == "ACTION" ) {
			if ( subTag.value == "PUT_PIECE"  ) { result.action = MOVE::ACTION::PUT_PIECE; }
			if ( subTag.value == "PUT_CARD"   ) { result.action = MOVE::ACTION::PUT_CARD; }
			if ( subTag.value == "MOVE_PIECE" ) { result.action = MOVE::ACTION::MOVE_PIECE; }
			if ( subTag.value == "TAKE_CARD"  ) { result.action = MOVE::ACTION::TAKE_CARD; }
			if ( subTag.value == "PASS"       ) { result.action = MOVE::ACTION::PASS; }
		}
		if ( subTag.name == "TURN" ) {
			if ( subTag.value == "BLACK"  ) { result.turn = TURN::BLACK; }
			if ( subTag.value == "WHITE"  ) { result.turn = TURN::WHITE; }
			if ( subTag.value == "NONE"  ) { result.turn = TURN::NONE; }
		}
		if ( subTag.name == "putPieceX" ) { result.putPieceX = Utils_Convert::String_to_Int( subTag.value );}
		if ( subTag.name == "putPieceY" ) { result.putPieceY = Utils_Convert::String_to_Int( subTag.value );}

		if ( subTag.name == "putCardX"         ) { result.putCardX = Utils_Convert::String_to_Int( subTag.value );}
		if ( subTag.name == "putCardY"         ) { result.putCardY = Utils_Convert::String_to_Int( subTag.value );}
		if ( subTag.name == "putCardId"        ) { result.putCardId = Utils_Convert::String_to_Int( subTag.value );}
		if ( subTag.name == "putCardName"      ) { 

			map<int, string>::iterator it = _cardNameList.begin();
			while ( it != _cardNameList.end() ) {

				if (it->second == subTag.value) {
					result.putCardId = it->first;
					break;
				}
				++it;
			}
		}
		if ( subTag.name == "putCardDirection" ) { 

			if      ( subTag.value == "up"    ) { result.putCardDirection = DIRECTION::UP;}
			else if ( subTag.value == "UP"    ) { result.putCardDirection = DIRECTION::UP;}
			else if ( subTag.value == "down"  ) { result.putCardDirection = DIRECTION::DOWN;}
			else if ( subTag.value == "DOWN"  ) { result.putCardDirection = DIRECTION::DOWN;}
			else if ( subTag.value == "left"  ) { result.putCardDirection = DIRECTION::LEFT;}
			else if ( subTag.value == "LEFT"  ) { result.putCardDirection = DIRECTION::LEFT;}
			else if ( subTag.value == "right" ) { result.putCardDirection = DIRECTION::RIGHT;}
			else if ( subTag.value == "RIGHT" ) { result.putCardDirection = DIRECTION::RIGHT;}
			else                                { result.putCardDirection = Utils_Convert::String_to_Int( subTag.value );}
		}

		if ( subTag.name == "movePieceFromX"       ) { result.movePieceFromX = Utils_Convert::String_to_Int( subTag.value );}
		if ( subTag.name == "movePieceFromY"       ) { result.movePieceFromY = Utils_Convert::String_to_Int( subTag.value );}
		if ( subTag.name == "movePieceToX"         ) { result.movePieceToX   = Utils_Convert::String_to_Int( subTag.value );}
		if ( subTag.name == "movePieceToY"         ) { result.movePieceToY   = Utils_Convert::String_to_Int( subTag.value );}
		if ( subTag.name == "movePieceAndIsCllide" ) { result.movePieceAndIsCllide = Utils_Convert::String_to_bool( subTag.value );}

		if ( subTag.name == "takeCardX" ) { result.takeCardX = Utils_Convert::String_to_Int( subTag.value );}
		if ( subTag.name == "takeCardY" ) { result.takeCardY = Utils_Convert::String_to_Int( subTag.value );}

	}


	return result;
}


//==========================================================
// Move が一致しているかどうか
//==========================================================
bool AI::MoveEquals ( const Move &moveA, const Move &moveB ) {

	bool result = true;

	///// アクションと手番
	if ( moveA.action != moveB.action ) { return false;}
	if ( moveA.turn   != moveB.turn   ) { return false;}

	///// 以下、アクションごとに、関連する値を比較
	if ( moveA.action == MOVE::ACTION::PUT_PIECE ) {

		if ( moveA.putPieceX != moveB.putPieceX ) { return false;}
		if ( moveA.putPieceY != moveB.putPieceY ) { return false;}
	}
	else if ( moveA.action == MOVE::ACTION::PUT_CARD ) {

		if ( moveA.putCardX != moveB.putCardX ) { return false;}
		if ( moveA.putCardY != moveB.putCardY ) { return false;}
		if ( moveA.putCardId != moveB.putCardId ) { return false;}
		if ( moveA.putCardDirection != moveB.putCardDirection ) { return false;}
	}
	else if ( moveA.action == MOVE::ACTION::MOVE_PIECE ) {

		if ( moveA.movePieceFromX != moveB.movePieceFromX ) { return false;}
		if ( moveA.movePieceFromY != moveB.movePieceFromY ) { return false;}
		if ( moveA.movePieceToX != moveB.movePieceToX ) { return false;}
		if ( moveA.movePieceToY != moveB.movePieceToY ) { return false;}
		if ( moveA.movePieceAndIsCllide != moveB.movePieceAndIsCllide ) { return false;}
	}
	else if ( moveA.action == MOVE::ACTION::TAKE_CARD ) {

		if ( moveA.takeCardX != moveB.takeCardX ) { return false;}
		if ( moveA.takeCardY != moveB.takeCardY ) { return false;}
	}
	else if ( moveA.action == MOVE::ACTION::PASS ) {
		// 特にチェック項目なし
	}


	return result;
}

//==========================================================
// 学習用手順構造体をファイルから読み込む
//==========================================================
Learning::LearningProgram AI::ReadLearningProgram( string fileName ) {

	Learning::LearningProgram result;

	NotXML::Tag learnTag = NotXML().ReadRoot( fileName );

	///// Root が <LerningProgram> だということにして、子要素でループ
	for ( unsigned int i = 0; i < learnTag.childList.size(); ++i ) {

		NotXML::Tag tag = learnTag.childList[i];

		if ( tag.name == "AI_File_Input" ) {
			result.AI_File_Input = tag.value;
		}
		else if ( tag.name == "AI_File_Output" ) {
			result.AI_File_Output = tag.value;
		}
		else if ( tag.name == "passBorderPoint" ) {
			result.passBorderPoint = Utils_Convert::String_to_Int ( tag.value );
		}
		else if ( tag.name == "examinCount" ) {
			result.examinCount = Utils_Convert::String_to_Int ( tag.value );
		}
		else if ( tag.name == "copyCount" ) {
			result.copyCount = Utils_Convert::String_to_Int ( tag.value );
		}
		else if ( tag.name == "Problem" ) {

			Learning::Problem problem;

			for ( unsigned int n = 0; n < tag.childList.size(); ++n ) {

				if (tag.childList[n].name == "File") {

					problem.problemFile = tag.childList[n].value;

				} else if (tag.childList[n].name == "Move") {

					int  point       = Utils_Convert::String_to_Int ( tag.childList[n].attribute["point"] );
					Move correctMove = Move_From_Tag( tag.childList[n] );

					pair<Move, int> answer;
					answer.first  = correctMove;
					answer.second = point;

					problem.answerList.push_back ( answer );
				}
			}

			result.problemList.push_back ( problem );
		}

	}

	return result;
}

//==========================================================
// 素子同士を掛け合わせる
//==========================================================
Parameters_PieceEvaluation AI::GeneticCrossover ( vector<Parameters_PieceEvaluation> PEEList, int fluctuation) {

	/*
		ルート素子６つ＋高さバイアス＝７要素である、という前提
	*/

	Parameters_PieceEvaluation result;

	int candidateCount = PEEList.size();

	///// 受け継ぎ：高さバイアス
	result.heightValue_Bias = FluctuatedCopy ( PEEList[ Utils_Cal::GetRandom( candidateCount ) ].heightValue_Bias, fluctuation, 5);
	
	///// 受け継ぎ可能な素子の個数：候補のうち、一番少ないルート素子の数
	/*
		掛け合わせる候補として指定された候補が持つ素子の数が同じ前提だけれども、
		一応、エラーが起きないように。
	*/
	int inheritableNumber = 0;
	for ( unsigned int i = 0; i > PEEList.size(); ++i ) {

		if (inheritableNumber <= PEEList[i].pieceEvaluateElementList.size()) {
			inheritableNumber = PEEList[i].pieceEvaluateElementList.size();
		}
	}

	///// 受け継ぎ：素子  掛け合わせる候補がもつルート素子の数が同じである前提
	for ( unsigned int i = 0; i < inheritableNumber; ++i ) {

		PieceEvaluateElement PEE = PEE_Copy( PEEList[ Utils_Cal::GetRandom( candidateCount ) ].pieceEvaluateElementList[i], fluctuation);
		result.pieceEvaluateElementList.push_back( PEE );
	}


	return result;
}
//==========================================================
// 素子同士を掛け合わせる：２個用甘口
//==========================================================
Parameters_PieceEvaluation AI::GeneticCrossover ( Parameters_PieceEvaluation PEE1, Parameters_PieceEvaluation PEE2, int fluctuation) {

	vector<Parameters_PieceEvaluation> PEEList;
	PEEList.push_back( PEE1 );
	PEEList.push_back( PEE2 );
	return GeneticCrossover( PEEList, fluctuation );
}
//==========================================================
// 素子同士を掛け合わせる：３個用甘口
//==========================================================
Parameters_PieceEvaluation AI::GeneticCrossover ( Parameters_PieceEvaluation PEE1, Parameters_PieceEvaluation PEE2, Parameters_PieceEvaluation PEE3, int fluctuation) {

	vector<Parameters_PieceEvaluation> PEEList;
	PEEList.push_back( PEE1 );
	PEEList.push_back( PEE2 );
	PEEList.push_back( PEE3 );
	return GeneticCrossover( PEEList, fluctuation );
}

//==========================================================
// 素子２個掛け合わせ；全組み合わせ
//==========================================================
vector<Parameters_PieceEvaluation> AI::GeneticCrossover_ALL ( Parameters_PieceEvaluation PEE1, Parameters_PieceEvaluation PEE2, int fluctuation) {

	vector<Parameters_PieceEvaluation> result;

	/*
		素子６＋高さバイアス＝７要素である前提。
	*/

	int elementSize = 7;
	int allSize = Utils_Cal::Pow ( 2, elementSize );
	for ( int i = 0; i < allSize; ++i ) {

		Parameters_PieceEvaluation PEE;

		///// 最初の桁は高さバイアスということにする
		int flag = Utils_Cal::Pow( 2, 0 );

		if ( (i & flag) == flag  ) {
			PEE.heightValue_Bias = PEE1.heightValue_Bias;
		} else {
			PEE.heightValue_Bias = PEE2.heightValue_Bias;
		}

		///// 以降の桁を、各ルート素子ということにする
		for ( int n = 1; n < elementSize; ++n ) {

			int PEEIndex = n - 1;
			int flag = Utils_Cal::Pow( 2, n );

			if ( (i & flag) == flag  ) {
				if ( PEEIndex <  PEE1.pieceEvaluateElementList.size()) { // 一応
					PEE.pieceEvaluateElementList.push_back ( PEE1.pieceEvaluateElementList[PEEIndex]);
				}
			} else {
				if ( PEEIndex <  PEE2.pieceEvaluateElementList.size()) { // 一応
					PEE.pieceEvaluateElementList.push_back ( PEE2.pieceEvaluateElementList[PEEIndex]);
				}
			}
		}

		result.push_back ( PEE );
	}

	return result;
}

//==========================================================
// 
//==========================================================
vector<int> AI::Get_ThrouableInfoEXList_ALL() {

	vector<int> result;

	result.push_back( THROUGHABLE_INFO::EX::NOT_THROUGHABLE );
	result.push_back( THROUGHABLE_INFO::EX::THROUGHABLE );
	result.push_back( THROUGHABLE_INFO::EX::MUST_TO_UP );
	result.push_back( THROUGHABLE_INFO::EX::MUST_TO_DOWN );
	result.push_back( THROUGHABLE_INFO::EX::MUST_TO_LEFT );
	result.push_back( THROUGHABLE_INFO::EX::MUST_TO_RIGHT );

	result.push_back( THROUGHABLE_INFO::EX::PIECE_EMPTY );
	result.push_back( THROUGHABLE_INFO::EX::PIECE_MY );
	result.push_back( THROUGHABLE_INFO::EX::PIECE_OPPONENT );

	result.push_back( THROUGHABLE_INFO::EX::OUT_OF_BOARD );
	result.push_back( THROUGHABLE_INFO::EX::CARD_TAKEABLE );

	return result;
}
//==========================================================
// 
//==========================================================
vector<int> AI::Get_ThrouableInfoEXList_Root() {

	vector<int> result;

	result.push_back( THROUGHABLE_INFO::EX::NOT_THROUGHABLE );
	result.push_back( THROUGHABLE_INFO::EX::THROUGHABLE );
	result.push_back( THROUGHABLE_INFO::EX::MUST_TO_UP );
	result.push_back( THROUGHABLE_INFO::EX::MUST_TO_DOWN );
	result.push_back( THROUGHABLE_INFO::EX::MUST_TO_LEFT );
	result.push_back( THROUGHABLE_INFO::EX::MUST_TO_RIGHT );


	return result;
}

