//=========================================================
//
// 定数
//
//=========================================================

#ifndef _AI_DEFINES_
#define _AI_DEFINES_

#include <string>

using namespace std;

namespace AI_Modules {

	namespace DIRECTION {

		const int NONE  = 0;
		const int LEFT  = 1;
		const int RIGHT = 2;
		const int UP    = 3;
		const int DOWN  = 4;

		const int LEFT_UP    = 5;
		const int LEFT_DOWN  = 6;
		const int RIGHT_UP   = 7;
		const int RIGHT_DOWN = 8;

	};

	namespace TURN {

		const int NONE  = 0;
		const int BLACK = 1;
		const int WHITE = 2;

		/*
		int SwitchTurn( int turn ) {
			if ( turn == BLACK) { return WHITE; } 
			if ( turn == WHITE) { return BLACK; } 
		}
		*/
	};

	namespace GRID {
		const int ENPTY = 0;
		const int BLACK = 1;
		const int WHITE = 2;
	};

	namespace THROUGHABLE_INFO {

		const int NOT_THROUGHABLE = 0;
		const int THROUGHABLE     = 1;
		const int MUST_TO_RIGHT = 2;
		const int MUST_TO_LEFT  = 3;
		const int MUST_TO_UP    = 4;
		const int MUST_TO_DOWN  = 5;

		///// 拡張
		namespace EX {
			const int NOT_THROUGHABLE = 0;
			const int THROUGHABLE     = 1;
			const int MUST_TO_RIGHT = 2;
			const int MUST_TO_LEFT  = 3;
			const int MUST_TO_UP    = 4;
			const int MUST_TO_DOWN  = 5;
			const int PIECE_EMPTY    = 10;
			const int PIECE_MY       = 11;
			const int PIECE_OPPONENT = 22;
			const int OUT_OF_BOARD   = 100;
			const int OPPONENT_AREA  = 200;
			const int CARD_TAKEABLE  = 300;
		};

	};

	namespace PILLAR_COVER {

		namespace PIECE_POSITION {

			const int NOT_INVOLVED            = 1;
			const int IN_THE_BOX              = 2;
			const int ABOVE_COVER             = 4;
			const int ABOVE_COVER_DIAGONALLY  = 8;
			const int UNDER_COVER_SIDE        = 16;
			const int UNDER_COVER             = 32;

		};

		namespace THROUGHABLE_INFO {

			const int WALL  = 0; // その陣営にとって、通れない
			const int PATH  = 1; // その陣営にとって、通れる
			const int EMPTY = 2; // 空きマス
		};

		namespace ALGORISM_FOR_RISK_DEGREE_ADDITION_TO_A_PILLAR {

			const int MAX_ONLY = 0;
			const int ADD      = 1;
		};

	};


	namespace MOVE {

		namespace ACTION {

			const int PASS       = 1;
			const int PUT_PIECE  = 2;
			const int MOVE_PIECE = 4;
			const int PUT_CARD   = 8;
			const int TAKE_CARD  = 16;

			// TAKE_CARD すらも入れてない。
			const int CURRENT_TERRAIN_ONLY = PASS | PUT_PIECE | MOVE_PIECE;

			// 王手をかけうる種類の手：カード取得を省く
			const int CHECKABLE = MOVE_PIECE | PUT_PIECE | PUT_CARD | PASS;

			// 王手を受けうる種類の手：カード設置を省く
			const int GUARDABLE = MOVE_PIECE | PUT_PIECE | TAKE_CARD | PASS;
		};

	};

	namespace EVALUATION {

		namespace RUTIN_SET {

			const int FOR_TERRAIN_SEARCH = 1;
			const int FOR_MIDDLE_STAGE   = 2;
			const int FOR_TREE_PICK_UP   = 3;
		};
		namespace RUTIN {

			///// 勝ってたら評価最高 → ここで結論が出る場合、以降を考えない
			const int GAME_OVER = 1;

			///// 自分の手番なら、少し評価高い
			const int CONSIDER_ONES_TURN = 2;

			///// 駒の価値を計算
			const int VALUE_OF_PIECES = 4;

			///// 地形の価値を計算
			const int VALUE_OF_TERRAIN = 8;

			///// カードの位置を計算：カードというか、移動制限。自陣付近に通れるマス目が多いと不利、みたいな。
			const int VALUE_OF_CARDS = 16;

			///// 王手がかかっていたら高得点ですよね？
			const int VALUE_OF_CHECK = 32;

		};

		namespace BLACKHOLE {
			const int BLACKHOLE            = 3000; // まさにブラックホールそのもの 
			const int MIGHT_BE_BLACKHOLE   = 1800; // ブラックホールかもしれない
			const int CAN_BE_BLACK_HOLE    = 400; // ブラックホールになるかもしれない（今は違う）
			const int FREE_FROM_BLACK_HOLE = 200; // パネルがない
			const int FAR_FROM_BLACKHOLE   = 0; // 無印パネルがある
		};

		namespace TERRAIN {
			const int BLIND_ALLEY = -5000; // 行き止まり
			const int FIRST_ENERGY = 9000; // 移動エネルギー。満タン値。
		};

		///// 外部向けの総括としての評価値というか、表現：すでに数値ではないので、文字列。設定ファイルとの相性もよいはず。
		namespace SUM_UP {

			const string WIN_MAX  = "WIN_MAX";  // 勝ち
			const string WIN_4    = "WIN_4";    // 勝ち同然（詰みを読みきっている）
			const string WIN_3    = "WIN_3";    // 非常に有利
			const string WIN_2    = "WIN_2";    // 有利
			const string WIN_1    = "WIN_1";    // 少し有利
			const string EVEN     = "EVEN";     // 互角
			const string LOSE_1   = "LOSE_1";   // 少し不利
			const string LOSE_2   = "LOSE_2";   // 不利
			const string LOSE_3   = "LOSE_3";   // 非常に不利
			const string LOSE_4   = "LOSE_4";   // 負け同然（相手に詰みがある）
			const string LOSE_MAX = "LOSE_MAX"; // 負け

			const string UNKNOWN = "UNKNOWN"; // 判定なし

		};
	};
};

#endif
