//=========================================================
//
// 局面、内部表現
//
//=========================================================

#ifndef _INNER_INTERPRETATION_
#define _INNER_INTERPRETATION_

#include <vector>
#include <new>

using namespace std;

#include "Move.h"
#include "Position.h"
#include "../Utils/Allocator.h"

namespace AI_Modules {

	class InnerInterPretation {
	
	public:
		
		InnerInterPretation();
		~InnerInterPretation();

		// placement newとplacement deleteを定義
		static void* operator new(size_t size, void* buf){ return buf; }
		static void* operator new(size_t size){ 
			return malloc(size); 
		}
		static void operator delete(void* p, void* buf){}
		static void operator delete(void* p){ free(p); }  // VC6.0では必要
		
		//=========================================================
		// static
		//=========================================================
		static int _numberOfInstances;

		//=========================================================
		// 盤上のカード
		//=========================================================
		struct CardOnBoard {

			int cardId;       // どのカードが
			int direction;    // どの向きか

			CardOnBoard() {
				cardId    = -1; // マイナスで、「なし」ということにする。
				direction = -1;
			}

			bool isEmpty() {
				return (cardId < 0);
			}
		};

		//=========================================================
		// エリア：グリッドをまとめてあいまいに解釈したもの
		//=========================================================
		struct Area {

			int gridNumber_X;  // 全部でマス目がいくつ_X方向
			int gridNumber_Y;  // 全部でマス目がいくつ_Y方向
			int blackNumber; // 黒駒の数
			int whiteNumber; // 白駒の数

			int cardNumber;            // カードの枚数
			CardOnBoard cardList[100]; // 置かれているカードの状態。
			/*
				その時点での cardNumber が枚数を表すとともに、
				その時点で cardList に追加する際のインデックスをも表す。
			*/

			int throuable_Number;       // 通れるマス目の数＿無印
			int throuable_UP_Number;    // 通れるマス目の数＿上強制
			int throuable_DOWN_Number;  // 通れるマス目の数＿下強制
			int throuable_RIGHT_Number; // 通れるマス目の数＿右強制
			int throuable_LEFT_Number;  // 通れるマス目の数＿左強制

			Area() {
				gridNumber_X  = 0;
				gridNumber_Y  = 0;
				blackNumber = 0;
				whiteNumber = 0;
				cardNumber  = 0;
				throuable_Number       = 0;
				throuable_UP_Number    = 0;
				throuable_DOWN_Number  = 0;
				throuable_LEFT_Number  = 0;
				throuable_RIGHT_Number = 0;
			}
		};


		//=========================================================
		// 初期設定
		//=========================================================
		void Init(); // 盤のサイズをそろえて、後は真っ白にする。
		void Init_SetCurrentTurn( int turn );
		void Init_PutCard( int x, int y, int cardId, int direction );
		void Init_PutPiece( int x, int y, int piece );
		void Init_SetThrouableInfo( int x, int y, int throuableInfo );
		void Init_SetHandPiece_BLACK( int num );
		void Init_SetHandPiece_WHITE( int num );
		void Init_AddHandCard_BLACK( int cardId );
		void Init_AddHandCard_WHITE( int cardId );
		void Init_ReflectCardAffection();

		void Init_RemoveCard( int x, int y, int cardId );
		void Init_RemoveCard( int x, int y, int cardId, int &direction); // &directionは、指定IDのカードがあった場合に、どの方向であったかを入れる戻り値用

		//=========================================================
		// 次の局面を取得
		//【引数】こう指すと
		//【戻値】こうなる
		//=========================================================
		InnerInterPretation* GetNext( Move move );
		InnerInterPretation* GetNext( Move move, Allocator<InnerInterPretation> *allocator );

		//=========================================================
		// 次の局面になっておくれ
		//【引数】こう指してね
		//=========================================================
		void GoNext( const Move &move );

		//=========================================================
		// 手を一つ巻き戻す
		//【戻り値】Undoできたら true 
		//=========================================================
		bool GoBack();

		//=========================================================
		// 可能な次の手のうち、勝つ手を取得
		//（現局面に王手が生じているかどうかを判定する主旨）
		//=========================================================
		//void GetLegalWinMoveList(vector<Move> &result);
		//bool IsCheck();
		/*
			これは、解釈版では、なし。
		*/


		//=========================================================
		// 可能な次の手を列挙
		//=========================================================
		vector<Move> GetLegalNextMoveList();
		void GetLegalNextMoveList(vector<Move> &result);

		//=========================================================
		// コピーを取得
		//=========================================================
		InnerInterPretation* GetCopy();
		InnerInterPretation* GetCopy(Allocator<InnerInterPretation> *allocator);
		void GetCopy( InnerInterPretation *useThisMemory );

		//=========================================================
		// 局面が同一かどうか
		//【戻り値】 true で「同一」
		//=========================================================
		bool IsEquals( InnerInterPretation *anotherOne );
		static int compare_int( const void *a, const void *b );
		static int compare_CardOnBoard( const void *a, const void *b );
		bool IsEquals_Area ( Area &a, Area &b );

		//=========================================================
		// コンソールに様子を表示：デバッグ用
		//=========================================================
		void ShowToConsole();

		///// 手番情報
		int _currentTurn; // 現在どっちの手番か？ TURN
		
		///// 盤面情報
		Area _areaList[3][3]; // 実際の盤面をここに集約する。

		//int         _gridList_ThrouableInfo[15][15]; // 通行情報レイヤー：通行情報　 THROUABLE_INFO
		//int         _gridList_Piece[15][15];         // 駒レイヤー      ：黒/白/なし GRID
		//CardOnBoard _gridList_Card[15][15];          // カードレイヤー  ：カードと向き
		//static int           _gridList_Init[15][15];     // 初期化用
		//static CardOnBoard   _gridList_Init_Card[15][15]; // 初期化用

		// サイズは Area の数を表す。
		int _boardSizeX;
		int _boardSizeY;

		///// 駒台情報：何個あるか、だけ。
		int _handPiece_BLACK;
		int _handPiece_WHITE;

		///// 手札情報：カード番号のリスト
		int _handCards_BLACK[100];
		int _handCards_WHITE[100];
		static int _handCards_Init[100];
		int _handCards_BLACK_Index;
		int _handCards_WHITE_Index;

		///// 今、何手目？
		int _totalMoveCount;
		void AddTotalMoveCount()       { ++_totalMoveCount; }
		void DecrementTotalMoveCount() { --_totalMoveCount; }
		int GetTotalMoveCount()        { return _totalMoveCount;}

		///// パス情報
		bool _isPassPreviousMove;
		bool _isPassPass;

	private:

		///// 盤上のカードの状態に対して通行可能情報が最新か？
		bool _isThrouableInfoLatest;

		///// Undo用
		MoveForBack _moveForBack;


		//=========================================================
		// 可能な次の手を列挙：子処理群
		//=========================================================
		void GetLegalNextMoveList_PutPiece  (vector<Move> &result );
		void GetLegalNextMoveList_MovePiece (vector<Move> &result );
		void GetLegalNextMoveList_TakeCard  (vector<Move> &result );
		void GetLegalNextMoveList_PutCard   (vector<Move> &result );

		// 全体分の空きマス数を効果範囲の数で累乗。
		// (空きマス数 / 全マス数)^効果範囲の数
		bool GetLegalNextMoveList_PutCard_Lot ( int blankGridNum, int allGridNum, int effectGridListNum);

		//void GetLegalNextMoveList_PutCard_TurnDetermined (vector<Move> &result, vector<int> &handCards );
		void GetLegalNextMoveList_PutCard_TurnDetermined (vector<Move> &result, int handCards[], int length );

		void GetLegalNextMoveList_MovePiece_toUP    (vector<Move> &result, int x, int y );
		void GetLegalNextMoveList_MovePiece_toDOWN  (vector<Move> &result, int x, int y );
		void GetLegalNextMoveList_MovePiece_toLEFT  (vector<Move> &result, int x, int y );
		void GetLegalNextMoveList_MovePiece_toRIGHT (vector<Move> &result, int x, int y );
		void GetLegalNextMoveList_MovePiece_toRIGHT_DOWN (vector<Move> &result, int x, int y );
		void GetLegalNextMoveList_MovePiece_toRIGHT_UP   (vector<Move> &result, int x, int y );
		void GetLegalNextMoveList_MovePiece_toLEFT_UP    (vector<Move> &result, int x, int y );
		void GetLegalNextMoveList_MovePiece_toLEFT_DOWN  (vector<Move> &result, int x, int y );

		void GetLegalNextMoveList_MovePiece_toUP_toWin    (vector<Move> &result, int x, int y );
		void GetLegalNextMoveList_MovePiece_toDOWN_toWin  (vector<Move> &result, int x, int y );
		void GetLegalNextMoveList_MovePiece_toRIGHT_DOWN_toWin (vector<Move> &result, int x, int y );
		void GetLegalNextMoveList_MovePiece_toRIGHT_UP_toWin   (vector<Move> &result, int x, int y );
		void GetLegalNextMoveList_MovePiece_toLEFT_UP_toWin    (vector<Move> &result, int x, int y );
		void GetLegalNextMoveList_MovePiece_toLEFT_DOWN_toWin  (vector<Move> &result, int x, int y );

		int GetLegalNextMoveList_MovePiece_CollidePieceColor ( const int &blackNumber, const int &whiteNumber );

		int GetLegalNextMoveList_TakeCard_HowManyCardsCanITake ( int allGridNum, int cardNum, int pieceNum );
		void GetLegalNextMoveList_WhichCardWllITake ( int *result, int cardNum, int takenCardNum );

		//=========================================================
		// 各方向への移動可？
		//=========================================================
		bool IsThrouableTo( int throuableInfo, int direction );

		//=========================================================
		// このエリアは、指定の方向への移動可？
		//【引数】
		// Area エリア
		// direction この方向
		//=========================================================
		bool IsThrouableTo_Area( Area &area, int direction );

		//=========================================================
		// このエリアは他の駒があって通れない？
		//【結果】
		// true で、空きがある
		//=========================================================
		bool IsEmpty_Area( Area &area);

		//=========================================================
		// このエリアに、入れる・通行可不可の意味で
		//【結果】
		// true で、入れる
		//=========================================================
		bool IsEnter_Area_Throuable( Area &area );


		//=========================================================
		// 次の局面になっておくれ：子処理群
		//=========================================================
		void GoNext_PutPiece  ( const Move &move );
		void GoNext_MovePiece ( const Move &move );
		void GoNext_PutCard   ( const Move &move );
		void GoNext_TakeCard  ( const Move &move );
		void GoNext_Pass      ( const Move &move );

		//=========================================================
		// 手を巻き戻して：子処理群
		//=========================================================
		bool GoBack_PutPiece  ();
		bool GoBack_MovePiece ();
		bool GoBack_PutCard   ();
		bool GoBack_TakeCard  ();
		bool GoBack_Pass      ();

		//=========================================================
		// 盤の範囲内？
		//=========================================================
		bool IsWithinBoardLimit( int x, int y );

		//=========================================================
		// このカードがありますか？
		//=========================================================
		int GetHandCardIndexOf_Black( int cardId);
		int GetHandCardIndexOf_White( int cardId);

		//=========================================================
		// n番目のカードを手放す
		//=========================================================
		void ReleaseCard_ByIndex_Black( int index);
		void ReleaseCard_ByIndex_White( int index);


	};
};
#endif
