//=========================================================
//
// Button (implement)
//
//=========================================================

#include "Button.h"
#include "TextImageFactory.h"
#include "../Utils/Utils_Cal.h"
#include "SFML_Utils.h"

using namespace GUI_SFML;


//=========================================================
// Button
//=========================================================
Button::Button(string label, sf::Vector2f size, int labelFontSize) {

	_label = label;
	
	//TextImageFactory::Create( _labelImage, _label, labelFontSize, sf::Color(0,0,0,255) );
	_labelText = TextImageFactory::Create ( _label, labelFontSize);
	
	float labelOriginX = _labelText.getGlobalBounds().width / 2.0f;
	Utils_Cal::RoundWildly (labelOriginX);

	float labelOriginY = _labelText.getGlobalBounds().height / 2.0f + labelFontSize/10.0f;
	Utils_Cal::RoundWildly (labelOriginY);
	
	_labelText.setOrigin( sf::Vector2f( labelOriginX, labelOriginY) );
	_labelText.setColor ( sf::Color(0,0,0,255) );
	
	/*
	_labelTexture.loadFromImage ( _labelImage );
	_sprite.setTexture(_labelTexture);
	_sprite.setOrigin( sf::Vector2f( _labelImage.getSize().x / 2.0f, _labelImage.getSize().y / 2.0f ) );
	*/
	/*
	// サイズを適当にまかなうケース
	if ( size.x < 0.0f || size.y < 0.0f ) {

		size.x = _labelImage.getSize().x + 5;
		size.y = _labelImage.getSize().y + 2;
	}
	*/

	SetSize(size);


	_color_Nomal = sf::Color(255,255,255,255);
	_color_Over  = sf::Color(255,000,000,255);
	_color_Push  = sf::Color(000,255,000,255);

	_spritePositionOffset = sf::Vector2f(0.0f, 0.0f);
	SetPosition(_position);

	_fontSize = labelFontSize;

	_isVisible = true;
	
	_currentState = BUTTON::STATE::NONAL;
	_pushedWhenMouseOut = false;
}
//=========================================================
// SetSize
//=========================================================
void Button::SetSize(sf::Vector2f arg) {

	_size = arg;

	_base = sf::RectangleShape (_size);
	_base.setOutlineThickness(1.0f);
	_base.setOutlineColor (sf::Color(0,0,0,255));
	_base.setFillColor    (sf::Color(255,255,255,255));

	//_sprite.SetPosition( _position + _size / 2.0f );
}

//=========================================================
// SetSpritePositionOffset
//=========================================================
void Button::SetSpritePositionOffset(sf::Vector2f arg) {
	_spritePositionOffset = arg;
}

//=========================================================
// ~Button
//=========================================================
Button::~Button() {



}


//=========================================================
// SetImageData
//=========================================================
void Button::SetImageData ( ImageProperty prop ) {
	
	_imageData = prop;
	
	_labelTexture.loadFromFile ( _imageData.imageFile );
	
	_labelSprite = sf::RectangleShape ( _imageData.size );
	
	_labelSprite.setTexture ( &_labelTexture );
	
	_labelSprite.setTextureRect (_imageData.rect_Normal);
	
}

//=========================================================
// Update
//=========================================================
void Button::Update(sf::RenderWindow &window) {
	
	if ( ! window.hasFocus() ) {
		return;
	}

	sf::Vector2f labelPos = _position + _size/2.0f;
	Utils_Cal::RoundWildly ( labelPos.x );
	Utils_Cal::RoundWildly ( labelPos.y );

	_labelText.setPosition( labelPos );
	
	
	sf::Vector2f sizeForMouseOver = _size;
	
	
	if ( _imageData.imageFile != "" ) {

		_labelSprite.setPosition ( _position );
		
		sizeForMouseOver = _imageData.size;
	}
	
	
	int mouseX = sf::Mouse::getPosition(window).x;
	int mouseY = sf::Mouse::getPosition(window).y;
	
	SFML_Utils::PosToScreenPos(mouseX, mouseY, window);
	
	
	bool isMouseOver = false;
	if
	(
		_position.x < mouseX && mouseX < _position.x + sizeForMouseOver.x
		&&
		_position.y < mouseY && mouseY < _position.y + sizeForMouseOver.y
	)
	{
		isMouseOver = true;
	}

	if ( isMouseOver ) {
		
		_base.setFillColor( _color_Over );

		if ( _currentState == BUTTON::STATE::NONAL ) {
			
			if ( sf::Mouse::isButtonPressed (sf::Mouse::Left ) ) {
				_pushedWhenMouseOut = true;
			} else {
				_pushedWhenMouseOut = false;
			}

			_currentState = BUTTON::STATE::OVER;
		
		} else if ( _currentState == BUTTON::STATE::OVER ) {
			
			if ( !_pushedWhenMouseOut ) {
			
				if ( sf::Mouse::isButtonPressed (sf::Mouse::Left ) ) {

					_currentState = BUTTON::STATE::PUSH;

				} else {
					_currentState = BUTTON::STATE::OVER;
				}

			} else {
			
				if ( !sf::Mouse::isButtonPressed (sf::Mouse::Left ) ) {
					_pushedWhenMouseOut = false;
				}

			}
		
		} else if ( _currentState == BUTTON::STATE::PUSH ) {

			_base.setFillColor( _color_Push );
			if ( sf::Mouse::isButtonPressed (sf::Mouse::Left ) ) {

				_currentState = BUTTON::STATE::PUSH;

			} else {
				_currentState = BUTTON::STATE::RELEASE;
			}
		
		} else if ( _currentState == BUTTON::STATE::RELEASE ) {

			_currentState = BUTTON::STATE::OVER;
		}

		/*
		bool isPush = sf::Mouse::isButtonPressed (sf::Mouse::Left );
	
		if (isPush && _currentState = BUTTON::STATE::NONAL) {
		
			_currentState = BUTTON::STATE::PUSH;
			
		} else {
			_base.setFillColor( _color_Over );

			if ( _currentState == BUTTON::STATE::PUSH ) {
				_currentState = BUTTON::STATE::RELEASE;
			} else {
				_currentState = BUTTON::STATE::OVER;
			}
		}
		*/

	} else {
		_base.setFillColor( _color_Nomal );
		_currentState = BUTTON::STATE::NONAL;
		_pushedWhenMouseOut = false;
	}
	
	if ( _imageData.imageFile != "" ) {
		
		if ( _currentState == BUTTON::STATE::NONAL   ) { _labelSprite.setTextureRect (_imageData.rect_Normal); }
		if ( _currentState == BUTTON::STATE::OVER    ) { _labelSprite.setTextureRect (_imageData.rect_Over); }
		if ( _currentState == BUTTON::STATE::PUSH    ) { _labelSprite.setTextureRect (_imageData.rect_Push); }
		if ( _currentState == BUTTON::STATE::RELEASE ) { _labelSprite.setTextureRect (_imageData.rect_Normal); }
	}
}

//=========================================================
// Observe
//=========================================================
bool Button::IsPush()    { return (_currentState == BUTTON::STATE::PUSH);    }
bool Button::IsOver()    { return (_currentState == BUTTON::STATE::OVER);    }
bool Button::IsRelease() { return (_currentState == BUTTON::STATE::RELEASE); }
bool Button::IsNomal()   { return (_currentState == BUTTON::STATE::NONAL);   }

//=========================================================
// SetColor
//=========================================================
void Button::SetColor_Nomal( sf::Color color ) { _color_Nomal = color; }
void Button::SetColor_Over ( sf::Color color ) { _color_Over  = color; }
void Button::SetColor_Push ( sf::Color color ) { _color_Push  = color; }

//=========================================================
// Draw
//=========================================================
void Button::Draw(sf::RenderWindow &window) {

	if (!_isVisible) { 
		return; 
	}

	if ( _imageData.imageFile == "" ) {

		window.draw(_base);
		
		/*
		_sprite.setPosition( _sprite.getPosition() + _spritePositionOffset );
		window.draw(_sprite);
		_sprite.setPosition( _sprite.getPosition() - _spritePositionOffset );
		*/
		
	
		window.draw(_labelText);
		
	} else {
		window.draw ( _labelSprite );
	}
}

//=========================================================
// SetPosition
//=========================================================
void Button::SetPosition(sf::Vector2f arg) {

	_position = arg;

	_base.setPosition( arg );
	//_sprite.setPosition( arg + _size / 2.0f );
}


