//=========================================================
//
// ファイルエクスプローラー
//
//=========================================================

#include "FileExplorer.h"
#include "ScrollBar.h"
#include "../Utils/Utils_File.h"
#include "../Utils/Utils_String.h"
#include "../Utils/Utils_Convert.h"
#include "../Utils/NotXML.h"
#include "../ScriptEngine/Application.h"
#include "../ScriptEngine/ClickableImage.h"
#include "SFML_Utils.h"
#include "TextImageFactory.h"


using namespace std;
using namespace GUI_SFML;

			
//=========================================================
// ファイルエクスプローラー内での一項目
//=========================================================

///// コンストラクタ
//FileInFileExplorer::FileInFileExplorer(string name, bool isFolder, sf::Font &font) {
FileInFileExplorer::FileInFileExplorer ( Utils_File::FolderEntity entity, sf::Font &font) {

	_entity = entity;

	_position    = sf::Vector2f(0.0f, 0.0f);
	//_size        = sf::Vector2f(100.0f, 10.0f);
	_isMouseOver = false;
	_isPressed   = false;
	_isReleased  = false;	
	
	_text.setFont(font);
	_text.setString ( "" );
	//_text.setString ( SFML_Utils::GetSfString ( GetMKFDisplayName (_entity.path, _entity.name) ) );
	
	_isFolder = _entity.isFolder;
		
	if (_isFolder) {
		_text.setColor  ( sf::Color::Yellow  );
	} else {
		_text.setColor  ( sf::Color::White );
	}
	
	_text.setCharacterSize ( 18 );
	
	_size = sf::Vector2f(0.0f, 0.0f);
	//_size = sf::Vector2f (_text.getGlobalBounds().width, _text.getGlobalBounds().height);
	
	// 外部から読んでもらうこととする。
	//SetupFileNameLabel();
	
	_view = NULL;
}

void FileInFileExplorer::SetupFileNameLabel() {
	
	if ( _entity.isFolder ) {
		
		_text.setString ( SFML_Utils::GetSfString ( GetArcDirDisplayName ( _entity.name) ) );
		_size = sf::Vector2f (_text.getGlobalBounds().width, _text.getGlobalBounds().height);
		
	} else {
	
		_text.setString ( SFML_Utils::GetSfString ( GetMKFDisplayName (_entity.path, _entity.name) ) );
		_size = sf::Vector2f (_text.getGlobalBounds().width, _text.getGlobalBounds().height);
	}
	
}

bool FileInFileExplorer::IsSetupFileNameLabelDone() {
	
	bool result = false;
	
	sf::String str = _text.getString();
	
	result = !( str.isEmpty() );
	
	
	return result;
}

			
///// 棋譜ファイルの表示用の文字列を取得
string FileInFileExplorer::GetMKFDisplayName ( string path, string fileName ) {
	
	string result = fileName;
	
	///// 拡張子を確認
	string extension = "";
	vector<string> anaForExtension = Utils_String::Split ( fileName, ".");
	if ( anaForExtension.size() > 0 ) {
		extension = anaForExtension [ anaForExtension.size() - 1 ];
	}

	//printf ("パス：%s\n", path.c_str());
	//printf ("ファイル名：%s\n", fileName.c_str());
	//printf ("拡張子：%s\n", extension.c_str());
	
	
	
	// 拡張子を確認
	if ( extension != "mkf" && extension != "MKF" ) {
		
		return result;
	}
	
	
	
	NotXML::Tag rootTag = NotXML().ReadRoot( path + fileName );
	
	NotXML::Tag battleTitleTag = NotXML::GetTagFromName ( rootTag, "BattleTitle" );
	NotXML::Tag timestampTag   = NotXML::GetTagFromName ( rootTag, "Timestamp" );
	
	///// こういう情報があるはず！
	/*
	<BattleTitle>練習試合レベル５</BattleTitle>
	<Timestamp>1565067918</Timestamp>
	*/
	
	///// 該当のタグがない場合は処理終了
	if ( battleTitleTag.name != "BattleTitle" ) {
		printf("BattleTitle なし\n");
		return result;
	}
	if ( timestampTag.name != "Timestamp" ) {
		printf("Timestamp なし\n");
		return result;
	}
	
	///// あるべき情報が取得できましたよ。
	string battleTitle = battleTitleTag.value;
	string timestamp   = timestampTag.value;
	
	
	///// タイムスタンプを日付っぽく変換するんです？
	if ( timestamp != "" ) {
		
		/*
		//time_t t = time(NULL);
		time_t t = (time_t)Utils_Convert::String_to_Long(timestamp);

		tm *localTime = localtime(&t);

		int year  = localTime->tm_year + 1900;
		int month = localTime->tm_mon + 1;
		int day   = localTime->tm_mday;
		int hour  = localTime->tm_hour;
		int min   = localTime->tm_min;
		int sec   = localTime->tm_sec;

		char date[512];

		//sprintf ( date, "%d年 %02d月 %02d日 %02d:%02d:%02d",
		sprintf ( date, "%d/%02d/%02d %02d:%02d:%02d",
			year, month, day, hour, min,sec
		);
		*/
		
		map<string, int> timeInfo = Utils_Convert::String_to_TimeInfo ( timestamp );

		char date[512];

		sprintf ( date, "%d/%02d/%02d %02d:%02d:%02d",
			timeInfo["year"], timeInfo["month"], timeInfo["day"], timeInfo["hour"], timeInfo["min"], timeInfo["sec"]
		);
		
		timestamp = date;
	
	}
	
	
	result = battleTitle + "(" + timestamp + ")";
	
	
	return result;
}	

///// アーカイブフォルダの表示用文字列を取得
string FileInFileExplorer::GetArcDirDisplayName ( string dirName ) {
	
	string result = dirName;
	
	
	// arc_20191014_20391204
	
	
	vector<string> ana1 = Utils_String::Split ( dirName, "_" );
	
	if ( ana1.size() != 3 ) { return dirName; }
	if ( ana1[0] != "arc"  ) { return dirName; }
	if ( ana1[1].size() != 8  ) { return dirName; }
	if ( ana1[2].size() != 8  ) { return dirName; }
	
	string yearFrom  = ana1[1].substr ( 0, 4 );
	string monthFrom = ana1[1].substr ( 4, 2 );
	string dayFrom   = ana1[1].substr ( 6, 2 );
	
	string yearTo  = ana1[2].substr ( 0, 4 );
	string monthTo = ana1[2].substr ( 4, 2 );
	string dayTo   = ana1[2].substr ( 6, 2 );
	
	result = yearFrom + "年" + monthFrom + "月" + dayFrom + "日 ～ " + yearTo + "年" + monthTo + "月" + dayTo + "日";
	
	
	return result;
}

///// デストラクタ
FileInFileExplorer::~FileInFileExplorer() {
	
}

///// フォルダアイコンをもらう
void FileInFileExplorer::SetFolderIconTexture ( sf::Texture& folderIconTexture ) { 

	//_folderIconTexture = folderIconTexture;
	
	_folderIconSprite.setTexture ( folderIconTexture );

}


///// 更新
void FileInFileExplorer::Update (sf::RenderWindow &window) {
	
	_text.setPosition ( _position );
	
	if ( _view != NULL ) {
		_view->Update(window);
	}

	sf::Vector2f textCurrentPos = _text.getPosition();
	if ( _isFolder ) {
		
		// フォルダアイコンの位置
		_folderIconSprite.setPosition 
		(
			_position.x, 
			_position.y - 2 // 適当
		);
		
		window.draw ( _folderIconSprite );
		
		// アイコンの分、フォルダ名を少し右にずらす。
		sf::Vector2f textAdjustedPos = textCurrentPos;
		textAdjustedPos.x += _folderIconSprite.getGlobalBounds().width + 2; // 適当
		
		_text.setPosition(textAdjustedPos);
	}
	
	
	///// マウス関連処理
	if ( Application::GetInstance()->HasFocus() ) {
		
		///// マウスオーバー判定
		sf::Vector2i mousePos = sf::Mouse::getPosition ( window );
		
		SFML_Utils::PosToScreenPos ( mousePos.x, mousePos.y, window );
		
		int boundsX = _text.getGlobalBounds().left;
		int boundsY = _position.y;// _text.getGlobalBounds().top;
		int boundsW = _text.getGlobalBounds().width;
		int boundsH = 30;//_text.getGlobalBounds().height;
		
		
		_isMouseOver =
		(
			boundsX < mousePos.x && mousePos.x < boundsX + boundsW 
			&&
			boundsY < mousePos.y && mousePos.y < boundsY + boundsH
		);
		
		
		///// クリック判定
		_isReleased = false;
		
		if ( _isPressed ) {
			
			if ( !sf::Mouse::isButtonPressed( sf::Mouse::Left ) ) {
				_isReleased = true;
			}
			
		}
		
		_isPressed = sf::Mouse::isButtonPressed( sf::Mouse::Left );
		
	} else {
		
		_isMouseOver = false;
		_isPressed   = false;
		_isReleased  = false;
	}
	
	
}

///// 描画
void FileInFileExplorer::Draw   (sf::RenderWindow &window) {

	if ( _isFolder ) {
		
		window.draw ( _folderIconSprite );
	}

			
	if ( _isMouseOver ) {
	
		sf::RectangleShape cursor = sf::RectangleShape ( sf::Vector2f ( _text.getGlobalBounds().width, _text.getGlobalBounds().height ) );
		cursor.setOutlineThickness(1.0f);
		cursor.setOutlineColor (sf::Color(10,10,200,255));
		cursor.setFillColor    (sf::Color(150,150,255,100));
		cursor.setPosition ( sf::Vector2f ( _text.getGlobalBounds().left, _text.getGlobalBounds().top ) );

		window.draw( cursor );
	}
	
	
	
	if ( _view != NULL ) {
		_view->Draw(window);
	} else {
		window.draw(_text);
	}
	
	// ラベルの位置を戻しておく
	// _text.setPosition(textCurrentPos);
}

///// クリック判定
bool FileInFileExplorer::IsClicked() {
	
	return ( _isMouseOver && _isReleased );
}


///// ソート用
bool FileInFileExplorer::compare ( FileInFileExplorer* a, FileInFileExplorer* b) {
	
	//printf("来てます？\n");
	
	bool result = false;
	
	// フォルダーが上
	result = ( a->IsFolder() && !b->IsFolder() );
	
	if ( a->IsFolder() == b->IsFolder() ) {
		
		// フォルダー同士
		if ( a->IsFolder() ) {
		
			result = (a->GetEntity().name < b->GetEntity().name);
			
		// ファイル同士
		} else {

			result = (a->GetEntity().name > b->GetEntity().name);
		}
		
	}
	
	return result;
}


///// その他
void         FileInFileExplorer::SetPosition( sf::Vector2f pos) {_position = pos;}
sf::Vector2f FileInFileExplorer::GetPosition() { return _position; }
sf::Vector2f FileInFileExplorer::GetSize    () { return _size;}
bool         FileInFileExplorer::IsMouseOver() { return _isMouseOver;}
void         FileInFileExplorer::SetView    (IFileInFileExplorer_View *view) { _view = view;}





static map<string, FileExplorer::PositionSaveData> FileExplorer::POSITION_SAVE; // ２回目以降、開いたときの位置再現用情報

//=========================================================
// コンストラクタ
//=========================================================
FileExplorer::FileExplorer() {
	
	//_currentFolder = NULL;
	//_parentFolder  = NULL;
						
	_position = sf::Vector2f ( 100.0f, 50.0f );
	_size     = sf::Vector2f ( 600.0f, 500.0f );
	
	_base = sf::RectangleShape ( _size );
	_base.setOutlineThickness(1.0f);
	_base.setOutlineColor ( sf::Color ( 0,0,0,255 ) );
	//_base.setFillColor    ( sf::Color ( 120,120,120, 200 ) );
	_base.setFillColor    ( sf::Color ( 50,50,50, 200 ) );
	_base.setPosition     ( _position ); 


	_toolBar = sf::RectangleShape ( sf::Vector2f (_size.x + 20, 30.0f ) );
	_toolBar.setOutlineThickness(1.0f);
	_toolBar.setOutlineColor ( sf::Color ( 0,0,0,255 ) );
	_toolBar.setFillColor    ( sf::Color ( 60,60,200, 200 ) );
	_toolBar.setPosition     ( _position + sf::Vector2f ( 0.0f, -31.0f ) ); 

	_addressBar = sf::RectangleShape ( sf::Vector2f (_toolBar.getSize().x - 50, _toolBar.getSize().y ) );
	_addressBar.setOutlineThickness(1.0f);
	_addressBar.setOutlineColor ( sf::Color ( 0,0,0,255 ) );
	_addressBar.setFillColor    ( sf::Color ( 255,255,255,255 ) );
	_addressBar.setPosition     ( _position + sf::Vector2f ( 50.0f, -31.0f ) ); 
	
		
	_font = TextImageFactory::_font;
	
	_selectBox = new SelectBox();
	_selectBox->SetDisplayCount    ( 16 );
	//_selectBox->SetFrameSize       ( sf::Vector2f ( 180.0f, 170.0f ) );
	_selectBox->SetFrameSize       ( sf::Vector2f ( 600.0f, 500.0f ) );
	_selectBox->SetPosition_Offset ( sf::Vector2f ( 10.0f, 10.0f ) );
	_selectBox->SetLengthOfOneItem ( 30.0f );  // 行の高さ
	
	_selectBox->SetPosition (_position);
	_selectBox->SetFrameColor(sf::Color::Transparent);
	_selectBox->SetFrameOutlineColor(sf::Color::Transparent);
	
	_selectBox->SetDefaultCursorOn ( false );
	
	_backButton = new Button ( "↑", sf::Vector2f(20.0f, 20.0f), 12 );
	_backButton->SetPosition ( _position + sf::Vector2f ( 7.0f, -28.0f ) );
	{
		GUI_SFML::Button::ImageProperty prop;
		prop.imageFile = "./image/gameBoard/toParentFolderIcon.png";
		prop.rect_Normal = sf::IntRect(0, 0, 34, 26);
		prop.rect_Over   = sf::IntRect(0, 26, 34, 26);
		prop.rect_Push   = sf::IntRect(0, 52, 34, 26);
		prop.size        = sf::Vector2f(34, 26);

		_backButton->SetImageData ( prop );
	}
	
	_terminateButton = new Button ( "×", sf::Vector2f(20.0f, 20.0f), 12 );
	_terminateButton->SetPosition ( _position + sf::Vector2f ( _size.x - 12, -29.0f ) );
	{
		GUI_SFML::Button::ImageProperty prop;
		prop.imageFile = "./image/gameBoard/terminateButton.png";
		prop.rect_Normal = sf::IntRect(0, 0, 34, 26);
		prop.rect_Over   = sf::IntRect(0, 26, 34, 26);
		prop.rect_Push   = sf::IntRect(0, 52, 34, 26);
		prop.size        = sf::Vector2f(34, 26);

		_terminateButton->SetImageData ( prop );
	}
	
	{
		ClickableImage::Property closeButtonProp;
		
		closeButtonProp.imageFile = "./image/gameBoard/backFromScoreSheetSelector.png";
		closeButtonProp.size = sf::Vector2f(100.0f, 30.0f);
		closeButtonProp.rect_Normal = sf::IntRect ( 0, 0,  100, 30 );
		closeButtonProp.rect_Over   = sf::IntRect ( 0, 30, 100, 30 );
		closeButtonProp.rect_Push   = sf::IntRect ( 0, 60, 100, 30 );
		
		_backToTitleButton = new ClickableImage(closeButtonProp);
		
		_backToTitleButton->SetPosition ( sf::Vector2f(695, 560) );
	}
	
	_folderIconTexture.loadFromFile ( "./image/gameBoard/folderIcon.png" );
	
	_drawHaltOnce = false;
	
	_isTerminated = false;
	
	_view = NULL;
	
	_isGenerateFileNameLabelDone = true; // フォルダを指定して読み込んだ時点でfalseにする。
	
	_rootFolder = "";
}


//=========================================================
// デストラクタ
//=========================================================
FileExplorer::~FileExplorer() {
	
	GUI_SFML::ScrollBar *scrollBar = _selectBox->GetScrollBar();
	int scrollPos = scrollBar->GetCurrentBarPosition();
	
	PositionSaveData positionSaveData;
	positionSaveData.lastFolder   = _currentFolder.path + _currentFolder.name;
	positionSaveData.scrollPos    = scrollPos;
	positionSaveData.parentFolder = _parentFolder;
	positionSaveData.isEnable     = true;
	
	printf ( "_parentFolder size : %d\n", _parentFolder.size() );
	printf ( " scrollPos         : %d\n", scrollPos );
	
	POSITION_SAVE [ _rootFolder ] = positionSaveData;
	
	///// 既存のものをクリアしておくよ
	for ( FileInFileExplorer *fileInfo : _fileList ) 
	{
		delete fileInfo;
	}
	_fileList.clear();	
	
	delete _selectBox;
	delete _backButton;
	delete _terminateButton;
	delete _backToTitleButton;
	
	if ( _view != NULL ) {
		delete _view;
	}
	
	
}
	
//=========================================================
// 更新
//=========================================================
void FileExplorer::Update(sf::RenderWindow &window) {
	
	
	_backButton->Update(window);
	_selectBox->Update(window);
	_terminateButton->Update(window);
	_backToTitleButton->Update(window);
	
	// マウスのホイールでリストをスクロールさせたい
	bool isMouseOver = _selectBox->IsMouseOver();
	if ( isMouseOver ) {
		
		int mouseWheel = Application::GetInstance()->_mouseWheel;
		
		if ( mouseWheel > 0 ) {
			//printf ("ホイール回転：上\n");
			_selectBox->ScrollUp();
		}
		if ( mouseWheel < 0 ) {
			//printf ("ホイール回転：下\n");
			_selectBox->ScrollDown();
		}
		
	}
	
	///// 上に戻る？
	if ( !_parentFolder.empty()) { // 上があって……
		
		//if ( sf::Keyboard::isKeyPressed(sf::Keyboard::A) ) {
		if ( _backButton->IsRelease() ) {
			
			SetupCurrentFolder( _parentFolder.top() );
			
			_parentFolder.pop();
			_drawHaltOnce = true;
			return;
		}
	}
	
	///// クリック判定をしましょうよ
	for ( FileInFileExplorer *fileInfo : _fileList ) 
	{
		// クリックされた？
		if ( fileInfo->IsClicked() ) 
		{
			
			// フォルダですか？
			if ( fileInfo->IsFolder() ) 
			{
				Utils_File::FolderEntity entity = fileInfo->GetEntity();
				
				printf(" フォルダを選択 %s\n", entity.name.c_str());
				
				// 次なる上フォルダとしておきまして、
				_parentFolder.push (_currentFolder);
				
				// フォルダを潜ってリストを更新するぞ！
				SetupCurrentFolder ( entity );
				
				_drawHaltOnce = true;
				break;
				
			}
			// じゃあ、ファイルってことですね？
			else
			{
				// 「選択したファイル名」に値を入れるぞ！
				_selectedFileName = fileInfo->GetEntity().path + fileInfo->GetEntity().name;
				
				printf(" ファイルを選択： %s\n", _selectedFileName.c_str());
			}
		}
	}
	///// ファイル名のラベル作成
	if ( !_isGenerateFileNameLabelDone ) {
		
		// 1フレームに何件ずつラベル作成するか
		int maxNum = 10;
		
		_isGenerateFileNameLabelDone = true;
		int num = 0;
		for ( FileInFileExplorer *fileInfo : _fileList ) 
		{
			if ( num >= maxNum ) {
				break;
			}
			
			if ( !fileInfo->IsSetupFileNameLabelDone() ) {
				
				_isGenerateFileNameLabelDone = false;
				
				fileInfo->SetupFileNameLabel();
				
				++num;
			}
		}
	}
	
	///// 閉じるボタン
	if ( _terminateButton->IsRelease() ) {
		_isTerminated = true;
	} else {
	
		// 閉じた後にもう一度開いた際、前回の「終了しました」が残っていて即閉じしてしまうのを防ぐ目的でしたっけ？
		// 毎フレーム、Updateの冒頭でやるのではダメなんでしたっけ？ 結局同じなのではなくて？
		_isTerminated = false;
	}
	///// 右下の戻るボタン
	if ( _backToTitleButton->IsClick() ) {
		_isTerminated = true;
	}
	
}

//=========================================================
// 閉じるボタン判定
//=========================================================
bool FileExplorer::IsTerminated() {
	
	bool result = false;
	
	//if( sf::Keyboard::isKeyPressed ( sf::Keyboard::Escape ) ) {
	//	result = true;
	//}
	
	result = _isTerminated;
	
	return result;
}


//=========================================================
// 描画
//=========================================================
void FileExplorer::Draw(sf::RenderWindow &window) {
	
	window.draw( _base );
	window.draw( _toolBar );
	window.draw( _addressBar );
	
	
	_backButton->Draw(window);
	_terminateButton->Draw(window);
	_backToTitleButton->Draw(window);
	
	window.draw ( _currentFolderLabel );

	
	///// なぜかというと、フォルダをクリックしてリストを更新した直後は SelectBoxの側でリストが更新されてないっぽいのです。え〜？
	if (_drawHaltOnce) {
		_drawHaltOnce = false;
	} else {
		_selectBox->Draw(window);
	}
	
	
}

//=========================================================
// フォルダを読み込む
// フィルターの意味するところは、表示「する」拡張子。
// コンマ区切りで並べる。空文字または省略でフィルターなし。つまり全て表示「する」
// フォルダはフィルタリングの対象にならない。つまり必ず表示される。
// 拡張子なしのファイルとなんらかの拡張子ありのファイルを表示対象にしたい？ 諦めてください。
//=========================================================		
void FileExplorer::ReadFolder ( bool isPreviousPositionToUse, string folderName, string filter ) {
	
	// 前回の位置情報を使うのかどうか。
	PositionSaveData positionSaveData;
	positionSaveData.scrollPos = -1;
	positionSaveData.isEnable  = false;
	
	// 初回読み込み？
	if ( _rootFolder == "" ) {
		_rootFolder = folderName;
		
		// 前回の位置保存データがあればそれを使う
		if ( isPreviousPositionToUse ) {
			
			if ( POSITION_SAVE.find ( _rootFolder ) != POSITION_SAVE.end()) {
				
				positionSaveData = POSITION_SAVE.find ( _rootFolder )->second;
				
				// 引数で指定されたフォルダを、前回最後に開いていたフォルダに置き換える。なくなっていたらどうなるかは乞うご期待。
				folderName = positionSaveData.lastFolder;
				
			}
		}
	}
	
	
	///// フィルターというか、表示「する」拡張子のリストを受け取る。コンマ区切りで。
	_filterList = Utils_String::Split( filter, "," );
	
	
	// ここでファイルアクセス
	
#if defined(_WIN32)

	Utils_File::GetFileList ( folderName, _folder );
    
	
#elif defined(__linux__)

	set<int> visitedDirList;
	Utils_File::GetFileList ( folderName, _folder, visitedDirList );

#endif
	
	//printf ("%s\n", _folder.path.c_str() );
	//for ( unsigned int i = 0; i < _folder.childList.size(); ++i ) {
	//	printf ( "\t%s\n", _folder.childList[i].name.c_str() );
	//}
	
	// 上「なし」ということにする
	while (!_parentFolder.empty()) { _parentFolder.pop(); }
	
	// まずはルート直下？を表示のですよー。
	_folder.path = folderName;
	SetupCurrentFolder(_folder);
	
	
	// 前回のスクロールバーの位置を再現する場合は再現する
	if ( positionSaveData.isEnable ) {
		
		ScrollBar *scrollBar = _selectBox->GetScrollBar();
		
		printf ( "last scroll bar pos : %d\n" , positionSaveData.scrollPos );
		
		scrollBar->SetCurrentBarPosition ( positionSaveData.scrollPos );
		
		printf ( "_parentFolder size : %d\n", positionSaveData.parentFolder.size() );
		
		_parentFolder = positionSaveData.parentFolder;
	}
}

//=========================================================
// このフォルダの中身を表示することにするよ！
//=========================================================
void FileExplorer::SetupCurrentFolder ( Utils_File::FolderEntity &folder) {
	
	///// ここで「現在位置」を更新：「上フォルダ」はどうかというと、潜るときにそのときの「現在位置」で上書きする
	_currentFolder = folder;

	///// 既存のものをクリアしておくよ
	//for ( FileInFileExplorer *fileInfo : _fileList ) 
	for ( unsigned int i = 0; i < _fileList.size(); ++i ) 
	{
		FileInFileExplorer *fileInfo = _fileList[i];
		
		_selectBox->RemoveItem(fileInfo);
		
		delete fileInfo;
	}
	_fileList.clear();
	
	_isGenerateFileNameLabelDone = false;

	
	///// 新たに用意するよ！
	for ( Utils_File::FolderEntity entity : folder.childList ) 
	{
		// フィルターにかけるのですよー。
		if ( !IsFiltered(entity) ) {
		
			//FileInFileExplorer *item = new FileInFileExplorer( entity.name, entity.isFolder, _font);
			FileInFileExplorer *item = new FileInFileExplorer( entity, _font);
			
			// アイコンを渡す。さしあたりフォルダアイコンだけ。
			item->SetFolderIconTexture(_folderIconTexture);
			
			_fileList.push_back(item);
		}
	}
	
	///// 並べ替えをしてから
	std::sort(_fileList.begin(), _fileList.end(), FileInFileExplorer::compare);
	
	///// _selectBox に入れる
	for ( FileInFileExplorer *item : _fileList ) {
		//printf( "\t%s\n", item->GetEntity().name.c_str() );
		_selectBox->AddItem(item);
	}
	
	///// カレントフォルダのラベル
	_currentFolderLabel = TextImageFactory::Create ( 
		_currentFolder.path + _currentFolder.name, 12 
	);
	_currentFolderLabel.setFillColor ( sf::Color::Black );
	_currentFolderLabel.setPosition ( _position + sf::Vector2f ( 60, -24 ) );
	
	string aa = _currentFolder.path + _currentFolder.name;
	printf ( "カレントフォルダのラベル作成しましたよ (created current folder label) : %s\n", aa.c_str() );
	
	printf ( "_selectBox->MoveToTop();\n" );
	_selectBox->MoveToTop();

}
//=========================================================
// フィルター判定： true で、フィルターに「引っかかる」、つまり、表示「しない」
//=========================================================
bool FileExplorer::IsFiltered ( Utils_File::FolderEntity &entity ) {

	bool result = false;
	
	// フォルダだったらフィルターにかけない
	if ( entity.isFolder ) {
		return false;
	}
	
	// 拡張子が欲しい欲しい欲しい！
	vector<string> teamotanto = Utils_String::Split(entity.name, ".");
	string extention = ( teamotanto.empty() ) ? "" : teamotanto.back();
	
	// ファイルということは、表示されない可能性があるということです。
	result = true;
	
	// さぁ、表示対象かな？
	for ( string filter : _filterList ) {
			
		// 表示対象だった！
		if ( extention == filter ) {
			result = false;
			break;
		}
	}
	
	return result;
}
			

//=========================================================
// 位置指定
//=========================================================
void FileExplorer::SetPosition(sf::Vector2f arg) { 
	_position = arg; 
} 
//=========================================================
// サイズ指定
//=========================================================
void FileExplorer::SetSize(sf::Vector2f arg) {
	
}




