//=========================================================
//
// ファイルエクスプローラー
//
//=========================================================

#ifndef _FILE_EXPLORER_
#define _FILE_EXPLORER_

#include <map>
#include <vector>
#include <string>
#include <stack>

#include <SFML/Window.hpp>
#include <SFML/Graphics.hpp>

#include "SelectBox.h"
#include "ISelectBoxItem.h"

#include "../Utils/Utils_File.h"

#include "Button.h"

class ClickableImage;

using namespace std;

namespace GUI_SFML {
	
	//=========================================================
	// フォルダ内の項目の見た目専用クラス、のインターフェイス
	//=========================================================
	class IFileInFileExplorer_View {
		public:
			
			~IFileInFileExplorer_View(){}
			virtual void Update (sf::RenderWindow &window) = 0;
			virtual void Draw   (sf::RenderWindow &window) = 0;
			virtual void         SetPosition( sf::Vector2f pos) = 0;
			virtual bool IsClicked() = 0;
		
	};
	
	//=========================================================
	// 一項目
	//=========================================================
	class FileInFileExplorer : public ISelectBoxItem {
		
		public:
			//FileInFileExplorer(string name, bool isFolder, sf::Font &font);
			FileInFileExplorer ( Utils_File::FolderEntity entity, sf::Font &font);
			~FileInFileExplorer();
			
			///// ISelectBoxItemとしての関数群
			virtual void Update (sf::RenderWindow &window);
			virtual void Draw   (sf::RenderWindow &window);
			virtual sf::Vector2f GetPosition();
			virtual void         SetPosition( sf::Vector2f pos);
			virtual sf::Vector2f GetSize();
			virtual bool IsMouseOver();
			
			void SetView(IFileInFileExplorer_View *view);
			bool IsClicked();
			bool IsFolder() { return _isFolder; }
			
			Utils_File::FolderEntity GetEntity() { return _entity; }
			
			static bool compare ( FileInFileExplorer* a, FileInFileExplorer* b);
			
			void SetFolderIconTexture ( sf::Texture& folderIconTexture );
			
			void SetupFileNameLabel();
			bool IsSetupFileNameLabelDone();
		
		private:

			sf::Vector2f _position;
			sf::Vector2f _size;
			bool _isMouseOver;
			
			sf::Text _text;
			
			bool _isFolder;
			
			IFileInFileExplorer_View *_view;
			
			Utils_File::FolderEntity _entity;
			
			bool _isPressed;
			bool _isReleased;
			
			//sf::Texture _folderIconTexture;
			sf::Sprite _folderIconSprite;
			
			
			///// 棋譜ファイルの表示用の文字列を取得
			string GetMKFDisplayName ( string path, string fileName );
		
			///// アーカイブフォルダの表示用の文字列を取得
			string GetArcDirDisplayName ( string dirName );
	};
	
	
	//=========================================================
	// 本体
	//=========================================================
	class FileExplorer {

		public:
			
			struct PositionSaveData {
				string lastFolder;
				int scrollPos;
				stack<Utils_File::FolderEntity> parentFolder;
				
				bool isEnable;
				
				PositionSaveData () {
					lastFolder = "";
					scrollPos = -1;
					isEnable = false;
				}
			};
			
			static map<string, PositionSaveData> POSITION_SAVE; // ２回目以降、開いたときの位置再現用情報 [ 初回ルートフォルダ, PositionSaveData  ]

			FileExplorer();
			~FileExplorer();

			void Update(sf::RenderWindow &window);
			void Draw(sf::RenderWindow &window);

			void SetPosition(sf::Vector2f arg);
			void SetPositionX( float x ) { _position.x = x; SetPosition(_position); }
			void SetPositionY( float y ) { _position.y = y; SetPosition(_position); }
			void SetSize(sf::Vector2f arg);
			
			//=========================================================
			// フォルダを読み込む
			// フィルターの意味するところは、表示「する」拡張子。
			// コンマ区切りで並べる。空文字または省略でフィルターなし。つまり全て表示「する」
			// フォルダはフィルタリングの対象にならない。つまり必ず表示される。
			// 拡張子なしのファイルとなんらかの拡張子ありのファイルを表示対象にしたい？ 諦めてください。
			//=========================================================			
			void ReadFolder ( bool isPreviousPositionToUse, string folderName, string filter = "" );
			
			///// 見た目用クラスをセット：こっちで解放まで担当しますよ。
			void SetItemView( IFileInFileExplorer_View *view ) {
				
				if ( _view != NULL ) {
					delete _view;
				}
				_view = view;
			}
			
			string GetSelectedFileName() { return _selectedFileName; }
			
			// 閉じるボタンを押したかどうか
			bool IsTerminated();
		
		private:
			
			sf::Font _font;
			
			sf::Vector2f _position;
			sf::Vector2f _size;
			int          _fontSize;

			sf::RectangleShape _base;
			
			sf::RectangleShape _toolBar;
			sf::RectangleShape _addressBar;
			
			
			// フォルダアイコン
			sf::Texture _folderIconTexture;
			//sf::Sprite  _folderIconSprite;
			
			Button *_backButton;        // 上に戻るボタン
			Button *_terminateButton;   // 閉じるボタン
			ClickableImage *_backToTitleButton; // タイトル画面へ戻るボタン
			bool _isTerminated;
			
			SelectBox *_selectBox;
			
			vector<string> _filterList;
			
			vector<FileInFileExplorer*> _fileList;
			
			bool _isGenerateFileNameLabelDone;
			
			// 指定されて読み込んだフォルダ以下の情報。ツリー構造つきだよ！
			Utils_File::FolderEntity _folder;

			Utils_File::FolderEntity _currentFolder;
			
			stack<Utils_File::FolderEntity> _parentFolder; // 上に戻るときに必要：スタックとする
			
			void SetupCurrentFolder ( Utils_File::FolderEntity &folder);
			
			
			IFileInFileExplorer_View *_view;
			
			
			bool _drawHaltOnce;
			
			string _selectedFileName;
			
			sf::Text _currentFolderLabel;
			
			string _rootFolder; // インスタンス生成後、最初に読み込みを指定されたフォルダを覚えておく（次回の位置再現用）。
			
			
			///// フィルター判定： true で、フィルターに「引っかかる」、つまり、表示「しない」
			bool IsFiltered ( Utils_File::FolderEntity &entity );
	};
};
#endif


