//=========================================================
//
// ImageFont (implement)
//
//=========================================================

#include "ImageFont.h"
#include "ImageManager.h"
#include "../Utils//NotXML.h"
#include "../Utils/Utils_Convert.h"
#include "../Utils/Utils_String.h"

using namespace GUI_SFML;


//=========================================================
// Constructor
//=========================================================
ImageFont::ImageFont( string fileName ) {

	NotXML::Tag root = NotXML().ReadRoot( fileName );

	///// Loop with ChildTags of Root
	for (unsigned int i = 0; i < root.childList.size(); ++i) {

		NotXML::Tag tag = root.childList[i];

		///// Making sure it's a motion Tag
		if (tag.name == "imageFile") {

			string    maskColorData = tag.attribute["maskColor"];
			sf::Color maskColor     = String_to_sfColor( maskColorData );

			sf::Image* image = ImageManager::GetImage( tag.value, maskColor );
			_texture.loadFromImage ( *image );
			_sprite.setTexture( _texture );

		} else if (tag.name == "char") {

			string rectData = tag.attribute["rect"];

			vector<string> elem = Utils_String::Split( rectData, "," );
			if ( elem.size() >= 4 ) {

				sf::IntRect rect;

				rect.left   = Utils_Convert::String_to_Int( Utils_String::Trim( elem[0] ) );
				rect.top    = Utils_Convert::String_to_Int( Utils_String::Trim( elem[1] ) );
				
				int right  = Utils_Convert::String_to_Int( Utils_String::Trim( elem[2] ) );
				int bottom = Utils_Convert::String_to_Int( Utils_String::Trim( elem[3] ) );

				rect.width  = right - rect.left;
				rect.height = bottom - rect.top;

				_charList[ tag.value[0] ] = rect;
			}

		}
	}

}
//=========================================================
// Destructor
//=========================================================
ImageFont::~ImageFont() {



}


//=========================================================
// Draw
//=========================================================
void ImageFont::Draw(sf::RenderWindow &window, string str) {

	sf::Vector2f formerPos = _sprite.getPosition();

	for (unsigned int i = 0; i < str.size(); ++i) {

		char c = str[i];

		map<char, sf::IntRect>::iterator findIt = _charList.find(c);
		if (findIt == _charList.end()) {
			continue;
		}

		sf::Rect<int> subRect = findIt->second;

		//_sprite.SetSubRect(subRect);
		_sprite.setTextureRect(subRect);

		window.draw( _sprite );

		_sprite.setPosition(
			_sprite.getPosition() +
			sf::Vector2f( (float)(subRect.width), 0.0f )
		);
	}


	_sprite.setPosition(formerPos);
}

//=========================================================
// SetPosition
//=========================================================
void ImageFont::SetPosition( sf::Vector2f pos ) {


	_position = pos;
	_sprite.setPosition( pos );
}

//=========================================================
// 文字列 → sf::Color
//=========================================================
sf::Color ImageFont::String_to_sfColor(string str, string splitter) {

	sf::Color result;

	vector<string> list = Utils_String::Split( str, splitter );

	///// number of elements is 4 :: with alpha
	if (list.size() == 4) {

		result.r = Utils_Convert::String_to_Int ( list[0] );
		result.g = Utils_Convert::String_to_Int ( list[1] );
		result.b = Utils_Convert::String_to_Int ( list[2] );
		result.a = Utils_Convert::String_to_Int ( list[3] );

	///// number of elements is 3 :: without alpha
	} else if (list.size() == 3) {

		result.r = Utils_Convert::String_to_Int ( list[0] );
		result.g = Utils_Convert::String_to_Int ( list[1] );
		result.b = Utils_Convert::String_to_Int ( list[2] );

	///// Invalid argument
	} else {
		// anything error processing?
	}

	return result;
}