//=========================================================
//
// ProgressBar (implement)
//
//=========================================================

#include "ProgressBar.h"
#include "../Utils/Utils_Cal.h"
#include "../Utils/Utils_Convert.h"
#include "TextImageFactory.h"

using namespace GUI_SFML;

//=========================================================
// コンストラクタ
//=========================================================
ProgressBar::ProgressBar( sf::Vector2f size, float maxValue, string unit) {

	_vanishCountDown = -1;

	_size         = size;
	_unit         = unit;
	_maxValue     = maxValue;

	_currentValue = 0.0f;
	_goalValue    = 0.0f;

	_frame = sf::RectangleShape ( size );
	_frame.setOutlineColor ( sf::Color( 128,64,32, 255 ) );
	_frame.setOutlineThickness ( 2.0f );
	_frame.setFillColor ( sf::Color( 0,0,0,0 ) );

	_base = sf::RectangleShape ( size );
	_base.setOutlineColor ( sf::Color( 0,0,0, 0 ) );
	_base.setOutlineThickness ( 0.0f );
	_base.setFillColor ( sf::Color( 200,200,200,150 ) );

	_bar = sf::RectangleShape ( sf::Vector2f( 0, 0) );
	_bar.setOutlineColor ( sf::Color( 0,0,0, 0 ) );
	_bar.setOutlineThickness ( 0.0f );
	_bar.setFillColor ( sf::Color( 200,200,255,200 ) );

	Update_ProgressRate();

}

//=========================================================
// デストラクタ
//=========================================================
ProgressBar::~ProgressBar() {

}

//=========================================================
// 更新
//=========================================================
void ProgressBar::Update (sf::RenderWindow &window) {

	if ( _vanishCountDown == 0) {
		return;
	} else {
		
		if ( _vanishCountDown > 0  ) {
			--_vanishCountDown;
		}
	}

	Update_Main();
}

//=========================================================
// 更新処理メイン部分
//=========================================================
void ProgressBar::Update_Main() {

	///// ゴールへ向かう？
	if ( _currentValue != _goalValue  ) {

		_currentValue = Utils_Cal::Nearlize ( _currentValue, _goalValue, 1.0f );

		///// じゃあ、バーを動かしましょう。
		Update_Bar();

		// 進捗率を更新
		Update_ProgressRate();
	}

	_base. setPosition ( _position );
	_frame.setPosition ( _position );
	_bar.  setPosition ( _position );
	_number_Sprite.setPosition ( _position + sf::Vector2f ( _size.x / 2.0f, _size.y / 2.0f ) );
}

//=========================================================
// バーの長さを更新
//=========================================================
void ProgressBar::Update_Bar() {

	///// じゃあ、バーを動かしましょう。
	// 長さを求める
	float barLength = (_maxValue != 0.0f ) ? _base.getSize().x * _currentValue / _maxValue : 0.0f;

	// セット。
	_bar.setSize ( sf::Vector2f ( barLength, _base.getSize().y ) );
}

//=========================================================
// 進捗率を求めて更新
//=========================================================
void ProgressBar::Update_ProgressRate() {

	int progressRate = (_maxValue != 0.0f ) ? _currentValue / _maxValue * 100.0f : 0.0f;

	string progressString = Utils_Convert::Int_to_String ( progressRate ) + _unit;

	TextImageFactory::Create( _number_Image, progressString, 16, sf::Color (0,0,0,255)) ;
	_number_Texture.loadFromImage ( _number_Image );
	_number_Sprite = sf::Sprite();
	_number_Sprite.setTexture ( _number_Texture );

	_number_Sprite.setOrigin( _number_Image.getSize().x / 2.0f, _number_Image.getSize().y / 2.0f );
}

//=========================================================
// 強引に値をセット
//=========================================================
void ProgressBar::SetValue_Forsibly ( float value ) {

	_goalValue = value;  
	if ( _goalValue > _maxValue) { _goalValue = _maxValue; } 
	_currentValue = _goalValue;

	///// じゃあ、バーを動かしましょう。
	Update_Bar();

	// 進捗率を更新
	Update_ProgressRate();

	Update_Main();

}	

//=========================================================
// 描画
//=========================================================
void ProgressBar::Draw (sf::RenderWindow &window) {

	if ( _vanishCountDown == 0) {
		return;
	}
	window.draw ( _base );
	window.draw ( _frame );
	window.draw ( _bar );
	window.draw ( _number_Sprite );
}
			
