//=========================================================
//
// サブルーチン集
// SFMLに依存するもの
//
//=========================================================

#include <string>
#include <SFML/Graphics.hpp>
#include "SFML_Utils.h"

#include "../Utils/Utils_String.h"
#include "../Utils/Utils_Convert.h"

using namespace std;
using namespace GUI_SFML;


//=========================================================
// string → sf::Color
//=========================================================
sf::Color SFML_Utils::Convert_string_to_color( string str, string delim ) {

	sf::Color result;

	vector<string> colorElemList = Utils_String::Split ( str, delim );

	result.r = ( colorElemList.size() > 0 ) ? Utils_Convert::String_to_Int (colorElemList[0]) : 0; 
	result.g = ( colorElemList.size() > 1 ) ? Utils_Convert::String_to_Int (colorElemList[1]) : 0; 
	result.b = ( colorElemList.size() > 2 ) ? Utils_Convert::String_to_Int (colorElemList[2]) : 0; 
	result.a = ( colorElemList.size() > 3 ) ? Utils_Convert::String_to_Int (colorElemList[3]) : 255; 

	return result;
}
//=========================================================
// sf::Color → string
//=========================================================
string SFML_Utils::Convert_color_to_string ( sf::Color color, string delim ) {

	string result = "";

	result += Utils_Convert::Int_to_String ( (int)color.r ) + delim;
	result += Utils_Convert::Int_to_String ( (int)color.g ) + delim;
	result += Utils_Convert::Int_to_String ( (int)color.b ) + delim;
	result += Utils_Convert::Int_to_String ( (int)color.a );

	return result;
}


//=========================================================
// std::string → sf::String
//=========================================================
sf::String SFML_Utils::GetSfString ( std::string str) {
	
	unsigned int index = 0;
	unsigned char *input = (unsigned char*)str.c_str();
	
	vector<unsigned int> codePointList;
	
	
	while ( index < str.size() ) 
	{
		int bytesInSequence = 0;
		unsigned int codePoint = utf8ToUtf32 (input + index, bytesInSequence);
		index += bytesInSequence;
		codePointList.push_back(codePoint);
		
	} // while	
	
	unsigned int *codePointPoiner = new unsigned int[ codePointList.size() + 1  ];
	
	for ( unsigned int i = 0; i < codePointList.size(); ++i ) {
		
		codePointPoiner[i] = codePointList[i];
	}
	codePointPoiner[codePointList.size()] = NULL;
	
	sf::String result = codePointPoiner;
	
	delete[] codePointPoiner;
	
	return result;
}


//=========================================================
// 座標値を、「画面上の座標」ということにする ん？ 逆かな？ よくわからなくなってきましたよ。
//=========================================================
void SFML_Utils::PosToScreenPos ( int &mouseX, int &mouseY, sf::RenderWindow &window ) {
	
	sf::Vector2i mousePos = sf::Vector2i(mouseX, mouseY);
	sf::Vector2f mouseInnerPos =  window.mapPixelToCoords( mousePos );
	mouseX = mouseInnerPos.x;
	mouseY = mouseInnerPos.y;
	
}



//=========================================================
// 参考：http://www.ryusuke.net/archives/601
//=========================================================
unsigned int SFML_Utils::utf8ToUtf32(unsigned char *input, int &bytesInSequence)
{

	unsigned char c1, c2, c3, c4, c5, c6;

	bytesInSequence = 1;
	if (!input)
	{
		return 0;
	}

	//0xxxxxxx (ASCII)      7bit
	c1 = input[0];
	if ((c1 & 0x80) == 0x00)
	{
		return c1;
	}

	//10xxxxxx              high-order byte
	if ((c1 & 0xc0) == 0x80)
	{
		return 0;
	}

	//0xFE or 0xFF          BOM (not utf-8)
	if (c1 == 0xfe || c1 == 0xFF )
	{
		return 0;
	}

	//110AAAAA 10BBBBBB     5+6bit=11bit
	c2 = input[1];
	if (((c1 & 0xe0) == 0xc0) &&
		((c2 & 0xc0) == 0x80))
	{
		bytesInSequence = 2;


		return ((c1 & 0x1f) << 6) | (c2 & 0x3f);
	}

	//1110AAAA 10BBBBBB 10CCCCCC        4+6*2bit=16bit
	c3 = input[2];
	if (((c1 & 0xf0) == 0xe0) &&
		((c2 & 0xc0) == 0x80) &&
		((c3 & 0xc0) == 0x80))
	{
		bytesInSequence = 3;
		return ((c1 & 0x0f) << 12) | ((c2 & 0x3f) << 6) | (c3 & 0x3f);
	}

	//1111 0AAA 10BBBBBB 10CCCCCC 10DDDDDD      3+6*3bit=21bit
	c4 = input[3];
	if (((c1 & 0xf8) == 0xf0) &&
		((c2 & 0xc0) == 0x80) &&
		((c3 & 0xc0) == 0x80) &&
		((c4 & 0xc0) == 0x80))
	{
		bytesInSequence = 4;
		return ((c1 & 0x07) << 18) | ((c2 & 0x3f) << 12) | ((c3 & 0x3f) << 6) | (c4 & 0x3f);
	}

	//1111 00AA 10BBBBBB 10CCCCCC 10DDDDDD 10EEEEEE     2+6*4bit=26bit
	c5 = input[4];
	if (((c1 & 0xfc) == 0xf0) &&
		((c2 & 0xc0) == 0x80) &&
		((c3 & 0xc0) == 0x80) &&
		((c4 & 0xc0) == 0x80) &&
		((c5 & 0xc0) == 0x80))
	{
		//bytesInSequence = 4; // 参考元では 4 となっていた。5ではなく？
		bytesInSequence = 5;
		return ((c1 & 0x03) << 24) | ((c2 & 0x3f) << 18) | ((c3 & 0x3f) << 12) | ((c4 & 0x3f) << 6) | (c5 & 0x3f);
	}

	//1111 000A 10BBBBBB 10CCCCCC 10DDDDDD 10EEEEEE 10FFFFFF        1+6*5bit=31bit
	c6 = input[5];
	if (((c1 & 0xfe) == 0xf0) &&
		((c2 & 0xc0) == 0x80) &&
		((c3 & 0xc0) == 0x80) &&
		((c4 & 0xc0) == 0x80) &&
		((c5 & 0xc0) == 0x80) &&
		((c6 & 0xc0) == 0x80))
	{
		//bytesInSequence = 4; // 参考元では 4 となっていた。6ではなく？
		bytesInSequence = 6;
		return ((c1 & 0x01) << 30) | ((c2 & 0x3f) << 24) | ((c3 & 0x3f) << 18) | ((c4 & 0x3f) << 12) | ((c5 & 0x3f) << 6) | (c6 & 0x3f);
	}

	return 0;
}


//=========================================================
// sf::IntRect -->> std::string 
//=========================================================
string SFML_Utils::Convert_IntRect_to_string ( sf::IntRect rect ) {
	
	string result = "";
	
	result += std::to_string ( rect.top   ) + "<>";
	result += std::to_string ( rect.left  ) + "<>";
	result += std::to_string ( rect.width ) + "<>";
	result += std::to_string ( rect.height);
	
	return result;
}

//=========================================================
// std::string -->> sf::IntRect
//=========================================================
sf::IntRect SFML_Utils::Convert_string_to_IntRect ( string str ) {

	sf::IntRect result;
	
	
	vector<string> elements = Utils_String::Split(str, "<>");
	
	if ( elements.size() == 4 ) {
		
		result.top    = std::stoi ( elements[0] );
		result.left   = std::stoi ( elements[1] );
		result.width  = std::stoi ( elements[2] );
		result.height = std::stoi ( elements[3] );
		
	} else {
		printf ( "データがおかしい。復元失敗。Convert_string_to_IntRect(%s)", str.c_str() );
	}
	
	return result;
	
}










