//=========================================================
//
// ScrollBar (implement)
//
//=========================================================

#include <stdio.h>
#include "ScrollBar.h"
#include "Button.h"
#include "SFML_Utils.h"

using namespace GUI_SFML;

//=========================================================
// ScrollBar
//=========================================================
ScrollBar::ScrollBar() {

	//_currentIndex   = 0;
	//_maxLength      = 0;
	_actualLength    = 0;
	_displayLength  = 10;
	
	_isMouseOver    = false;
	_isUpDownButton = true;

	_button_Up   = new Button("▲", sf::Vector2f(20.0f, 20.0f));
	_button_Down = new Button("▼", sf::Vector2f(20.0f, 20.0f));

	_button_Up  ->SetSpritePositionOffset( sf::Vector2f(1.0f, 0.0f) );
	_button_Down->SetSpritePositionOffset( sf::Vector2f(1.0f, 0.0f) );
}

//=========================================================
// ~ScrollBar
//=========================================================
ScrollBar::~ScrollBar() {

	delete _button_Up;
	delete _button_Down;
}

//=========================================================
// Update
//=========================================================
void ScrollBar::Update(sf::RenderWindow &window) {
	
	if ( !window.hasFocus() ) {
		return;
	}
	
	MakeBarShape();

	if (_isUpDownButton) {

		_button_Up->Update(window);
		_button_Down->Update(window);

		if (_button_Up->IsRelease()  ) {_barPosition.y -= GetBarMoveLength_par_OneItem(); }
		if (_button_Down->IsRelease()) {_barPosition.y += GetBarMoveLength_par_OneItem(); }
	}

	sf::Vector2i mousePos = sf::Mouse::getPosition(window);
	int  mouseX = mousePos.x;
	int  mouseY = mousePos.y;
	
	SFML_Utils::PosToScreenPos ( mouseX, mouseY, window );
	
	bool isPush = sf::Mouse::isButtonPressed ( sf::Mouse::Left );

	if ( isPush && IsMouseOver_to_BarUpper(mouseX, mouseY) )  { _barPosition.y -= GetBarMoveLength_par_OneItem(); }
	if ( isPush && IsMouseOver_to_BarDownner(mouseX, mouseY)) { _barPosition.y += GetBarMoveLength_par_OneItem(); }


	if ( !_isMouseOver) {

		if (IsMouseOver_to_Bar(mouseX, mouseY)) {
			_isMouseOver = true;
		}
	}
	if ( _isMouseOver && isPush ) {

		_barPosition.y += ( mouseY - _mouseY_previous);

		//GetCurrentIndex();

	} else {
		_isMouseOver = false;
	}

	_mouseX_previous = mouseX;
	_mouseY_previous = mouseY;

	///// Adjust position
	AdjustPosition();

}

void ScrollBar::AdjustPosition() {
	
	if ( _barPosition.y < _position.y ) {
		_barPosition.y = _position.y;
	} 
	if ( _barPosition.y > _position.y + _frameSize.y - _barSize.y ) {
		_barPosition.y = _position.y + _frameSize.y - _barSize.y;
	} 
}

///// IsMouseOver_to_Bar
bool ScrollBar::IsMouseOver_to_Bar(int mouseX, int mouseY) {

	bool result = false;

	if 
	( 
		_barPosition.x < mouseX && mouseX < (_barPosition.x + _barSize.x)
		&&
		_barPosition.y < mouseY && mouseY < (_barPosition.y + _barSize.y)
	)
	{
		result = true;
	}

	return result;
}
///// IsMouseOver_to_BarUpper
bool ScrollBar::IsMouseOver_to_BarUpper(int mouseX, int mouseY) {

	bool result = false;

	if 
	( 
		_position.x < mouseX && mouseX < ( _position.x + _frameSize.x)
		&&
		_position.y < mouseY && mouseY < _barPosition.y
	)
	{
		result = true;
		//printf("UpperPart\n");
	}

	return result;

}
///// IsMouseOver_to_BarDownner
bool ScrollBar::IsMouseOver_to_BarDownner(int mouseX, int mouseY) {

	bool result = false;

	if 
	( 
		_position.x < mouseX && mouseX < ( _position.x + _frameSize.x)
		&&
		_barPosition.y + _barSize.y < mouseY && mouseY < _position.y + _frameSize.y
	)
	{
		result = true;
		//printf("DownPart\n");
	}

	return result;

}

//=========================================================
// 最下段？
//=========================================================
bool ScrollBar::IsBottom() {

	bool result = false;
	
	// スクロールバーの必要なし、という場合も「最下段」とみなしますよね？ うん、みなすよ。よーし。
	if ( _actualLength <= _displayLength) {
		return true;
	}
	
	// あと少し下へ行くと目一杯、という判定の仕方です。
	if ( (_barPosition.y + 1.0f) > _position.y + _frameSize.y - _barSize.y ) {
		result = true;
	} 
	
	return result;
}


//=========================================================
// 上に移動
//=========================================================
void ScrollBar::MoveUp   (int count) {
	
	_barPosition.y -= GetBarMoveLength_par_OneItem() * count;
	
	AdjustPosition();
}
//=========================================================
// 下に移動
//=========================================================
void ScrollBar::MoveDown (int count) {

	_barPosition.y += GetBarMoveLength_par_OneItem() * count;
	
	AdjustPosition();
}


//=========================================================
// 最下段まで移動
//=========================================================
void ScrollBar::MoveToBottom() {
	
	// TODO ここでの MakeBarShape() が本当に必要なのかどうかは疑惑
	
	MakeBarShape();
	
	//printf ("_barPosition.y(%f)  _position.y(%f)  _frameSize.y(%f)  _barSize.y(%f)\n", _barPosition.y, _position.y, _frameSize.y, _barSize.y);
	
	_barPosition.y = _position.y + _frameSize.y - _barSize.y;

	MakeBarShape();

	//printf ("_barPosition.y(%f)  _position.y(%f)  _frameSize.y(%f)  _barSize.y(%f)\n", _barPosition.y, _position.y, _frameSize.y, _barSize.y);

}

//=========================================================
// 最上段まで移動
//=========================================================
void ScrollBar::MoveToTop() {
	
	_barPosition.y = _position.y;
}
		
		
//=========================================================
// GetBarMoveLength_par_OneItem
//=========================================================
float ScrollBar::GetBarMoveLength_par_OneItem() {

	float result = 0.0f;

	
	float movableRange = _frameSize.y - _barSize.y;

	//int capableLength = _frameSize.y / _length_of_oneItem;
	//int exceedLength  = _actualLength - capableLength;
	int exceedLength  = _actualLength - _displayLength;

	result = (exceedLength == 0) ? 0.0f : movableRange / (float)exceedLength;

	//result = (_maxLength == 0) ? 0.0f : movableRange / (float)_maxLength;


	return result;
}

//=========================================================
// GetCurrentIndex
//=========================================================
int ScrollBar::GetCurrentIndex() {

	int result = 0;
	
	// スクロールバーの必要なし、ということ。
	if ( _actualLength <= _displayLength) {
		return 0;
	}
	

	float movableRange = _frameSize.y - _barSize.y;
	float barPosition  = _barPosition.y - _position.y;
	
	int exceedLength  = _actualLength - _displayLength;

	float exactResult = (float)(exceedLength) * barPosition / movableRange;

	// 四捨五入しつつ int化
	result = exactResult + 0.5f;

	//printf( "result(%d) <- exactResult(%f) = exceedLength(%f) * barPosition(%f) / movableRange(%f)\n",  
	//		result, exactResult, exceedLength, barPosition, movableRange, result);

	return result;
}

//=========================================================
// SetCurrentIndex
//=========================================================
int ScrollBar::GetCurrentBarPosition () {
	
	return _barPosition.y;
	
}
void ScrollBar::SetCurrentBarPosition ( int posY ) {
	
	_barPosition.y = posY;
	
	AdjustPosition();
	
	
}
//=========================================================
// Draw
//=========================================================
void ScrollBar::Draw(sf::RenderWindow &window) {

	window.draw(_base);
	
	// バーが出るときだけ描画する
	if ( _actualLength > _displayLength) {
		window.draw(_bar);
	}

	if (_isUpDownButton) {
		_button_Up->Draw(window);
		_button_Down->Draw(window);
	}
}

//=========================================================
// SetPosition
//=========================================================
void ScrollBar::SetPosition(sf::Vector2f arg) {

	_position = arg;
	_base.setPosition( _position );

	//_barPosition = _position;
	_barPosition.x = _position.x;

	_button_Up  ->SetPosition( sf::Vector2f( _position.x, _position.y - _frameSize.x ) );
	_button_Down->SetPosition( sf::Vector2f( _position.x, _position.y + _frameSize.y ) );

}

//=========================================================
// SetFrameSize
//=========================================================
void ScrollBar::SetFrameSize(sf::Vector2f arg) {
	
	_frameSize = arg;

	_base = sf::RectangleShape(_frameSize);
	_base.setOutlineThickness(1.0f);
	_base.setOutlineColor ( sf::Color ( 0,0,0,255 ) );
	_base.setFillColor    ( sf::Color ( 255,50,50, 150 ) );
	_base.setPosition( _position );


	_button_Up  ->SetSize( sf::Vector2f( arg.x, arg.x ) );
	_button_Down->SetSize( sf::Vector2f( arg.x, arg.x ) );

	_button_Up  ->SetPosition( sf::Vector2f( _position.x, _position.y - _frameSize.x ) );
	_button_Down->SetPosition( sf::Vector2f( _position.x, _position.y + _frameSize.y ) );
}

//=========================================================
// MakeBarShape
//=========================================================
void ScrollBar::MakeBarShape() {
	
	//_currentIndex = GetCurrentIndex();
	
	/*
	float totalLength = _length_of_oneItem * _maxLength;
	float barLength   = (totalLength == 0) ? 0 : _frameSize.y * _frameSize.y / totalLength;

	if ( totalLength < _frameSize.y ) {
		barLength = _frameSize.y;
	}
	*/
	
	// バーが出ない状態なら何もしない。バーを消す
	if ( _actualLength <= _displayLength) {
		return;
	}

	float barLength = 0.0f;
	barLength = _frameSize.y * (float)( (float)_displayLength / (float)_actualLength );
	
	/*
	if ( _actualLength <= _displayLength) {
			
		barLength = _frameSize.y;
		
	} else {
		
		barLength = _frameSize.y * (float)( (float)_displayLength / (float)_actualLength );
	}
	*/
	

	_barSize = sf::Vector2f(_frameSize.x, barLength);


	_bar = sf::RectangleShape(_barSize);
	_bar.setOutlineThickness(1.0f);
	_bar.setOutlineColor ( sf::Color ( 0,0,0,255 ) );
	_bar.setFillColor    ( sf::Color ( 50,255,50, 150 ) );
	_bar.setPosition( _position );

	//	int _maxLength;
	//	int _currentIndex;

	_barPosition.x = _position.x;
	
	/*
	_barPosition.y += 
		(_maxLength == 0) ?
		0
		:
		(_frameSize.y - barLength) 
		* 
		
		(float)
		( 
			(float)_currentIndex / (float)_maxLength
		);
	*/


	_bar.setPosition( _barPosition );
}

//=========================================================
// SetMaxLength
//=========================================================
void ScrollBar::SetActualLength( int arg ) {

	_actualLength = arg;

	if ( _actualLength < 0 ) {
		_actualLength = 0;
	}
}
void ScrollBar::SetDisplayLength( int arg ) {
	_displayLength = arg;
}
/*
//=========================================================
// SetLengthOfOneItem
//=========================================================
void ScrollBar::SetLengthOfOneItem( float arg ) {
	_length_of_oneItem = arg;
}
*/

