//=========================================================
//
// GUI SelectBox(implement)
//
//=========================================================

#include <stdio.h>
#include "SelectBox.h"
#include "ISelectBoxItem.h"
#include "ScrollBar.h"
#include "SFML_Utils.h"

using namespace GUI_SFML;

//=========================================================
// Constructor
//=========================================================
SelectBox::SelectBox() {

	_position.x = 10.0f;
	_position.y = 10.0f;

	_position_Offset = sf::Vector2f(0.0f, 0.0f);

	_length_of_oneItem = 20.0f;

	_scrollBar = new ScrollBar();
	_scrollBar->SetActualLength(0);
	//_scrollBar->SetLengthOfOneItem(_length_of_oneItem);
	_scrollBar->SetFrameSize(sf::Vector2f(20.0f, 300.0f));
	_scrollBar->SetPosition( _position );
	_scrollBar->IsUpDownButton(true);

	SetFrameSize( sf::Vector2f(200.0f, 300.0f) );

	_frame.setPosition( _position );


	_displayCount = 10;
	_currentDisplayPosition = 0;

	_currentPointingItem = NULL;
	_currentPointingItenIndex = -1;
	
	_isMousePress   = false;
	_isMouseRelease = false;
	_isMouseOver    = false;
	
	_isDefaultCorsorOn = true;
}
//=========================================================
// SetLengthOfOneItem
//=========================================================
void SelectBox::SetLengthOfOneItem( float arg ) {

	_length_of_oneItem = arg;
	//_scrollBar->SetLengthOfOneItem(_length_of_oneItem);

}
//=========================================================
// Destructor
//=========================================================
SelectBox::~SelectBox() {

	delete _scrollBar;
}

//=========================================================
// Update
//=========================================================
void SelectBox::Update(sf::RenderWindow &window) {

	sf::Vector2f currentDrawPosition = _position + _position_Offset;
	
	
	///// マウス監視
	if ( _isMousePress && !sf::Mouse::isButtonPressed(sf::Mouse::Left)) {
		_isMouseRelease = true;
	} else if (_isMouseRelease == true) {
		_isMouseRelease = false;
	}
	_isMousePress = sf::Mouse::isButtonPressed(sf::Mouse::Left);
	
	
	///// マウスが乗っているかどうか
	int mouseX = sf::Mouse::getPosition(window).x;
	int mouseY = sf::Mouse::getPosition(window).y;
	
	SFML_Utils::PosToScreenPos ( mouseX, mouseY, window );
	
	if
	(
		_position.x < mouseX && mouseX < _position.x + _frameSize.x + _scrollBar->GetFrameSize().x
		&&
		_position.y < mouseY && mouseY < _position.y + _frameSize.y
	) 
	{
		_isMouseOver = true;
	} else {
		_isMouseOver = false;
	}
	
	
	_currentDisplayPosition = 0;
	_currentPointingItem    = NULL;
	
	///// スクロールバーの出番ですか？
	if ( _itemList.size() > _displayCount ) {
		
		_scrollBar->SetActualLength  ( _itemList.size() );
		_scrollBar->SetDisplayLength ( _displayCount    );

		_scrollBar->Update( window );
		_currentDisplayPosition = _scrollBar->GetCurrentIndex();
	}


	for (int i = 0; i < _displayCount; ++i) {

		int index = _currentDisplayPosition + i;
		if ( index >= _itemList.size() ) {
			break;
		}

		_itemList[index]->SetPosition( currentDrawPosition );

		_itemList[index]->Update(window);

		if ( _itemList[index]->IsMouseOver() ) {
			_currentPointingItem = _itemList[index];
			_currentPointingItenIndex = index;
		}



		//currentDrawPosition.y += _itemList[index]->GetSize().y;
		currentDrawPosition.y += _length_of_oneItem;

	}

}

//=========================================================
// スクロール
//=========================================================
void SelectBox::ScrollUp() {
	_scrollBar->MoveUp(1);
}
void SelectBox::ScrollDown() {
	_scrollBar->MoveDown(1);
}
		
		
void SelectBox::SetPosition( sf::Vector2f pos ) { 
	
	_position = pos;
	_frame.setPosition(pos);
	
	_scrollBar->SetPosition( sf::Vector2f( 
										_position.x + _frameSize.x,
										_position.y + 20.0f) );
}

//=========================================================
// Frame apparance
//=========================================================
void SelectBox::SetFrameColor             ( sf::Color color ) { _frame.setFillColor(color); }
void SelectBox::SetFrameOutlineColor      ( sf::Color color ) { _frame.setOutlineColor(color); }
void SelectBox::SetFrameOutlineThickness  ( float value     ) { _frame.setOutlineThickness(value); }
		

//=========================================================
// Draw
//=========================================================
void SelectBox::Draw(sf::RenderWindow &window) {

	window.draw( _frame );

	for (int i = 0; i < _displayCount; ++i) {

		int index = _currentDisplayPosition + i;
		if ( index >= _itemList.size() ) {
			break;
		}


		_itemList[index]->Draw(window);

	}

	if (_currentPointingItem != NULL) {
        
        if ( _isDefaultCorsorOn ) {
        
            _cursor = sf::RectangleShape (_currentPointingItem->GetSize());
            _cursor.setOutlineThickness(1.0f);
            _cursor.setOutlineColor (sf::Color(255,255,0,255));
            _cursor.setFillColor    (sf::Color(0,0,0,0));
            _cursor.setPosition ( _currentPointingItem->GetPosition() );

            window.draw( _cursor );
		}
	}


	_scrollBar->Draw( window );

}


//=========================================================
// 最下段まで移動
//=========================================================
void SelectBox::MoveToBottom() {
	
	_scrollBar->MoveToBottom();
}

//=========================================================
// 最上段まで移動
//=========================================================
void SelectBox::MoveToTop() {
	
	_scrollBar->MoveToTop();
}
		

//=========================================================
// 指定インデックスの項目が上にハミ出ていたら、スクロールを追いつかせる
//=========================================================
void SelectBox::Makeup_OutofUpperBounds   (int index) {
	
	// まず、スクロールバーが出ているかどうか
	if ( _displayCount < _itemList.size() ) {
		
		
		// 上にハミ出ている
		if ( index < _currentDisplayPosition ) {
			
			_scrollBar->MoveUp ( _currentDisplayPosition - index );
		}
	}
	
	// 登場人物たち
	//_itemList.size()
	//_currentDisplayPosition
	//_displayCount
}

//=========================================================
// 指定インデックスの項目が下にハミ出ていたら、スクロールを追いつかせる
//=========================================================
void SelectBox::Makeup_OutofDownnerBounds (int index) {
	
	// まず、スクロールバーが出ているかどうか
	if ( _displayCount < _itemList.size() ) {
		
		// 下にハミ出ている
		if ( index > _currentDisplayPosition + (_displayCount-1) ) {
			
			int moveCount = index - (_currentDisplayPosition + (_displayCount-1));
			
			//printf ( "スクロールバー %d件分 下に動いて欲しい\n", moveCount);
			//printf ( " %d > %d + %d \n",index, _currentDisplayPosition,(_displayCount-1));
			
			_scrollBar->MoveDown ( moveCount );
		}
	}
}




void SelectBox::AddItem( ISelectBoxItem *item ) {
	_itemList.push_back(item);
	
	_scrollBar->SetActualLength( _itemList.size() );
	_scrollBar->SetDisplayLength ( _displayCount );
}
void SelectBox::RemoveItem( ISelectBoxItem *item ) {

	for ( vector<ISelectBoxItem*>::iterator it = _itemList.begin(); it < _itemList.end(); ++it) {
		
		if ( item == (*it) ) {
			_itemList.erase( it );
			break;
		}
	}
	
	_scrollBar->SetActualLength( _itemList.size() );
	_scrollBar->SetDisplayLength ( _displayCount );
}
ISelectBoxItem* SelectBox::GetPointingItem() {
	return _currentPointingItem;
}
ISelectBoxItem* SelectBox::GetClickItem() {
	
	ISelectBoxItem *result = NULL;
	
	if ( _currentPointingItem != NULL ) {
		
		if ( _currentPointingItem->IsMouseOver() ) {
			
			if (_isMouseRelease) {
				result = _currentPointingItem;
			}
		}
	}
	
	return result;
}





void SelectBox::SetDisplayCount( int arg) {_displayCount = arg; }
void SelectBox::SetFrameSize( sf::Vector2f arg) {

	_frameSize = arg; 

	_frame = sf::RectangleShape (_frameSize);
	_frame.setOutlineThickness(1.0f);
	_frame.setOutlineColor (sf::Color(0,0,0,255));
	_frame.setFillColor    (sf::Color(255,255,255,150));

	_scrollBar->SetFrameSize( sf::Vector2f( 20.0f, arg.y - 40.0f )  );
}
