//=========================================================
//
// テキスト表示部分の「１文字」
//
//=========================================================

#include "TextChar.h"
#include "TextImageFactory.h"
#include "../Utils/Utils_String.h"

using namespace GUI_SFML;

//=========================================================
// 定数
//=========================================================
const int TextChar::TEXT_CHAR_TYPE_CHARACTER  = 0;
const int TextChar::TEXT_CHAR_TYPE_NEW_LINE   = 1;
const int TextChar::TEXT_CHAR_TYPE_NEW_PAGE   = 2;
const int TextChar::TEXT_CHAR_TYPE_WAIT_CLICK = 3;

//=========================================================
// コンストラクタ
//=========================================================
TextChar::TextChar(string c) {

	Init ( c, 32, TEXT_CHAR_TYPE_CHARACTER, sf::Color(255,255,255,255) );
}
TextChar::TextChar(string c, int size, int type) {

	Init ( c, size, type, sf::Color(255,255,255,255));
}
TextChar::TextChar(string c, int size, int type, sf::Color color) {

	Init ( c, size, type, color);
}

//=========================================================
// 初期設定
//=========================================================
void TextChar::Init ( string c, int size, int type, sf::Color color ) {

	_char      = c;
	_isVisible = true;
	_type      = type;

	// 通常の文字のときだけ、文字画像を取得
	if ( type == TEXT_CHAR_TYPE_CHARACTER ) {
			
		_sprite = TextImageFactory::Create ( c, size );
		_sprite.setColor(color);
		
		//_width = TextImageFactory::_width;
		if (Utils_String::isHalfWidth(c) ) {
			_width = size / 2.0f;
		} else {
			_width = size;
		}
		
		/*
		TextImageFactory::Create ( _image, c, size, color );

		_texture.loadFromImage ( _image );
		_texture.setSmooth ( true );

		_sprite.setTexture ( _texture );
		*/

	// 通常の文字じゃないなら、文字画像は取得しませんよ。
	} else {
		// 処理なし
	}
}

//=========================================================
// ふりがなを設定
//=========================================================
void TextChar::SetRuby ( string ruby ) {
	
	// 先に文字本体を設定していないとルビのサイズを決定できない。よろしいのですか？
	int rubySize = _sprite.getCharacterSize() / 2.0f; //1.5f;
	
	if (_sprite.getString().isEmpty()) {
		printf ( "文字本体が設定されていないのでフリガナを設定できない [%s]\n", ruby.c_str() );
	}
	
	_ruby = TextImageFactory::Create ( ruby, rubySize );
	_ruby.setOrigin( sf::Vector2f ( _ruby.getGlobalBounds().width / 2.0f, _ruby.getGlobalBounds().height ) );

}

//=========================================================
// デストラクタ
//=========================================================
TextChar::~TextChar() {

}

//=========================================================
// 更新
//=========================================================
void TextChar::Update ( sf::RenderWindow &window ) {

	_sprite.setPosition( _position.x, _position.y );
	
	if ( !_ruby.getString().isEmpty() ) {
		
		_ruby.setPosition( _position.x + _width / 2.0f, _position.y );
		
		// ルビを含めた幅はいくつなのでしたっけ？
		int totalWidth = GetWidth();
		
		if ( _width < totalWidth ) {
			
			int offset = (totalWidth  - _width) / 2;
			
			_sprite.setPosition( _position.x + offset, _position.y );
			
			// ただでさえ小さい文字なので、位置が少数になってブレるのは困るのですよー
			int rubyX = (int)( (_position.x + offset + (_width / 2)) + 0.5f);
			int rubyY = _position.y;
			
			_ruby.setPosition( rubyX , rubyY );
			
			
		}
		
	}
}

//=========================================================
// 描画
//=========================================================
void TextChar::Draw ( sf::RenderWindow &window ) {

	if ( _isVisible && _type == TEXT_CHAR_TYPE_CHARACTER) {
		
		window.draw( _sprite );
		
		if ( !_ruby.getString().isEmpty() ) {
			window.draw( _ruby );
		}
	}
}
//=========================================================
// サイズを取得：通常の文字じゃない場合はゼロ。
//=========================================================
//int TextChar::GetWidth () { return (_type == TEXT_CHAR_TYPE_CHARACTER) ? _image.getSize().x : 0; }
//int TextChar::GetHeight() { return (_type == TEXT_CHAR_TYPE_CHARACTER) ? _image.getSize().y : 0; }
int TextChar::GetHeight() { return (_type == TEXT_CHAR_TYPE_CHARACTER) ? _sprite.getGlobalBounds().height : 0; }
int TextChar::GetWidth () { 
	
	int result = 0;
	
	if ( _ruby.getString().isEmpty() ) {
		
		result = (_type == TEXT_CHAR_TYPE_CHARACTER) ? _width : 0; 
		
	} else {
		
		// ルビがある場合、ルビの幅と文字本体の幅の大きい方を「この文字の幅」とする
		
		result = _ruby.getGlobalBounds().width;
		
		int bodyWidth = (_type == TEXT_CHAR_TYPE_CHARACTER) ? _width : 0;
		
		if ( result < bodyWidth ) {
			result = bodyWidth;
		}
		
	}
	
	return result;
	//return (_type == TEXT_CHAR_TYPE_CHARACTER) ? _width : 0; 
}

























