//=========================================================
//
// AIさんのお世話係
//
//=========================================================

#include "AI_Manager.h"

#include <chrono>


//=========================================================
// コンストラクタ
//=========================================================
AI_Manager::AI_Manager() {

	printf ( "[AI_Manager::AI_Manager()]\n" );

	///// AIスレッド関連
	_guestAI = NULL;


#if defined(_WIN32)

	_AI_ThreadHandle = NULL;
	InitializeCriticalSection( &_criticalSection_For_AI );

#elif defined(__linux__)

	//_criticalSection_For_AI = PTHREAD_RECURSIVE_MUTEX_INITIALIZER_NP;
	_criticalSection_For_AI = PTHREAD_RECURSIVE_MUTEX_INITIALIZER;

#endif
	
	
}

//=========================================================
// デストラクタ
//=========================================================
AI_Manager::~AI_Manager() {

	printf ( "[AI_Manager::~AI_Manager()]\n" );

	EndThread();

	delete _currentProgresData.problemState;

	//DeleteCriticalSection( &_criticalSection_For_AI );
}


//=========================================================
// 思考開始：開始できないときは false  
//【引数】
// こっちでコピーをとって持ち回ります。
// なので、そっちはそっちで、そっちで生成したインスタンスの面倒を見てね。
//=========================================================
bool AI_Manager::ThinkStart ( AI_Modules::AI *AI, AI_Modules::InnerExpression *currentState ) {

	printf ( "[AI_Manager::ThinkStart()]\n" );

	bool result = false;;

	///// 思考スレッドが終わっていないときは？
	if ( IsStillActive_AI_Thread() ) {

		// 終了させます。
		EndThread();
	}

	///// いらっしゃいませ。
	_guestAI = AI;

	///// 進行状況を「開始直後：０％」にセット。
	_currentProgresData.progressData.progressRate = 0.0f;

	///// この局面について考え中。
	_currentProgresData.problemState = currentState->GetCopy();
	

	///// 思考スレッドを作成・開始
#if defined(_WIN32)

	_AI_ThreadHandle = (HANDLE)_beginthreadex ( NULL, 0, AI_Manager::AI_Thread_Entrance, this, 0, NULL );

#elif defined(__linux__)

	_AI_ThreadHandle = std::thread ( AI_Manager::AI_Thread_Entrance, this );
	_AI_ThreadHandle.detach();

#endif


	
	return result;
}

//=========================================================
// AI思考スレッド：入り口
// 引数に GameManagerインスタンスを渡して、インスタンスメソッドの思考スレッドを呼ぶ
//=========================================================
unsigned int AI_Manager::AI_Thread_Entrance(void *obj) {

	///// インスタンスを取得
	AI_Manager *self = (AI_Manager*)obj;

	///// インスタンスごとの処理を呼ぶ。
	self->AI_Thread();

	return 0;
}

//=========================================================
// AI思考スレッド：インスタンスごとの本体
//=========================================================
void AI_Manager::AI_Thread() {

	///// さあ、思考せよ
	// PEEファイル手作業模索用、という出自のメソッドなのでした。格上げさせてあげません？
	_guestAI->GetNextMove_PEE_Test( _currentProgresData.problemState, _currentProgresData.progressData );

	///// 思考終了。
	// 進捗を 100% にして、問題局面を解放。
	_currentProgresData.progressData.progressRate = 100.0f; // これは AI さんの側でもしてくれるはず。
	delete _currentProgresData.problemState;
	_currentProgresData.problemState = NULL;

	///// お客さんAI にも、お帰りいただく？
	_guestAI = NULL;
}



//=========================================================
// 進行状況を教えて。
//=========================================================
AI_Modules::ProgressData AI_Manager::GetThinkProgress() {

	AI_Modules::ProgressData result = _currentProgresData.progressData;

	return result;
}

//=========================================================
// クリア
// 進行状況を -1.0f に戻すとか、そのへんの処理
// また次回もよろしくおねがいします。
//=========================================================
void AI_Manager::Clear() {

	printf ( "[AI_Manager::Clear()]\n" );

	///// スレッド閉じて（一応）
	EndThread();

	///// 問題局面を掃除して、
	delete _currentProgresData.problemState;
	_currentProgresData.problemState = NULL;

	///// 進行状況を初期化。これで、進捗のパーセントは負の数になってくれてるはず。
	_currentProgresData.progressData = AI_Modules::ProgressData();

	_guestAI = NULL;
}

//=========================================================
// 中止
//=========================================================
void AI_Manager::ThinkStop() {

	///// スレッド閉じて、
	EndThread();

	///// 問題局面を掃除して、
	delete _currentProgresData.problemState;
	_currentProgresData.problemState = NULL;

	///// 進行状況を初期化。これで、進捗のパーセントは負の数になってくれてるはず。
	_currentProgresData.progressData = AI_Modules::ProgressData();

	_guestAI = NULL;

}

//=========================================================
// スレッドが残っていれば終了させる
//=========================================================
void AI_Manager::EndThread() {

	///// 思考スレッドが終わっていないときは？
	if ( IsStillActive_AI_Thread() ) {

		printf ( "[AI_Manager::EndThread()] スレッドが残っていた。終了させます。\n" );

		///// 終了させます。

		// 終了してね、とお願いして、
		_currentProgresData.progressData.isStop = true;

#if defined(_WIN32)

		// 終了を待って、
		WaitForSingleObject(_AI_ThreadHandle, INFINITE);
		
		// ハンドルを閉じる
		CloseHandle(_AI_ThreadHandle);
		
#elif defined(__linux__)
		
		while ( true ) {
			
			if ( !IsStillActive_AI_Thread() ) {
				break;
			}
			printf ( "AI_Manager::EndThread()：思考スレッドの終了を待機中……\n" );
			std::this_thread::sleep_for(std::chrono::milliseconds(1000));
		}
		
#endif


		printf ( "[AI_Manager::EndThread()] 終了させました。\n" );

	} else {

		printf ( "[AI_Manager::EndThread()] スレッドが残っていなかった。何もしません。\n" );

	}
}




//=========================================================
// AI思考スレッド（スレッドそのもの）が生きているかどうか？
//=========================================================
bool AI_Manager::IsStillActive_AI_Thread () {
	
	bool result = false;
	
#if defined(_WIN32)

	DWORD exitCode;
	GetExitCodeThread ( _AI_ThreadHandle, &exitCode );

	if ( exitCode == STILL_ACTIVE) {
		result = true;
	}
#elif defined(__linux__)
	
	if ( _currentProgresData.progressData.progressRate != 100.0f && _currentProgresData.progressData.progressRate > 0.0f) {
		result = true;
	}
#endif
 
	return result;
}
