//=========================================================
//
// 「対戦相手」としてのActor
//
//=========================================================

#include "Actor.h"
#include "Define.h"
#include "../Utils/Utils_Convert.h"
#include "../Utils/Utils_Lua.h"
#include "../Utils/Utils_String.h"
#include "../Utils/Utils_Others.h"
#include "CardFactory.h"
#include "Application.h"
#include "Image.h"
#include "GameManager.h"

//=========================================================
// 定数；リアクションを求めるときのコンテキスト
//=========================================================
//const string Actor::REACTION_CONTEXT_MOVED_BY_MESELF   = "MOVED_BY_MESELF";   // 自分が着手した
//const string Actor::REACTION_CONTEXT_MOVED_BY_OPPONENT = "MOVED_BY_OPPONENT"; // 相手が着手した

//=========================================================
// 定数；リアクションを問い合わせるためのコンテキスト情報用の区切り文字
//=========================================================
const string Actor::REACTION_CONTEXT_EQUALSIGN = "===";   // 等号
const string Actor::REACTION_CONTEXT_DELIMITER = "<@>"; // デリミタ

//=========================================================
// コンストラクタ
//=========================================================
Actor::Actor ( string scriptFileName, GUI_SFML::TextMessageWindow *textmessageWindow) {

	///// デフォルトでは、位置は画面の中央
	_position.x = APPLICATION::WINDOW_SIZE::WIDTH  / 2;
	_position.y = APPLICATION::WINDOW_SIZE::HEIGHT / 2;

	_lua = NULL;
	_isReverse = false;
	_textMessageWindow = textmessageWindow;
	//_framecount_TextMessage = 0;
	_isActive = true;
	
	_gameManager = NULL;

	// 自分は先手？ 後手？
	/*
		最初は後手だと思っている。
		局面評価値履歴がない状態で着手決定通知を受けたら、先手、ということ。
	*/
	//_myPlayingTurn = "SECOND";
	_myPlayingSide = "P2";
	_myName        = "";

	///// Lua読み込み
	_scriptFileName = scriptFileName;
	Script_Init();

	_faceImage = new Image();
	_faceImage->SetLabel("actor");
	_faceImage->SetFadeInSpeed(10);
	_faceImage->SetFadeOutSpeed(10);
	
	// マウスオーバーでの半透明機能オン
	_faceImage->SetMouseOverSeeThrough ( true );

	///// 表情画像のリストを登録しましょう。
	Script_GetImageList();
	
	
}

//=========================================================
// 初期化
//=========================================================
void Actor::Init() {

	/*
		機嫌を直す？
	*/
	Script_Init(); // これで、内部の感情値はクリアされてしまう。

	///// 自分は先手(FIRST)？ 後手(SECOND)？
	//_myPlayingTurn = "SECOND";
	
	///// どっち側：P1 or P2（ほぼP2であることでしょう：<Game>タグ内の <Participant side="P2">で指定）
	//_myPlayingSide = "P2";

	/*
		棋譜系の値も、Lua内で持てばいいのかも。Luaスクリプト次第では、前回の結果を引きずる、みたいな。
	*/
	///// （自分が）着手した結果の履歴。
	_thinkResultHistory.clear();

	///// 局面評価値の履歴
	//_stateEvaluationHistory.clear();

}

//=========================================================
// デストラクタ
//=========================================================
Actor::~Actor() {

	///// Lua 終了
	if ( _lua != NULL ) {
		lua_close ( _lua );
	}

	///// 受け取ったテキスト欄は、こっちで削除しますよ。
	//delete _textMessageWindow;
	// ↑これをするとdelete時にメモリエラーになる。
	/*
		別インスタンスで持つのではなく、同一オブジェクトを参照させてもらうこととする。
	*/
	delete _faceImage;
	_faceImage = NULL;
	
	//while ( _faceImageList.size() > 0 ) {
	//	delete _faceImageList.begin()->second;
	//	_faceImageList.erase ( _faceImageList.begin() );
	//}

}

//=========================================================
// 位置を指定（画像の位置もこの段階で反映させる）
//=========================================================
void Actor::SetPosition ( sf::Vector2f pos ) { 

	_position = pos; 

	///// 画像リスト：ラベル・画像
	//map <string, ImageSet*> _faceImageList;	
	
	/*
	for ( map<string, ImageSet*>::iterator it = _faceImageList.begin(); it != _faceImageList.end(); ++it ) {
		
		ImageSet *is = it->second; 
		is->sprite.setPosition ( _position );
		
		if ( _isReverse ) {
			is->sprite.setScale ( -1, 1 );
		}
	}
	*/
	_faceImage->SetPosition( pos );
	if ( _isReverse ) {
		_faceImage->FlipX();
	}

}
sf::Vector2f Actor::GetPosition () { 
	return _position; //_faceImage->GetCurrentPosition();
}
sf::Vector2f Actor::GetImagePositionCurrent () { return _faceImage->GetCurrentPosition(); }
sf::Vector2f Actor::GetImagePositionGoal    () { return _position; } // つまりこういうことでよいハズ。
	
//=========================================================
// 位置を移動する
//=========================================================
void Actor::MoveTo ( sf::Vector2f pos ) {
	
	// 内部用の値は即時に変更する
	_position = pos;
	
	// 顔画像は「移動」とする
	_faceImage->MoveTo ( pos );
	
}


//=========================================================
// P1側？ P2側？
//=========================================================
string Actor::GetPlayingSide() {

	///// 反転しているかどうかで決め付けてしまうのですよー。
	//string result = ( _isReverse ) ? "P2" : "P1";
	
	string result = _myPlayingSide;

	return result;
}

/*
//=========================================================
// 先手(FIRST)？ 後手(SECOND)？
//=========================================================
string Actor::GetPlayingTurn() {

	string result = _myPlayingTurn;

	return result;
}
*/


//=========================================================
// 更新
//=========================================================
void Actor::Update ( sf::RenderWindow &window ) {

	if (!_isActive) { return; }
	
	if ( _gameManager == NULL ) {
		printf ( "[%s] _gameManagerがないのでUpdate()を中止します。\n", _myName.c_str() );
	}
	
	///// 顔画像をお世話するよ
	_faceImage->Update(window);

	/*
	if ( Application::GetInstance()->_mouseState_Left == MOUSE::BUTTON::STATE::RELEASE) {

		_framecount_TextMessage = 60;
		_textMessageWindow->AddText_asFinishedText ( "/セリフ！" );
	}
	*/
	/*
	///// かけあい実験
	if ( _myPlayingSide == "P1" && Application::GetInstance()->_mouseState_Middle == MOUSE::BUTTON::STATE::RELEASE) {
		
		{
			KakeaiInfo kakeai;
			kakeai.participantId = "char2";
			kakeai.faceName      = "NORMAL";
			kakeai.remark        = "掛け合いパート1[c][l]";
			_kakeaiInfoList.push_back ( kakeai );
		}
		{
			KakeaiInfo kakeai;
			kakeai.participantId = "prof";
			kakeai.faceName      = "NORMAL";
			kakeai.remark        = "掛け合いパート2[c][l]";
			_kakeaiInfoList.push_back ( kakeai );
		}
		{
			KakeaiInfo kakeai;
			kakeai.participantId = "char2";
			kakeai.faceName      = "NORMAL";
			kakeai.remark        = "掛け合いパート3[c][l]";
			_kakeaiInfoList.push_back ( kakeai );
		}
		{
			KakeaiInfo kakeai;
			kakeai.participantId = "prof";
			kakeai.faceName      = "NORMAL";
			kakeai.remark        = "掛け合いパート4[c][l]";
			_kakeaiInfoList.push_back ( kakeai );
		}
	}
	*/
	
	///// あってもなくてもかけあい処理
	Kakeai();

}

//=========================================================
// 掛け合い残り存在確認
//=========================================================
bool Actor::IsKakeaiAllFlushed() {
	
	bool result = false;
	
	if ( _kakeaiInfoList.empty() ) {
		result = true;
	}
	
	return result;
}

//=========================================================
// 掛け合い処理
//=========================================================
void Actor::Kakeai() {
	
	///// テキスト欄、完了してないなら何もしない。
	if (!_textMessageWindow->IsAllFinished()) {
		return;
	}
	
	///// かけあいデータがないなら何もしない
	if ( _kakeaiInfoList.empty() ) {
		return;
	}
	
	///// 先頭のデータを取り出して実行する
	vector<KakeaiInfo>::iterator it = _kakeaiInfoList.begin();
	KakeaiInfo kakeaiInfo = *it;
	
	//kakeaiInfo.
	printf ("掛け合い一件ありました：ID[%s], FACE[%s], REMARK[%s]\n", kakeaiInfo.participantId.c_str(), kakeaiInfo.faceName.c_str(), kakeaiInfo.remark.c_str());
	
	//_textMessageWindow->AddText ( kakeaiInfo.remark );
	//_gameManager->AcceptKakeai ( kakeaiInfo.participantId, kakeaiInfo.remark, kakeaiInfo.faceName );
	_gameManager->AcceptKakeai ( kakeaiInfo );
	
	
	///// 実行したら削除する
	_kakeaiInfoList.erase(it);

}

//=========================================================
// 掛け合い用：セリフと表情を受け付ける（GameManagerから）
//=========================================================
void Actor::AcceptKakeai ( string remark, string faceName, string operation ) {
	
	ReactionDataSet react;
	react.remark   = remark;
	react.faceName = faceName;
	
	ReactionExecute ( react );
	
	// MOVE-TO:400*560
	if ( operation != "" ) {
		
		if ( Utils_String::IsContainsPattern ( operation, "MOVE-TO" ) ) {
			
			vector<string> elems = Utils_String::Split ( operation, ":" );
			string valuesString  = ( elems.size() >= 2 ) ? elems[1] : "0*0";
			
			vector<string> positionInfo = Utils_String::Split ( valuesString, "*" );
			string xString  = ( elems.size() >= 2 ) ? positionInfo[0] : "0"; // ２つないとおかしい、という意味で
			string yString  = ( elems.size() >= 2 ) ? positionInfo[1] : "0";
			
			int toX = Utils_Convert::String_to_Int ( xString );
			int toY = Utils_Convert::String_to_Int ( yString );
			
			MoveTo ( sf::Vector2f( toX, toY ) );
		
			
		}
		
	}
	
}


//=========================================================
// 描画
//=========================================================
void Actor::Draw   ( sf::RenderWindow &window ) {

	if (!_isActive) { return; }

	///// 顔画像を表示するよ
	/*
	map<string, ImageSet*>::iterator it = _faceImageList.find ( _faceName_Temporary );
	if ( it != _faceImageList.end() ) {
		window.draw ( it->second->sprite );
	}
	*/
	_faceImage->Draw(window);

	///// セリフ欄
	// TODO セリフの扱いはまた後で
	/*
	if ( _framecount_TextMessage > 0 ) {
		_textMessageWindow->Draw(window);
	}
	*/

}

//=========================================================
// 着手時のエフェクトを決定する
//【引数】
// 着手を思考した結果
//=========================================================
Actor::MoveEffectData Actor::DecideMoveEffect ( AI_Modules::ProgressData thinkResult ) {

	/*
		ついでに、思考結果を、履歴に保存してもいる。
	*/

	Actor::MoveEffectData result;


	// ちなみに、このとき、局面評価値の履歴が０件なら、自分は先手、ということ。
	//_myPlayingTurn = "FIRST";
	
	// ↑つまり、こういう判定をして、値を入れるってことでいいんですよね？？？
	//if ( _thinkResultHistory.size() > 0 ) {
	//	//_myPlayingTurn = "SECOND";
	//}

	// Lua関数さん、Lua関数さん
	result = Script_GetMoveEffectLevel ( thinkResult.evaluationSumUp );//, _thinkResultHistory );

	// 今回の思考結果を履歴に入れておく：つまり今回分は履歴に含まずに判定処理を行うということ。
	_thinkResultHistory.push_back ( thinkResult );

	return result;
}

//=========================================================
// 外部からの通知：試合が始まったところ
//=========================================================
void Actor::Notice_BattleStart () {
	
	// Lua側にリアクションを問い合わせる
	string contextInfo = MakeContextForReaction_BATTLE_START();
	ReactionViaLua ( contextInfo );
}
//=========================================================
// 外部からの通知：試合が終了するところ
//=========================================================
void Actor::Notice_BattleEnd ( string battleResult ) {
	
	/*
		battleResult : P1Win or P2Win or Draw
	*/

	// Lua側にリアクションを問い合わせる
	string contextInfo = MakeContextForReaction_BATTLE_END ( battleResult );
	ReactionViaLua ( contextInfo );
}


//=========================================================
// 外部からの通知：手番開始したところ
//=========================================================
void Actor::Notice_TurnStart   ( string participantId ) {
	
	// Lua側にリアクションを問い合わせる
	string contextInfo = MakeContextForReaction_TURN_START ( participantId );
	ReactionViaLua ( contextInfo );
}


//=========================================================
// 外部からの通知：着手をしようとするところ
//=========================================================
void Actor::Notice_MoveIsReady ( string participantId, AI_Modules::Move attemptingMove, string sumUpEval) {
	
	// Lua側にリアクションを問い合わせる
	string contextInfo = MakeContextForReaction_BEFORE_MOVE ( participantId, attemptingMove, sumUpEval);
	ReactionViaLua ( contextInfo );
}
 
//=========================================================
// 外部からの通知：着手が行われました
//=========================================================
void Actor::Notice_MoveIsDone ( string participantId, AI_Modules::Move latestMove, AI_Modules::InnerExpression *latestState ) {

	/*
		◆ ここですること（していたこと） ◆
		・最新の盤面を評価して、リストに入れる。
		・評価値（の変化）を基づいて感情値を変動させる（Lua使用）
	*/

	///// どうにかして、局面についての評価値を得る。Lua経由で。
	//int evaluationForTheState = Script_EvaluateState ( latestState );

	///// 局面についての評価値を履歴に入れる
	//_stateEvaluationHistory.push_back ( evaluationForTheState );

	///// 局面評価値の履歴に基づいて、感情値を変動させる。Lua内で。
	/*
		引数：評価値の履歴。
	*/
	//Script_EmotionProceed ( _stateEvaluationHistory );

	
	
	// 着手が行われたことにかかり、Lua側に最新情報の通知をする
	Script_SetNotificationInfo_MoveDone ( participantId, latestMove, latestState);
	

	///// 感情値にもとづいてリアクションを起こしたり起こさなかったり。Lua経由で
	///// リアクションを問い合わせる：着手完了後イベント
	string contextInfo = MakeContextForReaction_AFTER_MOVE ( participantId, latestMove, latestState );
	ReactionViaLua ( contextInfo );

}

//=========================================================
// リアクションをLua側に問い合わせて実行：public じゃなくてもいいのかもしれない
//=========================================================
//void Actor::ReactionViaLua ( AI_Modules::Move latestMove ) {
void Actor::ReactionViaLua ( string contextInfo ) {

	/*
		この箇所は別メソッドにした方がいいのかも。
		リアクション起動処理。
		
		というわけで、しました。
		
		ここでしていること
			・コンテキスト情報を引数で受け取って、
			・Lua側に問い合わせて、
			・リアクションがあれば実行
		
		イベント別のコンテキスト情報作成は各イベント用の情報作成メソッドにて。
		
	*/
	/*
	// 自分の手かどうか。
	bool isMyMove = false;
	if      ( latestMove.turn == AI_Modules::TURN::BLACK) { isMyMove = ( GetPlayingSide() == "P1" ); }
	else if ( latestMove.turn == AI_Modules::TURN::WHITE) { isMyMove = ( GetPlayingSide() == "P2" ); }

	// リアクションを得るコンテキストは、「自分が着手した」または「相手が着手した」
	string context = "";
	if ( isMyMove ) { context = REACTION_CONTEXT_MOVED_BY_MESELF; }
	else            { context = REACTION_CONTEXT_MOVED_BY_OPPONENT; }
	*/

	ReactionDataSet react = Script_React ( contextInfo );
	
	// 掛け合いデータが指定されている場合、受け取る。
	if ( react.kakeaiInfo != "" ) {
		
		printf ( "Luaから来た掛け合い情報：%s\n", react.kakeaiInfo.c_str() );
		
		vector<string> infoList = Utils_String::Split ( react.kakeaiInfo, "<>" );
		for ( unsigned int i = 0; i < infoList.size(); ++i ) {
			
			KakeaiInfo kakeaiInfo;
			
			map<string, string> info = Utils_String::ParseStringAsMap ( infoList[i], ",", "=" );
			
			
			kakeaiInfo.participantId = UTILS_OTHERS::GetValueFromMap ( info, "ID",        "" );
			kakeaiInfo.faceName      = UTILS_OTHERS::GetValueFromMap ( info, "FACE",      "" );
			kakeaiInfo.remark        = UTILS_OTHERS::GetValueFromMap ( info, "REMARK",    "" );
			kakeaiInfo.operation     = UTILS_OTHERS::GetValueFromMap ( info, "OPERATION", "" );
			
			printf ("掛け合い情報一件追加：ID[%s], FACE[%s], REMARK[%s], OPERATION[%s]\n", kakeaiInfo.participantId.c_str(), kakeaiInfo.faceName.c_str(), kakeaiInfo.remark.c_str(), kakeaiInfo.operation.c_str());
			
			// 追加。消さない。
			_kakeaiInfoList.push_back ( kakeaiInfo );
		}
		
		// ここで1フレーム分、かけあいを開始しておく：フレームの関係で、着手時に掛け合いがあるのにないものとして処理が進んでしまうのを防ぐ
		Kakeai();

	// 掛け合いデータが指定されていない場合は、通常のリアクション情報を実行する
	} else {
		
		/////
		// 掛け合い指定の方が通常リアクションよりも優先ということにする。
		ReactionExecute ( react );
	}
	
	
	
	
}

//=========================================================
// リアクション実行
//=========================================================
void Actor::ReactionExecute ( ReactionDataSet react ) {
	
	// 現在とは違う画像にするのであれば、そのようにする。
	if ( react.faceName != "" && react.faceName != _currentFaceName) {

		_currentFaceName = react.faceName;
		_faceImage->ChangeImage ( _faceImageFileName [ _currentFaceName ], Image::CHANGE_IMAGE_EFFECT_FADE_IN );
	}
	
	
	///// セリフを言う
	/*
		このリアクション問い合わせを別メソッドのしたのを同じ理由で、
		セリフを言うこの箇所もメソッドを分けるのがいいのかも。
	*/
	string remark    = "";
	string nameLabel = "";
	if ( react.remarkWithAnotherNameLabel != "" ) {
		
		vector<string> rrrrrr = Utils_String::Split ( react.remarkWithAnotherNameLabel, "<>" );
		            nameLabel = ( rrrrrr.size() >= 1 ) ? rrrrrr[0] : "";
					   remark = ( rrrrrr.size() >= 2 ) ? rrrrrr[1] : "";
					   
	} else if ( react.remark != "" ) {
		
		remark    = react.remark;
		nameLabel = _myName;
	}
	
	if ( remark != "" ) {

		printf ( "セリフ指定（名札[%s]）：%s\n", remark.c_str(), nameLabel.c_str() );
		
		_textMessageWindow->SetSpeakerName ( nameLabel );
		
		// 必ず新ページで。
		//_textMessageWindow->AddText( "[p]" + remark );
		
		// やはり改ページも自前で制御したい。
		_textMessageWindow->AddText( remark );
		
		/*
			テキスト欄が表示完了していない間はその他の動作がブロックされる。
			試合進行側にて。
		*/
	}
	
	
	
}



//=========================================================
// リアクションを問い合わせるためのコンテキスト情報を作成：BATTLE_START
//=========================================================
string Actor::MakeContextForReaction_BATTLE_START () {

	string result = "EVENT" + REACTION_CONTEXT_EQUALSIGN + "BATTLE_START";

	return result;
	
	
}
//=========================================================
// リアクションを問い合わせるためのコンテキスト情報を作成：TURN_START
//=========================================================
string Actor::MakeContextForReaction_TURN_START   ( string participantId ) {

	string result = "EVENT" + REACTION_CONTEXT_EQUALSIGN + "TURN_START";
	result += REACTION_CONTEXT_DELIMITER + "PARTICIPANT_ID" + REACTION_CONTEXT_EQUALSIGN + participantId;

	return result;
	
}
//=========================================================
// リアクションを問い合わせるためのコンテキスト情報を作成：BEFORE_MOVE
//=========================================================
string Actor::MakeContextForReaction_BEFORE_MOVE  ( string participantId, AI_Modules::Move attemptingMove, string sumUpEval) {

	string result = "EVENT" + REACTION_CONTEXT_EQUALSIGN + "BEFORE_MOVE";

	result += REACTION_CONTEXT_DELIMITER + "PARTICIPANT_ID" + REACTION_CONTEXT_EQUALSIGN + participantId;
	result += REACTION_CONTEXT_DELIMITER + "MOVE"           + REACTION_CONTEXT_EQUALSIGN + MoveToString(attemptingMove);
	result += REACTION_CONTEXT_DELIMITER + "SUMUP_EVAL"     + REACTION_CONTEXT_EQUALSIGN + sumUpEval;

	return result;
	
	
}
//=========================================================
// リアクションを問い合わせるためのコンテキスト情報を作成：AFTER_MOVE
//=========================================================
string Actor::MakeContextForReaction_AFTER_MOVE   ( string participantId, AI_Modules::Move latestMove, AI_Modules::InnerExpression *latestState ) {

	string result = "EVENT" + REACTION_CONTEXT_EQUALSIGN + "AFTER_MOVE";

	result += REACTION_CONTEXT_DELIMITER + "PARTICIPANT_ID" + REACTION_CONTEXT_EQUALSIGN + participantId;
	result += REACTION_CONTEXT_DELIMITER + "MOVE"           + REACTION_CONTEXT_EQUALSIGN + MoveToString(latestMove);
	
	// 文字列が大きくなりすぎるので、ここでは渡さない。着手完了通知の側でも渡しているので、必要ならLua側でそれを保存して使いまわしてもらうこととする
	//result += REACTION_CONTEXT_DELIMITER + "STATE"          + REACTION_CONTEXT_EQUALSIGN + InnerExpressionToString(latestState);

	return result;
	
}

//=========================================================
// リアクションを問い合わせるためのコンテキスト情報を作成：BATTLE_END
//=========================================================
string Actor::MakeContextForReaction_BATTLE_END   ( string battleResult ) {
	
	///// どっちサイドの勝ちで、自分はどっちサイドなのか、という情報を知らせる
	string winSide = "";
	string mySide  = _myPlayingSide;
	
	if ( battleResult == "P1Win" ) { winSide = "P1"; }
	if ( battleResult == "P2Win" ) { winSide = "P2"; }
	
	string result = "EVENT" + REACTION_CONTEXT_EQUALSIGN + "BATTLE_END";

	result += REACTION_CONTEXT_DELIMITER + "WIN_SIDE" + REACTION_CONTEXT_EQUALSIGN + winSide;
	result += REACTION_CONTEXT_DELIMITER + "MY_SIDE"  + REACTION_CONTEXT_EQUALSIGN + mySide;

	return result;
}
	
	
	
	
	
	
int MethodTest ( lua_State *lua ) {
	
	string a = "文字列";
	printf ("C++側のコード\n");
	
	
	// 引数を受け取る
	string arg1 = lua_tostring( lua, 1 );
	int arg2 = lua_tointeger( lua, 2 );
	string arg3 = lua_tostring( lua, 3 );

	printf ("arg1 : %s\n", arg1.c_str());
	printf ("arg2 : %d\n", arg2);
	printf ("arg3 : %s\n", arg3.c_str());
	
	// 戻り値をセット
	//lua_pushinteger(lua, 1000 + arg1);
	//lua_pushstring(lua, a.c_str());

	//return 2; // 戻り値の数

	return 0;
}

/*
Actor* Actor::LuaedObject = NULL;

int Actor::LuaTest( lua_State *lua ) {
	Actor::LuaedObject->LuaTest_Object(lua);
}

int Actor::LuaTest_Object( lua_State *lua ) {

	string a = "文字列";
	printf ("C++側のコード：クラスメソッドの実験ですよ。%s\n", GetName().c_str());
	
	
	// 引数を受け取る
	string arg1 = lua_tostring( lua, 1 );
	int arg2 = lua_tointeger( lua, 2 );
	string arg3 = lua_tostring( lua, 3 );

	printf ("arg1 : %s\n", arg1.c_str());
	printf ("arg2 : %d\n", arg2);
	printf ("arg3 : %s\n", arg3.c_str());
	
	// 戻り値をセット
	//lua_pushinteger(lua, 1000 + arg1);
	//lua_pushstring(lua, a.c_str());

	//return 2; // 戻り値の数

	return 0;
}
*/


//=========================================================
// 外部スクリプトとの連携部分：初期設定
//=========================================================
void Actor::Script_Init() {

	// 既存のLuaオブジェクトがあったらクリアしておく
	if ( _lua != NULL ) {
		lua_close(_lua);
	}
	// Lua 初期設定
	_lua = luaL_newstate();
	luaL_openlibs ( _lua );

	// Luaファイル読み込み
	if ( luaL_dofile (_lua, _scriptFileName.c_str()) ) {

		// エラーメッセージ
		string error_message = lua_tostring(_lua, -1);
		printf ( "Error：luaL_dofile: %s\n", error_message.c_str());

		///// Lua 終了
		lua_close ( _lua );
		_lua = NULL;

	// エラーなし
	} else {
		// よかったよかった。

		// 初期設定関数を呼び出す
		lua_getglobal ( _lua, "Init" );
		lua_pcall ( _lua, 0, 0, 0 );

	}
	
	/*
	lua_register ( _lua, "Method", MethodTest );
	lua_register ( _lua, "LuaTest", Actor::LuaTest );
	
	Actor::LuaedObject = this;
	
	lua_getglobal ( _lua, "RegistClassID" );
	lua_pushstring ( _lua, "aaaaaaaaaaaaaaaaaa" );

	// 関数呼び出し
	if ( lua_pcall ( _lua, 1, 0, 0 ) != 0 ) {
		printf("error : %s\n", lua_tostring ( _lua, -1) );
		lua_pop ( _lua, 1);
	}
	*/
}

//=========================================================
// 外部スクリプトとの連携部分：表情として使う画像リストを取得
//=========================================================
void Actor::Script_GetImageList() {

	/*
		◆ ここの目的 ◆
		・map <string, ImageSet> _imageList に 値を登録すること
		・初期状態の 顔画像名を取得すること

	*/

	// Lua関数をスタックに積む
	lua_getglobal ( _lua, "GetFaceImageList" );
	
	// 関数呼び出し
	if ( lua_pcall ( _lua, 0, 2, 0 ) != 0 ) {

		printf("error : %s\n", lua_tostring ( _lua, -1) );
		lua_pop ( _lua, 1);
	}

	// 第１戻り値：顔画像のファイル名のリスト  「ID1:ファイル名1,ID2:ファイル名2,ID3:ファイル名3...」
	string faceImageListString = Utils_Lua::GetReturnValue_asString ( _lua, 1 );
	map<string, string> faceImageList =  Utils_Convert::ReadStringAsMap ( faceImageListString, ",", "=" );

	for ( map<string, string>::iterator it = faceImageList.begin(); it != faceImageList.end(); ++it ) {

		string name     = it->first;
		string fileName = it->second;

		_faceImageFileName[name] = fileName;
		
		/*
		ImageSet *imageSet = new ImageSet();
		imageSet->image.loadFromFile ( it->second );
		imageSet->texture.loadFromImage ( imageSet->image );
		imageSet->sprite.setTexture ( imageSet->texture );
		imageSet->sprite.setOrigin ( imageSet->image.getSize().x / 2.0f, imageSet->image.getSize().y / 2.0f );

		imageSet->sprite.setPosition ( _position );
		if ( _isReverse ) { imageSet->sprite.setScale ( -1, 1 ); }
		
		
		_faceImageList[name] = imageSet;
		*/
	}

	// 第２戻り値：最初の顔画像
	_currentFaceName = Utils_Lua::GetReturnValue_asString ( _lua, 2 );
	_faceImage->ChangeImage ( _faceImageFileName[_currentFaceName], Image::CHANGE_IMAGE_EFFECT_NONE );
	
	/*
	_faceName_Temporary = "";
	_faceName_Base      = Utils_Lua::GetReturnValue_asString ( _lua, 2 );
	_framecount_ForCurrentFace = 0;
	_faceImage->ChangeImage ( _faceImageFileName[_faceName_Base], Image::CHANGE_IMAGE_EFFECT_NONE );
	*/

	// 戻り値をスタックから降ろす
	lua_pop ( _lua, -2 );
	lua_pop ( _lua, -1 );
}

//=========================================================
// スクリプトとの連携部分：着手時のエフェクトを決定
//=========================================================
Actor::MoveEffectData Actor::Script_GetMoveEffectLevel ( string EVALUATION_SUM_UP ) { //, vector<AI_Modules::ProgressData> thinkResultHistory ) {

	Actor::MoveEffectData result;

	result.effectLevel = 0;

	// これはLua側で履歴を保存することとする
	/*
	// 今までの思考結果の履歴から、総括評価値の履歴を、Luaに渡せる形にする
	string evalSumUpForLua = "";
	for ( unsigned int i = 0; i < _thinkResultHistory.size(); ++i) {

		if ( i > 0 ) { evalSumUpForLua += ","; }

		evalSumUpForLua += _thinkResultHistory[i].evaluationSumUp;
	}
	*/


	// Lua関数をスタックに積む
	lua_getglobal ( _lua, "GetMoveEffectLevel" );

	// 引数をセット
	lua_pushstring ( _lua, EVALUATION_SUM_UP.c_str() );
	//lua_pushstring ( _lua, evalSumUpForLua.c_str() );

	// 関数呼び出し
	//if ( lua_pcall ( _lua, 2, 3, 0 ) != 0 ) {
	if ( lua_pcall ( _lua, 1, 3, 0 ) != 0 ) {

		printf("error : %s\n", lua_tostring ( _lua, -1) );
		lua_pop ( _lua, 1);
	}

	// 第１戻り値
	int effectLevel = lua_tointeger ( _lua, 1 );

	// 第２戻り値
	size_t length;
	const char* cstr = lua_tolstring (_lua, 2, &length);
	string cutInType = string( cstr, length );

	// 第３戻り値
	cstr = lua_tolstring( _lua, 3, &length);
	string cutInParameters = string( cstr, length );

	printf ("Lua からお返事。effectLevel : %d,  cutInName : %s,  cutInParameters : %s \n", effectLevel, cutInType.c_str(), cutInParameters.c_str() );

	// 戻り値を整理
	result.effectLevel = effectLevel;
	result.cutInType   = cutInType;
	result.cutInParameters = Utils_Convert::ReadStringAsMap ( cutInParameters );
	
	// 名札は本人名義で固定してしまう。
	//result.cutInParameters["nameLabel"] = _myName;
	
	// 改行を冒頭に追加したりは？ しない。直前のセリフと続きにしたいかもしれないので。
	

	// 戻り値をスタックから降ろす
	lua_pop ( _lua, -3 );
	lua_pop ( _lua, -2 );
	lua_pop ( _lua, -1 );

	return result;
}


//=========================================================
// スクリプトとの連携部分：Lua側へ、着手が行われた旨の通知をする
//=========================================================
void Actor::Script_SetNotificationInfo_MoveDone ( string participantId, AI_Modules::Move latestMove, AI_Modules::InnerExpression *latestState) {
	
	// Lua関数をスタックに積む
	lua_getglobal ( _lua, "SetNotificationInfo_MoveDone" );
	
	// 引数として渡せる形に加工：指し手
	string moveString = MoveToString ( latestMove );
	
	// 引数として渡せる形に加工：局面を文字列化
	string latestStateString = InnerExpressionToString ( latestState );
	
	// 引数をセット
	lua_pushstring  ( _lua, participantId.c_str() );
	lua_pushstring  ( _lua, moveString.c_str() );
	lua_pushstring  ( _lua, latestStateString.c_str() );

	// 関数呼び出し（引数の数、戻り値の数、エラー関数？）
	if ( lua_pcall ( _lua, 3, 0, 0 ) != 0) {
		printf("error  Actor::Script_SetNotificationInfo_MoveDone : %s\n", lua_tostring ( _lua, -1) );
		lua_pop ( _lua, 1);
	}
}

//=========================================================
// InnerExpression を Lua に渡せるように文字列化
//=========================================================
string Actor::InnerExpressionToString ( AI_Modules::InnerExpression *state ) {
	
	string stateString = "";
	stateString +=  "boardWidth="   + Utils_Convert::Int_to_String( state->_boardSizeX );
	stateString += "<>boardHeight="  + Utils_Convert::Int_to_String( state->_boardSizeY );
	stateString += "<>BOARD="        + SerializeInnerExpression_BOARD        ( state );
	stateString += "<>PIECE_BOX_P1=" + SerializeInnerExpression_PIECE_BOX_P1 ( state );
	stateString += "<>PIECE_BOX_P2=" + SerializeInnerExpression_PIECE_BOX_P2 ( state );
	stateString += "<>CARD_BOX_P1="  + SerializeInnerExpression_CARD_BOX_P1  ( state );
	stateString += "<>CARD_BOX_P2="  + SerializeInnerExpression_CARD_BOX_P2  ( state );	
	
	return stateString;
}

//=========================================================
// Move を Lua に渡せるように文字列化
//=========================================================
string Actor::MoveToString ( AI_Modules::Move moveData ) {

	string moveString = "";
	if ( moveData.action == AI_Modules::MOVE::ACTION::MOVE_PIECE ) {
		
		moveString = "Action=MOVE_PIECE";
		moveString += ",movePieceFromX=" + Utils_Convert::Int_to_String ( moveData.movePieceFromX );
		moveString += ",movePieceFromY=" + Utils_Convert::Int_to_String ( moveData.movePieceFromY );
		moveString += ",movePieceToX="   + Utils_Convert::Int_to_String ( moveData.movePieceToX   );
		moveString += ",movePieceToY="   + Utils_Convert::Int_to_String ( moveData.movePieceToY   );
		
		if (moveData.movePieceAndIsCllide) {
			moveString += ",isCollide=true";
		} else {
			moveString += ",isCollide=false";
		}
	}
	else if ( moveData.action == AI_Modules::MOVE::ACTION::PUT_PIECE ) {

		moveString = "Action=PUT_PIECE";
		moveString += ",putPieceX=" + Utils_Convert::Int_to_String ( moveData.putPieceX );
		moveString += ",putPieceY=" + Utils_Convert::Int_to_String ( moveData.putPieceY );
	}
	else if ( moveData.action == AI_Modules::MOVE::ACTION::PUT_CARD ) {

		Card::Property cardProperty = CardFactory::GetCardProperty ( moveData.putCardId );

		moveString = "Action=PUT_CARD";
		moveString += ",putCardId="        + Utils_Convert::Int_to_String ( moveData.putCardId        );
		moveString += ",putCardX="         + Utils_Convert::Int_to_String ( moveData.putCardX         );
		moveString += ",putCardY="         + Utils_Convert::Int_to_String ( moveData.putCardY         );
		moveString += ",putCardDirection=" + Utils_Convert::Int_to_String ( moveData.putCardDirection );
		moveString += ",putCardName="      + cardProperty.cardName;
	}
	else if ( moveData.action == AI_Modules::MOVE::ACTION::TAKE_CARD ) {
		
		// カードを取る指し手の場合、指し手データmove自体には、カードの情報は入れてない模様。
		// 盤上の座標と「取る」という動作情報のみ。
		// つまり takeCardCardId が入っておらず、カード名が取得できない
		Card::Property cardProperty = CardFactory::GetCardProperty ( moveData.takeCardCardId );
		
		
		// 今試合中のGamaManagerさんへの参照を取得して、
		
		
		
		// 指定座標のカードを問い合わせる
		
		
		// ↑ということをここでするのは話がヤヤコシクなるので、ヤメにして、
		// どうせGameManager側から呼ばれるだけなので、必要な情報を埋めた上で呼んでもらうことにする

		moveString = "Action=TAKE_CARD";
		moveString += ",takeCardCardId=" + Utils_Convert::Int_to_String ( moveData.takeCardCardId );
		moveString += ",takeCardX="      + Utils_Convert::Int_to_String ( moveData.takeCardX      );
		moveString += ",takeCardY="      + Utils_Convert::Int_to_String ( moveData.takeCardY      );
		moveString += ",takeCardName="   + cardProperty.cardName;
	
	}
	
	return moveString;
}

//=========================================================
// スクリプトとの連携部分：局面を評価
//=========================================================
int Actor::Script_EvaluateState ( AI_Modules::InnerExpression *state ) {

	int result = 0;

	// Lua関数をスタックに積む
	lua_getglobal ( _lua, "EvaluateState" );

	// 引数をセット：局目を文字列化した値
	string playingSide =  ((_isReverse) ? "P2" : "P1"); // 反転してるならP2に決まってますよね。
	int boardWidth      = state->_boardSizeX;
	int boardHeight     = state->_boardSizeY;
	string BOARD        = SerializeInnerExpression_BOARD        ( state );
	string PIECE_BOX_P1 = SerializeInnerExpression_PIECE_BOX_P1 ( state );
	string PIECE_BOX_P2 = SerializeInnerExpression_PIECE_BOX_P2 ( state );
	string CARD_BOX_P1  = SerializeInnerExpression_CARD_BOX_P1  ( state );
	string CARD_BOX_P2  = SerializeInnerExpression_CARD_BOX_P2  ( state );

	
	//printf ( "playingSide：%s\n", playingSide.c_str() );
	//printf ( "boardWidth：%d\n", boardWidth );
	//printf ( "boardHeight：%d\n", boardHeight );
	// //printf ( "BOARD：%s\n", BOARD.c_str() );
	//printf ( "PIECE_BOX_P1：%s\n", PIECE_BOX_P1.c_str() );
	//printf ( "PIECE_BOX_P2：%s\n", PIECE_BOX_P2.c_str() );
	//printf ( "CARD_BOX_P1：%s\n", CARD_BOX_P1.c_str() );
	//printf ( "CARD_BOX_P2：%s\n", CARD_BOX_P2.c_str() );

	lua_pushstring  ( _lua, playingSide.c_str());
	lua_pushinteger ( _lua, boardWidth );
	lua_pushinteger ( _lua, boardHeight );
	lua_pushstring  ( _lua, BOARD.c_str() );
	lua_pushstring  ( _lua, PIECE_BOX_P1.c_str() );
	lua_pushstring  ( _lua, PIECE_BOX_P2.c_str() );
	lua_pushstring  ( _lua, CARD_BOX_P1.c_str() );
	lua_pushstring  ( _lua, CARD_BOX_P2.c_str() );

	// 関数呼び出し
	if ( lua_pcall ( _lua, 8, 1, 0 ) != 0 ) {

		printf("error  Actor::Script_EvaluateState  : %s\n", lua_tostring ( _lua, -1) );
		lua_pop ( _lua, 1);
	}

	// 第１戻り値
	result = lua_tointeger ( _lua, 1 );

	// 戻り値をスタックから降ろす
	lua_pop ( _lua, -1 );

	return result;
}

// 感情値変動はLua側で自主的にしてもらう。C++側の担当するものではない
/*
//=========================================================
// スクリプトとの連携部分：感情値変動
//=========================================================
void Actor::Script_EmotionProceed ( vector<int> stateEvaluationHistory  ) {

	// 評価値の履歴を Lua に渡せるように文字列化する
	string evalHistory = Utils_Convert::vectorInt_to_string ( stateEvaluationHistory );

	// Lua関数をスタックに積む
	lua_getglobal ( _lua, "EmotionProceed" );

	// 引数をセット
	lua_pushstring ( _lua, evalHistory.c_str() );

	// 関数呼び出し
	if ( lua_pcall ( _lua, 1, 0, 0 ) != 0) {
		printf("error  Actor::Script_EmotionProceed : %s\n", lua_tostring ( _lua, -1) );
		lua_pop ( _lua, 1);
	}
}
*/


//=========================================================
// スクリプトとの連携部分：リアクション
//=========================================================
Actor::ReactionDataSet Actor::Script_React ( string contextInfo ) {

	Actor::ReactionDataSet result;

	// Lua関数をスタックに積む
	lua_getglobal ( _lua, "React" );

	// 引数をセット
	lua_pushstring ( _lua, contextInfo.c_str() );

	// 関数呼び出し
	if ( lua_pcall ( _lua, 1, 4, 0 ) != 0 ) {

		// エラーメッセージ
		string error_message = lua_tostring(_lua, -1);
		printf ( "Error：Script_React::lua_pcall: %s\n", error_message.c_str());
	}

	// 戻り値を回収
	result.remark                     = Utils_Lua::GetReturnValue_asString ( _lua, 1 );
	result.faceName                   = Utils_Lua::GetReturnValue_asString ( _lua, 2 );
	result.remarkWithAnotherNameLabel = Utils_Lua::GetReturnValue_asString ( _lua, 3 );
	result.kakeaiInfo                 = Utils_Lua::GetReturnValue_asString ( _lua, 4 );

	// 戻り値をスタックから降ろす
	lua_pop ( _lua, -4 );
	lua_pop ( _lua, -3 );
	lua_pop ( _lua, -2 );
	lua_pop ( _lua, -1 );

	return result;
}




//=========================================================
// 口出しモードでの「これでいい？」のリアクションを求める
//=========================================================
Actor::InterfereMode_AskConfirmationReactionData Actor::InterfereMode_AskConfirmationReaction ( AI_Modules::Move moveSuggestedByAI, string sumupEval ) {
	
	printf ( "口出しモードでの「これでいい？」のリアクションを求める by %s\n", _myName.c_str() );
	
	InterfereMode_AskConfirmationReactionData result;
	
	
	string remark   = ""; //"「プレイヤーに確認」";
	string faceName = "";
	
	///// ここから、Luaとのやりとり
	{
		// Lua関数をスタックに積む：口出しモードのアクターなら、このメソッドを用意しておいてね、ということ。
		lua_getglobal ( _lua, "ReactForInterferenceMode_AskForPlayerConfirmation" );
		
		string contextInfo = "";
		contextInfo += "MOVE"       + REACTION_CONTEXT_EQUALSIGN + MoveToString(moveSuggestedByAI) + REACTION_CONTEXT_DELIMITER;
		contextInfo += "SUMUP_EVAL" + REACTION_CONTEXT_EQUALSIGN + sumupEval;
		
		// 引数をセット
		lua_pushstring ( _lua, contextInfo.c_str() );

		// 関数呼び出し
		if ( lua_pcall ( _lua, 1, 4, 0 ) != 0 ) {

			// エラーメッセージ
			string error_message = lua_tostring(_lua, -1);
			printf ( "Error：Script_React::lua_pcall: %s\n", error_message.c_str());
		}

		// 戻り値を回収
		remark   = Utils_Lua::GetReturnValue_asString ( _lua, 1 );
		faceName = Utils_Lua::GetReturnValue_asString ( _lua, 2 );
		result.selectOptionLabel_agree = Utils_Lua::GetReturnValue_asString ( _lua, 3 ); // 同意の選択肢
		result.selectOptionLabel_no    = Utils_Lua::GetReturnValue_asString ( _lua, 4 ); // ダメの選択肢

		// 戻り値をスタックから降ろす
		lua_pop ( _lua, -4 );
		lua_pop ( _lua, -3 );
		lua_pop ( _lua, -2 );
		lua_pop ( _lua, -1 );
	}
	
	
	

	///// セリフを言う
	if ( remark != "" ) {

		_textMessageWindow->SetSpeakerName ( _myName );
		
		// 必ず新ページで。。完了済みとして。
		_textMessageWindow->AddText_asFinishedText ( "[p]" + remark );
	}	
	
	if ( faceName != "" && faceName != _currentFaceName  ) {
		
		if ( _faceImageFileName.find(faceName) != _faceImageFileName.end() ) {
			
			_currentFaceName = faceName;
			_faceImage->ChangeImage(_faceImageFileName[faceName], Image::CHANGE_IMAGE_EFFECT_FADE_IN);

		} else {
			printf ("顔画像IDなし（%s） at Actor::InterfereMode_AskConfirmationReaction\n", faceName.c_str());
		}
		
		
	}
	
	///// ↑ここまで、Actor側での処理
	///// ↓あとは呼出側（GameManager）での処理
	/*
		これでいい？ に同意か口出しかの選択肢ラベル
	*/
	
	return result;
}

//=========================================================
// 口出しモードでの「じゃあ○○くん教えて！」のリアクションを求める
//=========================================================
void Actor::InterfereMode_AskPlayersMoveReaction ( AI_Modules::Move moveSuggestedByAI, string sumupEval ) {
	
	string remark   = "「じゃあ○○くん教えて」";
	string faceName = "";


	
	///// ここから、Luaとのやりとり
	{
		// Lua関数をスタックに積む：口出しモードのアクターなら、このメソッドを用意しておいてね、ということ。
		lua_getglobal ( _lua, "ReactForInterferenceMode_AskForPlayerMove" );
		
		string contextInfo = "";
		contextInfo += "MOVE"       + REACTION_CONTEXT_EQUALSIGN + MoveToString(moveSuggestedByAI) + REACTION_CONTEXT_DELIMITER;
		contextInfo += "SUMUP_EVAL" + REACTION_CONTEXT_EQUALSIGN + sumupEval;
		
		// 引数をセット
		lua_pushstring ( _lua, contextInfo.c_str() );

		// 関数呼び出し
		if ( lua_pcall ( _lua, 1, 2, 0 ) != 0 ) {

			// エラーメッセージ
			string error_message = lua_tostring(_lua, -1);
			printf ( "Error：Script_React::lua_pcall: %s\n", error_message.c_str());
		}

		// 戻り値を回収
		remark   = Utils_Lua::GetReturnValue_asString ( _lua, 1 );
		faceName = Utils_Lua::GetReturnValue_asString ( _lua, 2 );

		// 戻り値をスタックから降ろす
		lua_pop ( _lua, -2 );
		lua_pop ( _lua, -1 );
	}



	///// セリフを言う
	if ( remark != "" ) {

		_textMessageWindow->SetSpeakerName ( _myName );
		
		// 必ず新ページで。完了済みとして。
		_textMessageWindow->AddText_asFinishedText ( "[p]" + remark );
	}	
	
	if ( faceName != "" && faceName != _currentFaceName  ) {
		
		if ( _faceImageFileName.find(faceName) != _faceImageFileName.end() ) {
			
			_currentFaceName = faceName;
			_faceImage->ChangeImage(_faceImageFileName[faceName], Image::CHANGE_IMAGE_EFFECT_FADE_IN);

		} else {
			printf ("顔画像IDなし（%s） at Actor::InterfereMode_AskPlayersMoveReaction\n", faceName.c_str());
		}
		
	}
	
	///// ↑ここまで、Actor側での処理
	///// ↓あとは呼出側（GameManager）での処理
	/*
		戻り値なしか。
	*/

}

//=========================================================
// 候補手からの選択モードでプレイヤーに選択肢を示すときのリアクション
//=========================================================
void Actor::ChoiceMode_AskPlayerDecisionReaction ( AI_Modules::Move moveSuggestedByAI, string sumupEval ) {

	
	string remark   = ""; //「どれがいいかな？」
	string faceName = "";

	
	///// ここから、Luaとのやりとり
	{
		// Lua関数をスタックに積む：口出しモードのアクターなら、このメソッドを用意しておいてね、ということ。
		lua_getglobal ( _lua, "ReactForChoiceMode_AskForPlayerDecision" );
		
		string contextInfo = "";
		contextInfo += "MOVE"       + REACTION_CONTEXT_EQUALSIGN + MoveToString(moveSuggestedByAI) + REACTION_CONTEXT_DELIMITER;
		contextInfo += "SUMUP_EVAL" + REACTION_CONTEXT_EQUALSIGN + sumupEval;
		
		// 引数をセット
		lua_pushstring ( _lua, contextInfo.c_str() );

		// 関数呼び出し
		if ( lua_pcall ( _lua, 1, 2, 0 ) != 0 ) {

			// エラーメッセージ
			string error_message = lua_tostring(_lua, -1);
			printf ( "Error：Script_React::lua_pcall: %s\n", error_message.c_str());
		}

		// 戻り値を回収
		remark   = Utils_Lua::GetReturnValue_asString ( _lua, 1 );
		faceName = Utils_Lua::GetReturnValue_asString ( _lua, 2 );

		// 戻り値をスタックから降ろす
		lua_pop ( _lua, -2 );
		lua_pop ( _lua, -1 );
	}



	///// セリフを言う
	if ( remark != "" ) {

		_textMessageWindow->SetSpeakerName ( _myName );
		
		// 必ず新ページで。完了済みとして。
		_textMessageWindow->AddText_asFinishedText ( "[p]" + remark );
	}	
	
	if ( faceName != "" && faceName != _currentFaceName  ) {
		
		if ( _faceImageFileName.find(faceName) != _faceImageFileName.end() ) {
			
			_currentFaceName = faceName;
			_faceImage->ChangeImage(_faceImageFileName[faceName], Image::CHANGE_IMAGE_EFFECT_FADE_IN);

		} else {
			printf ("顔画像IDなし（%s） at Actor::InterfereMode_AskPlayersMoveReaction\n", faceName.c_str());
		}
		
	}
	
}
	
	
//=========================================================
// 指し手の文字列表現を取得（選択モード時の選択肢のラベル用）
//=========================================================
string Actor::GetMoveString ( AI_Modules::Move mv ) {
	
	string result = "";
	
	///// ここから、Luaとのやりとり
	{
		// Lua関数をスタックに積む：口出しモードのアクターなら、このメソッドを用意しておいてね、ということ。
		lua_getglobal ( _lua, "MoveToString" );
		
		string contextInfo = "";
		contextInfo = MoveToString(mv);
		
		// 引数をセット
		lua_pushstring ( _lua, contextInfo.c_str() );

		// 関数呼び出し
		if ( lua_pcall ( _lua, 1, 1, 0 ) != 0 ) {

			// エラーメッセージ
			string error_message = lua_tostring(_lua, -1);
			printf ( "Error：Script_React::lua_pcall: %s\n", error_message.c_str());
		}

		// 戻り値を回収
		result   = Utils_Lua::GetReturnValue_asString ( _lua, 1 );

		// 戻り値をスタックから降ろす
		lua_pop ( _lua, -1 );
	}
	
	return result;
}


//=========================================================
// InnerExpression を Lua に渡せるように文字列化：盤面
//=========================================================
string Actor::SerializeInnerExpression_BOARD ( AI_Modules::InnerExpression *state ) {

	string result = "";

	for ( int y = 0; y < state->_boardSizeY; ++y ) {
		for ( int x = 0; x < state->_boardSizeX; ++x ) {
			
			if ( result != "" ) { result += "&"; }
			result += SerializeInnerExpression_GRID ( state, x, y );
		}
	}

	return result;
}
//=========================================================
// InnerExpression を Lua に渡せるように文字列化：グリッド
//=========================================================
string Actor::SerializeInnerExpression_GRID ( AI_Modules::InnerExpression *state, int x, int y ) {

	string result = "";

	///// 座標。Lua では 1始まりなので、+1 しておく。
	result +=  "X=" + Utils_Convert::Int_to_String ( x + 1 );
	result += ",Y=" + Utils_Convert::Int_to_String ( y + 1 );

	if      ( state->_gridList_Piece[y][x] == AI_Modules::GRID::BLACK) { result += ",PIECE=BLACK"; }
	else if ( state->_gridList_Piece[y][x] == AI_Modules::GRID::WHITE) { result += ",PIECE=WHITE"; }
	else if ( state->_gridList_Piece[y][x] == AI_Modules::GRID::ENPTY) { result += ",PIECE="; }

	if ( state->_gridList_Card[y][x].isEmpty()) {
		result += ",CARD=";
	} else {
		int cardID = state->_gridList_Card[y][x].cardId;
		result += ",CARD=" + CardFactory::GetCardProperty ( cardID ).cardName;
	}

	int panel = state->_gridList_ThrouableInfo[y][x];
	if      ( panel == AI_Modules::THROUGHABLE_INFO::NOT_THROUGHABLE ) { result += ",PANEL=";             }
	else if ( panel == AI_Modules::THROUGHABLE_INFO::THROUGHABLE     ) { result += ",PANEL=THROUGHABLE";  }
	else if ( panel == AI_Modules::THROUGHABLE_INFO::MUST_TO_UP      ) { result += ",PANEL=MUST_TO_UP";   }
	else if ( panel == AI_Modules::THROUGHABLE_INFO::MUST_TO_DOWN    ) { result += ",PANEL=MUST_TO_DOWN"; }
	else if ( panel == AI_Modules::THROUGHABLE_INFO::MUST_TO_RIGHT   ) { result += ",PANEL=MUST_TO_RIGHT";}
	else if ( panel == AI_Modules::THROUGHABLE_INFO::MUST_TO_LEFT    ) { result += ",PANEL=MUST_TO_LEFT"; }

	return result;
}
//=========================================================
// InnerExpression を Lua に渡せるように文字列化：駒台。P1
//=========================================================
string Actor::SerializeInnerExpression_PIECE_BOX_P1 ( AI_Modules::InnerExpression *state ) {

	string result = "";

	///// 駒の数、だけ
	result = Utils_Convert::Int_to_String ( state->_handPiece_BLACK );

	return result;
}
//=========================================================
// InnerExpression を Lua に渡せるように文字列化：駒台。P2
//=========================================================
string Actor::SerializeInnerExpression_PIECE_BOX_P2 ( AI_Modules::InnerExpression *state ) {

	string result = "";

	///// 駒の数、だけ
	result = Utils_Convert::Int_to_String ( state->_handPiece_WHITE );

	return result;
}
//=========================================================
// InnerExpression を Lua に渡せるように文字列化：カード置き場。P1
//=========================================================
string Actor::SerializeInnerExpression_CARD_BOX_P1 ( AI_Modules::InnerExpression *state ) {

	string result = "";

	///// カードの枚数、だけ
	result = Utils_Convert::Int_to_String ( state->_handCards_BLACK_Index );

	return result;
}
//=========================================================
// InnerExpression を Lua に渡せるように文字列化：カード置き場。P2
//=========================================================
string Actor::SerializeInnerExpression_CARD_BOX_P2 ( AI_Modules::InnerExpression *state ) {

	string result = "";

	///// カードの枚数、だけ
	result = Utils_Convert::Int_to_String ( state->_handCards_WHITE_Index );

	return result;
}


//=========================================================
// シリアライズ
//=========================================================
string Actor::Serialize () {
	
	string result = "";
	
	string delimiter = ",";
	
	// TODO あれ？ これだけ？ さしあたり。
	result += _scriptFileName + delimiter; 
	result += (_isActive ? "true" : "false") + delimiter;
	result += _myPlayingSide;
	
	return result;
}


//=========================================================
// デシリアライズ
//=========================================================
Actor* Actor::DeSerialize (string data, GUI_SFML::TextMessageWindow *textMessageWindow) {
	
	Actor *result = NULL;
	
	string delimiter = ",";
	
	vector<string> dataList = Utils_String::Split ( data, delimiter );
	
	if ( dataList.size() < 3 ) {
		printf ("セーブデータが壊れています Actor::DeSerialize ([%s])\n", data.c_str());
		return NULL;
	}
	
	// 今はこれだけ。
	string scriptFileName = dataList[0];
	bool   isActive       = Utils_String::isPositiveWord ( dataList[1] ) ;
	string myPlayingSide  = dataList[2];
	
	// TODO myName も必要になるってことですよね。
	
	result = new Actor ( scriptFileName, textMessageWindow );
	result->SetActive ( isActive );
	result->SetPlayingSide ( myPlayingSide );
	
	return result;
}


















