//=========================================================
//
// 「対戦相手」としてのActor
//
//=========================================================

#ifndef _ACTOR_
#define _ACTOR_

#include <./SFML/Graphics.hpp>
#include "../AI/AI.h"
#include "../AI/InnerExpression.h"

#include "../GUI_SFML/TextMessageWindow.h"

#include <vector>
#include <map>
#include <string>

#include <lua.hpp>
//#include <lua.h>

class Image;
class GameManager;

using namespace std;

class Actor {

public:

	//=========================================================
	// 定数；リアクションを求めるときのコンテキスト
	//=========================================================
	//static const string REACTION_CONTEXT_MOVED_BY_MESELF;   // 自分が着手した
	//static const string REACTION_CONTEXT_MOVED_BY_OPPONENT; // 相手が着手した
	
	//=========================================================
	// 定数；リアクションを問い合わせるためのコンテキスト情報用の区切り文字
	//=========================================================
	static const string REACTION_CONTEXT_EQUALSIGN; // 等号
	static const string REACTION_CONTEXT_DELIMITER; // デリミタ
	


	//=========================================================
	// 着手時のエフェクトを指定するデータのセット
	//=========================================================
	struct MoveEffectData {

		int                 effectLevel;
		string              cutInType;
		map<string, string> cutInParameters;

		MoveEffectData () {
			effectLevel = 0;
			cutInType   = "";
		}
	};
	
	//=========================================================
	// 口出しモード時の「これでいい？」の確認時の挙動（GameManager側に渡す情報）
	// セリフと表情はアクター側で処理する。
	//=========================================================
	struct InterfereMode_AskConfirmationReactionData {
		
		string selectOptionLabel_agree; // 「それでいい」の選択肢の文言
		string selectOptionLabel_no;    // 「ダメ（プレイヤー口出しする）」の選択肢の文言
	};
	
	
	//=========================================================
	// 掛け合いデータ
	//=========================================================
	struct KakeaiInfo {
		
		string participantId; // 参加者ID
		string faceName;      // 表情名
		string remark;        // セリフ
		string operation;     // その他の操作。強制終了とか。
		
		KakeaiInfo() {
			participantId = "";
			faceName      = "";
			remark        = "";
			operation     = "";
		}
		
	};


	//=========================================================
	// コンストラクタ
	//【引数】
	// スクリプトファイル名
	// セリフ表示用のテキスト欄
	//=========================================================
	Actor ( string scriptFileName, GUI_SFML::TextMessageWindow *textMessageWindow);
	~Actor();

	void Init   ();
	void Update ( sf::RenderWindow &window );
	void Draw   ( sf::RenderWindow &window );

	//=========================================================
	// 着手時のエフェクトを決定する
	//【引数】
	// 着手を思考した結果
	//=========================================================
	MoveEffectData DecideMoveEffect ( AI_Modules::ProgressData thinkResult );

	//=========================================================
	// 外部からの通知：着手が行われました、など
	//=========================================================
	void Notice_BattleStart (); // 試合開始
	void Notice_TurnStart   ( string participantId ); // 手番開始
	void Notice_MoveIsDone  ( string participantId, AI_Modules::Move latestMove, AI_Modules::InnerExpression *latestState);
	void Notice_MoveIsReady ( string participantId, AI_Modules::Move attemptingMove, string sumUpEval ); // 着手するところ
	void Notice_BattleEnd   ( string battleResult  ); // 試合終了するところ
	
	//=========================================================
	// リアクションをLua側に問い合わせて実行：public じゃなくてもいいのかもしれない
	//=========================================================
	void ReactionViaLua (  string contextInfo );
	
	
	

	//=========================================================
	// 口出しモードでの「これでいい？」のリアクションを求める
	//=========================================================
	InterfereMode_AskConfirmationReactionData InterfereMode_AskConfirmationReaction ( AI_Modules::Move moveSuggestedByAI, string sumupEval );
	
	//=========================================================
	// 口出しモードでの「じゃあ○○くん教えて！」のリアクションを求める
	//=========================================================
	void InterfereMode_AskPlayersMoveReaction ( AI_Modules::Move moveSuggestedByAI, string sumupEval );
	
	
	//=========================================================
	// 候補手からの選択モードでプレイヤーに選択肢を示すときのリアクション
	//=========================================================
	void ChoiceMode_AskPlayerDecisionReaction ( AI_Modules::Move moveSuggestedByAI, string sumupEval );
	
	//=========================================================
	// 位置を指定する
	//=========================================================
	void SetPosition ( sf::Vector2f pos );
	sf::Vector2f GetPosition ();
	sf::Vector2f GetImagePositionCurrent ();
	sf::Vector2f GetImagePositionGoal ();

	//=========================================================
	// 位置を移動する
	//=========================================================
	void MoveTo ( sf::Vector2f pos );

	//=========================================================
	// 反転設定
	//=========================================================
	void SetReverse ( bool arg ) { _isReverse = arg; }

	//=========================================================
	// P1側？ P2側？
	//=========================================================
	string GetPlayingSide();
	void   SetPlayingSide( string side ) { _myPlayingSide = side;}

	//=========================================================
	// 表示名設定
	//=========================================================
	string GetName() { return _myName; }
	void   SetName( string name ) { _myName = name;}
	
	

	//=========================================================
	// 先手(FIRST)？ 後手(SECOND)？
	//=========================================================
	//string GetPlayingTurn();

	void SetActive ( bool arg ) { _isActive = arg; }
	bool GetActive ()           { return _isActive; }
	
	
	string Serialize ();
	static Actor* DeSerialize ( string data, GUI_SFML::TextMessageWindow *textMessageWindow );
	
	
	//static int LuaTest( lua_State *lua ); // こっちを登録して
	//int LuaTest_Object( lua_State *lua ); // こっちを呼び出す
	
	//static Actor* LuaedObject;
	
	//=========================================================
	// 指し手の文字列表現を取得（選択モード時の選択肢のラベル用）
	//=========================================================
	string GetMoveString ( AI_Modules::Move mv );
	
	
	
	//=========================================================
	// 掛け合い用：セリフと表情を受け付ける
	//=========================================================
	void AcceptKakeai ( string remark, string faceName, string operation );
	
	
	GameManager *_gameManager; // 参加中のゲーム盤。デシリアライズのことまで考えるとオブジェクトの持ち回り経路が面倒なのでUpdateの直前に毎フレーム渡してもらうこととする。
	
	
	//=========================================================
	// 掛け合い存在確認：未実行の掛け合いが残っているかどうか。
	// 最後のセリフが「現在実行中」ならば、「残りはない」という結果になる。
	// そこについてはテキストウィンドウの文字が全表示済みかどうかの判定と合わせて判断のこと
	//=========================================================
	bool IsKakeaiAllFlushed();
	
private:

	bool _isActive; // 活性状態？


	//=========================================================
	// 画像セット
	//=========================================================
	struct ImageSet {
		sf::Image    image;
		sf::Texture  texture;
		sf::Sprite   sprite;
	};

	//=========================================================
	// リアクションデータセット
	//=========================================================
	struct ReactionDataSet {

		string remark;                     // 発言
		string faceName;                   // 顔画像名
		string remarkWithAnotherNameLabel; // 名札を指定して発言。空文字は「未指定」扱い。じゃあ書式は「名札文字列<>発言内容」で。	

		string kakeaiInfo;

		ReactionDataSet () {
			remark             = "";
			faceName           = "";
			remarkWithAnotherNameLabel = "";
			
			kakeaiInfo = "";
			
		}
		/*
		string remark             ; // セリフ       ：空文字はセリフなしを意味する
		int    remark_framecount; ; // セリフを表示するフレーム数（持続時間）
		string baseFaceName       ; // ベース表情名 ：空文字は変化なしを意味する
		string tempFaceName       ; // 一時表情名   ：空文字は変化なしを意味する
		int    tempFaceFrameCount ; // 一時表情の持続フレーム数

		ReactionDataSet () {
			remark       = "";
			remark_framecount = 0;
			baseFaceName = "";
			tempFaceName = "";
			tempFaceFrameCount = 0;
		}
		*/
	};
	
	///// 位置
	sf::Vector2f _position;

	///// Luaオブジェクト：具体的な処理内容はここに委託するよ。
	lua_State *_lua;

	///// Luaスクリプトファイルのファイル名：一応覚えておいてもいいんじゃない？
	string _scriptFileName;

	///// 画像リスト：ラベル・画像
	//map <string, ImageSet*> _faceImageList;
	
	// 画像オブジェクトと、ファイル名リスト
	map <string, string> _faceImageFileName;
	Image *_faceImage;
	
	/*
	string _faceName_Temporary;        // 今表示している表情
	string _faceName_Base;             // 現在のベース表情
	int    _framecount_ForCurrentFace; // 現在のベース表情を続けるフレーム数
	*/
	string _currentFaceName; // 現在表示している表情
	bool   _isReverse;

	///// セリフをいうためのテキスト欄：本体のコピー。カットインエフェクトのことを思い出すのです。
	GUI_SFML::TextMessageWindow *_textMessageWindow;

	///// この値がある間、セリフを表示する
	//int _framecount_TextMessage;

	///// （自分が）着手した結果の履歴。
	// 結局使ってないけども、今後の拡張用に、持っておいて悪いことはないかな。
	vector<AI_Modules::ProgressData> _thinkResultHistory; 

	///// 局面評価値の履歴（Luaで算出した分の）
	//vector<int> _stateEvaluationHistory;

	///// 自分は先手(FIRST)？ 後手(SECOND)？
	string _myPlayingTurn;
	
	///// 自分はP2側？ P1側？
	string _myPlayingSide;
	
	///// 表示名
	string _myName;
	
	
	//=========================================================
	// 掛け合い処理
	//=========================================================
	void Kakeai();
	vector<KakeaiInfo> _kakeaiInfoList;
	
	//=========================================================
	// リアクション実行
	//=========================================================
	void ReactionExecute ( ReactionDataSet reactionData );
	

	//=========================================================
	// 外部スクリプトとの連携部分：つまりLua関連
	//=========================================================
	void            Script_Init();
	void            Script_GetImageList ();
	MoveEffectData  Script_GetMoveEffectLevel ( string EVALUATION_SUM_UP );//, vector<AI_Modules::ProgressData> thinkResultHistory );
	int             Script_EvaluateState ( AI_Modules::InnerExpression *state );
	ReactionDataSet Script_React ( string contextInfo );
	void            Script_SetNotificationInfo_MoveDone ( string participantId, AI_Modules::Move latestMove, AI_Modules::InnerExpression *latestState);

	//void            Script_EmotionProceed ( vector<int> stateEvaluationHistory );

	//=========================================================
	// リアクションを問い合わせるためのコンテキスト情報を作成
	//=========================================================
	string MakeContextForReaction_BATTLE_START ();
	string MakeContextForReaction_TURN_START   ( string participantId );
	string MakeContextForReaction_BEFORE_MOVE  ( string participantId, AI_Modules::Move attemptingMove, string sumUpEval);
	string MakeContextForReaction_AFTER_MOVE   ( string participantId, AI_Modules::Move latestMove, AI_Modules::InnerExpression *latestState );
	string MakeContextForReaction_BATTLE_END   ( string battleResult );


	//=========================================================
	// InnerExpression を Lua に渡せるように文字列化
	//=========================================================
	string InnerExpressionToString ( AI_Modules::InnerExpression *state );
		string SerializeInnerExpression_BOARD        ( AI_Modules::InnerExpression *state );
		string SerializeInnerExpression_GRID         ( AI_Modules::InnerExpression *state, int x, int y );
		string SerializeInnerExpression_PIECE_BOX_P1 ( AI_Modules::InnerExpression *state );
		string SerializeInnerExpression_PIECE_BOX_P2 ( AI_Modules::InnerExpression *state );
		string SerializeInnerExpression_CARD_BOX_P1  ( AI_Modules::InnerExpression *state );
		string SerializeInnerExpression_CARD_BOX_P2  ( AI_Modules::InnerExpression *state );
	
	//=========================================================
	// Move を Lua に渡せるように文字列化
	//=========================================================
	string MoveToString ( AI_Modules::Move moveData );
	
	
	
	
	
};
#endif
