//=========================================================
//
// BGMクラス。SFML的な。
//
//=========================================================

#include "../Utils/Utils_Convert.h"
#include "BGM.h"

//=========================================================
// 定数：トランジションエフェクト
//=========================================================
const string BGM::CHANGE_BGM_EFFECT_NONE              = "";                  // エフェクトなし
const string BGM::CHANGE_BGM_EFFECT_FADE_OUT_IN       = "FADE_OUT_IN";       // 元のをフェードアウト してから 次のをフェードイン
const string BGM::CHANGE_BGM_EFFECT_FADE_OUT_IN_SIMUL = "FADE_OUT_IN_SIMUL"; // 元のをフェードアウト しながら 次のをフェードイン

//=========================================================
// コンストラクタ
//=========================================================
BGM::BGM() {

	///// ボインタなのですね。
	_sound_Current = new sf::Music();
	_sound_Next    = new sf::Music();

	///// BGMなのでループするのです？
	_loop = true;
	_sound_Current->setLoop(_loop);
	_sound_Next->setLoop(_loop);
	
	_currentPlayingLabel = "";
	
	_sound_Current_OpenResult = false;
	_sound_Next_OpenResult    = false;
	
	
	_volume = 100.0f;
	SetVolume (_volume);

	_pitch = 1.0f;
	SetPitch ( _pitch );
}

//=========================================================
// デストラクタ
//=========================================================
BGM::~BGM() {

	delete _sound_Current;
	delete _sound_Next;
}

//=========================================================
// ファイルを開く
//=========================================================
void BGM::OPEN_FROM_FILE ( sf::Music *music, string fileName, bool &openResult ) {

	// 空文字指定 は "演奏しない" の意。
	if ( fileName == "" ) {
		
		openResult = false;
		
		// 演奏中であるならば止めておく
		if ( music->getStatus() == sf::Music::Playing || music->getStatus() == sf::Music::Paused ) {
			music->stop();
		}
		
	} else {
	
		openResult = music->openFromFile ( fileName );
		
		if ( !openResult ) {
			
			printf ("[BGM::OPEN_FROM_FILE] ファイルを開けませんでした。[%s]\n", fileName.c_str() );
		}
	}
	
}

//=========================================================
// 演奏開始する
//=========================================================
void BGM::PLAY_MUSIC ( sf::Music *music, bool openResult ) {
	
	if ( openResult ) {
		
		music->play();
		
	}
}

	
//=========================================================
// 音量設定
//=========================================================
void BGM::SetVolume( float value ) {
	
	_volume = value;
	
	_sound_Current->setVolume ( _volume );
	//_sound_Next->setVolume    ( _volume );
}
float BGM::GetVolume () {
	return _volume;
}
//=========================================================
// ピッチ設定
//=========================================================
void BGM::SetPitch ( float value ) {

	_pitch = value;
	
	_sound_Current->setPitch ( _pitch );
	
}
float BGM::GetPitch () {
	return _pitch;
}


//=========================================================
// トランジションのためのボリューム増分値を取得
// 「音量設定」が演出としてのトランジションの時間に影響させないため
//=========================================================
float BGM::GetVolumeIncrementalValue_forTransition () {
	
	float result = 1.0f;
	
	result = _volume / 100.0f;
	
	return result;
}
	
//=========================================================
// ファイル名とラベルを指定して登録。
// ラベルがファイル名のエイリアスになる。
//=========================================================
void BGM::Regist ( string fileName, string label ) {

	///// 同じ名前のものがあったら警告でも出しておいてあげる
	if ( _registHistory.find ( label ) != _registHistory.end() ) {

		printf ( " ラベル [%s] のBGMが上書きされました。%s → %s \n", label.c_str(), _registHistory[label].c_str(), fileName.c_str() );
	}

	// ラベルと、ファイル名の対応を覚えておく。セーブ/ロード対象。
	_registHistory [ label ] = fileName;
}

//=========================================================
// BGM変更。
//=========================================================
void BGM::Change ( string label, string transitionEffect) {
	
	///// ここが呼ばれた時点で、前回のトランジションが終了してない場合は？
	/*
	if ( _transitionEffect != CHANGE_BGM_EFFECT_NONE ) {

		///// フェードアウト中の「現在のBGM」を止めて、
		_sound_Current->stop();
		delete _sound_Current;
		
		///// フェードイン中の次のBGMを、「現在のBGM」に格上げ。ボリュームそのままに。
		_sound_Current = _sound_Next;
		_sound_Next = new sf::Music();
		_sound_Next->setLoop(_loop);
		_sound_Next->setPitch(_pitch);
		
		printf("はい、このケースっす\n");
	}
	*/
	if ( _transitionEffect != CHANGE_BGM_EFFECT_NONE ) {
	
		// この時点で鳴っているうちボリュームが大きい方を「現在のBGM」ということにする
		if (_sound_Current->getVolume() > _sound_Next->getVolume() ) {
			
			// まだ次のBGMのフェードインが始まっていない、ということ（FADE_OUT_INの場合。SIMULも似たようなものであるはず）
			
			// このたび指定された「次のBGM」のために席をゆずる（SIMULの場合、_sound_Nextが既に鳴り始めている場合があるので、ポインタを手放す前に後始末をしておく。背後でいつまでもBGMが消えない現象への対策）
			_sound_Next->stop();
			delete _sound_Next;
			
			_sound_Next = new sf::Music();
			_sound_Next->setLoop(_loop);
			_sound_Next->setPitch(_pitch);
			_sound_Next->setVolume(0.0f);

			//printf("こっち？\n");

		} else {
			
			// すでに次のBGMのフェードインが始まっていた場合、ということ（FADE_OUT_INの場合。SIMULも似たようなものであるはず）
			
			// 「次のBGM」を「現在のBGM」ということにして、
			_sound_Current->stop();
			delete _sound_Current;
			_sound_Current = _sound_Next;
			
			// このたび指定された「新たな次のBGM」に備える。
			_sound_Next = new sf::Music();
			_sound_Next->setLoop(_loop);
			_sound_Next->setPitch(_pitch);
			_sound_Next->setVolume(0.0f);
			
			//printf("はい、このケースっす\n");
			
		}
		
	}
	
	_transitionEffect = transitionEffect;
	 
	if      ( _transitionEffect == BGM::CHANGE_BGM_EFFECT_NONE              ) { Transition_Preparation_NONE              (label); }
	else if ( _transitionEffect == BGM::CHANGE_BGM_EFFECT_FADE_OUT_IN       ) { Transition_Preparation_FADE_OUT_IN       (label); }
	else if ( _transitionEffect == BGM::CHANGE_BGM_EFFECT_FADE_OUT_IN_SIMUL ) { Transition_Preparation_FADE_OUT_IN_SIMUL (label); }

	// セーブ用。演奏中のラベルを覚えておく（トランジション中であっても、変更後、のラベルになる、ということですね）
	_currentPlayingLabel = label;
}

//=========================================================
// 毎フレーム呼んでもらう。
//=========================================================
void BGM::Update() {

	if      ( _transitionEffect == BGM::CHANGE_BGM_EFFECT_FADE_OUT_IN       ) { Transition_Processing_FADE_OUT_IN(); }
	else if ( _transitionEffect == BGM::CHANGE_BGM_EFFECT_FADE_OUT_IN_SIMUL ) { Transition_Processing_FADE_OUT_IN_SIMUL(); }

}



//=========================================================
// トランジションエフェクト準備：エフェクトなし
//=========================================================
void BGM::Transition_Preparation_NONE ( string label ) {
	
	///// 今、演奏中？
	bool isPlaying = (_sound_Current->getStatus() == sf::SoundSource::Playing);

	///// バッファを取得。
	//sf::SoundBuffer *buf = GetSoundBuffer ( label );

	///// ラベルに対応するファイル名を取得
	string fileName = ( _registHistory.find (label) != _registHistory.end() ) ? _registHistory[label] : "";

	///// いっぺん止めて。
	_sound_Current->stop();

	///// ストリーム先のファイルを切り替える
	if ( fileName != "" ) {
		
		OPEN_FROM_FILE ( _sound_Current, fileName, _sound_Current_OpenResult);
		
		PLAY_MUSIC ( _sound_Current, _sound_Current_OpenResult );
		
		_sound_Current->setVolume ( _volume ); 
		_sound_Current->setPitch(_pitch);

		/*
		if (_sound_Current->openFromFile ( fileName ))
		{

			_sound_Current->play();
			_sound_Current->setVolume ( _volume ); 
			
		} else {
			printf ( "BGMオープン失敗 [%s]\n", fileName.c_str());
		}
		*/
	} 
	else 
	///// 
	{
		printf ( "BGM：ファイル名取得できず Label:[%s]  ファイル名:[%s]\n", label.c_str(), fileName.c_str());
		
		///// TODO BGMで空ファイルを指定したときの segmentation fault error 関連
		///// これをやると高確率で落ちる、というサンプルとして残しておく
		// 元々鳴っていた状態のときに空ファイルを指定してここへ来る
		// 元々鳴っていない状態のときなら、これをやっても落ちない。原理は不明。
		//_sound_Current->openFromFile ( "" );
		//_sound_Current->play();

		// TODO 何かクリアするべき？
		// これでクリアになってくれる？
		//_sound_Current->openFromFile ( "" );
	}
	
	///// ロード時、もともと鳴っていた状態から鳴っていない状態になると、落ちる。以下をコメントアウトにて落ちなくなった。
	///// 再開。もともと、演奏中じゃなかったとしても？ うん。
	//_sound_Current->play();

	///// ボリュームは元に戻す、ということで問題ないですよね。
	//_sound_Current->setVolume ( _volume ); 
}
//=========================================================
// トランジションエフェクト準備：フェードアウトしてからイン
//=========================================================
void BGM::Transition_Preparation_FADE_OUT_IN ( string label ) {

	///// ラベルに対応するファイル名を取得
	string fileName = ( _registHistory.find (label) != _registHistory.end() ) ? _registHistory[label] : "";


	///// 取得したファイル名を、「次のBGM」に入れる。
	OPEN_FROM_FILE ( _sound_Next, fileName, _sound_Next_OpenResult);
	
	// 次のBGMのボリュームはゼロ。
	_sound_Next->setVolume(0.0f);
	
	// 次のBGMは、まだ鳴らさない。
	_sound_Next->stop();
	
	/*
	if ( fileName != "" ) {
			
		_sound_Next->openFromFile ( fileName );
		OPEN_FROM_FILE ( _sound_Next, fileName, _sound_Next_OpenResult);

		// 次のBGMのボリュームはゼロ。
		_sound_Next->setVolume(0.0f);

		// 次のBGMは、まだ鳴らさない。
		_sound_Next->stop();

	} else {
		_sound_Next->openFromFile ( "" );
	}
	*/
	
}
//=========================================================
// トランジションエフェクト準備：フェードアウトしながらイン
//=========================================================
void BGM::Transition_Preparation_FADE_OUT_IN_SIMUL ( string label ) {

	///// ラベルに対応するファイル名を取得
	string fileName = ( _registHistory.find (label) != _registHistory.end() ) ? _registHistory[label] : "";

	///// 取得したファイル名を、「次のBGM」に入れる。
	OPEN_FROM_FILE ( _sound_Next, fileName, _sound_Next_OpenResult);

	// 次のBGMのボリュームはゼロ。
	_sound_Next->setVolume(0.0f);
	
	// 次のBGMを、鳴らしておく。
	//_sound_Next->play();
	PLAY_MUSIC ( _sound_Next, _sound_Next_OpenResult );

	/*
	if ( fileName != "" ) {

		_sound_Next->openFromFile ( fileName );

		// 次のBGMのボリュームはゼロ。
		_sound_Next->setVolume(0.0f);

		// 次のBGMを、鳴らしておく。
		_sound_Next->play();

	} else {
		_sound_Next->openFromFile ( "" );
	}
	*/
}

//=========================================================
// トランジションエフェクト実行中：エフェクトなし
//=========================================================
void BGM::Transition_Processing_NONE () {

	// ここには来ませんよね。
}
//=========================================================
// トランジションエフェクト実行中：フェードアウトしてからイン
//=========================================================
void BGM::Transition_Processing_FADE_OUT_IN () {

	///// 現在のBGMはすでにフェードアウト済み？
	if ( _sound_Current->getVolume() <= 0.0f ) {

		///// 次のBGMは既にフェードイン済み？
		if ( _sound_Next->getVolume() >= _volume ) {

			///// 次を現に置き換えて、エフェクト終了
			delete _sound_Current;
			_sound_Current = _sound_Next;
			_sound_Next = new sf::Music();
			_sound_Next->setLoop(_loop);
			_sound_Next->setPitch(_pitch);


			_transitionEffect = CHANGE_BGM_EFFECT_NONE;
			
		///// まだ
		} else {

			// フェードイン
			float nextVol = _sound_Next->getVolume() + GetVolumeIncrementalValue_forTransition();
			nextVol = ( nextVol < _volume ) ? nextVol : _volume;
			_sound_Next->setVolume( nextVol );
		}

	///// まだ
	} else {

		// フェードアウト
		float nextVol = _sound_Current->getVolume() - GetVolumeIncrementalValue_forTransition();
		nextVol = ( nextVol > 0.0f ) ? nextVol : 0.0f;
		_sound_Current->setVolume( nextVol );

		// フェードアウト終了したら、次のBGMを鳴らす
		if ( nextVol == 0.0f ) {
			
			//_sound_Next->play();
			PLAY_MUSIC (_sound_Next, _sound_Next_OpenResult);

			// 一応、現BGMを止めておく（すでに停まっていた場合と、そろえておく意図もある）
			_sound_Current->stop();

		}
		// というか、今、BGM鳴ってない？
		if ( _sound_Current->getStatus() == sf::SoundSource::Stopped ) {

			// じゃあ、フェードアウト終了したことにする。
			_sound_Current->setVolume( 0.0f );
			
			//_sound_Next->play();
			PLAY_MUSIC (_sound_Next, _sound_Next_OpenResult);
		}
	}

}
//=========================================================
// トランジションエフェクト実行中：フェードアウトしながらイン
//=========================================================
void BGM::Transition_Processing_FADE_OUT_IN_SIMUL () {

	///// フェードアウト/イン 終了してます？
	bool isEnd_FadeOut = false;
	bool isEnd_FadeIn  = false;

	///// 現在のBGMはすでにフェードアウト済み？
	if ( _sound_Current->getVolume() <= 0 ) {

		isEnd_FadeOut = true;

	///// まだ
	} else {

		// フェードアウト
		float nextVol = _sound_Current->getVolume() - GetVolumeIncrementalValue_forTransition();
		nextVol = ( nextVol > 0.0f ) ? nextVol : 0.0f;
		_sound_Current->setVolume( nextVol );
	}

	///// 次のBGMは既にフェードイン済み？
	if ( _sound_Next->getVolume() >= _volume ) {

		isEnd_FadeIn = true;

	///// まだ
	} else {

		// フェードイン
		float nextVol = _sound_Next->getVolume() + GetVolumeIncrementalValue_forTransition();
		nextVol = ( nextVol < _volume ) ? nextVol : _volume;
		_sound_Next->setVolume( nextVol );
	}

	///// 両方とも終了してる？
	if ( isEnd_FadeOut && isEnd_FadeIn ) {

		///// 次を現に置き換えて、エフェクト終了
		delete _sound_Current;
		_sound_Current = _sound_Next;
		_sound_Next = new sf::Music();
		_sound_Next->setLoop(_loop);
		_sound_Next->setPitch(_pitch);


		_transitionEffect = CHANGE_BGM_EFFECT_NONE;
	}
}

//=========================================================
// ループ設定
//=========================================================
void BGM::SetLoop (bool loop) {
	
	_loop = loop;
	_sound_Current->setLoop(_loop);
	_sound_Next->setLoop(_loop);
}
//=========================================================
// セーブデータ作成
//=========================================================
BGM::SaveData BGM::GetSaveData () {

	BGM::SaveData result;

	result.currentPlayingLabel = _currentPlayingLabel;
	result.pitch               = std::to_string(_pitch);
	result.loop                = _loop;
	//result.registHistory       = _registHistory;

	return result;
}
//=========================================================
// セーブデータを元に再現
//=========================================================
void BGM::Load ( SaveData &saveData ) {

	_currentPlayingLabel = saveData.currentPlayingLabel;
	
	if ( saveData.pitch != "") {
		_pitch = std::stof( saveData.pitch );
	} else {
		_pitch = 1.0f;
	}
	SetLoop (saveData.loop);
	
	//_registHistory       = saveData.registHistory;

	// 演奏する。
	Change ( _currentPlayingLabel );
	
	SetPitch(_pitch);

}
