//=========================================================
//
// カメラクラス
// 
//=========================================================

#include "Application.h"
#include "Camera.h"

#include "../Utils/Utils_Cal.h"

string Camera::CURRENT_ACTION_NONE   = "none";
string Camera::CURRENT_ACTION_MOVETO = "moveto";

	
//=========================================================
// コンストラクタ
//=========================================================
Camera::Camera() {
	
	// この時点で Application クラスの側で view が準備できていること
	_view = Application::GetInstance()->_window.getDefaultView();
	
	_currentAction = Camera::Camera::CURRENT_ACTION_NONE;
	
	// これの初期値はダミーということなのですね。
	_goalX = 0;
	_goalY = 0;
	_speed = 0;
	
	_verticalReverseStatus = "";
	_IsVerticalRotate = false;

}

//=========================================================
// デストラクタ
//=========================================================
Camera::~Camera() {
	
}
	
//=========================================================
// 更新
//=========================================================
void Camera::Update() {
	
	// 現時点のアクションの処理を行う
	if ( _currentAction == Camera::CURRENT_ACTION_MOVETO ) {
		Action_MoveTo();
	}
	
	
	Action_VerticalStatus();
	
}


//=========================================================
// アクション：上下反転してくれ/しないでいてくれ
//=========================================================
void Camera::VerticalReverse ( bool arg ) {
	
	_IsVerticalRotate = arg;
}

//=========================================================
// アクション：上下反転
//=========================================================
void Camera::Action_VerticalStatus() {
	
	sf::View currentView = Application::GetInstance()->_window.getView();
	
	int currentRotate = (int)currentView.getRotation();
	
	///// 上下反転してくれ、と言われている状態であるとき
	if ( _IsVerticalRotate ) {

		if ( currentRotate == 180) {

			// 反転していますよ

		} else {
		
			int distance = Utils_Cal::Abs ( currentRotate, 180 );
			int increment = distance / 5;
			if ( increment <= 0 ) { increment = 1;}
			
			int nextRotate = Utils_Cal::Nearlize ( currentRotate, 180, increment);
			
			currentView.setRotation ( nextRotate );
			
			Application::GetInstance()->_window.setView(currentView);
			
		}
		
		
		
	///// 上下反転していないでくれ、という状態であるとき
	} else {

		if ( currentRotate == 0) {

			// 反転していますよ

		} else {
			
			int goal = 0;
			if ( currentRotate >= 180 ) { goal = 360;}
		
			int distance = Utils_Cal::Abs ( currentRotate, goal );
			int increment = distance / 5;
			if ( increment <= 0 ) { increment = 1;}
			
			int nextRotate = Utils_Cal::Nearlize ( currentRotate, goal, increment);
			
			if ( nextRotate == 360 ) { nextRotate = 0; }
			
			currentView.setRotation ( nextRotate );
			
			Application::GetInstance()->_window.setView(currentView);
			
		}		
		
	}
	
	
	
	
}


//=========================================================
// アクション：移動
//=========================================================
void Camera::Action_MoveTo() {
	
	
	sf::Vector2f currentPos = _view.getCenter();
	
	pair<float, float> resultPos = Utils_Cal::MoveTo( currentPos.x, currentPos.y, _goalX, _goalY, _speed );
	
	float distance = Utils_Cal::GetDistance ( currentPos.x, currentPos.y,  resultPos.first, resultPos.second);
	
	// 終了判定ですよ
	if ( distance < 1.0f ) {
		
		resultPos.first  = _goalX;
		resultPos.second = _goalY;
		
		_currentAction = Camera::CURRENT_ACTION_NONE;
	}
	
	_view.setCenter ( resultPos.first, resultPos.second );
	Application::GetInstance()->_window.setView ( _view );
	
}

//=========================================================
// このオブジェクトの設定値を実際の画面に適用する
//=========================================================
void Camera::ApplyThisCameraToTheActualScreen() {
	
	/*
		画面揺れエフェクトを実行した後に呼ぶといいのかもしれない
	*/
	
	Application::GetInstance()->_window.setView ( _view );
	
}

//=========================================================
// リセット
//=========================================================
void Camera::Reset() {
	
	_view = Application::GetInstance()->_window.getDefaultView();
	
	Application::GetInstance()->_window.setView ( _view );
	
	
	// 一応
	_currentAction = Camera::Camera::CURRENT_ACTION_NONE;
	_goalX = 0;
	_goalY = 0;
	_speed = 0;
	
}

//=========================================================
// 指定先に移動する
//=========================================================
void Camera::MoveTo ( int x, int y, int speed ) {
	
	_goalX = x;
	_goalY = y;
	_speed = speed;
	_currentAction = Camera::Camera::CURRENT_ACTION_MOVETO;
	

	_view = Application::GetInstance()->_window.getView();

}

//=========================================================
// 現在位置からの相対位置に移動する
//=========================================================
void Camera::Move ( int x, int y) {

	_view = Application::GetInstance()->_window.getView();
	
	_view.move ( x, y );
	
	Application::GetInstance()->_window.setView ( _view );	
}

//=========================================================
// 座標を指定
//=========================================================
void Camera::SetPosition ( int x, int y ) {

	_view = Application::GetInstance()->_window.getView();
	
	_view.setCenter ( x, y );
	
	Application::GetInstance()->_window.setView ( _view );	
}

//=========================================================
// アクションが終了しているか
//=========================================================
bool Camera::IsActionEnd() {
	
	return ( _currentAction == Camera::CURRENT_ACTION_NONE );
	
}

//=========================================================
// セーブデータ作成
//=========================================================
Camera::SaveData Camera::GetSaveData () {

	Camera::SaveData result;
		
	// 動いている最中のとき
	if (_currentAction == Camera::CURRENT_ACTION_MOVETO ) {
		
		result.centerX = _goalX;
		result.centerY = _goalY;

	// アクションなしのとき
	} else {
		
		result.centerX = _view.getCenter().x;
		result.centerY = _view.getCenter().y;
	}
	
	
	
	//result.registHistory       = _registHistory;

	return result;
}
//=========================================================
// セーブデータを元に再現
//=========================================================
void Camera::Load ( SaveData &saveData ) {
	
	// つまりこういうことか
	SetPosition ( saveData.centerX, saveData.centerY );
	
	_currentAction = Camera::CURRENT_ACTION_NONE;
	
	
}









