//=========================================================
//
// クリックできる画像
//
//=========================================================

#include "ClickableImage.h"
#include "Application.h"
#include "Define.h"

//=========================================================
// コンストラクタ
//=========================================================
ClickableImage::ClickableImage ( Property prop ) {

	_prop = prop;

	Init();
}

//=========================================================
// デストラクタ
//=========================================================
ClickableImage::~ClickableImage () {

}

//=========================================================
// 初期設定
//=========================================================
void ClickableImage::Init() {

	_texture.loadFromFile ( _prop.imageFile );

	_sprite = sf::RectangleShape ( _prop.size );

	_rect_Normal = _prop.rect_Normal;
	_rect_Over   = _prop.rect_Over;
	_rect_Push   = _prop.rect_Push;

	_position = sf::Vector2f ( 0, 0 );

	_sprite.setTexture ( &_texture );


	_currentState = STATE_NORMAL;
	_isClick = false;
}
	
//=========================================================
// セーブ用のデータを作成
//=========================================================
ClickableImage::SaveData ClickableImage::GetSaveData () {
	
	ClickableImage::SaveData result;
	
	result.prop     = _prop;
	result.position = _position;
	
	return result;
}

//=========================================================
// ロード
//=========================================================
void ClickableImage::Load ( ClickableImage::SaveData &saveData ) {
	
	_prop = saveData.prop;
	Init();
	_position = saveData.position;
}
	
	
//=========================================================
// 更新
//=========================================================
void ClickableImage::Update (sf::RenderWindow &window) {

	///// 状態：通常
	if ( _currentState == STATE_NORMAL ) { 

		// マウスが乗っていたら → 状態：マウスオーバー
		if ( IsMouseOver() ) {
			_currentState = STATE_OVER;
		}

		_isClick = false;

	///// 状態：マウスオーバー	
	} else if ( _currentState == STATE_OVER   ) { 
		
		// マウスが乗っていない → 状態：通常
		if ( !IsMouseOver() ) {
			_currentState = STATE_NORMAL;
		}

		// マウスを押している → 状態：プッシュ
		if ( Application::GetInstance()->_mouseState_Left == MOUSE::BUTTON::STATE::HAVE_PRESSED ) {
			_currentState = STATE_PUSH;
		}

		_isClick = false;
	
	///// 状態：プッシュ
	} else if ( _currentState == STATE_PUSH   ) { 

		// マウスが乗っていない → 状態：通常
		if ( !IsMouseOver() ) {
			_currentState = STATE_NORMAL;
		}

		// マウスを押していない → 状態：マウスオーバー
		if ( Application::GetInstance()->_mouseState_Left != MOUSE::BUTTON::STATE::PRESS ) {
			_currentState = STATE_OVER;

			// つまりマウスを放した
			_isClick = true;
		}
	}


	sf::IntRect currentRect;
	if      ( _currentState == STATE_NORMAL ) { currentRect = _rect_Normal; }
	else if ( _currentState == STATE_OVER   ) { currentRect = _rect_Over  ; }
	else if ( _currentState == STATE_PUSH   ) { currentRect = _rect_Push  ; }

	_sprite.setTextureRect (currentRect);

	_sprite.setPosition (_position);
}

//=========================================================
// マウスが乗っている？
//=========================================================
bool ClickableImage::IsMouseOver() {

	bool result = false;

	int mouseX = Application::GetInstance()->_mouseX;
	int mouseY = Application::GetInstance()->_mouseY;
	
	if
	(
		_position.x <= mouseX && mouseX <= _position.x + _sprite.getGlobalBounds().width 
		&&
		_position.y <= mouseY && mouseY <= _position.y + _sprite.getGlobalBounds().height 
	) 
	{
		result = true;
	}


	return result;
}


//=========================================================
// 描画
//=========================================================
void ClickableImage::Draw   (sf::RenderWindow &window) {

	window.draw ( _sprite );
}



//=========================================================
// 画像を差し替え
//=========================================================
void ClickableImage::SetImageFile  ( string fileName ) {

	_texture.loadFromFile ( fileName );
	_sprite.setTexture ( &_texture );
}

//=========================================================
// サイズを差し替え
//=========================================================
void ClickableImage::SetSize ( sf::Vector2f size ) {

	_sprite.setSize ( size );
}


//=========================================================
// 矩形を差し替え：通常時
//=========================================================
void ClickableImage::SetRectNormal ( sf::IntRect rect ) {

	_rect_Normal = rect;
}

//=========================================================
// 矩形を差し替え：マウスオーバー
//=========================================================
void ClickableImage::SetRectOrver  ( sf::IntRect rect ) {

	_rect_Over = rect;
}

//=========================================================
// 矩形を差し替え：押下時
//=========================================================
void ClickableImage::SetRectPush   ( sf::IntRect rect ) {

	_rect_Push = rect;
}





