//=========================================================
//
// デリゲータ関連
//
//=========================================================

/*
	使い方メモ：
	
	デリゲータ基底クラスの中に、演算子オーバーロードを宣言して、
	デリゲータクラスの中に、対応する演算子を実装。

	あとは引数ごとに対応するものをそれぞれ用意。

*/

#ifndef _DELIGATE_
#define _DELIGATE_

#include <string>

//#include "struct.h"

#include "../AI/Move.h"

using namespace std;


//=========================================================
// デリゲータ基底クラス
//=========================================================
class DelegateBase {
public:
   DelegateBase(){};
   virtual ~DelegateBase(){};

   // T型を引数に持つ戻り値無しの関数のように呼べるようにせよ！
   virtual void operator()( int value ) = 0;
   virtual void operator()( int x, int y ) = 0;
   virtual void operator()( float value ) = 0;
   virtual void operator()() = 0;
   virtual void operator()(string value ) = 0;
   virtual void operator()( AI_Modules::Move move ) = 0;
}; 


//=========================================================
// デリゲータクラス
//=========================================================
template < class T >
class Delegate : public DelegateBase {

public:
	Delegate(){}
	virtual ~Delegate(){}

	
	//=========================================================
	// オペレータ実装
	//=========================================================
	virtual void operator()()                     { (m_obj->*m_func_void)(); }
	virtual void operator()( int value )          { (m_obj->*m_func_int)( value );}
	virtual void operator()( int x, int y )       { (m_obj->*m_func_int_int)( x, y );}
	virtual void operator()( float value )        { (m_obj->*m_func_float)( value );}
	virtual void operator()( string value )       { (m_obj->*m_func_string)( value );}
	virtual void operator()( AI_Modules::Move value ) { (m_obj->*m_func_move)( value );}

	typedef void (T::*EventFunc_void    )( void     );
	typedef void (T::*EventFunc_int     )( int      );
	typedef void (T::*EventFunc_int_int )( int, int );
	typedef void (T::*EventFunc_float   )( float    );
	typedef void (T::*EventFunc_string  )( string   );
	typedef void (T::*EventFunc_move    )( AI_Modules::Move );

	//=========================================================
	// オブジェクトとメソッドを登録
	//=========================================================
	void set( T* obj, EventFunc_void    func ) {m_obj = obj; m_func_void      = func;}
	void set( T* obj, EventFunc_int     func ) {m_obj = obj; m_func_int       = func;}
	void set( T* obj, EventFunc_int_int func ) {m_obj = obj; m_func_int_int   = func;}
	void set( T* obj, EventFunc_float   func ) {m_obj = obj; m_func_float     = func;}
	void set( T* obj, EventFunc_string  func ) {m_obj = obj; m_func_string    = func;}
	void set( T* obj, EventFunc_move    func ) {m_obj = obj; m_func_move      = func;}

	//=========================================================
	// デリゲータ生成関数
	//=========================================================
	///// 引数void
	static DelegateBase* createDelegator_void( T* obj, void (T::*func)( void ) ) {
		Delegate* dg = new Delegate();
		dg->set( obj, func );
		return dg;
	}
	///// 引数int
	static DelegateBase* createDelegator_int( T* obj, void (T::*func)( int ) ) {
		Delegate* dg = new Delegate;
		dg->set( obj, func );
		return dg;
	}
	///// 引数int,int
	static DelegateBase* createDelegator_int_int( T* obj, void (T::*func)( int, int ) ) {
		Delegate* dg = new Delegate;
		dg->set( obj, func );
		return dg;
	}
	///// 引数float
	static DelegateBase* createDelegator_float( T* obj, void (T::*func)( float ) ) {
		Delegate* dg = new Delegate;
		dg->set( obj, func );
		return dg;
	}
	///// 引数string
	static DelegateBase* createDelegator_string( T* obj, void (T::*func)( string ) ) {
		Delegate* dg = new Delegate;
		dg->set( obj, func );
		return dg;
	}
	///// 引数move
	static DelegateBase* createDelegator_move( T* obj, void (T::*func)( AI_Modules::Move ) ) {
		Delegate* dg = new Delegate;
		dg->set( obj, func );
		return dg;
	}

protected:

	///// オブジェクト
	T* m_obj;

	///// 関数ポインタ
	EventFunc_void        m_func_void;
	EventFunc_int         m_func_int;
	EventFunc_int_int     m_func_int_int;
	EventFunc_float       m_func_float;
	EventFunc_string      m_func_string;
	EventFunc_move        m_func_move;
}; 
#endif

