//=========================================================
//
// フリーバトルセレクター
//
//=========================================================

#ifndef _FREE_BATTLE_SELECTOR_
#define _FREE_BATTLE_SELECTOR_

#include <SFML/Graphics.hpp>
#include <string>
using namespace std;

#include "../GUI_SFML/Button.h"
#include "../GUI_SFML/ScrollBar.h"

class ScriptEngine;
class ClickableImage;
class Image;

class FreeBattleIcon {
	
public:
	
	struct FreeBattleIconProp {
		string id;
		string imageFile;
		string battleTitle;
		string variableForUnlock;
		string lockedIconImageFile;
		
		bool isSecret;
		int secretX;
		int secretY;
		int secretWidth;
		int secretHeight;
		
		FreeBattleIconProp () {
			id = "";
			imageFile = "";
			battleTitle = "";
			variableForUnlock = "";
			lockedIconImageFile = "";
			
			isSecret = false;
			secretX = 0;
			secretY = 0;
			secretWidth = 0;
			secretHeight = 0;
		}
	};
	
	FreeBattleIcon ( FreeBattleIconProp prop, ScriptEngine *scriptEngine, sf::Texture &lockedIconTexture );
	~FreeBattleIcon();
	
	void Update ( sf::RenderWindow &window );
	void Draw   ( sf::RenderWindow &window );
	
	void SetPosition ( int x, int y );
	
	bool IsSelected() { return _isSelected; }
	string GetId() { return _prop.id; }

	sf::Vector2f GetSize();
	
	void ClearSelectStatus();
	
private:
	
	bool _isEmpty; // 空欄フラグ

	FreeBattleIconProp _prop;
	
	sf::Texture _iconTexture;
	sf::Sprite _iconSprite;
	sf::Text   _battleTitleLabel;
	
	sf::Texture _lockedIconTexture;
	sf::Sprite _lockedIconSprite;
	
	sf::RectangleShape _back_Frame;
	sf::RectangleShape _cursor_Frame;
	
	bool _isUnlocked; // アンロックされているかどうか
	
	ScriptEngine *_scriptEngine;
	

	bool _isMouseover;
	bool _clickedHere;
	bool _isSelected;
	bool _isPressed;
	
	bool IsUnlocked();
	
};







class FreeBattleSelector {

public:

	FreeBattleSelector ( ScriptEngine *scriptEngine );
	~FreeBattleSelector();


	void Update ( sf::RenderWindow &window );
	void Draw   ( sf::RenderWindow &window );

	///// 表示/非表示
	bool IsActive() { return _isActive; }
	void SetActive ( bool value ) { _isActive = value; };
	
	
	///// 選択されているかどうか
	bool IsSelected();
	
	///// 選択されているもののIDを取得
	string GetSelectedId();
	
	void   SetVariableForResult ( string variableForResult ) { _variableForResult = variableForResult; }
	string GetVariableForResult () { return _variableForResult; }
	
	void AddIcon ( FreeBattleIcon::FreeBattleIconProp prop, int gridX, int gridY );
	
	///// 表の枠を設定：じゃあ、これをすると中身全部消えるってことで。
	void SetMatrix ( int hCount, int vCount, int displayVCount );
	
	///// 選択状況をクリア
	void ClearSelectStatus();
		
private:

	// 表示中？
	bool _isActive;
	
	sf::Vector2f       _position;     // 位置。デフォルトで画面の左上
	sf::RectangleShape _back_Frame;   // 下地。デフォルト用
	
	string _variableForResult; // 結果を入れる変数名
	
	int _displayVCount;
	
	GUI_SFML::ScrollBar *_scrollBar;
	
	ClickableImage *_closeButton;

	Image *_caption;

	
	bool _isSelected;
	string _selectedId;
	
	ScriptEngine *_scriptEngine;

	
	vector<vector<FreeBattleIcon*>> _itemList; // フリーバトルのリスト
	int _gridXCount;
	int _gridYCount;
	
	sf::Texture _lockedIconTexture;
	
	vector<FreeBattleIcon*> _itemList_secret; // フリーバトルのリスト。シークレット枠。
	
	void AllClear();
	FreeBattleIcon* GetIcon ( int x, int y);
};
#endif


