//=========================================================
//
// メニューでの設定項目の選択肢
//
//=========================================================

#include "Application.h"
#include "MenuItemOption.h"
#include "../Utils/Utils_Cal.h"

#include "../GUI_SFML/TextImageFactory.h"
#include "../GUI_SFML/SFML_Utils.h"

//=========================================================
// コンストラクタ
//=========================================================
MenuItemOption::MenuItemOption ( string label, string value ) {

	_value = value;
	_label = label;

	TextImageFactory::Create ( _imageSet.image, label, 20, sf::Color(0,0,0,255) );
	_imageSet.Update();
	_imageSet.sprite.setOrigin ( _imageSet.image.getSize().x / 2.0f, _imageSet.image.getSize().y / 2.0f );

	_frame = sf::RectangleShape ( sf::Vector2f( _imageSet.image.getSize().x, _imageSet.image.getSize().y) );

	_frame_Selected = sf::RectangleShape ( sf::Vector2f( _imageSet.image.getSize().x + 2.0f, _imageSet.image.getSize().y + 2.0f) );
	_frame_Selected.setFillColor ( sf::Color(0,0,0,0) );
	_frame_Selected.setOutlineColor ( sf::Color(255,255,0,255) );
	_frame_Selected.setOutlineThickness ( 1.0f );
	_frame_Selected.setOrigin ( _frame.getSize() / 2.0f );

	_isMouseOver = false;
	_isSelected  = false;
}

//=========================================================
// デストラクタ
//=========================================================
MenuItemOption::~MenuItemOption() {

}

//=========================================================
// 更新
//=========================================================
void MenuItemOption::Update ( sf::RenderWindow &window ) {

	_frame.setPosition ( _position );
	//_imageSet.sprite.setPosition ( _position + _frame.getSize() / 2.0f );
	
	sf::Vector2f labelPos;
	labelPos.x = _position.x + _imageSet.sprite.getGlobalBounds().width  / 2.0f;
	labelPos.y = _position.y + _imageSet.sprite.getGlobalBounds().height / 2.0f;
	Utils_Cal::RoundWildly ( labelPos.x );
	Utils_Cal::RoundWildly ( labelPos.y );
	_imageSet.sprite.setPosition ( labelPos );
	
	//_frame_Selected.setPosition ( _imageSet.sprite.getPosition() );
	_frame_Selected.setPosition ( _frame.getPosition() );
	_frame_Selected.setOrigin(1.0f, 1.0f);

	ObserveMouseOver();

	if ( _isMouseOver ) {
		_frame.setFillColor ( sf::Color( 200,200,255,150 ) );
	} else {
		_frame.setFillColor ( sf::Color( 255,255,255,255 ) );
	}
}

//=========================================================
// 描画
//=========================================================
void MenuItemOption::Draw   ( sf::RenderWindow &window ) {

	window.draw ( _frame );
	window.draw ( _imageSet.sprite );

	if ( _isSelected ) {
		window.draw ( _frame_Selected );
	}
}


//=========================================================
// マウスが乗っているかどうかを監視
//=========================================================
void MenuItemOption::ObserveMouseOver() {

	int mouseX = Application::GetInstance()->_mouseX;
	int mouseY = Application::GetInstance()->_mouseY;

	sf::Vector2f size = _frame.getSize();

	_isMouseOver = false;
	if
	(
		_position.x < mouseX && mouseX < _position.x + size.x
		&&
		_position.y < mouseY && mouseY < _position.y + size.y
	)
	{
		_isMouseOver = true;

		//printf ("%s マウス乗ってます\n", _label.c_str());
	}

}








