//=========================================================
//
// セーブデータ。１件分を表す。
//
//=========================================================

#include "SaveData.h"
#include "Application.h"
#include "../GUI_SFML/TextImageFactory.h"
#include "../GUI_SFML/SFML_Utils.h"
#include "../Utils/Utils_String.h"
#include "../Utils/Utils_Convert.h"

//=========================================================
// 内部定数
//=========================================================
const int SaveData::MOUSE_STATE_NONE     = 0; // 押していない
const int SaveData::MOUSE_STATE_PUSH     = 1; // 押したところ
const int SaveData::MOUSE_STATE_PRESS    = 2; // 押している
const int SaveData::MOUSE_STATE_RELEASE  = 3; // 放したところ

//=========================================================
// 定数：データの型・セーブ対象としての
//=========================================================
const int SaveData::SAVE_DATA_TYPE_NONE   = 0;
const int SaveData::SAVE_DATA_TYPE_INT    = 1;
const int SaveData::SAVE_DATA_TYPE_STRING = 2;


//=========================================================
// コンストラクタ
// ファイル名を必ず受け取るのです。
//=========================================================
SaveData::SaveData( string fileName, int slotNum ) {

	_fileName = fileName;

	_date = "----年 -- 月 -- 日 --：--：--";

	_ss_Size.x = 120.0f;
	_ss_Size.y = 90.0f;

	// 一度も Load を呼んでない状態だから。
	_isValid = false;
	
	_chapterName = "";


	///// 最低限の見た目を整えます。
	{
		///// 枠というか下地というか
		_frame = sf::RectangleShape ( sf::Vector2f ( 530.0f, 95.0f ) );
		_frame.setFillColor         ( sf::Color( 100,100,100,200 ) );
		_frame.setOutlineColor      ( sf::Color( 0,0,0,255 ) );
		_frame.setOutlineThickness  ( 1.0f );

		///// マウスが乗ったときのカーソル。
		_cursor = sf::RectangleShape ( sf::Vector2f ( 530.0f, 95.0f ) );
		_cursor.setFillColor         ( sf::Color( 255,255,255,50 ) );
		_cursor.setOutlineColor      ( sf::Color( 255,255,0,255 ) );
		_cursor.setOutlineThickness  ( 2.0f );

		///// スロット番号
		char slotNumStr[64];
		sprintf ( slotNumStr, "[%02d]", slotNum );
		_slotNum = slotNum;
		TextImageFactory::Create ( _image_slotNumber.image, slotNumStr, 20, sf::Color(255,255,255,255));
		_image_slotNumber.Update();

		///// SS表示枠
		_SS_Frame = sf::RectangleShape ( _ss_Size );
		_SS_Frame.setFillColor         ( sf::Color( 200,200,200,200 ) );
		_SS_Frame.setOutlineColor      ( sf::Color( 150,150,255,255 ) );
		_SS_Frame.setOutlineThickness  ( 1.0f );

		///// No Image
		/*
		TextImageFactory::Create ( _image_SS_NO.image, "No Image", 20, sf::Color(255,255,255,255) );
		_image_SS_NO.Update();
		*/
		_image_SS_NO.texture.loadFromFile("./image/menu/noData.png");
		_image_SS_NO.sprite.setTexture(_image_SS_NO.texture);

		_image_SS_NO.sprite.setOrigin (
			_image_SS_NO.texture.getSize().x / 2.0f,
			_image_SS_NO.texture.getSize().y / 2.0f
		);

		///// SS
		UpdateSS();

	}


	_mouseState = MOUSE_STATE_NONE;

}

//=========================================================
// デストラクタ
//=========================================================
SaveData::~SaveData() {

}

//=========================================================
// 見た目をセットアップ（可変部分）
//=========================================================
void SaveData::SetupAppearance() {

	///// 日付表示
	{
		//sf::Vector2f pos = _image_Date.sprite.getPosition ();
		//TextImageFactory::Create ( _image_Date.image, _date, 18, sf::Color( 255,255,255,255 ) );
		//_image_Date.Update();
		//_image_Date.sprite.setPosition( pos );
		
		_dateTextLabel = TextImageFactory::Create ( _date, 18 );
		_dateTextLabel.setFillColor        ( sf::Color::White );
		//_dateTextLabel.setOutlineColor     ( sf::Color::Black );
		//_dateTextLabel.setOutlineThickness ( 10 );
		
		_chapterNameTextLabel = TextImageFactory::Create ( _chapterName, 18);
		_chapterNameTextLabel.setFillColor        ( sf::Color::White );
		//_chapterNameTextLabel.setOutlineColor     ( sf::Color::Black );
		//_chapterNameTextLabel.setOutlineThickness ( 10 );

	}

	///// SS
	{
		// ２回実行されません？
		//UpdateSS();

		// これは、SS更新後にも呼ばれてなんだか二度手間なので、別枠とする。
	}

}

//=========================================================
// チャプター名を設定
//=========================================================
void SaveData::SetChapterName ( string chapterName ) {
	
	_chapterName = chapterName;
	
}


//=========================================================
// 扱うファイルを変更する
//=========================================================
void SaveData::ReplaceFileName ( string fileName ) {

	_fileName = fileName;

	LoadAppearance();

}

//=========================================================
// SS 更新
//=========================================================
void SaveData::UpdateSS () {

	// ロード成功？
	if (_image_SS.image.loadFromFile ( GetSSFileName() )) {

		_image_SS.Update();

		// 多少、意味があるみたいだぞ？ 小さく表示したときの潰れ具合が、気持ち、軽減している。
		_image_SS.texture.setSmooth(true);

		float width  = _image_SS.image.getSize().x;
		float height = _image_SS.image.getSize().y;

		float regularWidth  = _ss_Size.x;
		float regularHeight = _ss_Size.y;

		_image_SS.sprite.setScale ( regularWidth / width, regularHeight / height  );

		_image_SS.isValid = true;

		// こんなところで位置を？
		_image_SS         .sprite.setPosition ( _position + sf::Vector2f( 50, 2 ) );

	// ロードできず？ ファイルがないとか。
	} else {

		_image_SS.isValid = false;

	}
}


//=========================================================
// SSファイル名
//=========================================================
string SaveData::GetSSFileName() {

	// SS のファイル名  拡張子を.png とする。
	string extention  = "png";
	string ssFileName = "";
	if ( _fileName.size() - extention.size() >= 0 ) {

		string fileItself = _fileName.substr ( 0, _fileName.size() - extention.size());

		ssFileName = fileItself + extention;
	}

	return ssFileName;
}

//=========================================================
// 読み込み
// 見た目部分だけ
//=========================================================
void SaveData::LoadAppearance() {

	///// 読み込みまして、
	map< string, SaveDataUnit > loadDataList;

	set<string> requiredKey;
	requiredKey.insert ( "year" );
	requiredKey.insert ( "month" );
	requiredKey.insert ( "day" );
	requiredKey.insert ( "hour" );
	requiredKey.insert ( "min" );
	requiredKey.insert ( "sec" );
	requiredKey.insert ( "chapterName" );

	ReadSaveData ( loadDataList, requiredKey );
	//ReadSaveData ( loadDataList );

	// ファイルがあれば、中身もそれなりであるはず、という思い込み、または思い切り。
	// 中身がない＝ファイル読み込みエラー、とみなす
	if ( loadDataList.size() <= 0 ) {

		_isValid = false;
		_date = "----年 -- 月 -- 日 --：--：--";
		_chapterName = "";

		return;

	} else {

		// いずれにせよ、ここ Load を呼ばないことには true にはなりえない。
		_isValid = true;

		char date[512];
		sprintf ( date, "%d年 %02d月 %02d日 %02d:%02d:%02d",
			loadDataList [ "year"  ].GetData_as_Int(),
			loadDataList [ "month" ].GetData_as_Int(),
			loadDataList [ "day"   ].GetData_as_Int(),
			loadDataList [ "hour"  ].GetData_as_Int(),
			loadDataList [ "min"   ].GetData_as_Int(),
			loadDataList [ "sec"   ].GetData_as_Int()
		);
		_date = date;

		_chapterName = loadDataList [ "chapterName" ].GetData_as_String();
	}

	///// 見た目を更新
	SetupAppearance();

	UpdateSS();
}


//=========================================================
// 読み込み
// ファイルを読み込んで内容を受け取る。
// もともと持っていた変数の値とかは、消される。
//=========================================================
void SaveData::Load () {

	///// 読み込みまして、
	map< string, SaveDataUnit > loadDataList;
	ReadSaveData ( loadDataList );

	// ファイルがあれば、中身もそれなりであるはず、という思い込み、または思い切り。
	// 中身がない＝ファイル読み込みエラー、とみなす
	if ( loadDataList.size() <= 0 ) {

		_isValid = false;
		_date = "----年 -- 月 -- 日 --：--：--";
		_chapterName = "";
		
		return;

	} else {

		// いずれにせよ、ここ Load を呼ばないことには true にはなりえない。
		_isValid = true;

		char date[512];
		sprintf ( date, "%d年 %02d月 %02d日 %02d:%02d:%02d",
			loadDataList [ "year"  ].GetData_as_Int(),
			loadDataList [ "month" ].GetData_as_Int(),
			loadDataList [ "day"   ].GetData_as_Int(),
			loadDataList [ "hour"  ].GetData_as_Int(),
			loadDataList [ "min"   ].GetData_as_Int(),
			loadDataList [ "sec"   ].GetData_as_Int()
		);
		_date = date;

		_chapterName = loadDataList [ "chapterName"].GetData_as_String();
	}


	/*
		loadDataList から、loadDataList [ "sec"   ].GetData_as_Int() のように、データを取り出して、
		_saveData_Select などの各オブジェクト用のセーブデータを再現。
	*/

	///// セーブ用のシリアライズデータ再現：画像リスト _imageList
	{

		///// イメージは何件だった？
		int    imadeListLength = loadDataList [ "imageListLength" ].GetData_as_Int();

		_saveData_ImageList.clear();

		for ( int i = 0; i < imadeListLength; ++i ) {

			Image::SaveData image;

			string index = Utils_Convert::Int_to_String(i);

			image.position_Goal.x       = loadDataList [ "image_position_Goal_X-"       + index ].GetData_as_Int();
			image.position_Goal.y       = loadDataList [ "image_position_Goal_Y-"       + index ].GetData_as_Int();
			image.position_Current.x    = loadDataList [ "image_position_Current_X-"    + index ].GetData_as_Int();
			image.position_Current.y    = loadDataList [ "image_position_Current_Y-"    + index ].GetData_as_Int();

			image.rotation              = loadDataList [ "image_rotation-"              + index ].GetData_as_Int();
			image.scale.x               = loadDataList [ "image_scale_X-"               + index ].GetData_as_Int();
			image.scale.y               = loadDataList [ "image_scale_Y-"               + index ].GetData_as_Int();

			image.label                 = loadDataList [ "image_label-"                 + index ].GetData_as_String();
			image.z                     = loadDataList [ "image_z-"                     + index ].GetData_as_Int();
			image.image_fileName        = loadDataList [ "image_image_fileName-"        + index ].GetData_as_String();
			image.maskColorString       = loadDataList [ "image_maskColorString-"       + index ].GetData_as_String();
			image.wipeOutImage_fileName = loadDataList [ "image_wipeOutImage_fileName-" + index ].GetData_as_String();
			image.wipeInImage_fileName  = loadDataList [ "image_wipeInImage_fileName-"  + index ].GetData_as_String();
			image.fadeOutSpeed          = loadDataList [ "image_fadeOutSpeed-"          + index ].GetData_as_Int();
			image.fadeInSpeed           = loadDataList [ "image_fadeInSpeed-"           + index ].GetData_as_Int();
			image.wipeOutSpeed          = loadDataList [ "image_wipeOutSpeed-"          + index ].GetData_as_Int();
			image.wipeInSpeed           = loadDataList [ "image_wipeInSpeed-"           + index ].GetData_as_Int();

			image.moveForm                  = loadDataList [ "image_moveForm-"                  + index ].GetData_as_String();
			image.moveForm_spring_softness  = loadDataList [ "image_moveForm_spring_softness-"  + index ].GetData_as_String();
			image.moveForm_linear_increment = loadDataList [ "image_moveForm_linear_increment-" + index ].GetData_as_String();

			_saveData_ImageList.push_back ( image );

		}
	}

	///// セーブ用のシリアライズデータ再現：ボタンリスト
	{
		///// 何件だった？
		int length = loadDataList [ "buttonListLength" ].GetData_as_Int();

		_saveData_ButtonList.clear();

		for ( int i = 0; i < length; ++i ) {

			ScriptButton::SaveData button;

			string index = Utils_Convert::Int_to_String(i);

			button.position.x  = loadDataList [ "button_position_X-" + index ].GetData_as_Int();
			button.position.y  = loadDataList [ "button_position_Y-"  + index].GetData_as_Int();
			button.size.x      = loadDataList [ "button_size_X-"      + index].GetData_as_Int();
			button.size.y      = loadDataList [ "button_size_Y-"      + index].GetData_as_Int();
			button.label       = loadDataList [ "button_label-"       + index].GetData_as_String();
			button.fontSize    = loadDataList [ "button_fontSize-"    + index].GetData_as_Int();
			button.name        = loadDataList [ "button_name-"        + index].GetData_as_String();
			button.jumpToLabel = loadDataList [ "button_jumpToLabel-" + index].GetData_as_String();
			button.isVisible   = Utils_Convert::String_to_bool ( loadDataList [ "button_visible-"     + index].GetData_as_String() );

			_saveData_ButtonList.push_back ( button );
		}
	}
	///// セーブ用のシリアライズデータ再現：クリック画像リスト
	{
		///// 何件だった？
		int length = loadDataList [ "clickableImageListLength" ].GetData_as_Int();

		_saveData_ClickableImageList.clear();

		for ( int i = 0; i < length; ++i ) {

			ClickableImage::SaveData clickableImage;

			string index = Utils_Convert::Int_to_String(i);

			clickableImage.position.x     = loadDataList [ "clickableImage_posX-"     + index ].GetData_as_Int();
			clickableImage.position.y     = loadDataList [ "clickableImage_posY-"     + index ].GetData_as_Int();
			clickableImage.prop.imageFile = loadDataList [ "clickableImage_fileName-" + index ].GetData_as_String();
			clickableImage.prop.size.x    = loadDataList [ "clickableImage_sizeX-"    + index ].GetData_as_Int();
			clickableImage.prop.size.y    = loadDataList [ "clickableImage_sizeY-"    + index ].GetData_as_Int();
			clickableImage.prop.name      = loadDataList [ "clickableImage_name-"     + index ].GetData_as_String();
			clickableImage.prop.rect_Normal = GUI_SFML::SFML_Utils::Convert_string_to_IntRect (loadDataList [ "clickableImage_rect_Normal-" + index ].GetData_as_String() );
			clickableImage.prop.rect_Over   = GUI_SFML::SFML_Utils::Convert_string_to_IntRect (loadDataList [ "clickableImage_rect_Over-"   + index ].GetData_as_String() );
			clickableImage.prop.rect_Push   = GUI_SFML::SFML_Utils::Convert_string_to_IntRect (loadDataList [ "clickableImage_rect_Push-"   + index ].GetData_as_String() );

			_saveData_ClickableImageList.push_back ( clickableImage );
		}

	}
	///// セーブ用のシリアライズデータ再現：テキストラベルリスト
	{
		///// 何件だった？
		int length = loadDataList [ "textLabelListLength" ].GetData_as_Int();

		_saveData_textLabelList.clear();

		for ( int i = 0; i < length; ++i ) {

			TextLabel::SaveData saveData;

			string index = Utils_Convert::Int_to_String(i);

			saveData.name       = loadDataList [ "textLabel_name-"      + index ].GetData_as_String();
			saveData.text       = loadDataList [ "textLabel_text-"      + index ].GetData_as_String();
			saveData.size       = loadDataList [ "textLabel_size-"      + index ].GetData_as_Int();
			saveData.position.x = loadDataList [ "textLabel_positionX-" + index ].GetData_as_Int();
			saveData.position.y = loadDataList [ "textLabel_positionY-" + index ].GetData_as_Int();
			saveData.color      = GUI_SFML::SFML_Utils::Convert_string_to_color ( loadDataList [ "textLabel_color-"     + index ].GetData_as_String() );

			_saveData_textLabelList.push_back ( saveData );
		}

	}
	///// セーブ用のシリアライズデータ再現：ゲーム盤
	{
		// TODO さしあたり、試合中のセーブ/ロードのことは考えないこととする

		///// 何件だった？
		int length = loadDataList [ "gameManagerListLength" ].GetData_as_Int();

		_saveData_GameManagerList.clear();

		for ( int i = 0; i < length; ++i ) {

			GameManager::SaveData saveData;

			string index = Utils_Convert::Int_to_String(i);

			saveData.name                    = loadDataList [ "gameManager_name-"                   + index ].GetData_as_String();
			saveData.gameBoardSerialized     = loadDataList [ "gameManager_board-"                  + index ].GetData_as_String();
			saveData.evaluationSumUp_FromAI  = loadDataList [ "gameManager_evaluationSumUp_FromAI-" + index ].GetData_as_String();
			saveData.gameRecordSerialized    = loadDataList [ "gameManager_gameRecordSerialized-"   + index ].GetData_as_String();
			saveData.astralProjection_physicalState  = loadDataList [ "gameManager_astralProjection_physicalState-" + index ].GetData_as_String();

			saveData.boardCenterX            = loadDataList [ "gameManager_boardCenterX-"           + index ].GetData_as_Int();
			saveData.boardCenterY            = loadDataList [ "gameManager_boardCenterY-"           + index ].GetData_as_Int();

			saveData.actorPosisionX          = loadDataList [ "gameManager_actorPositionX-"         + index ].GetData_as_Int();
			saveData.actorPosisionY          = loadDataList [ "gameManager_actorPositionY-"         + index ].GetData_as_Int();

			saveData.operationState          = loadDataList [ "gameManager_operationState-"         + index ].GetData_as_Int();
			saveData.isActive                = Utils_String::isPositiveWord(loadDataList [ "gameManager_isActive-"                + index ].GetData_as_String());
			saveData.isFreeOperation         = Utils_String::isPositiveWord(loadDataList [ "gameManager_isFreeOperation-"         + index ].GetData_as_String());
			saveData.isRendered              = Utils_String::isPositiveWord(loadDataList [ "gameManager_isRendered-"              + index ].GetData_as_String());
			saveData.IsSelectableMoveListNew = Utils_String::isPositiveWord(loadDataList [ "gameManager_IsSelectableMoveListNew-" + index ].GetData_as_String());
			saveData.isAstralProjection      = Utils_String::isPositiveWord(loadDataList [ "gameManager_isAstralProjection-"      + index ].GetData_as_String());
			saveData.is_MoveEffectGoing      = Utils_String::isPositiveWord(loadDataList [ "gameManager_is_MoveEffectGoing-"      + index ].GetData_as_String());


			saveData.kifuViewerPosition.x      = loadDataList [ "gameManager_kifuViewerPosition.x-"   + index ].GetData_as_Int();
			saveData.kifuViewerPosition.y      = loadDataList [ "gameManager_kifuViewerPosition.y-"   + index ].GetData_as_Int();

			saveData.selectingGridX      = loadDataList [ "gameManager_selectingGridX-"   + index ].GetData_as_Int();
			saveData.selectingGridY      = loadDataList [ "gameManager_selectingGridY-"   + index ].GetData_as_Int();
			saveData.operationMemo_belongsToWhichPieceBox = loadDataList [ "gameManager_operationMemo_belongsToWhichPieceBox-" + index ].GetData_as_String();
			saveData.operationMemo_whatColorThePieceWas   = loadDataList [ "gameManager_operationMemo_whatColorThePieceWas-"   + index ].GetData_as_Int();
			saveData.operationMemo_belongsToWhichCardBox  = loadDataList [ "gameManager_operationMemo_belongsToWhichCardBox-"  + index ].GetData_as_String();
			saveData.operationMemo_whichCardTheCardWas    = loadDataList [ "gameManager_operationMemo_whichCardTheCardWas-"    + index ].GetData_as_String();

			saveData.operationData.isLock      = Utils_String::isPositiveWord(loadDataList [ "gameManager_operationData.isLock-" + index ].GetData_as_String());
			saveData.operationData.currentTurn = loadDataList [ "gameManager_operationData.currentTurn-" + index ].GetData_as_String();

			saveData.operationData.currentTurnIndex = loadDataList [ "gameManager_operationData.currentTurnIndex-" + index ].GetData_as_Int();
			string turnOrderString                  = loadDataList [ "gameManager_operationData.turnOrder-" + index ].GetData_as_String();
			saveData.operationData.turnOrder = Utils_String::Split ( turnOrderString, "<+*+>" );

			//saveData.operationData.first       = loadDataList [ "gameManager_operationData.first-"       + index ].GetData_as_String();
			//saveData.operationData.second      = loadDataList [ "gameManager_operationData.second-"      + index ].GetData_as_String();
			//saveData.operationData.operatorP1  = loadDataList [ "gameManager_operationData.operatorP1-"  + index ].GetData_as_String();
			//saveData.operationData.operatorP2  = loadDataList [ "gameManager_operationData.operatorP2-"  + index ].GetData_as_String();

			saveData.battleInfo.isBattlingNow  = Utils_String::isPositiveWord(loadDataList [ "gameManagaer_battleInfo.isBattlingNow-" + index ].GetData_as_String());
			saveData.battleInfo.result         = loadDataList [ "gameManagaer_battleInfo.result-" + index ].GetData_as_String();

			saveData.selectableMoveList = MoveFromString_List ( loadDataList [ "gameManager_selectableMoveList-" + index ].GetData_as_String() );

			saveData.progressData_AfterThinking = ProgressDataFromString( loadDataList [ "gameManager_progressData_AfterThinking-" + index ].GetData_as_String() );

			//saveData.AI_fileName     = loadDataList [ "gameManager_AI_fileName-"     + index ].GetData_as_String();
			//saveData.actorSerialized = loadDataList [ "gameManager_actorSerialized-" + index ].GetData_as_String();
			saveData.participantsString = loadDataList [ "gameManager_participantsString-" + index ].GetData_as_String();
			
			saveData.battleTitle = loadDataList [ "gameManager_battleTitle-" + index ].GetData_as_String();
			
			_saveData_GameManagerList.push_back ( saveData );
		}


	}

	///// セーブ用のシリアライズデータ再現：スクリプト進行状況 _saveData_ScriptProgress
	{
		///// カレントプロセスのラベルと、state
		//_saveData_ScriptProgress.currentProcessLabel = loadDataList [ "scriptProgress_currentProcessLabel" ].GetData_as_String();
		_saveData_ScriptProgress.state               = loadDataList [ "scriptProgress_state" ].GetData_as_Int();

		///// プロセスの数は何件だった？
		int    processListLength = loadDataList [ "scriptProcessSaveDataList" ].GetData_as_Int();

		_saveData_ScriptProgress.processSaveDataList.clear();

		for ( int i = 0; i < processListLength; ++i ) {

			ScriptProcess_SaveData process;

			string index = Utils_Convert::Int_to_String(i);

			process.label                = loadDataList [ "scriptProcess_label-"                + index ].GetData_as_String();
			//process.referingProcessLabel = loadDataList [ "scriptProcess_referingProcessLabel-" + index ].GetData_as_String();
			process.currentUnitIndex     = loadDataList [ "scriptProcess_currentUnitIndex-"     + index ].GetData_as_Int();

			_saveData_ScriptProgress.processSaveDataList.push_back ( process );
		}
	}

	///// セーブ用のシリアライズデータ再現：変数 数値 _variable_Local_Int
	{
		_saveData_variable_Int.clear();

		for ( map<string, SaveDataUnit>::iterator it = loadDataList.begin(); it != loadDataList.end(); ++it) {


			// キーに "LVstring_"がついてる？
			string suffix = "LVint_";
			string key    = it->first;

			string key_suffix     = ( key.size() >= suffix.size() ) ? key.substr(0, suffix.size()) : "";
			string key_secondHalf = ( key.substr ( key_suffix.size()) );

			if ( key_suffix == suffix ) {

				// ここでは、キーのままに受け取る
				_saveData_variable_Int [ key ] = it->second.GetData_as_Int();
			}
		}
	}

	///// セーブ用のシリアライズデータ再現：変数 文字列 _variable_Local_String
	{
		/*
		for ( map<string, string>::iterator it = _saveData_variable_String.begin(); it != _saveData_variable_String.end(); ++it) {

			// 元の変数リストのキーに "LVstring_" がついている。
			saveDataList.push_back ( SaveDataUnit ( it->first,  it->second ));
		}
		*/

		_saveData_variable_String.clear();

		for ( map<string, SaveDataUnit>::iterator it = loadDataList.begin(); it != loadDataList.end(); ++it) {


			// キーに "LVstring_"がついてる？
			string suffix = "LVstring_";
			string key    = it->first;

			string key_suffix     = ( key.size() >= suffix.size() ) ? key.substr(0, suffix.size()) : "";
			string key_secondHalf = ( key.substr ( key_suffix.size()) );

			if ( key_suffix == suffix ) {

				// ここでは、キーのままに受け取る
				_saveData_variable_String [ key ] = it->second.GetData_as_String();
			}
		}
	}

	///// セーブ用のシリアライズデータ再現：テキスト欄
	{
		_saveData_TextMessageWindow.addTextHistory.clear();

		string addText_latest = loadDataList [ "textWindow_addText_latest" ].GetData_as_String();
		_saveData_TextMessageWindow.addText_latest = addText_latest;

		_saveData_TextMessageWindow.stepped = loadDataList [ "textWindow_stepped" ].GetData_as_Int();


		string isShown = loadDataList [ "textWindow_isShown" ].GetData_as_String();
		_saveData_TextMessageWindow.isShown = Utils_Convert::String_to_bool ( isShown );

		_saveData_TextMessageWindow.speakerName = loadDataList [ "textWindow_speakerName" ].GetData_as_String();

	}
	///// セーブ用のシリアライズデータ再現：BGM
	{
		_saveData_BGM.currentPlayingLabel = loadDataList [ "bgm_currentPlayingLabel" ].GetData_as_String();
		_saveData_BGM.pitch               = loadDataList [ "bgm_pitch"               ].GetData_as_String();
		_saveData_BGM.loop                = Utils_Convert::String_to_bool( loadDataList [ "bgm_loop"               ].GetData_as_String() );
		//string value                      = loadDataList [ "bgm_registHistory" ].GetData_as_String();
		//_saveData_BGM.registHistory       = Utils_Convert::string_to_map_string_string ( value );
	}
	///// セーブ用のシリアライズデータ再現：効果音
	{
		//string value                     = loadDataList [ "soundManager_labelList" ].GetData_as_String();
		//_saveData_SoundManager.labelList = Utils_Convert::string_to_map_string_string ( value );
	}
	///// セーブ用のシリアライズデータ再現：Camera
	{
		_saveData_Camera.centerX = loadDataList [ "camera_centerX" ].GetData_as_Int();
		_saveData_Camera.centerY = loadDataList [ "camera_centerY" ].GetData_as_Int();
	}

	///// セーブ用のシリアライズデータ再現：選択肢
	{
		string value = loadDataList [ "select_optionList" ].GetData_as_String();
		_saveData_Select.optionList = Utils_Convert::string_to_pairList_string_string ( value );

		_saveData_Select.gameBoardId = loadDataList [ "select_gameBoardId" ].GetData_as_String();
		_saveData_Select.positionX   = loadDataList [ "select_positionX" ].GetData_as_Int();
		_saveData_Select.positionY   = loadDataList [ "select_positionY" ].GetData_as_Int();
		
		vector<AI_Modules::Move> moveList = MoveFromString_List( loadDataList [ "select_moveList" ].GetData_as_String() );
		_saveData_Select.moveList = moveList;

	}
	///// セーブ用のシリアライズデータ再現：イベント監視・クリック画像
	{

		_saveData_waitClickableImageDataList = loadDataList [ "wait_ClickableImageDataList" ].GetData_as_String();
	}
	///// セーブ用のシリアライズデータ再現：イベント監視
	{

		_saveData_waitEventDataList = loadDataList [ "wait_EventDataList" ].GetData_as_String();
	}
	///// セーブ用のシリアライズデータ再現：イベント監視・盤面操作
	{

		_saveData_waitGameOperationDataList = loadDataList [ "wait_GameOperationDataList" ].GetData_as_String();
	}
	///// セーブ用のシリアライズデータ資源：ScriptEngineその他
	{
		_saveData_ScriptEngineMiscellaneous = loadDataList [ "ScriptEngineMiscellaneous" ].GetData_as_String();
	}


	///// 見た目を更新
	SetupAppearance();

}

//=========================================================
// 書き込み
// ファイルに書き込む。
// ファイルのもともとの内容は、消される。
//=========================================================
void SaveData::Save () {

	vector< SaveDataUnit > saveDataList;

	time_t t = time(NULL);
	
	map<string, int> timeInfo = Utils_Convert::Long_to_TimeInfo ( t );
	
	/*
	tm *localTime = localtime(&t);
	
	int year  = localTime->tm_year + 1900;
	int month = localTime->tm_mon + 1;
	int day   = localTime->tm_mday;
	int hour  = localTime->tm_hour;
	int min   = localTime->tm_min;
	int sec   = localTime->tm_sec;
	*/
	int year  = timeInfo["year"];
	int month = timeInfo["month"];
	int day   = timeInfo["day"];
	int hour  = timeInfo["hour"];
	int min   = timeInfo["min"];
	int sec   = timeInfo["sec"];
	
	saveDataList.push_back ( SaveDataUnit ( "year",  year  ));
	saveDataList.push_back ( SaveDataUnit ( "month", month ));
	saveDataList.push_back ( SaveDataUnit ( "day",   day   ));
	saveDataList.push_back ( SaveDataUnit ( "hour",  hour  ));
	saveDataList.push_back ( SaveDataUnit ( "min",   min   ));
	saveDataList.push_back ( SaveDataUnit ( "sec",   sec   ));

	char date[512];

	sprintf ( date, "%d年 %02d月 %02d日 %02d:%02d:%02d",
		year,
		month,
		day  ,
		hour ,
		min  ,
		sec
	);
	_date = date;

	saveDataList.push_back ( SaveDataUnit ( "chapterName",   _chapterName   ));

	///// セーブ用のシリアライズデータ作成：変数 数値 _variable_Local_Int
	{
		for ( map<string, int>::iterator it = _saveData_variable_Int.begin(); it != _saveData_variable_Int.end(); ++it) {

			// 元の変数リストのキーに "LVint_" がついている。
			saveDataList.push_back ( SaveDataUnit ( it->first,  it->second ));
		}
	}

	///// セーブ用のシリアライズデータ作成：変数 文字列 _variable_Local_String
	{
		for ( map<string, string>::iterator it = _saveData_variable_String.begin(); it != _saveData_variable_String.end(); ++it) {

			// 元の変数リストのキーに "LVstring_" がついている。
			saveDataList.push_back ( SaveDataUnit ( it->first,  it->second ));
		}
	}

	///// セーブ用のシリアライズデータ作成：画像リスト _imageList
	{

		// イメージの数を記録しておく
		/*
			ロード時には、この数をまず取得して、
			その件数分のイメージデータ（のキー）が保存されているはず、と前提して、
			セーブデータを解析する。
		*/
		saveDataList.push_back ( SaveDataUnit ( "imageListLength" , _saveData_ImageList.size() ));

		for ( unsigned int i = 0; i < _saveData_ImageList.size(); ++i ) {

			//_saveData_ImageList[i].
			/*
			struct SaveData {

				sf::Vector2f position_Goal;     // 目的地
				sf::Vector2f position_Current;  // 現在ただいまの表示位置

				int z;
				string label;

				string image_fileName; // 画像
				string maskColorString; // マスク未設定 を空文字で表したい。だからあえて、この段階で文字列化しておくのです。


				string wipeOutImage_fileName;
				string wipeInImage_fileName;

				int wipeOutSpeed;
				int wipeInSpeed;

				int fadeOutSpeed;
				int fadeInSpeed;
			};
			*/
			saveDataList.push_back ( SaveDataUnit ( "image_position_Goal_X-"       + Utils_Convert::Int_to_String(i),  _saveData_ImageList[i].position_Goal.x       ));
			saveDataList.push_back ( SaveDataUnit ( "image_position_Goal_Y-"       + Utils_Convert::Int_to_String(i),  _saveData_ImageList[i].position_Goal.y       ));
			saveDataList.push_back ( SaveDataUnit ( "image_position_Current_X-"    + Utils_Convert::Int_to_String(i),  _saveData_ImageList[i].position_Current.x    ));
			saveDataList.push_back ( SaveDataUnit ( "image_position_Current_Y-"    + Utils_Convert::Int_to_String(i),  _saveData_ImageList[i].position_Current.y    ));

			saveDataList.push_back ( SaveDataUnit ( "image_rotation-"                + Utils_Convert::Int_to_String(i),  _saveData_ImageList[i].rotation               ));
			saveDataList.push_back ( SaveDataUnit ( "image_scale_X-"                 + Utils_Convert::Int_to_String(i),  _saveData_ImageList[i].scale.x                ));
			saveDataList.push_back ( SaveDataUnit ( "image_scale_Y-"                 + Utils_Convert::Int_to_String(i),  _saveData_ImageList[i].scale.y                ));

			saveDataList.push_back ( SaveDataUnit ( "image_label-"                 + Utils_Convert::Int_to_String(i),  _saveData_ImageList[i].label                 ));
			saveDataList.push_back ( SaveDataUnit ( "image_z-"                     + Utils_Convert::Int_to_String(i),  _saveData_ImageList[i].z                     ));
			saveDataList.push_back ( SaveDataUnit ( "image_image_fileName-"        + Utils_Convert::Int_to_String(i),  _saveData_ImageList[i].image_fileName        ));
			saveDataList.push_back ( SaveDataUnit ( "image_maskColorString-"       + Utils_Convert::Int_to_String(i),  _saveData_ImageList[i].maskColorString       ));
			saveDataList.push_back ( SaveDataUnit ( "image_wipeOutImage_fileName-" + Utils_Convert::Int_to_String(i),  _saveData_ImageList[i].wipeOutImage_fileName ));
			saveDataList.push_back ( SaveDataUnit ( "image_wipeInImage_fileName-"  + Utils_Convert::Int_to_String(i),  _saveData_ImageList[i].wipeInImage_fileName  ));
			saveDataList.push_back ( SaveDataUnit ( "image_fadeOutSpeed-"          + Utils_Convert::Int_to_String(i),  _saveData_ImageList[i].fadeOutSpeed          ));
			saveDataList.push_back ( SaveDataUnit ( "image_fadeInSpeed-"           + Utils_Convert::Int_to_String(i),  _saveData_ImageList[i].fadeInSpeed           ));
			saveDataList.push_back ( SaveDataUnit ( "image_wipeOutSpeed-"          + Utils_Convert::Int_to_String(i),  _saveData_ImageList[i].wipeOutSpeed          ));
			saveDataList.push_back ( SaveDataUnit ( "image_wipeInSpeed-"           + Utils_Convert::Int_to_String(i),  _saveData_ImageList[i].wipeInSpeed           ));

			saveDataList.push_back ( SaveDataUnit ( "image_moveForm-"                  + Utils_Convert::Int_to_String(i),  _saveData_ImageList[i].moveForm           ));
			saveDataList.push_back ( SaveDataUnit ( "image_moveForm_spring_softness-"  + Utils_Convert::Int_to_String(i),  _saveData_ImageList[i].moveForm_spring_softness  ));
			saveDataList.push_back ( SaveDataUnit ( "image_moveForm_linear_increment-" + Utils_Convert::Int_to_String(i),  _saveData_ImageList[i].moveForm_linear_increment ));
		}
	}
	///// セーブ用のシリアライズデータ作成：ボタンリスト
	{

		// イメージの数を記録しておく
		saveDataList.push_back ( SaveDataUnit ( "buttonListLength" , _saveData_ButtonList.size() ));

		/*
			struct SaveData {

				sf::Vector2f position;
				string       label;
				sf::Vector2f size;
				int          fontSize;
				bool         isVisible;
				string       jumpToLabel;
				string       name;
			};
		*/
		for ( unsigned int i = 0; i < _saveData_ButtonList.size(); ++i ) {

			string index = Utils_Convert::Int_to_String(i);

			saveDataList.push_back ( SaveDataUnit ( "button_position_X-"  + index,  _saveData_ButtonList[i].position.x  ));
			saveDataList.push_back ( SaveDataUnit ( "button_position_Y-"  + index,  _saveData_ButtonList[i].position.y  ));
			saveDataList.push_back ( SaveDataUnit ( "button_size_X-"      + index,  _saveData_ButtonList[i].size.x      ));
			saveDataList.push_back ( SaveDataUnit ( "button_size_Y-"      + index,  _saveData_ButtonList[i].size.y      ));
			saveDataList.push_back ( SaveDataUnit ( "button_label-"       + index,  _saveData_ButtonList[i].label       ));
			saveDataList.push_back ( SaveDataUnit ( "button_fontSize-"    + index,  _saveData_ButtonList[i].fontSize    ));
			saveDataList.push_back ( SaveDataUnit ( "button_name-"        + index,  _saveData_ButtonList[i].name        ));
			saveDataList.push_back ( SaveDataUnit ( "button_jumpToLabel-" + index,  _saveData_ButtonList[i].jumpToLabel ));
			saveDataList.push_back ( SaveDataUnit ( "button_visible-"     + index,  Utils_Convert::Bool_to_string ( _saveData_ButtonList[i].isVisible )  ));
		}
	}

	///// セーブ用のシリアライズデータ作成：クリック画像リスト
	{

		// イメージの数を記録しておく
		saveDataList.push_back ( SaveDataUnit ( "clickableImageListLength" , _saveData_ClickableImageList.size() ));

		/*
			struct SaveData {

				Property prop;
				sf::Vector2f position;

			};
			struct Property {

				string       imageFile;
				sf::Vector2f size;
				sf::IntRect  rect_Normal;
				sf::IntRect  rect_Over;
				sf::IntRect  rect_Push;

				string name;
			};
		*/
		for ( unsigned int i = 0; i < _saveData_ClickableImageList.size(); ++i ) {

			string index = Utils_Convert::Int_to_String(i);

			ClickableImage::SaveData saveData = _saveData_ClickableImageList[i];
			ClickableImage::Property prop = saveData.prop;

			string fileName = prop.imageFile;

			float sizeX = prop.size.x;
			float sizeY = prop.size.y;

			string rect_Normal = GUI_SFML::SFML_Utils::Convert_IntRect_to_string ( prop.rect_Normal );
			string rect_Over   = GUI_SFML::SFML_Utils::Convert_IntRect_to_string ( prop.rect_Over );
			string rect_Push   = GUI_SFML::SFML_Utils::Convert_IntRect_to_string ( prop.rect_Push );

			string name = prop.name;

			float posX = saveData.position.x;
			float posY = saveData.position.y;

			saveDataList.push_back ( SaveDataUnit ( "clickableImage_fileName-"     + index,  fileName    ));
			saveDataList.push_back ( SaveDataUnit ( "clickableImage_sizeX-"        + index,  (int)sizeX  ));
			saveDataList.push_back ( SaveDataUnit ( "clickableImage_sizeY-"        + index,  (int)sizeY  ));
			saveDataList.push_back ( SaveDataUnit ( "clickableImage_rect_Normal-"  + index,  rect_Normal ));
			saveDataList.push_back ( SaveDataUnit ( "clickableImage_rect_Over-"    + index,  rect_Over   ));
			saveDataList.push_back ( SaveDataUnit ( "clickableImage_rect_Push-"    + index,  rect_Push   ));
			saveDataList.push_back ( SaveDataUnit ( "clickableImage_name-"         + index,  name        ));
			saveDataList.push_back ( SaveDataUnit ( "clickableImage_posX-"         + index,  (int)posX   ));
			saveDataList.push_back ( SaveDataUnit ( "clickableImage_posY-"         + index,  (int)posY   ));

		}
	}

	///// セーブ用のシリアライズデータ作成：テキストラベルリスト
	{

		// イメージの数を記録しておく
		saveDataList.push_back ( SaveDataUnit ( "textLabelListLength" , _saveData_textLabelList.size() ));

		for ( unsigned int i = 0; i < _saveData_textLabelList.size(); ++i ) {

			string index = Utils_Convert::Int_to_String(i);

			TextLabel::SaveData saveData = _saveData_textLabelList[i];

			saveDataList.push_back ( SaveDataUnit ( "textLabel_name-"      + index,  saveData.name    ));
			saveDataList.push_back ( SaveDataUnit ( "textLabel_text-"      + index,  saveData.text    ));
			saveDataList.push_back ( SaveDataUnit ( "textLabel_size-"      + index,  saveData.size    ));
			saveDataList.push_back ( SaveDataUnit ( "textLabel_color-"     + index,  GUI_SFML::SFML_Utils::Convert_color_to_string( saveData.color)));
			saveDataList.push_back ( SaveDataUnit ( "textLabel_positionX-" + index,  (int)saveData.position.x ));
			saveDataList.push_back ( SaveDataUnit ( "textLabel_positionY-" + index,  (int)saveData.position.y ));
		}
	}
	///// セーブ用のシリアライズデータ作成：ゲーム盤
	{
		// TODO さしあたり試合中のセーブ/ロードのことは考えないこととする

		// 件数を記録しておく
		saveDataList.push_back ( SaveDataUnit ( "gameManagerListLength" , _saveData_GameManagerList.size() ));

		for ( unsigned int i = 0; i < _saveData_GameManagerList.size(); ++i ) {

			string index = Utils_Convert::Int_to_String(i);

			GameManager::SaveData saveData = _saveData_GameManagerList[i];

			saveDataList.push_back ( SaveDataUnit ( "gameManager_name-"                   + index,  saveData.name    ));
			saveDataList.push_back ( SaveDataUnit ( "gameManager_board-"                  + index,  saveData.gameBoardSerialized    ));
			saveDataList.push_back ( SaveDataUnit ( "gameManager_evaluationSumUp_FromAI-" + index,  saveData.evaluationSumUp_FromAI ));
			saveDataList.push_back ( SaveDataUnit ( "gameManager_astralProjection_physicalState-" + index,  saveData.astralProjection_physicalState ));
			saveDataList.push_back ( SaveDataUnit ( "gameManager_gameRecordSerialized-"           + index,  saveData.gameRecordSerialized ));

			saveDataList.push_back ( SaveDataUnit ( "gameManager_boardCenterX-"   + index,  saveData.boardCenterX  ));
			saveDataList.push_back ( SaveDataUnit ( "gameManager_boardCenterY-"   + index,  saveData.boardCenterY  ));
			saveDataList.push_back ( SaveDataUnit ( "gameManager_actorPositionX-" + index,  saveData.actorPosisionX  ));
			saveDataList.push_back ( SaveDataUnit ( "gameManager_actorPositionY-" + index,  saveData.actorPosisionY  ));

			saveDataList.push_back ( SaveDataUnit ( "gameManager_operationState-" + index,  saveData.operationState  ));
			saveDataList.push_back ( SaveDataUnit ( "gameManager_isActive-"                + index,  (saveData.isActive                ? "true" : "false") ));
			saveDataList.push_back ( SaveDataUnit ( "gameManager_isFreeOperation-"         + index,  (saveData.isFreeOperation         ? "true" : "false") ));
			saveDataList.push_back ( SaveDataUnit ( "gameManager_isRendered-"              + index,  (saveData.isRendered              ? "true" : "false") ));
			saveDataList.push_back ( SaveDataUnit ( "gameManager_IsSelectableMoveListNew-" + index,  (saveData.IsSelectableMoveListNew ? "true" : "false") ));
			saveDataList.push_back ( SaveDataUnit ( "gameManager_isAstralProjection-"      + index,  (saveData.isAstralProjection      ? "true" : "false") ));
			saveDataList.push_back ( SaveDataUnit ( "gameManager_is_MoveEffectGoing-"      + index,  (saveData.is_MoveEffectGoing      ? "true" : "false") ));
			saveDataList.push_back ( SaveDataUnit ( "gameManager_kifuViewerPosition.x-"   + index,  (int)saveData.kifuViewerPosition.x  ));
			saveDataList.push_back ( SaveDataUnit ( "gameManager_kifuViewerPosition.y-"   + index,  (int)saveData.kifuViewerPosition.y  ));
			saveDataList.push_back ( SaveDataUnit ( "gameManager_selectingGridX-"         + index,  saveData.selectingGridX  ));
			saveDataList.push_back ( SaveDataUnit ( "gameManager_selectingGridY-"         + index,  saveData.selectingGridY  ));
			saveDataList.push_back ( SaveDataUnit ( "gameManager_operationMemo_belongsToWhichPieceBox-" + index, saveData.operationMemo_belongsToWhichPieceBox ));
			saveDataList.push_back ( SaveDataUnit ( "gameManager_operationMemo_whatColorThePieceWas-"   + index, saveData.operationMemo_whatColorThePieceWas   ));
			saveDataList.push_back ( SaveDataUnit ( "gameManager_operationMemo_belongsToWhichCardBox-"  + index, saveData.operationMemo_belongsToWhichCardBox  ));
			saveDataList.push_back ( SaveDataUnit ( "gameManager_operationMemo_whichCardTheCardWas-"    + index, saveData.operationMemo_whichCardTheCardWas    ));

			saveDataList.push_back ( SaveDataUnit ( "gameManager_operationData.isLock-"         + index,  (saveData.operationData.isLock    ? "true" : "false") ));
			saveDataList.push_back ( SaveDataUnit ( "gameManager_operationData.currentTurn-"    + index,  saveData.operationData.currentTurn    ));
			saveDataList.push_back ( SaveDataUnit ( "gameManager_operationData.currentTurnIndex-" + index,  saveData.operationData.currentTurnIndex ));

			string turnOrderString = "";
			for ( unsigned int n = 0; n < saveData.operationData.turnOrder.size(); ++n ) {
				if ( turnOrderString != "" ) { turnOrderString += "<+*+>"; }
				turnOrderString += saveData.operationData.turnOrder[n];
			}
			saveDataList.push_back ( SaveDataUnit ( "gameManager_operationData.turnOrder-" + index,  turnOrderString ));


			//saveDataList.push_back ( SaveDataUnit ( "gameManager_operationData.first-"          + index,  saveData.operationData.first    ));
			//saveDataList.push_back ( SaveDataUnit ( "gameManager_operationData.second-"         + index,  saveData.operationData.second    ));
			//saveDataList.push_back ( SaveDataUnit ( "gameManager_operationData.operatorP1-"     + index,  saveData.operationData.operatorP1    ));
			//saveDataList.push_back ( SaveDataUnit ( "gameManager_operationData.operatorP2-"     + index,  saveData.operationData.operatorP2    ));

			saveDataList.push_back ( SaveDataUnit ( "gameManager_battleInfo.isBattlingNow-" + index,  (saveData.battleInfo.isBattlingNow    ? "true" : "false") ));
			saveDataList.push_back ( SaveDataUnit ( "gameManager_battleInfo.result-"        + index,  saveData.battleInfo.result    ));

			saveDataList.push_back ( SaveDataUnit ( "gameManager_selectableMoveList-"         + index, MoveToString_List ( saveData.selectableMoveList ) ));
			saveDataList.push_back ( SaveDataUnit ( "gameManager_progressData_AfterThinking-" + index, ProgressDataToString ( saveData.progressData_AfterThinking ) ));

			//saveDataList.push_back ( SaveDataUnit ( "gameManager_AI_fileName-"     + index,  saveData.AI_fileName     ));
			//saveDataList.push_back ( SaveDataUnit ( "gameManager_actorSerialized-" + index,  saveData.actorSerialized ));
			saveDataList.push_back ( SaveDataUnit ( "gameManager_participantsString-" + index,  saveData.participantsString ));

			saveDataList.push_back ( SaveDataUnit ( "gameManager_battleTitle-" + index,  saveData.battleTitle ));

		}

	}
	///// セーブ用のシリアライズデータ作成：スクリプト進行状況 _saveData_ScriptProgress
	{
		///// プロセスの数を記録しておく
		saveDataList.push_back ( SaveDataUnit ( "scriptProcessSaveDataList" , _saveData_ScriptProgress.processSaveDataList.size()));

		for ( unsigned int i = 0; i < _saveData_ScriptProgress.processSaveDataList.size(); ++i ) {

			saveDataList.push_back ( SaveDataUnit ( "scriptProcess_label-"                + Utils_Convert::Int_to_String(i), _saveData_ScriptProgress.processSaveDataList[i].label                ));
			//saveDataList.push_back ( SaveDataUnit ( "scriptProcess_referingProcessLabel-" + Utils_Convert::Int_to_String(i), _saveData_ScriptProgress.processSaveDataList[i].referingProcessLabel ));
			saveDataList.push_back ( SaveDataUnit ( "scriptProcess_currentUnitIndex-"     + Utils_Convert::Int_to_String(i), _saveData_ScriptProgress.processSaveDataList[i].currentUnitIndex     ));
		}

		///// カレントプロセスのラベルと、state
		//saveDataList.push_back ( SaveDataUnit ( "scriptProgress_currentProcessLabel", _saveData_ScriptProgress.currentProcessLabel ));
		saveDataList.push_back ( SaveDataUnit ( "scriptProgress_state"              , _saveData_ScriptProgress.state               ));
	}

	///// セーブ用のシリアライズデータ作成：テキスト欄
	{
		string addText_latest = _saveData_TextMessageWindow.addText_latest;
		string isShown        = Utils_Convert::Bool_to_string ( _saveData_TextMessageWindow.isShown );

		saveDataList.push_back ( SaveDataUnit ( "textWindow_addText_latest",  addText_latest ));
		saveDataList.push_back ( SaveDataUnit ( "textWindow_stepped",  _saveData_TextMessageWindow.stepped ));

		saveDataList.push_back ( SaveDataUnit ( "textWindow_isShown",  Utils_Convert::Bool_to_string (_saveData_TextMessageWindow.isShown) ));
		saveDataList.push_back ( SaveDataUnit ( "textWindow_speakerName",  _saveData_TextMessageWindow.speakerName ));

	}
	///// セーブ用のシリアライズデータ作成：BGM
	{
		//string registHistory = Utils_Convert::map_string_string_to_string  ( _saveData_BGM.registHistory );
		//saveDataList.push_back ( SaveDataUnit ( "bgm_registHistory",  registHistory ));
		saveDataList.push_back ( SaveDataUnit ( "bgm_currentPlayingLabel",  _saveData_BGM.currentPlayingLabel ));
		saveDataList.push_back ( SaveDataUnit ( "bgm_pitch",  _saveData_BGM.pitch ));
		saveDataList.push_back ( SaveDataUnit ( "bgm_loop",  Utils_Convert::Bool_to_string (_saveData_BGM.loop )));
	}
	///// セーブ用のシリアライズデータ作成：効果音
	{
		//string labelList = Utils_Convert::map_string_string_to_string  ( _saveData_SoundManager.labelList );
		//saveDataList.push_back ( SaveDataUnit ( "soundManager_labelList",  labelList ));
	}

	///// セーブ用のシリアライズデータ作成：Camera
	{
		saveDataList.push_back ( SaveDataUnit ( "camera_centerX",  _saveData_Camera.centerX ));
		saveDataList.push_back ( SaveDataUnit ( "camera_centerY",  _saveData_Camera.centerY ));
	}

	///// セーブ用のシリアライズデータ作成：選択肢
	{
		string optionList = Utils_Convert::pairList_string_string_to_string ( _saveData_Select.optionList );
		
		string moveStringList = MoveToString_List ( _saveData_Select.moveList );

		saveDataList.push_back ( SaveDataUnit ( "select_optionList",  optionList ));
		saveDataList.push_back ( SaveDataUnit ( "select_gameBoardId", _saveData_Select.gameBoardId ));
		saveDataList.push_back ( SaveDataUnit ( "select_positionX",   _saveData_Select.positionX ));
		saveDataList.push_back ( SaveDataUnit ( "select_positionY",   _saveData_Select.positionY ));
		saveDataList.push_back ( SaveDataUnit ( "select_moveList",  moveStringList ));
		



	}
	///// セーブ用のシリアライズデータ作成：イベント監視・クリック画像
	{
		saveDataList.push_back ( SaveDataUnit ( "wait_ClickableImageDataList",  _saveData_waitClickableImageDataList ));
	}
	///// セーブ用のシリアライズデータ作成：イベント監視
	{
		saveDataList.push_back ( SaveDataUnit ( "wait_EventDataList",  _saveData_waitEventDataList ));
	}
	///// セーブ用のシリアライズデータ作成：イベント監視・盤面操作
	{
		saveDataList.push_back ( SaveDataUnit ( "wait_GameOperationDataList",  _saveData_waitGameOperationDataList ));
	}
	///// セーブ用のシリアライズデータ作成：ScriptEngineその他
	{
		saveDataList.push_back ( SaveDataUnit ( "ScriptEngineMiscellaneous",  _saveData_ScriptEngineMiscellaneous ));
	}



	WriteSaveData ( saveDataList );

	// セーブしたのですから、有効なデータ、ってことでいいですよね？
	_isValid = true;

	// セーブしたのですから、見た目を作り直しておく
	SetupAppearance();

	/*

	vector< SaveDataUnit > saveDataList;

	saveDataList.push_back ( SaveDataUnit ( "data1", 100 ) );
	saveDataList.push_back ( SaveDataUnit ( "data2", "ABCDEFG" ) );
	saveDataList.push_back ( SaveDataUnit ( "data3", 2000 ) );
	saveDataList.push_back ( SaveDataUnit ( "data4", "XYZ" ) );

	WriteSaveData ( saveDataList );


	map< string, SaveDataUnit > loadDataList;
	ReadSaveData ( loadDataList );

	int res = loadDataList["data1"].GetData_as_Int();
	string str = loadDataList["data2"].GetData_as_String();
	int res2 = loadDataList["data3"].GetData_as_Int();
	string str3 = loadDataList["data4"].GetData_as_String();
	*/
}



//=========================================================
// 状態をセーブ：スクリプトの進行状況
//=========================================================
void SaveData::Save_ScriptProgess ( ScriptEngine *scriptEngine ) {

	_saveData_ScriptProgress = scriptEngine->GetSaveData_ScriptProgress();

}
//=========================================================
// 状態をロード：スクリプトの進行状況
//=========================================================
void SaveData::Load_ScriptProgess ( ScriptEngine *scriptEngine ) {

	scriptEngine->Load_ScriptProgress ( _saveData_ScriptProgress );
}

//=========================================================
// 状態をセーブ：イベント監視情報・クリック画像
//=========================================================
void SaveData::Save_WaitClickableImageDataList ( ScriptEngine *scriptEngine ) {

	_saveData_waitClickableImageDataList = scriptEngine->Serialize_WatchClickableImageData();

}
//=========================================================
// 状態をロード：イベント監視情報・クリック画像
//=========================================================
void SaveData::Load_WaitClickableImageDataList ( ScriptEngine *scriptEngine ) {

	scriptEngine->DeSerialize_WatchClickableImageData( _saveData_waitClickableImageDataList );

}


//=========================================================
// 状態をセーブ：イベント監視情報
//=========================================================
void SaveData::Save_WaitEventDataList( ScriptEngine *scriptEngine ) {

	_saveData_waitEventDataList = scriptEngine->Serialize_WatchEventData();

}
//=========================================================
// 状態をロード：イベント監視情報
//=========================================================
void SaveData::Load_WaitEventDataList( ScriptEngine *scriptEngine ) {

	scriptEngine->DeSerialize_WatchEventData( _saveData_waitEventDataList );

}

//=========================================================
// 状態をセーブ：イベント監視情報・盤面操作待ち受け情報
//=========================================================
void SaveData::Save_WaitGameOperationDataList( ScriptEngine *scriptEngine ) {

	_saveData_waitGameOperationDataList = scriptEngine->Serialize_WatchGameOperationData();

}
//=========================================================
// 状態をロード：イベント監視情報・盤面操作待ち受け情報
//=========================================================
void SaveData::Load_WaitGameOperationDataList( ScriptEngine *scriptEngine ) {

	scriptEngine->DeSerialize_WatchGameOperationData(_saveData_waitGameOperationDataList );

}
//=========================================================
// 状態をセーブ：変数＿Int
//=========================================================
void SaveData::Save_LoclVariable_Int ( map <string, int>    &variable_Local_Int ) {

	_saveData_variable_Int.clear();

	for ( map<string, int>::iterator it = variable_Local_Int.begin(); it != variable_Local_Int.end(); ++it ) {

		///// キーに目印をつけて、受け取る。ほかのキーとかぶらないようにするため。
		string key = "LVint_" + it->first;
		_saveData_variable_Int [ key ] = it->second;
	}
}

//=========================================================
// 状態をロード：変数＿Int
//=========================================================
void SaveData::Load_LoclVariable_Int ( map <string, int>    &variable_Local_Int ) {

	variable_Local_Int.clear();

	for ( map<string, int>::iterator it = _saveData_variable_Int.begin(); it != _saveData_variable_Int.end(); ++it ) {

		///// キーに目印をつけた目印をはずして、渡す。
		string key    = it->first;

		// キーに "LVstring_"がついてる？
		string suffix = "LVint_";

		string key_suffix     = ( key.size() >= suffix.size() ) ? key.substr(0, suffix.size()) : "";
		string key_secondHalf = ( key.substr ( key_suffix.size()) );

		variable_Local_Int [ key_secondHalf ] = it->second;
	}
}

//=========================================================
// 状態をセーブ：変数＿String
//=========================================================
void SaveData::Save_LoclVariable_String ( map <string, string> &variable_Local_String ) {

	_saveData_variable_String.clear();

	for ( map<string, string>::iterator it = variable_Local_String.begin(); it != variable_Local_String.end(); ++it ) {

		///// キーに目印をつけて、受け取る。ほかのキーとかぶらないようにするため。
		string key = "LVstring_" + it->first;
		_saveData_variable_String [ key ] = it->second;
	}
}

//=========================================================
// 状態をロード：変数＿String
//=========================================================
void SaveData::Load_LoclVariable_String ( map <string, string> &variable_Local_String ) {

	variable_Local_String.clear();

	for ( map<string, string>::iterator it = _saveData_variable_String.begin(); it != _saveData_variable_String.end(); ++it ) {

		///// キーに目印をつけた目印をはずして、渡す。
		string key    = it->first;

		// キーに "LVstring_"がついてる？
		string suffix = "LVstring_";

		string key_suffix     = ( key.size() >= suffix.size() ) ? key.substr(0, suffix.size()) : "";
		string key_secondHalf = ( key.substr ( key_suffix.size()) );

		variable_Local_String [ key_secondHalf ] = it->second;
	}
}

//=========================================================
// 状態をセーブ：テキスト欄
//=========================================================
void SaveData::Save_TextMessageWindow   ( GUI_SFML::TextMessageWindow *textMessageWindow ) {

	if (textMessageWindow != NULL) {
		_saveData_TextMessageWindow = textMessageWindow->GetSaveData();
	} else {
		//_saveData_TextMessageWindow.isExist = false;
	}

}
//=========================================================
// 状態をロード：テキスト欄
//=========================================================
void SaveData::Load_TextMessageWindow   ( GUI_SFML::TextMessageWindow *textMessageWindow ) {

	// テキスト欄が NULL ってことは、定義されてないってことですよね。
	// デフォルトで、定義自体は常にされていることにする？
	if ( textMessageWindow != NULL ) {
		textMessageWindow->Load (_saveData_TextMessageWindow);
	}
}

//=========================================================
// 状態をセーブ：BGM
//=========================================================
void SaveData::Save_BGM ( BGM *bgm  ) {

	/*
		_saveData_BGM.registHistory; は、map<string, string> の文字列シリアライズフォームとして、保存のこと。
	*/

	_saveData_BGM = bgm->GetSaveData();
}
//=========================================================
// 状態をロード：BGM
//=========================================================
void SaveData::Load_BGM ( BGM *bgm  ) {

	bgm->Load ( _saveData_BGM );
}
//=========================================================
// 状態をセーブ：効果音
//=========================================================
void SaveData::Save_SoundManager ( SoundManager *soundManager ) {

	_saveData_SoundManager =  soundManager->GetSaveData();
}
//=========================================================
// 状態をロード：効果音
//=========================================================
void SaveData::Load_SoundManager ( SoundManager *soundManager ) {

	soundManager->Load ( _saveData_SoundManager );
}

//=========================================================
// 状態をセーブ：Camera
//=========================================================
void SaveData::Save_Camera ( Camera *camera  ) {

	_saveData_Camera = camera->GetSaveData();
}
//=========================================================
// 状態をロード：Camera
//=========================================================
void SaveData::Load_Camera ( Camera *camera  ) {

	camera->Load ( _saveData_Camera );
}

//=========================================================
// 状態をセーブ：ImageList
//=========================================================
void SaveData::Save_Image ( vector < Image*> &imageList ) {

	// 今持っているセーブデータを消しておきます。一応。
	_saveData_ImageList.clear();

	for ( unsigned int i = 0; i < imageList.size(); ++i ) {

		_saveData_ImageList.push_back (
			imageList[i]->GetSaveData()
		);
	}

}
//=========================================================
// 状態をロード：ImageList
//=========================================================
void SaveData::Load_Image ( vector < Image*> &imageList ) {

	// いったん、渡されたリストをクリアしてしまますよ？
	for ( unsigned int i = 0; i < imageList.size(); ++i ) {

		Image *image = imageList[i];
		delete image;
	}
	imageList.clear();

	// 持っているセーブデータをもとに、イメージリストを再現
	for ( unsigned int i = 0; i < _saveData_ImageList.size(); ++i ) {

		Image *image = new Image();

		image->Load ( _saveData_ImageList[i] );

		imageList.push_back ( image );
	}

}

//=========================================================
// 状態をセーブ：ボタン
//=========================================================
void SaveData::Save_Button ( map < string, ScriptButton*> &scriptButtonList ) {

	_saveData_ButtonList.clear();

	for ( map<string, ScriptButton*>::iterator it = scriptButtonList.begin(); it != scriptButtonList.end(); ++it) {

		_saveData_ButtonList.push_back ( it->second->GetSaveData() );
	}

}
//=========================================================
// 状態をロード：ボタン
//=========================================================
void SaveData::Load_Button ( map < string, ScriptButton*> &scriptButtonList ) {

	///// まず、現在のものをクリアしておきまして、
	for ( map<string, ScriptButton*>::iterator it = scriptButtonList.begin(); it != scriptButtonList.end(); ++it) {

		delete it->second;
	}
	scriptButtonList.clear();

	///// これだと、ロードしたとき、Z位置は乱れるのかも？ 重ならないように位置を指定してね。
	for ( unsigned int i = 0; i < _saveData_ButtonList.size(); ++i ) {

		ScriptButton *button = ScriptButton::Load ( _saveData_ButtonList[i] );

		scriptButtonList [ _saveData_ButtonList[i].name  ] = button;
	}

}

//=========================================================
// 状態をセーブ：クリック可能画像
//=========================================================
void SaveData::Save_ClickableImage ( map<string, ClickableImage*> &clickableImageList ) {

	_saveData_ClickableImageList.clear();

	for ( map<string, ClickableImage*>::iterator it = clickableImageList.begin(); it != clickableImageList.end(); ++it) {

		_saveData_ClickableImageList.push_back ( it->second->GetSaveData() );
	}

}
//=========================================================
// 状態をセーブ：テキストラベル
//=========================================================
void SaveData::Save_TextLabelList ( map < string, TextLabel*> &textLabelList ) {

	_saveData_textLabelList.clear();

	for ( map<string, TextLabel*>::iterator it = textLabelList.begin(); it != textLabelList.end(); ++it) {

		_saveData_textLabelList.push_back ( it->second->GetSaveData() );
	}
}
//=========================================================
// 状態をセーブ：ゲーム盤
//=========================================================
void SaveData::Save_GameManagerList(  map < string, GameManager*> &gameManagerList ) {

	_saveData_GameManagerList.clear();

	for ( map<string, GameManager*>::iterator it = gameManagerList.begin(); it != gameManagerList.end(); ++it) {

		_saveData_GameManagerList.push_back ( it->second->GetSaveData() );
	}
}

//=========================================================
// 状態をロード：ゲーム盤
//=========================================================
void SaveData::Load_GameManagerList ( map<string, GameManager*> &gameManagerList, ScriptEngine *scriptEngine ) {

	///// まず、現在のものをクリアしておきまして、
	for ( map<string, GameManager*>::iterator it = gameManagerList.begin(); it != gameManagerList.end(); ++it) {

		delete it->second;
	}
	gameManagerList.clear();

	///// これだと、ロードしたとき、Z位置は乱れるのかも？ 重ならないように位置を指定してね。
	for ( unsigned int i = 0; i < _saveData_GameManagerList.size(); ++i ) {

		GameManager::SaveData saveData = _saveData_GameManagerList[i];

		GameManager *obj = new GameManager ( saveData.name, scriptEngine );

		obj->Load ( saveData );

		gameManagerList [ saveData.name  ] = obj;
	}

}

//=========================================================
// 状態をロード：クリック可能画像
//=========================================================
void SaveData::Load_ClickableImage ( map<string, ClickableImage*> &clickableImageList ) {

	///// まず、現在のものをクリアしておきまして、
	for ( map<string, ClickableImage*>::iterator it = clickableImageList.begin(); it != clickableImageList.end(); ++it) {

		delete it->second;
	}
	clickableImageList.clear();

	///// これだと、ロードしたとき、Z位置は乱れるのかも？ 重ならないように位置を指定してね。
	for ( unsigned int i = 0; i < _saveData_ClickableImageList.size(); ++i ) {

		ClickableImage *obj = new ClickableImage ( _saveData_ClickableImageList[i].prop ); // ここはダミーのようなもの

		obj->Load(_saveData_ClickableImageList[i]);

		clickableImageList [ _saveData_ClickableImageList[i].prop.name  ] = obj;
	}

}
//=========================================================
// 状態をロード：テキストラベル
//=========================================================
void SaveData::Load_TextLabelList( map<string, TextLabel*> &textLabelList, sf::Font &font ) {

	///// まず、現在のものをクリアしておきまして、
	for ( map<string, TextLabel*>::iterator it = textLabelList.begin(); it != textLabelList.end(); ++it) {

		delete it->second;
	}
	textLabelList.clear();

	///// これだと、ロードしたとき、Z位置は乱れるのかも？ 重ならないように位置を指定してね。
	for ( unsigned int i = 0; i < _saveData_textLabelList.size(); ++i ) {

		TextLabel::SaveData saveData = _saveData_textLabelList[i];

		TextLabel *obj = new TextLabel ( saveData.name );

		obj->SetFont     (font);
		obj->SetText     (saveData.text);
		obj->SetPosition (saveData.position);
		obj->SetColor    (saveData.color);
		obj->SetSize     (saveData.size);

		textLabelList [ saveData.name  ] = obj;
	}

}

//=========================================================
// 状態をセーブ：選択肢
//=========================================================
void SaveData::Save_Select ( Select *select ) {

	/*
		_saveData_Select.optionList は vector < pair < string, string> > の文字列版として保存のこと
	*/

	_saveData_Select = select->GetSaveData();
}
//=========================================================
// 状態をロード：選択肢
//=========================================================
void SaveData::Load_Select ( Select *select ) {

	select->Load ( _saveData_Select );
}

//=========================================================
// セーブ：ScriptEngineその他
//=========================================================
void SaveData::Save_ScriptEngineMiscellaneous  ( ScriptEngine *scriptEngine ) {

	_saveData_ScriptEngineMiscellaneous = scriptEngine->Serialize_Miscellaneous();
}
//=========================================================
// ロード：ScriptEngineその他
//=========================================================
void SaveData::Load_ScriptEngineMiscellaneous  ( ScriptEngine *scriptEngine ) {

	scriptEngine->DeSerialize_Miscellaneous ( _saveData_ScriptEngineMiscellaneous );
}


//=========================================================
// 書き込める形式で書き込む
//=========================================================
void SaveData::WriteSaveData ( vector<SaveDataUnit> &saveDataUnitList ) {

	/*
		この段階で、キーの前後に<>をつけて書き込む。
	*/

	///// ファイルを開く。
	FILE *pFile;
	pFile = fopen( _fileName.c_str(), "wb");

	for ( unsigned int i = 0; i < saveDataUnitList.size(); ++i ) {

		// キーを書き込む。末尾に : をつけて。
		string key = saveDataUnitList[i].key + ":";
		for ( int n = 0; n < key.size(); ++n ) {
			fwrite ( &key[n], sizeof(char), 1, pFile);
		}

		// 型を書き込む
		fwrite ( &saveDataUnitList[i].type, sizeof(saveDataUnitList[i].type), 1, pFile);

		// 長さを書き込む
		fwrite ( &saveDataUnitList[i].dataLength, sizeof(saveDataUnitList[i].dataLength), 1, pFile);

		// データを書き込む
		fwrite ( saveDataUnitList[i].data, saveDataUnitList[i].dataLength, 1, pFile);

		// この段階で、データは消してしまっていい？
		saveDataUnitList[i].Clear();
	}

	///// 書き込み終了。ファイルを閉じる。
	fclose(pFile);
}

//=========================================================
// ともかく読み込みます。
//=========================================================
void SaveData::ReadSaveData ( map<string, SaveDataUnit> &result ) {

	set<string> requiredKey;
	ReadSaveData (result, requiredKey);
}
void SaveData::ReadSaveData ( map<string, SaveDataUnit> &result, set<string> requiredKey ) {

	bool isCaringRequiredKeys = ( !requiredKey.empty() );
	int index = 0; // デバッグ用

	///// ファイルを開く
	FILE *pFile;
	pFile = fopen (_fileName.c_str(), "rb");

	///// ファイル、開けた？
	if (pFile != NULL) {

		bool isContinule = true;
		while ( isContinule ) {

			//printf ( "セーブデータ読み込み中...(%d)\n", index );

			///// 一件分
			SaveDataUnit unit;

			///// キーを読み込む。
			string key = "";
			while (true) {
				char c;

				isContinule = fread (&c, sizeof(char), 1, pFile);
				if ( !isContinule ) { break; }

				if ( c == ':' ) {
					break;
				} else {
					key.push_back ( c );
				}
			}
			if ( !isContinule ) { break; }

			///// 同じキーがあった？ それは変ですね。
			if ( result.find ( key ) != result.end() ) {
				printf ( "Error : 同じキーがあった。(%s)\n", key.c_str());

			}

			///// 大丈夫のようなので、キーを受け取る。
			unit.key = key;

			///// 型を読み込む
			isContinule = fread ( &unit.type, sizeof(int), 1, pFile);
			if ( !isContinule ) { break; }

			///// サイズを読み込む
			isContinule = fread ( &unit.dataLength, sizeof(int), 1, pFile);
			if ( !isContinule ) { break; }

			///// データ本体を読み込む
			if ( unit.dataLength > 0 ) {
				unit.data = new unsigned char[ unit.dataLength ];
				isContinule = fread ( unit.data, unit.dataLength, 1, pFile);
			} else {
				unit.data = NULL;
			}
			if ( !isContinule ) { break; }

			///// 確保しておく。
			result [ unit.key ] = unit;


			if ( isCaringRequiredKeys ) {


				// 要素を探索する
				if( requiredKey.find( unit.key ) != requiredKey.end() )
				{
					requiredKey.erase ( requiredKey.find( unit.key ) );

					//printf("指定のキー %s でした。\n", unit.key.c_str());
				}


				// 指定のキーを全て読み込んだらループを出る
				if ( requiredKey.empty() ) {

					//printf ( "指定のキーを全て読み込んだ。ループを出る(%d)。\n", index );
					isContinule = false;
				}
			}

			++index;
		}

		///// ファイルを閉じる
		fclose(pFile);

	} else {
		printf ("ファイルを開けませんでした（%s）\n",_fileName.c_str());
	}

}

//=========================================================
// 更新
//=========================================================
void SaveData::Update ( sf::RenderWindow &window ) {

	///// マウスの状態を監視
	ObserveMouseState ( window );
	ObserveMouseOver  ( window, _frame.getSize() );

	_frame.setPosition ( _position );
	_cursor.setPosition ( _position );

	_chapterNameTextLabel    .setPosition ( _position + sf::Vector2f( 200, 20 ) );
	_dateTextLabel           .setPosition ( _position + sf::Vector2f( 200, 60 ) );
	//_image_Date.sprite       .setPosition ( _position + sf::Vector2f( 200, 60 ) );
	_image_slotNumber .sprite.setPosition ( _position + sf::Vector2f( 5, 35 ) );
	_SS_Frame                .setPosition ( _position + sf::Vector2f( 50, 2 ) );
	_image_SS         .sprite.setPosition ( _position + sf::Vector2f( 50, 2 ) );
	_image_SS_NO      .sprite.setPosition ( _position + sf::Vector2f( 50, 2 ) + _SS_Frame.getSize() / 2.0f );


}

//=========================================================
// 描画
//=========================================================
void SaveData::Draw   ( sf::RenderWindow &window ) {

	///// 下地を表示して
	window.draw ( _frame );

	/*
		なんやかんやと中身を表示して
	*/

	window.draw ( _SS_Frame );
	if ( _image_SS.isValid ) {
		window.draw ( _image_SS.sprite );
	} else {
		window.draw ( _image_SS_NO.sprite );
	}
	window.draw ( _image_slotNumber.sprite);
	//window.draw ( _image_Date.sprite );
	window.draw ( _dateTextLabel );
	window.draw ( _chapterNameTextLabel );

	///// マウスが乗っているときには、カーソルを表示するよ。
	if ( _isMouseover ) {
		window.draw ( _cursor );
	}

}



//=========================================================
// マウスの状態を監視
//=========================================================
void SaveData::ObserveMouseState ( sf::RenderWindow &window ) {

	/*
	///// ウィンドウが最前面じゃないときには、マウス操作を受け付けない。
	HWND handle     =  window.getSystemHandle();
	HWND foreWindow = GetForegroundWindow();

	bool isTopWindow = ( handle == foreWindow );
	*/

	if ( !(Application::GetInstance()->HasFocus()) ) {

		_mouseState = MOUSE_STATE_NONE;
		return;
	}


	if ( _mouseState == MOUSE_STATE_NONE ) {

		if ( sf::Mouse::isButtonPressed ( sf::Mouse::Left ) ) {
			_mouseState = MOUSE_STATE_PUSH;
		}
	}
	else if ( _mouseState == MOUSE_STATE_PUSH ) {

		if ( sf::Mouse::isButtonPressed ( sf::Mouse::Left ) ) {
			_mouseState = MOUSE_STATE_PRESS;
		} else {
			_mouseState = MOUSE_STATE_RELEASE;
		}
	}
	else if ( _mouseState == MOUSE_STATE_PRESS ) {

		if ( !sf::Mouse::isButtonPressed ( sf::Mouse::Left ) ) {
			_mouseState = MOUSE_STATE_RELEASE;
		}
	}
	else if ( _mouseState == MOUSE_STATE_RELEASE ) {

		_mouseState = MOUSE_STATE_NONE;
	}

}

//=========================================================
// マウスが乗ってるかどうかを監視
//=========================================================
void SaveData::ObserveMouseOver (  sf::RenderWindow &window, sf::Vector2f size ) {

	///// マウス、乗ってる？
	int mouseX = sf::Mouse::getPosition ( window ).x;
	int mouseY = sf::Mouse::getPosition ( window ).y;
	
	SFML_Utils::PosToScreenPos(mouseX, mouseY, window);
	
	if
	(
		_position.x <= (float)mouseX && (float)mouseX <= _position.x + size.x  &&
		_position.y <= (float)mouseY && (float)mouseY <= _position.y + size.y
	)
	{
		_isMouseover = true;
	} else {
		_isMouseover = false;
	}
}




//=========================================================
// 指し手構造体<-->文字列：
//=========================================================
string SaveData::MoveToString ( AI_Modules::Move &move) {

	string result = "";

/*
	struct Move {

		int turn; // 黒が？ 白が？

		// 何をする？ 駒を打つ・カードを置く・駒を動かす・カードを取る
		int action;

		// 駒を打つ場合：どこへ？
		int putPieceX;
		int putPieceY;

		// カードを置く場合：どのカードを？ どこへ？ どの向きに？
		int putCardId;
		int putCardX;
		int putCardY;
		int putCardDirection;

		// 駒を動かす場合：どこから？ どこへ？ ぶつける？
		int movePieceFromX;
		int movePieceFromY;
		int movePieceToX;
		int movePieceToY;
		int movePieceAndCllidedPieceColor; // GRID 「エリア」で解釈する場合に必要。直視する場合は不要。

		// カードを取る場合：どこの？
		int takeCardX;
		int takeCardY;
		int takeCardCardId; // 「エリア」で解釈する場合に必要。直視する場合には不要。

		bool movePieceAndIsCllide;
	}
*/
	string delim = ",";

	result += std::to_string ( move.turn ) + delim;
	result += std::to_string ( move.action ) + delim;
	result += std::to_string ( move.putPieceX ) + delim;
	result += std::to_string ( move.putPieceY ) + delim;
	result += std::to_string ( move.putCardId ) + delim;
	result += std::to_string ( move.putCardX ) + delim;
	result += std::to_string ( move.putCardY ) + delim;
	result += std::to_string ( move.putCardDirection ) + delim;
	result += std::to_string ( move.movePieceFromX ) + delim;
	result += std::to_string ( move.movePieceFromY ) + delim;
	result += std::to_string ( move.movePieceToX ) + delim;
	result += std::to_string ( move.movePieceToY ) + delim;
	result += std::to_string ( move.movePieceAndCllidedPieceColor ) + delim;
	result += std::to_string ( move.takeCardX ) + delim;
	result += std::to_string ( move.takeCardY ) + delim;
	result += std::to_string ( move.takeCardCardId ) + delim;
	result += ( move.movePieceAndIsCllide ? "true" : "false" ) ;


	return result;
}

//=========================================================
// 指し手構造体<-->文字列：
//=========================================================
AI_Modules::Move SaveData::MoveFromString ( string data) {

	AI_Modules::Move result;

	if ( data == "") {
		return result;
	}

	vector<string> dataList = Utils_String::Split ( data, "," );

	if ( dataList.size() < 17 ) {
		printf ( "セーブデータが壊れている：AI_Modules::Move SaveData::MoveFromString ( %s )\n", data.c_str() );
	} else {

		result.turn                          = std::stoi ( dataList[0] );
		result.action                        = std::stoi ( dataList[1] );
		result.putPieceX                     = std::stoi ( dataList[2] );
		result.putPieceY                     = std::stoi ( dataList[3] );
		result.putCardId                     = std::stoi ( dataList[4] );
		result.putCardX                      = std::stoi ( dataList[5] );
		result.putCardY                      = std::stoi ( dataList[6] );
		result.putCardDirection              = std::stoi ( dataList[7] );
		result.movePieceFromX                = std::stoi ( dataList[8] );
		result.movePieceFromY                = std::stoi ( dataList[9] );
		result.movePieceToX                  = std::stoi ( dataList[10] );
		result.movePieceToY                  = std::stoi ( dataList[11] );
		result.movePieceAndCllidedPieceColor = std::stoi ( dataList[12] );
		result.takeCardX                     = std::stoi ( dataList[13] );
		result.takeCardY                     = std::stoi ( dataList[14] );
		result.takeCardCardId                = std::stoi ( dataList[15] );
		result.movePieceAndIsCllide          = Utils_String::isPositiveWord ( dataList[16] );

	}

	return result;
}

//=========================================================
// 指し手構造体<-->文字列：
//=========================================================
string SaveData::MoveToString_List ( vector<AI_Modules::Move> &moveList) {

	string result = "";

	for ( unsigned int i = 0; i < moveList.size(); ++i ) {

		if ( i > 0 ) {
			result += "<>";
		}

		result += MoveToString ( moveList[i] );
	}
	printf ("指し手構造体<-->文字列 : %s \n", result.c_str());
	return result;
}

//=========================================================
// 指し手構造体<-->文字列：
//=========================================================
vector<AI_Modules::Move> SaveData::MoveFromString_List ( string data) {

	vector<AI_Modules::Move> result;

	for ( string moveString : Utils_String::Split(data, "<>") ) {

		AI_Modules::Move move = MoveFromString ( moveString );
		result.push_back ( move );
	}

	return result;
}

//=========================================================
// AI_Modules::ProgressData <--> 文字列 ：
//=========================================================
string SaveData::ProgressDataToString ( AI_Modules::ProgressData &progressData ) {

/*
	struct ProgressData {

		float          progressRate;    // 0〜100 %（負の数は開始前を表すとする）
		vector<string> messageLogs;     // 出来事を随時、追加したりしなかったり。
		Move           answerMove;      // 回答（進捗 100% のとき、意味をなす）
		string         evaluationSumUp; // 外部向けの評価値  EVALUATION::SUM_UP

		bool           isStop;          // これが true になっているのを確認すると、そこで思考を止める。

		ProgressData () {
			progressRate = -1.0f;
			isStop = false;
			evaluationSumUp = EVALUATION::SUM_UP::EVEN;
		}
	};
*/
	string result = "";

	string delim = "<*>";

	string messageLog = "";
	for ( unsigned int i = 0; i < progressData.messageLogs.size(); ++i ) {
		if ( i > 0 ) { messageLog += "<@>"; }
		messageLog += progressData.messageLogs[i];
	}

	result += std::to_string ( progressData.progressRate ) + delim;
	result += messageLog + delim;
	result += MoveToString ( progressData.answerMove ) + delim;
	result += progressData.evaluationSumUp + delim;
	result += ( progressData.isStop ? "true" : "false" );

	return result;
}

//=========================================================
// AI_Modules::ProgressData <--> 文字列 ：
//=========================================================
AI_Modules::ProgressData SaveData::ProgressDataFromString ( string data ) {

	//printf ( "セーブデータが壊れていないかな？？？：SaveData::ProgressDataFromString ( %s )\n ", data.c_str() );

	AI_Modules::ProgressData result;

	vector<string> dataList = Utils_String::Split( data, "<*>" );

	if ( dataList.size() < 5 ) {
		printf ( "セーブデータが壊れています：AI_Modules::ProgressData SaveData::ProgressDataFromString ( %s ) \n", data.c_str() );
	} else {

		result.progressRate    = std::stof(dataList[0]);
		if ( dataList[1] != "" ) { result.messageLogs     = Utils_String::Split ( dataList[1], "<@>"); }
		result.answerMove      = MoveFromString ( dataList[2] );
		result.evaluationSumUp = dataList[3];
		result.isStop          = Utils_String::isPositiveWord(dataList[4] );
	}


	return result;
}














