//=========================================================
//
// セーブデータ。１件分を表す。
//
//=========================================================

#ifndef _SAVE_DATA_
#define _SAVE_DATA_

#include "SFML/Graphics.hpp"
#include <string.h>
#include <map>

#include "../GUI_SFML/TextMessageWindow.h"
#include "BGM.h"
#include "Camera.h"
#include "SoundManager.h"
#include "Image.h"
#include "Select.h"
#include "ScriptButton.h"
#include "ClickableImage.h"
#include "ScriptEngine.h"
#include "TextLabel.h"
#include "GameManager.h"

using namespace std;


class SaveData {

public:

	//=========================================================
	// 定数：データの型・セーブ対象としての
	//=========================================================
	static const int SAVE_DATA_TYPE_NONE;
	static const int SAVE_DATA_TYPE_INT;
	static const int SAVE_DATA_TYPE_STRING;

	//=========================================================
	// セーブする情報の最小単位
	//=========================================================
	struct SaveDataUnit {

		string key;  // キーは半角英数字とする。前後に<>をつけるのは、こちら側での処理とする。使う側は気にしない。
		int    type;
		int    dataLength;
		unsigned char   *data;

		///// コンストラクタ
		SaveDataUnit () {
			key        = "";
			type       = SAVE_DATA_TYPE_NONE;
			dataLength = 0;
			data       = NULL;
		}
		SaveDataUnit ( string key, int value ) {
			data = NULL;
			MakeInto_Int( key, value );
		}
		SaveDataUnit ( string key, string value ) {
			data = NULL;
			MakeInto_String( key, value );
		}

		///// 後始末：呼んでね。
		void Clear() {
			delete[] data;
			data = NULL;
		}

		///// Int型のデータを入れる。
		void MakeInto_Int( string key_arg, int value ) {

			// 先客は消しますよ？ いいですね？
			delete[] data;
			data = NULL;

			key        = key_arg;
			type       = SAVE_DATA_TYPE_INT;
			dataLength = sizeof(int);
			data       = new unsigned char[ dataLength ];

			memcpy( data,  &value, sizeof(int) );
		}

		///// String型のデータを入れる。
		void MakeInto_String( string key_arg, string value ) {

			// 先客は消しますよ？ いいですね？
			delete[] data;
			data = NULL;

			key        = key_arg;
			type       = SAVE_DATA_TYPE_STRING;
			dataLength = value.size();

			if ( dataLength == 0 ) {

				data = NULL;

			} else {

				data       = new unsigned char[ dataLength ];

				for ( int i = 0; i < dataLength; ++i ) {

					data[i] = value[i];
				}
			}
		}

		///// 数値型だとして、取得
		int GetData_as_Int () {
			
			int result = 0;

			if ( type == SAVE_DATA_TYPE_INT && data != NULL ) {
				memcpy( &result, data,sizeof(int));
			}

			return result;
		}
		///// 文字列型だとして、取得
		string GetData_as_String () {

			string result = "";

			if ( type == SAVE_DATA_TYPE_STRING && data != NULL) {

				for ( int i = 0; i < dataLength; ++i ) {
					result.push_back ( data[i] );
				}
			}
			return result;
		}
	};

	//=========================================================
	// コンストラクタ
	// ファイル名を必ず受け取るのです。
	//=========================================================
	SaveData( string fileName, int slotNum );

	//=========================================================
	// デストラクタ
	//=========================================================
	~SaveData();

	//=========================================================
	// 読み込み
	// ファイルを読み込んで内容を受け取る。
	// もともと持っていた変数の値とかは、消される。
	//=========================================================
	void Load();

	//=========================================================
	// 読み込み
	// 見た目部分だけ
	//=========================================================
	void LoadAppearance();

	//=========================================================
	// 書き込み
	// ファイルに書き込む。
	// ファイルのもともとの内容は、消される。
	//=========================================================
	void Save();

	//=========================================================
	// セーブ（このSaveDataオブジェクトに、状態を）
	//=========================================================
	void Save_ScriptProgess       ( ScriptEngine *scriptEngine );
	void Save_LoclVariable_Int    ( map <string, int>    &variable_Local_Int    );
	void Save_LoclVariable_String ( map <string, string> &variable_Local_String );
	void Save_TextMessageWindow   ( GUI_SFML::TextMessageWindow *textMessageWindow );
	void Save_BGM                 ( BGM *bgm  );
	void Save_Camera              ( Camera *camera );
	void Save_Image               ( vector < Image*> &imageList );
	void Save_Button              ( map < string, ScriptButton*> &scriptButtonList );
	void Save_ClickableImage      ( map<string, ClickableImage*> &clickableImageList );
	void Save_Select              ( Select *select );
	void Save_SoundManager        ( SoundManager *soundManager );
	void Save_TextLabelList       ( map < string, TextLabel*> &textLabelList);
	void Save_GameManagerList     ( map < string, GameManager*> &gameManagerList);
	void Save_WaitClickableImageDataList ( ScriptEngine *scriptEngine );
	void Save_WaitEventDataList          ( ScriptEngine *scriptEngine );
	void Save_WaitGameOperationDataList  ( ScriptEngine *scriptEngine );
	void Save_ScriptEngineMiscellaneous  ( ScriptEngine *scriptEngine );
	
	//=========================================================
	// ロード（この SaveDataオブジェクトから、状態を）
	//=========================================================
	void Load_ScriptProgess       ( ScriptEngine *scriptEngine );
	void Load_LoclVariable_Int    ( map <string, int>    &variable_Local_Int    );
	void Load_LoclVariable_String ( map <string, string> &variable_Local_String );
	void Load_TextMessageWindow   ( GUI_SFML::TextMessageWindow *textMessageWindow );
	void Load_BGM                 ( BGM *bgm  );
	void Load_Camera              ( Camera *camera  );
	void Load_Image               ( vector < Image*> &imageList );
	void Load_Button              ( map < string, ScriptButton*> &scriptButtonList );
	void Load_ClickableImage      ( map<string, ClickableImage*> &clickableImageList );
	void Load_Select              ( Select *select );
	void Load_SoundManager        ( SoundManager *soundManager );
	void Load_TextLabelList       ( map < string, TextLabel*> &textLabelList, sf::Font &font );
	void Load_GameManagerList     ( map < string, GameManager*> &gameManagerList, ScriptEngine *scriptEngine);
	void Load_WaitClickableImageDataList ( ScriptEngine *scriptEngine);
	void Load_WaitEventDataList          ( ScriptEngine *scriptEngine );
	void Load_WaitGameOperationDataList  ( ScriptEngine *scriptEngine );
	void Load_ScriptEngineMiscellaneous  ( ScriptEngine *scriptEngine );

	//=========================================================
	// 描画機能があるのです。
	//=========================================================
	void Update ( sf::RenderWindow &window );
	void Draw   ( sf::RenderWindow &window );

	//=========================================================
	// 位置は外部からいじられることになってるんですよね？ いいですよ。わかってます。
	//=========================================================
	void SetPosition ( sf::Vector2f pos ) { _position = pos; }
	void SetPositionX ( int x ) { _position.x = x; }
	void SetPositionY ( int y ) { _position.y = y; }


	//=========================================================
	// 選択された？ あえて「クリック」とはいわないでみる。それは操作の形式に過ぎないから。
	//=========================================================
	//bool IsSelected() { return ( _mouseState == MOUSE_STATE_RELEASE && _isMouseover ); }
	
	bool IsMouseReleased() { return ( _mouseState == MOUSE_STATE_RELEASE && _isMouseover ); }
	bool IsMousePushed  () { return ( _mouseState == MOUSE_STATE_PUSH    && _isMouseover ); }

	///// n番にセーブします、とか言いたいですよね。
	int GetSlotNum() { return _slotNum; }

	//=========================================================
	// SSファイル名
	//=========================================================
	string GetSSFileName();

	//=========================================================
	// SS 更新
	//=========================================================
	void UpdateSS();
	
	//=========================================================
	// 扱うファイルを変更する
	//=========================================================
	void ReplaceFileName ( string fileName );
	
	string GetFileName () { return _fileName; }
	
	//=========================================================
	// 有効なセーブデータか？
	//=========================================================
	bool IsValid() { return _isValid; }


	//=========================================================
	// 指し手構造体<-->文字列 ：あくまでもセーブ/ロードの目的ということ
	//=========================================================
	static string MoveToString ( AI_Modules::Move &move);
	static AI_Modules::Move MoveFromString ( string data );
	static string MoveToString_List ( vector<AI_Modules::Move> &moveList);
	static vector<AI_Modules::Move> MoveFromString_List ( string data);
	
	
	//=========================================================
	// AI_Modules::ProgressData <--> 文字列 ：あくまでもセーブ/ロードの目的ということ
	//=========================================================
	static string ProgressDataToString ( AI_Modules::ProgressData &progressData );
	static AI_Modules::ProgressData ProgressDataFromString ( string data );
	
	
	void SetChapterName ( string chapterName );
	string GetChapterName () { return _chapterName; }
	
private:

	struct Image_set {

		sf::Image   image;
		sf::Texture texture;
		sf::Sprite  sprite;
		bool isValid; // 必要なときだけ使う。いつでもアテになる値が入っているわけではない。

		void Update() {
			texture.loadFromImage(image);
			sprite = sf::Sprite();
			sprite.setTexture(texture);
		}
	};

	///// 有効なセーブデータ？
	bool _isValid;

	///// 表示位置。左上。
	sf::Vector2f _position;

	///// バインド？ わかちがたく関連づけられるファイルのファイル名。ファイルのファイル名。
	string _fileName;
	int    _slotNum;

	///// セーブデータとしての顔。表示用の部品としても持つわけですよ。
	sf::RectangleShape _frame;     // 表示枠。
	sf::RectangleShape _cursor;    // カーソル
	sf::RectangleShape _SS_Frame;  // スクリーンショット画像の下地。主眼は枠。SSなしのときには背景。

	sf::Vector2f _ss_Size;

	Image_set _image_SS;           // スクリーンショット画像。
	Image_set _image_SS_NO;        // スクリーンショット画像なし
	Image_set _image_slotNumber;   // スロット番号
	//Image_set _image_Caption;      // 見出し。章のタイトルなど。
	Image_set _image_ReminderText; // 直前のテキスト。
	//Image_set _image_Date;         // 日付
	string _date;
	//string _year, _month, _day, _hour, _min, _sec;
	
	string _chapterName;
	sf::Text _chapterNameTextLabel;
	sf::Text _dateTextLabel;


	///// セーブする中身
	map < string, int>    _saveData_variable_Int;       // ローカル変数：数値
	map < string, string> _saveData_variable_String;    // ローカル変数：文字列

	GUI_SFML    ::TextMessageWindow::SaveData _saveData_TextMessageWindow;
	BGM         ::SaveData                    _saveData_BGM;
	SoundManager::SaveData                    _saveData_SoundManager;
	Camera      ::SaveData                    _saveData_Camera;
	Select      ::SaveData                    _saveData_Select;
	vector < Image::SaveData >                _saveData_ImageList;
	vector < ScriptButton::SaveData   >       _saveData_ButtonList; 
	vector < ClickableImage::SaveData >       _saveData_ClickableImageList; 
	ScriptEngine::SaveData_ScriptProgress     _saveData_ScriptProgress;
	string                                    _saveData_waitClickableImageDataList;
	string                                    _saveData_waitEventDataList;
	string                                    _saveData_waitGameOperationDataList;
	vector < TextLabel::SaveData >            _saveData_textLabelList;
	vector < GameManager::SaveData >          _saveData_GameManagerList;
	string                                    _saveData_ScriptEngineMiscellaneous;

	 // 再開ポイント：どのプロセルの、どのユニット。と、何文字目？
		/*
			セーブされる可能性のあるユニット、は限定されている。
			・テキスト
		*/

	/*
		画面に表示していた部品の状態、も再現すべき対象ですよね。
		画像は、どうしてたか、とか。

		GUI_SFML::TextMessageWindow *_textMessageWindow; // テキスト欄
		vector < Image* > _imageList; 
		BGM *_BGM;
		SoundManager *_soundManager; // 効果音はセーブしなくていいかも。
		Select       *_select;

		これらのオブジェクトの状態を、保存・再現。
		BGMの「どの箇所から」までは、再現しなくてもいいけど、何を「演奏中」または「演奏停止中」は再現したい。


	*/


	///// マウスの状態。なんか同じパターンですよねー。うんうん。あ、聞き流しましたね。
	int _mouseState;
	bool _isMouseover;

	//=========================================================
	// 内部定数：マウスの状態。
	//=========================================================
	static const int MOUSE_STATE_NONE;    // 押していない
	static const int MOUSE_STATE_PUSH;    // 押したところ
	static const int MOUSE_STATE_PRESS;   // 押している
	static const int MOUSE_STATE_RELEASE; // 放したところ

	//=========================================================
	// マウスの状態を監視
	//=========================================================
	void ObserveMouseState ( sf::RenderWindow &window );

	//=========================================================
	// マウスが乗ってるかどうかを監視
	//=========================================================
	void ObserveMouseOver (  sf::RenderWindow &window, sf::Vector2f size );


	//=========================================================
	// 書き込める形式で書き込む
	//=========================================================
	void WriteSaveData ( vector<SaveDataUnit> &saveDataUnit );

	//=========================================================
	// ともかく読み込みます。
	//=========================================================
	void ReadSaveData ( map<string, SaveDataUnit> &result );
	void ReadSaveData ( map<string, SaveDataUnit> &result, set<string> requiredKey );


	//=========================================================
	// 見た目をセットアップ
	//=========================================================
	void SetupAppearance();
	

	
	
	
	
	
	
	
	
};

#endif

