//=========================================================
//
// スクリプトエンジン
//
//=========================================================

#ifndef _SCRIPT_ENGINE_
#define _SCRIPT_ENGINE_

#include <SFML/Graphics.hpp>
#include <SFML/Audio.hpp>

#include "../GUI_SFML/TextMessageWindow.h"

#include "ScriptProcess.h"
#include "../Utils/NotXML.h"

#include <map>
#include <vector>

#include <IScenarioLogger.h>

using namespace std;

class Image;
class BGM;
class SoundManager;
class Menu;

class SelectOption;
class Select;

class SaveData;
class GlobalSaveData;
class ReadTextManager;
class ScriptButton;
class TextLabel;

class GameManager;
class Effect_CurtainClose;

class ClickableImage;
class Fade;

class Camera;

class FreeBattleSelector;
class ResignButton;

// 動作テスト
class ScoreSheetViewer;
//class ScenarioLogViewer;
#include "../GUI_SFML/ScenarioLogViewer.h"
#include "../GUI_SFML/FileExplorer.h"

using namespace GUI_SFML;

#include "../AI/Move.h"

class ScriptEngine : public GUI_SFML::IReadManagerReceptionist, public IScenarioLogger {

public:

	// 動作テスト
	//ScoreSheetViewer *_scoreSheetViewer;
	//ScenarioLogViewer *_scenarioLogViewer;
	//FileExplorer *_fileExplorer;

	//=========================================================
	// イベント監視情報
	//=========================================================
	struct WatchEventData {

		string           type;        // GameMove / Button
		AI_Modules::Move move_Before; // イベント監視開始時点での最終着手。
		AI_Modules::Move move;
		int              moveAction;
		string           isCollide; // true. false, 空文字で設定なし、とする。
		string           gameManagerName;
		string           buttonName;

		string jumpToLabel;
		int    count;

		WatchEventData() {
			type            = "";
			buttonName      = "";
			gameManagerName = "";
			jumpToLabel     = "";
			count           = 1;
			moveAction = -1;
			isCollide  = "";
		}
	};

	//=========================================================
	// ゲーム盤の操作イベント監視情報
	//=========================================================
	struct WatchGameOperationData {

		AI_Modules::Move move_Before; // イベント監視開始時点での最終着手。
		int              scoreLength_Before; // これまでの手数
		AI_Modules::Move move;
		int              moveAction;
		string           isCollide; // true. false, 空文字で設定なし、とする。
		string           gameManagerName;

		string jumpToLabel;
		int    count;

		WatchGameOperationData() {
			gameManagerName = "";
			jumpToLabel     = "";
			count           = 1;
			moveAction = -1;
			isCollide  = "";
			scoreLength_Before = 0;
		}
	};


	//=========================================================
	// クリック画像監視情報
	//=========================================================
	struct WatchClickableImageData {

		string cImageName;    // この名前のクリック画像がクリックされたら
		string jumpToLabel;   // このラベルにジャンプする

		WatchClickableImageData() {
			cImageName  = "";
			jumpToLabel = "";
		}
	};

	//=========================================================
	// 試合情報
	//=========================================================
	struct BattleData {

		static const int PHASE_BATTLE_NOT_STARTED   = 0; // 試合中ではない。
		static const int PHASE_BEFORE_ACTOR_APPEARS = 1; // カーテンを閉める：閉まりきったら、アクター登場。
		static const int PHASE_ACTOR_APPEARED       = 2; // カーテンを開ける：開け始めると同時に試合開始
		static const int PHASE_BATTLING             = 3; // 試合中。
		static const int PHASE_AFTER_BATTLE         = 4; // 試合後。
		static const int PHASE_BEFORE_ACTOR_LEAVES  = 5; // カーテンを閉める：閉まりきったら、アクター退場。
		static const int PHASE_ACTOR_LEFT           = 6; // カーテンを開ける：開け始めたら、試合処理を終了。

		string gameManagerName;

		string playerSide;  // プレイヤーはどっち？ P1？ P2？ 勝ち負けの意味を左右するよ。

		// 試合結果に応じてジャンプする先：これは、変数ごと分けるんじゃなくもっと柔軟に？
		string jumpTo_WIN;
		string jumpTo_LOSE;
		string jumpTo_DRAW;

		// 試合結果を入れる変数名：空文字のときは、試合結果を格納しない。
		// 該当の変数が宣言されていないときは、宣言する。
		string resultVariable;

		// 試合中は非表示にしておく画像のラベルリスト
		vector<string> hideImageLabelList;

		// 試合後に余韻を残すフレームカウント
		int frameCount_AfterBattle;

		string curtainEffect_begin;  // カーテンエフェクト：試合開始時
		string curtainEffect_end;    // カーテンエフェクト：試合終了時
		string curtainEffect_after;  // カーテンエフェクト：試合終了後

		// フェイズ
		int phase;

		BattleData() {
			playerSide      = "P1";
			gameManagerName = "";
			resultVariable  = "";
			jumpTo_WIN      = "";
			jumpTo_LOSE     = "";
			jumpTo_DRAW     = "";
			phase           = PHASE_BATTLE_NOT_STARTED;
			frameCount_AfterBattle = 60;

			curtainEffect_begin = "";
			curtainEffect_end   = "";
			curtainEffect_after = "";
		}
	};

	//=========================================================
	// コンストラクタ / デストラクタ
	//=========================================================
	ScriptEngine();
	~ScriptEngine();

	//=========================================================
	// 初期設定：ルートプロセスを含むスクリプトファイルを指定
	// ここで指定されたファイルの先頭に書かれているプロセスをルート扱い、でよいのかも。
	//
	// じゃなくて、スクリプトファイルが置かれているフォルダを指定する。
	//=========================================================
	void Init ( string scriptDir );

	void Update ( sf::RenderWindow &window );
	void Draw   ( sf::RenderWindow &window );

	bool IsRendered() { return _isRendered; }

	void SaveTo   ( SaveData *saveData, bool isTakeSS = true, string message = "セーブしました" ); // SSを撮るかどうか
	void LoadFrom ( SaveData *saveData, string message = "ロードしました" );
	void LoadPreparation();


	//=========================================================
	// セーブ/ロード用のインターフェイス  スクリプトの進行状況
	//=========================================================
	struct SaveData_ScriptProgress {

		/*
			プロセスは、必ず、現時点のプロセスを最下層とする、入れ子構造になっているはず。
		*/

		vector < ScriptProcess_SaveData > processSaveDataList;
		//string currentProcessLabel; // 先頭の子がカレント。
		int    state;

	};
	SaveData_ScriptProgress GetSaveData_ScriptProgress();
	void                    Load_ScriptProgress ( SaveData_ScriptProgress &saveData_ScriptProgress );


	//=========================================================
	// セーブ/ロード用のインターフェイス：イベント待ち受け情報
	//=========================================================
	string Serialize_WatchClickableImageData();
	void   DeSerialize_WatchClickableImageData( string str );
	string Serialize_WatchEventData();
	void   DeSerialize_WatchEventData( string str );
	string Serialize_WatchGameOperationData();
	void   DeSerialize_WatchGameOperationData( string str );

	//=========================================================
	// セーブ/ロード用のインターフェイス：システム部品というかその他というか、入れ忘れというか。
	//=========================================================
	string Serialize_Miscellaneous ();
	void   DeSerialize_Miscellaneous ( string str );


	//=========================================================
	// 既読管理への窓口係りとして。
	//=========================================================
	int  GetReadFinishedLength ();
	bool IsSkip();
	bool IsWannaSkip();
	void Finished ();
	void Finished ( int charLength );
	void SkipEnd();
	void SkippingNow(bool arg);
	bool _isSkippingNow;


	//=========================================================
	// 読み返しログ関連
	//=========================================================
	void   WriteScenarioLog (string log);
	string ReadScenarioLog ();
	void   ClearScenarioLog();


	//=========================================================
	// 全てのプロセスが終了している？
	//=========================================================
	int IsAllProcessEnd() { return (_state == STATE_NOT_ACTIVE); }


	void  SetVolumeBGM ( float value );
	float GetVolumeBGM ();
	void  SetVolumeSE ( float value );
	float GetVolumeSE ();

	void SaveConfig();
	void LoadConfig();


	sf::Font* GetFont() { return &_font; }

	// あれ？ 何これ？ GameManager側でActorをDeSerializeするときのための裏口。
	static GUI_SFML::TextMessageWindow *textMessageWindowParaTi;

	// もうね、
	GameManager* GetGameManager ( string name ) { return GameManager_Get(name); }
	
	
	// フリーバトルが解放されているかどうか。フリーバトル専用のドロナワメソッド。
	bool IsUnlockedFreeBattle ( string variableName );
	
	
	///// システムメッセージ
	void SystemMessage ( string message );
	
private:

	//map < string, ScriptProcess* > _processList;
	//string _currentProcessLabel;
	ScriptProcess *_currentProcess;

	///// 現在の状態
	int _state;

	///// プロセスのラベルと、定義されているファイル名の対応表。
	map < string, string> _processLabel_ScriptFileName_Table;

	///// 描画を１フレーム休む。（つまり、ロード直後、各オブジェクトの表示位置が狂っているとき、一周回ってUpdateをかけてからにしたい）
	//bool _isRestDraw;

	bool _isRendered;

	///// セーブデータ用にSSを撮影して欲しい
	SaveData *_takeSSforThisSaveData;

	// ファイル名を入れておくと、次回の描画時に SS を撮って保存してくれる。windowがないと撮影できないとかなんとか。
	string    _takeSStoThisFile;

	///// 少し待ちたいときがある。
	int _waitFrame;

	///// 画像ファイル名のエイリアス集
	map <string, string> _imageFileNameAliases;


	///// 読み返しログ関連
	string _scenarioLog;

	///// ファイルを選択したときに結果を入れる変数、の変数名。ローカル文字列変数を使うよ。
	string _fileSelectVariableName;

	// TODO これをテキストウィンドウのフォントとして使えてる？ TextImageFactory に文字列を渡してますよね？ フォントファイルの
	// TextLabelではこれを使っている。
	sf::Font _font;

	// ウィンドウがフォーカスを持っているかどうか。windowがないと判定できないというのでは不便なので、更新時に判定して持ち回る。
	bool _hasFocus;

	// アクターさんに通知を出した後ですよ
	string _actorReactionState = "";
	
	// チャプター名
	string _chapterName;

	//=========================================================
	// スクリプトエンジンとして標準装備している部品
	//=========================================================
	GUI_SFML::TextMessageWindow *_textMessageWindow; // テキスト欄
	ReadTextManager             *_readTextManager;   // 既読管理クラス
	vector < Image* >            _imageList;         // 画像リスト
	map < string, TextLabel*>    _textLabelList;
	map < string, ScriptButton*> _scriptButtonList;
	BGM                         *_BGM;
	SoundManager                *_soundManager;
	Select                      *_select;
	Menu                        *_menu;
	Fade                        *_fade;
	map<string, GameManager*>    _gameManagerList;
	map<string, ClickableImage*> _clickableImageList;  // クリック画像リスト
	FileExplorer                *_fileExplorer; // ファイル選択さん
	ClickableImage              *_endAstralProjectionButton; // 検討モード終了ボタン
	Camera                      *_camera;
	Image                       *_skipIcon;          // 既読スキップアイコン
	Image                       *_skipIcon_inactive; // 既読スキップアイコン（非アクティブ）
	FreeBattleSelector          *_freeBattleSelector;
	ResignButton                *_resignButton; // 投了ボタン

	map<string, GUI_SFML::TextMessageWindow::Design> _textMessageWindowDesignList; // テキスト欄のデザイン集
	string _currentTextWindowDesignID; // 今使っているテキスト欄のデザインID

	void SaveTextWindowDesignAs ( string designID );
	void LoadTextWindowDesign ( string designID );

	//=========================================================
	// 変数
	//【メモ】
	// ここでいうグローバルというのは、セーブデータ間で共有するという意味。
	// 当該のセーブデータ内のみをスコープする変数はローカルと呼ばれる。
	//=========================================================
	map < string, int >    _variable_Local_Int;
	map < string, string > _variable_Local_String;
	map < string, int >    _variable_Global_Int;
	map < string, string > _variable_Global_String;

	///// セーブデータ間で共有する変数を保存/再現するための部品
	GlobalSaveData *_globalSaveData;
	
	///// 変数をコンソールに表示
	void ShowVariablesToConsole();

	//=========================================================
	// イベント監視関連
	//=========================================================
	vector<WatchEventData>          _watchEventDataList;
	vector<WatchGameOperationData>  _watchGameOperationDataList;
	vector<WatchClickableImageData> _watchClickableImageDataList;

	//=========================================================
	// 状態を表す定数
	//=========================================================
	static const int STATE_NOT_ACTIVE;            // 停止中。
	static const int STATE_WAIT_UNTIL_UNIT_END;   // 現在のユニットの処理が終わるのを待っている（実行中）
	static const int STATE_CURRENT_UNIT_FINISHED; // 現在のユニットの処理は終了している。（次のユニットを実行せよ）
	static const int STATE_RESUME;                // 現在のユニットを開始処理からやり直す

	//=========================================================
	// スクリプト進行
	//=========================================================
	void Update_ProgressScript();

	//=========================================================
	// サブルーチンを呼び出す
	//=========================================================
	void Gosub ( string processLabel );

	//=========================================================
	// プロセスタグとスクリプトファイル名の対応表を作成
	//=========================================================
	void MakeProcessIndex ( string scriptDir );

	//=========================================================
	// スクリプトファイルを読み込む
	//=========================================================
	//void ReadScriptFile ( string fileName );

	//=========================================================
	// ラベルを指定して、Process を読み込む
	//=========================================================
	ScriptProcess* ReadProcess ( string label );

	//=========================================================
	// プロセスタグを解釈
	//=========================================================
	ScriptProcess* ReadProcessTag ( NotXML::Tag &processTag );


	//=========================================================
	// ユニット開始処理
	//=========================================================
	void UnitStart ( ScriptUnit &unit );
	void UnitStart_DefineTextWindow   ( ScriptUnit &unit );
	void UnitStart_TextWindowDesign   ( ScriptUnit &unit );
	void UnitStart_Text               ( ScriptUnit &unit );
	void UnitStart_TextAlias_Regist   ( ScriptUnit &unit );
	void UnitStart_Protagonist_Regist ( ScriptUnit &unit );
	void UnitStart_Skipping           ( ScriptUnit &unit );
	void UnitStart_Print              ( ScriptUnit &unit );
	void UnitStart_SystemMessage      ( ScriptUnit &unit );
	void UnitStart_Image              ( ScriptUnit &unit );
	void UnitStart_Image_Regist       ( ScriptUnit &unit );
	void UnitStart_ImageGroup         ( ScriptUnit &unit );
	void UnitStart_BGM_Regist         ( ScriptUnit &unit );
	void UnitStart_BGM_Change         ( ScriptUnit &unit );
	void UnitStart_BGM_Config         ( ScriptUnit &unit );
	void UnitStart_Sound_Regist       ( ScriptUnit &unit );
	void UnitStart_Sound              ( ScriptUnit &unit );
	void UnitStart_TextLabel          ( ScriptUnit &unit );
	void UnitStart_Select             ( ScriptUnit &unit );
	void UnitStart_Variable_Define    ( ScriptUnit &unit );
	void UnitStart_Variable_Release   ( ScriptUnit &unit );
	void UnitStart_Variable_Operation ( ScriptUnit &unit );
	void UnitStart_Variable_Console   ( ScriptUnit &unit );
	void UnitStart_IF                 ( ScriptUnit &unit );
	void UnitStart_Switch             ( ScriptUnit &unit );
	void UnitStart_Gosub              ( ScriptUnit &unit );
	void UnitStart_Button             ( ScriptUnit &unit );
	void UnitStart_Game               ( ScriptUnit &unit );
	void UnitStart_Game_Release       ( ScriptUnit &unit );
	void UnitStart_Move               ( ScriptUnit &unit );
	void UnitStart_Put                ( ScriptUnit &unit );
	void UnitStart_Remove             ( ScriptUnit &unit );
	void UnitStart_RemoveAll          ( ScriptUnit &unit );
	void UnitStart_WatchEvent         ( ScriptUnit &unit );
	void UnitStart_WatchGameOperation ( ScriptUnit &unit );
	void UnitStart_Jump               ( ScriptUnit &unit );
	void UnitStart_Wait               ( ScriptUnit &unit );
	void UnitStart_WaitClick          ( ScriptUnit &unit );
	void UnitStart_Battle             ( ScriptUnit &unit );
	void UnitStart_AstralProjection   ( ScriptUnit &unit );
	void UnitStart_Fade               ( ScriptUnit &unit );
	void UnitStart_Shake              ( ScriptUnit &unit );
	void UnitStart_Camera             ( ScriptUnit &unit );
	void UnitStart_Title              ( ScriptUnit &unit );
	void UnitStart_Chapter            ( ScriptUnit &unit );
	void UnitStart_Save               ( ScriptUnit &unit );
	void UnitStart_Load               ( ScriptUnit &unit );
	void UnitStart_WriteSaveData      ( ScriptUnit &unit );
	void UnitStart_FileSelect         ( ScriptUnit &unit );
	void UnitStart_ClickableImage     ( ScriptUnit &unit );
	void UnitStart_ClearClickableImage( ScriptUnit &unit );
	void UnitStart_DefineControlTag   ( ScriptUnit &unit );
	void UnitStart_WaitForClickableImages ( ScriptUnit &unit );
	void UnitStart_FreeBattleSelector ( ScriptUnit &unit );

	bool ImageTagSub ( NotXML::Tag &imageTag );

	//=========================================================
	// ユニット実行中（終了を待機）
	//=========================================================
	void UnitProcessing ( ScriptUnit &unit );
	void UnitProcessing_DefineTextWindow   ( ScriptUnit &unit );
	void UnitProcessing_TextWindowDesign   ( ScriptUnit &unit );
	void UnitProcessing_Text               ( ScriptUnit &unit );
	void UnitProcessing_TextAlias_Regist   ( ScriptUnit &unit );
	void UnitProcessing_Protagonist_Regist ( ScriptUnit &unit );
	void UnitProcessing_Skipping           ( ScriptUnit &unit );
	void UnitProcessing_Print              ( ScriptUnit &unit );
	void UnitProcessing_SystemMessage      ( ScriptUnit &unit );
	void UnitProcessing_Image              ( ScriptUnit &unit );
	void UnitProcessing_Image_Regist       ( ScriptUnit &unit );
	void UnitProcessing_Image_Group        ( ScriptUnit &unit );
	void UnitProcessing_BGM_Regist         ( ScriptUnit &unit );
	void UnitProcessing_BGM_Change         ( ScriptUnit &unit );
	void UnitProcessing_BGM_Config         ( ScriptUnit &unit );
	void UnitProcessing_Sound_Regist       ( ScriptUnit &unit );
	void UnitProcessing_Sound              ( ScriptUnit &unit );
	void UnitProcessing_TextLabel          ( ScriptUnit &unit );
	void UnitProcessing_Select             ( ScriptUnit &unit );
	void UnitProcessing_Variable_Define    ( ScriptUnit &unit );
	void UnitProcessing_Variable_Release   ( ScriptUnit &unit );
	void UnitProcessing_Variable_Operation ( ScriptUnit &unit );
	void UnitProcessing_Variable_Console   ( ScriptUnit &unit );
	void UnitProcessing_IF                 ( ScriptUnit &unit );
	void UnitProcessing_Switch             ( ScriptUnit &unit );
	void UnitProcessing_Gosub              ( ScriptUnit &unit );
	void UnitProcessing_Button             ( ScriptUnit &unit );
	void UnitProcessing_Game               ( ScriptUnit &unit );
	void UnitProcessing_Game_Release       ( ScriptUnit &unit );
	void UnitProcessing_Move               ( ScriptUnit &unit );
	void UnitProcessing_Put                ( ScriptUnit &unit );
	void UnitProcessing_Remove             ( ScriptUnit &unit );
	void UnitProcessing_RemoveAll          ( ScriptUnit &unit );
	void UnitProcessing_WatchEvent         ( ScriptUnit &unit );
	void UnitProcessing_WatchGameOperation ( ScriptUnit &unit );
	void UnitProcessing_Jump               ( ScriptUnit &unit );
	void UnitProcessing_Wait               ( ScriptUnit &unit );
	void UnitProcessing_WaitClick          ( ScriptUnit &unit );
	void UnitProcessing_Battle             ( ScriptUnit &unit );
	void UnitProcessing_AstralProjection   ( ScriptUnit &unit );
	void UnitProcessing_Fade               ( ScriptUnit &unit );
	void UnitProcessing_Shake              ( ScriptUnit &unit );
	void UnitProcessing_Camera             ( ScriptUnit &unit );
	void UnitProcessing_Title              ( ScriptUnit &unit );
	void UnitProcessing_Chapter            ( ScriptUnit &unit );
	void UnitProcessing_Save               ( ScriptUnit &unit );
	void UnitProcessing_Load               ( ScriptUnit &unit );
	void UnitProcessing_WriteSaveData      ( ScriptUnit &unit );
	void UnitProcessing_FileSelect         ( ScriptUnit &unit );
	void UnitProcessing_ClickableImage     ( ScriptUnit &unit );
	void UnitProcessing_ClearClickableImage( ScriptUnit &unit );
	void UnitProcessing_DefineControlTag   ( ScriptUnit &unit );
	void UnitProcessing_WaitForClickableImages ( ScriptUnit &unit );
	void UnitProcessing_FreeBattleSelector ( ScriptUnit &unit );
	



	//=========================================================
	// イメージリスト関連
	//=========================================================
	void   Image_Update      ( sf::RenderWindow &window ); // 更新
	void   Image_Draw_Back   ( sf::RenderWindow &window ); // 描画＿背景
	void   Image_Draw_Object ( sf::RenderWindow &window ); // 描画＿オブジェクト
	void   Image_ZSort  ();                                // Zでソート
	void   Image_Release ();
	void   Image_Release (string label);
	Image* Image_Get    ( string label );             // ラベルを指定して取得

	//=========================================================
	// クリック画像関連
	//=========================================================
	void ClickableImage_Update  ( sf::RenderWindow &window );
	void ClickableImage_Draw    ( sf::RenderWindow &window );
	void ClickableImage_Release ();
	ClickableImage* ClickableImage_Get ( string name );

	//=========================================================
	// スクリプトボタン関連
	//=========================================================
	void Button_Update ( sf::RenderWindow &window );
	void Button_Draw   ( sf::RenderWindow &window );
	void Button_Release ();
	ScriptButton* Button_Get ( string name );

	//=========================================================
	// テキストラベル関連
	//=========================================================
	void TextLabel_Update    ( sf::RenderWindow &window );
	void TextLabel_Draw      ( sf::RenderWindow &window );
	void TextLabel_Release   ();
	void TextLabel_Release   ( string name );
	TextLabel* TextLabel_Get ( string name );


	//=========================================================
	// ゲーム盤関連
	//=========================================================
	void GameManager_ReleaseAll  ();
	void GameManager_Release     ( string name );
	void GameManager_Update      ( sf::RenderWindow &window );
	void GameManager_Draw        ( sf::RenderWindow &window );
	void GameManager_Draw_ScoreSheet ( sf::RenderWindow &window );
	GameManager* GameManager_Get ( string name );

	BattleData _battleData;  // 試合情報。一度に複数の試合をすることはない、はず。
	Effect_CurtainClose *_effect_Curtain_Close;

	vector<string> _hideImageLabelList_ForAstralProjection; // 検討モードのために非表示にした画像リスト。後で元に戻す。

	bool GameManager_IsHoldingCard ();

	//=========================================================
	// 試合処理関連
	//=========================================================
	void UnitProcessing_Battle_PHASE_PHASE_BEFORE_ACTOR_APPEARS (); // カーテンを閉める：閉まりきったら、アクター登場。
	void UnitProcessing_Battle_PHASE_PHASE_ACTOR_APPEARED       (); // カーテンを開ける：開け始めると同時に試合開始
	void UnitProcessing_Battle_PHASE_PHASE_BATTLING             (); // 試合中。
	void UnitProcessing_Battle_PHASE_PHASE_AFTER_BATTLE         (); // 試合後。
	void UnitProcessing_Battle_PHASE_PHASE_BEFORE_ACTOR_LEAVES  (); // カーテンを閉める：閉まりきったら、アクター退場。
	void UnitProcessing_Battle_PHASE_PHASE_ACTOR_LEFT           (); // カーテンを開ける：開け始めたら、試合処理を終了。


	//=========================================================
	// 変数関連
	//=========================================================

	///// この名前の変数は、数値ですか？：スコープは問いません
	bool IsInt ( string variableName );

	///// この名前の変数は、文字列ですか？：スコープは問いません。
	bool IsString ( string variableName );

	///// この名前の変数の値（数値）をください。：存在しない場合に、何が返ってくるかは、保証しません。
	int GetIntVariable ( string variableName );

	///// この名前の変数の値（文字列）をください。：存在しない場合に、何が返ってくるかは、知りません。
	string GetStringVariable ( string variableName );

	///// この数値を、この名前の変数に代入してください：スコープは問いません。存在しない場合は無視してください。
	void SetIntValue ( string variableName, int value );

	///// この文字列を、この名前の変数に代入してください：スコープは問いません。存在しない場合は無視してください。
	void SetStringValue ( string variableName, string value );

	///// この文字列を、ローカル変数に代入してください。存在する場合は上書きです。
	void SetLocalString ( string variableName, string value );

	///// グローバル変数をセーブ
	void SaveGlobalData();

	///// グローバル変数を、ファイルからロード
	void LoadGlobalData();

	///// クイックセーブ
	void QuickSave();

	///// クイックロード
	void QuickLoad();


	///// ファイル名をエイリアスとして解釈してみる
	string ParseImageFileNameAsAlias ( string maybeAlias );

	///// 座標を表す文字列の末尾は '%' かもしれないのでした
	int ConvertCoordinateX_from_string ( string coordinate );
	int ConvertCoordinateY_from_string ( string coordinate );

	///// 画面ゆれエフェクトの終了待機用
	string _isShakeEnd;
	
	
	///// SS用。画像圧縮
	sf::Image CompressImage ( sf::Image &srcImage, int newWidth, int newHeight );
	

	///// 実験
	/*
	int _n;
	sf::Image _image1;
	sf::Image _image2;
	sf::Texture _texture1;
	sf::Texture _texture2;
	sf::Sprite _sprite1;
	sf::Sprite _sprite2;

	sf::Image _image0;
	sf::Texture _texture0;
	sf::Sprite _sprite0;
	*/
	//sf::Sound _sound;
	//sf::SoundBuffer _soundBuf;

	//SelectOption *_selectOption;
	//Select       *_select;
	//SaveData *_saveData;

	//GUI_SFML::ProgressBar *_progressBar;

	//=========================================================
	// Lua関数呼び出し：エフェクトレベル取得
	//【戻り値】
	// EFFECT_LEVEL     エフェクトレベル
	// CUTIN_TYPE       カットインの種類
	// CUTIN_PARAMETERS カットインに渡すパラメータ
	//=========================================================
	//map<string, string> GetMoveEffectLevel ( string scriptFileName, GameManager *gameManager );



};
#endif
