//=========================================================
//
// スクリプトエンジン（クリック画像関連）
//
//=========================================================

#include "Application.h"
#include "ScriptEngine.h"
#include "ScriptProcess.h"
#include "ScriptUnit.h"
#include "Image.h"
#include "BGM.h"
#include "SoundManager.h"
#include "SelectOption.h"
#include "Select.h"
#include "SaveData.h"
#include "GlobalSaveData.h"
#include "Menu.h"
#include "ReadTextManager.h"
#include "ScriptButton.h"
#include "ClickableImage.h"
#include "Game_Utils.h"
#include "EffectManager.h"
#include "Effect_BattleStart.h"
#include "Effect_BattleWin.h"
#include "Effect_BattleLose.h"
#include "Effect_BattleDraw.h"

#include "../Utils/NotXML.h"
#include "../Utils/Utils_Convert.h"
#include "../Utils/Utils_String.h"
#include "../Utils/Utils_File.h"
#include "../Utils/Utils_Others.h"
#include "../GUI_SFML/TextMessageWindow.h"
#include "../GUI_SFML/SFML_Utils.h"

#include "CardFactory.h"
#include "GameManager.h"
#include "Piece.h"

#include "../AI/AI.h"
#include "../AI/Move.h"

#include "Effect_CuiInTest001.h"
#include "Effect_CuiInTest002.h"
#include "Effect_CutIn_COME_AND_GO.h"

#include <lua.hpp>
//#include <lua.h>

using namespace std;
using namespace GUI_SFML;


//=========================================================
// ユニット開始処理：クリックできる画像
//=========================================================
void ScriptEngine::UnitStart_ClickableImage ( ScriptUnit &unit ) {

	///// 名前をもとに既存のボタンを取得。なければ新規に生成。あればそれに対して操作を行う。
	string name = NotXML::GetAttribute ( unit.unitTag, "name" );


	ClickableImage *cImage = ClickableImage_Get ( name );

	///// 登録されてなかった。新規に作成する。
	if ( cImage == NULL ) {

		sf::Vector2f position;
		ClickableImage::Property initProp;
		
		initProp.name = name;

		///// 内部のタグでループ
		for ( unsigned int i = 0; i < unit.unitTag.childList.size(); ++i) {

			NotXML::Tag tag = unit.unitTag.childList[i];

			///// 位置
			if ( tag.name == "Position" ) {

				position.x = Utils_Convert::String_to_Float ( NotXML::GetAttribute ( tag, "x", "0") );
				position.y = Utils_Convert::String_to_Float ( NotXML::GetAttribute ( tag, "y", "0") );
			}
			///// サイズ
			else if ( tag.name == "Size" ) {

				initProp.size.x = Utils_Convert::String_to_Float ( NotXML::GetAttribute ( tag, "width", "0") );
				initProp.size.y = Utils_Convert::String_to_Float ( NotXML::GetAttribute ( tag, "height", "0") );
			}
			///// 画像
			else if ( tag.name == "ImageFile" ) {
				initProp.imageFile = ParseImageFileNameAsAlias (tag.value);
			}
			///// 矩形
			else if ( tag.name == "Rect" ) {

				string state = NotXML::GetAttribute ( tag, "state", "");

				sf::IntRect rect;
				rect.left   = Utils_Convert::String_to_Int ( NotXML::GetAttribute ( tag, "left",   "0") );
				rect.top    = Utils_Convert::String_to_Int ( NotXML::GetAttribute ( tag, "top",    "0") );
				rect.width  = Utils_Convert::String_to_Int ( NotXML::GetAttribute ( tag, "width",  "0") );
				rect.height = Utils_Convert::String_to_Int ( NotXML::GetAttribute ( tag, "height", "0") );

				if      ( state == "normal" ) { initProp.rect_Normal = rect; }
				else if ( state == "over"   ) { initProp.rect_Over   = rect; }
				else if ( state == "push"   ) { initProp.rect_Push   = rect; }

			}
		}

		// インスタンスを生成して、
		cImage = new ClickableImage ( initProp );
		cImage->SetPosition ( position );

		// リストに追加
		_clickableImageList [ name ] = cImage;


	///// 新規じゃない場合：更新
	} else {

		///// 内部のタグでループ
		for ( unsigned int i = 0; i < unit.unitTag.childList.size(); ++i) {

			NotXML::Tag tag = unit.unitTag.childList[i];

			///// 位置
			if ( tag.name == "Position" ) {

				sf::Vector2f position;
				position.x = Utils_Convert::String_to_Float ( NotXML::GetAttribute ( tag, "x", "0") );
				position.y = Utils_Convert::String_to_Float ( NotXML::GetAttribute ( tag, "y", "0") );

				cImage->SetPosition ( position );
			}
			///// サイズ
			else if ( tag.name == "Size" ) {

				sf::Vector2f size;
				size.x = Utils_Convert::String_to_Float ( NotXML::GetAttribute ( tag, "width", "0") );
				size.y = Utils_Convert::String_to_Float ( NotXML::GetAttribute ( tag, "height", "0") );

				cImage->SetSize ( size );
			}
			///// 画像
			else if ( tag.name == "ImageFile" ) {
				cImage->SetImageFile ( ParseImageFileNameAsAlias (tag.value) );
			}
			///// 矩形
			else if ( tag.name == "Rect" ) {

				string state = NotXML::GetAttribute ( tag, "state", "");

				sf::IntRect rect;
				rect.left   = Utils_Convert::String_to_Int ( NotXML::GetAttribute ( tag, "left",   "0") );
				rect.top    = Utils_Convert::String_to_Int ( NotXML::GetAttribute ( tag, "top",    "0") );
				rect.width  = Utils_Convert::String_to_Int ( NotXML::GetAttribute ( tag, "width",  "0") );
				rect.height = Utils_Convert::String_to_Int ( NotXML::GetAttribute ( tag, "height", "0") );

				if      ( state == "normal" ) { cImage->SetRectNormal ( rect ); }
				else if ( state == "over"   ) { cImage->SetRectOrver  ( rect ); }
				else if ( state == "push"   ) { cImage->SetRectPush   ( rect ); }

			}
		}
	}

	// 処理終了
	_state = ScriptEngine::STATE_CURRENT_UNIT_FINISHED;
	
	
	// 今回は描画しなくていいですよね？
	//_isRestDraw = true;
	
}


//=========================================================
// 更新
//=========================================================
void ScriptEngine::ClickableImage_Update  ( sf::RenderWindow &window ) {

	for ( map<string, ClickableImage*>::iterator it = _clickableImageList.begin(); it != _clickableImageList.end(); ++it ) {

		it->second->Update ( window );
	}

}

//=========================================================
// 描画
//=========================================================
void ScriptEngine::ClickableImage_Draw    ( sf::RenderWindow &window ) {

	for ( map<string, ClickableImage*>::iterator it = _clickableImageList.begin(); it != _clickableImageList.end(); ++it ) {

		it->second->Draw ( window );
	}
}

//=========================================================
// リリース
//=========================================================
void ScriptEngine::ClickableImage_Release ( ) {

	while ( _clickableImageList.size() > 0) {

		delete _clickableImageList.begin()->second;
		_clickableImageList.erase ( _clickableImageList.begin() );
	}
}

//=========================================================
// 取得
//=========================================================
ClickableImage* ScriptEngine::ClickableImage_Get ( string name ) {

	ClickableImage *result = NULL;


	map<string, ClickableImage*>::iterator it = _clickableImageList.find ( name );
	if ( it != _clickableImageList.end() ) {

		result = it->second;

	}

	return result;
}



//=========================================================
// ユニット開始処理：クリック画像・クリック待ち受け
//=========================================================
void ScriptEngine::UnitStart_WaitForClickableImages ( ScriptUnit &unit ) {

	///// 既存の待ち受け情報を削除
	_watchClickableImageDataList.clear();
	
	for ( unsigned int i = 0; i < unit.unitTag.childList.size(); ++i ) {

		NotXML::Tag tag = unit.unitTag.childList[i];

		if ( tag.name == "ClickableImage") {

			WatchClickableImageData data;
			data.cImageName  = NotXML::GetAttribute ( tag, "name", "" );
			data.jumpToLabel = NotXML::GetAttribute ( tag, "jumpTo", "" );

			_watchClickableImageDataList.push_back ( data );
		}

	}

	///// 実行中：_watchClickableImageDataList が０件でも。確認してあげないの？ ヒドイ！
	_state = ScriptEngine::STATE_WAIT_UNTIL_UNIT_END;
}

//=========================================================
// ユニット実行中：クリック画像・クリック待ち受け
//=========================================================
void ScriptEngine::UnitProcessing_WaitForClickableImages ( ScriptUnit &unit ) {
	
	Application *app = Application::GetInstance();
	
	for ( unsigned int i = 0; i < _watchClickableImageDataList.size(); ++i ) {

		string name = _watchClickableImageDataList[i].cImageName;
		
		ClickableImage *cImage = ClickableImage_Get ( name );

		if ( cImage == NULL ) {
			continue;
		}
		
		// クリックされてるかな？
		bool isClick = cImage->IsClick();
		
		// たとえクリックされてなかったとしても、今はキーボードでも操作できる時代なのだ！
		if ( _hasFocus ) {
			
			if      ( i ==  0 && app->_releasedKeyCode == sf::Keyboard:: F1 ) { isClick = true; }
			else if ( i ==  1 && app->_releasedKeyCode == sf::Keyboard:: F2 ) { isClick = true; }
			else if ( i ==  2 && app->_releasedKeyCode == sf::Keyboard:: F3 ) { isClick = true; }
			else if ( i ==  3 && app->_releasedKeyCode == sf::Keyboard:: F4 ) { isClick = true; }
			else if ( i ==  4 && app->_releasedKeyCode == sf::Keyboard:: F5 ) { isClick = true; }
			else if ( i ==  5 && app->_releasedKeyCode == sf::Keyboard:: F6 ) { isClick = true; }
			else if ( i ==  6 && app->_releasedKeyCode == sf::Keyboard:: F7 ) { isClick = true; }
			else if ( i ==  7 && app->_releasedKeyCode == sf::Keyboard:: F8 ) { isClick = true; }
			else if ( i ==  8 && app->_releasedKeyCode == sf::Keyboard:: F9 ) { isClick = true; }
			else if ( i ==  9 && app->_releasedKeyCode == sf::Keyboard::F10 ) { isClick = true; }
			else if ( i == 10 && app->_releasedKeyCode == sf::Keyboard::F11 ) { isClick = true; }
			else if ( i == 11 && app->_releasedKeyCode == sf::Keyboard::F12 ) { isClick = true; }
			else if ( i == 12 && app->_releasedKeyCode == sf::Keyboard::F13 ) { isClick = true; }
			else if ( i == 13 && app->_releasedKeyCode == sf::Keyboard::F14 ) { isClick = true; }
			else if ( i == 14 && app->_releasedKeyCode == sf::Keyboard::F15 ) { isClick = true; }
				
		}
		
		// クリックされてたら次の実行先をセットして終了
		if ( isClick ) {

			unit.nextUnitLabel = _watchClickableImageDataList[i].jumpToLabel;
			_state = ScriptEngine::STATE_CURRENT_UNIT_FINISHED;
			
			// 終わっていいですよね？ うん、いいよ。
			break;
		}
		
		
	}

}

//=========================================================
// ユニット開始処理：クリックできる画像をクリア
//=========================================================
void ScriptEngine::UnitStart_ClearClickableImage ( ScriptUnit &unit ) {

	ClickableImage_Release();
	_state = ScriptEngine::STATE_CURRENT_UNIT_FINISHED;
}








