//=========================================================
//
// スクリプトエンジン
//
//=========================================================

#include "Application.h"
#include "ScriptEngine.h"
#include "ScriptProcess.h"
#include "ScriptUnit.h"
#include "Image.h"
#include "BGM.h"
#include "SoundManager.h"
#include "SelectOption.h"
#include "Select.h"
#include "SaveData.h"
#include "GlobalSaveData.h"
#include "Menu.h"
#include "ReadTextManager.h"
#include "ScriptButton.h"
#include "Game_Utils.h"
#include "EffectManager.h"
#include "Effect_BattleStart.h"
#include "Effect_BattleWin.h"
#include "Effect_BattleLose.h"
#include "Effect_BattleDraw.h"
#include "Effect_Curtain_Close.h"

#include "../Utils/NotXML.h"
#include "../Utils/Utils_Convert.h"
#include "../Utils/Utils_String.h"
#include "../Utils/Utils_File.h"
#include "../Utils/Utils_Others.h"
#include "../GUI_SFML/TextMessageWindow.h"
#include "../GUI_SFML/SFML_Utils.h"

#include "CardFactory.h"
#include "GameManager.h"
#include "GameRecord.h"
#include "Piece.h"
#include "Actor.h"

#include "../AI/AI.h"
#include "../AI/Move.h"

#include "Effect_CuiInTest001.h"
#include "Effect_CuiInTest002.h"
#include "Effect_CutIn_COME_AND_GO.h"
#include "Effect_CutIn_SPEAK.h"
#include "Effect_CurtainOpen.h"
#include "Effect_ScriptedOperation.h"

#include "ResignButton.h"

#include <lua.hpp>
//#include <lua.h>

using namespace std;
using namespace GUI_SFML;


//=========================================================
// ユニット開始処理：試合
//=========================================================
void ScriptEngine::UnitStart_Battle ( ScriptUnit &unit ) {

	///// 最低限あるはずのもの：どのゲーム盤？
	string gameManagerName = NotXML::GetTagFromName ( unit.unitTag, "Game" ).value;

	///// ゲーム盤、ないとは言わせませんよ？
	GameManager *gameManager = GameManager_Get ( gameManagerName );

	// え、ないの？ マジで？ おはなしになりませんね。
	if ( gameManager == NULL ) {
		_state = ScriptEngine::STATE_CURRENT_UNIT_FINISHED;
		printf ( "Warning：ゲーム盤がないので試合できません：%s\n", gameManagerName.c_str() );
		return;
	}

	///// 試合開始時点の状態でセーブしておく
	string fileName = "./data/save/qsave/beforeBattle";
	SaveData *quickSaveData = new SaveData ( fileName + ".sav", 0 );

	// ロード時にはこのタグの開始処理からやって欲しいので、「そういうことに」しておく
	_state = ScriptEngine::STATE_RESUME;

	// すぐに delete してしまうので、SSの撮影は別ルートで行う
	SaveTo (quickSaveData, false, "");
	delete quickSaveData;

	// その別ルートとは、これだ！
	_takeSStoThisFile = fileName + ".png";

	// クイックセーブが終わったことだし、_state を元に戻しておきたい気がしますが、
	// どのみち、この関数の最後で STATE_WAIT_UNTIL_UNIT_END にしているので、それでよしとする。


	///// ここからが本番。試合情報をクリアしますよ。一度に一回ずつしか試合しませんよね？ よね？
	_battleData = BattleData();
	_battleData.gameManagerName = gameManagerName;
	_battleData.phase = BattleData::PHASE_BEFORE_ACTOR_APPEARS;
	

	///// 投了ボタンはデフォルトではオフ
	_resignButton->SetActive(false);


	///// 子要素さんこんにちわ。
	for ( unsigned int i = 0; i < unit.unitTag.childList.size(); ++i ) {

		NotXML::Tag tag = unit.unitTag.childList[i];

		// プレイヤーはどっち側？
		if ( tag.name == "PlayerSide" ) {

			// "P1" か "P2" が入ってくると信じる！
			_battleData.playerSide = tag.value;

		// 試合結果を入れる変数名を指定する
		} else if ( tag.name == "ResultVariable" ) {

			_battleData.resultVariable = tag.value;

		// 試合中は非表示にしておく画像
		} else if ( tag.name == "HideImage" ) {

			_battleData.hideImageLabelList.push_back ( tag.value );

		// カーテンエフェクト
		//} else if ( tag.name == "CurtainEffect" ) {
		} else if ( tag.name == "CurtainEffect_Begin" ) {

			_battleData.curtainEffect_begin = tag.value;

		} else if ( tag.name == "CurtainEffect_End" ) {

			_battleData.curtainEffect_end = tag.value;

		} else if ( tag.name == "CurtainEffect_After" ) {

			_battleData.curtainEffect_after = tag.value;

		} else if ( tag.name == "ResignButton" ) {
		
			bool isResignButtonOn = Utils_String::isPositiveWord(tag.value);
			
			if ( isResignButtonOn ) {
				_resignButton->SetActive(true);
			}
			
			string positionDesignation = NotXML::GetAttribute(tag, "position", "");

			if ( positionDesignation != "" ) {
			
				vector<string> posData = Utils_String::Split ( positionDesignation, "," );
				int x = ( posData.size() >= 2 ) ? Utils_Convert::String_to_Int( posData[0] ) : 0;
				int y = ( posData.size() >= 2 ) ? Utils_Convert::String_to_Int( posData[1] ) : 0;

				printf ("投了ボタンの位置を指定（%d, %d）\n", x, y);
				
				_resignButton->SetPosition ( x, y );

			} else {
				printf ("投了ボタンの位置はデフォルト\n");
				_resignButton->SetPositionToDefault();
			}
		}

		/* 試合結果に応じて、試合終了後にジャンプする設定は使わないこととする。
		} else if ( tag.name == "Result" ) {

			for ( unsigned int n = 0; n < tag.childList.size(); ++n ) {

				NotXML::Tag subTag = tag.childList[n];
				if      ( subTag.name == "Win"  ) { _battleData.jumpTo_WIN  = subTag.value; }
				else if ( subTag.name == "Draw" ) { _battleData.jumpTo_DRAW = subTag.value; }
				else if ( subTag.name == "Lose" ) { _battleData.jumpTo_LOSE = subTag.value; }

			}

		*/


	}
	///// ゲーム盤さんにも、準備をしていただくのですよ。
	gameManager->BattleStart();

	///// アクターは、まだ登場しないのです。
	//string AI_Name = "";

	//gameManager->GetActor( AI_Name )->SetActive(false);
	gameManager->SetAllActorsActivity(false);

	///// これから試合なので、気分を一新してね。
	//gameManager->GetActor( AI_Name )->Init();
	gameManager->InitAllActors();

	///// カーテンエフェクトをいろいろと用意して指定できるようにするといい？ 今は「使う」「使わない」だけ。
	bool isUsingBeginCurtain = false;

	if ( _battleData.curtainEffect_begin != "" ) {
		isUsingBeginCurtain = true;
	}

	///// カーテンエフェクトあり
	if (isUsingBeginCurtain) {

		///// カーテン閉じるエフェクトを開始しておく
		//delete _effect_Curtain_Close; // deleteしない。万一二重に生成しても、以前のオブジェクトはすでにエフェクトマネージャーに渡されているはず。そして、以降の処理では、ここで生成したオブジェクトに用がある。
		_effect_Curtain_Close = new Effect_CurtainClose_001();
		Application::GetInstance()->_effectManager->Add(_effect_Curtain_Close);


		_battleData.phase = BattleData::PHASE_BEFORE_ACTOR_APPEARS;


	///// カーテンエフェクトなし
	} else {

		///// 指定の画像を非表示にする
		for ( unsigned int i = 0; i < _battleData.hideImageLabelList.size(); ++i ) {

			Image *image = Image_Get ( _battleData.hideImageLabelList[i] );
			if ( image != NULL ) {

				image->SetActive(false);
			}

		}

		///// アクター登場
		//gameManager->GetActor( AI_Name )->SetActive(true);
		gameManager->SetAllActorsActivity(true);

		///// 試合開始のエフェクトを出す
		IEffect *effectBattleStart = new Effect_BattleStart();
		Application::GetInstance()->_effectManager->Add ( effectBattleStart );


		// じゃあ、このへんでアクターのみなさんに試合開始のお知らせを
		// カーテンエフェクトありの場合は、nitProcessing_Battle_PHASE_PHASE_ACTOR_APPEARED() 側で同じことをする。
		//GameManager *gameManager = GameManager_Get ( _battleData.gameManagerName );
		vector<Actor*> allActors = gameManager->GetAllActors();
		for ( unsigned int i = 0 ; i < allActors.size(); ++i ) {
			allActors[i]->Notice_BattleStart();
		}


		/*
		// 口出しモードの実験：内容決め打ちで
		GameManager::InterferenceModeInfo ifmi;
		ifmi.isActive = true;
		ifmi.maxTimes = 3;
		ifmi.currentTimes = 0;
		gameManager->SetInterferenceModeInfo(ifmi);
		gameManager->InterferenceMode_StartWaitingForAIMove();
		*/

		// 口出しモード：最初の手番の参加者の設定を読み込む
		gameManager->DeployInterferenceModeFromParticipant ( gameManager->GetCurrentTurnParticipant().id );
		gameManager->InterferenceMode_StartWaitingForAIMove();

		// 候補手からの選択モード：最初の手番の参加者の設定を読み込む
		gameManager->DeployChoiceModeFromParticipant ( gameManager->GetCurrentTurnParticipant().id );
		gameManager->ChoiceMode_NewTurnStarts();

		// 現在の手番を設定
		gameManager->RenewCurentTurnNamePanel();

		// ペアバトルのチームを開始する
		gameManager->TeamNewTurnStart();


		_battleData.phase = BattleData::PHASE_BATTLING;

	}


	///// 処理開始するよ！
	_state = ScriptEngine::STATE_WAIT_UNTIL_UNIT_END;
}


//=========================================================
// ユニット実行中（終了を待機）：試合
//=========================================================
void ScriptEngine::UnitProcessing_Battle ( ScriptUnit &unit ) {

	///// フェイズで分岐
	if      ( _battleData.phase == BattleData::PHASE_BEFORE_ACTOR_APPEARS ) { UnitProcessing_Battle_PHASE_PHASE_BEFORE_ACTOR_APPEARS (); }
	else if ( _battleData.phase == BattleData::PHASE_ACTOR_APPEARED       ) { UnitProcessing_Battle_PHASE_PHASE_ACTOR_APPEARED       (); }
	else if ( _battleData.phase == BattleData::PHASE_BATTLING             ) { UnitProcessing_Battle_PHASE_PHASE_BATTLING             (); }
	else if ( _battleData.phase == BattleData::PHASE_AFTER_BATTLE         ) { UnitProcessing_Battle_PHASE_PHASE_AFTER_BATTLE         (); }
	else if ( _battleData.phase == BattleData::PHASE_BEFORE_ACTOR_LEAVES  ) { UnitProcessing_Battle_PHASE_PHASE_BEFORE_ACTOR_LEAVES  (); }
	else if ( _battleData.phase == BattleData::PHASE_ACTOR_LEFT           ) { UnitProcessing_Battle_PHASE_PHASE_ACTOR_LEFT           (); }

}


//=========================================================
// 試合ユニット実行：カーテンを閉める：閉まりきったら、アクター登場。
//=========================================================
void ScriptEngine::UnitProcessing_Battle_PHASE_PHASE_BEFORE_ACTOR_APPEARS () {

	/*
		カーテンが閉まりきったかどうかの判定処理。
		エフェクトオブジェクトがエフェクトマネージャーに削除される直前の、スリリングなタイミング。
	*/
	if ( _effect_Curtain_Close->IsClosed() ) {

		///// 指定の画像を非表示にする
		for ( unsigned int i = 0; i < _battleData.hideImageLabelList.size(); ++i ) {

			Image *image = Image_Get ( _battleData.hideImageLabelList[i] );
			if ( image != NULL ) {

				image->SetActive(false);
			}

		}

		///// アクターを登場させる
		string AI_Name = "";
		GameManager *gameManager = GameManager_Get ( _battleData.gameManagerName );
		//gameManager->GetActor( AI_Name )->SetActive(true);
		gameManager->SetAllActorsActivity(true);


		///// 次のフェイズへ
		_battleData.phase = BattleData::PHASE_ACTOR_APPEARED;

	}

}

//=========================================================
// 試合ユニット実行：カーテンを開ける：開け始めると同時に試合開始
//=========================================================
void ScriptEngine::UnitProcessing_Battle_PHASE_PHASE_ACTOR_APPEARED () {

	///// カーテンを開けるエフェクトを開始する
	IEffect *effectCurtainOpen = new Effect_CurtainOpen001();
	Application::GetInstance()->_effectManager->Add ( effectCurtainOpen );

	///// 試合開始のエフェクトを出す
	IEffect *effectBattleStart = new Effect_BattleStart();
	Application::GetInstance()->_effectManager->Add ( effectBattleStart );

	// じゃあ、このへんでアクターのみなさんに試合開始のお知らせを
	GameManager *gameManager = GameManager_Get ( _battleData.gameManagerName );
	vector<Actor*> allActors = gameManager->GetAllActors();
	for ( unsigned int i = 0 ; i < allActors.size(); ++i ) {
		allActors[i]->Notice_BattleStart();
	}

	// 口出しモード：最初の手番の参加者の設定を読み込む
	gameManager->DeployInterferenceModeFromParticipant ( gameManager->GetCurrentTurnParticipant().id );
	gameManager->InterferenceMode_StartWaitingForAIMove();

	// 候補手からの選択モード：最初の手番の参加者の設定を読み込む
	gameManager->DeployChoiceModeFromParticipant ( gameManager->GetCurrentTurnParticipant().id );
	gameManager->ChoiceMode_NewTurnStarts();

	// 現在の手番を設定
	gameManager->RenewCurentTurnNamePanel();

	// ペアバトルのチームを開始する
	gameManager->TeamNewTurnStart();

	///// 次のフェイズへ
	_battleData.phase = BattleData::PHASE_BATTLING;

}

///// 指し手を画面に表示する実験用として、マウスクリック待ち
//bool isMouseClicked = false;

//=========================================================
// 試合ユニット実行：試合中。
//=========================================================
void ScriptEngine::UnitProcessing_Battle_PHASE_PHASE_BATTLING () {

	///// えっと、どのゲーム盤、でしたっけ？
	GameManager *gameManager = GameManager_Get ( _battleData.gameManagerName );
	if ( gameManager == NULL ) {
		printf ( "Error：ゲーム盤がないので試合できません。 %s\n", _battleData.gameManagerName.c_str());
		_state = ScriptEngine::STATE_CURRENT_UNIT_FINISHED;
	}

	///// 試合を進めてね。
	gameManager->Update_Battle();


	///// 口出しモードとして処理しますか？
	if ( gameManager->IsInterferenceMode() ) {

		// AIの思考結果を待つ
		if ( gameManager->IsAIThinkFinished() && !gameManager->IsMoveEffectGoing() ) {

			///// これが試行結果です
			AI_Modules::ProgressData AI_thinkResult = gameManager->GetProgressData_AfterThinking();



			///// 口出しモードでの更新
			gameManager->Update_InterferenceMode ( AI_thinkResult );


		// AI思考中
		} else {
			// はーい。
		}


		///// 口出しモード：この段階ですでに、次のターンのAI指し手候補の待ち受け状態にしていいのかな？
		//gameManager->InterferenceMode_StartWaitingForAIMove();


	///// 候補手からの選択モードとして処理しますか？
	} else if ( gameManager->IsChoiceMode() ) {

		// AIの思考結果を待つ
		if ( gameManager->IsAIThinkFinished() && !gameManager->IsMoveEffectGoing() ) {

			///// これが試行結果です
			AI_Modules::ProgressData AI_thinkResult = gameManager->GetProgressData_AfterThinking();

			///// 口出しモードでの更新
			gameManager->Update_ChoiceMode ( AI_thinkResult );


		// AI思考中
		} else {
			// はーい。
		}


	///// AI、思考完了してる？（IsAIThinkFinishedが「AIの手番で」という判定も含んでいる）
	} else if ( gameManager->IsAIThinkFinished() && !gameManager->IsMoveEffectGoing() ) {

		//printf ( "口出しモードではない扱いで、AI思考完了\n" );

		///// これが試行結果です
		AI_Modules::ProgressData AI_thinkResult = gameManager->GetProgressData_AfterThinking();

		// 候補手からの選択モードの場合、プレイヤーが選んだものに置き換える
		AI_thinkResult = gameManager->ReplaceToPlayersChoiceIfNeeded ( AI_thinkResult );

		/*
		///// 指し手を画面に表示する実験用として、マウスクリック待ち
		AI_Modules::Move demonstratingMove = AI_thinkResult.answerMove;
		if ( gameManager->GetCurrentTurnParticipant().side == "P1" ) { demonstratingMove.turn = 1; }
		if ( gameManager->GetCurrentTurnParticipant().side == "P2" ) { demonstratingMove.turn = 2; }
		else { demonstratingMove.turn = 1; }

		gameManager->SetMoveDemonstration ( demonstratingMove );
		if ( !isMouseClicked ) {
			printf ( "指し手決定：マウスクリック待ち\n" );

			if ( Application::GetInstance()->_mouseState_Left == MOUSE::BUTTON::STATE::RELEASE ) {
				isMouseClicked = true;
			} else {
				return;
			}
		}
		gameManager->ClearMoveDemonstration();
		*/



		/*
			全参加アクターのみなさんに通知（着手直前イベント）
			ここでリアクションがある場合に、その終了を待機しなくていいのか？
			そういうことがしたい場合は本人の手番限定になりますが着手エフェクトで実現することとしてはどうか。

			と言いつつ、_actorReactionStateでレジュームみたいな感じのことをゴリゴリ書いてみる。
		*/
		if ( _actorReactionState == "BEFORE_NOTIFYING" ) {

			vector<Actor*> actors = gameManager->GetAllActors();
			for ( unsigned int i = 0; i < actors.size(); ++i ) {
				Actor *a = actors[i];
				a->Notice_MoveIsReady ( gameManager->GetOperationData().currentTurn, AI_thinkResult.answerMove, AI_thinkResult.evaluationSumUp );
			}
			_actorReactionState = "WAIT_REACTION";
		}

		// アクターのリアクション中なら中断
		if ( _actorReactionState == "WAIT_REACTION" ) {


			if ( gameManager->IsBattlingNow() && !_textMessageWindow->IsAllFinished() ) {
				return;

			} else {

				// ここまでは「テキスト欄が表示完了しているかどうか」の判定。
				// 掛け合いの切り替わりのタイミングという可能性もある
				// したがって、全アクターの掛け合いデータが残っているかどうかをここで確認する

				vector<Actor*> actors = gameManager->GetAllActors();
				for ( unsigned int i = 0; i < actors.size(); ++i ) {
					Actor *a = actors[i];

					if ( !a->IsKakeaiAllFlushed() ) {
						return;
					}
				}

				_actorReactionState = "REACTION_FINISHED";
			}
		}
		_actorReactionState = "BEFORE_NOTIFYING";

		///// 着手時のエフェクトレベルを決める。（0〜100）

		/*
		// アクターさんに来ていただく（本当はAI名を指定したい）
		string AI_Name = "";
		Actor *actor = gameManager->GetActor ( AI_Name );
		*/
		// 現在の手番のアクターさんに来ていただく
		Actor *actor = gameManager->GetCurrentTurnActor();

		// アクターさんいらっしゃらない場合
		if ( actor == NULL ) {

			// 素通りで指し手を実行。
			int effectLevel = 0;
			gameManager->ExecuteAIMove ( effectLevel );
		}

		// アクターさんにお問い合わせします。
		Actor::MoveEffectData moveEffectData = actor->DecideMoveEffect(AI_thinkResult);

		///// カットインなし の 場合
		if ( moveEffectData.cutInType == "" ) {

			///// エフェクトレベルを指定して、AIさん着手実行。
			gameManager->ExecuteAIMove ( moveEffectData.effectLevel );

		///// カットインあり
		//} else if ( moveEffectData.cutInType == "COME_AND_GO" ) {
		} else if ( moveEffectData.cutInType != "" ) {

			// エフェクトインスタンスは、こちらで作る
			IEffect *cutInEffect = NULL;

			// カットインタイプ：COME_AND_GO
			if ( moveEffectData.cutInType == "COME_AND_GO" ) {

				cutInEffect = new Effect_CutIn_COME_AND_GO (
					gameManager->GetEndMoveFromAICutIn(),
					moveEffectData.effectLevel,
					//( _textMessageWindow != NULL ) ? _textMessageWindow->GetSameSetuppedInstance() : NULL,
					_textMessageWindow,
					moveEffectData.cutInParameters
				);

			// カットインタイプ：SCRIPT
			} else if ( moveEffectData.cutInType == "SCRIPT" ) {

				cutInEffect = new Effect_ScriptedOperation (
					gameManager->GetEndMoveFromAICutIn(),
					moveEffectData.effectLevel,
					_textMessageWindow,
					moveEffectData.cutInParameters
				);

			// カットインタイプ：SPEAK
			} else if ( moveEffectData.cutInType == "SPEAK" ) {

				cutInEffect = new Effect_CutIn_SPEAK (
					gameManager->GetEndMoveFromAICutIn(),
					moveEffectData.effectLevel,
					//( _textMessageWindow != NULL ) ? _textMessageWindow->GetSameSetuppedInstance() : NULL,
					_textMessageWindow,
					moveEffectData.cutInParameters
				);

			} else {
				printf ("[Warning] 未定義のカットインタイプ [%s]\n", moveEffectData.cutInType.c_str());
			}

			// あとはGamemanagerさんの側で「着手実行中」としてもらう。：さもないと、ずっと、ここに来つづける。
			gameManager->ExecuteAIMoveWithCutIn (cutInEffect);
		}

//isMouseClicked = false;
	}

	///// 試合、終わった？
	string battleResult = gameManager->GetBattleResult();
	
	if ( battleResult != "" || _resignButton->IsResignDesided() ) {

		string battleResultDescription = gameManager->GetBattleResultDescription();
		
		string win_draw_lose = ""; // ちょっともう、てきとうすぎるんですけどー。

		printf("battleResult : %s\n", battleResult.c_str());
		printf("playerSide   : %s\n", _battleData.playerSide.c_str());
		printf("description  : %s\n", battleResultDescription.c_str());
		

		/*
			とりあえず P1 黒、P2 白、として、勝敗判定。
		*/
		if ( battleResult == "Draw") {
			win_draw_lose = "draw";
		}
		else if ( battleResult == "force-quit") {
			win_draw_lose = "force-quit";
			gameManager->GetScoreRecorder()->AddEndNotation ( "中断" );
		}
		else if ( _battleData.playerSide == "P1" || _battleData.playerSide == "p1" ) {

			if ( battleResult == "P1Win") {
				win_draw_lose = "win";
				gameManager->GetScoreRecorder()->AddEndNotation ( battleResultDescription );
			}
			else if ( battleResult == "P2Win") {
				win_draw_lose = "lose";
				gameManager->GetScoreRecorder()->AddEndNotation ( battleResultDescription );
			}
			else if ( _resignButton->IsResignDesided() ) {
				win_draw_lose = "lose";
				gameManager->GetScoreRecorder()->AddEndNotation ( "P2の勝ち（P1投了）" );
			}

		}
		else if ( _battleData.playerSide == "P2" || _battleData.playerSide == "p2" ) {

			if ( battleResult == "P1Win") {
				win_draw_lose = "lose";
				gameManager->GetScoreRecorder()->AddEndNotation ( battleResultDescription );
			}
			else if ( battleResult == "P2Win") {
				win_draw_lose = "lose";
				gameManager->GetScoreRecorder()->AddEndNotation ( battleResultDescription );
			}
			else if ( _resignButton->IsResignDesided() ) {
				win_draw_lose = "lose";
				gameManager->GetScoreRecorder()->AddEndNotation ( "P1の勝ち（P2投了）" );
			}
		}

		///// ここでアクターさんたちにお知らせ
		vector<Actor*> actors = gameManager->GetAllActors();
		for ( unsigned int i = 0; i < actors.size(); ++i ) {
			Actor* actor = actors[i];

			// battleResult={P1Win or P1Win or Draw}
			actor->Notice_BattleEnd ( battleResult );
		}


		///// 結果に応じて、エフェクトを出す。
		if ( win_draw_lose == "win" ) {

			IEffect *effect = new Effect_BattleWin();
			Application::GetInstance()->_effectManager->Add(effect);

			//unit.nextUnitLabel = _battleData.jumpTo_WIN;
		}
		else if ( win_draw_lose == "lose" ) {

			IEffect *effect = new Effect_BattleLose();
			Application::GetInstance()->_effectManager->Add(effect);

			//unit.nextUnitLabel = _battleData.jumpTo_LOSE;
		}
		else if ( win_draw_lose == "draw" ) {

			IEffect *effect = new Effect_BattleDraw();
			Application::GetInstance()->_effectManager->Add(effect);

			//unit.nextUnitLabel = _battleData.jumpTo_DRAW;
		}
		else if ( win_draw_lose == "force-quit" ) {

			// とりあえず、なし。

		}



		// 試合結果を、指定の変数に格納する
		if ( _battleData.resultVariable != "" ) {

			// 対象は、ローカルの文字列変数。
			// なければ宣言する。宣言というか、下記のコードで作られますよね。
			_variable_Local_String [ _battleData.resultVariable ] = win_draw_lose;

			/*
				GameManager が生成する「試合結果」は、P1Win /  P2Win
				それを、ここで、プレイヤー目線での「勝ちか負けか」に変換し、指定の変数に格納している。
			*/
		}


		// 処理終了（次のタグへ進む、ということ）
		//_state = ScriptEngine::STATE_CURRENT_UNIT_FINISHED;

		///// 次のフェイズへ
		_battleData.phase = BattleData::PHASE_AFTER_BATTLE;

		// ゲーム盤の側でも、試合終了処理
		gameManager->BattleEnd();

		// 投了ボタン、表示していたかどうかに関係なく、ともかくオフにしておく
		_resignButton->SetActive(false);
		
	}
}


//=========================================================
// 試合ユニット実行：試合後。
//=========================================================
void ScriptEngine::UnitProcessing_Battle_PHASE_PHASE_AFTER_BATTLE () {

	/*
		このフェイズを終える条件は？
		フレーム数経過？
		マウスクリック？
	*/

	///// フレーム数経過
	//--_battleData.frameCount_AfterBattle;

	///// マウスクリック
	if ( Application::GetInstance()->_mouseState_Left == MOUSE::BUTTON::STATE::RELEASE ) {
		_battleData.frameCount_AfterBattle = 0;
	}

	/*

		ここで、アクターのリアクションが残っている場合も待機というような処理を入れるとよさそう。


	*/


	///// 次のフェイズへ？
	if ( _battleData.frameCount_AfterBattle <= 0 ) {

		_battleData.phase = BattleData::PHASE_BEFORE_ACTOR_LEAVES;


		///// カーテンを閉じるエフェクトを開始

		if ( _battleData.curtainEffect_end == "001" ) {
			_effect_Curtain_Close = new Effect_CurtainClose_001();
		}
		else if ( _battleData.curtainEffect_end == "002" ) {
			_effect_Curtain_Close = new Effect_CurtainClose_002();
		}
		else if ( _battleData.curtainEffect_end == "003" ) {
			
			Effect_CurtainClose_001 *e = new Effect_CurtainClose_001();
			e->SetWaitFrameAfterClose(30);
			
			_effect_Curtain_Close = e;
		}
		else {
			_effect_Curtain_Close = new Effect_CurtainClose_001();
		}


		Application::GetInstance()->_effectManager->Add ( _effect_Curtain_Close );
	}


}


//=========================================================
// 試合ユニット実行：カーテンを閉める：閉まりきったら、アクター退場。
//=========================================================
void ScriptEngine::UnitProcessing_Battle_PHASE_PHASE_BEFORE_ACTOR_LEAVES () {

	if ( !Application::GetInstance()->_effectManager->IsExist(_effect_Curtain_Close) || _effect_Curtain_Close->IsClosed() ) {

		///// アクター退場というか非表示。
		string AI_Name = "";
		GameManager *gameManager = GameManager_Get ( _battleData.gameManagerName );
		//gameManager->GetActor ( AI_Name )->SetActive(false);
		gameManager->SetAllActorsActivity(false);

		/*
			アクターオブジェクトを解放はしない。
			ゲーム盤設定タグでアクターを設定しなおせば、いったん解放されて、新たにオブジェクトが生成される。
			それをしないまま、続けて試合を行うと、アクター内部の感情値が引き継がれる。
			というわけでもなく、試合前に Init を呼んでいるのですけれども、
			その気になれば、そういうふうにもできるのかも、という可能性を におわせておく。
		*/


		///// 非表示にしていた画像を再表示する。もともと非表示だったものも、ここで再表示されてしまうけどいいの？ 今のところ、そういうケースはないはず。なにしろこれのために非活性フラグを追加したのだから。根本的にまずいことには変わりない。
		for ( unsigned int i = 0; i < _battleData.hideImageLabelList.size(); ++i ) {

			Image *image = Image_Get ( _battleData.hideImageLabelList[i] );
			if ( image != NULL ) {

				image->SetActive(true);
			}
		}

		///// 次のフェイズへ
		_battleData.phase = BattleData::PHASE_ACTOR_LEFT;
	}

}


//=========================================================
// 試合ユニット実行：カーテンを開ける：開け始めたら、試合処理を終了。
//=========================================================
void ScriptEngine::UnitProcessing_Battle_PHASE_PHASE_ACTOR_LEFT () {

	///// カーテンを開けるエフェクトを開始
	//IEffect *effectCurtainOpen = new Effect_CurtainOpen001();

	///// 終了後のカーテンエフェクト：フェードイン
	if ( _battleData.curtainEffect_after == "" || _battleData.curtainEffect_after == "FADE_IN" ) {

		printf("終了後のカーテンエフェクト：フェードイン\n");

		IEffect *effectCurtainOpen = new Effect_CurtainOpen001();
		Application::GetInstance()->_effectManager->Add ( effectCurtainOpen );

	///// 終了後のカーテンエフェクト：なし（処理はスクリプト側でするので、ちょっと閉じたままにしておいて、という意味合い）
	} else if ( _battleData.curtainEffect_after == "KEEP_CURTAIN_CLOSED_FOR_A_WHILE" ) {

		printf("終了後のカーテンエフェクト：KEEP_CURTAIN_CLOSED_FOR_A_WHILE\n");

		IEffect *effectCurtainOpen = new Effect_CurtainOpen002();
		Application::GetInstance()->_effectManager->Add ( effectCurtainOpen );

	} else if ( _battleData.curtainEffect_after == "none" ) {

		printf("終了後のカーテンエフェクト：なし\n");

	} else {
		printf("終了後のカーテンエフェクト：その他（%s）\n", _battleData.curtainEffect_after.c_str());
	}

	///// 試合タグ終了
	_state = ScriptEngine::STATE_CURRENT_UNIT_FINISHED;

	///// 試合も終了
	_battleData.phase = BattleData::PHASE_BATTLE_NOT_STARTED;
}




/*
//=========================================================
// Lua関数呼び出し：エフェクトレベル(とかを)取得
//【戻り値】
// EFFECT_LEVEL     エフェクトレベル
// CUTIN_TYPE       カットインの種類
// CUTIN_PARAMETERS カットインに渡すパラメータ
//=========================================================
map<string, string> ScriptEngine::GetMoveEffectLevel ( string scriptFileName, GameManager *gameManager ) {

	map<string, string> result;

	result["EFFECT_LEVEL"    ] = "0";
	result["CUTIN_TYPE"      ] = "";
	result["CUTIN_PARAMETERS"] = "";


	///// ファイル名、ありますよね？
	if ( scriptFileName == "" ) {
		return result;
	}

	// Lua 初期設定
	lua_State *lua = luaL_newstate();
	luaL_openlibs ( lua );

	// Luaファイル読み込み
	if ( luaL_dofile (lua, scriptFileName.c_str()) ) {

		// エラー時のメッセージ表示
		string error_message = lua_tostring(lua, -1);

		printf ( "Error：luaL_dofile: %s\n", error_message.c_str());

		///// Lua 終了
		lua_close ( lua );

	// エラーなし
	} else {

		// Lua関数をスタックに積む
		lua_getglobal ( lua, "GetMoveEffectLevel" );

		// 変数 を lua空間に登録
		//
		//	必要な情報は：
		//	・総括評価値
		//	・棋譜に入っている情報：
		//		このAIが下した総括評価値のリスト、をもらえると、いい感じ？
		//		それをどうにかしてLuaに渡しましょう。
		//
		//	// 書式メモ：
		//	lua_pushnumber ( lua, );
		//	lua_setglobal  ( lua, "TEST");
		//
		AI_Modules::ProgressData AI_thinkResult = gameManager->GetProgressData_AfterThinking();

		// 総括評価値・今回の指し手に対する
		lua_pushstring ( lua, AI_thinkResult.evaluationSumUp.c_str() );
		lua_setglobal  ( lua, "EVALUATION_SUM_UP");

		// 総括評価値リスト：これまでの分
		lua_pushstring ( lua, Utils_Convert::vectorString_to_string ( gameManager->GetGameRecord()->GetSumupEvaluationList( gameManager->GetOperationData().GetCurrentTurn_As_Side() ) ).c_str() );
		lua_setglobal  ( lua, "EVALUATION_SUM_UP_List_so_far");

		// 引数をセット
		//lua_pushnumber ( lua, 100 );

		// 関数呼び出し
		lua_pcall ( lua, 0, 3, 0 );

		// 第１戻り値
		int effectLevel = lua_tointeger ( lua, 1 );

		// 第２戻り値
		size_t length;
		const char* cstr = lua_tolstring(lua, 2, &length);
		string cutInType = string( cstr, length );

		// 第３戻り値
		cstr = lua_tolstring(lua, 3, &length);
		string cutInParameters = string( cstr, length );

		printf ("Lua からお返事。effectLevel : %d,  cutInName : %s,  cutInParameters : %s \n", effectLevel, cutInType.c_str(), cutInParameters.c_str() );

		// 戻り値を整理
		result["EFFECT_LEVEL"    ] = Utils_Convert::Int_to_String ( effectLevel );
		result["CUTIN_TYPE"      ] = cutInType;
		result["CUTIN_PARAMETERS"] = cutInParameters;

		// 戻り値をスタックから降ろす
		lua_pop ( lua, 1 );
		lua_pop ( lua, 2 );
		lua_pop ( lua, 3 );

		///// Lua 終了
		lua_close ( lua );
	}

	return result;
}
*/
