//=========================================================
//
// 選択肢リスト？
//
//=========================================================

#include "Define.h"
#include "Select.h"
#include "SelectOption.h"
#include <IScenarioLogger.h>
#include "Application.h"
#include "Game_Utils.h"
#include "../AI/Move.h"
#include "../Utils/Utils_String.h"
#include "../Utils/Utils_Cal.h"

#include "ScriptEngine.h"
#include "GameManager.h"

//=========================================================
// コンストラクタ
//=========================================================
Select::Select ( ScriptEngine *scriptEndine ) {

	///// よしなに、よしなに。
	_position.x = 0;
	_position.y = 0;
	_rowGap     = 5;

	_position_FromCenter_default = sf::Vector2f(0.0f, -50.0f);
	_position_FromCenter         = _position_FromCenter_default;


	_scenarioLogger = NULL;
	_scriptEngine = scriptEndine;

    _gameBoardId = "";

	// 保存用の値も初期化しておくよ
	_posX_forSave = 0;
	_posY_forSave = 0;

}
//=========================================================
// デストラクタ
//=========================================================
Select::~Select() {

	Clear();
}

//=========================================================
// 選択肢リストをセット
//=========================================================
void Select::SetUpSelectOptions ( vector<SelectOption*> selectOptionList ) {

	///// 先客がいらしたら、お引取りいただく。
	Clear();

	///// まるごと受け取ればいいのですね？ 引渡し元は、生成しっぱなしってことで。
	_selectOptionList = selectOptionList;


	///// 表示位置を、いい感じに、とりはからいたい。願望。願望のままじゃ終わらせないんだから！
	/*
		欲しい情報：
			・横幅：１番大きい子の表示幅
			・縦幅：全部並べたら、どんだけになる？

		その上で決める情報：
			左上の座標。_position に入れる。
	*/

	///// １番おっきぃ子だーれだ？：実は、「誰」かはわからないのですね。うんうん。いわなくてよいですよー。
	int maxWidth = 0;

	///// 全部並べたときの縦幅もついでに知りたい。知りたい！
	int totalHeight = 0;

	for ( unsigned int i = 0; i < _selectOptionList.size(); ++i ) {

		int width = _selectOptionList[i]->GetSizeX();
		if ( width > maxWidth ) {
			maxWidth = width;
		}

		totalHeight += _selectOptionList[i]->GetSizeY();
	}
	totalHeight += _rowGap * ( _selectOptionList.size() - 1 ); // ０件だったら？ それはそれで構わない。どうせ表示されない。


	///// そんなキミたちをー、画面のまんなかに、表示したいと思うのですよー。
	_position.x = (APPLICATION::WINDOW_SIZE::WIDTH  / 2) - maxWidth / 2;
	_position.y = (APPLICATION::WINDOW_SIZE::HEIGHT / 2) - totalHeight / 2;

	int cursorY = _position.y;
	for ( unsigned int i = 0; i < _selectOptionList.size(); ++i ) {

		_selectOptionList[i]->SetPositionX ( _position.x + _position_FromCenter.x );
		_selectOptionList[i]->SetPositionY ( cursorY     + _position_FromCenter.y);

		cursorY += _selectOptionList[i]->GetSizeY() + _rowGap;
	}


	///// あとね、なんかね、横幅は、そろえたいんですって。ね？ね？
	for ( unsigned int i = 0; i < _selectOptionList.size(); ++i ) {

		_selectOptionList[i]->SetSizeX ( maxWidth );
	}


	///// 読み返しログ
	if (_scenarioLogger != NULL && _selectOptionList.size() > 0) {

		string log = "\n\t選択肢\n";

		for ( unsigned int i = 0; i < _selectOptionList.size(); ++i ) {

			log += "\t\t・" + _selectOptionList[i]->GetText() + "\n";
		}

		_scenarioLogger->WriteScenarioLog(log);
	}
}

//=========================================================
// クリア
//=========================================================
void Select::Clear() {

	while ( _selectOptionList.size() > 0 ) {
		delete *( _selectOptionList.begin() );
		_selectOptionList.erase(_selectOptionList.begin());
	}
	_selectOptionList.clear();

}


//=========================================================
// どうしても表示位置を動かしたい場合がある。口出しモードのときとか。
//=========================================================
void Select::Move ( float x, float y  ) {

	for ( unsigned int i = 0; i < _selectOptionList.size(); ++i ) {

		sf::Vector2f currentPos = _selectOptionList[i]->GetPosition();

		_selectOptionList[i]->SetPositionX ( currentPos.x + x );
		_selectOptionList[i]->SetPositionY ( currentPos.y + y );
	}


	// 多分、こうしておけばつじつまが合うのではありませんか？
	_position_FromCenter = _position_FromCenter_default + sf::Vector2f ( x, y );
	//_position_FromCenter.x += x;
	//_position_FromCenter.y += y;

	// 保存用
	_posX_forSave = x;
	_posY_forSave = y;
}

//=========================================================
// 更新
//=========================================================
void Select::Update ( sf::RenderWindow &window ) {

	// 見た目上邪魔な場合に備えて、非表示のオプション：表示を非にするだけでなく、動作も停止しておく
	if ( sf::Keyboard::isKeyPressed ( sf::Keyboard::LAlt  ) || sf::Keyboard::isKeyPressed ( sf::Keyboard::RAlt ) ) {
		return;
	}

	/*
		_selectOptionList が空なら、素通りする
	*/

    GameManager *gameManager = _scriptEngine->GetGameManager ( _gameBoardId );
    if ( gameManager != NULL ) {
        gameManager->ClearMoveDemonstration();
    }

	for ( unsigned int i = 0; i < _selectOptionList.size(); ++i ) {

        SelectOption *option = _selectOptionList[i];
		
		option->Update ( window );

		// マウスが乗っている場合のこちら側での処理
		if ( option->IsMouseover() ) {

            //NotXML::Tag moveTag = option->GetMoveTag();
            AI_Modules::Move mv = option->GetMove();

			//if ( moveTag.name != "" ) {
			if ( mv.action != 0 ) {

                //AI_Modules::Move mv = Game_Utils::Move_From_Tag ( moveTag );

                if ( gameManager != NULL ) {

                    gameManager->SetMoveDemonstration ( mv );

                }



            } else {
                // 指し手タグなし
            }


		}


	}

}
//=========================================================
// 描画
//=========================================================
void Select::Draw   ( sf::RenderWindow &window ) {

	// 見た目上邪魔な場合に備えて、非表示のオプション
	if ( sf::Keyboard::isKeyPressed ( sf::Keyboard::LAlt  ) || sf::Keyboard::isKeyPressed ( sf::Keyboard::RAlt ) ) {
		return;
	}


	for ( unsigned int i = 0; i < _selectOptionList.size(); ++i ) {

		_selectOptionList[i]->Draw ( window );
	}
}

//=========================================================
// 選択された選択肢のジャンプ先ラベルを取得：未選択なら空文字
//=========================================================
string Select::GetSelectOption() {

	/*
		選択肢が存在しない場合も、自動的に空文字が返る。
	*/

	string result = "";

	for ( unsigned int i = 0; i < _selectOptionList.size(); ++i ) {

		if (_selectOptionList[i]->IsSelected()) {

			result = _selectOptionList[i]->GetJumpToLabel();
			break;
		}

	}

	///// キーボードでも選択肢を選べる時代になった！
	if ( Application::GetInstance()->HasFocus() && result == "") {

		// 選択肢が15個以上あるときはどうすればいいのだろう？ 諦めればいいのですよ。なるほど！ よ〜し！
		do {
			if (_selectOptionList.size() > 0 && Application::GetInstance()->_releasedKeyCode == sf::Keyboard::F1) {
				result = _selectOptionList[0]->GetJumpToLabel(); break;
			}
			if (_selectOptionList.size() > 1 && Application::GetInstance()->_releasedKeyCode == sf::Keyboard::F2) {
				result = _selectOptionList[1]->GetJumpToLabel(); break;
			}
			if (_selectOptionList.size() > 2 && Application::GetInstance()->_releasedKeyCode == sf::Keyboard::F3) {
				result = _selectOptionList[2]->GetJumpToLabel(); break;
			}
			if (_selectOptionList.size() > 3 && Application::GetInstance()->_releasedKeyCode == sf::Keyboard::F4) {
				result = _selectOptionList[3]->GetJumpToLabel(); break;
			}
			if (_selectOptionList.size() > 4 && Application::GetInstance()->_releasedKeyCode == sf::Keyboard::F5) {
				result = _selectOptionList[4]->GetJumpToLabel(); break;
			}
			if (_selectOptionList.size() > 5 && Application::GetInstance()->_releasedKeyCode == sf::Keyboard::F6) {
				result = _selectOptionList[5]->GetJumpToLabel(); break;
			}
			if (_selectOptionList.size() > 6 && Application::GetInstance()->_releasedKeyCode == sf::Keyboard::F7) {
				result = _selectOptionList[6]->GetJumpToLabel(); break;
			}
			if (_selectOptionList.size() > 7 && Application::GetInstance()->_releasedKeyCode == sf::Keyboard::F8) {
				result = _selectOptionList[7]->GetJumpToLabel(); break;
			}
			if (_selectOptionList.size() > 8 && Application::GetInstance()->_releasedKeyCode == sf::Keyboard::F9) {
				result = _selectOptionList[8]->GetJumpToLabel(); break;
			}
			if (_selectOptionList.size() > 9 && Application::GetInstance()->_releasedKeyCode == sf::Keyboard::F10) {
				result = _selectOptionList[9]->GetJumpToLabel(); break;
			}
			if (_selectOptionList.size() > 10 && Application::GetInstance()->_releasedKeyCode == sf::Keyboard::F11) {
				result = _selectOptionList[10]->GetJumpToLabel(); break;
			}
			if (_selectOptionList.size() > 11 && Application::GetInstance()->_releasedKeyCode == sf::Keyboard::F12) {
				result = _selectOptionList[11]->GetJumpToLabel(); break;
			}
			if (_selectOptionList.size() > 12 && Application::GetInstance()->_releasedKeyCode == sf::Keyboard::F13) {
				result = _selectOptionList[12]->GetJumpToLabel(); break;
			}
			if (_selectOptionList.size() > 13 && Application::GetInstance()->_releasedKeyCode == sf::Keyboard::F14) {
				result = _selectOptionList[13]->GetJumpToLabel(); break;
			}
			if (_selectOptionList.size() > 14 && Application::GetInstance()->_releasedKeyCode == sf::Keyboard::F15) {
				result = _selectOptionList[14]->GetJumpToLabel(); break;
			}

		} while (false);

	}
	
	
	///// コンマ区切りで複数のラベルをランダムに選択できるのです
	if ( result != "" ) {
		
		vector<string> labels = Utils_String::Split(result, ",");
		
		int randomIndex = Utils_Cal::GerRandom ( labels.size() );
		
		result = labels[randomIndex];
		
	}
	
	

	return result;
}
//=========================================================
// 選択された選択肢の文字列を取得：未選択なら空文字
//=========================================================
string Select::GetSelectText() {

	/*
		選択肢が存在しない場合も、自動的に空文字が返る。
	*/

	string result = "";

	for ( unsigned int i = 0; i < _selectOptionList.size(); ++i ) {

		if (_selectOptionList[i]->IsSelected()) {

			result = _selectOptionList[i]->GetText();
			break;
		}

	}

	return result;
}


//=========================================================
// マウスが乗っている選択肢のジャンプ先ラベルを取得
//=========================================================
string Select::GetMouseoverOptionJumpLabel() {

	string result = "";

	for ( unsigned int i = 0; i < _selectOptionList.size(); ++i ) {

		if (_selectOptionList[i]->IsMouseover()) {

			result = _selectOptionList[i]->GetJumpToLabel();
			break;
		}

	}


	return result;
}

//=========================================================
// セーブ用のデータを作成
//=========================================================
Select::SaveData Select::GetSaveData () {

	Select::SaveData result;

	for ( unsigned int i = 0; i < _selectOptionList.size(); ++i ) {

		string text        = _selectOptionList[i]->GetText();
		string jumpToLabel = _selectOptionList[i]->GetJumpToLabel();

		pair < string, string> p;
		p.first  = text;
		p.second = jumpToLabel;

		result.optionList.push_back ( p );
		
		
		AI_Modules::Move mv = _selectOptionList[i]->GetMove();
		result.moveList.push_back ( mv );
		
	}

	result.positionX = _posX_forSave;
	result.positionY = _posY_forSave;
	result.gameBoardId = _gameBoardId;

	return result;
}
//=========================================================
// ロード
//=========================================================
void Select::Load ( SaveData &saveData ) {

	///// 一応、お掃除しておきまして、
	Clear();

	vector < SelectOption*> optionList;
	vector < AI_Modules::Move > moveList;

	for ( unsigned int i = 0; i < saveData.optionList.size(); ++i ) {

		string text        = saveData.optionList[i].first;
		string jumpToLabel = saveData.optionList[i].second;

		SelectOption *option = new SelectOption ( text, jumpToLabel );


		optionList.push_back (option);
	}

	_gameBoardId = saveData.gameBoardId;
	Move
    (
        saveData.positionX,
        saveData.positionY
    );

	///// さぁ、セットアップだ！
	SetUpSelectOptions ( optionList );
	
	
	///// セットアップできているハズのものへ、指し手タグを入れる
	for ( unsigned int i = 0; i < _selectOptionList.size(); ++i ) {
        
		AI_Modules::Move mv = saveData.moveList[i];

		_selectOptionList[i]->SetMove ( mv );
        
	}
	
}















