//=========================================================
//
// 選択肢クラス：１つの選択肢
//
//=========================================================

#include "SelectOption.h"
#include "Application.h"
#include "../GUI_SFML/TextImageFactory.h"


//=========================================================
// 内部定数
//=========================================================
const int SelectOption::MOUSE_STATE_NONE     = 0; // 押していない
const int SelectOption::MOUSE_STATE_PRESS    = 1; // 押している
const int SelectOption::MOUSE_STATE_RELEASE  = 2; // 放したところ


//=========================================================
// コンストラクタ
//=========================================================
SelectOption::SelectOption ( string text, string jumpToLabel ) {
	Init(text, jumpToLabel, InitProperties());
}

SelectOption::SelectOption ( string text, string jumpToLabel, InitProperties initProp ) {
	Init(text, jumpToLabel, initProp);
}

//=========================================================
// 初期設定
//=========================================================
void SelectOption::Init ( string text, string jumpToLabel, InitProperties initProp ) {

	_text        = text;
	_jumpToLabel = jumpToLabel;

	///// 文言の周りに、マージンが欲しい？
	sf::Vector2f _marginSize = initProp.marginSize;

	///// 文言
	// [[deprecated("怪しい実装：sf::Text を囲む矩形のサイズと位置がままならず、マジックナンバーでごまかしている。")]]
	_textImage = TextImageFactory::Create ( text, initProp.fontSize);
	_textImage.setColor  ( initProp.fontColor  );
	_textImage.setOrigin ( initProp.textOrigin  );
	
	///// 背景
	_backRect = sf::RectangleShape (
		sf::Vector2f ( 
			_textImage.getGlobalBounds().width + initProp.backRectWidthAdjustValue,
			_textImage.getGlobalBounds().height
		) + _marginSize
	);
	_backRect.setFillColor        ( initProp.backRectFillColor        );
	_backRect.setOutlineThickness ( initProp.backRectOutlineThickness );
	_backRect.setOutlineColor     ( initProp.backRectOutlineColor     );
	_backRect.setOrigin           ( _marginSize / 2.0f                );
	
	///// カーソル
	_cursorRect = sf::RectangleShape (
		sf::Vector2f ( 
			_textImage.getGlobalBounds().width,
			_textImage.getGlobalBounds().height
		) + _marginSize
	);
	_cursorRect.setFillColor        ( initProp.cursorRectFillColor        );
	_cursorRect.setOutlineThickness ( initProp.cursorRectOutlineThickness );
	_cursorRect.setOutlineColor     ( initProp.cursorRectOutlineColor     );
	_cursorRect.setOrigin           ( _marginSize / 2.0f                  );


	_mouseState  = MOUSE_STATE_NONE;
	_isMouseover = false;
}
/*
void SelectOption::Init ( string text, string jumpToLabel, InitProperties initProp ) {

	_text        = text;
	_jumpToLabel = jumpToLabel;

	///// 文言の周りに、マージンが欲しい？
	sf::Vector2f _marginSize ( 10.0f, 10.0f );

	///// 文言
	//TextImageFactory::Create ( _image_Text.image, text, 30, sf::Color(255,255,255,255));
	//_image_Text.texture.loadFromImage ( _image_Text.image );
	//_image_Text.sprite.setTexture ( _image_Text.texture );
	
	// [[deprecated("怪しい実装：sf::Text を囲む矩形のサイズと位置がままならず、マジックナンバーでごまかしている。")]]
	_textImage = TextImageFactory::Create ( text, 30);
	_textImage.setColor ( sf::Color(255,255,255,255) );
	_textImage.setOrigin (sf::Vector2f(0.0f, 5.0f)  );
	
	///// 背景
	_backRect = sf::RectangleShape (
		sf::Vector2f ( 
			//_image_Text.image.getSize().x,
			//_image_Text.image.getSize().y
			_textImage.getGlobalBounds().width + 10,
			_textImage.getGlobalBounds().height
		) + _marginSize
	);
	_backRect.setFillColor ( sf::Color ( 50,50,50,150 ) );
	_backRect.setOutlineThickness (0.0f);
	_backRect.setOutlineColor ( sf::Color ( 0,0,0,0 ) );
	_backRect.setOrigin ( _marginSize / 2.0f);
	
	///// カーソル
	_cursorRect = sf::RectangleShape (
		sf::Vector2f ( 
			//_image_Text.image.getSize().x,
			//_image_Text.image.getSize().y
			_textImage.getGlobalBounds().width,
			_textImage.getGlobalBounds().height
		) + _marginSize
	);
	_cursorRect.setFillColor ( sf::Color ( 200,200,255,180 ) );
	_cursorRect.setOutlineThickness (0.0f);
	_cursorRect.setOutlineColor ( sf::Color ( 0,0,0,0 ) );
	_cursorRect.setOrigin ( _marginSize / 2.0f );


	_mouseState  = MOUSE_STATE_NONE;
	_isMouseover = false;
}
*/

//=========================================================
// デストラクタ
//=========================================================
SelectOption::~SelectOption() {

}

//=========================================================
// 更新
//=========================================================
void SelectOption::Update ( sf::RenderWindow &window ) {

	//_image_Text.sprite.setPosition ( _position );
	_backRect  .setPosition ( _position );
	_cursorRect.setPosition ( _position );
	
	_textImage.setPosition ( _position );
	
	//printf ( "%s(%f, %f)\n", _text.c_str() , _position.x, _position.y );
	
	///// マウス押下状態
	ObserveMouseState (window);


	///// マウス、乗ってる？
	int mouseX = sf::Mouse::getPosition ( window ).x;
	int mouseY = sf::Mouse::getPosition ( window ).y;
	
	// カメラの位置に合わせてマウスの位置を解釈しなおす
	sf::Vector2i mousePos = sf::Vector2i(mouseX, mouseY);
	sf::Vector2f mouseInnerPos =  Application::GetInstance()->_window.mapPixelToCoords( mousePos );
	mouseX = mouseInnerPos.x;
	mouseY = mouseInnerPos.y;
	
	
	if 
	( 
		_position.x <= (float)mouseX && (float)mouseX <= _position.x + _backRect.getSize().x  &&
		_position.y <= (float)mouseY && (float)mouseY <= _position.y + _backRect.getSize().y  
	) 
	{
		_isMouseover = true;
	} else {
		_isMouseover = false;
	}

	//printf("(%d, %d)\n", mouseX, mouseY);
}

//=========================================================
// 描画
//=========================================================
void SelectOption::Draw  ( sf::RenderWindow &window ) {

	window.draw ( _backRect  );
	window.draw ( _textImage );
	//window.draw ( _image_Text.sprite  );

	if (_isMouseover) {
		window.draw( _cursorRect );
	}

}

//=========================================================
// マウス乗っている？
//=========================================================
bool SelectOption::IsMouseover() {
	
	return _isMouseover;
}

//=========================================================
// 選択された？（マウス押された？）
//=========================================================
bool SelectOption::IsSelected () {

	bool result = false;


	///// マウスが乗っていて、押されたなら、
	result = (_isMouseover && ( _mouseState == MOUSE_STATE_RELEASE ) );

	return result;
}


//=========================================================
// マウスの状態を監視
//=========================================================
void SelectOption::ObserveMouseState ( sf::RenderWindow &window ) {


	///// ウィンドウが最前面じゃないときには、マウス操作を受け付けない。
	/*
	HWND handle     =  window.getSystemHandle();
	HWND foreWindow = GetForegroundWindow();

	bool isTopWindow = ( handle == foreWindow );
	*/

	if ( !Application::GetInstance()->HasFocus() ) {
		_mouseState = MOUSE_STATE_NONE;
		return;
	}


	if ( _mouseState == MOUSE_STATE_NONE ) {

		if ( sf::Mouse::isButtonPressed ( sf::Mouse::Left ) ) {
			_mouseState = MOUSE_STATE_PRESS;
		}
	}
	else if ( _mouseState == MOUSE_STATE_PRESS ) {

		if ( !sf::Mouse::isButtonPressed ( sf::Mouse::Left ) ) {
			_mouseState = MOUSE_STATE_RELEASE;
		}
	}
	else if ( _mouseState == MOUSE_STATE_RELEASE ) {

		_mouseState = MOUSE_STATE_NONE;
	}

}

