//=========================================================
//
// 選択肢クラス：１つの選択肢
//
//=========================================================

#ifndef _SELECT_OPTION_
#define _SELECT_OPTION_

#include <SFML/Graphics.hpp>
#include <string>

#include "../Utils/NotXML.h"
#include "../AI/Define.h"
#include "../AI/Move.h"

using namespace std;

class SelectOption {

public:
	/*
	struct ImageSet {
		sf::Image   image;
		sf::Texture texture;
		sf::Sprite  sprite;

		ImageSet () {
		}
	};
	*/

	struct InitProperties {

		sf::Vector2f marginSize;
		int          fontSize;
		sf::Color    fontColor;
		sf::Vector2f textOrigin;

		int backRectWidthAdjustValue;

		sf::Color  backRectFillColor;
		float      backRectOutlineThickness;
		sf::Color  backRectOutlineColor;

		sf::Color   cursorRectFillColor;
		float       cursorRectOutlineThickness;
		sf::Color   cursorRectOutlineColor;

		InitProperties () {
			marginSize = sf::Vector2f( 10.0f, 10.0f );
			fontSize = 30;
			fontColor = sf::Color(255,255,255,255);
			textOrigin = sf::Vector2f(0.0f, 5.0f);
			backRectWidthAdjustValue = 10;

			backRectFillColor = sf::Color ( 50,50,50,150 );
			backRectOutlineThickness = 0.0f;
			backRectOutlineColor = sf::Color ( 0,0,0,0 );

			cursorRectFillColor = sf::Color ( 200,200,255,180 );
			cursorRectOutlineThickness = 0.0f;
			cursorRectOutlineColor = sf::Color ( 0,0,0,0 );
		}
	};

	SelectOption ( string text, string jumpToLabel );
	SelectOption ( string text, string jumpToLabel, InitProperties initProp );
	void Init ( string text, string jumpToLabel, InitProperties initProp );
	~SelectOption();

	void Update ( sf::RenderWindow &window );
	void Draw   ( sf::RenderWindow &window );

	bool IsSelected (); // 選択された？（マウス押された？）
	string GetJumpToLabel() { return _jumpToLabel; }
	bool IsMouseover();

	void SetPosition  ( sf::Vector2f value ) { _position = value; }
	void SetPositionX ( int value ) { SetPosition ( sf::Vector2f( (float)value, _position.y ) ); }
	void SetPositionY ( int value ) { SetPosition ( sf::Vector2f( _position.x, (float)value ) ); }
	sf::Vector2f GetPosition  () { return _position; }

	int GetSizeX() { return _backRect.getSize().x; }
	int GetSizeY() { return _backRect.getSize().y; }


	string GetText() { return _text; }

	void SetSizeX ( int value ) {
		_backRect.setSize (
			sf::Vector2f( (float)value, _backRect.getSize().y)
		);
		_cursorRect.setSize (
			sf::Vector2f( (float)value, _cursorRect.getSize().y)
		);
	}

	void SetMove ( AI_Modules::Move move ) { _move = move; }
	AI_Modules::Move GetMove() { return _move; }
	//void SetMoveTag ( NotXML::Tag moveTag ) { _moveTag = moveTag; }
	//NotXML::Tag GetMoveTag () { return _moveTag; }

private:

	//ImageSet _image_Text;           // 表示するイメージ：文言
	sf::Text _textImage;            // 表示するイメージ：文言（sf::Text)
	sf::RectangleShape _backRect;   // 背景
	sf::RectangleShape _cursorRect; // カーソル

	//ImageSet _Image_Back;    // 表示するイメージ：背景
	//ImageSet _Image_Cursor;  // 表示するイメージ：カーソル（マウスオーバー時）

	string _text;         // 表示する文言
	string _jumpToLabel;  // クリックされたときのジャンプ先

	sf::Vector2f _position;

	int _mouseState;

	bool _isMouseover;

    // 指し手タグ（次の一手の選択肢用）
	//NotXML::Tag _moveTag;
	AI_Modules::Move _move;


	//=========================================================
	// 内部定数
	//=========================================================
	static const int MOUSE_STATE_NONE;    // 押していない
	static const int MOUSE_STATE_PRESS;   // 押している
	static const int MOUSE_STATE_RELEASE; // 放したところ

	//=========================================================
	// マウスの状態を監視
	//=========================================================
	void ObserveMouseState ( sf::RenderWindow &window );


};

#endif
