//=========================================================
//
// 効果音管理クラス
//
//=========================================================

#include "SoundManager.h"
#include "../Utils/Utils_Cal.h"

//=========================================================
// コンストラクタ
//=========================================================
SoundManager::SoundManager() {
	
	_volume = 100.0f;
	
	///// チャンネルを用意しておく。いくつ？ 適当に。
	for ( int i = 0; i < 10; ++i ) {
		sf::Sound *sound = new sf::Sound();
		_soundList.push_back( sound );
	}
}

//=========================================================
// デストラクタ
//=========================================================
SoundManager::~SoundManager() {

	while ( _soundBufferList.size() > 0 ) {
		delete _soundBufferList.begin()->second;
		_soundBufferList.erase ( _soundBufferList.begin() );
	}

	while ( _soundList.size() > 0 ) {
		delete *_soundList.begin();
		_soundList.erase ( _soundList.begin() );
	}
}


//=========================================================
// 音量設定
//=========================================================
void SoundManager::SetVolume ( float value ) {
	
	_volume = value;
	
	for (unsigned int i = 0; i < _soundList.size(); ++i) {
		
		_soundList[i]->setVolume ( _volume );
		
	}
	
}
float SoundManager::GetVolume() {
	return _volume;
}
	

//=========================================================
// 鳴らす
//=========================================================
void SoundManager::Play ( string fileName, bool preload ) {

	///// 未登録？
	map<string, sf::SoundBuffer*>::iterator it = _soundBufferList.find ( fileName );
	if ( it == _soundBufferList.end() ) {

		// 登録しましょう。まずは、読み込んで。
		sf::SoundBuffer *buf = new sf::SoundBuffer();
		buf->loadFromFile ( fileName );

		// なんか、すっごくいっぱい登録されてるんですけど
		if ( _soundBufferList.size() > 10 ) {

			// 適当に消しといて
			// map なので順番がどうなってるかわからないのは百も承知の上で。
			delete _soundBufferList.begin()->second;
			_soundBufferList.erase ( _soundBufferList.begin() );
		}

		// 登録しました。
		_soundBufferList[ fileName ] = buf;
	}


	///// 鳴らします？ 登録するだけじゃなくて。
	if ( !preload ) {

		// この段階では、あるに決まってる。決まってますよね。
		sf::SoundBuffer *buf = _soundBufferList.find ( fileName )->second;

		// あいてるチャンネルがあるといいな。
		bool isBlankChannenExist = false;

		// あいてるチャンネルはどこかな？
		for ( unsigned int i = 0; i < _soundList.size(); ++i ) {

			// あっ、空いてるみたいだよ！
			if (_soundList[i]->getStatus() == sf::SoundSource::Stopped) {

				// 鳴らそう！ 鳴らそう！
				_soundList[i]->setBuffer ( *buf );
				_soundList[i]->play();

				// わーいわーい。
				isBlankChannenExist = true;
				break;
			}
		}

		// どこも空いてなかったの？
		if ( !isBlankChannenExist ) {

			// じゃあ、どうしましょう。適当に割り込みましょうか。イヒヒ。
			int lot = Utils_Cal::GerRandom ( _soundList.size() );
			_soundList[lot]->setBuffer( *buf );
		}
	}

}

//=========================================================
// ラベルからファイル名を検索して鳴らす
//=========================================================
void SoundManager::Play_FromLabel ( string label, bool preload ) {

	///// さて、ラベルが登録されてるかな？
	if ( _labelList.find(label) != _labelList.end() ) {

		string fileName = _labelList [ label ];

		Play ( fileName, preload );
	}
}

//=========================================================
// ラベルを登録
//=========================================================
void SoundManager::Regist ( string label, string fileName ) {

	///// 同じのがあったら、一応、ワーニングを出してあげる。
	if ( _labelList.find(label) != _labelList.end() ) {
		printf ( "効果音ラベルが重複しています。上書き登録します。%s=%s\n", label.c_str(), fileName.c_str() );
	}

	_labelList[label] = fileName;

}


//=========================================================
// セーブデータ作成
//=========================================================
SoundManager::SaveData SoundManager::GetSaveData () {

	SoundManager::SaveData result;

	//result.labelList = _labelList;

	return result;
}
//=========================================================
// セーブデータを元に再現
//=========================================================
void SoundManager::Load ( SaveData &saveData ) {

	//_labelList = saveData.labelList;
}
