//=========================================================
//
// カットインエフェクト：COME_AND_GO
//
//=========================================================

#include "Effect_CutIn_COME_AND_GO.h"
#include "Define.h"
#include "../Utils/Utils_Cal.h"
#include "../Utils/Utils_Convert.h"
#include "../GUI_SFML/SoundPlayer.h"
#include "../Utils/Utils_Others.h"


//=========================================================
// コンストラクタ
//=========================================================
Effect_CutIn_COME_AND_GO::Effect_CutIn_COME_AND_GO ( DelegateBase *endMethod, int effectLevel, GUI_SFML::TextMessageWindow *textMessageWindow, map<string, string> parameters ) {

	printf ("カットインエフェクト：COME_AND_GO\n");

	// 引数を受け取る
	_textMessageWindow = textMessageWindow;
	_endMethod         = endMethod;
	_effectLevel       = effectLevel;

	// キャラ画像・パート1〜3用
	_fameImageMask  = sf::Color( 0, 255, 0, 255 );
	_faceImageFile1 = UTILS_OTHERS::GetValueFromMap ( parameters, "faceImage1", "" );
	_faceImageFile2 = UTILS_OTHERS::GetValueFromMap ( parameters, "faceImage2", _faceImageFile1 );
	_faceImageFile3 = UTILS_OTHERS::GetValueFromMap ( parameters, "faceImage3", _faceImageFile1 );
	
	// セリフの名札
	_nameLabel = UTILS_OTHERS::GetValueFromMap ( parameters, "nameLabel", "" );
	
	// セリフ・パート1〜3開始時
	_phrase1 = UTILS_OTHERS::GetValueFromMap ( parameters, "phrase1", "" );
	_phrase2 = UTILS_OTHERS::GetValueFromMap ( parameters, "phrase2", "" );
	_phrase3 = UTILS_OTHERS::GetValueFromMap ( parameters, "phrase3", "" );

	//効果音・パート1〜3開始時
	_sound1 = UTILS_OTHERS::GetValueFromMap ( parameters, "sound1", "" );
	_sound2 = UTILS_OTHERS::GetValueFromMap ( parameters, "sound2", "" );
	_sound3 = UTILS_OTHERS::GetValueFromMap ( parameters, "sound3", "" );

	// 1フレーム毎の移動増分（速度）
	_speed1to2 = Utils_Cal::Abs ( Utils_Convert::String_to_Int ( UTILS_OTHERS::GetValueFromMap ( parameters, "speed1to2", "20" ) ) );
	_speed2    =                  Utils_Convert::String_to_Int ( UTILS_OTHERS::GetValueFromMap ( parameters, "speed2"   , "0"  ) );
	_speed2to3 = Utils_Cal::Abs ( Utils_Convert::String_to_Int ( UTILS_OTHERS::GetValueFromMap ( parameters, "speed2to3", "20" ) ) );

	// パート2に停止しているフレーム数
	_part2FrameCount = Utils_Convert::String_to_Int ( UTILS_OTHERS::GetValueFromMap ( parameters, "part2FrameCount",   "30" ) );

	// パート2として停止する位置(X)
	_part2StayPositionX = Utils_Convert::String_to_Int ( UTILS_OTHERS::GetValueFromMap ( parameters, "part2StayPositionX",  Utils_Convert::Int_to_String((APPLICATION::WINDOW_SIZE::WIDTH / 2.0f)) ) );

	// 全体の位置Y
	_positionY = Utils_Convert::String_to_Int ( UTILS_OTHERS::GetValueFromMap ( parameters, "positionY", "300") );
	
	// 移動方向 
	_direction = UTILS_OTHERS::GetValueFromMap ( parameters, "direction", "left_to_right" );

	// 移動方向に合わせて、開始位置と終了位置を求める
	if ( _direction == "left_to_right" ) {
		
		_position_Start = sf::Vector2f ( -200, _positionY );
		_position_End   = sf::Vector2f ( APPLICATION::WINDOW_SIZE::WIDTH + 200 , _positionY );

	} else {
		_position_Start = sf::Vector2f ( APPLICATION::WINDOW_SIZE::WIDTH + 200, _positionY );
		_position_End   = sf::Vector2f ( -200, _positionY );
	}

	// 追加アニメーションエフェクト・ファイル名で指定・パート1〜3開始時
	_animation1 = UTILS_OTHERS::GetValueFromMap ( parameters, "animation1", "" );
	_animation2 = UTILS_OTHERS::GetValueFromMap ( parameters, "animation2", "" );
	_animation3 = UTILS_OTHERS::GetValueFromMap ( parameters, "animation3", "" );

	// 背景として使用する画像
	_backImageAlpha = Utils_Convert::String_to_Int ( UTILS_OTHERS::GetValueFromMap ( parameters, "backImageAlpha", "200" ) );
	_backImageFile  = UTILS_OTHERS::GetValueFromMap ( parameters, "backImage", "" );

	_state = "";
	_frameCount = 0;
	_isEnd = false;
}

//=========================================================
// デストラクタ
//=========================================================
Effect_CutIn_COME_AND_GO::~Effect_CutIn_COME_AND_GO() {

	printf ("カットインエフェクト COME_AND_GO 終了\n");

	///// 消しちゃいますよ。消しちゃいますよー。消してるってことを忘れないで！ 忘れないで！
	//delete _textMessageWindow;

}

//=========================================================
// パート１の開始準備処理
//=========================================================
void Effect_CutIn_COME_AND_GO::Preparation_forPart1() {

	///// パート1用の顔画像を作成
	_imageFace.image.loadFromFile        ( _faceImageFile1 );
	_imageFace.image.createMaskFromColor ( _fameImageMask ); 
	_imageFace.texture.loadFromImage     (_imageFace.image);
	_imageFace.sprite.setTexture         (_imageFace.texture);
	_imageFace.sprite.setOrigin ( _imageFace.image.getSize().x / 2, _imageFace.image.getSize().y / 2);

	///// パート１の準備なので、背景画像も準備するの。
	_imageBack.image.loadFromFile    ( _backImageFile );
	_imageBack.texture.loadFromImage (_imageBack.image);
	_imageBack.sprite.setTexture     (_imageBack.texture);
	_imageBack.sprite.setColor       ( sf::Color ( 255,255,255,_backImageAlpha ) );

	///// 背景画像・部分矩形
	_rect_Back.left = 0;
	_rect_Back.top  = 0;
	_rect_Back.width  = _imageBack.image.getSize().x;
	_rect_Back.height = _imageFace.image.getSize().y; // 顔画像の高さに合わせる。
	_imageBack.sprite.setTextureRect ( _rect_Back );
	

	_positionBack = sf::Vector2f ( 0, _positionY - _imageFace.image.getSize().y / 2 );
	_positionFace = _position_Start;

	_imageBack.sprite.setPosition ( _positionBack );
	_imageFace.sprite.setPosition ( _positionFace );


	// 効果音
	GUI_SFML::SoundPlayer::Play ( _sound1 );

	// セリフ
	if ( _textMessageWindow != NULL && _phrase1 != "") {
		
		_textMessageWindow->SetSpeakerName ( _nameLabel );
			
		// 必ず完了済みのテキストとして。一言で言い切ってくれないとエフェクトとして困る、ということで。
		// リアクションの発言とは扱いが違うのですよ、ということ。
		_textMessageWindow->AddText_asFinishedText ( _phrase1 );
	}
}
//=========================================================
// パート２の開始準備処理
//=========================================================
void Effect_CutIn_COME_AND_GO::Preparation_forPart2() {

	///// パート2用の顔画像を作成
	if ( _faceImageFile2 != _faceImageFile1 ) {
		_imageFace.image.loadFromFile        ( _faceImageFile2 );
		_imageFace.image.createMaskFromColor ( _fameImageMask ); 
		_imageFace.texture.loadFromImage     (_imageFace.image);
		_imageFace.sprite.setTexture         (_imageFace.texture);
		_imageFace.sprite.setOrigin ( _imageFace.image.getSize().x / 2, _imageFace.image.getSize().y / 2);
	}
	_imageFace.sprite.setPosition ( _positionFace );


	// 効果音
	GUI_SFML::SoundPlayer::Play ( _sound2 );

	// セリフ
	if ( _textMessageWindow != NULL && _phrase2 != "") {
		_textMessageWindow->SetSpeakerName ( _nameLabel );
		_textMessageWindow->AddText_asFinishedText ( _phrase2 );
	}
}
//=========================================================
// パート３の開始準備処理
//=========================================================
void Effect_CutIn_COME_AND_GO::Preparation_forPart3() {

	///// パート3用の顔画像を作成
	if ( _faceImageFile3 != _faceImageFile2 ) {
		_imageFace.image.loadFromFile        ( _faceImageFile3 );
		_imageFace.image.createMaskFromColor ( _fameImageMask ); 
		_imageFace.texture.loadFromImage     (_imageFace.image);
		_imageFace.sprite.setTexture         (_imageFace.texture);
		_imageFace.sprite.setOrigin ( _imageFace.image.getSize().x / 2, _imageFace.image.getSize().y / 2);
	}
	_imageFace.sprite.setPosition ( _positionFace );


	// 効果音
	GUI_SFML::SoundPlayer::Play ( _sound3 );

	// セリフ
	if ( _textMessageWindow != NULL && _phrase3 != "") {
		_textMessageWindow->SetSpeakerName ( _nameLabel );
		_textMessageWindow->AddText_asFinishedText ( _phrase3 );
	}
}


//=========================================================
// 更新
//=========================================================
bool Effect_CutIn_COME_AND_GO::Update(sf::RenderWindow &window) {

	if ( _isEnd ) { return _isEnd; }

	
	///// 背景を動かす
	_rect_Back.top += 100;
	if ( _rect_Back.top + _rect_Back.height > _imageBack.image.getSize().y ) {
		_rect_Back.top = 0;
	}
	_imageBack.sprite.setTextureRect ( _rect_Back );



	///// 状態：初期
	if ( _state == "" ) {

		// パート１の準備をする
		Preparation_forPart1();

		// パート１へ。
		_state = "1";

	///// 状態：パート１
	} else if ( _state == "1" ) {

		// 顔画像が移動する
		_positionFace.x = (int)Utils_Cal::Nearlize ( _positionFace.x, _part2StayPositionX, _speed1to2 );

		// 真ん中まで来た？
		if ( _positionFace.x == _part2StayPositionX ) {

			// パート２の準備をする
			Preparation_forPart2();

			_state = "2";
			_frameCount = 0;
		}

	///// 状態；パート２
	} else if ( _state == "2" ) {

		_frameCount += 1;

		_positionFace.x += _speed2;

		///// フレーム数めいっぱい？
		if ( _frameCount >= _part2FrameCount ) {

			///// パート３の準備
			Preparation_forPart3();

			_state = "3";
		}

	///// 状態：パート３
	} else if ( _state == "3" ) {

		_positionFace.x = (int)Utils_Cal::Nearlize ( _positionFace.x, _position_End.x, _speed2to3 );

		// めいっぱい？
		if ( _positionFace.x == _position_End.x ) {

			// 着手メソッドを実行する
			if ( _endMethod != NULL ) {
				(*_endMethod)( _effectLevel );
			}

			// 状態４（終了）へ
			_state = "4";
		}

	///// 終了
	} else if ( _state == "4" ) {

		_isEnd = true;
	}


	_imageBack.sprite.setPosition ( _positionBack );
	_imageFace.sprite.setPosition ( _positionFace );


	if ( _textMessageWindow != NULL ) {
		_textMessageWindow->Update(window);
	}

	return _isEnd;
}

//=========================================================
// 描画
//=========================================================
void Effect_CutIn_COME_AND_GO::Draw(sf::RenderWindow &window) {

	window.draw ( _imageBack.sprite  );
	window.draw ( _imageFace.sprite  );


	if ( _textMessageWindow != NULL ) {
		_textMessageWindow->Draw(window);
	}

}







