//=========================================================
//
// 着手エフェクト＿駒移動（ぶつけない）
//
//=========================================================

#include "Effect_MovePiece.h"

#include "Piece.h"
#include "Grid.h"
#include "../GUI_SFML/SoundPlayer.h"
#include "Effect_SimpleParticle.h"
#include "Effect_Animation.h"
#include "Application.h"
#include "EffectManager.h"
#include "Effect_ImpactWave_Rash.h"
#include "Define.h"


//=========================================================
// 動作のステージ
//=========================================================
const int Effect_MovePiece::STATE_0_BEGGINING_TO_GO = 0; // 駒が動き始める
const int Effect_MovePiece::STATE_0_GOING           = 1; // 駒が動いている
const int Effect_MovePiece::STATE_0_ARRIVE_AT       = 2; // 駒が移動先に到着した

const int Effect_MovePiece::STATE_1_BEGGINING_TO_GO = 10; // 駒が動き始める
const int Effect_MovePiece::STATE_1_GOING           = 11; // 駒が動いている
const int Effect_MovePiece::STATE_1_ARRIVE_AT       = 12; // 駒が移動先に到着した

const int Effect_MovePiece::STATE_2_BEGGINING_TO_GO       = 20; // 光出現
const int Effect_MovePiece::STATE_2_ENTERING_TO_LIGHT     = 21; // 光に吸い込まれている
const int Effect_MovePiece::STATE_2_WARPING               = 22; // 移動中というか、吸い込まれ終わっている
const int Effect_MovePiece::STATE_2_SECOND_LIGHT_APPEARS  = 23; // 移動先に光出現
const int Effect_MovePiece::STATE_2_SECOND_LIGHT_GOING_ON = 24; // 移動先に光が出現している最中

const int Effect_MovePiece::STATE_3_SETUP           = 30; // 初回処理。集中線エフェクト開始
const int Effect_MovePiece::STATE_3_BEFORE_MOVING   = 31; // 動き始める前。集中線エフェクト
const int Effect_MovePiece::STATE_3_GOING           = 32; // 動いている
const int Effect_MovePiece::STATE_3_ARRIVED         = 33; // 移動先に到着。衝撃波エフェクト

const int Effect_MovePiece::STATE_FINISHED          = 3; // レベル共通：終了している。


//=========================================================
// コンストラクタ
//=========================================================
Effect_MovePiece::Effect_MovePiece (
		
		DelegateBase     *endMethod,
		AI_Modules::Move  move,
		int               effectLevel,
		Grid             *grid_From,   // このグリッドから
		Grid             *grid_To,     // このグリッドへ移動
		Piece            *piece        // この駒を移動
) 
{

	///// 引数を受け取って
	_endMethod   = endMethod;
	_grid_From   = grid_From;
	_grid_To     = grid_To;
	_piece       = piece;
	_effectLevel = effectLevel;
	_move        = move;

	///// 初期状態
	_isEnd        = false;
	_frameCount_SinceCurrentState = 0;

	///// エフェクトレベル、操作する？
	if ( effectLevel < 0 ) {

		_effectLevel = 0;
		if      ( sf::Keyboard::isKeyPressed ( sf::Keyboard::Num1 ) ) { _effectLevel = 1; }
		else if ( sf::Keyboard::isKeyPressed ( sf::Keyboard::Num2 ) ) { _effectLevel = 2; }
		else if ( sf::Keyboard::isKeyPressed ( sf::Keyboard::Num3 ) ) { _effectLevel = 3; }

	///// 操作しない：引数で渡されたまま、受け取る
	} else {
		_effectLevel = effectLevel;
	}

	///// エフェクトレベル別に、初期状態をセット。
	if        ( _effectLevel == 0 ) { _currentState = STATE_0_BEGGINING_TO_GO;
	} else if ( _effectLevel == 1 ) { _currentState = STATE_1_BEGGINING_TO_GO;
	} else if ( _effectLevel == 2 ) { _currentState = STATE_2_BEGGINING_TO_GO;
	} else if ( _effectLevel == 3 ) { _currentState = STATE_3_SETUP; }


	///// アニメーションエフェクトはみなさんNULLですよね。
	_animation_Concentration = NULL;
	//_animation_Impact        = NULL;
	_animation_LightPillar_Appear    = NULL;
	_animation_LightPillar_Disappear = NULL;

	printf ( "駒移動エフェクト_%d (引数 effectLevel:%d) 開始\n", _effectLevel, effectLevel );

}


//=========================================================
// デストラクタ
//=========================================================
Effect_MovePiece::~Effect_MovePiece() {

	printf ("駒移動エフェクト終了\n");

	delete _animation_Concentration;
	delete _animation_LightPillar_Appear;
	delete _animation_LightPillar_Disappear;
}

//=========================================================
// 更新
//=========================================================
bool Effect_MovePiece::Update(sf::RenderWindow &window) {

	if ( _isEnd ) { return _isEnd; }

	if      ( _effectLevel == 0 ) { Update_EffectLEvel_0 ( window ); }
	else if ( _effectLevel == 1 ) { Update_EffectLEvel_1 ( window ); }
	else if ( _effectLevel == 2 ) { Update_EffectLEvel_2 ( window ); }
	else if ( _effectLevel == 3 ) { Update_EffectLEvel_3 ( window ); }

	return _isEnd;
}


//=========================================================
// エフェクトレベル０の更新処理
//=========================================================
void Effect_MovePiece::Update_EffectLEvel_0 (sf::RenderWindow &window) {

	///// 状態：動き始め
	if ( _currentState == STATE_0_BEGGINING_TO_GO ) {

		// 駒を from から to のグリッドへ移動させる。
		_grid_To->SetPiece   ( _piece );
		_grid_From->SetPiece ( NULL );

		// 動き始めを表す効果音
		GUI_SFML::SoundPlayer::Play ( "./sound/se_good_pk01/swing29.wav" );

		// 状態遷移：移動中
		_currentState = STATE_0_GOING;

	///// 状態：移動中
	} else if ( _currentState == STATE_0_GOING ) {

		// 移動が終わったかな？
		if ( _piece->IsMovingStop() ) {

			// 状態遷移：到着
			_currentState = STATE_0_ARRIVE_AT;

		}

	///// 状態：到着
	} else if ( _currentState == STATE_0_ARRIVE_AT ) {

		// 到着を表す効果音
		//GUI_SFML::SoundPlayer::Play ( "./sound/se_good_pk01/step07.wav" );

		// 終了処理
		printf ("駒移動：終了処理\n");

		(*_endMethod)(_move);
		
		printf ("駒移動：終了処理後\n");
		
		_isEnd       = true;
		_currentState = STATE_FINISHED;

		printf ("駒移動：到着\n");

	///// 終了
	} else if ( _currentState == STATE_FINISHED ) {

		/*
			ここにはこないはず
		*/
		_isEnd = true;

	}

}
//=========================================================
// エフェクトレベル１の更新処理
//=========================================================
void Effect_MovePiece::Update_EffectLEvel_1 (sf::RenderWindow &window) {

	///// 毎フレーム、駒がある位置に旋風エフェクトを出す
	{
		///// ファイル名：移動方向によっては、反転バージョンのを使う
		string whirlEffectFile = "./effect/whirlWind/define.dat";
		if 
		( 
			_grid_From->GetPosition().x < _grid_To->GetPosition().x // 左から右へ
			||
			_grid_From->GetPosition().y > _grid_To->GetPosition().y // 下から上へ
		) 
		{
			whirlEffectFile = "./effect/whirlWind/define_reverse.dat";
		}

		///// エフェクトを生成して EffectManager に登録
		GUI_SFML::Animation *animation = new GUI_SFML::Animation ();
		animation->Init ( whirlEffectFile );
		animation->SetPosition ( _piece->GetCurrentPosition() );

		IEffect *effect = new Effect_Animation ( animation );
		Application::GetInstance()->_effectManager->Add ( effect );
	}

	///// 状態：動き始め
	if ( _currentState == STATE_1_BEGGINING_TO_GO ) {

		// 駒を from から to のグリッドへ移動させる。
		_grid_To->SetPiece   ( _piece );
		_grid_From->SetPiece ( NULL );

		// 動き始めを表す効果音
		GUI_SFML::SoundPlayer::Play ( "./sound/se_good_pk01/whip00.wav" );

		// 状態遷移：移動中
		_currentState = STATE_1_GOING;

	///// 状態：移動中
	} else if ( _currentState == STATE_1_GOING ) {

		// 移動が終わったかな？
		if ( _piece->IsMovingStop() ) {

			// 状態遷移：到着
			_currentState = STATE_1_ARRIVE_AT;

		}

	///// 状態：到着
	} else if ( _currentState == STATE_1_ARRIVE_AT ) {

		// 到着を表す効果音
		//GUI_SFML::SoundPlayer::Play ( "./sound/se_good_pk01/step07.wav" );

		// 終了処理
		(*_endMethod)(_move);
		_isEnd       = true;
		_currentState = STATE_FINISHED;

		printf ("駒移動：到着\n");

	///// 終了
	} else if ( _currentState == STATE_FINISHED ) {

		/*
			ここにはこないはず
		*/
		_isEnd = true;

	}

}
//=========================================================
// エフェクトレベル２の更新処理
//=========================================================
void Effect_MovePiece::Update_EffectLEvel_2 (sf::RenderWindow &window) {

	++_frameCount_SinceCurrentState;

	sf::Vector2f lightPillarOffset ( 0.0f, 12.0f );

	///// 状態：動き始め：光出現
	if ( _currentState == STATE_2_BEGGINING_TO_GO ) {

		_animation_LightPillar_Appear = new GUI_SFML::Animation();
		_animation_LightPillar_Appear->Init("./effect/lightPillar/define_Appear.dat");
		_animation_LightPillar_Appear->SetPosition ( _grid_From->GetCenterPosition() + lightPillarOffset );

		// 状態遷移：光に吸い込まれる
		_currentState = STATE_2_ENTERING_TO_LIGHT;
		_frameCount_SinceCurrentState = 0;

	///// 状態：光に吸い込まれる
	} else if ( _currentState == STATE_2_ENTERING_TO_LIGHT ) {

		// 光が出てきているところ？
		if ( _animation_LightPillar_Appear != NULL ) { 

			_animation_LightPillar_Appear->Update(window);

			// 光出現終了？
			if ( !_animation_LightPillar_Appear->IsActive() ) {

				delete _animation_LightPillar_Appear;
				_animation_LightPillar_Appear = NULL;

				// 光消滅エフェクト開始
				_animation_LightPillar_Disappear = new GUI_SFML::Animation();
				_animation_LightPillar_Disappear->Init("./effect/lightPillar/define_Disappear.dat");
				_animation_LightPillar_Disappear->SetPosition ( _grid_From->GetCenterPosition() + lightPillarOffset );

				_animation_LightPillar_Disappear->Update( window );

				// 駒を消すというか、マス目から外す：これで、誰からも描画されなくなる
				_grid_From->SetPiece ( NULL );
			}
		}

		// 光が消えているところ？
		if ( _animation_LightPillar_Disappear != NULL ) {

			_animation_LightPillar_Disappear->Update( window );

			// 光が消え終わった？
			if ( !_animation_LightPillar_Disappear->IsActive() ) {

				delete _animation_LightPillar_Disappear;
				_animation_LightPillar_Disappear = NULL;

				// 余韻エフェクト開始
				GUI_SFML::Animation *animation_LightPillar_Remain = new GUI_SFML::Animation();
				animation_LightPillar_Remain->Init("./effect/lightPillar/define_Remain.dat");
				animation_LightPillar_Remain->SetPosition( _grid_From->GetCenterPosition() );

				IEffect *effect = new Effect_Animation ( animation_LightPillar_Remain );
				Application::GetInstance()->_effectManager->Add ( effect );

				// 状態遷移：吸い込まれ終わった。移動中？
				_currentState = STATE_2_WARPING;
				_frameCount_SinceCurrentState = 0;
			}

		}

	///// 状態：吸い込まれ終わった。移動中？
	} else if ( _currentState == STATE_2_WARPING ) {

		// 少しタメを入れて、次の状態へ遷移
		if ( _frameCount_SinceCurrentState > 20 ) {

			_currentState = STATE_2_SECOND_LIGHT_APPEARS;
			_frameCount_SinceCurrentState = 0;
		}

	///// 状態：移動先に光出現
	} else if ( _currentState == STATE_2_SECOND_LIGHT_APPEARS ) {

		_animation_LightPillar_Appear = new GUI_SFML::Animation();
		_animation_LightPillar_Appear->Init("./effect/lightPillar/define_Appear.dat");
		_animation_LightPillar_Appear->SetPosition ( _grid_To->GetCenterPosition() + lightPillarOffset );

		_currentState = STATE_2_SECOND_LIGHT_GOING_ON;
		_frameCount_SinceCurrentState = 0;

	///// 状態：光から出てきているところ
	} else if ( _currentState == STATE_2_SECOND_LIGHT_GOING_ON ) {

		// 光が出てきているところ？
		if ( _animation_LightPillar_Appear != NULL ) { 

			_animation_LightPillar_Appear->Update(window);

			// 光出現終了？
			if ( !_animation_LightPillar_Appear->IsActive() ) {

				delete _animation_LightPillar_Appear;
				_animation_LightPillar_Appear = NULL;

				// 光消滅エフェクト開始
				_animation_LightPillar_Disappear = new GUI_SFML::Animation();
				_animation_LightPillar_Disappear->Init("./effect/lightPillar/define_Disappear.dat");
				_animation_LightPillar_Disappear->SetPosition ( _grid_To->GetCenterPosition() + lightPillarOffset );
				_animation_LightPillar_Disappear->Update( window );

				// 駒を移動先に出現させる
				_piece->SetPosition ( _grid_To->GetCenterPosition() );
				_grid_To->SetPiece ( _piece );
			}
		}

		// 光が消えているところ？
		if ( _animation_LightPillar_Disappear != NULL ) {

			_animation_LightPillar_Disappear->Update( window );

			// 光が消え終わった？
			if ( !_animation_LightPillar_Disappear->IsActive() ) {

				delete _animation_LightPillar_Disappear;
				_animation_LightPillar_Disappear = NULL;

				// 余韻エフェクト開始
				GUI_SFML::Animation *animation_LightPillar_Remain = new GUI_SFML::Animation();
				animation_LightPillar_Remain->Init("./effect/lightPillar/define_Remain.dat");
				animation_LightPillar_Remain->SetPosition( _grid_To->GetCenterPosition() );

				IEffect *effect = new Effect_Animation ( animation_LightPillar_Remain );
				Application::GetInstance()->_effectManager->Add ( effect );


				// 状態遷移：吸い込まれ終わった。移動中？
				_currentState = STATE_FINISHED;
				_frameCount_SinceCurrentState = 0;

				// ここで処理を戻す
				(*_endMethod)(_move);
			}

		}

	///// 終了
	} else if ( _currentState == STATE_FINISHED ) {

		/*
			ここにはこないはず
		*/
		_isEnd = true;

	}

}

//=========================================================
// エフェクトレベル３の更新処理
//=========================================================
void Effect_MovePiece::Update_EffectLEvel_3 (sf::RenderWindow &window) {

	///// 状態：初回処理。集中線エフェクトを開始
	if ( _currentState == STATE_3_SETUP ) {

		///// 集中線エフェクト開始
		_animation_Concentration = new GUI_SFML::Animation();
		_animation_Concentration->Init ( "./effect/movePiece_concentration/define.dat" );
		_animation_Concentration->SetPosition ( _piece->GetCurrentPosition() - sf::Vector2f(0, 12) );

		// 状態遷移：動き始める前・集中線エフェクト
		_currentState = STATE_3_BEFORE_MOVING;

	///// 状態：動き始める前・集中線エフェクト
	} else if ( _currentState == STATE_3_BEFORE_MOVING ) {

		// 集中線エフェクトが終わったら、次の状態へ遷移
		if ( !_animation_Concentration->IsActive() ) {

			delete _animation_Concentration;
			_animation_Concentration = NULL;

			// 駒を移動先のマス目へ入れる
			_grid_From->SetPiece ( NULL );
			_grid_To->SetPiece ( _piece );

			// 状態遷移：移動中
			_currentState = STATE_3_GOING;

		} else {

			// 集中線エフェクトを更新
			_animation_Concentration->Update(window);

		}

	///// 状態：移動中
	} else if ( _currentState == STATE_3_GOING ) {

		// 毎フレーム、駒の現在地に軌跡エフェクトを出す
		Effect_SimpleParticle::Property prop;
			prop.imageFile = "./effect/movePiece_particle/gra_effect_lightA_convert.png";
			prop.imageMask = "0,0,0";
			prop.color_Start = sf::Color ( 255,255,255,200 );
			prop.color_End   = sf::Color ( 255,255,255,0 );
			prop.totalFrameCount = 20;
			prop.imageAlphaGenerate_baseBlack = true;
			prop.position = _piece->GetCurrentPosition();

		IEffect *effect = new Effect_SimpleParticle(prop);
		Application::GetInstance()->_effectManager->Add ( effect );


		// 移動が終わったかな？
		if ( _piece->IsMovingStop() ) {

			// 状態遷移：到着
			_currentState = STATE_3_ARRIVED;

			// 衝撃波エフェクト開始
			ImpactWave ( _piece->GetGoalPosition() );

			/*
			_animation_Impact = new GUI_SFML::Animation();
			_animation_Impact->Init ( "./effect/movePiece_impact/define.dat" );
			_animation_Impact->SetPosition ( _piece->GetGoalPosition() );
			*/

			// この段階で終了メソッドを呼んで処理を戻す
			(*_endMethod)(_move);

		}

	///// 状態：到着
	} else if ( _currentState == STATE_3_ARRIVED ) {

		_isEnd = true;
		_currentState = STATE_FINISHED;

		/*
		// 衝撃波エフェクトが終了したら、終了
		if ( _animation_Impact == NULL || !_animation_Impact->IsActive() ) {

			delete _animation_Impact;
			_animation_Impact = NULL;

			_isEnd = true;
			_currentState = STATE_FINISHED;
		} else {
			_animation_Impact->Update(window);
		}
		*/
		printf ("駒移動：到着\n");

	///// 終了
	} else if ( _currentState == STATE_FINISHED ) {

		/*
			ここにはこないはず
		*/
		_isEnd = true;

	}
}

//=========================================================
// 描画
//=========================================================
void Effect_MovePiece::Draw(sf::RenderWindow &window) {

	if ( _isEnd ) { _isEnd; }

	if      ( _effectLevel == 0 ) { Draw_EffectLEvel_0 ( window ); }
	else if ( _effectLevel == 1 ) { Draw_EffectLEvel_1 ( window ); }
	else if ( _effectLevel == 2 ) { Draw_EffectLEvel_2 ( window ); }
	else if ( _effectLevel == 3 ) { Draw_EffectLEvel_3 ( window ); }
}

//=========================================================
// エフェクトレベル０の描画処理
//=========================================================
void Effect_MovePiece::Draw_EffectLEvel_0 (sf::RenderWindow &window) {
	// なし
}
//=========================================================
// エフェクトレベル１の描画処理
//=========================================================
void Effect_MovePiece::Draw_EffectLEvel_1 (sf::RenderWindow &window) {
	// なし
}
//=========================================================
// エフェクトレベル２の描画処理
//=========================================================
void Effect_MovePiece::Draw_EffectLEvel_2 (sf::RenderWindow &window) {

	if ( _animation_LightPillar_Appear != NULL ) {
		_animation_LightPillar_Appear->Draw(window);
	}

	if ( _animation_LightPillar_Disappear != NULL ) {
		_animation_LightPillar_Disappear->Draw(window);
	}


}
//=========================================================
// エフェクトレベル３の描画処理
//=========================================================
void Effect_MovePiece::Draw_EffectLEvel_3 (sf::RenderWindow &window) {

	if ( _animation_Concentration != NULL ) {
		_animation_Concentration->Draw(window);
	}
	/*
	if ( _animation_Impact != NULL ) {
		_animation_Impact->Draw(window);
	}
	*/
}

//=========================================================
// 衝撃波エフェクト作成：移動レベル３用のアレ
//=========================================================
void Effect_MovePiece::ImpactWave ( sf::Vector2f positon ) {

	// 音はここで出す
	GUI_SFML::SoundPlayer::Play ( "./sound/se_good_pk01/bom11.wav" );

	Effect_ImpactWave::Property prop;
	prop.color = sf::Color ( 255,255,180, 100 );
	prop.maxRadius = APPLICATION::WINDOW_SIZE::WIDTH / 1.5;
	prop.speed = 20;
	prop.position = positon;
	prop.soundName = "";

		
	map<unsigned int, Effect_ImpactWave::Property> impactList;
	for ( int i = 0; i < 5; ++i ) {
		impactList [ i * 5 ] = prop;
	}

	Effect_Impact_Rash *effect = new Effect_Impact_Rash ( impactList );

	Application::GetInstance()->_effectManager-> Add ( effect );
}
