//=========================================================
//
// 着手エフェクト＿駒移動（ぶつけない）
//
//=========================================================

#ifndef _EFFECT_MOVE_PIECE_
#define _EFFECT_MOVE_PIECE_

#include "IEffect.h"
#include "Delegate.h"
#include "../GUI_SFML/Animation.h"
#include "../AI/Move.h"

#include <vector>

class Piece;
class Grid;
class Effect_SimpleParticle;

class Effect_MovePiece : public IEffect {

public:

	///// オフセットアニメーション用
	struct OffsetAnimationFrame {

		int x;
		int y;
		int frameCount_Max;
		int frameCount_Current;

		OffsetAnimationFrame () {
			x = 0;
			y = 0;
			frameCount_Max     = 0;
			frameCount_Current = 0;
		}
		OffsetAnimationFrame ( int arg_x, int arg_y, int arg_frame_Max) {
			x = arg_x;
			y = arg_y;
			frameCount_Max     = arg_frame_Max;
			frameCount_Current = 0;
		}
	};

	///// オフセットアニメーション用
	struct OffsetAnimation {
		vector < OffsetAnimationFrame > offsetList;
		int index;

		OffsetAnimation() {
			index = -1;
		}
	};

	Effect_MovePiece (
		
		DelegateBase     *endMethod,
		AI_Modules::Move  move,
		int               effectLevel,
		Grid             *grid_From,   // このグリッドから
		Grid             *grid_To,     // このグリッドへ移動
		Piece            *piece        // この駒を移動
	);


	~Effect_MovePiece();

	bool Update(sf::RenderWindow &window);
	void Draw(sf::RenderWindow &window);

private:

	bool _isEnd;

	///// 着手終了時に呼ぶメソッド。GameManager側で用意する。
	DelegateBase *_endMethod;

	///// 監視対象のオブジェクト
	AI_Modules::Move _move;   // つまりこの着手ですよ。
	Grid        *_grid_From;  // このグリッドから
	Grid        *_grid_To;    // このグリッドへ移動
	Piece       *_piece;      // この駒を移動

	///// アニメーションエフェクト
	GUI_SFML::Animation *_animation_Concentration; // 集中線
	//GUI_SFML::Animation *_animation_Impact;        // 衝撃波
	GUI_SFML::Animation *_animation_LightPillar_Appear;     // 光の柱出現
	GUI_SFML::Animation *_animation_LightPillar_Disappear;  // 光の柱消滅

	int _frameCount_SinceCurrentState;

	///// オフセットアニメーション
	//vector < OffsetAnimationFrame > _offsetList_For2_SoarCard; // カード取得時、上空に飛ばされる
	//int _offsetList_PutCard_index;
	//OffsetAnimation _offsetAnimation_For2_SoarCard; // カード取得時、上空に飛ばされる

	///// エフェクトレベルはいくつか？
	int _effectLevel;

	///// 今の状態
	int _currentState;

	//=========================================================
	// 動作のステージ
	//=========================================================
	static const int STATE_0_BEGGINING_TO_GO; // 駒が動き始める
	static const int STATE_0_GOING;           // 駒が動いている
	static const int STATE_0_ARRIVE_AT;       // 駒が移動先に到着した

	static const int STATE_1_BEGGINING_TO_GO; // 駒が動き始める
	static const int STATE_1_GOING;           // 駒が動いている
	static const int STATE_1_ARRIVE_AT;       // 駒が移動先に到着した

	static const int STATE_2_BEGGINING_TO_GO;       // 光出現
	static const int STATE_2_ENTERING_TO_LIGHT;     // 光に吸い込まれている
	static const int STATE_2_WARPING;               // 移動中というか、吸い込まれ終わっている
	static const int STATE_2_SECOND_LIGHT_APPEARS;  // 移動先に光出現
	static const int STATE_2_SECOND_LIGHT_GOING_ON; // 移動先に光が出現している最中

	static const int STATE_3_SETUP;           // 初回処理。集中線エフェクト開始
	static const int STATE_3_BEFORE_MOVING;   // 動き始める前。集中線エフェクト終了を待つ
	static const int STATE_3_GOING;           // 動いている
	static const int STATE_3_ARRIVED;         // 移動先に到着。衝撃波エフェクト

	static const int STATE_FINISHED; // レベル共通：終了している。


	//=========================================================
	// オフセットリスト生成
	//=========================================================
	//void MakeOffsetList_For_0();
	//void MakeOffsetList_For_3();
	//void MakeOffsetList_For_2_SoarCard();


	//=========================================================
	// エフェクトレベルごとに更新処理を分ける
	//=========================================================
	void Update_EffectLEvel_0 (sf::RenderWindow &window);
	void Update_EffectLEvel_1 (sf::RenderWindow &window);
	void Update_EffectLEvel_2 (sf::RenderWindow &window);
	void Update_EffectLEvel_3 (sf::RenderWindow &window);

	//=========================================================
	// エフェクトレベルごとに描画処理を分ける
	//=========================================================
	void Draw_EffectLEvel_0 (sf::RenderWindow &window);
	void Draw_EffectLEvel_1 (sf::RenderWindow &window);
	void Draw_EffectLEvel_2 (sf::RenderWindow &window);
	void Draw_EffectLEvel_3 (sf::RenderWindow &window);


	//=========================================================
	// 衝撃波エフェクト作成：移動レベル３用のアレ
	//=========================================================
	void ImpactWave ( sf::Vector2f positon );
};

#endif
