//=========================================================
//
// 着手エフェクト＿駒移動（ぶつける）
//
//=========================================================

#include "Effect_MovePiece_Collide.h"

#include "Piece.h"
#include "Grid.h"
#include "PieceBox.h"
#include "../GUI_SFML/SoundPlayer.h"
#include "../Utils/Utils_Cal.h"
#include "Effect_Animation.h"
#include "Application.h"
#include "EffectManager.h"


//=========================================================
// 動作のステージ
//=========================================================
const int Effect_MovePiece_Collide::STATE_0_BEGGINING_TO_GO = 0; // 駒が動き始める
const int Effect_MovePiece_Collide::STATE_0_GOING           = 1; // 駒が動いている
const int Effect_MovePiece_Collide::STATE_0_ARRIVE_AT       = 2; // 駒が移動先に到着した
const int Effect_MovePiece_Collide::STATE_FINISHED          = 3; // レベル共通：終了している。

const int Effect_MovePiece_Collide::STATE_1_BEGGINING_TO_GO = 10; // 駒が動き始める
const int Effect_MovePiece_Collide::STATE_1_GOING           = 11; // 駒が動いている
const int Effect_MovePiece_Collide::STATE_1_ARRIVE_AT       = 12; // 駒が移動先に到着した

const int Effect_MovePiece_Collide::STATE_2_BEGGINING_TO_AIM = 20; // 照準エフェクトを出す
const int Effect_MovePiece_Collide::STATE_2_AIMING           = 21; // 照準エフェクト進行中
const int Effect_MovePiece_Collide::STATE_2_SHOOT            = 22; // 発射
const int Effect_MovePiece_Collide::STATE_2_GOING            = 23; // 駒が動いている
const int Effect_MovePiece_Collide::STATE_2_ARRIVE_AT        = 24; // 駒が移動先に到着した

const int Effect_MovePiece_Collide::STATE_3_CHARGE          = 30; // 駒がタメを作る
const int Effect_MovePiece_Collide::STATE_3_WAIT_TO_GO      = 31; // たまった状態で待つ
const int Effect_MovePiece_Collide::STATE_3_GOING           = 32; // 駒が動いている
const int Effect_MovePiece_Collide::STATE_3_ARRIVE_AT       = 33; // 駒が移動先に到着した


Piece* Effect_MovePiece_Collide::LAST_COLLIDED_PIECE = NULL;

//=========================================================
// コンストラクタ
//=========================================================
Effect_MovePiece_Collide::Effect_MovePiece_Collide (
		
		DelegateBase     *endMethod,
		AI_Modules::Move  move,
		int               effectLevel,
		Grid             *grid_From,      // このグリッドから
		Grid             *grid_To,        // このグリッドへ移動
		Piece            *piece,          // この駒を移動
		Piece            *piece_Collided, // この駒にぶつける
		PieceBox         *pieceBox,       // ぶつけられた駒が戻る駒台
		Grid             *grid_Collided   // ぶつけられる駒がいるグリッド
) 
{
	LAST_COLLIDED_PIECE = NULL;

	///// 引数を受け取って
	_endMethod      = endMethod;
	_grid_From      = grid_From;
	_grid_To        = grid_To;
	_piece          = piece;
	_piece_Collided = piece_Collided;
	_pieceBox       = pieceBox;
	_effectLevel    = effectLevel;
	_grid_Collided  = grid_Collided;
	_move           = move;

	///// 初期状態
	_isEnd        = false;
	_frameCount_Disposable = 0;

	///// アニメーションエフェクトはみなさんNULLですよね。
	_animation_Collide = NULL;
	_animation_Sight   = NULL;

	///// エフェクトレベル、操作する？
	if ( effectLevel < 0 ) {

		_effectLevel = 0;
		if      ( sf::Keyboard::isKeyPressed ( sf::Keyboard::Num1 ) ) { _effectLevel = 1; }
		else if ( sf::Keyboard::isKeyPressed ( sf::Keyboard::Num2 ) ) { _effectLevel = 2; }
		else if ( sf::Keyboard::isKeyPressed ( sf::Keyboard::Num3 ) ) { _effectLevel = 3; }

	///// 操作しない：引数で渡されたまま、受け取る
	} else {
		_effectLevel = effectLevel;
	}

	///// エフェクトレベル別に、初期状態をセット。
	if      ( _effectLevel == 0 ) { _currentState = STATE_0_BEGGINING_TO_GO; }
	else if ( _effectLevel == 1 ) { _currentState = STATE_1_BEGGINING_TO_GO; }
	else if ( _effectLevel == 2 ) { _currentState = STATE_2_BEGGINING_TO_AIM;}
	else if ( _effectLevel == 3 ) { _currentState = STATE_3_CHARGE;          }

	printf ( "駒移動（ぶつける）エフェクト_%d (引数 effectLevel:%d) 開始\n", _effectLevel, effectLevel );
}


//=========================================================
// デストラクタ
//=========================================================
Effect_MovePiece_Collide::~Effect_MovePiece_Collide() {

	printf ("駒移動（ぶつける）エフェクト終了\n");

	delete _animation_Collide;
	delete _animation_Sight;
	
	LAST_COLLIDED_PIECE = _piece_Collided;
}

//=========================================================
// 更新
//=========================================================
bool Effect_MovePiece_Collide::Update(sf::RenderWindow &window) {

	if ( _isEnd ) { return _isEnd; }

	if      ( _effectLevel == 0 ) { Update_EffectLevel_0 ( window ); }
	else if ( _effectLevel == 1 ) { Update_EffectLevel_1 ( window ); }
	else if ( _effectLevel == 2 ) { Update_EffectLevel_2 ( window ); }
	else if ( _effectLevel == 3 ) { Update_EffectLevel_3 ( window ); }

	return _isEnd;
}


//=========================================================
// エフェクトレベル０の更新処理
//=========================================================
void Effect_MovePiece_Collide::Update_EffectLevel_0 (sf::RenderWindow &window) {

	///// 状態：動き始め
	if ( _currentState == STATE_0_BEGGINING_TO_GO ) {

		// 駒を from から、浮かせる
		_grid_From->SetPiece ( NULL );

		// 動き始めを表す効果音
		GUI_SFML::SoundPlayer::Play ( "./sound/se_good_pk01/swing29.wav" );

		// 状態遷移：移動中
		_currentState = STATE_0_GOING;

	///// 状態：移動中
	} else if ( _currentState == STATE_0_GOING ) {

		// 駒を目的地へ移動させる
		float goalX = (_grid_Collided->GetCenterPosition().x + _grid_To->GetCenterPosition().x ) / 2.0f;
		float goalY = (_grid_Collided->GetCenterPosition().y + _grid_To->GetCenterPosition().y ) / 2.0f;

		pair<float, float> nextPos = Utils_Cal::MoveTo (
			_piece->GetGoalPosition().x, _piece->GetGoalPosition().y,
			goalX, goalY,
			25.0f
		);
		_piece->SetPosition ( sf::Vector2f ( nextPos.first, nextPos.second ) );

		// 移動が終わったかな？
		//if ( _piece->IsMovingStop() ) {
		if ( nextPos.first == goalX && nextPos.second == goalY ) {

			// 状態遷移：到着
			_currentState = STATE_0_ARRIVE_AT;

			// 衝突エフェクトを作成
			_animation_Collide = new GUI_SFML::Animation();
			_animation_Collide->Init ( "./effect/collide_0/define.dat" );

			_animation_Collide->SetPosition (
				(_piece->GetCurrentPosition() + _piece_Collided->GetCurrentPosition()) / 2.0f
			);

			// ぶつけられた駒を駒台に移動
			_grid_Collided->SetPiece(NULL);
			_pieceBox->AddPiece ( _piece_Collided );

			// 駒を、目的地のグリッドに入れる。
			_grid_To->SetPiece ( _piece );

			// ここで終了処理を出す。
			(*_endMethod)(_move);

		}

	///// 状態：到着
	} else if ( _currentState == STATE_0_ARRIVE_AT ) {

		// 衝突エフェクトが終わった？
		if ( !_animation_Collide->IsActive() ) {

			_isEnd       = true;
			_currentState = STATE_FINISHED;

			delete _animation_Collide;
			_animation_Collide = NULL;
		}

	///// 終了
	} else if ( _currentState == STATE_FINISHED ) {

		/*
			ここにはこないはず
		*/
		_isEnd = true;

	}

	if ( _currentState == STATE_0_GOING ) {
		_piece->Update ( window, _piece->GetGoalPosition() );
	}

	if ( _animation_Collide != NULL ) {
		_animation_Collide->Update(window);
	}
}

//=========================================================
// エフェクトレベル１の更新処理
//=========================================================
void Effect_MovePiece_Collide::Update_EffectLevel_1 (sf::RenderWindow &window) {

	///// 状態：動き始め
	if ( _currentState == STATE_1_BEGGINING_TO_GO ) {

		// 駒がある位置に旋風エフェクトを出す
		WhirlWind();

		// 駒を from から、浮かせる
		_grid_From->SetPiece ( NULL );

		// 動き始めを表す効果音
		GUI_SFML::SoundPlayer::Play ( "./sound/se_good_pk01/whip00.wav" );

		// 状態遷移：移動中
		_currentState = STATE_1_GOING;

	///// 状態：移動中
	} else if ( _currentState == STATE_1_GOING ) {

		// 駒がある位置に旋風エフェクトを出す
		WhirlWind();

		// 駒を目的地へ移動させる
		float goalX = (_grid_Collided->GetCenterPosition().x + _grid_To->GetCenterPosition().x ) / 2.0f;
		float goalY = (_grid_Collided->GetCenterPosition().y + _grid_To->GetCenterPosition().y ) / 2.0f;

		pair<float, float> nextPos = Utils_Cal::MoveTo (
			_piece->GetGoalPosition().x, _piece->GetGoalPosition().y,
			goalX, goalY,
			25.0f
		);
		_piece->SetPosition ( sf::Vector2f ( nextPos.first, nextPos.second ) );

		// 移動が終わったかな？
		//if ( _piece->IsMovingStop() ) {
		if ( nextPos.first == goalX && nextPos.second == goalY ) {

			// 状態遷移：到着
			_currentState = STATE_1_ARRIVE_AT;

			// 衝突エフェクトを作成
			_animation_Collide = new GUI_SFML::Animation();
			_animation_Collide->Init ( "./effect/collide_1/define.dat" );

			_animation_Collide->SetPosition (
				(_piece->GetCurrentPosition() + _piece_Collided->GetCurrentPosition()) / 2.0f
			);

			// ぶつけられた駒を駒台に移動
			_grid_Collided->SetPiece(NULL);
			_pieceBox->AddPiece ( _piece_Collided );

			// 駒を、目的地のグリッドに入れる。
			_grid_To->SetPiece ( _piece );

			// ここで終了処理を出す。
			(*_endMethod)(_move);

		}

	///// 状態：到着
	} else if ( _currentState == STATE_1_ARRIVE_AT ) {

		// 衝突エフェクトが終わった？
		if ( !_animation_Collide->IsActive() ) {

			_isEnd       = true;
			_currentState = STATE_FINISHED;

			delete _animation_Collide;
			_animation_Collide = NULL;
		}

	///// 終了
	} else if ( _currentState == STATE_FINISHED ) {

		/*
			ここにはこないはず
		*/
		_isEnd = true;

	}

	if ( _currentState == STATE_1_GOING ) {
		_piece->Update ( window, _piece->GetGoalPosition() );
	}

	if ( _animation_Collide != NULL ) {
		_animation_Collide->Update(window);
	}
}

//=========================================================
// エフェクトレベル２の更新処理
//=========================================================
void Effect_MovePiece_Collide::Update_EffectLevel_2 (sf::RenderWindow &window) {

	///// 状態：狙い始め
	if ( _currentState == STATE_2_BEGGINING_TO_AIM ) {

		///// 照準エフェクトを出す
		_animation_Sight = new GUI_SFML::Animation();
		_animation_Sight->Init ( "./effect/sight/define.dat" );
		_animation_Sight->SetPosition ( _piece_Collided->GetCurrentPosition() );

		_currentState = STATE_2_AIMING;

	///// 状態：狙い中
	} else if ( _currentState == STATE_2_AIMING ) {

		_animation_Sight->Update(window);

		///// 狙い終わったら発射する
		if ( !_animation_Sight->IsActive() ) {

			delete _animation_Sight;
			_animation_Sight = NULL;

			GUI_SFML::SoundPlayer::Play ( "./sound/se_good_pk01/gun14_c.wav" );

			_currentState = STATE_2_SHOOT;
		}

	///// 状態：発射
	} else if ( _currentState == STATE_2_SHOOT ) {

		// 駒を from から、浮かせる
		_grid_From->SetPiece ( NULL );

		_currentState = STATE_2_GOING;

		// 煙を出す
		GUI_SFML::Animation *animation = new GUI_SFML::Animation();
		animation->Init ( "./effect/gun_smoke/define.dat" );
		animation->SetPosition ( _grid_From->GetCenterPosition() );
		IEffect *effect = new Effect_Animation ( animation );
		Application::GetInstance()->_effectManager->Add ( effect );

	///// 駒が動いている
	} else if ( _currentState == STATE_2_GOING ) {

		// 駒を目的地へ移動させる
		float goalX = (_grid_Collided->GetCenterPosition().x + _grid_To->GetCenterPosition().x ) / 2.0f;
		float goalY = (_grid_Collided->GetCenterPosition().y + _grid_To->GetCenterPosition().y ) / 2.0f;

		pair<float, float> nextPos = Utils_Cal::MoveTo (
			_piece->GetGoalPosition().x, _piece->GetGoalPosition().y,
			goalX, goalY,
			50.0f
		);
		_piece->SetPosition ( sf::Vector2f ( nextPos.first, nextPos.second ) );

		// 移動が終わったかな？
		//if ( _piece->IsMovingStop() ) {
		if ( nextPos.first == goalX && nextPos.second == goalY ) {

			// 状態遷移：到着
			_currentState = STATE_2_ARRIVE_AT;

			// 衝突エフェクトを作成
			_animation_Collide = new GUI_SFML::Animation();
			_animation_Collide->Init ( "./effect/collide_2/define.dat" );

			_animation_Collide->SetPosition (
				(_piece->GetCurrentPosition() + _piece_Collided->GetCurrentPosition()) / 2.0f
			);

			// ぶつけられた駒を駒台に移動
			_grid_Collided->SetPiece(NULL);
			_pieceBox->AddPiece ( _piece_Collided );

			// 駒を、目的地のグリッドに入れる。
			_grid_To->SetPiece ( _piece );

			// ここで終了処理を出す。
			(*_endMethod)(_move);

		}


	///// 駒が移動先に到着した
	} else if ( _currentState == STATE_2_ARRIVE_AT ) {

		// 衝突エフェクトが終わった？
		if ( !_animation_Collide->IsActive() ) {

			_isEnd       = true;
			_currentState = STATE_FINISHED;

			delete _animation_Collide;
			_animation_Collide = NULL;
		}

	///// 終了
	} else if ( _currentState == STATE_FINISHED ) {

		/*
			ここにはこないはず
		*/
		_isEnd = true;

	}


	if ( _currentState == STATE_2_GOING ) {
		_piece->Update ( window, _piece->GetGoalPosition() );
	}

	if ( _animation_Collide != NULL ) {
		_animation_Collide->Update(window);
	}
}

//=========================================================
// エフェクトレベル３の更新処理
//=========================================================
void Effect_MovePiece_Collide::Update_EffectLevel_3 (sf::RenderWindow &window) {

	///// 状態：タメを作る
	if ( _currentState == STATE_3_CHARGE ) {

		///// 最初だけ、タメてる感じの音を出す
		if ( _frameCount_Disposable == 0 ) {
			GUI_SFML::SoundPlayer::Play ( "./sound/se_good_pk01/biri03.wav" );
		}
		++_frameCount_Disposable;

		///// 行き先と逆向きに、少し進む

		// 目的地
		float goalX = (_grid_Collided->GetCenterPosition().x + _grid_To->GetCenterPosition().x ) / 2.0f;
		float goalY = (_grid_Collided->GetCenterPosition().y + _grid_To->GetCenterPosition().y ) / 2.0f;

		// 現在地
		float startX = _piece->GetGoalPosition().x;
		float startY = _piece->GetGoalPosition().y;

		// 逆向きの目的地
		pair<float, float> oppositePos = Utils_Cal::MoveTo (
			startX, startY,
			goalX, goalY,
			-50.0f
		);

		// に、向けて、オフセットをタメる。
		float currentOffsetX = _piece->GetOffset().x;
		float currentOffsetY = _piece->GetOffset().y;

		// 逆向きに進む
		pair<float, float> nextPos = Utils_Cal::MoveTo (
			startX + currentOffsetX, startY + currentOffsetY,
			oppositePos.first, oppositePos.second,
			5.0f
		);

		_piece->SetOffsetX ( nextPos.first  - startX );
		_piece->SetOffsetY ( nextPos.second - startY );


		///// タメきった？
		if ( Utils_Cal::Abs( (startX + currentOffsetX) - oppositePos.first) <= 1 && Utils_Cal::Abs( (startY + currentOffsetY) - oppositePos.second) <= 1 ) {

			// 状態遷移：たまった状態で少し待つ
			_currentState = STATE_3_WAIT_TO_GO;
			_frameCount_Disposable = 0;
		}

	///// 状態：たまった状態で少し待つ
	} else if ( _currentState == STATE_3_WAIT_TO_GO ) {

		if ( _frameCount_Disposable == 0) {

			// 衝突アニメーションを用意しておく。ローディングで一瞬止まらないように。（タイミング的に、ここなら止まっても悪くはない。）
			_animation_Collide = new GUI_SFML::Animation();
			_animation_Collide->Init ( "./effect/collide_3/define.dat" );
		}

		++_frameCount_Disposable;

		if ( _frameCount_Disposable >= 20 ) {

			_currentState = STATE_3_GOING;

			// オフセット込みの現在地を実際の位置とする
			//_piece->SetPosition ( sf::Vector2f ( startX + currentOffsetX, startY + currentOffsetY ) );
			_piece->SetPosition ( _piece->GetGoalPosition() + _piece->GetOffset() );

			// オフセットはクリア
			_piece->SetOffsetX (0);
			_piece->SetOffsetY (0);

			// 駒を from から、浮かせる
			_grid_From->SetPiece ( NULL );

			// 動き始めを表す効果音
			GUI_SFML::SoundPlayer::Play ( "./sound/se_good_pk01/swing27.wav" );
		}

	///// 状態：移動中
	} else if ( _currentState == STATE_3_GOING ) {

		// 駒を目的地へ移動させる
		float goalX = (_grid_Collided->GetCenterPosition().x + _grid_To->GetCenterPosition().x ) / 2.0f;
		float goalY = (_grid_Collided->GetCenterPosition().y + _grid_To->GetCenterPosition().y ) / 2.0f;

		pair<float, float> nextPos = Utils_Cal::MoveTo (
			_piece->GetGoalPosition().x, _piece->GetGoalPosition().y,
			goalX, goalY,
			30.0f
		);
		_piece->SetPosition ( sf::Vector2f ( nextPos.first, nextPos.second ) );

		// 移動が終わったかな？
		if ( nextPos.first == goalX && nextPos.second == goalY ) {

			// 状態遷移：到着
			_currentState = STATE_3_ARRIVE_AT;

			// 衝突エフェクトを準備（位置を決めることができるのはこの時点）
			_animation_Collide->SetPosition (
				(_piece->GetCurrentPosition() + _piece_Collided->GetCurrentPosition()) / 2.0f
			);

			// ぶつけられた駒を駒台に移動
			_grid_Collided->SetPiece(NULL);
			_pieceBox->AddPiece ( _piece_Collided );

			// 駒を、目的地のグリッドに入れる。
			_grid_To->SetPiece ( _piece );

			// ここで終了処理を出す？
			//(*_endMethod)(_move);

		}

	///// 状態：到着
	} else if ( _currentState == STATE_3_ARRIVE_AT ) {

		_animation_Collide->Update(window);

		// 衝突エフェクトが終わった？
		if ( !_animation_Collide->IsActive() ) {

			_isEnd       = true;
			_currentState = STATE_FINISHED;

			// 爆発エフェクトの最中に相手に動かれると、何が起きたのか、わかりにくい。
			(*_endMethod)(_move);

			delete _animation_Collide;
			_animation_Collide = NULL;
		}

	///// 終了
	} else if ( _currentState == STATE_FINISHED ) {

		/*
			ここにはこないはず
		*/
		_isEnd = true;

	}

	if ( _currentState == STATE_3_GOING ) {
		_piece->Update ( window, _piece->GetGoalPosition() );
	}

}

//=========================================================
// 描画
//=========================================================
void Effect_MovePiece_Collide::Draw(sf::RenderWindow &window) {

	if ( _isEnd ) { _isEnd; }

	if      ( _effectLevel == 0 ) { Draw_EffectLevel_0 ( window ); }
	else if ( _effectLevel == 1 ) { Draw_EffectLevel_1 ( window ); }
	else if ( _effectLevel == 2 ) { Draw_EffectLevel_2 ( window ); }
	else if ( _effectLevel == 3 ) { Draw_EffectLevel_3 ( window ); }
}

//=========================================================
// エフェクトレベル０の描画処理
//=========================================================
void Effect_MovePiece_Collide::Draw_EffectLevel_0 (sf::RenderWindow &window) {

	if ( _currentState == STATE_0_GOING ) {
		_piece->Draw ( window );
	}

	if ( _animation_Collide != NULL ) {
		_animation_Collide->Draw(window);
	}
}

//=========================================================
// エフェクトレベル１の描画処理
//=========================================================
void Effect_MovePiece_Collide::Draw_EffectLevel_1 (sf::RenderWindow &window) {

	if ( _currentState == STATE_1_GOING ) {
		_piece->Draw ( window );
	}

	if ( _animation_Collide != NULL ) {
		_animation_Collide->Draw(window);
	}
}
//=========================================================
// エフェクトレベル２の描画処理
//=========================================================
void Effect_MovePiece_Collide::Draw_EffectLevel_2 (sf::RenderWindow &window) {

	if ( _animation_Sight != NULL ) {
		_animation_Sight->Draw(window);
	}
	if ( _currentState == STATE_2_GOING ) {
		_piece->Draw ( window );
	}
	if ( _animation_Collide != NULL ) {
		_animation_Collide->Draw(window);
	}
}

//=========================================================
// エフェクトレベル３の描画処理
//=========================================================
void Effect_MovePiece_Collide::Draw_EffectLevel_3 (sf::RenderWindow &window) {

	if ( _currentState == STATE_3_GOING ) {
		_piece->Draw ( window );
	}

	if ( _animation_Collide != NULL && _currentState == STATE_3_ARRIVE_AT ) {
		_animation_Collide->Draw(window);
	}
}

//=========================================================
// 駒の現在地に旋風エフェクトを出す
//=========================================================
void Effect_MovePiece_Collide::WhirlWind() {

	///// ファイル名：移動方向によっては、反転バージョンのを使う
	string whirlEffectFile = "./effect/whirlWind/define.dat";
	if 
	( 
		_grid_From->GetPosition().x < _grid_To->GetPosition().x // 左から右へ
		||
		_grid_From->GetPosition().y > _grid_To->GetPosition().y // 下から上へ
	) 
	{
		whirlEffectFile = "./effect/whirlWind/define_reverse.dat";
	}

	///// エフェクトを生成して EffectManager に登録
	GUI_SFML::Animation *animation = new GUI_SFML::Animation ();
	animation->Init ( whirlEffectFile );
	animation->SetPosition ( _piece->GetCurrentPosition() );

	IEffect *effect = new Effect_Animation ( animation );
	Application::GetInstance()->_effectManager->Add ( effect );
}

