//=========================================================
//
// カード設置エフェクト。AI用
//
//=========================================================

#ifndef _EFFECT_PUT_CARD_AI_
#define _EFFECT_PUT_CARD_AI_

#include "IEffect.h"
#include "Delegate.h"
#include "../GUI_SFML/Animation.h"
#include "../AI/Move.h"

#include <vector>

class Card;
class Grid;

using namespace std;

class Effect_PutCard_AI : public IEffect {

public:

	///// オフセットアニメーション用
	struct OffsetAnimationFrame {

		int x;
		int y;
		int frameCount_Max;
		int frameCount_Current;

		OffsetAnimationFrame () {
			x = 0;
			y = 0;
			frameCount_Max     = 0;
			frameCount_Current = 0;
		}
		OffsetAnimationFrame ( int arg_x, int arg_y, int arg_frame_Max) {
			x = arg_x;
			y = arg_y;
			frameCount_Max     = arg_frame_Max;
			frameCount_Current = 0;
		}
	};

	///// コンストラクタ：必要なオブジェクトを受け取る
	Effect_PutCard_AI ( 
		DelegateBase     *endMethod,
		AI_Modules::Move  move,
		int               effectLevel,
		Card            *card,
		Grid             *grid
	);
	~Effect_PutCard_AI();


	bool Update(sf::RenderWindow &window);
	void Draw(sf::RenderWindow &window);

private:

	bool _isEnd;

	///// 着手終了時に呼ぶメソッド。GameManager側で用意する。
	DelegateBase *_endMethod;

	///// 監視対象のオブジェクト
	AI_Modules::Move _move;  // つまりこの着手ですよ。
	Card        *_card;      // 設置するカード
	Grid        *_goalGrid;  // 設置先グリッド

	///// アニメーションエフェクト
	GUI_SFML::Animation *_animation_PutCard;    // カード叩きつけ
	GUI_SFML::Animation *_animation_HoldCard;   // カードを保持中
	GUI_SFML::Animation *_animation_RisingCard; // カード上昇中（叩きつけ前の）

	///// カードを設置するときの勢いオフセット
	vector < OffsetAnimationFrame > _offsetList_PutCard; // カード設置時の叩きつけ動作
	int _offsetList_PutCard_index;

	///// エフェクトレベルはいくつか？
	int _effectLevel;

	///// 今の状態
	int _currentState;

	//=========================================================
	// カード設置動作のステージ
	//=========================================================
	static const int STATE_PICK_UP;           // カードをカード台から持ち上げた
	static const int STATE_GOING_TO_THE_GRID; // カードが設置先グリッドに向かっている
	static const int STATE_AT_THE_GRID;       // カードが設置先グリッドに到着した
	static const int STATE_FINISHED;          // カードをグリッドに設置し終えた


	//=========================================================
	// カード叩きつけオフセットリスト生成
	//=========================================================
	void MakeOffsetList_For_0();
	void MakeOffsetList_For_3();

	//=========================================================
	// 設置１用衝撃波発生
	//=========================================================
	void ImpactWave ( sf::Vector2f positon );

	//=========================================================
	// 設置２用効果
	//=========================================================
	void Energy ( sf::Vector2f positon );
};

#endif
