//=========================================================
//
// 着手エフェクト＿カード設置＿人間の操作
//
//=========================================================

#ifndef _EFFECT_PUT_CARD_HUMAN_
#define _EFFECT_PUT_CARD_HUMAN_

#include "IEffect.h"
#include "Delegate.h"
#include "../GUI_SFML/Animation.h"

#include <vector>

class Card;
class CardBox;
class Board;
class Grid;
class PsychoGage;

using namespace std;

class Effect_PutCard_Human : public IEffect {

public:

	///// オフセットアニメーション用
	struct OffsetAnimationFrame {

		int x;
		int y;
		int frameCount_Max;
		int frameCount_Current;

		OffsetAnimationFrame () {
			x = 0;
			y = 0;
			frameCount_Max     = 0;
			frameCount_Current = 0;
		}
		OffsetAnimationFrame ( int arg_x, int arg_y, int arg_frame_Max) {
			x = arg_x;
			y = arg_y;
			frameCount_Max     = arg_frame_Max;
			frameCount_Current = 0;
		}
	};

	///// コンストラクタ：必要なオブジェクトを受け取る
	Effect_PutCard_Human ( 
		DelegateBase *endMethod,
		Card         *card,
		CardBox      *cardBox,
		Board        *board,
		int           turn,
		PsychoGage   *psychoGage
	);
	~Effect_PutCard_Human();

	bool Update(sf::RenderWindow &window);
	void Draw(sf::RenderWindow &window);

private:

	bool _isEnd;

	///// 着手終了時に呼ぶメソッド。GameManager側で用意する。
	DelegateBase *_endMethod;

	///// 監視対象のオブジェクト
	Card        *_card;
	CardBox     *_cardBox;
	Board       *_board;
	int          _turn;     // Move のいうところの TURN
	Grid        *_goalGrid; // 設置先グリッド
	PsychoGage  *_psychoGage; // 意志ゲージ
	
	vector < vector < Grid* > > _allGridList;
	AI_Modules::Move _move;

	///// アニメーションエフェクト
	GUI_SFML::Animation *_animation_PutCard;    // カード叩きつけ
	GUI_SFML::Animation *_animation_HoldCard;   // カードを保持中
	GUI_SFML::Animation *_animation_RisingCard; // カード上昇中（叩きつけ前の）

	///// カードを設置するときの勢いオフセット
	vector < OffsetAnimationFrame > _offsetList_PutCard; // カード設置時の叩きつけ動作
	int _offsetList_PutCard_index;

	///// 今の状態。
	int _currentState;

	///// エフェクトレベルはいくつか？：つまり操作が介入するポイントごとに記録する
	int _effectLevel_toHoldCard; // カードを持ったときのエフェクトレベル
	int _effectLevel_toPutCard;  // カード設置先を決めたときのエフェクトレベル


	//=========================================================
	// 駒設置動作のステージ
	//=========================================================
	static const int STATE_BEFORE;            // まだ何もしていない
	static const int STATE_HOLDING_NOW;       // 駒をマウスで持っている
	static const int STATE_GOING_TO_THE_GRID; // 設置先グリッドに向かっている
	static const int STATE_FINISHED;          // 駒をグリッドに設置し終えた


	//=========================================================
	// 駒の現在地を探る
	//=========================================================
	bool IsCardInCardBox();
	bool IsCardHeldByMouse();
	bool IsCardSetInGrid();

	//=========================================================
	// カード、設置済みだとして、どの座標に設置した？
	//=========================================================
	pair<int, int> GetCardSettingGrid();

	//=========================================================
	// カード叩きつけオフセットリスト生成
	//=========================================================
	void MakeOffsetList_For_0();
	void MakeOffsetList_For_3();

	//=========================================================
	// 設置１用衝撃波発生
	//=========================================================
	void ImpactWave ( sf::Vector2f positon );

	//=========================================================
	// 設置２用効果
	//=========================================================
	void Energy ( sf::Vector2f positon );

	//=========================================================
	// エフェクトレベル用の操作判定
	//=========================================================
	bool IsEffectInput1();
	bool IsEffectInput2();
	bool IsEffectInput3();
};

#endif

