//=========================================================
//
// 駒設置エフェクト。AI用
//
//=========================================================

#include "Effect_PutPiece_AI.h"
#include "Piece.h"
#include "PieceBox.h"
#include "PieceBox.h"
#include "Board.h"
#include "Grid.h"
#include "../GUI_SFML/SoundPlayer.h"
#include "../Utils/Utils_Cal.h"
#include "Effect_Animation.h"
#include "Application.h"
#include "EffectManager.h"
#include "Effect_ImpactWave.h"
#include "Effect_ImpactWave_Rash.h"


//=========================================================
// 駒設置動作のステージ
//=========================================================
const int Effect_PutPiece_AI::STATE_PICK_UP           = 0; // 駒を駒台から持ち上げた
const int Effect_PutPiece_AI::STATE_GOING_TO_THE_GRID = 1; // 駒が設置先グリッドに向かっている
const int Effect_PutPiece_AI::STATE_AT_THE_GRID       = 2; // 駒が設置先グリッドに到着した
const int Effect_PutPiece_AI::STATE_FINISHED          = 3; // 駒をグリッドに設置し終えた


//=========================================================
// コンストラクタ：必要なオブジェクトを受け取る
//=========================================================
Effect_PutPiece_AI::Effect_PutPiece_AI ( 
		DelegateBase     *endMethod,   // 終了時の通知先
		AI_Modules::Move  move,        // この着手ですよ。
		int               effectLevel, // エフェクトレベル
		Piece            *piece,       // 設置する駒
		Grid             *grid         // 設置先グリッド
)
{

	_endMethod   = endMethod;
	_move        = move;
	_effectLevel = effectLevel;
	_piece       = piece;
	_goalGrid    = grid; 

	_isEnd = false;

	_currentState = STATE_PICK_UP;


	_animation_PutPiece    = NULL;
	_animation_HoldPiece   = NULL;
	_animation_RisingPiece = NULL;
	_offsetList_PutPiect_index = -1;

	printf ( "駒設置エフェクト（AI用）開始\n" );

}

//=========================================================
// 駒叩きつけオフセットリスト生成＿For0
//=========================================================
void Effect_PutPiece_AI::MakeOffsetList_For_0() {

	_offsetList_PutPiect.clear();

}
//=========================================================
// 駒叩きつけオフセットリスト生成＿For3
//=========================================================
void Effect_PutPiece_AI::MakeOffsetList_For_3() {

	_offsetList_PutPiect.clear();

	int y = 0; int jump = -30; int gravity = 2;
	while ( y >= 0 ) { // 着地したら終了

		int frameCount = 1;

		// 上昇中はゆっくり
		//if ( jump < 0 ) {
		//	frameCount = 2;
		//}
		_offsetList_PutPiect.push_back ( OffsetAnimationFrame ( 0, jump, frameCount ) );

		y    -= jump;
		jump += gravity;

		// 下降中は速度統一
		if ( jump > 0 ) {
			jump = 30;
		}
	}

}

//=========================================================
// デストラクタ
//=========================================================
Effect_PutPiece_AI::~Effect_PutPiece_AI() {

	printf ( "駒設置エフェクト（AI用）終了\n" );

	delete _animation_PutPiece;
	delete _animation_HoldPiece;
	delete _animation_RisingPiece;
}


//=========================================================
// 更新
//=========================================================
bool Effect_PutPiece_AI::Update(sf::RenderWindow &window) {

	if (_isEnd) { return _isEnd; }


	///// 現在の状態：駒を持ったところ
	if ( _currentState == STATE_PICK_UP ) {

		///// エフェクトレベル０
		if ( _effectLevel == 0 ) {

			// 駒つかみ効果音：AIには不要かもしれない。
			//GUI_SFML::SoundPlayer::Play ( "./sound/se_good_pk01/open26.wav" );

			// 駒を設置先グリッドに設置する。
			_goalGrid->SetPiece ( _piece );

		///// エフェクトレベル１
		} else if ( _effectLevel == 1 ) {

			// 駒つかみ効果音：AIには不要かもしれない。
			//GUI_SFML::SoundPlayer::Play ( "./sound/se_good_pk01/open25.wav" );

			// 駒を設置先グリッドに設置する。
			_goalGrid->SetPiece ( _piece );

		///// エフェクトレベル２
		} else if ( _effectLevel == 2 ) {

			// 駒つかみ効果音：AIには不要かもしれない。
			//GUI_SFML::SoundPlayer::Play ( "./sound/se_good_pk01/power25.wav" );

			// 駒を設置先グリッドに設置する。
			_goalGrid->SetPiece ( _piece );

		///// エフェクトレベル３
		} else if ( _effectLevel == 3 ) {

			// 駒保持エフェクトを作成
			_animation_HoldPiece = new GUI_SFML::Animation();
			_animation_HoldPiece->Init("./effect/holdPieceTest/define.dat");
			_animation_HoldPiece->SetPosition ( _piece->GetGoalPosition() );

			// 駒つかみ音
			GUI_SFML::SoundPlayer::Play ( "./sound/se_good_pk01/mecha05.wav" );
		}

		// 状態遷移 → 設置先グリッドに向かっているところ
		_currentState = STATE_GOING_TO_THE_GRID;


	///// 現在の状態：駒が設置先グリッドに向かっているところ
	} else if ( _currentState == STATE_GOING_TO_THE_GRID ) {

		///// 到着した？
		if ( _piece->IsMovingStop() ) {

			///// エフェクトレベル０
			if ( _effectLevel == 0 ) {

				// ここで終わってもいいけど、１フレーム伸ばしてみる。音のタイミング的に。
				
				// 状態遷移 → 設置先グリッドにいる
				_currentState = STATE_AT_THE_GRID;

			///// エフェクトレベル１
			} else if ( _effectLevel == 1 ) {

				// ここで終わってもいいけど、１フレーム伸ばしてみる。音のタイミング的に。
				
				// 状態遷移 → 設置先グリッドにいる
				_currentState = STATE_AT_THE_GRID;

			///// エフェクトレベル２
			} else if ( _effectLevel == 2 ) {

				// ここで終わってもいいけど、１フレーム伸ばしてみる。音のタイミング的に。
				
				// 状態遷移 → 設置先グリッドにいる
				_currentState = STATE_AT_THE_GRID;

			///// エフェクトレベル３
			} else if ( _effectLevel == 3 ) {

				///// 駒を目的のグリッドの位置に移動させる
				sf::Vector2f piecePos = _piece->GetGoalPosition();
				sf::Vector2f goalPos  = _goalGrid->GetCenterPosition();

				pair<float, float> nextPos = Utils_Cal::MoveTo ( piecePos.x, piecePos.y, goalPos.x,  goalPos.y, 10.0f );

				_piece->SetPosition ( sf::Vector2f( nextPos.first, nextPos.second ) );


				///// 到着した？
				if ( nextPos.first == goalPos.x && nextPos.second == goalPos.y) {

					// 駒保持エフェクトを停止
					delete _animation_HoldPiece;
					_animation_HoldPiece = NULL;

					// 駒上昇エフェクトを生成
					_animation_RisingPiece = new GUI_SFML::Animation();
					_animation_RisingPiece->Init("./effect/risingPieceTest/define.dat");

					// 駒上昇音
					GUI_SFML::SoundPlayer::Play ( "./sound/se_good_pk01/noise05.wav" );

					// 駒叩きつけオフセットアニメーション開始：0にすると開始になる。普段は -1
					_offsetList_PutPiect_index = 0;
					MakeOffsetList_For_3();

					// グリッドに駒を入れる
					_goalGrid->SetPiece ( _piece );

					// 状態遷移 → 設置先グリッドにいる
					_currentState = STATE_AT_THE_GRID;

				}
			}
		}

	///// 現在の状態：駒が設置先グリッドにいる
	} else if ( _currentState == STATE_AT_THE_GRID ) {

		///// エフェクトレベル０
		if ( _effectLevel == 0 ) {

			// 駒設置
			GUI_SFML::SoundPlayer::Play ( "./sound/se_good_pk01/hit15.wav" );

			printf ("駒移動完了（AI用）\n");

			(*_endMethod)(_move);
			_isEnd = true;

			// 終了
			_currentState = STATE_FINISHED;

		///// エフェクトレベル１
		} else if ( _effectLevel == 1 ) {

			// 駒設置
			GUI_SFML::SoundPlayer::Play ( "./sound/se_good_pk01/hit27.wav" );

			// 衝撃波
			ImpactWave ( _goalGrid->GetCenterPosition() );

			printf ("駒移動完了（AI用）\n");

			(*_endMethod)(_move);
			_isEnd = true;

			// 終了
			_currentState = STATE_FINISHED;

		///// エフェクトレベル２
		} else if ( _effectLevel == 2 ) {

			// 駒設置
			GUI_SFML::SoundPlayer::Play ( "./sound/se_good_pk01/mecha04.wav" );

			// エフェクト
			Energy ( _goalGrid->GetCenterPosition() );

			printf ("駒移動完了（AI用）\n");

			(*_endMethod)(_move);
			_isEnd = true;

			// 終了
			_currentState = STATE_FINISHED;

		///// エフェクトレベル３
		} else if ( _effectLevel == 3 ) {

			///// 駒たたきつけオフセットアニメーション
			if (_offsetList_PutPiect_index != -1) {

				// 現在のコマのカウントが0（初回）のときのみ、駒にオフセットを追加する。
				if ( _offsetList_PutPiect[_offsetList_PutPiect_index].frameCount_Current == 0 ) {

					_piece->SetOffsetX ( _piece->GetOffset().x + _offsetList_PutPiect[_offsetList_PutPiect_index].x );
					_piece->SetOffsetY ( _piece->GetOffset().y + _offsetList_PutPiect[_offsetList_PutPiect_index].y );


					///// 叩きつけオフセットアニメが下降を始めたら、駒設置エフェクトを作成
					if ( _animation_PutPiece == NULL && _offsetList_PutPiect[_offsetList_PutPiect_index].y > 0 ) {

						_animation_PutPiece = new GUI_SFML::Animation();
						_animation_PutPiece->Init("./effect/putPieceTest/define.dat");
						_animation_PutPiece->SetPosition ( _goalGrid->GetCenterPosition() );

						// 上昇エフェクトは終了
						delete _animation_RisingPiece;
						_animation_RisingPiece = NULL;
					}

				}

				// Maxになったら
				if (_offsetList_PutPiect[_offsetList_PutPiect_index].frameCount_Current >= _offsetList_PutPiect[_offsetList_PutPiect_index].frameCount_Max) {

					// カウントを０に戻して
					_offsetList_PutPiect[_offsetList_PutPiect_index].frameCount_Current = 0;

					// 次へ
					++_offsetList_PutPiect_index;

					// 終了？
					if (_offsetList_PutPiect_index >= _offsetList_PutPiect.size()) {
						_offsetList_PutPiect_index = -1; // 終了。

						// 駒のオフセットをクリア
						_piece->SetOffset ( 0, 0 );
					}

				// まだMaxじゃないなら、カウントを進める
				} else {
					_offsetList_PutPiect[_offsetList_PutPiect_index].frameCount_Current += 1;
				}
			}

			///// 移動完了？
			if ( _piece->IsMovingStop() && _offsetList_PutPiect_index == -1 ) {
			
				if ( !_animation_PutPiece->IsActive() )  {

					printf ("駒移動完了（AI用）\n");

					(*_endMethod)(_move);
					_isEnd = true;
				}
			}

		} ///// エフェクトレベルで分岐 if else

	///// 現在の状態：終了
	} else if ( _currentState == STATE_FINISHED) {

		/*
			ここにはこないはず
		*/
		_isEnd = true;
	}




	////// アニメーションエフェクト；駒設置
	if ( _animation_PutPiece != NULL ) {
		_animation_PutPiece->Update(window);
	}
	////// アニメーションエフェクト；駒保持
	if ( _animation_HoldPiece != NULL ) {
		_animation_HoldPiece->SetPosition ( _piece->GetCurrentPosition() );
		_animation_HoldPiece->Update(window);
	}
	////// アニメーションエフェクト；駒上昇
	if ( _animation_RisingPiece != NULL ) {
		_animation_RisingPiece->SetPosition ( _piece->GetCurrentPosition() + _piece->GetOffset() );
		_animation_RisingPiece->Update(window);
	}

	///// 駒の面倒を見てあげないといけないかもしれない。どこにも所属していない状態だから。
	if ( _goalGrid->GetPiece() != _piece ) {
		_piece->Update ( window, _piece->GetCurrentPosition() );
	}

	return _isEnd;
}

//=========================================================
// 描画
//=========================================================
void Effect_PutPiece_AI::Draw(sf::RenderWindow &window) {

	if (_isEnd) {return;}

	///// 駒の面倒を見てあげないといけないかもしれない。どこにも所属していない状態だから。
	if ( _goalGrid->GetPiece() != _piece ) {
		_piece->Draw ( window );
	}

	///// アニメーションエフェクト：駒設置
	if ( _animation_PutPiece != NULL ) {
		_animation_PutPiece->Draw ( window );
	}
	////// アニメーションエフェクト；駒保持
	if ( _animation_HoldPiece != NULL ) {
		_animation_HoldPiece->Draw ( window );
	}
	////// アニメーションエフェクト；駒上昇
	if ( _animation_RisingPiece != NULL ) {
		_animation_RisingPiece->Draw ( window );
	}
	
}
//=========================================================
// 衝撃波エフェクト作成：設置レベル１用のアレ
//=========================================================
void Effect_PutPiece_AI::ImpactWave ( sf::Vector2f positon ) {

	// 音はここで出す
	//GUI_SFML::SoundPlayer::Play ( "./sound/se_good_pk01/bom11.wav" );

	Effect_ImpactWave::Property prop;
	prop.color = sf::Color ( 20,20,100, 100 );
	prop.maxRadius = 100; // APPLICATION::WINDOW_SIZE::WIDTH / 1.5;
	prop.speed = 15;
	prop.position = positon;
	prop.soundName = "";

		
	map<unsigned int, Effect_ImpactWave::Property> impactList;
	for ( int i = 0; i < 5; ++i ) {
		impactList [ i * 2 ] = prop;
	}

	Effect_Impact_Rash *effect = new Effect_Impact_Rash ( impactList );

	Application::GetInstance()->_effectManager-> Add ( effect );
}

//=========================================================
// 駒設置２用効果
//=========================================================
void Effect_PutPiece_AI::Energy ( sf::Vector2f positon ) {

	{
		GUI_SFML::Animation *animation = new GUI_SFML::Animation();
		animation->Init("./effect/putPiece_energy/define_bottom.dat");
		animation->SetPosition( positon );

		IEffect *effect = new Effect_Animation ( animation );
		Application::GetInstance()->_effectManager->Add(effect);
	}
	{
		GUI_SFML::Animation *animation = new GUI_SFML::Animation();
		animation->Init("./effect/putPiece_energy/define.dat");
		animation->SetPosition( positon );

		IEffect *effect = new Effect_Animation ( animation );
		Application::GetInstance()->_effectManager->Add(effect);
	}

}