//=========================================================
//
// 駒設置エフェクト。AI用
//
//=========================================================

#ifndef _EFFECT_PUT_PIECE_AI_
#define _EFFECT_PUT_PIECE_AI_

#include "IEffect.h"
#include "Delegate.h"
#include "../GUI_SFML/Animation.h"
#include "../AI/Move.h"

#include <vector>

class Piece;
class Grid;

using namespace std;

class Effect_PutPiece_AI : public IEffect {

public:

	///// オフセットアニメーション用
	struct OffsetAnimationFrame {

		int x;
		int y;
		int frameCount_Max;
		int frameCount_Current;

		OffsetAnimationFrame () {
			x = 0;
			y = 0;
			frameCount_Max     = 0;
			frameCount_Current = 0;
		}
		OffsetAnimationFrame ( int arg_x, int arg_y, int arg_frame_Max) {
			x = arg_x;
			y = arg_y;
			frameCount_Max     = arg_frame_Max;
			frameCount_Current = 0;
		}
	};

	///// コンストラクタ：必要なオブジェクトを受け取る
	Effect_PutPiece_AI ( 
		DelegateBase     *endMethod,
		AI_Modules::Move  move,
		int               effectLevel,
		Piece            *piece,
		Grid             *grid
	);
	~Effect_PutPiece_AI();


	bool Update(sf::RenderWindow &window);
	void Draw(sf::RenderWindow &window);

private:

	bool _isEnd;

	///// 着手終了時に呼ぶメソッド。GameManager側で用意する。
	DelegateBase *_endMethod;

	///// 監視対象のオブジェクト
	AI_Modules::Move _move;  // つまりこの着手ですよ。
	Piece        *_piece;    // 設置する駒
	Grid         *_goalGrid; // 設置先グリッド

	///// アニメーションエフェクト
	GUI_SFML::Animation *_animation_PutPiece;    // 駒叩きつけ
	GUI_SFML::Animation *_animation_HoldPiece;   // 駒を保持中
	GUI_SFML::Animation *_animation_RisingPiece; // 駒上昇中（叩きつけ前の）

	///// 駒を設置するときの勢いオフセット
	vector < OffsetAnimationFrame > _offsetList_PutPiect; // 駒設置時の叩きつけ動作
	int _offsetList_PutPiect_index;

	///// エフェクトレベルはいくつか？
	int _effectLevel;

	///// 今の状態
	int _currentState;

	//=========================================================
	// 駒設置動作のステージ
	//=========================================================
	static const int STATE_PICK_UP;           // 駒を駒台から持ち上げた
	static const int STATE_GOING_TO_THE_GRID; // 駒が設置先グリッドに向かっている
	static const int STATE_AT_THE_GRID;       // 駒が設置先グリッドに到着した
	static const int STATE_FINISHED;          // 駒をグリッドに設置し終えた


	//=========================================================
	// 駒叩きつけオフセットリスト生成
	//=========================================================
	void MakeOffsetList_For_0();
	void MakeOffsetList_For_3();

	//=========================================================
	// 駒設置１用衝撃波発生
	//=========================================================
	void ImpactWave ( sf::Vector2f positon );

	//=========================================================
	// 駒設置２用効果
	//=========================================================
	void Energy ( sf::Vector2f positon );
};

#endif
