//=========================================================
//
// 着手エフェクト＿駒設置＿人間の操作
//
//=========================================================

#ifndef _EFFECT_PUT_PIECE_HUMAN_
#define _EFFECT_PUT_PIECE_HUMAN_

#include "IEffect.h"
#include "Delegate.h"
#include "../GUI_SFML/Animation.h"

#include <vector>

class Piece;
class PieceBox;
class Board;
class Grid;
class PsychoGage;

using namespace std;

class Effect_PutPiece_Human : public IEffect {

public:

	///// オフセットアニメーション用
	struct OffsetAnimationFrame {

		int x;
		int y;
		int frameCount_Max;
		int frameCount_Current;

		OffsetAnimationFrame () {
			x = 0;
			y = 0;
			frameCount_Max     = 0;
			frameCount_Current = 0;
		}
		OffsetAnimationFrame ( int arg_x, int arg_y, int arg_frame_Max) {
			x = arg_x;
			y = arg_y;
			frameCount_Max     = arg_frame_Max;
			frameCount_Current = 0;
		}
	};

	///// コンストラクタ：必要なオブジェクトを受け取る
	Effect_PutPiece_Human ( 
		DelegateBase *endMethod,
		Piece        *piece,
		PieceBox     *pieceBox,
		Board        *board,
		int           turn,
		PsychoGage   *psychoGage
	);
	~Effect_PutPiece_Human();

	bool Update(sf::RenderWindow &window);
	void Draw(sf::RenderWindow &window);
	
	void Terminate();

private:

	bool _isEnd;

	///// 着手終了時に呼ぶメソッド。GameManager側で用意する。
	DelegateBase *_endMethod;

	///// 監視対象のオブジェクト
	Piece        *_piece;
	PieceBox     *_pieceBox;
	Board        *_board;
	int           _turn;      // Move のいうところの TURN
	Grid        *_goalGrid;   // 設置先グリッド
	PsychoGage  *_psychoGage; // 意志ゲージ
	
	vector < vector < Grid* > > _allGridList;
	AI_Modules::Move _move;

	///// アニメーションエフェクト
	GUI_SFML::Animation *_animation_PutPiece;    // 駒叩きつけ
	GUI_SFML::Animation *_animation_HoldPiece;   // 駒を保持中
	GUI_SFML::Animation *_animation_RisingPiece; // 駒上昇中（叩きつけ前の）

	///// 駒を設置するときの勢いオフセット
	vector < OffsetAnimationFrame > _offsetList_PutPiece; // 駒設置時の叩きつけ動作
	int _offsetList_PutPiece_index;

	///// 今の状態。
	int _currentState;

	///// エフェクトレベルはいくつか？：つまり操作が介入するポイントごとに記録する
	int _effectLevel_toHoldPiece; // 駒を持ったときのエフェクトレベル
	int _effectLevel_toPutPiece;  // 駒設置先を決めたときのエフェクトレベル


	//=========================================================
	// 駒設置動作のステージ
	//=========================================================
	static const int STATE_BEFORE;            // まだ何もしていない
	static const int STATE_HOLDING_NOW;       // 駒をマウスで持っている
	static const int STATE_GOING_TO_THE_GRID; // 設置先グリッドに向かっている
	static const int STATE_FINISHED;          // 駒をグリッドに設置し終えた


	//=========================================================
	// 駒の現在地を探る
	//=========================================================
	bool IsPieceInPieceBox();
	bool IsPieceHeldByMouse();
	bool IsPieceSetInGrid();

	//=========================================================
	// 駒、設置済みだとして、どの座標に設置した？
	//=========================================================
	pair<int, int> GetPieceSettingGrid();

	//=========================================================
	// 駒叩きつけオフセットリスト生成
	//=========================================================
	void MakeOffsetList_For_0();
	void MakeOffsetList_For_3();


	//=========================================================
	// 駒設置１用衝撃波発生
	//=========================================================
	void ImpactWave ( sf::Vector2f positon );

	//=========================================================
	// 駒設置２用効果
	//=========================================================
	void Energy ( sf::Vector2f positon );


	//=========================================================
	// エフェクトレベル用の操作判定
	//=========================================================
	bool IsEffectInput1();
	bool IsEffectInput2();
	bool IsEffectInput3();

};

#endif

