//=========================================================
//
// 千日手の警告メッセージ
//
//=========================================================

#include "Effect_RepetitionMoveAlertMessage.h"
#include "../GUI_SFML/SFML_Utils.h"
#include "Define.h"
#include "../Application.h"
#include "../GUI_SFML/TextImageFactory.h"
#include "../GUI_SFML/SFML_Utils.h"
#include "../GUI_SFML/SoundPlayer.h"
#include "../Utils/Utils_Convert.h"


using namespace GUI_SFML;

//=========================================================
// コンストラクタ
//=========================================================
Effect_RepetitionMoveAlertMessage::Effect_RepetitionMoveAlertMessage ( int currentCount, int maxCount ) {
	
	_position = sf::Vector2f ( APPLICATION::WINDOW_SIZE::WIDTH / 2.0f, APPLICATION::WINDOW_SIZE::HEIGHT / 2.0f + 5.0f );
	
	string message1 = "同じ局面が" + Utils_Convert::Int_to_String(currentCount) + "回出現しました。";
	string message2 = Utils_Convert::Int_to_String(maxCount + 1) + "回目で千日手となります。";
	
	_text1.setFont ( TextImageFactory::_font );
	_text1.setString ( SFML_Utils::GetSfString ( message1) );
	_text2.setFont ( TextImageFactory::_font );
	_text2.setString ( SFML_Utils::GetSfString ( message2) );
	
	int fontSize = 24;
	_lineSpacing  = fontSize + (fontSize / 2.0f);

	_text1.setCharacterSize ( fontSize ); _text1.setColor ( sf::Color::White );
	_text2.setCharacterSize ( fontSize ); _text2.setColor ( sf::Color::White );

	//printf ( "%f,%f\n" , _text.getGlobalBounds().width, _text.getGlobalBounds().height );
	
	//printf ("%f\n", adjustValue);
	
	float adjustValue = -10;////fontSize;//10.0f; // 適当。
	//_text1.setOrigin ( _text1.getGlobalBounds().width / 2.0f, _text1.getGlobalBounds().height / 2.0f + adjustValue - );
	_text1.setOrigin ( _text1.getGlobalBounds().width / 2.0f, _text1.getGlobalBounds().height + adjustValue + _lineSpacing );
	_text1.setPosition(_position);

	_text2.setOrigin ( _text2.getGlobalBounds().width / 2.0f, adjustValue);
	//_text2.setPosition(_position);

	float rectWidth = 
		( _text1.getGlobalBounds().width > _text2.getGlobalBounds().width )
		?
		_text1.getGlobalBounds().width
		:
		_text2.getGlobalBounds().width
		;
	float rectHeight = 
		_text1.getGlobalBounds().height +
		_text2.getGlobalBounds().height +
		_lineSpacing;

	sf::Vector2f rectSize = sf::Vector2f( 
		rectWidth + (fontSize * 2) , 
		rectHeight + (fontSize * 2)
	);
	//sf::Vector2f rectSize = sf::Vector2f( _text.getGlobalBounds().width, _text.getGlobalBounds().height);
	_backRect = sf::RectangleShape ( rectSize );
	_backRect.setOutlineColor ( sf::Color::Blue );
	_backRect.setFillColor ( sf::Color ( 100,100,250,255 ) );
	_backRect.setOutlineThickness(1.0f);
	_backRect.setOrigin ( _backRect.getGlobalBounds().width / 2.0f, _backRect.getGlobalBounds().height / 2.0f );
	
	
	_sound = "./sound/se_good_pk01/push11.wav";
	
	_isEnd = false;
	_isFirst = true;

}


//=========================================================
// デストラクタ
//=========================================================
Effect_RepetitionMoveAlertMessage::~Effect_RepetitionMoveAlertMessage() {
	
}

//=========================================================
// 更新
//=========================================================
bool Effect_RepetitionMoveAlertMessage::Update(sf::RenderWindow &window) {
	
	if ( _isFirst ) {
		_isFirst = false;
		GUI_SFML::SoundPlayer::Play ( _sound );
	}
	
	int colorFadeRate = 2;
	
	sf::Color text1Color =  _text1.getColor();
	if ( (text1Color.a - colorFadeRate) <= 0 ) { text1Color.a = 0;} else { text1Color.a -= colorFadeRate;}
	_text1.setColor ( text1Color );

	sf::Color text2Color =  _text2.getColor();
	if ( (text2Color.a - colorFadeRate) <= 0 ) { text2Color.a = 0;} else { text2Color.a -= colorFadeRate;}
	_text2.setColor ( text2Color );

	sf::Color backFillColor =  _backRect.getFillColor();
	if ( (backFillColor.a - colorFadeRate) <= 0 ) { backFillColor.a = 0;} else { backFillColor.a -= colorFadeRate;}
	_backRect.setFillColor ( backFillColor );

	sf::Color backOutlineColor =  _backRect.getOutlineColor();
	if ( (backOutlineColor.a - colorFadeRate) <= 0 ) { backOutlineColor.a = 0;} else { backOutlineColor.a -= colorFadeRate;}
	_backRect.setOutlineColor ( backOutlineColor );

	if ( text1Color.a <= 0 || text2Color.a <= 0 || backFillColor.a <= 0 || backOutlineColor.a <= 0) {
		_isEnd = true;
		return _isEnd;
	}
	
	// 移動
	_position += sf::Vector2f ( 0.0f, -1.0f );
	
	// カメラに合わせて座標を調整
	sf::Vector2f screenPos = Application::GetInstance()->_window.mapPixelToCoords ( sf::Vector2i ( _position.x, _position.y ) );
	
	
	_backRect.setPosition ( screenPos );
	_text1.setPosition ( screenPos );
	_text2.setPosition ( screenPos );
	

	return _isEnd;
}

//=========================================================
// 描画
//=========================================================
void Effect_RepetitionMoveAlertMessage::Draw(sf::RenderWindow &window) {
	
	if ( !_isEnd ) {
		window.draw(_backRect);
		window.draw(_text1);
		window.draw(_text2);
	}
}

