//=========================================================
//
// Luaのスクリプトで制御するエフェクト
//
//=========================================================

#include "IEffect.h"

#include <./SFML/Graphics.hpp>
#include "../GUI_SFML/Animation.h"
#include "../Image.h"
#include "../GUI_SFML/TextMessageWindow.h"
#include "../GUI_SFML/SoundPlayer.h"
#include <string>
#include <map>
#include "Delegate.h"
#include <lua.hpp>

using namespace std;


class Effect_ScriptedOperation : public IEffect {

public:

	Effect_ScriptedOperation ( DelegateBase *endMethod, int effectLevel, GUI_SFML::TextMessageWindow *textMessageWindow, map<string, string> parameters );
	~Effect_ScriptedOperation ();

	bool Update(sf::RenderWindow &window);
	void Draw(sf::RenderWindow &window);
	
	
	static Effect_ScriptedOperation* LuaedObject;

	
private:

	//=========================================================
	// Lua側から呼び出される処理群 private でもいいの？ いいらしい。
	//=========================================================
	// セリフを言う
	static int FromLua_Remark( lua_State *lua );
	int FromLua_Remark_Object( lua_State *lua );

	// 音を鳴らす
	static int FromLua_PlaySound( lua_State *lua );
	int FromLua_PlaySound_Object( lua_State *lua );

	// 画像を表示する
	static int FromLua_PutImage( lua_State *lua );
	int FromLua_PutImage_Object( lua_State *lua );
	
	// 画像のアルファ値を指定する
	static int FromLua_SetAlphaToImage( lua_State *lua );
	int FromLua_SetAlphaToImage_Object( lua_State *lua );

	// 画像の描画領域を設定する
	static int FromLua_SetTextureRectToImage( lua_State *lua );
	int FromLua_SetTextureRectToImage_Object( lua_State *lua );

	// 画像の中心点を設定する
	static int FromLua_SetOriginToImage( lua_State *lua );
	int FromLua_SetOriginToImageToImage_Object( lua_State *lua );
	
	//static int LuaTest( lua_State *lua );
	//int LuaTest_Object( lua_State *lua );
	

	struct ImageSet {

		sf::Image   image;
		sf::Texture texture;
		sf::Sprite  sprite;

		ImageSet () {
		}
		ImageSet( string file ) {
			image.loadFromFile(file);
			texture.loadFromImage(image);
			sprite.setTexture(texture);
		}

		void ToCenter() {
			sprite.setOrigin ( image.getSize().x / 2, image.getSize().y / 2 );
		}

	};

	bool _isEnd;
	int  _frameCount;
	
	///// 内部的に持つ部品
	map<string, ImageSet*> _imageList; // 画像
	map<string, string>    _soundList; // 音（ファイル名）
	map<string, string>    _textList;  // セリフ
	map<string, string>    _nameLabel; // 名札
	
	// その他のパラメータ：Lua側に渡す。どう使うのかはLua側の受け持ち
	string _otherParams;
	
	///// 終了時に呼ぶメソッド：つまり着手・エフェクトレベル(int) を１つ渡すメソッド。
	DelegateBase *_endMethod;
	int           _effectLevel;

	// テキスト欄
	GUI_SFML::TextMessageWindow *_textMessageWindow;
	
	
	///// Luaオブジェクト：具体的な処理内容はここに委託するよ。
	lua_State *_lua;

	///// Luaスクリプトファイルのファイル名：一応覚えておいてもいいんじゃない？
	string _scriptFileName;
	
	void Script_Init();
	
	string Update_with_Lua();
	void   Draw_with_Lua(sf::RenderWindow &window);
	
	
	
	// Lua経由で画像を表示する際の予約用
	struct ImagePutReserveInfo {
		Effect_ScriptedOperation::ImageSet *imageSet;
		int positionX;
		int positionY;
	};
	vector< ImagePutReserveInfo > _imagePutReserveList;
	

	
	
};

