//=========================================================
//
// 簡易パーティクル
//
//=========================================================

#include "Effect_SimpleParticle.h"
#include "../GUI_SFML/SFML_Utils.h"

//=========================================================
// コンストラクタ
//=========================================================
Effect_SimpleParticle::Effect_SimpleParticle ( Property prop ) {

	_prop = prop;

	// 画像をファイルから読み込むよ！
	_imageSet.image.loadFromFile    ( _prop.imageFile );

	// マスク指定？
	if ( _prop.imageMask != "" ) {
		_imageSet.image.createMaskFromColor(
			GUI_SFML::SFML_Utils::Convert_string_to_color ( _prop.imageMask )
		);
	}

	// 黒っぽいところほど透明にする処理をする？
	if ( _prop.imageAlphaGenerate_baseBlack ) {
		for (int y = 0; y < _imageSet.image.getSize().y; ++y) {
			for (int x = 0; x < _imageSet.image.getSize().x; ++x) {

				sf::Color color = _imageSet.image.getPixel( x, y );
				color.a = color.r;
				_imageSet.image.setPixel( x, y, color );

			}
		}
	}

	_imageSet.texture.loadFromImage ( _imageSet.image );
	_imageSet.sprite.setTexture     ( _imageSet.texture );
	_imageSet.sprite.setOrigin      ( _imageSet.image.getSize().x / 2.0f, _imageSet.image.getSize().y / 2.0f  );
	_imageSet.sprite.setPosition    ( _prop.position );
	_imageSet.sprite.setColor       ( _prop.color_Start );


	// 色の増分を決める
	if ( _prop.totalFrameCount != 0 ) {

		_color_frameAddingAmount_r = ((float)_prop.color_End.r - (float)_prop.color_Start.r) / (float)_prop.totalFrameCount;
		_color_frameAddingAmount_g = ((float)_prop.color_End.g - (float)_prop.color_Start.g) / (float)_prop.totalFrameCount;
		_color_frameAddingAmount_b = ((float)_prop.color_End.b - (float)_prop.color_Start.b) / (float)_prop.totalFrameCount;
		_color_frameAddingAmount_a = ((float)_prop.color_End.a - (float)_prop.color_Start.a) / (float)_prop.totalFrameCount;

		printf ("(%d,%d,%d,%d)\n", 
			_color_frameAddingAmount_r,
			_color_frameAddingAmount_g,
			_color_frameAddingAmount_b,
			_color_frameAddingAmount_a);

	} else {
		// 救済措置：０なんて指定しないでー。
		_color_frameAddingAmount_r = (_prop.color_End.r - _prop.color_Start.r);
		_color_frameAddingAmount_g = (_prop.color_End.g - _prop.color_Start.g);
		_color_frameAddingAmount_b = (_prop.color_End.b - _prop.color_Start.b);
		_color_frameAddingAmount_a = (_prop.color_End.a - _prop.color_Start.a);

	}

	_isEnd = false;
	_currentFrameCount = 0;

}

//=========================================================
// デストラクタ
//=========================================================
Effect_SimpleParticle::~Effect_SimpleParticle() {


}

//=========================================================
// 更新
//=========================================================
bool Effect_SimpleParticle::Update(sf::RenderWindow &window) {

	sf::Color currentColor = _imageSet.sprite.getColor();

	currentColor.r += _color_frameAddingAmount_r;
	currentColor.g += _color_frameAddingAmount_g;
	currentColor.b += _color_frameAddingAmount_b;
	currentColor.a += _color_frameAddingAmount_a;

	_imageSet.sprite.setColor ( currentColor );
	

	++_currentFrameCount;
	if ( _currentFrameCount > _prop.totalFrameCount ) { // =を含まないのは、最後の状態で描画して欲しいから。
		_isEnd = true;
	}

	return _isEnd;
}
//=========================================================
// 描画
//=========================================================
void Effect_SimpleParticle::Draw(sf::RenderWindow &window) {

	window.draw(_imageSet.sprite);
}



