//=========================================================
//
// 着手エフェクト＿カード取得
//
//=========================================================

#include "Effect_TakeCard.h"

#include "Card.h"
#include "CardBox.h"
#include "Piece.h"
#include "Grid.h"
#include "../GUI_SFML/SoundPlayer.h"
#include "CardFactory.h"
#include "Effect_Animation.h"
#include "EffectManager.h"
#include "Application.h"
#include "Effect_TakeCardEnergy.h"

//=========================================================
// カード設置動作のステージ
//=========================================================
const int Effect_TakeCard::STATE_0_PICK_UP          = 0; // エフェクトレベル０：取得するカードを選んだ
const int Effect_TakeCard::STATE_0_GOING_TO_THE_BOX = 1; // エフェクトレベル０：カードがカード置き場に移動中
const int Effect_TakeCard::STATE_0_AT_THE_BOX       = 2; // エフェクトレベル０：カードがカード置き場に到着した

const int Effect_TakeCard::STATE_1_LIGHT_RING_APPEARS  = 10; // 光の輪エフェクト開始
const int Effect_TakeCard::STATE_1_LIGHT_RING_GOING_ON = 11; // 光の輪が縮まる
const int Effect_TakeCard::STATE_1_GOING_TO_THE_BOX    = 12; // カードをカード置き場に移動させる

const int Effect_TakeCard::STATE_2_PICK_UP          = 20; // エフェクトレベル２：取得するカードを選んだ
const int Effect_TakeCard::STATE_2_SOAR             = 21; // エフェクトレベル２：カードが上空へ飛ばされる
const int Effect_TakeCard::STATE_2_GOING_TO_THE_BOX = 22; // エフェクトレベル２：カードがカード置き場に移動中
const int Effect_TakeCard::STATE_2_AT_THE_BOX       = 23; // エフェクトレベル２：カードがカード置き場に到着している

const int Effect_TakeCard::STATE_3_HYPERSPACE_APPEARS  = 30; // 亜空間出現
const int Effect_TakeCard::STATE_3_HYPERSPACE_GOING_ON = 31; // 亜空間活動中 → カード消滅 → 亜空間消滅
const int Effect_TakeCard::STATE_3_CARD_APPEARS        = 32; // カード、カード置き場上空に出現、落下開始
const int Effect_TakeCard::STATE_3_CARD_LANDED         = 33; // カード、カード置き場に着地

const int Effect_TakeCard::STATE_FINISHED           = 3; // レベル共通：終了している。


Card* Effect_TakeCard::LAST_TAKEN_CARD = NULL;

//=========================================================
// コンストラクタ
//=========================================================
Effect_TakeCard::Effect_TakeCard (
		
		DelegateBase     *endMethod,
		AI_Modules::Move  move,
		int               effectLevel,
		Grid            *grid,      // このグリッドから取る
		Card            *card,      // このカードを取る         
		Piece           *piece,     // この駒で取る
		CardBox         *cardBox    // このカード置き場に入れる
		) 
{
	///// 引数を受け取って
	_endMethod   = endMethod;
	_grid        = grid;
	_card        = card;
	_piece       = piece;
	_cardBox     = cardBox;
	_effectLevel = effectLevel;
	_cardDirection = _card->GetCurrentDirection();
	_move        = move;

	///// 初期状態
	_isEnd        = false;
	
	// エフェクト開始とともに、「最後の指し手」のメモを消しておく
	LAST_TAKEN_CARD = NULL;

	///// エフェクトレベル、操作する？
	if ( effectLevel < 0 ) {

		_effectLevel = 0;
		if      ( sf::Keyboard::isKeyPressed ( sf::Keyboard::Num1 ) ) { _effectLevel = 1; }
		else if ( sf::Keyboard::isKeyPressed ( sf::Keyboard::Num2 ) ) { _effectLevel = 2; }
		else if ( sf::Keyboard::isKeyPressed ( sf::Keyboard::Num3 ) ) { _effectLevel = 3; }

	///// 操作しない：引数で渡されたまま、受け取る
	} else {
		_effectLevel = effectLevel;
	}

	///// エフェクトレベル別に、初期状態をセット。
	if      ( _effectLevel == 0 ) { _currentState = STATE_0_PICK_UP; } 
	else if ( _effectLevel == 1 ) { _currentState = STATE_1_LIGHT_RING_APPEARS; }
	else if ( _effectLevel == 2 ) { _currentState = STATE_2_PICK_UP; }
	else if ( _effectLevel == 3 ) { _currentState = STATE_3_HYPERSPACE_APPEARS; }


	///// アニメーションエフェクトはみなさんNULLですよね。
	//_animation_PutPiece    = NULL;
	//_animation_HoldPiece   = NULL;
	//_animation_RisingPiece = NULL;
	//_offsetList_PutPiect_index = -1;
	_animation_TakeCardWind   = NULL;
	_animation_CardArrivesBox = NULL;
	_animation_HyperSpace     = NULL;
	_effect_TakeCardEnergy    = NULL;


	printf ( "カード取得エフェクト_%d (引数 effectLevel:%d) 開始\n", _effectLevel, effectLevel );

}

//=========================================================
// オフセットリスト生成
//=========================================================
void Effect_TakeCard::MakeOffsetList_For_2_SoarCard() {

	_offsetAnimation_For2_SoarCard.offsetList.clear();

	int y = 0; int jump = -25; int gravity = 2;
	while ( jump < 0 ) { // 頂点に達したら終了

		int frameCount = 1;

		_offsetAnimation_For2_SoarCard.offsetList.push_back ( OffsetAnimationFrame ( 0, jump, frameCount ) );

		y    -= jump;
		jump += gravity;
	}

}

//=========================================================
// デストラクタ
//=========================================================
Effect_TakeCard::~Effect_TakeCard() {

	printf ( "カード取得エフェクト終了\n" );

	delete _animation_TakeCardWind;
	delete _animation_CardArrivesBox;
	delete _animation_HyperSpace;
	
	
	// エフェクト終了とともに、「最後の指し手用のメモ」に書き込む
	LAST_TAKEN_CARD = _card;
	
}

//=========================================================
// 更新
//=========================================================
bool Effect_TakeCard::Update(sf::RenderWindow &window) {

	if ( _isEnd ) { return _isEnd; }

	///// エフェクトレベル別の更新処理
	if      ( _effectLevel == 0 ) { Update_EffectLEvel_0 (window); }
	else if ( _effectLevel == 1 ) { Update_EffectLEvel_1 (window); }
	else if ( _effectLevel == 2 ) { Update_EffectLEvel_2 (window); }
	else if ( _effectLevel == 3 ) { Update_EffectLEvel_3 (window); }


	return _isEnd;
}

//=========================================================
// エフェクトレベル別の更新処理：エフェクトレベル０
//=========================================================
void Effect_TakeCard::Update_EffectLEvel_0 (sf::RenderWindow &window) {


	///// 現在の状態：カードを選んだところ
	if ( _currentState == STATE_0_PICK_UP ) {

		// グリッドからカードを取って、
		_grid->SetCard ( NULL );

		// カード置き場にカードを入れる
		_cardBox->AddCard ( _card );

		// カードを盤面から取った効果音
		GUI_SFML::SoundPlayer::Play ( "./sound/se_good_pk01/puu23.wav" );

		// 状態遷移：カード置き場に向かう
		_currentState = STATE_0_GOING_TO_THE_BOX;

	///// 現在の状態：カードがカード置き場に向かっているところ
	} else if ( _currentState == STATE_0_GOING_TO_THE_BOX ) {

		// カードの動きが止まったら、到着の合図だよ。
		if ( _card->IsMovingStop() ) {

			// 状態遷移：カード置き場にいます。
			_currentState = STATE_0_AT_THE_BOX;
		}

	///// 現在の状態：カードがカード置き場にいます。
	} else if ( _currentState == STATE_0_AT_THE_BOX ) {

		// カード置き場に到着した効果音
		GUI_SFML::SoundPlayer::Play ( "./sound/se_good_pk01/puu24.wav" );

		// 終了処理
		(*_endMethod)(_move);
		_isEnd       = true;
		_currentState = STATE_FINISHED;

		printf ("カード取得完了\n");

	///// 現在の状態：終了している
	} else if ( _currentState == STATE_FINISHED ) {

		/*
			ここには来ないはず。
		*/
		_isEnd = true;
	}

}

//=========================================================
// エフェクトレベル別の更新処理：エフェクトレベル１
//=========================================================
void Effect_TakeCard::Update_EffectLEvel_1 (sf::RenderWindow &window) {

	///// 現在の状態：光の輪出現
	if ( _currentState == STATE_1_LIGHT_RING_APPEARS ) {

		Effect_TakeCardEnergy::Property prop;
		prop.firstRadius = 200;
		prop.minRadius   = 20;
		prop.particleNumber = 20;
		prop.position = _card->GetCurrentPosition();
		prop.speed = -20.0f;
		prop.radiusShrinkSpeed = 4;
		prop.animationConfigFile = "./effect/takeCard_energy/define.dat";

		_effect_TakeCardEnergy = new Effect_TakeCardEnergy(prop);

		Application::GetInstance()->_effectManager->Add ( _effect_TakeCardEnergy );

		_currentState = STATE_1_LIGHT_RING_GOING_ON;

		// 出現音
		GUI_SFML::SoundPlayer::Play ( "./sound/se_good_pk01/mecha23_a.wav" );

	///// 現在の状態：光の輪縮まる
	} else if ( _currentState == STATE_1_LIGHT_RING_GOING_ON ) {

		///// 縮まりきった？
		if ( _effect_TakeCardEnergy->IsRadiusMin() ) {

			// 移動開始音
			GUI_SFML::SoundPlayer::Play ( "./sound/se_good_pk01/byoro06.wav" );

			// カードをカード置き場へ移動
			_grid->SetCard(NULL);
			_cardBox->AddCard(_card);

			// 終了
			(*_endMethod)(_move);
			_currentState = STATE_1_GOING_TO_THE_BOX;
		}

	///// 現在の状態：カード置き場へ移動中
	} else if ( _currentState == STATE_1_GOING_TO_THE_BOX ) {

		_effect_TakeCardEnergy->SetPosition ( _card->GetCurrentPosition() );

		// 移動終了？
		if ( _card->IsMovingStop() ) {

			// 光の輪、アニメーション開始（消滅へ向かう）
			_effect_TakeCardEnergy->AnimationGo();

			_currentState = STATE_FINISHED;
		}


	///// 現在の状態：終了している
	} else if ( _currentState == STATE_FINISHED ) {

		/*
			ここには来ないはず。
		*/
		_isEnd = true;
	}

}

//=========================================================
// エフェクトレベル別の更新処理：エフェクトレベル２
//=========================================================
void Effect_TakeCard::Update_EffectLEvel_2 (sf::RenderWindow &window) {

	///// 現在の状態：カードを選んだところ
	if ( _currentState == STATE_2_PICK_UP ) {

		// 竜巻アニメーション作成
		_animation_TakeCardWind = new GUI_SFML::Animation();
		_animation_TakeCardWind->Init ( "./effect/soarCardWind/define.dat");
		_animation_TakeCardWind->SetPosition ( _grid->GetCenterPosition() );

		// カード上昇オフセットを作成
		MakeOffsetList_For_2_SoarCard();
		_offsetAnimation_For2_SoarCard.index = 0;

		// 効果音、は、竜巻アニメに含むことにする。

		// 状態遷移：カード上昇中へ
		_currentState = STATE_2_SOAR;

	///// 現在の状態：カード上昇中
	} else if ( _currentState == STATE_2_SOAR ) {

		// カードをくるくるする
		_card->RollingRight ();

		// 名前が長いので、短く受け止めておく
		int                          &index = _offsetAnimation_For2_SoarCard.index;
		vector<OffsetAnimationFrame> &list  = _offsetAnimation_For2_SoarCard.offsetList;

		///// カードたたきつけオフセットアニメーション
		if ( index != -1) {

			// 現在のコマのカウントが0（初回）のときのみ、カードにオフセットを追加する。
			if ( list[index].frameCount_Current == 0 ) {

				_card->SetOffsetX ( _card->GetOffset().x + list[index].x );
				_card->SetOffsetY ( _card->GetOffset().y + list[index].y );
			}

			// Maxになったら
			if ( list[index].frameCount_Current >= list[index].frameCount_Max) {

				// カウントを０に戻して
				list[index].frameCount_Current = 0;

				// 次へ
				++index;

				// 終了？
				if (index >= list.size()) {
					index = -1; // 終了。
				}

			// まだMaxじゃないなら、カウントを進める
			} else {
				list[index].frameCount_Current += 1;
			}
		}

		///// 上昇オフセットアニメ終了？
		if ( index == -1 ) {

			// オフセットで上昇しているカードのその位置を、実際の位置とする。
			_card->SetPosition ( _card->GetGoalPosition() + _card->GetOffset() );

			// カードのオフセットをクリア
			_card->SetOffset ( 0, 0 );

			// カードをカード置き場にセット
			_grid    -> SetCard ( NULL );
			_cardBox -> AddCard ( _card );

			// カード置き場に向かう瞬間の効果音
			GUI_SFML::SoundPlayer::Play ( "./sound/se_good_pk01/swing02.wav" );

			// 状態遷移：カード置き場に向かっているところ
			_currentState = STATE_2_GOING_TO_THE_BOX;
		}


	///// 現在の状態：カードがカード置き場に向かっているところ
	} else if ( _currentState == STATE_2_GOING_TO_THE_BOX ) {

		// カードをくるくるする
		_card->RollingRight ();

		// 到着した？
		if ( _card->IsMovingStop() ) {

			// この瞬間、着手エフェクトとしては、終了通知を出してしまう。設置エフェクト終了を待たずに次の操作ができるように。
			(*_endMethod)(_move);

			// 到着アニメーション作成
			_animation_CardArrivesBox = new GUI_SFML::Animation();
			_animation_CardArrivesBox->Init ( "./effect/cardArrivesBox/define.dat");
			_animation_CardArrivesBox->SetPosition ( _card->GetGoalPosition() );


			// 状態遷移：
			_currentState = STATE_2_AT_THE_BOX;
		}

	///// 現在の状態：カードがカード置き場に到着している
	} else if ( _currentState == STATE_2_AT_THE_BOX ) {

		// まだカードをくるくるする
		/* 
			このカードが解放されていないという保証はあるか？
			１つ手前の段階で着手終了通知を出しているので、心配。
			
			普通にゲームを進めている限り、ゲーム中にカードを消す、ということはないはず。
			シナリオスクリプト上で操作したら？

			カードファクトリーに存在を問い合わせることとする。
			（カードの生成・解放は、必ずカードファクトリーを通している前提）
		*/
		if ( CardFactory::IsExist ( _card ) ) {
			_card->RollingRight ();
		}

		// カード到着アニメーションが終わっている？
		if ( !_animation_CardArrivesBox->IsActive() ) {

			// カードの向きを戻す：もどさない！
			//_card->SetCurrentDirection ( _cardDirection );

			// 終了処理
			//(*_endMethod)(_move); // は、前の段階で通知してしまっている。エフェクトの終了を待ちたくない！
			_isEnd       = true;
			_currentState = STATE_FINISHED;

			printf ("カード取得完了\n");
		}

	///// 現在の状態：終了している
	} else if ( _currentState == STATE_FINISHED ) {

		/*
			ここには来ないはず。
		*/
		_isEnd       = true;
		_currentState = STATE_FINISHED;
	}

	if ( _animation_TakeCardWind != NULL ) {
		_animation_TakeCardWind->Update ( window );
	}
	if ( _animation_CardArrivesBox != NULL ) {
		_animation_CardArrivesBox->Update ( window );
	}
}


//=========================================================
// エフェクトレベル別の更新処理：エフェクトレベル３
//=========================================================
void Effect_TakeCard::Update_EffectLEvel_3 (sf::RenderWindow &window) {

	//(*_endMethod)(_move);

	///// 現在の状態：亜空間出現
	if ( _currentState == STATE_3_HYPERSPACE_APPEARS ) {

		_animation_HyperSpace = new GUI_SFML::Animation();
		_animation_HyperSpace->Init ( "./effect/takeCard_hyperspace/define.dat" );
		_animation_HyperSpace->SetPosition ( _card->GetCurrentPosition() );

		// 左上に一瞬何かが表示されるのを防ぐ。というか、初回刺激の１回だけでいいはずだったのに。つまり想定外の仕様です。バグともいいますね。なんで２回呼ばないといけないんでしょうね？ 子猫に訊いてもそっぽむくだけ。
		_animation_HyperSpace->Update(window);
		_animation_HyperSpace->Update(window);

		_frameCount_sinceCurrentState = 0;
		_currentState = STATE_3_HYPERSPACE_GOING_ON;

	///// 現在の状態：亜空間活動中
	} else if ( _currentState == STATE_3_HYPERSPACE_GOING_ON ) {

		++_frameCount_sinceCurrentState;

		///// 亜空間進行
		_animation_HyperSpace->Update ( window );

		///// 少し経ったらカード消滅
		if ( _frameCount_sinceCurrentState > 10 ) {

			_grid->SetCard ( NULL );
		}

		///// 亜空間終了？
		if ( !_animation_HyperSpace->IsActive() ) {

			delete _animation_HyperSpace;
			_animation_HyperSpace = NULL;

			_frameCount_sinceCurrentState = 0;
			_currentState = STATE_3_CARD_APPEARS;
		}



	///// 現在の状態：カード、カード置き場上空に出現
	} else if ( _currentState == STATE_3_CARD_APPEARS ) {

		sf::Vector2f cardBoxPos = _cardBox->GetPosition();
		cardBoxPos.y -= 200;
		cardBoxPos.x += _cardBox->GetBoxWidth() / 2;
		_card->SetPosition ( cardBoxPos );

		_cardBox->AddCard ( _card );

		_currentState = STATE_3_CARD_LANDED;

		// 処理を戻す
		(*_endMethod)(_move);

	///// 現在の状態：カード着地
	} else if ( _currentState == STATE_3_CARD_LANDED ) {

		///// 着地した？
		if (_card->IsMovingStop()) {

			GUI_SFML::Animation *animation = new GUI_SFML::Animation();
			animation->Init ( "./effect/takeCard_hyperspace_landed/define.dat" );
			animation->SetPosition ( _card->GetCurrentPosition() );

			Effect_Animation *effect = new Effect_Animation ( animation );
			Application::GetInstance()->_effectManager->Add ( effect );

			_isEnd = true;
			_currentState = STATE_FINISHED;
		}


	///// 現在の状態：終了。ここにはこないはず
	} else if ( _currentState == STATE_FINISHED ) {
		_isEnd = true;
	}

}

//=========================================================
// 描画
//=========================================================
void Effect_TakeCard::Draw(sf::RenderWindow &window) {

	if ( _isEnd ) { return; }

	///// エフェクトレベル別の描画処理
	if      ( _effectLevel == 0 ) { Draw_EffectLEvel_0 (window); }
	else if ( _effectLevel == 1 ) { Draw_EffectLEvel_1 (window); }
	else if ( _effectLevel == 2 ) { Draw_EffectLEvel_2 (window); }
	else if ( _effectLevel == 3 ) { Draw_EffectLEvel_3 (window); }

}
//=========================================================
// エフェクトレベル別の描画処理：エフェクトレベル０
//=========================================================
void Effect_TakeCard::Draw_EffectLEvel_0 (sf::RenderWindow &window) {

	// 処理なし
}
//=========================================================
// エフェクトレベル別の描画処理：エフェクトレベル１
//=========================================================
void Effect_TakeCard::Draw_EffectLEvel_1 (sf::RenderWindow &window) {

	// 処理なし
}
//=========================================================
// エフェクトレベル別の描画処理：エフェクトレベル２
//=========================================================
void Effect_TakeCard::Draw_EffectLEvel_2 (sf::RenderWindow &window) {

	if ( _animation_TakeCardWind != NULL ) {
		_animation_TakeCardWind->Draw ( window );
	}
	if ( _animation_CardArrivesBox != NULL ) {
		_animation_CardArrivesBox->Draw ( window );
	}
}

//=========================================================
// エフェクトレベル別の描画処理：エフェクトレベル３
//=========================================================
void Effect_TakeCard::Draw_EffectLEvel_3 (sf::RenderWindow &window) {

	if ( _animation_HyperSpace != NULL ) {
		_animation_HyperSpace->Draw ( window );
	}
}






