//=========================================================
//
// 着手エフェクト＿カード取得
//
//=========================================================

#ifndef _EFFECT_TAKE_CARD_
#define _EFFECT_TAKE_CARD_

#include "IEffect.h"
#include "Delegate.h"
#include "../GUI_SFML/Animation.h"
#include "../AI/Move.h"

#include <vector>

class Card;
class CardBox;
class Piece;
class Grid;
class Effect_TakeCardEnergy;

class Effect_TakeCard : public IEffect {

public:
	
	static Card* LAST_TAKEN_CARD; // 最後の指し手を強調表示する目的のもの

	///// オフセットアニメーション用
	struct OffsetAnimationFrame {

		int x;
		int y;
		int frameCount_Max;
		int frameCount_Current;

		OffsetAnimationFrame () {
			x = 0;
			y = 0;
			frameCount_Max     = 0;
			frameCount_Current = 0;
		}
		OffsetAnimationFrame ( int arg_x, int arg_y, int arg_frame_Max) {
			x = arg_x;
			y = arg_y;
			frameCount_Max     = arg_frame_Max;
			frameCount_Current = 0;
		}
	};

	///// オフセットアニメーション用
	struct OffsetAnimation {
		vector < OffsetAnimationFrame > offsetList;
		int index;

		OffsetAnimation() {
			index = -1;
		}
	};

	Effect_TakeCard (
		
		DelegateBase     *endMethod,
		AI_Modules::Move  move,
		int               effectLevel,
		Grid            *grid,      // このグリッドから取る
		Card            *card,      // このカードを取る         
		Piece           *piece,     // この駒で取る
		CardBox         *cardBox    // このカード置き場に入れる
		);

	~Effect_TakeCard();

	bool Update(sf::RenderWindow &window);
	void Draw(sf::RenderWindow &window);


private:

	bool _isEnd;

	///// 着手終了時に呼ぶメソッド。GameManager側で用意する。
	DelegateBase *_endMethod;

	///// 監視対象のオブジェクト
	AI_Modules::Move _move;  // つまりこの着手ですよ。
	Grid        *_grid;      // このグリッドから取る
	Card        *_card;      // 取得するカード
	CardBox     *_cardBox;   // カードを追加するカード置き場
	Piece       *_piece;     // この駒で取る
	int          _cardDirection; // カードの向き。エフェクトとして向きをいじった後、元に戻すために覚えておく。

	///// アニメーションエフェクト
	//GUI_SFML::Animation *_animation_PutCard;    // カード叩きつけ
	//GUI_SFML::Animation *_animation_HoldCard;   // カードを保持中
	//GUI_SFML::Animation *_animation_RisingCard; // カード上昇中（叩きつけ前の）
	GUI_SFML::Animation *_animation_TakeCardWind;   // カードを取得時の竜巻
	GUI_SFML::Animation *_animation_CardArrivesBox; // カードがカード置き場に到着
	GUI_SFML::Animation *_animation_HyperSpace;     // 亜空間
	Effect_TakeCardEnergy *_effect_TakeCardEnergy;  // 光の輪

	///// オフセットアニメーション
	//vector < OffsetAnimationFrame > _offsetList_For2_SoarCard; // カード取得時、上空に飛ばされる
	//int _offsetList_PutCard_index;
	OffsetAnimation _offsetAnimation_For2_SoarCard; // カード取得時、上空に飛ばされる

	///// エフェクトレベルはいくつか？
	int _effectLevel;

	///// 今の状態
	int _currentState;
	int _frameCount_sinceCurrentState;

	//=========================================================
	// カード設置動作のステージ
	//=========================================================
	static const int STATE_0_PICK_UP;          // 取得するカードを選んだ
	static const int STATE_0_GOING_TO_THE_BOX; // カードがカード置き場に移動中
	static const int STATE_0_AT_THE_BOX;       // カードがカード置き場に到着している

	static const int STATE_1_LIGHT_RING_APPEARS;  // 光の輪エフェクト開始
	static const int STATE_1_LIGHT_RING_GOING_ON; // 光の輪が縮まる
	static const int STATE_1_GOING_TO_THE_BOX;    // カードをカード置き場に移動させる

	static const int STATE_2_PICK_UP;          // 取得するカードを選んだ
	static const int STATE_2_SOAR;             // カードが上空へ飛ばされる
	static const int STATE_2_GOING_TO_THE_BOX; // カードがカード置き場に移動中
	static const int STATE_2_AT_THE_BOX;       // カードがカード置き場に到着している

	static const int STATE_3_HYPERSPACE_APPEARS;  // 亜空間出現
	static const int STATE_3_HYPERSPACE_GOING_ON; // 亜空間活動中 → カード消滅 → 亜空間消滅
	static const int STATE_3_CARD_APPEARS;        // カード、カード置き場上空に出現、落下開始
	static const int STATE_3_CARD_LANDED;         // カード、カード置き場に着地

	static const int STATE_FINISHED; // レベル共通：終了している。


	//=========================================================
	// オフセットリスト生成
	//=========================================================
	//void MakeOffsetList_For_0();
	//void MakeOffsetList_For_3();
	void MakeOffsetList_For_2_SoarCard();


	//=========================================================
	// エフェクトレベルごとに更新処理を分ける
	//=========================================================
	void Update_EffectLEvel_0 (sf::RenderWindow &window);
	void Update_EffectLEvel_1 (sf::RenderWindow &window);
	void Update_EffectLEvel_2 (sf::RenderWindow &window);
	void Update_EffectLEvel_3 (sf::RenderWindow &window);

	//=========================================================
	// エフェクトレベルごとに描画処理を分ける
	//=========================================================
	void Draw_EffectLEvel_0 (sf::RenderWindow &window);
	void Draw_EffectLEvel_1 (sf::RenderWindow &window);
	void Draw_EffectLEvel_2 (sf::RenderWindow &window);
	void Draw_EffectLEvel_3 (sf::RenderWindow &window);
};

#endif


