//=========================================================
//
// カード取得時のエフェクト
//
//=========================================================

#include "Effect_TakeCardEnergy.h"

#include <math.h>

//=========================================================
// コンストラクタ
//=========================================================
Effect_TakeCardEnergy::Effect_TakeCardEnergy (  Property prop ) {

	_isEnd = false;
	_prop = prop;
	_position = prop.position;
	_isAnimationGo = false;

	int particleNumber = prop.particleNumber;
	_radius = prop.firstRadius;

	// 角度の増分
	float angleStride = 6.28f / (float)particleNumber;


	_animation = new GUI_SFML::Animation();
	_animation->Init ( _prop.animationConfigFile );
	
	sf::RenderWindow window;
	_animation->Update ( window );

	// 初期角度と表示する物体だけセット
	for ( int i = 0; i < particleNumber; ++i ) {

		Particle particle;
		particle.radian = (float)i * angleStride;

		_particleList.push_back (particle);
	}


}

//=========================================================
// デストラクタ
//=========================================================
Effect_TakeCardEnergy::~Effect_TakeCardEnergy() {

	delete _animation;

}

//=========================================================
// 更新
//=========================================================
bool Effect_TakeCardEnergy::Update(sf::RenderWindow &window) {

	if ( _isEnd ) { return _isEnd; }

	///// くるくる
	for ( int i = 0; i < _particleList.size(); ++i ) {

		_particleList[i].radian += _prop.speed;

		if ( _particleList[i].radian > 6.28f ) {
			_particleList[i].radian -= 6.28f;
		}

		_particleList[i].x = cos ( _particleList[i].radian ) * (float)_radius;
		_particleList[i].y = sin ( _particleList[i].radian ) * (float)_radius;

	}

	///// アニメーション進める？
	if ( _isAnimationGo ) {
		_animation->Update ( window );
	}

	///// まだ半径が最小値じゃないなら半径を縮める
	if ( _radius < _prop.minRadius ) {
		//_isEnd = true;
	} else {
		_radius -= _prop.radiusShrinkSpeed;
	}

	///// 終了判定：アニメーションが終わってる？
	_isEnd = !_animation->IsActive();


	return _isEnd;
}

//=========================================================
// 描画
//=========================================================
void Effect_TakeCardEnergy::Draw(sf::RenderWindow &window) {

	for ( unsigned int i = 0; i < _particleList.size(); ++i ) {

		sf::Vector2f position;
		position.x = _particleList[i].x;
		position.y = _particleList[i].y;
		_animation->SetPosition ( position + _position );
		_animation->Draw ( window );
	}

}
