//=========================================================
//
// Card
//
//=========================================================

#include "Card.h"
#include "Grid.h"
#include "Application.h"
#include "Define.h"
#include "CardBackDisplayPanel.h"
#include "../Utils/Utils_Cal.h"
#include "../GUI_SFML/ImageManager.h"
#include "../GUI_SFML/TextImageFactory.h"
#include <SFML/Graphics.hpp>

//=========================================================
// static
//=========================================================
CardBackDisplayPanel* Card::_cardBackDisplayPanel = NULL;

//=========================================================
// コンストラクタ
//=========================================================
Card::Card(Property prop) {
	
	_prop = prop;

	_image = ImageManager::GetImage( _prop.imageFileName, _prop.transparentColor );
	if (!_texture.loadFromImage ( *_image )) { printf("1 Card::Card() : [%s]\n", _prop.imageFileName.c_str()); }
	_sprite.setTexture( _texture );
	_sprite.setOrigin( _image->getSize().x / 2, _image->getSize().y / 2 );
	
	// 画像に合わせて強調表示用の矩形を作成する
	_shape_highlight = sf::RectangleShape ( sf::Vector2f ( _sprite.getGlobalBounds().width, _sprite.getGlobalBounds().height) );
	_shape_highlight.setFillColor ( sf::Color(0,0,255,0));
	_shape_highlight.setOutlineColor (sf::Color(0,0,255,255));
	_shape_highlight.setOutlineThickness (1.0f);
	_shape_highlight.setOrigin ( _image->getSize().x / 2, _image->getSize().y / 2 );
	//_shape_highlight.setOrigin ( _shape_highlight.getGlobalBounds().width/2 , _shape_highlight.getGlobalBounds().height/2 );
	
	_isReservedToDrawHighlight = false;
	
	int fontSize = 10;
	if ( _prop.cardLabel.size() >= 8 ) {
		fontSize = 8;
	}
	
	if ( _prop.cardLabel != "" ) {
	
		TextImageFactory::Create(_nameLabel, _prop.cardLabel, fontSize, sf::Color( 0,0,0,255 ));
		if (!_nameLabel_Texture.loadFromImage( _nameLabel) ) {printf("2 Card::Card() : [%s]\n", _prop.cardLabel.c_str()); }
		_nameLabel_Sprite.setTexture(_nameLabel_Texture);
		_nameLabel_Sprite.setOrigin( _nameLabel.getSize().x / 2.0f, _nameLabel.getSize().y / 2.0f );
	}
	
	_currentDirection = DIRECTION::UP;
	_isMouseOver = false;

	_rotationAngle_Goal = 0;
	_rotationAngle_Current = 0;

	_position_Offset.x = 0;
	_position_Offset.y = 0;

	_isAutonomous = true;
	
	_isWarpToGoal = false;
}

//=========================================================
// デストラクタ
//=========================================================
Card::~Card() {

}


//=========================================================
// 更新
//=========================================================
void Card::Update(sf::RenderWindow &window, sf::Vector2f pos) {

	_position_Goal = pos;

	if ( _isWarpToGoal ) {
		
		_position_Current = _position_Goal;	
		_isWarpToGoal = false;
	
	} else {
		
		// 現在地と目標地の距離の、
		float distance = Utils_Cal::GetDistance( _position_Current.x, _position_Current.y, _position_Goal.x, _position_Goal.y );

		// 半分の距離を、
		float moveDistance = ( distance < 3.0f ) ? distance : distance / 3.0f;

		// 移動する
		pair<float, float> moveResult = 
			Utils_Cal::MoveTo ( _position_Current.x, _position_Current.y, _position_Goal.x, _position_Goal.y, moveDistance );

		_position_Current.x = moveResult.first;
		_position_Current.y = moveResult.second;
	}

	// 反映
	_sprite.setPosition( _position_Current + _position_Offset );
	_shape_highlight.setPosition ( _sprite.getPosition() );

	sf::Vector2f nameLabelOffset(0.0f, 0.0f);
	//nameLabelOffset.y = _image->GetHeight() / 2.0f - _nameLabel.GetHeight() / 2.0f;
	//nameLabelOffset.x = _image->GetWidth() / 2.0f - _nameLabel.GetWidth() / 2.0f;

	_nameLabel_Sprite.setPosition( _position_Current + nameLabelOffset + _position_Offset );


	// マウスオーバー判定（この位置、移動処理の後で行う）
	MouseOverCheck();

	// 自治更新処理
	AutonomousUpdate ( window );

	// 回転監視
	Rolling();

	// マウス真ん中ボタンクリックで、カードの性能パネルを表示
	if (_isMouseOver) {

		if (Application::GetInstance()->_mouseState_Middle == MOUSE::BUTTON::STATE::RELEASE) {

			if (_cardBackDisplayPanel != NULL) {

				if ( _cardBackDisplayPanel->GetCurrentShowingCardName() != _prop.cardName) {

					_cardBackDisplayPanel->SetCardProperty( _prop );
					_cardBackDisplayPanel->Show();
					Application::GetInstance()->_mouseState_Middle = MOUSE::BUTTON::STATE::NONE;
				
				} else {

					_cardBackDisplayPanel->Hide();
					Application::GetInstance()->_mouseState_Middle = MOUSE::BUTTON::STATE::NONE;

				}
			}
		}
	}
}

//=========================================================
// 自治処理
//=========================================================
void Card::AutonomousUpdate ( sf::RenderWindow &window ) {

	if (!_isAutonomous) {
		return;
	}

	///// 保持されている（またはマウスが乗っている）
	//if ( this == Application::GetInstance()->RefferMouseClingObj() || _isMouseOver) {
	
	// マウスが乗っているだけでは回転できないことにする。その理由は右クリックメニューとの衝突だったりする。
	if ( this == Application::GetInstance()->RefferMouseClingObj()) {

		///// 右クリック押下
		if (Application::GetInstance()->_mouseState_Right == MOUSE::BUTTON::STATE::HAVE_PRESSED) {

			Rolling_By_Operatiion();
		}

	///// 保持されていない
	} else {
		//_currentDirection = DIRECTION::UP;
	}
}


//=========================================================
// 回転監視
//=========================================================
void Card::Rolling() {
	
	if (_isWarpToGoalAngle) {
		
		_rotationAngle_Current = _rotationAngle_Goal;
		_isWarpToGoalAngle = false;
	}

	// 回転が必要
	if ( _rotationAngle_Current != _rotationAngle_Goal) {

		float rate =  abs(_rotationAngle_Current -_rotationAngle_Goal) / 3.0f;
		if ( rate < 1.0f ) { rate = 1.0f; }
		_rotationAngle_Current = (int)Utils_Cal::Nearlize( (float)_rotationAngle_Current, (float)_rotationAngle_Goal, rate );

	// 回転が止まっている
	} else {

		while ( _rotationAngle_Goal > 360  ) {
			_rotationAngle_Goal -= 360;
		}
		while ( _rotationAngle_Goal < 0  ) {
			_rotationAngle_Goal += 360;
		}
		_rotationAngle_Current = _rotationAngle_Goal;

	}
	_sprite.setRotation( (float)_rotationAngle_Current );
	_nameLabel_Sprite.setRotation( (float)_rotationAngle_Current );

}
//=========================================================
// 右クリックしたときの回転
//=========================================================
void Card::Rolling_By_Operatiion() {

	_currentDirection = RollDirection( _currentDirection, DIRECTION::RIGHT );

	_rotationAngle_Goal += 90;
}

//=========================================================
// 描画
//=========================================================
void Card::Draw(sf::RenderWindow &window) {

	// マウスにくっついているときには普通に描画する。
	if (Application::GetInstance()->RefferMouseClingObj() == this) {
		DrawActual(window);
		return;
	}

	// 後で前面に描画してもらう → マス目の裏側に入ってしまうのを防ぐ。
	Application::GetInstance()->AddLaterDraw_Card( this );
}

//=========================================================
// 後で前面に描画
//=========================================================
void Card::DrawActual(sf::RenderWindow &window) {

	window.draw(_sprite);
	//window.draw(_nameLabel_Sprite); // 文字で描画すると崩れたりするので画像のみとする。ラベルは棋譜やログ専用

	if ( _isReservedToDrawHighlight ) {
		DrawHighlight(window);
	}
}

//=========================================================
// 強調表示を描画、の予約（アクターと重なる場合の対策）
//=========================================================
void Card::DrawHighlight() {
	
	_isReservedToDrawHighlight = true;
}
//=========================================================
// 強調表示を描画
//=========================================================
void Card::DrawHighlight(sf::RenderWindow &window) {

	window.draw (_shape_highlight);
	
	_isReservedToDrawHighlight = false;
}

//=========================================================
// 指し手の仮表示用の描画処理
//=========================================================
void Card::DrawDemonstration(sf::RenderWindow &window, Grid *grid, int direction) {
	
	if ( grid != NULL ) {
		
		sf::Vector2f pos_org   = _sprite.getPosition();
		sf::Color    color_org = _sprite.getColor();
		float     rotation_org = _sprite.getRotation();
	
		sf::Vector2f pos_temp   =  grid->GetPosition();
		pos_temp.x   +=  grid->GetSize().x / 2.0f;
		pos_temp.y   +=  grid->GetSize().y / 2.0f;
		sf::Color    color_temp = color_org; color_temp.a = 100;
		float     rotation_temp = 0;
		
		if      ( direction == DIRECTION::UP    ) { rotation_temp = 0;}
		else if ( direction == DIRECTION::RIGHT ) { rotation_temp = 90;}
		else if ( direction == DIRECTION::DOWN  ) { rotation_temp = 180;}
		else if ( direction == DIRECTION::LEFT  ) { rotation_temp = 270;}
		
		_sprite.setPosition (pos_temp);
		_sprite.setColor    (color_temp);
		_sprite.setRotation (rotation_temp);
		
		window.draw(_sprite);
		//window.draw(_nameLabel_Sprite);

		_sprite.setPosition (pos_org);
		_sprite.setColor    (color_org);
		_sprite.setRotation (rotation_org);
		
	}
	
}

//=========================================================
// 位置を決める。びよーん なし
//=========================================================
void Card::SetPosition( sf::Vector2f pos) {

	///// びよーん なしで指定の位置になる。
	_position_Goal    = pos;
	_position_Current = pos;
}

//=========================================================
// 影響するグリッドの位置情報をください。
//=========================================================
vector< Card::GridAffection > Card::GetAffectionList() {

	vector< Card::GridAffection > result;

	// 動きが終わってなければ値を返さない。
	/*
	if (_position_Current != _position_Goal) {
		return result;
	}
	*/
	// 回転の動きも？
	/*
	if ( _rotationAngle_Current != _rotationAngle_Goal) {
		return result;
	}
	*/

	// オフセットアニメーションが残っている間は値を返さない
	if ( _position_Offset != sf::Vector2f(0.0f, 0.0f) ) {
		return result;
	}


	for (unsigned int i = 0; i < _prop.gridAffectionList.size(); ++i) {

		// 向きで変換して
		Card::GridAffection affection = 
			ConvertAffectionGrid_by_Direction(
				_prop.gridAffectionList[i],
				_currentDirection
			);

		// 入れる
		result.push_back(affection);
	}


	return result;
}

//=========================================================
// 影響するグリッドの位置情報をください。:向きを指定して
//=========================================================
vector< Card::GridAffection > Card::GetAffectionList( int direction ) {

	vector< Card::GridAffection > result;

	for (unsigned int i = 0; i < _prop.gridAffectionList.size(); ++i) {

		// 向きで変換して
		Card::GridAffection affection = 
			ConvertAffectionGrid_by_Direction(
				_prop.gridAffectionList[i],
				direction
			);

		// 入れる
		result.push_back(affection);
	}

	return result;
}

//=========================================================
// 影響を及ぼす位置情報を向きで変換
//=========================================================
Card::GridAffection Card::ConvertAffectionGrid_by_Direction( GridAffection src, int direction ) {

	Card::GridAffection result = src;

	if (direction == DIRECTION::UP) {
		result = src;
	}
	else if (direction == DIRECTION::LEFT) {
		
		result.x = src.y;
		result.y = -src.x;
		result.movableDirection = RollDirection( src.movableDirection, direction );
	}
	else if (direction == DIRECTION::DOWN) {
		
		result.y = -src.y;
		result.x = -src.x;
		result.movableDirection = RollDirection( src.movableDirection, direction );
	}
	else if (direction == DIRECTION::RIGHT) {
		
		result.x = -src.y;
		result.y = src.x;
		result.movableDirection = RollDirection( src.movableDirection, direction );
	}

	return result;
}

//=========================================================
// 向きを向きで回転させる
//=========================================================
int Card::RollDirection( int direction_src, int direction_move ) {

	int result = direction_src;

	if (direction_src == DIRECTION::LEFT) {

		if      ( direction_move == DIRECTION::LEFT ) { result = DIRECTION::DOWN; }
		else if ( direction_move == DIRECTION::DOWN ) { result = DIRECTION::RIGHT;}
		else if ( direction_move == DIRECTION::RIGHT) { result = DIRECTION::UP;   }
		else if ( direction_move == DIRECTION::UP   ) { result = DIRECTION::LEFT; }
	} 
	else if (direction_src == DIRECTION::DOWN) {

		if      ( direction_move == DIRECTION::LEFT ) { result = DIRECTION::RIGHT; }
		else if ( direction_move == DIRECTION::DOWN ) { result = DIRECTION::UP;    }
		else if ( direction_move == DIRECTION::RIGHT) { result = DIRECTION::LEFT;  }
		else if ( direction_move == DIRECTION::UP   ) { result = DIRECTION::DOWN;  }
	} 
	else if (direction_src == DIRECTION::RIGHT) {

		if      ( direction_move == DIRECTION::LEFT ) { result = DIRECTION::UP;   }
		else if ( direction_move == DIRECTION::DOWN ) { result = DIRECTION::LEFT; }
		else if ( direction_move == DIRECTION::RIGHT) { result = DIRECTION::DOWN; }
		else if ( direction_move == DIRECTION::UP   ) { result = DIRECTION::RIGHT;}
	} 
	else if (direction_src == DIRECTION::UP) {

		if      ( direction_move == DIRECTION::LEFT ) { result = DIRECTION::LEFT;  }
		else if ( direction_move == DIRECTION::DOWN ) { result = DIRECTION::DOWN;  }
		else if ( direction_move == DIRECTION::RIGHT) { result = DIRECTION::RIGHT; }
		else if ( direction_move == DIRECTION::UP   ) { result = DIRECTION::UP;    }
	} 

	return result;
}

//=========================================================
// マウスオーバー判定
//=========================================================
void Card::MouseOverCheck() {

	int mouseX = Application::GetInstance()->_mouseX;
	int mouseY = Application::GetInstance()->_mouseY;

	sf::Vector2f size( _image->getSize());
	sf::Vector2f position = _position_Current - size / 2.0f;

	if
	(
		position.x < mouseX && mouseX < (position.x + size.x) 
		&&
		position.y < mouseY && mouseY < (position.y + size.y) 
	) 
	{
		//printf("mouse on\n");

		_isMouseOver = true;

	} else {
		_isMouseOver = false;
	}
}

//=========================================================
// 向きをセット
//=========================================================
void Card::SetCurrentDirection( int direstion ) {

	if      ( direstion == DIRECTION::UP    ) { _rotationAngle_Goal = 0;}
	else if ( direstion == DIRECTION::LEFT  ) { _rotationAngle_Goal = -90;}
	else if ( direstion == DIRECTION::DOWN  ) { _rotationAngle_Goal = -180;}
	else if ( direstion == DIRECTION::RIGHT ) { _rotationAngle_Goal = -270;}

	_currentDirection = direstion;

}

//=========================================================
// 右回転
//=========================================================
void Card::RollingRight () {

	_currentDirection = RollDirection( _currentDirection, DIRECTION::RIGHT );

	_rotationAngle_Goal += 90;
}
//=========================================================
// 左回転
//=========================================================
void Card::RollingLeft  () {

	_currentDirection = RollDirection( _currentDirection, DIRECTION::LEFT );

	_rotationAngle_Goal -= 90;
}






