//=========================================================
//
// カード生成管理
//
//=========================================================

#include "CardFactory.h"
#include "../Utils/NotXML.h"
#include "../Utils/Utils_String.h"
#include "../Utils/Utils_Convert.h"
#include "../GUI_SFML/SFML_Utils.h"
#include "Define.h"

//=========================================================
// static
//=========================================================
map<string, int>         CardFactory::_cardName_cardId_mapping;
map<int, Card::Property> CardFactory::_cardPropertyList;
set<Card*>               CardFactory::_generatedCardList;

//=========================================================
// 最初に呼ぶ。設定ファイルを読み込む
//=========================================================
void CardFactory::Init() {

	int cardNumSequence = 0;

	vector<NotXML::Tag> tagList = NotXML().Read("./data/cardConfig.xml");

	for (unsigned int i = 0; i < tagList.size(); ++i) {

		NotXML::Tag tag = tagList[i];

		if (tag.name == "card") {

			Card::Property cardProperty;

			cardProperty.cardName         = NotXML::GetTagFromName( tag, "name").value;
			cardProperty.cardLabel        = NotXML::GetTagFromName( tag, "label").value;
			cardProperty.imageFileName    = NotXML::GetTagFromName( tag, "image").attribute["file"];
			cardProperty.transparentColor = GUI_SFML::SFML_Utils::Convert_string_to_color( NotXML::GetTagFromName( tag, "image").attribute["transparentColor"]);

			vector<NotXML::Tag> gridTagList = NotXML::GetTagListFromName( tag, "grid");
			for ( unsigned int n = 0; n < gridTagList.size(); ++n ) {

				Card::GridAffection affection;

				affection.x = Utils_Convert::String_to_Int ( gridTagList[n].attribute["x"] );
				affection.y = Utils_Convert::String_to_Int ( gridTagList[n].attribute["y"] );
				affection.movableDirection = String_to_DIRECTION ( gridTagList[n].attribute["direction"] );

				cardProperty.gridAffectionList.push_back( affection );
			}

			cardProperty.processId = cardNumSequence;
			_cardName_cardId_mapping[cardProperty.cardName] = cardNumSequence;

			_cardPropertyList[ cardNumSequence ] = cardProperty;
			++cardNumSequence;
		}

	}

}

//=========================================================
// 最後に呼ぶ。カードを解放する。
//=========================================================
void CardFactory::Release() {

	while (_generatedCardList.size() > 0) {

		Card *card = *(_generatedCardList.begin());

		delete card;

		_generatedCardList.erase(_generatedCardList.begin());
	}

}

//==========================================================
// カードをオブジェクト指定で解放
//==========================================================
void CardFactory::Release( Card *obj ) {

	//set<Card*> _generatedCardList;
	set<Card*>::iterator it = _generatedCardList.begin();
	while ( it != _generatedCardList.end() ) {

		Card *card = *it;
		if ( obj == card ) {

			_generatedCardList.erase( it );
			break;
		}
		++it;
	}
}

//=========================================================
// オブジェクトの存在を確認
//=========================================================
bool CardFactory::IsExist ( Card *obj ) {

	bool result = false;

	set<Card*>::iterator it = _generatedCardList.begin();
	while ( it != _generatedCardList.end() ) {

		Card *card = *it;
		if ( obj == card ) {

			result = true;
			break;
		}
		++it;
	}

	return result;
}


//=========================================================
// カードをください
//=========================================================
Card* CardFactory::GetCard( string cardName ) {

	Card *result = NULL;


	if ( _cardName_cardId_mapping.find( cardName ) != _cardName_cardId_mapping.end()) {

		int id = _cardName_cardId_mapping.find( cardName )->second;

		if (_cardPropertyList.find( id ) != _cardPropertyList.end()) {

			Card::Property cardProperty = _cardPropertyList.find( id )->second;

			Card *card = new Card( cardProperty );

			_generatedCardList.insert(card);

			result = card;
		}
	}

	return result;
}
//=========================================================
// ちょっと、プロパティを見せて
//=========================================================
Card::Property CardFactory::GetCardProperty ( int cardID ) {

	Card::Property result;

	if ( _cardPropertyList.find(cardID) != _cardPropertyList.end() ) {
		result = _cardPropertyList.find(cardID)->second;
	}

	return result;
}


/*
//=========================================================
// 文字列 → sf::Color
//=========================================================
sf::Color CardFactory::String_to_sfColor(string str, string splitter) {

	sf::Color result;

	vector<string> list = Utils_String::Split( str, splitter );

	///// number of elements is 4 :: with alpha
	if (list.size() == 4) {

		result.r = Utils_Convert::String_to_Int ( list[0] );
		result.g = Utils_Convert::String_to_Int ( list[1] );
		result.b = Utils_Convert::String_to_Int ( list[2] );
		result.a = Utils_Convert::String_to_Int ( list[3] );

	///// number of elements is 3 :: without alpha
	} else if (list.size() == 3) {

		result.r = Utils_Convert::String_to_Int ( list[0] );
		result.g = Utils_Convert::String_to_Int ( list[1] );
		result.b = Utils_Convert::String_to_Int ( list[2] );

	///// Invalid argument
	} else {
		// anything error processing?
	}

	return result;
}
*/
//==========================================================
// カード名からIDを参照
//==========================================================
int CardFactory::GetCardID ( string cardName ) {

	int result = -1;

	if ( _cardName_cardId_mapping.find( cardName ) != _cardName_cardId_mapping.end()) {

		result = _cardName_cardId_mapping[ cardName ];
	}


	return result;
}


//=========================================================
// 文字列 → DIRECTION
//=========================================================
int CardFactory::String_to_DIRECTION(string str) {

	int result = DIRECTION::NONE;

	if      (str == "left" ) { result = DIRECTION::LEFT;  }
	else if (str == "right") { result = DIRECTION::RIGHT; }
	else if (str == "up"   ) { result = DIRECTION::UP;    }
	else if (str == "down" ) { result = DIRECTION::DOWN;  }
	else if (str == "LEFT" ) { result = DIRECTION::LEFT;  }
	else if (str == "RIGHT") { result = DIRECTION::RIGHT; }
	else if (str == "UP"   ) { result = DIRECTION::UP;    }
	else if (str == "DOWN" ) { result = DIRECTION::DOWN;  }

	return result;
}













