//=========================================================
//
// 棋譜
//
//=========================================================

#include "GameRecord.h"

#include <fstream>
#include <iostream>

#include "CardFactory.h"

#include "../Utils/NotXML.h"
#include "../Utils/Utils_String.h"
#include "../Utils/Utils_Convert.h"
#include "Game_Utils.h"

//=========================================================
// コンストラクタ
//【引数】
// 開始局面：解放はこちらで行う。というか解放してしまいますので、ここを呼び出す側は firstState の所有権を手放したと心得てね
//=========================================================
GameRecord::GameRecord ( AI_Modules::InnerExpression *firstState ) {

	_scoreSheetViewer = new ScoreSheetViewer();
	_isShown = false;
	
	_battleTitle = "";

	///// 開始局面を登録
	Record firstRecord;
	firstRecord.isFirstState    = true;
	firstRecord.innerExpression = firstState;

	Add(firstRecord);

}

//=========================================================
// コンストラクタ：棋譜ファイル指定版
//=========================================================
GameRecord::GameRecord ( string fileName ) {

	_isShown = false;
	_scoreSheetViewer = NULL;
	
	_battleTitle = "";

	///// Kifuタグが最上位であるはず。
	NotXML::Tag kifuTag = NotXML().ReadRoot( fileName );

	GenerateRecordListFromTag ( kifuTag );

	/*
	_scoreSheetViewer = new ScoreSheetViewer();
	_isShown = false;

	// これを継ぎ盤として、棋譜を再現していく。
	AI_Modules::InnerExpression *currentInnerExpression = NULL;


	///// Kihuタグが最上位であるはず。
	NotXML::Tag kifuTag = NotXML().ReadRoot( fileName );


	///// 初期状態タグリスト
	vector <NotXML::Tag> firstStateTagList = NotXML::GetTagListFromName( kifuTag, "FirstState" );

	///// 初期状態タグなし
	if ( firstStateTagList.size() == 0 ) {

		// デフォルトとする
		// TODO 棋譜ファイル読み込み時のデフォルト初期局面をどこか別ファイルにでも定義しておきたいかもしれない？ そんな必要はない？ ハードコーディング大好き？
		currentInnerExpression = new AI_Modules::InnerExpression();

		currentInnerExpression->Init();

		currentInnerExpression->_currentTurn = AI_Modules::TURN::BLACK;

		currentInnerExpression->Init_SetHandPiece_BLACK(5);
		currentInnerExpression->Init_SetHandPiece_WHITE(5);
		currentInnerExpression->Init_AddHandCard_BLACK ( CardFactory::GetCardID("utsu") );
		currentInnerExpression->Init_AddHandCard_BLACK ( CardFactory::GetCardID("utsu") );
		currentInnerExpression->Init_AddHandCard_BLACK ( CardFactory::GetCardID("utsu") );
		currentInnerExpression->Init_AddHandCard_BLACK ( CardFactory::GetCardID("syoudou") );
		currentInnerExpression->Init_AddHandCard_BLACK ( CardFactory::GetCardID("syoudou") );
		currentInnerExpression->Init_AddHandCard_BLACK ( CardFactory::GetCardID("syoudou") );
		currentInnerExpression->Init_AddHandCard_BLACK ( CardFactory::GetCardID("iinogare") );
		currentInnerExpression->Init_AddHandCard_BLACK ( CardFactory::GetCardID("iinogare") );
		currentInnerExpression->Init_AddHandCard_BLACK ( CardFactory::GetCardID("iinogare") );
		currentInnerExpression->Init_AddHandCard_BLACK ( CardFactory::GetCardID("keno") );
		currentInnerExpression->Init_AddHandCard_BLACK ( CardFactory::GetCardID("keno") );
		currentInnerExpression->Init_AddHandCard_BLACK ( CardFactory::GetCardID("keno") );
		currentInnerExpression->Init_AddHandCard_BLACK ( CardFactory::GetCardID("ginen") );
		currentInnerExpression->Init_AddHandCard_BLACK ( CardFactory::GetCardID("ginen") );
		currentInnerExpression->Init_AddHandCard_BLACK ( CardFactory::GetCardID("ginen") );

		currentInnerExpression->Init_AddHandCard_WHITE ( CardFactory::GetCardID("utsu") );
		currentInnerExpression->Init_AddHandCard_WHITE ( CardFactory::GetCardID("utsu") );
		currentInnerExpression->Init_AddHandCard_WHITE ( CardFactory::GetCardID("utsu") );
		currentInnerExpression->Init_AddHandCard_WHITE ( CardFactory::GetCardID("syoudou") );
		currentInnerExpression->Init_AddHandCard_WHITE ( CardFactory::GetCardID("syoudou") );
		currentInnerExpression->Init_AddHandCard_WHITE ( CardFactory::GetCardID("syoudou") );
		currentInnerExpression->Init_AddHandCard_WHITE ( CardFactory::GetCardID("iinogare") );
		currentInnerExpression->Init_AddHandCard_WHITE ( CardFactory::GetCardID("iinogare") );
		currentInnerExpression->Init_AddHandCard_WHITE ( CardFactory::GetCardID("iinogare") );
		currentInnerExpression->Init_AddHandCard_WHITE ( CardFactory::GetCardID("keno") );
		currentInnerExpression->Init_AddHandCard_WHITE ( CardFactory::GetCardID("keno") );
		currentInnerExpression->Init_AddHandCard_WHITE ( CardFactory::GetCardID("keno") );
		currentInnerExpression->Init_AddHandCard_WHITE ( CardFactory::GetCardID("ginen") );
		currentInnerExpression->Init_AddHandCard_WHITE ( CardFactory::GetCardID("ginen") );
		currentInnerExpression->Init_AddHandCard_WHITE ( CardFactory::GetCardID("ginen") );

		Record firstRecord;
		firstRecord.innerExpression = currentInnerExpression->GetCopy();
		firstRecord.isFirstState = true;
		Add(firstRecord);


	} else {

		///// 準備
		currentInnerExpression = new AI_Modules::InnerExpression();
		currentInnerExpression->Init();
		currentInnerExpression->_currentTurn = AI_Modules::TURN::BLACK;


		///// 最初に書かれている FirstStateタグを見る。
		NotXML::Tag firstStateTag = firstStateTagList[0];

		///// 駒タグ
		vector<NotXML::Tag> pieceTagList = NotXML::GetTagListFromName(firstStateTag, "Piece");
		for ( NotXML::Tag tag : pieceTagList) {

			string box = NotXML::GetAttribute ( tag, "box", "" );

			// P1 の駒置き場？
			if ( box == "p1" ) {
				currentInnerExpression->Init_SetHandPiece_BLACK ( std::stoi ( NotXML::GetAttribute ( tag, "count", "0" ) ) );
			}
			// P2 の駒置き場？
			else if ( box == "p2" ) {
				currentInnerExpression->Init_SetHandPiece_WHITE ( std::stoi ( NotXML::GetAttribute ( tag, "count", "0" ) ) );
			}
			// 盤面ということなのですね。
			else {

				string color = NotXML::GetAttribute ( tag, "color", "" );
				int colorNum = AI_Modules::TURN::BLACK;
				if      ( color == "p1") { colorNum = AI_Modules::TURN::BLACK; }
				else if ( color == "p2") { colorNum = AI_Modules::TURN::WHITE; }

				currentInnerExpression->Init_PutPiece (
					std::stoi (  NotXML::GetAttribute ( tag, "x", "0" ) ),
					std::stoi (  NotXML::GetAttribute ( tag, "y", "0" ) ),
					colorNum
				);
			}
		}

		///// カードタグ
		vector<NotXML::Tag> cardTagList = NotXML::GetTagListFromName(firstStateTag, "Card");
		for ( NotXML::Tag tag : cardTagList) {

			string box = NotXML::GetAttribute ( tag, "box", "" );
			string cardName = NotXML::GetAttribute ( tag, "name", "" );

			// P1 のカード置き場？
			if ( box == "p1" ) {
				currentInnerExpression->Init_AddHandCard_BLACK ( CardFactory::GetCardID( NotXML::GetAttribute ( tag, "name", "" ) ) );
			}
			// P2 のカード置き場？
			else if ( box == "p2" ) {
				currentInnerExpression->Init_AddHandCard_WHITE ( CardFactory::GetCardID( NotXML::GetAttribute ( tag, "name", "" ) ) );
			}
			// 盤面ということなのですね。
			else {

				string directionStr = NotXML::GetAttribute ( tag, "direction", "UP" );
				int    direction    = Game_Utils::Direction_From_String ( directionStr );

				currentInnerExpression->Init_PutCard (
					std::stoi (  NotXML::GetAttribute ( tag, "x", "0" ) ),
					std::stoi (  NotXML::GetAttribute ( tag, "y", "0" ) ),
					CardFactory::GetCardID( NotXML::GetAttribute ( tag, "name", "" )),
					direction
				);
			}
		}

		///// 手番タグ
		NotXML::Tag turnTag = NotXML::GetTagFromName(firstStateTag, "Turn");
		if      ( turnTag.value == "P1" ) { currentInnerExpression->_currentTurn = AI_Modules::TURN::BLACK; }
		else if ( turnTag.value == "P2" ) { currentInnerExpression->_currentTurn = AI_Modules::TURN::WHITE; }


		Record firstRecord;
		firstRecord.innerExpression = currentInnerExpression->GetCopy();
		firstRecord.isFirstState = true;
		Add(firstRecord);
	}


	///// 着手タグリスト
	vector <NotXML::Tag> moveTagList = NotXML::GetTagListFromName( kifuTag, "Move" );
	for ( NotXML::Tag tag : moveTagList) {

		printf ("Moveタグ！\n");

		Record record;

		record.move.turn = currentInnerExpression->_currentTurn; //(currentInnerExpression->_currentTurn ==  AI_Modules::TURN::BLACK) ? AI_Modules::TURN::WHITE : AI_Modules::TURN::BLACK;
		record.side = (record.move.turn == AI_Modules::TURN::BLACK) ? "P1" : "P2";

		string action = NotXML::GetAttribute (tag, "action", "");

		if ( action == "PUT_CARD" ) {

			string cardName  = NotXML::GetAttribute (tag, "cardName", "");
			string direction = NotXML::GetAttribute (tag, "direction", "");
			string x         = NotXML::GetAttribute (tag, "x", "");
			string y         = NotXML::GetAttribute (tag, "y", "");

			record.move.action = AI_Modules::MOVE::ACTION::PUT_CARD;
			record.move.putCardId = CardFactory::GetCardID ( cardName );
			record.move.putCardX  = std::stoi(x);
			record.move.putCardY  = std::stoi(y);
			record.move.putCardDirection = Game_Utils::Direction_From_String(direction);

		}
		else if ( action == "PUT_PIECE" ) {

			string x = NotXML::GetAttribute (tag, "x", "");
			string y = NotXML::GetAttribute (tag, "y", "");

			record.move.action = AI_Modules::MOVE::ACTION::PUT_PIECE;
			record.move.putPieceX  = std::stoi(x);
			record.move.putPieceY  = std::stoi(y);

		}
		else if ( action == "MOVE_PIECE" ) {

			string fromX   = NotXML::GetAttribute (tag, "fromX", "");
			string fromY   = NotXML::GetAttribute (tag, "fromY", "");
			string toX     = NotXML::GetAttribute (tag, "toX",   "");
			string toY     = NotXML::GetAttribute (tag, "toY",   "");
			string collode = NotXML::GetAttribute (tag, "collide",   "false");

			record.move.action = AI_Modules::MOVE::ACTION::MOVE_PIECE;
			record.move.movePieceFromX  = std::stoi(fromX);
			record.move.movePieceFromY  = std::stoi(fromY);
			record.move.movePieceToX    = std::stoi(toX);
			record.move.movePieceToY    = std::stoi(toY);
			record.move.movePieceAndIsCllide = Utils_String::isPositiveWord(collode);

		}
		else if ( action == "TAKE_CARD" ) {

			string x = NotXML::GetAttribute (tag, "x", "");
			string y = NotXML::GetAttribute (tag, "y", "");

			// TODO takeCard って、カードID いる？

			record.move.action = AI_Modules::MOVE::ACTION::TAKE_CARD;
			record.move.takeCardX  = std::stoi(x);
			record.move.takeCardY  = std::stoi(y);

		}


		record.innerExpression = currentInnerExpression->GetNext( record.move );

		currentInnerExpression->GoNext( record.move );

		Add ( record );

	}


	// 継ぎ盤を解放しておく
	delete currentInnerExpression;
	*/
}


//=========================================================
// コンストラクタ：初期状態なし
//=========================================================
GameRecord::GameRecord () {
	_scoreSheetViewer = new ScoreSheetViewer();
	_isShown = false;
	
	_battleTitle = "";
}

//=========================================================
// デストラクタ
//=========================================================
GameRecord::~GameRecord() {

	ReleaseAllRecord();

	if ( _scoreSheetViewer != NULL ) {
		delete _scoreSheetViewer;
	}
}


//=========================================================
// 棋譜タグの元になる文字列を元に棋譜を読み込む
//=========================================================
void GameRecord::GenerateRecordListFromString ( string &kifuString ) {

	// 文字列をタグ化いたしまして
	NotXML::Tag kifuTag = NotXML().ReadRoot_fromString( kifuString );

	// タグを元に棋譜を読み込む
	GenerateRecordListFromTag (kifuTag);
}

//=========================================================
// 棋譜タグを元に棋譜を読み込む
//=========================================================
void GameRecord::GenerateRecordListFromTag ( NotXML::Tag &kifuTag ) {

	// 既存のものはクリアしておく
	ReleaseAllRecord();

	if ( _scoreSheetViewer != NULL ) {
		delete _scoreSheetViewer;
	}
	_scoreSheetViewer = new ScoreSheetViewer();

	// これを継ぎ盤として、棋譜を再現していく。
	AI_Modules::InnerExpression *currentInnerExpression = NULL;

	///// 試合タイトル
	_battleTitle = "";
	NotXML::Tag battleTitleTag = NotXML::GetTagFromName( kifuTag, "BattleTitle" );
	_battleTitle = battleTitleTag.value;

		
	///// 初期状態タグリスト
	vector <NotXML::Tag> firstStateTagList = NotXML::GetTagListFromName( kifuTag, "FirstState" );

	///// 初期状態タグなし
	if ( firstStateTagList.size() == 0 ) {

		// デフォルトとする
		// TODO 棋譜ファイル読み込み時のデフォルト初期局面をどこか別ファイルにでも定義しておきたいかもしれない？ そんな必要はない？ ハードコーディング大好き？
		currentInnerExpression = new AI_Modules::InnerExpression();

		currentInnerExpression->Init();

		currentInnerExpression->_currentTurn = AI_Modules::TURN::BLACK;

		currentInnerExpression->Init_SetHandPiece_BLACK(5);
		currentInnerExpression->Init_SetHandPiece_WHITE(5);
		currentInnerExpression->Init_AddHandCard_BLACK ( CardFactory::GetCardID("utsu") );
		currentInnerExpression->Init_AddHandCard_BLACK ( CardFactory::GetCardID("utsu") );
		currentInnerExpression->Init_AddHandCard_BLACK ( CardFactory::GetCardID("utsu") );
		currentInnerExpression->Init_AddHandCard_BLACK ( CardFactory::GetCardID("syoudou") );
		currentInnerExpression->Init_AddHandCard_BLACK ( CardFactory::GetCardID("syoudou") );
		currentInnerExpression->Init_AddHandCard_BLACK ( CardFactory::GetCardID("syoudou") );
		currentInnerExpression->Init_AddHandCard_BLACK ( CardFactory::GetCardID("iinogare") );
		currentInnerExpression->Init_AddHandCard_BLACK ( CardFactory::GetCardID("iinogare") );
		currentInnerExpression->Init_AddHandCard_BLACK ( CardFactory::GetCardID("iinogare") );
		currentInnerExpression->Init_AddHandCard_BLACK ( CardFactory::GetCardID("keno") );
		currentInnerExpression->Init_AddHandCard_BLACK ( CardFactory::GetCardID("keno") );
		currentInnerExpression->Init_AddHandCard_BLACK ( CardFactory::GetCardID("keno") );
		currentInnerExpression->Init_AddHandCard_BLACK ( CardFactory::GetCardID("ginen") );
		currentInnerExpression->Init_AddHandCard_BLACK ( CardFactory::GetCardID("ginen") );
		currentInnerExpression->Init_AddHandCard_BLACK ( CardFactory::GetCardID("ginen") );

		currentInnerExpression->Init_AddHandCard_WHITE ( CardFactory::GetCardID("utsu") );
		currentInnerExpression->Init_AddHandCard_WHITE ( CardFactory::GetCardID("utsu") );
		currentInnerExpression->Init_AddHandCard_WHITE ( CardFactory::GetCardID("utsu") );
		currentInnerExpression->Init_AddHandCard_WHITE ( CardFactory::GetCardID("syoudou") );
		currentInnerExpression->Init_AddHandCard_WHITE ( CardFactory::GetCardID("syoudou") );
		currentInnerExpression->Init_AddHandCard_WHITE ( CardFactory::GetCardID("syoudou") );
		currentInnerExpression->Init_AddHandCard_WHITE ( CardFactory::GetCardID("iinogare") );
		currentInnerExpression->Init_AddHandCard_WHITE ( CardFactory::GetCardID("iinogare") );
		currentInnerExpression->Init_AddHandCard_WHITE ( CardFactory::GetCardID("iinogare") );
		currentInnerExpression->Init_AddHandCard_WHITE ( CardFactory::GetCardID("keno") );
		currentInnerExpression->Init_AddHandCard_WHITE ( CardFactory::GetCardID("keno") );
		currentInnerExpression->Init_AddHandCard_WHITE ( CardFactory::GetCardID("keno") );
		currentInnerExpression->Init_AddHandCard_WHITE ( CardFactory::GetCardID("ginen") );
		currentInnerExpression->Init_AddHandCard_WHITE ( CardFactory::GetCardID("ginen") );
		currentInnerExpression->Init_AddHandCard_WHITE ( CardFactory::GetCardID("ginen") );

		Record firstRecord;
		firstRecord.innerExpression = currentInnerExpression->GetCopy();
		firstRecord.isFirstState = true;
		Add(firstRecord);

	} else {

		///// 準備
		currentInnerExpression = new AI_Modules::InnerExpression();
		currentInnerExpression->Init();
		currentInnerExpression->_currentTurn = AI_Modules::TURN::BLACK;


		///// 最初に書かれている FirstStateタグを見る。
		NotXML::Tag firstStateTag = firstStateTagList[0];

		///// 駒タグ
		vector<NotXML::Tag> pieceTagList = NotXML::GetTagListFromName(firstStateTag, "Piece");
		for ( NotXML::Tag tag : pieceTagList) {

			string box = NotXML::GetAttribute ( tag, "box", "" );

			// P1 の駒置き場？
			if ( box == "p1" ) {
				currentInnerExpression->Init_SetHandPiece_BLACK ( std::stoi ( NotXML::GetAttribute ( tag, "count", "0" ) ) );
			}
			// P2 の駒置き場？
			else if ( box == "p2" ) {
				currentInnerExpression->Init_SetHandPiece_WHITE ( std::stoi ( NotXML::GetAttribute ( tag, "count", "0" ) ) );
			}
			// 盤面ということなのですね。
			else {

				string color = NotXML::GetAttribute ( tag, "color", "" );
				int colorNum = AI_Modules::TURN::BLACK;
				if      ( color == "p1") { colorNum = AI_Modules::TURN::BLACK; }
				else if ( color == "p2") { colorNum = AI_Modules::TURN::WHITE; }

				currentInnerExpression->Init_PutPiece (
					std::stoi (  NotXML::GetAttribute ( tag, "x", "0" ) ),
					std::stoi (  NotXML::GetAttribute ( tag, "y", "0" ) ),
					colorNum
				);
			}
		}

		///// カードタグ
		vector<NotXML::Tag> cardTagList = NotXML::GetTagListFromName(firstStateTag, "Card");
		for ( NotXML::Tag tag : cardTagList) {

			string box = NotXML::GetAttribute ( tag, "box", "" );
			string cardName = NotXML::GetAttribute ( tag, "name", "" );

			// P1 のカード置き場？
			if ( box == "p1" ) {
				currentInnerExpression->Init_AddHandCard_BLACK ( CardFactory::GetCardID( NotXML::GetAttribute ( tag, "name", "" ) ) );
			}
			// P2 のカード置き場？
			else if ( box == "p2" ) {
				currentInnerExpression->Init_AddHandCard_WHITE ( CardFactory::GetCardID( NotXML::GetAttribute ( tag, "name", "" ) ) );
			}
			// 盤面ということなのですね。
			else {

				string directionStr = NotXML::GetAttribute ( tag, "direction", "UP" );
				int    direction    = Game_Utils::Direction_From_String ( directionStr );

				currentInnerExpression->Init_PutCard (
					std::stoi (  NotXML::GetAttribute ( tag, "x", "0" ) ),
					std::stoi (  NotXML::GetAttribute ( tag, "y", "0" ) ),
					CardFactory::GetCardID( NotXML::GetAttribute ( tag, "name", "" )),
					direction
				);
			}
		}

		///// 手番タグ
		NotXML::Tag turnTag = NotXML::GetTagFromName(firstStateTag, "Turn");
		if      ( turnTag.value == "P1" ) { currentInnerExpression->_currentTurn = AI_Modules::TURN::BLACK; }
		else if ( turnTag.value == "P2" ) { currentInnerExpression->_currentTurn = AI_Modules::TURN::WHITE; }


		Record firstRecord;
		firstRecord.innerExpression = currentInnerExpression->GetCopy();
		firstRecord.isFirstState = true;
		Add(firstRecord);
	}

	///// 着手タグリスト
	vector <NotXML::Tag> moveTagList = NotXML::GetTagListFromName( kifuTag, "Move" );
	for ( NotXML::Tag tag : moveTagList) {

		//printf ("Moveタグ！\n");

		Record record;

		record.move.turn = currentInnerExpression->_currentTurn; //(currentInnerExpression->_currentTurn ==  AI_Modules::TURN::BLACK) ? AI_Modules::TURN::WHITE : AI_Modules::TURN::BLACK;
		record.side = (record.move.turn == AI_Modules::TURN::BLACK) ? "P1" : "P2";

		string action = NotXML::GetAttribute (tag, "action", "");

		if ( action == "PUT_CARD" ) {

			string cardName  = NotXML::GetAttribute (tag, "cardName", "");
			string direction = NotXML::GetAttribute (tag, "direction", "");
			string x         = NotXML::GetAttribute (tag, "x", "");
			string y         = NotXML::GetAttribute (tag, "y", "");

			record.move.action = AI_Modules::MOVE::ACTION::PUT_CARD;
			record.move.putCardId = CardFactory::GetCardID ( cardName );
			record.move.putCardX  = std::stoi(x);
			record.move.putCardY  = std::stoi(y);
			record.move.putCardDirection = Game_Utils::Direction_From_String(direction);

		}
		else if ( action == "PUT_PIECE" ) {

			string x = NotXML::GetAttribute (tag, "x", "");
			string y = NotXML::GetAttribute (tag, "y", "");

			record.move.action = AI_Modules::MOVE::ACTION::PUT_PIECE;
			record.move.putPieceX  = std::stoi(x);
			record.move.putPieceY  = std::stoi(y);

		}
		else if ( action == "MOVE_PIECE" ) {

			string fromX   = NotXML::GetAttribute (tag, "fromX", "");
			string fromY   = NotXML::GetAttribute (tag, "fromY", "");
			string toX     = NotXML::GetAttribute (tag, "toX",   "");
			string toY     = NotXML::GetAttribute (tag, "toY",   "");
			string collode = NotXML::GetAttribute (tag, "collide",   "false");

			record.move.action = AI_Modules::MOVE::ACTION::MOVE_PIECE;
			record.move.movePieceFromX  = std::stoi(fromX);
			record.move.movePieceFromY  = std::stoi(fromY);
			record.move.movePieceToX    = std::stoi(toX);
			record.move.movePieceToY    = std::stoi(toY);
			record.move.movePieceAndIsCllide = Utils_String::isPositiveWord(collode);

		}
		else if ( action == "TAKE_CARD" ) {

			string x = NotXML::GetAttribute (tag, "x", "");
			string y = NotXML::GetAttribute (tag, "y", "");

			// TODO takeCard って、カードID いる？

			record.move.action = AI_Modules::MOVE::ACTION::TAKE_CARD;
			record.move.takeCardX  = std::stoi(x);
			record.move.takeCardY  = std::stoi(y);

		}


		record.innerExpression = currentInnerExpression->GetNext( record.move );

		currentInnerExpression->GoNext( record.move );

		Add ( record );

	}
	
	///// 終局タグリスト。１つしかないハズではありますが、一応。
	vector <NotXML::Tag> endTagList = NotXML::GetTagListFromName( kifuTag, "End" );
	for ( NotXML::Tag tag : endTagList) {
		
		string description = NotXML::GetAttribute(tag, "description", "終了");
		AddEndNotation(description);
		
	}

	// 継ぎ盤を解放しておく
	delete currentInnerExpression;

}


//=========================================================
// 棋譜の中身を解放（ビューワーには伝えない）
//=========================================================
void GameRecord::ReleaseAllRecord() {

	for ( Record record : _recordList ) {

		delete record.innerExpression;
	}

	_recordList.clear();
}

//=========================================================
// なんと、描画機能があるのでした。
//=========================================================
void GameRecord::Update (sf::RenderWindow &window) {

	// たとえ非表示でも更新処理は行っておく。本当にそれで正解なの？ 多分本当に正解です。多分絶対本当に正解です。
	// 問題はないはずですが、見えないところで動いているというのが不気味なので、やはり見た目に合わせておく。
	if ( _isShown )  {
		_scoreSheetViewer->Update(window);
	}

}
void GameRecord::Draw   (sf::RenderWindow &window) {

	if ( _isShown )  {
		_scoreSheetViewer->Draw(window);
	}

}


//=========================================================
// 一手追加
//=========================================================
void GameRecord::Add ( Record record ) {

	_recordList.push_back ( record );

	_scoreSheetViewer->Add ( RecordToString(record) );
	_scoreSheetViewer->MoveToBottom();


}

//=========================================================
// 終局の文言を追加
//=========================================================
void GameRecord::AddEndNotation ( string description ) {
	
	Record record;
	record.endDescription = description;
	
	// 「局面」はこの時点での最後の局面を受け継いでおく（ナビで「最後へ」を選んだときのため）
	if ( _recordList.size() > 0 ) {
		
		Record lastRecord = _recordList[_recordList.size()-1];
		record.innerExpression = lastRecord.innerExpression->GetCopy();
	}
	
	
	_recordList.push_back ( record );

	_scoreSheetViewer->Add ( RecordToString(record), true );
	_scoreSheetViewer->MoveToBottom();


}


//=========================================================
// 最終手へ移動
//=========================================================
void GameRecord::MoveToBottom() {
	_scoreSheetViewer->MoveToBottom();
}

//=========================================================
// インデックスが変化した？
//=========================================================
bool GameRecord::IsIndexChanged() {

	return _scoreSheetViewer->IsIndexChanged();
}
//=========================================================
// 「次へ」ボタンを押したのでしたっけ？
//=========================================================
bool GameRecord::IsNextPushed() {
	return _scoreSheetViewer->IsNextPushed();
}
//=========================================================
// 現在のインデックスを取得
//=========================================================
GameRecord::Record GameRecord::GetCurrentRecord() {

	Record result;

	int index = _scoreSheetViewer->GetCurrentIndex();

	if ( index != -1 && index < _recordList.size() ) {

		result = _recordList [ index ];
	}

	return result;
}
//=========================================================
// 現在のインデックスを設定
//=========================================================
void GameRecord::SetCurrentIndex ( int index ) {

	_scoreSheetViewer->SetCurrentIndex(index);
}



//=========================================================
// ガイドボタン表示/非表示
//=========================================================
void GameRecord::ShowGuideButtons() { _scoreSheetViewer->ShowGuideButtons(); }
void GameRecord::HideGuideButtons() { _scoreSheetViewer->ShowGuideButtons(); }


//=========================================================
// 棋譜の表示位置を指定
//=========================================================
void GameRecord::SetPosition( sf::Vector2f pos ) {

	_scoreSheetViewer->SetPosition(pos);
}


//=========================================================
// 局面のリスト、という形式で取得
//=========================================================
vector<AI_Modules::InnerExpression*> GameRecord::GetInnerExpressionsList() {

	// 参照を渡すのみ。

	vector<AI_Modules::InnerExpression*> result;

	for ( unsigned int i = 0; i < _recordList.size(); ++i ) {

		result.push_back ( _recordList[i].innerExpression );
	}

	return result;
}


//=========================================================
// sumup評価値のリストをください。
//【引数】
// 手番指定：P1 の分だけ / P2 の分だけ / 空文字で、両方の
//=========================================================
vector<string> GameRecord::GetSumupEvaluationList ( string side ) {

	vector<string> result;

	for ( unsigned int i = 0; i < _recordList.size(); ++i ) {

		if ( side == "" || _recordList[i].side == side ) {

			if ( _recordList[i].side == "P1" ) {
				result.push_back ( _recordList[i].evaluationSumUp_From_P1 );
			}
			else if ( _recordList[i].side == "P2" ) {
				result.push_back ( _recordList[i].evaluationSumUp_From_P2 );
			}
		}
	}

	return result;
}


//=========================================================
// 棋譜をファイルに書き出す
//=========================================================
void GameRecord::OutputScoreSheet ( string fileName, time_t t, string battleTitle ) {

	string scoreSheet = "";
	/*
	for ( unsigned int i = 0; i < _recordList.size(); ++i ) {

		Record record = _recordList[i];

		scoreSheet += RecordToString ( _recordList[i] );
	}
	*/

	scoreSheet += "<Kifu>\n";
	
	///// 試合タイトル
	scoreSheet += "\t<BattleTitle>" + battleTitle + "</BattleTitle>\n";
	
	///// タイムスタンプ
	scoreSheet += "\t<Timestamp>" + Utils_Convert::Int_to_String(t) + "</Timestamp>\n\n";

	///// 人間用の文字列に変換
	scoreSheet += "\t<Omake>\n";
	for ( unsigned int i = 0; i < _recordList.size(); ++i ) {

		Record record = _recordList[i];
		scoreSheet += "\t\t" + RecordToString( _recordList[i] );
	}
	scoreSheet += "\t</Omake>\n\n";


	///// 初期局面


    if ( _recordList.size() > 0 ) {

        scoreSheet += "<FirstState>\n";

        Record firstRecord = _recordList[0];

        AI_Modules::InnerExpression *firstState = firstRecord.innerExpression;
        
        /*
        if ( firstRecord.isFirstState ) {
            scoreSheet += "初期局面ですよ。\n";
        } else {
            scoreSheet += "初期局面じゃないんですって？？？\n";
        }
        */
        
        /*
        
        InnerExpression より。関係ありそうな箇所

		///// 手番情報
		int _currentTurn; // 現在どっちの手番か？ TURN
		
		///// 盤面情報
		int         _gridList_ThrouableInfo[15][15]; // 通行情報レイヤー：通行情報　 THROUABLE_INFO
		int         _gridList_Piece[15][15];         // 駒レイヤー      ：黒/白/なし GRID
		CardOnBoard _gridList_Card[15][15];          // カードレイヤー  ：カードと向き
		static int           _gridList_Init[15][15];     // 初期化用
		static CardOnBoard   _gridList_Init_Card[15][15]; // 初期化用
		int _boardSizeX;
		int _boardSizeY;

		///// 駒台情報：何個あるか、だけ。
		int _handPiece_BLACK;
		int _handPiece_WHITE;

		///// 手札情報：カード番号のリスト
		int _handCards_BLACK[100];
		int _handCards_WHITE[100];
		static int _handCards_Init[100];
		int _handCards_BLACK_Index;
		int _handCards_WHITE_Index;
        
        */
        
        string turn = "";
        if ( firstState->_currentTurn == AI_Modules::TURN::BLACK) { turn = "P1";}
        if ( firstState->_currentTurn == AI_Modules::TURN::WHITE) { turn = "P2";}
        
        
        
        
        // 最初の手番
        scoreSheet += "\t<Turn>" + turn + "</Turn>\n\n";
        
        
        
        
        // 駒台の駒
        scoreSheet += "\t<Piece box=\"p1\" count=\"" + to_string(firstState->_handPiece_BLACK) + "\" />\n";
        scoreSheet += "\t<Piece box=\"p2\" count=\"" + to_string(firstState->_handPiece_WHITE) + "\" />\n";
        
        
        
        
        
        // カード置き場のカード：P1
        for ( int i = 0; i < firstState->_handCards_BLACK_Index; ++i ) {
            
            string side = "p1";
            int cardId = firstState->_handCards_BLACK[i];
             
            string cardName  = "";
            
            Card::Property cardProperty = CardFactory::GetCardProperty(cardId);
            cardName = cardProperty.cardName;
            
            // カード置き場のカードの向きは「UP」で固定としてしまいますよ。
            string direction = "UP";
            
            // <Card name="iinogare"  direction="DOWN" box="p1" />
            scoreSheet += "\t<Card ";

            scoreSheet += "name=\""      + cardName  + "\" ";
            scoreSheet += "direction=\"" + direction + "\" ";
            scoreSheet += "box=\"" + side + "\" ";
            
            scoreSheet += "/>\n";
             
        }

        
        // カード置き場のカード：P2
        for ( int i = 0; i < firstState->_handCards_WHITE_Index; ++i ) {
            
            string side = "p2";
            int cardId = firstState->_handCards_WHITE[i];
             
            string cardName  = "";
            
            Card::Property cardProperty = CardFactory::GetCardProperty(cardId);
            cardName = cardProperty.cardName;
            
            // カード置き場のカードの向きは「UP」で固定としてしまいますよ。
            string direction = "UP";
            
            // <Card name="iinogare"  direction="DOWN" box="p1" />
            scoreSheet += "\t<Card ";

            scoreSheet += "name=\""      + cardName  + "\" ";
            scoreSheet += "direction=\"" + direction + "\" ";
            scoreSheet += "box=\"" + side + "\" ";
            
            scoreSheet += "/>\n";
             
        }
        
        
        // 盤上の駒とカード
        for ( int y = 0; y < firstState->_boardSizeY; ++y ) {
            for ( int x = 0; x < firstState->_boardSizeX; ++x ) {
                
                //_gridList_Piece[15][15]
                //_gridList_Card[15][15] (型 は CardOnBoard)
                
                if ( firstState->_gridList_Piece[y][x] == AI_Modules::GRID::BLACK) { 
                    scoreSheet += "\t<Piece color=\"p1\" x=\"" + to_string(x) + "\" y=\"" + to_string(y) + "\" />\n";
                }
                if ( firstState->_gridList_Piece[y][x] == AI_Modules::GRID::WHITE) { 
                    scoreSheet += "\t<Piece color=\"p2\" x=\"" + to_string(x) + "\" y=\"" + to_string(y) + "\" />\n";
                }
                
                /*
                    struct CardOnBoard {

                        int cardId;       // どのカードが
                        int direction;    // どの向きか                
                */
                
                AI_Modules::InnerExpression::CardOnBoard card = firstState->_gridList_Card[y][x];
                
                
                if ( !card.isEmpty() ) {
                    
                    string cardName  = "";
                    
                    Card::Property cardProperty = CardFactory::GetCardProperty(card.cardId);
                    cardName = cardProperty.cardName;

                    string direction = "";
                    
                    if ( card.direction == AI_Modules::DIRECTION::UP    ) { direction = "UP";    }
                    if ( card.direction == AI_Modules::DIRECTION::DOWN  ) { direction = "DOWN";  }
                    if ( card.direction == AI_Modules::DIRECTION::LEFT  ) { direction = "LEFT";  }
                    if ( card.direction == AI_Modules::DIRECTION::RIGHT ) { direction = "RIGHT"; }
                    
                    
                    // <Card name="iinogare"  direction="DOWN" x="5" y="5" />
                    scoreSheet += "\t<Card ";

                    scoreSheet += "name=\""      + cardName  + "\" ";
                    scoreSheet += "direction=\"" + direction + "\" ";
                    scoreSheet += "x=\"" + to_string(x) + "\" ";
                    scoreSheet += "y=\"" + to_string(y) + "\" ";
                    
                    scoreSheet += "/>\n";
                    
                    
                }
                
                
            }
        }
        
        
        
        scoreSheet += "</FirstState>\n\n";
    }


	///// 棋譜ファイル用の指し手コードに変換
	for ( unsigned int i = 0; i < _recordList.size(); ++i ) {

		Record record = _recordList[i];

		scoreSheet += RecordToKifuFileString ( _recordList[i] );
	}

	scoreSheet += "</Kifu>\n";

	///// ファイルに出力
	std::ofstream ofs;
	ofs.open ( fileName.c_str(), std::ios::out );

		ofs << scoreSheet << std::endl;

	ofs.close();

}


//=========================================================
// 現時点の全棋譜を文字列化（シリアライズ用・なので、人間用のタグは書かない）
//=========================================================
string GameRecord::SerializeAllRecordToString () {

	string scoreSheet = "";

	scoreSheet += "<Kifu>\n";


	///// 棋譜ファイル用の指し手コードに変換
	for ( unsigned int i = 0; i < _recordList.size(); ++i ) {

		Record record = _recordList[i];

		scoreSheet += RecordToKifuFileString ( _recordList[i] );
	}

	scoreSheet += "</Kifu>\n";


	return scoreSheet;
}


//=========================================================
// 指し手の内部表現を棋譜ファイル用の文字列に変換
//=========================================================
string GameRecord::RecordToKifuFileString ( const Record &record ) {

	string result = "";

	// 初期局面？
	if (record.isFirstState) {
		return result;
	}

	// 終局行？
	if (record.endDescription != "") {
		result += "\t<End description=\"" + record.endDescription + "\" />\n";
		
		return result;
	}

	result += "\t<Move action=\"";

	if ( record.move.action == AI_Modules::MOVE::ACTION::PUT_PIECE) {

		string x = to_string ( record.move.putPieceX );
		string y = to_string ( record.move.putPieceY );

		result += "PUT_PIECE\" x=\"" + x + "\" y=\"" + y + "\"";

	} else if ( record.move.action == AI_Modules::MOVE::ACTION::PUT_CARD) {

		string x = to_string ( record.move.putCardX );
		string y = to_string ( record.move.putCardY );
		string cardName  = CardFactory::GetCardProperty( record.move.putCardId).cardName;
		string direction = Game_Utils::Direction_From_int ( record.move.putCardDirection);

		result += "PUT_CARD\" x=\"" + x + "\" y=\"" + y + "\" cardName=\"" + cardName + "\" direction=\"" + direction + "\"";

	} else if ( record.move.action == AI_Modules::MOVE::ACTION::MOVE_PIECE) {

		string fromX = to_string ( record.move.movePieceFromX );
		string fromY = to_string ( record.move.movePieceFromY );
		string toX   = to_string ( record.move.movePieceToX );
		string toY   = to_string ( record.move.movePieceToY );
		string collide =  (record.move.movePieceAndIsCllide ) ? "true" : "false";

		result += "MOVE_PIECE\" fromX=\"" + fromX + "\" fromY=\"" + fromY + "\" toX=\"" + toX + "\" toY=\"" + toY + "\" collide=\"" + collide + "\"";

	} else if ( record.move.action == AI_Modules::MOVE::ACTION::TAKE_CARD) {

		string x = to_string ( record.move.takeCardX );
		string y = to_string ( record.move.takeCardY );
		string cardName  = CardFactory::GetCardProperty( record.move.takeCardCardId).cardName;

		result += "TAKE_CARD\" x=\"" + x + "\" y=\"" + y + "\" cardName=\"" + cardName + "\"";
		
	// おかしいですね。
	} else {
		result += Utils_Convert::Int_to_String(record.move.action) +  "\" error=\"invalidData\"";
		
	}

	result += " />\n";

	return result;
}



//=========================================================
// 指し手の内部表現を棋譜ビューワー用の文言に変換
//=========================================================
string GameRecord::RecordToString ( const Record &record ) {

	string result = "";

	if ( record.isFirstState ) {

		result = "--------------- 開始局面 ---------------\n";

	} else if ( record.endDescription != "" ) {
	
		result = record.endDescription + "\n";
	
	} else {

		if (record.move.action == AI_Modules::MOVE::ACTION::PUT_PIECE ) {

			result += record.side + " ";
			result += "駒設置 (" + to_string ( record.move.putPieceX) + "," + to_string ( record.move.putPieceY) + ")\n";
		}
		else if (record.move.action == AI_Modules::MOVE::ACTION::MOVE_PIECE ) {

			result += record.side + " ";

			if (record.move.movePieceAndIsCllide) {

				result += "駒ぶつける (" + 
					      to_string ( record.move.movePieceFromX) + "," + to_string ( record.move.movePieceFromY) + ") -> " +
					"(" + to_string ( record.move.movePieceToX  ) + "," + to_string ( record.move.movePieceToY  ) + ")\n";

			} else {

				result += "駒移動 (" +
					      to_string ( record.move.movePieceFromX) + "," + to_string ( record.move.movePieceFromY) + ") -> " +
					"(" + to_string ( record.move.movePieceToX  ) + "," + to_string ( record.move.movePieceToY  ) + ")\n";
			}
		}
		else if (record.move.action == AI_Modules::MOVE::ACTION::PUT_CARD ) {

			result += record.side + " ";
			string cardName = CardFactory::GetCardProperty (record.move.putCardId).cardLabel; //cardName;

			string directionLabel = "";
            if ( record.move.putCardDirection == AI_Modules::DIRECTION::UP    ) { directionLabel = "上向き"; }
            if ( record.move.putCardDirection == AI_Modules::DIRECTION::DOWN  ) { directionLabel = "下向き"; }
            if ( record.move.putCardDirection == AI_Modules::DIRECTION::LEFT  ) { directionLabel = "左向き"; }
            if ( record.move.putCardDirection == AI_Modules::DIRECTION::RIGHT ) { directionLabel = "右向き"; }

			result += "カード設置 " + cardName + " " +
				
				// 位置
				"(" + to_string ( record.move.putCardX) + "," + to_string ( record.move.putCardY) + ")" + 
				
				// 向き
				directionLabel + "\n";
		}
		else if (record.move.action == AI_Modules::MOVE::ACTION::TAKE_CARD ) {

			result += record.side + " ";
			string cardName = CardFactory::GetCardProperty (record.move.takeCardCardId).cardName;

			result += "カード取得 " + cardName + " " +
				to_string ( record.move.takeCardX) + to_string ( record.move.takeCardY) + "\n";
		}
	}


	return result;
}


//=========================================================
// セーブ/ロード対応
//=========================================================
string GameRecord::Serialize() {

	string result = "";

/*
	vector<Record> _recordList;
	ScoreSheetViewer *_scoreSheetViewer;
	bool _isShown;
*/
	string delim = "<>";

	string recordList       = SerializeAllRecordToString ();
	string scoreSheetViewer = ( _scoreSheetViewer == NULL ) ? "NULL" : _scoreSheetViewer->Serialize();
	string isShown          = ( _isShown ) ? "true" : "false";

	result  = recordList + delim;
	result += scoreSheetViewer + delim;
	result += isShown;

	return result;
}

//=========================================================
// セーブ/ロード対応
//=========================================================
void GameRecord::DeSerialize( string data ) {

	vector<string> dataList = Utils_String::Split ( data, "<>" );

	if ( dataList.size() < 3 ) {
		printf ( "セーブデータが壊れています： GameRecord::DeSerialize( %s )\n", data.c_str());
	}

	string recordList       = dataList[0];
	string scoreSheetViewer = dataList[1];
	string isShown          = dataList[2];

	GenerateRecordListFromString ( recordList );

	if ( _scoreSheetViewer != NULL ) {
		delete _scoreSheetViewer;
		_scoreSheetViewer = NULL;
	}

	if ( scoreSheetViewer != "NULL" ) {
		_scoreSheetViewer = new ScoreSheetViewer();
		_scoreSheetViewer->DeSerialize(scoreSheetViewer);
	}

	_isShown = Utils_String::isPositiveWord(isShown);

}















