//=========================================================
//
// 棋譜
//
//=========================================================

#ifndef _GAME_RECORD_
#define _GAME_RECORD_

#include "../AI/Move.h"
#include "../AI/InnerExpression.h"
#include "../AI/Define.h"

#include <vector>


#include "ScoreSheetViewer.h"
#include "../Utils/NotXML.h"

using namespace std;

class GameRecord {

public:

	//=========================================================
	// 棋譜上の１手を表す構造体
	//=========================================================
	struct Record {

		bool                         isFirstState;            // 開始局面ですよ。
		string                       endDescription;          // 終局の文言ですよ。（これが空文字かどうかで終局行かどうかのフラグを兼ねることとする）
		AI_Modules::Move             move;                    // 着手
		AI_Modules::InnerExpression *innerExpression;         // 局面
		string                       side;                    // 手番 P1 / P2
		string                       evaluationSumUp_From_P1; // P1の番だったとき、P1からの評価値（外部向け総括）
		string                       evaluationSumUp_From_P2; // P2の番だったとき、P1からの評価値（外部向け総括）
		
		Record () {
			move.action             = AI_Modules::MOVE::ACTION::PASS;
			innerExpression         = NULL;
			side                    = "";
			evaluationSumUp_From_P1 = "";
			evaluationSumUp_From_P2 = "";
			isFirstState            = false;
			endDescription          = "";
		}
	};

	//=========================================================
	// コンストラクタ
	//【引数】
	// 開始局面
	//=========================================================
	GameRecord ();
	GameRecord ( AI_Modules::InnerExpression *firstState );
	GameRecord ( string fileName );
	~GameRecord ();
	
	
	//=========================================================
	// なんと、描画機能があるのでした。
	//=========================================================
	void Update (sf::RenderWindow &window);
	void Draw   (sf::RenderWindow &window);

	//=========================================================
	// 一手追加
	//=========================================================
	void Add ( Record record );
	
	//=========================================================
	// 終局の文言を追加
	//=========================================================
	void AddEndNotation ( string description );
	
	
	
	//=========================================================
	// ビューワー操作
	//=========================================================
	// 棋譜の表示位置
	void SetPosition( sf::Vector2f pos );
	void MoveToBottom();
	//void MoveToTop();
	//bool   IsClicked();
	//Record GetClickedRecord();
	
	bool IsIndexChanged();
	Record GetCurrentRecord();
	bool IsNextPushed();
	void SetCurrentIndex ( int index );
	
	void ShowGuideButtons();
	void HideGuideButtons();
	
	// 表示/非表示
	void Show() { _isShown = true; }
	void Hide() { _isShown = false; }
	bool IsShown() { return _isShown; }
	
	//=========================================================
	// 全部取得
	//=========================================================
	vector<Record> GetRecordList() { return _recordList; }
	
	
	//=========================================================
	// 局面のリスト、という形式で取得
	//=========================================================
	vector<AI_Modules::InnerExpression*> GetInnerExpressionsList();

	//=========================================================
	// sumup評価値のリストをください。
	//【引数】
	// 手番指定：P1 の分だけ / P2 の分だけ / 空文字で、両方の
	//=========================================================
	vector<string> GetSumupEvaluationList ( string side = "" );
	
	
	//=========================================================
	// 棋譜をファイルに書き出す
	//=========================================================
	//void OutputScoreSheet ( string fileName, string battleTitle = "" );
	void OutputScoreSheet ( string fileName, time_t t, string battleTitle = "" );
	
	string RecordToString ( const Record &record );
	string RecordToKifuFileString ( const Record &record );
	
	
	//=========================================================
	// セーブ/ロード対応
	//=========================================================
	string Serialize();
	void DeSerialize( string data );
	
	string GetBattleTitle() { return _battleTitle; }
	void SetBattleTitle ( string battleTitle ) { _battleTitle = battleTitle; } // ロード時にGameManager側から入れる専用かもしれない

private:
	
	vector<Record> _recordList;
	
	ScoreSheetViewer *_scoreSheetViewer;
	
	bool _isShown;
		
	
	void ReleaseAllRecord();
	
	string _battleTitle;
	
	string SerializeAllRecordToString ();
	
	///// 棋譜タグを元に棋譜を読み込む
	void GenerateRecordListFromTag ( NotXML::Tag &kifuTag );
	
	///// 棋譜タグの元になる文字列を元に棋譜を読み込む
	void GenerateRecordListFromString ( string &kifuString );
	
};
#endif

