//=========================================================
//
// Gme_Utils (implement)
//
//=========================================================

#include "Game_Utils.h"
#include "../AI/Define.h"
#include "../Utils/Utils_Convert.h"
#include "../Utils/Utils_Cal.h"
#include "CardFactory.h"
#include "Define.h"

//=========================================================
// タグからMove構造体に変換
//=========================================================
AI_Modules::Move Game_Utils::Move_From_Tag ( const NotXML::Tag &moveTag ) {

	AI_Modules::Move result;

	///// 本当にMoveタグですか？
	if (moveTag.name != "Move") {
		result.action = AI_Modules::MOVE::ACTION::PASS;
		return result;
	}
	
	///// 一行形式タグの場合
	if ( moveTag.childList.size() == 0 ) {

        // つまり、棋譜に記載されるタグの形式、ということ
        
        // ACTION
        string action = NotXML::GetAttribute ( moveTag, "action", "" );
        result.action = AI_Modules::MOVE::ACTION::PUT_PIECE;

        // TURN
        string turn   = NotXML::GetAttribute ( moveTag, "turn", "" ); // BLACK or WHITE

        if ( turn == "BLACK"  ) { result.turn = AI_Modules::TURN::BLACK; }
        if ( turn == "WHITE"  ) { result.turn = AI_Modules::TURN::WHITE; }
        if ( turn == "NONE"  )  { result.turn = AI_Modules::TURN::NONE; }
        
        
        if ( action == "PUT_PIECE" ) {
            
            printf ( "Moveタグ：PUT_PIECE\n" );
            
            // <Move action="PUT_PIECE" x="6" y="0" />
        
            string x = NotXML::GetAttribute ( moveTag, "x", "0" );
            string y = NotXML::GetAttribute ( moveTag, "y", "0" );
            
            result.putPieceX = stoi(x);
            result.putPieceY = stoi(y);
        
        } else if ( action == "PUT_CARD" ) {
            
            printf ( "Moveタグ：PUT_CARD\n" );
            
            // <Move action="PUT_CARD" x="1" y="8" cardName="ginen" direction="RIGHT" />
            
            string x         = NotXML::GetAttribute ( moveTag, "x", "0" );
            string y         = NotXML::GetAttribute ( moveTag, "y", "0" );
            string direction = NotXML::GetAttribute ( moveTag, "direction", "UP" );
            string cardName  = NotXML::GetAttribute ( moveTag, "cardName", "" );
            
            
            result.action = AI_Modules::MOVE::ACTION::PUT_CARD;
            result.putCardX = stoi(x);
            result.putCardY = stoi(y);
            
            result.putCardId = CardFactory::GetCardID ( cardName );
        
            if      ( direction == "up"    ) { result.putCardDirection = AI_Modules::DIRECTION::UP;}
            else if ( direction == "UP"    ) { result.putCardDirection = AI_Modules::DIRECTION::UP;}
            else if ( direction == "down"  ) { result.putCardDirection = AI_Modules::DIRECTION::DOWN;}
            else if ( direction == "DOWN"  ) { result.putCardDirection = AI_Modules::DIRECTION::DOWN;}
            else if ( direction == "left"  ) { result.putCardDirection = AI_Modules::DIRECTION::LEFT;}
            else if ( direction == "LEFT"  ) { result.putCardDirection = AI_Modules::DIRECTION::LEFT;}
            else if ( direction == "right" ) { result.putCardDirection = AI_Modules::DIRECTION::RIGHT;}
            else if ( direction == "RIGHT" ) { result.putCardDirection = AI_Modules::DIRECTION::RIGHT;}
            else                             { result.putCardDirection = Utils_Convert::String_to_Int( direction );}
            
        
        } else if ( action == "MOVE_PIECE" ) {
            
            printf ( "Moveタグ：MOVE_PIECE\n" );
            
            // <Move action="MOVE_PIECE" fromX="3" fromY="2" toX="3" toY="6" collide="false" />
            
            string fromX     = NotXML::GetAttribute ( moveTag, "fromX", "0" );
            string fromY     = NotXML::GetAttribute ( moveTag, "fromY", "0" );
            string toX       = NotXML::GetAttribute ( moveTag, "toX", "0" );
            string toY       = NotXML::GetAttribute ( moveTag, "toY", "0" );
            string collide   = NotXML::GetAttribute   ( moveTag, "collide", "false" );
            
            result.action         = AI_Modules::MOVE::ACTION::MOVE_PIECE;
            result.movePieceFromX = stoi(fromX);
            result.movePieceFromY = stoi(fromY);
            result.movePieceToX   = stoi(toX);
            result.movePieceToY   = stoi(toY);
            result.movePieceAndIsCllide = Utils_Convert::String_to_bool(collide);
            

        } else if ( action == "TAKE_CARD" ) {
            
            printf ( "Moveタグ：TAKE_CARD\n" );
            
            // <Move action="TAKE_CARD" x="2" y="3" cardName="" />

            string x = NotXML::GetAttribute ( moveTag, "x", "0" );
            string y = NotXML::GetAttribute ( moveTag, "y", "0" );
            
            result.action = AI_Modules::MOVE::ACTION::TAKE_CARD;
            result.takeCardX = stoi(x);
            result.takeCardY = stoi(y);

        }
	
	
	
	///// サブタグ形式の場合
	} else {

        ///// 中身のタグでループ
        for ( unsigned int i = 0; i < moveTag.childList.size(); ++i ) {

            NotXML::Tag subTag = moveTag.childList[i];

            if ( subTag.name == "ACTION" ) {
                if ( subTag.value == "PUT_PIECE"  ) { result.action = AI_Modules::MOVE::ACTION::PUT_PIECE; }
                if ( subTag.value == "PUT_CARD"   ) { result.action = AI_Modules::MOVE::ACTION::PUT_CARD; }
                if ( subTag.value == "MOVE_PIECE" ) { result.action = AI_Modules::MOVE::ACTION::MOVE_PIECE; }
                if ( subTag.value == "TAKE_CARD"  ) { result.action = AI_Modules::MOVE::ACTION::TAKE_CARD; }
                if ( subTag.value == "PASS"       ) { result.action = AI_Modules::MOVE::ACTION::PASS; }
            }
            if ( subTag.name == "TURN" ) {
                if ( subTag.value == "BLACK"  ) { result.turn = AI_Modules::TURN::BLACK; }
                if ( subTag.value == "WHITE"  ) { result.turn = AI_Modules::TURN::WHITE; }
                if ( subTag.value == "NONE"  ) { result.turn = AI_Modules::TURN::NONE; }
            }
            if ( subTag.name == "putPieceX" ) { result.putPieceX = Utils_Convert::String_to_Int( subTag.value );}
            if ( subTag.name == "putPieceY" ) { result.putPieceY = Utils_Convert::String_to_Int( subTag.value );}

            if ( subTag.name == "putCardX"         ) { result.putCardX = Utils_Convert::String_to_Int( subTag.value );}
            if ( subTag.name == "putCardY"         ) { result.putCardY = Utils_Convert::String_to_Int( subTag.value );}
            if ( subTag.name == "putCardId"        ) { result.putCardId = Utils_Convert::String_to_Int( subTag.value );}
            if ( subTag.name == "putCardName"      ) { result.putCardId = CardFactory::GetCardID ( subTag.value );}
            if ( subTag.name == "putCardDirection" ) { 

                if      ( subTag.value == "up"    ) { result.putCardDirection = AI_Modules::DIRECTION::UP;}
                else if ( subTag.value == "UP"    ) { result.putCardDirection = AI_Modules::DIRECTION::UP;}
                else if ( subTag.value == "down"  ) { result.putCardDirection = AI_Modules::DIRECTION::DOWN;}
                else if ( subTag.value == "DOWN"  ) { result.putCardDirection = AI_Modules::DIRECTION::DOWN;}
                else if ( subTag.value == "left"  ) { result.putCardDirection = AI_Modules::DIRECTION::LEFT;}
                else if ( subTag.value == "LEFT"  ) { result.putCardDirection = AI_Modules::DIRECTION::LEFT;}
                else if ( subTag.value == "right" ) { result.putCardDirection = AI_Modules::DIRECTION::RIGHT;}
                else if ( subTag.value == "RIGHT" ) { result.putCardDirection = AI_Modules::DIRECTION::RIGHT;}
                else                                { result.putCardDirection = Utils_Convert::String_to_Int( subTag.value );}
            }

            if ( subTag.name == "movePieceFromX"       ) { result.movePieceFromX = Utils_Convert::String_to_Int( subTag.value );}
            if ( subTag.name == "movePieceFromY"       ) { result.movePieceFromY = Utils_Convert::String_to_Int( subTag.value );}
            if ( subTag.name == "movePieceToX"         ) { result.movePieceToX   = Utils_Convert::String_to_Int( subTag.value );}
            if ( subTag.name == "movePieceToY"         ) { result.movePieceToY   = Utils_Convert::String_to_Int( subTag.value );}
            if ( subTag.name == "movePieceAndIsCllide" ) { result.movePieceAndIsCllide = Utils_Convert::String_to_bool( subTag.value );}

            if ( subTag.name == "takeCardX" ) { result.takeCardX = Utils_Convert::String_to_Int( subTag.value );}
            if ( subTag.name == "takeCardY" ) { result.takeCardY = Utils_Convert::String_to_Int( subTag.value );}

        }
    }


	return result;

}

//=========================================================
// 文字列を方向に変換
//=========================================================
int Game_Utils::Direction_From_String ( string &direction ) {

	int result = DIRECTION::NONE;

	if      ( direction == "left"  || direction == "LEFT"  ) { result = DIRECTION::LEFT;  }
	else if ( direction == "right" || direction == "RIGHT" ) { result = DIRECTION::RIGHT; }
	else if ( direction == "up"    || direction == "UP"    ) { result = DIRECTION::UP;    }
	else if ( direction == "down"  || direction == "DOWN"  ) { result = DIRECTION::DOWN;  }

	return result;
}

//=========================================================
// 数値を方向文字列に変換
//=========================================================
string Game_Utils::Direction_From_int ( int direction ) {
	
	string result = "";

	if      ( direction == DIRECTION::LEFT  ) { result = "LEFT"; }
	else if ( direction == DIRECTION::RIGHT ) { result = "RIGHT";}
	else if ( direction == DIRECTION::UP    ) { result = "UP";   }
	else if ( direction == DIRECTION::DOWN  ) { result = "DOWN"; }

	return result;
	
}
	

//=========================================================
// ぶつける移動のとき、ぶつける相手の駒の位置を取得
//=========================================================
pair<int, int> Game_Utils::GetCollidedPiecePosition ( const AI_Modules::Move &move ) {

	pair<int, int> result = pair<int, int>(-1, -1);

	// 駒移動じゃないとか、ぶつけない、とかのとき、結果はどうなっても知りませんよー。
	if ( move.action != AI_Modules::MOVE::ACTION::MOVE_PIECE || !move.movePieceAndIsCllide) {
		return result;
	}

	result.first  = move.movePieceToX + Utils_Cal::Sign ( move.movePieceToX - move.movePieceFromX);
	result.second = move.movePieceToY + Utils_Cal::Sign ( move.movePieceToY - move.movePieceFromY);


	return result;
}


