//=========================================================
//
// 駒置き場
//
//=========================================================

#include "PieceBox.h"
#include "Piece.h"
#include "Application.h"
#include "Define.h"
#include "../Utils/Utils_Convert.h"
#include "../Utils/Utils_String.h"
#include "../Utils/Utils_Cal.h"

#include <SFML/Graphics.hpp>

#include <vector>

//=========================================================
// コンストラクタ
//=========================================================
PieceBox::PieceBox() {

	_boxSize = sf::Vector2f( 150.0f, 30.0f );

	_box = sf::RectangleShape (_boxSize);
	_box.setFillColor (sf::Color(255,255,200,200));
	_box.setOutlineColor (sf::Color(0,0,0,0));
	_box.setOutlineThickness ( 0.0f);

	_isMouseOver  = false;
	_isAutonomous = true;

	_mouseOverPiece = NULL;
}
//=========================================================
// デストラクタ
//=========================================================
PieceBox::~PieceBox() {

}

//=========================================================
// 更新
//=========================================================
void PieceBox::Update(sf::RenderWindow &window, sf::Vector2f pos) {

	_box.setPosition( pos );

	MouseOverCheck();

	sf::Vector2f currentPiecePos = pos + sf::Vector2f(15.0f, 15.0f);

	_mouseOverPiece = NULL;

	//=========================================================
	// 各駒を更新
	//=========================================================
	//for ( unsigned int i = 0; i < _pieceList.size(); ++i ) {
	for ( vector<Piece*>::iterator it = _pieceList.begin(); it != _pieceList.end(); ++it) {

		Piece *piece = *(it);

		piece->Update( window, currentPiecePos);

		currentPiecePos += sf::Vector2f( 30.0f, 0.0f );

		// マウスが乗っていたら、覚えておく。
		if ( piece->IsMouseOver() ) {
			_mouseOverPiece = piece;
		}
	}

	// 自治更新処理
	AutonomousUpdate ( window );
}

//=========================================================
// 自治更新処理
//=========================================================
void PieceBox::AutonomousUpdate ( sf::RenderWindow &window ) {

	if ( !_isAutonomous ) {
		return;
	}

	//=========================================================
	// マウス操作で駒を置いたり取ったり。
	//=========================================================
	// この駒置き場にマウスが乗っていて、
	if (_isMouseOver) {

		// マウスに駒がくっついている
		if (Application::GetInstance()->IsMouseClingObj_Piece()) {

			// 左押下
			if (Application::GetInstance()->_mouseState_Left == MOUSE::BUTTON::STATE::RELEASE) {

				// マウスから駒を放して、この駒置き場に追加
				IMouseClingableObject *obj = Application::GetInstance()->DrawMouseClingObj();
				Piece *piece = dynamic_cast<Piece*>( obj );

				AddPiece(piece);
			}

		// マウスに空きがある
		} else if (!Application::GetInstance()->IsMouseClingObjExist()){

			// 駒置き場内の駒にマウスが乗っていて、左押下
			if ( _mouseOverPiece != NULL && Application::GetInstance()->_mouseState_Left == MOUSE::BUTTON::STATE::RELEASE) {

				// マウスにくっつけて、その駒は駒置き場から放す
				Application::GetInstance()->AddMouseClingObj( _mouseOverPiece );
				RemovePiece ( _mouseOverPiece );
			}

		}

	}

}

//=========================================================
// マウスオーバー判定
//=========================================================
void PieceBox::MouseOverCheck() {

	int mouseX = Application::GetInstance()->_mouseX;
	int mouseY = Application::GetInstance()->_mouseY;

	sf::Vector2f position = _box.getPosition();

	if
	(
		position.x < mouseX && mouseX < (position.x + _boxSize.x) 
		&&
		position.y < mouseY && mouseY < (position.y + _boxSize.y) 
	) 
	{
		//printf("mouse on\n");

		_isMouseOver = true;

	} else {
		_isMouseOver = false;
	}
}

//=========================================================
// 駒を消す（オブジェクトを指定）
//=========================================================
void PieceBox::RemovePiece( Piece *obj ) {

	for ( vector<Piece*>::iterator it = _pieceList.begin(); it != _pieceList.end(); ++it) {

		Piece *piece = *(it);

		if (piece == obj) {
			
			_pieceList.erase(it);
			break;
		}
	}
}

//=========================================================
// 描画
//=========================================================
void PieceBox::Draw(sf::RenderWindow &window) {

	window.draw(_box);

	for ( unsigned int i = 0; i < _pieceList.size(); ++i ) {

		_pieceList[i]->DrawActual(window);
	}
}
//=========================================================
// 駒を一つ取り出す
//=========================================================
Piece* PieceBox::TakePiece() {

	Piece *result = NULL;

	if ( _pieceList.size() > 0 ) {

		int lotNum = Utils_Cal::GerRandom ( _pieceList.size() );

		vector<Piece*>::iterator it = _pieceList.begin();
		for ( int i = 0; i < lotNum; ++i ) {
			++it;
		}

		result = *(it);

		_pieceList.erase( it );

	} else {

printf ( "駒台に駒がない\n" );

	}
	return result;
}
//=========================================================
// 駒を一つ取り出す：オブジェクトを指定する
//=========================================================
Piece* PieceBox::TakePiece ( Piece *obj ) {

	Piece *result = NULL;

	for ( vector<Piece*>::iterator it = _pieceList.begin(); it != _pieceList.end(); ++it ) {

		if ( (*it) == obj ) {
			result = obj;
			_pieceList.erase ( it );
			break;
		}
	}

	return result;
}

//==========================================================
// ファイル出力用に文字列を生成
//==========================================================
string PieceBox::Output() {

	string result = "";

	//vector<Piece*> _pieceList;
	for (unsigned int i = 0; i < _pieceList.size(); ++i) {

		Piece *piece = _pieceList[i];
		if ( piece != NULL ) {

			if (result != "") {
				result += ",";
			}
			if ( piece->GetColor() == Piece::Color::BLACK ) {
				result += "BLACK";
			}
			else if ( piece->GetColor() == Piece::Color::WHITE ) {
				result += "WHITE";
			}
		}
	}

	return result;
}

//==========================================================
// ファイル出力用の文字列から再現
//==========================================================
void PieceBox::Read( string data ) {

	///// 一度解放
	/*
	// 解放は別のところでまとめてすることとする。駒台クラスは駒オブジェクトのメモリ管理をしない。
	for (unsigned int i = 0; i < _pieceList.size(); ++i) {
		delete _pieceList[i];
	}
	*/
	_pieceList.clear();

	vector<string> list = Utils_String::Split( data, "," );
	for (unsigned int i = 0; i < list.size(); ++i) {

		if ( list[i] == "BLACK" ) {

			Piece *piece = new Piece( Piece::Color::BLACK );
			_pieceList.push_back( piece );
		}
		else if ( list[i] == "WHITE" ) {

			Piece *piece = new Piece( Piece::Color::WHITE );
			_pieceList.push_back( piece );
		}

	}

}





