//=========================================================
//
// 気力ゲージ
//
//=========================================================

#include "Application.h"
#include "Define.h"
#include "PsychoGage.h"


//=========================================================
// コンストラクタ
//=========================================================
PsychoGage::PsychoGage() {

	_currentValue = 0;
	_maxValue     = 100;
	_minValue     = 0;

	_isPressButton = false;

	_position.x = 400;
	_position.y = 300;

	sf::Vector2f size = sf::Vector2f ( 20, 200 );

	_gageColor_level0 = sf::Color ( 255,255,255,200 );
	_gageColor_level1 = sf::Color (  50, 50,255,200 );
	_gageColor_level2 = sf::Color ( 250,250, 50,200 );
	_gageColor_level3 = sf::Color ( 255,  0,  0,200 );
	_gageColor_level3_blink= sf::Color ( 0, 0,0, 0 );
	_blink = false;

	_rect_Frame = sf::RectangleShape( size );
	_rect_Frame.setOutlineThickness ( 1 );
	_rect_Frame.setOutlineColor ( sf::Color ( 255,0,0,255 ) );
	_rect_Frame.setFillColor ( sf::Color ( 0,0,0,0 ) );

	_rect_Base = sf::RectangleShape( size );
	_rect_Base.setOutlineThickness ( 1 );
	_rect_Base.setOutlineColor ( sf::Color ( 0,0,0,0 ) );
	_rect_Base.setFillColor ( sf::Color ( 0,0,0,100 ) );

	_rect_Gage = sf::RectangleShape( size );
	_rect_Gage.setOutlineThickness ( 0 );
	_rect_Gage.setOutlineColor ( sf::Color ( 0,0,0,0 ) );
	_rect_Gage.setFillColor ( _gageColor_level0 );

}

//=========================================================
// デストラクタ
//=========================================================
PsychoGage::~PsychoGage() {

}

//=========================================================
// 更新
//=========================================================
void PsychoGage::Update ( sf::RenderWindow &window ) {

	///// マウスを押している
	//if ( Application::GetInstance()->_mouseState_Left == MOUSE::BUTTON::STATE::PRESS ) {
	if ( _isPressButton ) {

		++_currentValue;
		if ( _currentValue >= _maxValue ){
			_currentValue = _maxValue;
		}

	///// マウスを押していない
	} else {

		--_currentValue;
		if ( _currentValue <= _minValue ){
			_currentValue = _minValue;
		}

	}

	///// 押し続けなさい（高飛車に）
	_isPressButton = false;


	///// ゲージの長さを求める
	int gageLength = (float)_rect_Frame.getSize().y * ( (float)_currentValue / (float)_maxValue );

	_rect_Gage.setSize ( sf::Vector2f (
		_rect_Frame.getSize().x, 
		gageLength
	));

	///// 位置をセット
	_rect_Frame.setPosition ( _position );
	_rect_Base.setPosition ( _position );
	_rect_Gage.setPosition ( _position + sf::Vector2f ( 0, _rect_Frame.getSize().y - _rect_Gage.getSize().y  ) );


	///// ゲージの色をセット
	if      ( _currentValue == _maxValue       ) { 

		if ( _blink ) {
			_rect_Gage.setFillColor ( _gageColor_level3_blink );
		} else {
			_rect_Gage.setFillColor ( _gageColor_level3 );
		}
		_blink = !_blink;
	}
	else if ( _currentValue >  _maxValue * 2/3 ) { _rect_Gage.setFillColor ( _gageColor_level2 );}
	else if ( _currentValue >  _maxValue * 1/3 ) { _rect_Gage.setFillColor ( _gageColor_level1 );}
	else                                         { _rect_Gage.setFillColor ( _gageColor_level0 );}
}

//=========================================================
// 描画
//=========================================================
void PsychoGage::Draw ( sf::RenderWindow &window ) {

	window.draw ( _rect_Base );
	window.draw ( _rect_Gage );
	window.draw ( _rect_Frame );
}

//=========================================================
// エフェクトレベルとして値を取得
//=========================================================
int PsychoGage::GetValueAsEffectLevel() {

	/*
		つまり 0〜3 に換算ということ
	*/

	int result = 0;

	if      ( _currentValue <  _maxValue * 1/3 ) { result = 0; }
	else if ( _currentValue <  _maxValue * 2/3 ) { result = 1; }
	else if ( _currentValue <  _maxValue       ) { result = 2; }
	else if ( _currentValue == _maxValue       ) { result = 3; }

	return result;
}





