//=========================================================
//
// 棋譜ビューワー
//
//=========================================================

#include "ScoreSheetViewer.h"
#include "../GUI_SFML/SFML_Utils.h"
#include "Application.h"
#include "../Utils/Utils_String.h"
#include "../GUI_SFML/TextImageFactory.h"

using namespace GUI_SFML;


//=========================================================
// ビューワーの１行
//=========================================================
ScoreSheetCell::ScoreSheetCell( string moveNotation, const sf::Font &font ,
					sf::Sprite *backGroundImage,
					sf::Sprite *focusImage, 
					sf::Sprite *mouseoverImage,
					int index
)
{
	
	_moveNotation_text = moveNotation;
	
	// [何手目]の文言を追加
	string fullDisplayString = "";
	if ( index == 0 ) { 
		// 開始局面
		fullDisplayString = moveNotation; 
	} else if ( index == -1 ) {
		// 終局の文言
		fullDisplayString = " " + moveNotation; 
	} else {
		fullDisplayString = "[" + to_string(index) + "] " + moveNotation; 
	}
	
	_moveNotation_image.setFont ( font );
	_moveNotation_image.setString ( SFML_Utils::GetSfString(fullDisplayString));
	
	_moveNotation_image.setCharacterSize ( 12 );
	_moveNotation_image.setColor ( sf::Color::Black );
	
	_moveNotation_image.setOrigin ( _moveNotation_image.getOrigin() + sf::Vector2f(-2.0f,-2.0f) );
	
	_backGroundImage = backGroundImage;
	_focusImage      = focusImage;
	_mouseoverImage  = mouseoverImage;
	
	//printf ("棋譜ビューワーに追加：%s\n", _moveNotation_text.c_str());
	
}
ScoreSheetCell::~ScoreSheetCell() {

	//printf ("棋譜ビューワーから削除：%s\n", _moveNotation_text.c_str());
}

//=========================================================
// ビューワーの１行：更新
//=========================================================
void ScoreSheetCell::Update(sf::RenderWindow &window) {
	//printf("sheetcell[%s]", _moveNotation_text.c_str());
}
//=========================================================
// ビューワーの１行：描画
//=========================================================
void ScoreSheetCell::Draw  (sf::RenderWindow &window) {
	
	// フォーカス用の背景
	if ( _isFocus) {
		
		if (_focusImage != NULL) {
			_focusImage->setPosition(_position);
			window.draw((*_focusImage));
		}
		
	// 通常の背景
	} else {
		
		if ( IsMouseOver()  ) {

            if ( _mouseoverImage != NULL) {
                _mouseoverImage->setPosition(_position);
                window.draw((*_mouseoverImage));
            }
		
		} else {
		
            if ( _backGroundImage != NULL ) {
                _backGroundImage->setPosition(_position);
                window.draw((*_backGroundImage));
            }
		}
	}
	
	// 中身
	window.draw(_moveNotation_image);
	
}


//=========================================================
// ビューワーの１行：マウスオーバー判定
//=========================================================
bool ScoreSheetCell::IsMouseOver() {
	
	//sf::Vector2i mousePos = sf::Mouse::getPosition ( Application::GetInstance()->_window );
	
	sf::Vector2i mousePos;
	mousePos.x = Application::GetInstance()->_mouseX;
	mousePos.y = Application::GetInstance()->_mouseY;
	
	/*
	int boundsX = _moveNotation_image.getGlobalBounds().left;
	int boundsY = _moveNotation_image.getGlobalBounds().top;
	int boundsW = _moveNotation_image.getGlobalBounds().width;
	int boundsH = _moveNotation_image.getGlobalBounds().height;
	*/
	int boundsX = _position.x;
	int boundsY = _position.y;
	int boundsW = _backGroundImage->getGlobalBounds().width;
	int boundsH = _backGroundImage->getGlobalBounds().height;
	
	
	bool result =
	(
		boundsX < mousePos.x && mousePos.x < boundsX + boundsW 
		&&
		boundsY < mousePos.y && mousePos.y < boundsY + boundsH
	);
	
	return result;
}

//=========================================================
// ビューワーの１行：その他
//=========================================================
sf::Vector2f ScoreSheetCell::GetPosition() {
	
	return _position;
}

void ScoreSheetCell::SetPosition( sf::Vector2f pos) {
	
	_position = pos;
	
	_moveNotation_image.setPosition ( _position );
}

sf::Vector2f ScoreSheetCell::GetSize() {
	
	sf::Vector2f result;
	result.x = _moveNotation_image.getGlobalBounds().width;
	result.y = _moveNotation_image.getGlobalBounds().height;
	
	return result;
}



//=========================================================
// コンストラクタ
//=========================================================
ScoreSheetViewer::ScoreSheetViewer() {
	
	
	//_font.loadFromFile ( "/home/kifc000/Documents/フォント/umefont_530/ume-tgc4.ttf" );
	_font = TextImageFactory::_font;
	
	// 棋譜ビューワー本体の背景
	_backGroundTexture.loadFromFile ( "./image/gameBoard/kifuSelectboxBack.png" );
	_backGroundSprite.setTexture ( _backGroundTexture );
	
	// ナビゲーションパネルの背景
	_guideButtonsBackGroundTexture.loadFromFile ( "./image/gameBoard/scoreSheetNavigationPanelBack.png" );
	_guideButtonsBackGroundSprite.setTexture ( _guideButtonsBackGroundTexture );
	_guideButtonsBackGroundSprite.setPosition ( sf::Vector2f(20.0f, 410.0f) );
	
	_position.x = 0.0f;
	_position.y = 0.0f;
	
	_backGroundSprite.setPosition ( _position );
	
	_currentIndex = -1;
	_isIndexChanged = false;
	_isNextPushed   = false;
	
	_selectBox = new SelectBox();
	
	_selectBox->SetDisplayCount    ( 8 );
	_selectBox->SetFrameSize       ( sf::Vector2f ( 260.0f, 170.0f ) );//sf::Vector2f ( 180.0f, 170.0f ) );
	_selectBox->SetPosition_Offset ( sf::Vector2f ( 5.0f, 5.0f ) );//sf::Vector2f ( 110.0f, 10.0f ) );
	_selectBox->SetLengthOfOneItem ( 20.0f );
	_selectBox->SetDefaultCursorOn ( false );
	
	_guideButtonsOn = false;
	
	_scoreSheetCellList.clear();
	
	///// セルの背景
	_cellBackGroundTexture.loadFromFile("./image/gameBoard/scoreSheetCellBack.png");
	_cellFocusTexture.     loadFromFile("./image/gameBoard/scoreSheetCellFocus.png");
	_cellMouseOverTexture. loadFromFile("./image/gameBoard/scoreSheetCellMouseOver.png");
	
	_cellBackGroundImage = new sf::Sprite();
	_cellBackGroundImage->setTexture(_cellBackGroundTexture);
	
	_cellFocusImage = new sf::Sprite();
	_cellFocusImage->setTexture(_cellFocusTexture);
	
	_cellMouseOverImage = new sf::Sprite();
	_cellMouseOverImage->setTexture(_cellMouseOverTexture);

		
	
	///// ナビゲーションパネルのボタン群：「検討を終了」ボタンは ScriptEngine の持ち物です。ここでは関知せず。
	{
		ClickableImage::Property prop;
		prop.imageFile = "./image/gameBoard/kifuPlaybackButtons.png";
		prop.rect_Normal = sf::IntRect(0, 0, 50, 30);
		prop.rect_Over   = sf::IntRect(0, 30, 50, 30);
		prop.rect_Push   = sf::IntRect(0, 60, 50, 30);
		prop.size        = sf::Vector2f(50, 30);

		_guideButton_toFirst = new ClickableImage(prop);
	}
	{
		ClickableImage::Property prop;
		prop.imageFile = "./image/gameBoard/kifuPlaybackButtons.png";
		prop.rect_Normal = sf::IntRect(50, 0, 50, 30);
		prop.rect_Over   = sf::IntRect(50, 30, 50, 30);
		prop.rect_Push   = sf::IntRect(50, 60, 50, 30);
		prop.size        = sf::Vector2f(50, 30);

		_guideButton_toPrev = new ClickableImage(prop);
	}
	{
		ClickableImage::Property prop;
		prop.imageFile = "./image/gameBoard/kifuPlaybackButtons.png";
		prop.rect_Normal = sf::IntRect(100, 0, 50, 30);
		prop.rect_Over   = sf::IntRect(100, 30, 50, 30);
		prop.rect_Push   = sf::IntRect(100, 60, 50, 30);
		prop.size        = sf::Vector2f(50, 30);
		
		_guideButton_toNext = new ClickableImage(prop);
	}
	{
		ClickableImage::Property prop;
		prop.imageFile = "./image/gameBoard/kifuPlaybackButtons.png";
		prop.rect_Normal = sf::IntRect(150, 0, 50, 30);
		prop.rect_Over   = sf::IntRect(150, 30, 50, 30);
		prop.rect_Push   = sf::IntRect(150, 60, 50, 30);
		prop.size        = sf::Vector2f(50, 30);

		_guideButton_toLast = new ClickableImage(prop);
	}
	

}

//=========================================================
// 位置指定
//=========================================================
void ScoreSheetViewer::SetPosition ( sf::Vector2f pos ) {
	
	_position = pos;
	_backGroundSprite.setPosition ( _position );
}

//=========================================================
// デストラクタ
//=========================================================
ScoreSheetViewer::~ScoreSheetViewer() {

	while ( !_scoreSheetCellList.empty() ) {
	
		delete (*_scoreSheetCellList.begin());
		_scoreSheetCellList.erase(_scoreSheetCellList.begin());
	}
	
	delete _guideButton_toFirst;
	delete _guideButton_toPrev;
	delete _guideButton_toNext;
	delete _guideButton_toLast;
	
	delete _selectBox;
	
	delete _cellBackGroundImage;
	delete _cellFocusImage;
	delete _cellMouseOverImage;

}

//=========================================================
// 更新
//=========================================================
void ScoreSheetViewer::Update (sf::RenderWindow &window) {
	
	
	_selectBox->SetPosition ( _position );
	_backGroundSprite.setPosition ( _position );
	
	_selectBox->Update(window);
	
	// マウスのホイールでリストをスクロールさせたい
	bool isMouseOver = _selectBox->IsMouseOver();
	if ( isMouseOver ) {
		
		int mouseWheel = Application::GetInstance()->_mouseWheel;
		
		if ( mouseWheel > 0 ) {
			//printf ("ホイール回転：上\n");
			_selectBox->ScrollUp();
		}
		if ( mouseWheel < 0 ) {
			//printf ("ホイール回転：下\n");
			_selectBox->ScrollDown();
		}
		
	}
	
	_isIndexChanged = false;
	_isNextPushed   = false;

	
	// クリックしていたら、インデックスをその位置に移動させる
	int clickIndex = GetClickedIndex();
	if ( clickIndex != -1 ) {
		_currentIndex = clickIndex;
		
		_isIndexChanged = true;
	}
	
	///// フォーカスを更新
	for ( unsigned int i = 0; i < _scoreSheetCellList.size(); ++i ) {
		
		ScoreSheetCell *cell = _scoreSheetCellList[i];
		
		if ( i == _currentIndex ) {
			cell->Focus(true);
		} else {
			cell->Focus(false);
		}
	}
	
	///// ガイドボタン関連
	if ( _guideButtonsOn ) {
		
		sf::Vector2f guideButtonsOffset(-218, 0);
	
		_guideButton_toFirst -> SetPosition ( _position + guideButtonsOffset + sf::Vector2f ( 0, 0 ) );
		_guideButton_toPrev  -> SetPosition ( _position + guideButtonsOffset + sf::Vector2f ( 52, 0 ) );
		_guideButton_toNext  -> SetPosition ( _position + guideButtonsOffset + sf::Vector2f ( 104, 0 ) );
		_guideButton_toLast  -> SetPosition ( _position + guideButtonsOffset + sf::Vector2f ( 156, 0 ) );
	
		_guideButton_toFirst->Update(window);
		_guideButton_toPrev->Update(window);
		_guideButton_toNext->Update(window);
		_guideButton_toLast->Update(window);
		
		
		GuideButtonWatching();
	}
	
}

//=========================================================
// ガイドボタン押下判定
//=========================================================
void ScoreSheetViewer::GuideButtonWatching() {
	
	// 非表示のときは処理なし
	if ( !_guideButtonsOn ) {
		return;
	}
	
	// リストが空のときも処理なし
	if ( _scoreSheetCellList.size() == 0 ) {
		return;
	}
	
	// 開始位置へ
	if (_guideButton_toFirst->IsClick() ) {
		
		_currentIndex = 0;
		_isIndexChanged = true;
		
		_selectBox->MoveToTop();

	}
	// 次へ
	else if (_guideButton_toNext ->IsClick() ) {
		
		_currentIndex++;
		
		if ( _currentIndex >= _scoreSheetCellList.size() ) {
			_currentIndex = _scoreSheetCellList.size() - 1;
		} else {
			
			// 「次へ」を押したときには指し手をなめらかに再生したいじゃないですか。
			_isNextPushed = true;
		}
		
		_selectBox->Makeup_OutofDownnerBounds(_currentIndex);
		
		_isIndexChanged = true;
		
	}
	// 前へ
	else if (_guideButton_toPrev ->IsClick() ) {
		
		_currentIndex--;
		
		if ( _currentIndex < 0 ) {
			_currentIndex = 0;
		}
		
		_selectBox->Makeup_OutofUpperBounds(_currentIndex);
		
		_isIndexChanged = true;
	}
	// 最後へ
	else if (_guideButton_toLast ->IsClick() ) {
		
		_currentIndex = _scoreSheetCellList.size() - 1;
		
		_selectBox->MoveToBottom();
		
		_isIndexChanged = true;
	}
	
}
//=========================================================
// クリックした項目のインデックスを取得
//=========================================================
int ScoreSheetViewer::GetClickedIndex() {
	
	int result = -1;
	
	ISelectBoxItem *obj = _selectBox->GetClickItem();
	
	if ( obj != NULL ) {
		
		result = _selectBox->GetPointingItemIndex();
		
		printf("クリックした項目のインデックス : %d\n", result);
	}
		
	return result;
}


//=========================================================
// 描画
//=========================================================
void ScoreSheetViewer::Draw   (sf::RenderWindow &window) {
	
	
	if ( _guideButtonsOn ) {
			
		window.draw(_guideButtonsBackGroundSprite);
		_guideButton_toFirst->Draw(window);
		_guideButton_toPrev->Draw(window);
		_guideButton_toNext->Draw(window);
		_guideButton_toLast->Draw(window);
		
	}	

	window.draw(_backGroundSprite);

	
	_selectBox->Draw(window);

}

//=========================================================
// 一件追加
//=========================================================
void ScoreSheetViewer::Add   ( string moveNotation, bool isEndDescription ) {
    
    ///////
    // 通し番号はこちらでつける、と、セーブ/ロードのたびに手数表示が重なっていくので、こちらではつけない
    // 
    // 最初のは「開始局面」のハズなので「0」とする。
    // （非表示）
    //
    //
    /*
    string turnIndexLabel = "";
    int turnIndex = _scoreSheetCellList.size();
    if ( turnIndex != 0 ) {
        turnIndexLabel = "[" + to_string ( turnIndex ) + "] ";

    ///// 「開始局面」をもうちょっと右寄せにしたい。もう適当。
    } else {
        turnIndexLabel = " ";
    }
    
    moveNotation = turnIndexLabel + Utils_String::Trim(moveNotation);
    */
    int turnIndex = _scoreSheetCellList.size();
    if ( isEndDescription ) { turnIndex = -1; } 
    
    moveNotation  = Utils_String::Trim(moveNotation);
    	
	ScoreSheetCell *cell;
	cell = new ScoreSheetCell (moveNotation, _font , _cellBackGroundImage, _cellFocusImage, _cellMouseOverImage, turnIndex );
	
	_scoreSheetCellList.push_back(cell);
	
	_selectBox->AddItem(cell);
	
	
	// 一番後ろのインデックスを指すようにする。
	_currentIndex = _scoreSheetCellList.size() - 1;
}

//=========================================================
// クリア
//=========================================================
void ScoreSheetViewer::Clear () {
	
	while ( !_scoreSheetCellList.empty() ) {
		
		ScoreSheetCell *cell = (*_scoreSheetCellList.begin());
		
		_scoreSheetCellList.erase(_scoreSheetCellList.begin());
		
		_selectBox->RemoveItem(cell);
		
		delete cell;

	}
	
	_currentIndex = -1;
}

//=========================================================
// 最下段に移動
//=========================================================
void ScoreSheetViewer::MoveToBottom() {
	
	_currentIndex = _scoreSheetCellList.size() - 1;
	_selectBox->MoveToBottom();
}


//=========================================================
// 最上段に移動
//=========================================================
void ScoreSheetViewer::MoveToTop() {
	
	_selectBox->MoveToTop();
	
	// インデックスもゼロに。
	_currentIndex = 0;
	
	// と見せかけて、中身が空だったらマズイ。
	if ( _scoreSheetCellList.size() == 0 ) {
		_currentIndex = -1;
	}
}

//=========================================================
// 現在のインデックスを設定
//=========================================================
void ScoreSheetViewer::SetCurrentIndex(int index) {
	
	int before = _currentIndex;
	
	// 仰せのままに
	_currentIndex = index;
	
	// と見せかけて、中身が空だったらマズイ。
	if ( _scoreSheetCellList.size() == 0 ) {
		_currentIndex = -1;
	}
	
	// さらに、中身がリストを超過していたらマズイ
	if ( _currentIndex >= _scoreSheetCellList.size() ) {
		_currentIndex = _scoreSheetCellList.size() -1;
	}
	
	// 変化した？
	if (before != _currentIndex) {
		_isIndexChanged = true;
	}
}

//=========================================================
// シリアライズ
//=========================================================
string ScoreSheetViewer::Serialize() {
	
	string result = "";

/*
[セーブロード対象]

	sf::Vector2f _position;
	
	// 棋譜の現在位置
	int  _currentIndex;
	bool _isIndexChanged;
	bool _isNextPushed;
	
	bool _guideButtonsOn;

	vector<ScoreSheetCell*> _scoreSheetCellList; 
		==> 中の string さえあれば再現できる。というわけで string のリストとしてシリアライズする

[非対象]
	sf::Texture _backGroundTexture;
	sf::Sprite  _backGroundSprite;
	SelectBox *_selectBox;
	sf::Font _font;
		
		==> デザインはシステム固定ということにしてしまう。
*/
	string variableDelim  = "<@-@>";
	string interCellDelim = "*";
	
	for ( unsigned int i = 0; i < _scoreSheetCellList.size(); ++i ) {
		
		if ( i > 0 ) { result += interCellDelim; }
		
		ScoreSheetCell *cell = _scoreSheetCellList[i];
		result += cell->GetText();
	}
	result += variableDelim;
	
	result += std::to_string ( (int)_position.x ) + variableDelim;
	result += std::to_string ( (int)_position.y ) + variableDelim;
	result += std::to_string ( _currentIndex    ) + variableDelim;
	result += ( _isIndexChanged ) ? "true" : "false" + variableDelim;
	result += ( _isNextPushed   ) ? "true" : "false" + variableDelim;
	result += ( _guideButtonsOn ) ? "true" : "false" + variableDelim;
	
	
printf( "ScoreSheetViewer::Serialize()したよ！ ==> %s\n", result.c_str() );
	
	return result;
}


//=========================================================
// デシリアライズ
//=========================================================
void ScoreSheetViewer::DeSerialize( string data ) {
	
	string variableDelim  = "<@-@>";
	string interCellDelim = "*";
	
	vector<string> dataList = Utils_String::Split ( data, variableDelim );
	
	if ( dataList.size() < 7 ) {
		printf ( "セーブデータが壊れています：ScoreSheetViewer::DeSerialize( %s )\n", data.c_str() );
	}
	
	Clear();
	
	vector<string> cellDataList = Utils_String::Split ( dataList[0], interCellDelim);
	for ( unsigned int i = 0; i < cellDataList.size(); ++i ) {
		
		Add ( cellDataList[i], ( i == cellDataList.size()-1  ) );
	}
	
	_position.x   = std::stoi ( dataList[1] );
	_position.y   = std::stoi ( dataList[2] );
	_currentIndex = std::stoi ( dataList[3] );
	
	_isIndexChanged = Utils_String::isPositiveWord ( dataList[4] );
	_isNextPushed   = Utils_String::isPositiveWord ( dataList[5] );
	_guideButtonsOn = Utils_String::isPositiveWord ( dataList[6] );
	
	SetCurrentIndex(_currentIndex);
	SetPosition(_position);
	
}













