//=========================================================
//
// Utils_Cal (implement)
//
//=========================================================

#include "Utils_Cal.h"
#include<math.h>
#include <stdarg.h>
#include "mt19937ar.h"
#include <time.h>

//=========================================================
// From -> To with the distance
//=========================================================
//sf::Vector2f Utils_Cal::MoveTo ( sf::Vector2f from, sf::Vector2f to, float distance ) {
pair<float, float> Utils_Cal::MoveTo ( float from_x, float from_y, float to_x, float to_y, float distance ) {

	//sf::Vector2f result = from;
	pair<float, float> result( from_x, from_y );


	float d =  sqrt( pow(to_x - from_x, 2) + pow(to_y - from_y, 2) );

	float rate = ( d != 0.0f ) ? (distance / d) : 0.0f;

	result.first  += (to_x - from_x) * rate;
	result.second += (to_y - from_y) * rate;
	

	// Exceed
	if (from_x < to_x) {
		if ( to_x < result.first) {
			result.first  = to_x;
			result.second = to_y;
		}
	}
	if (from_x > to_x) {
		if ( to_x > result.first) {
			result.first  = to_x;
			result.second = to_y;
		}
	}
	if (from_y < to_y) {
		if ( to_y < result.second) {
			result.first  = to_x;
			result.second = to_y;
		}
	}
	if (from_y > to_y) {
		if ( to_y > result.second) {
			result.first  = to_x;
			result.second = to_y;
		}
	}
	

	return result;
}

//=========================================================
// Get Distance
//=========================================================
//float Utils_Cal::GetDistance ( sf::Vector2f from, sf::Vector2f to ) {
float Utils_Cal::GetDistance ( float from_x, float from_y,  float to_x, float to_y) {

	float result = 0.0f;

	result =  sqrt( pow(to_x - from_x, 2) + pow(to_y - from_y, 2) );
 
	return result;

}

//=========================================================
// Get Angle
//=========================================================
float Utils_Cal::GetAngleOfLineBetweenTwoPoints(float p1X, float p1Y, float p2X, float p2Y) {

	float PI = 3.14159265f;
	float xDiff = p2X - p1X;
	float yDiff = p2Y - p1Y;
	return atan2(yDiff, xDiff) * (180 / PI);
}
	
//=========================================================
// Nearlize
//=========================================================
float Utils_Cal::Nearlize( float from, float to, float distance ) {

	float result = from;

	if ( from > to ) {

		result -= distance;

		if (result < to) {
			result = to;
		}
	}
	else if ( result < to ) {

		result += distance;

		if (result > to) {
			result = to;
		}
	} 

	return result;
}

//=========================================================
// 符号を求める
//=========================================================
int Utils_Cal::Sign( int a ) {

	int result = 0;

	if (a < 0) {
		result = -1;
	}
	else if (a > 0) {
		result = 1;
	}

	return result;

}

//=========================================================
// int用のabs
//=========================================================
unsigned int Utils_Cal::Abs( int a, int b ) {

	unsigned int result = 0;

	if ( a > b ) {
		result = a - b;
	}
	else if ( b > a ) {
		result = b - a;
	}

	return result;
}
unsigned int Utils_Cal::Abs( int a ) {

	unsigned int result = 0;

	if ( a > 0 ) {
		result = a;
	}
	else if ( a < 0 ) {
		result = -a;
	}

	return result;
}

//=========================================================
// int用のpow
// a^b
//=========================================================
int Utils_Cal::Pow(int a, int b) {

	int result = 1;

	for (int i = 0; i < b; ++i) {

		result *= a;
	}

	return result;
}

//=========================================================
// 乱数
//=========================================================
int Utils_Cal::GetRandom( int max ) {

	/*
		max は含まない
	*/

	///// 1以上にしてね。
	if (max <= 0) {
		return 0;
	}
	
	init_genrand( static_cast<unsigned int>( time(NULL) ) );
	
	//int result = rand()%max;
	int result = genrand_int31() % max;

	return result;
}
int Utils_Cal::GerRandom( int max ) {
	return GetRandom( max );
}

//==========================================================
// コイントス
// lot / all でくじ引きをして、あたりが出たら true
//==========================================================
bool Utils_Cal::MultipleCoinToss ( int lot, int all ) {

	bool result = false;

	result = GerRandom ( all ) < lot;

	return result;
}

//=========================================================
// Permutation
//【引数】
// left Ｐ right
//=========================================================
int Utils_Cal::Permutation ( int left, int right ) {

	int result = 1;

	for (int i = 0; i < right; ++i) {

		result *= (left - i);

	}

	return result;
}

//=========================================================
// 可変長引数で、最大値チェック
//【引数】
// argNum 引数の個数「この argNum 個の数の中で」
//【結果】
// 最初の引数が最大値だったら、true
//=========================================================
bool Utils_Cal::IsMax ( int argNum, ... ) {

	bool result = true;

	va_list args;

	va_start(args, argNum);

	// 最初の引数が基準
	int firstArg;

	for (int i = 0; i < argNum; ++i) {

		// 引数を順番に受け取る
		int iVal = va_arg(args, int);

		// 最初の引数は比較対象
		if ( i == 0 ) {
			firstArg = iVal;

		// ２番目以降の引数を、最初の引数を比較する
		} else {

			if ( iVal > firstArg ) {
				result = false;
				break;
			}
		}
	}

	va_end(args);

	return result;
}


//=========================================================
// 重複なしで乱数
//=========================================================
void Utils_Cal::GetRandom_WithoutDuplicate ( set<int> &result, unsigned int n, int min, int max ) {

	// 結果用変数をクリアしてしまいますよ？
	result.clear();

	// 指定の個数になるまで繰り返す。
	while ( result.size() < n ) {

		// くじびきして
		int lot = Utils_Cal::GerRandom( max-min ) + min;

		// 未出の場合のみ、結果に加える。
		if ( result.find( lot ) ==  result.end()) {

			result.insert ( lot );
		}
	}
}

//==========================================================
// 間に入っている
// 真上を含むかどうか true で含む
//==========================================================
bool Utils_Cal::IsBetween( int thisValue, int a, int b, bool isOverIncluded ) {

	bool result = false;

	int bigger  = ( a < b ) ? b : a;
	int smaller = ( a < b ) ? a : b;

	if ( smaller < thisValue && thisValue < bigger ) {

		result = true;

	} else {

		if (isOverIncluded) {

			if ( thisValue == a || thisValue == b ) {
				result = true;
			}
		}
	}

	return result;
}
//=========================================================
// 無理やりな四捨五入というか整数値に丸める
//=========================================================
void Utils_Cal::RoundWildly ( float & value ) {

	int a = value;
	value = a;
}

