//=========================================================
//
// ファイル関係のサブルーチン
//
//=========================================================

#include "Utils_File.h"
#include "Utils_String.h"
#include <fstream>
#include <iostream>


#if defined(_WIN32)


//=========================================================
// 指定ディレクトリ以下のファイル名一覧を取得 (for Windows)
//=========================================================
void Utils_File::GetFileList ( string dir, vector<string> &result, vector<string> path ) {

	char currentDir[512];
	GetCurrentDirectory ( 512, currentDir );

	SetCurrentDirectory ( dir.c_str() );

	// パス履歴に追加。空文字＝カレントディレクトリは、入れない。
	if ( dir != "" ) {
		path.push_back( dir );
	}

	WIN32_FIND_DATA fd;
	HANDLE hSearch = FindFirstFile( "*.*", &fd );

	if( hSearch == INVALID_HANDLE_VALUE ) {
		printf ("Error ： Utils_File::GetFileList ( %s ) にて\n", dir.c_str());
	}

	while (true) {

		///// ファイルか、ディレクトリか？
		if ( fd.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY) {
			
			//printf ("フォルダ名：%s\n", fd.cFileName);

			string dirName = fd.cFileName;

			if ( dirName != "." && dirName != "..") {
				GetFileList ( dirName, result, path );
			}

		} else {

			//printf ("ファイル名：%s\n", fd.cFileName);

			string fileName = "";
			for ( unsigned int i = 0; i < path.size(); ++i ) {
				
				//if ( i > 0 ) { fileName += "/";	}
				fileName += path[i];
				fileName += "/";
			}

			fileName += fd.cFileName;

			result.push_back ( fileName );

		}

		if( !FindNextFile( hSearch, &fd ) )
		{
			if( GetLastError() == ERROR_NO_MORE_FILES )
			{
				//printf ("列挙終了\n");
				break;
			}
			else
			{
				printf ("Error ： Utils_File::GetFileList ( %s ) にて\n", dir.c_str());
				break;
			}
		}
	}

	FindClose( hSearch );


	// カレントディレクトリを元に戻す
	SetCurrentDirectory ( currentDir );

	// パス履歴から、はずしておく
	if ( dir != "" ) { // ← つまり、履歴に入れなかったなら、pop_back もしない、という意味。
		path.pop_back();
	}
}


//=========================================================
// 指定ディレクトリ以下のファイル名一覧を取得・フォルダ構造を保存しつつ (for Windows)
//=========================================================
void Utils_File::GetFileList ( string dir, FolderEntity &result, vector<string> path ) {

    //printf ( "ここを読む : %s\n", dir.c_str() );
    
	char currentDir[512];
	GetCurrentDirectory ( 512, currentDir );

	SetCurrentDirectory ( dir.c_str() );

	// パス履歴に追加。空文字＝カレントディレクトリは、入れない。
	if ( dir != "" ) {
		path.push_back( dir );
	}

	WIN32_FIND_DATA fd;
	HANDLE hSearch = FindFirstFile( "*.*", &fd );

	if( hSearch == INVALID_HANDLE_VALUE ) {
		printf ("Error ： Utils_File::GetFileList ( %s ) にて\n", dir.c_str());
	}

	while (true) {

		///// ファイルか、ディレクトリか？
		
		///// ディレクトリの場合
		if ( fd.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY) {
			
			//printf ("フォルダ名：%s\n", fd.cFileName);

			string dirName = fd.cFileName;

			if ( dirName != "." && dirName != "..") {
			    
			    
                string pathName = dir + "/" + dirName;
			    
				//GetFileList ( dirName, result, path );

                FolderEntity entity;
                entity.name = dirName;
                entity.path = dir + "/";
                entity.isFolder = true;
                
                //entity.isRoot = false;
                //entity.parent = result;
                
                

                //printf("L 潜る %s\n", pathName.c_str());
                GetFileList ( dirName, entity, path );
                
                result.childList.push_back(entity);				
				
			}
        
        ///// ファイルの場合
		} else {

			//printf ("ファイル名：%s\n", fd.cFileName);

			string pathName = "";
			for ( unsigned int i = 0; i < path.size(); ++i ) {
				
				//if ( i > 0 ) { fileName += "/";	}
				pathName += path[i];
				pathName += "/";
			}

			string fileName = fd.cFileName;

			//result.push_back ( fileName );

            FolderEntity entity;
            entity.name = fileName;
            entity.path = pathName; // + "/";
            entity.isFolder = false;
            
            result.childList.push_back(entity);
		}

		if( !FindNextFile( hSearch, &fd ) )
		{
			if( GetLastError() == ERROR_NO_MORE_FILES )
			{
				//printf ("列挙終了\n");
				break;
			}
			else
			{
				printf ("Error ： Utils_File::GetFileList ( %s ) にて\n", dir.c_str());
				break;
			}
		}
	}

	FindClose( hSearch );


	// カレントディレクトリを元に戻す
	SetCurrentDirectory ( currentDir );

	// パス履歴から、はずしておく
	if ( dir != "" ) { // ← つまり、履歴に入れなかったなら、pop_back もしない、という意味。
		path.pop_back();
	}
}



#elif defined(__linux__)

//=========================================================
// 指定ディレクトリ以下のファイル名一覧を取得 (for Linux)
//=========================================================
void Utils_File::GetFileList ( string dirName, vector<string> &result, set<int> visitedDirList ) {
	
	DIR *dir;
	struct dirent *dp;
	struct stat statInfo;
	
	dir = opendir ( dirName.c_str() );
	
	if ( dir == NULL ) {
		printf ( "開けず ( %s ) \n", dirName.c_str() );
		closedir(dir);
		return;
	}
	
	stat( dirName.c_str(), &statInfo );
	visitedDirList.insert ( statInfo.st_ino );
	
	
	for( dp=readdir(dir); dp != NULL; dp=readdir(dir) )
	{
		std::string name = dp->d_name;
		
		if ( name != ".." && name != ".")
		{
			
			//for (int i = 0; i < depth; ++i) { printf("    "); }
			//printf("%s\n",dp->d_name);
			
			string pathName = dirName + "/" + name;
			
			
			// ファイルの情報を取得
			if ( stat( pathName.c_str(), &statInfo ) ) {
				printf( "Failed to get stat %s \n", pathName.c_str() );
				break;
			}
			
			// 潜ってみる？
			if ( S_ISDIR( statInfo.st_mode ) ) {
				
				if ( visitedDirList.count(statInfo.st_ino) == 0 ) {
				
					//printf("フォルダですね\n");
					GetFileList ( pathName, result, visitedDirList );
				} else {
					//printf ("フォルダだけれども既訪\n");
				}
			
			// ファイルですね。
			} else {
				//printf("フォルダじゃないですね : %s\n", d.c_str());
				
				result.push_back ( pathName );
			}
		}
	}

	closedir(dir);	
}


//=========================================================
// 指定ディレクトリ以下のファイル名一覧を取得・フォルダ構造を保存しつつ (for Linux)
//=========================================================
void Utils_File::GetFileList ( string dirName, FolderEntity &result, set<int> visitedDirList ) {
	
	DIR *dir;
	struct dirent *dp;
	struct stat statInfo;
	
	dir = opendir ( dirName.c_str() );
	
	if ( dir == NULL ) {
		printf ( "開けず ( %s ) \n", dirName.c_str() );
		closedir(dir);
		return;
	}
	
	stat( dirName.c_str(), &statInfo );
	visitedDirList.insert ( statInfo.st_ino );
	
	
	
	for( dp=readdir(dir); dp != NULL; dp=readdir(dir) )
	{
		std::string name = dp->d_name;
		
		if ( name != ".." && name != ".")
		{
			
			//for (int i = 0; i < depth; ++i) { printf("    "); }
			//printf("%s\n",dp->d_name);
			
			string pathName = dirName + "/" + name;
			
			// ファイルの情報を取得
			if ( stat( pathName.c_str(), &statInfo ) ) {
				printf( "Failed to get stat %s \n", pathName.c_str() );
				break;
			}
			
			// 潜ってみる？
			if ( S_ISDIR( statInfo.st_mode ) ) {
				
				if ( visitedDirList.count(statInfo.st_ino) == 0 ) {
				
					//printf("フォルダですね\n");
					FolderEntity entity;
					entity.name = name;
					entity.path = dirName + "/";
					entity.isFolder = true;
					
					//entity.isRoot = false;
					//entity.parent = result;

					GetFileList ( pathName, entity, visitedDirList );
					
					result.childList.push_back(entity);
					
				} else {
					//printf ("フォルダだけれども既訪\n");
				}
			
			// ファイルですね。
			} else {
				//printf("フォルダじゃないですね : %s\n", d.c_str());
				
				FolderEntity entity;
				entity.name = name;
				entity.path = dirName + "/";
				entity.isFolder = false;
				
				result.childList.push_back(entity);
				
			}
		}
	}

	closedir(dir);		
}

#endif



//=========================================================
// ファイルをコピー（上書き）
// 参考：http://ameblo.jp/topazbc/entry-10095277159.html
//=========================================================
bool Utils_File::Copy ( string from, string to ) {
	
	if ( from == to ) {
		printf ( "コピー元とコピー先が同一のファイルです %s\n", from.c_str() );
		return false;
	}
	if ( from == "" ) {
		printf ( "コピー元が空文字です %s => %s\n", from.c_str(), to.c_str() );
		return false;
	}
	if ( to == "" ) {
		printf ( "コピー先が空文字です %s => %s\n", from.c_str(), to.c_str() );
		return false;
	}
	
#if defined(_WIN32)
	
	printf ( "Utils_File::Copy for Windows  %s -> %s \n", from.c_str(), to.c_str() );
	
	BOOL result = CopyFile(
		from.c_str(), // 既存のファイルの名前
		to.c_str(), // 新しいファイルの名前
		FALSE // ファイルが存在する場合の動作
	);
	
	return result;
	
#elif defined(__linux__)
	
	printf ( "Utils_File::Copy for Linux  %s -> %s \n", from.c_str(), to.c_str() );
	
	// コピー元をオープンする
	std::ifstream ifs ( from.c_str() );
	if ( !ifs ) {
		std::cerr << "file open error: " << from.c_str() << '\n';
		return false;
	}
	
	// コピー先をオープンする
	std::ofstream ofs ( to.c_str() );
	if ( !ofs ) {
		std::cerr << "file open error: " << to.c_str() << '\n';
		return false;
	}

	// コピー
	ofs << ifs.rdbuf() << std::flush;

	// エラーチェック
	if ( !ifs ) {
		std::cerr << "I/O error(from): " << from.c_str() << '\n';
		return false;
	}
	
	if ( !ofs ) {
		std::cerr << "I/O error(to): " << to.c_str() << '\n';
		return false;
	}

	return true;

#endif	

}






//=========================================================
// ファイルからロード
//=========================================================
map<string, string> Utils_File::LoadFromTextFile ( string fileName ) {
	
	printf ( "Load from \"%s\"\n", fileName.c_str() );
	
	map<string, string> result;
	
	std::ifstream ifs;
	
	ifs.open( fileName.c_str() );
	
	string row;
	while ( getline(ifs, row) ) {

		printf("%s\n", row.c_str());
		
		vector<string> elem = Utils_String::Split ( row, "=" );
		
		string key   = ( elem.size() > 0 ) ? elem[0] : "";
		string value = ( elem.size() > 1 ) ? elem[1] : "";
		
		if ( key != "" ) {
			
			if ( result.find(key) != result.end() ) {
				printf("[Warning] Utils::LoadFromFile : 同じキーが存在しています(%s)\n", key.c_str());
			}
			
			result[key] = value;
		}
	}

	ifs.close();
	
	
	return result;	
}
	
//=========================================================
// ファイルにセーブ
//=========================================================
void Utils_File::SaveToTextFile ( map<string, string> &list, string fileName) {
	
	printf ( "Save to \"%s\"\n", fileName.c_str() );
	
	FILE *file;
	file = fopen( fileName.c_str() ,"w");
	
	for ( map<string,string>::iterator i = list.begin(); i != list.end(); ++i ) {
		
		string key   = i->first;
		string value = i->second;
		
		printf ("%s=%s\n", key.c_str(), value.c_str());
		fprintf (file, "%s=%s\n", key.c_str(), value.c_str() );
	}
		
	fclose(file);	
}


//=========================================================
// ファイルにテキストを書き込む（上書き）
//=========================================================
void Utils_File::CreateAndOverwrite ( string fileName, string text ) {

	std::ofstream ofs;

	ofs.open ( fileName.c_str(), std::ios::out );

	if ( !ofs ) {

		printf ("File[%s] could not opened.\n", fileName.c_str());

	} else {

		ofs << text << std::endl;
		ofs.close();
	}
	
}

//=========================================================
// ファイルのテキストを読み込む
//=========================================================
string Utils_File::ReadTextFile ( string fileName ) {
	
	string result = "";
	
	std::ifstream ifs ( fileName.c_str() );
	
	if (!ifs)
	{
		printf ("File[%s] could not opened.\n", fileName.c_str());
		
	} else {
	
		//ifs >> result;
		std::string str((std::istreambuf_iterator<char>(ifs)), std::istreambuf_iterator<char>());
		result = str;
		ifs.close();
	}
	
	return result;
}


















