//=========================================================
//
// ファイル関係のサブルーチン
//
//=========================================================

#ifndef _UTILS_FILE_
#define _UTILS_FILE_

#include <vector>
#include <string>
#include <map>
#include <set>

#if defined(_WIN32)

	#include <Windows.h>

#elif defined(__linux__)

	#include <set>
	#include <dirent.h>
	#include <sys/types.h>
	#include <sys/stat.h>

#endif

using namespace std;

class Utils_File {

public:
	
	struct FolderEntity {
		
		string name;
		string path; // name含まず
		vector<FolderEntity> childList;
		//FolderEntity &parent;
		bool isFolder;
		//bool isRoot;
		
		// childListが空でもファイルとは限らないのでした。（空フォルダ）
		
		
		FolderEntity() {
			name = "";
			path = "";
			isFolder = false;
			//isRoot   = true; // フォルダのときだけ関係ある。
			childList.clear();
		}
	};
	
	#if defined(_WIN32)
	
	//=========================================================
	// 指定ディレクトリ以下のファイル名一覧を取得 (for Windows)
	//=========================================================
	static void GetFileList ( string dir, vector<string> &result, vector<string> path = vector<string>() );

	//=========================================================
	// 指定ディレクトリ以下のファイル名一覧を取得・フォルダ構造を保存しつつ (for Windows)
	//=========================================================
	static void GetFileList ( string dirName, FolderEntity &result, vector<string> path = vector<string>() );


	#elif defined(__linux__)
	
	
	//=========================================================
	// 指定ディレクトリ以下のファイル名一覧を取得 (for Linux)
	//=========================================================
	static void GetFileList ( string dirName, vector<string> &result, set<int> visitedDirList );
	
	//=========================================================
	// 指定ディレクトリ以下のファイル名一覧を取得・フォルダ構造を保存しつつ (for Linux)
	//=========================================================
	static void GetFileList ( string dirName, FolderEntity &result, set<int> visitedDirList );
	
	#endif

	
	//=========================================================
	// ファイルをコピー（上書き）
	//=========================================================
	static bool Copy ( string from, string to );
	
	
	//=========================================================
	// テキストファイルからロード
	//=========================================================
	static map<string, string> LoadFromTextFile ( string fileName );
		
	//=========================================================
	// テキストファイルにセーブ
	//=========================================================
	static void SaveToTextFile ( map<string, string> &list, string fileName);
	
	
	//=========================================================
	// ファイルにテキストを書き込む（上書き）
	//=========================================================
	static void CreateAndOverwrite ( string fileName, string text );
	
	//=========================================================
	// ファイルのテキストを読み込む
	//=========================================================
	static string ReadTextFile ( string fileName );
};

#endif


