//=========================================================
//
// スクリプトエンジン
//
//=========================================================

#include "Application.h"
#include "ScriptEngine.h"
#include "ScriptProcess.h"
#include "ScriptUnit.h"
#include "SceneScript.h"
#include "Image.h"
#include "BGM.h"
#include "SoundManager.h"
#include "SelectOption.h"
#include "Select.h"
#include "SaveData.h"
#include "GlobalSaveData.h"
#include "Menu.h"
#include "ReadTextManager.h"
#include "ScriptButton.h"
#include "ClickableImage.h"
#include "TextLabel.h"
#include "Fade.h"
#include "FreeBattleSelector.h"
#include "ResignButton.h"
#include "Game_Utils.h"
#include "Camera.h"
#include "EffectManager.h"
#include "Effect_BattleStart.h"
#include "Effect_BattleWin.h"
#include "Effect_BattleLose.h"
#include "Effect_BattleDraw.h"
#include "Effect_SystemMessage.h"
#include "Effect_Shake.h"

#include "../Utils/NotXML.h"
#include "../Utils/Utils_Convert.h"
#include "../Utils/Utils_String.h"
#include "../Utils/Utils_File.h"
#include "../Utils/Utils_Others.h"
#include "../GUI_SFML/TextMessageWindow.h"
#include "../GUI_SFML/SFML_Utils.h"
#include "../GUI_SFML/SoundPlayer.h"
#include "../GUI_SFML/TextImageFactory.h"

#include "CardFactory.h"
#include "GameManager.h"
#include "Piece.h"

#include "../AI/AI.h"
#include "../AI/Move.h"

#include "Effect_CuiInTest001.h"
#include "Effect_CuiInTest002.h"
#include "Effect_CutIn_COME_AND_GO.h"

#include <lua.hpp>
//#include <lua.h>

#include <stdio.h>
#include <fstream>
#include <iostream>
using namespace std;
using namespace GUI_SFML;


// 動作テスト
#include "ScoreSheetViewer.h"
#include "../GUI_SFML/FileExplorer.h"

//=========================================================
// 状態を表す定数
//=========================================================
const int ScriptEngine::STATE_NOT_ACTIVE            = 0; // 停止中
const int ScriptEngine::STATE_WAIT_UNTIL_UNIT_END   = 1; // 現在のユニットの処理が終わるのを待っている（実行中）
const int ScriptEngine::STATE_CURRENT_UNIT_FINISHED = 2; // 現在のユニットの処理は終了している。（次のユニットを実行せよ）
const int ScriptEngine::STATE_RESUME                = 3; // 現在のユニットを開始処理からやり直す


//=========================================================
// つまり Actor用の裏口
//=========================================================
GUI_SFML::TextMessageWindow* ScriptEngine::textMessageWindowParaTi = NULL;

//=========================================================
// コンストラクタ
//=========================================================
ScriptEngine::ScriptEngine() {

	_state = ScriptEngine::STATE_NOT_ACTIVE;
	//_currentProcessLabel = "";
	_currentProcess = NULL;

	///// TextImageFactory とは別に持っている。
	// TextLabel のためにここで持つことにしたのですが、統一しませんか？

	_font.loadFromFile ( TextImageFactory::_fontFileName );

	_textMessageWindow = NULL;
	_readTextManager   = new ReadTextManager();
	_BGM               = new BGM();
	_soundManager      = new SoundManager();
	_select            = new Select(this);
	_menu              = new Menu(this);
	_fade              = new Fade();
	_globalSaveData    = new GlobalSaveData ( "./data/save/save.sav" );
	_fileExplorer      = NULL;
	_endAstralProjectionButton = NULL;
	_camera            = new Camera();
	_skipIcon          = new Image();
	_skipIcon_inactive = new Image();
	_freeBattleSelector = NULL;
	
	_skipIcon->ChangeImage("./image/menu/skippingIcon.png");
	_skipIcon->SetPosition( sf::Vector2f ( 40.0f, 570.0f ) ); //sf::Vector2f ( 770.0f, 580.0f ) );

	_skipIcon_inactive->ChangeImage("./image/menu/skippingIcon_inactive.png");
	_skipIcon_inactive->SetPosition( sf::Vector2f ( 40.0f, 570.0f ) ); //sf::Vector2f ( 770.0f, 580.0f ) );
	
	_isSkippingNow = false;
	
	//_isRestDraw = false;
	_takeSSforThisSaveData = NULL;

	_scenarioLog = "";
	ClearScenarioLog();

	_select->SetScenarioLogger(this);
	//_select->_scriptEngine = this; // privateにしました

	// グローバルデータを読み込む。
	LoadGlobalData();

	_waitFrame = 0;

	//_scriptButton = new ScriptButton ( "ボタン", sf::Vector2f(150, 30), 16);
	//_scriptButton->SetPosition ( sf::Vector2f(100, 150) );
	//_scriptButton->SetJumpToLabel("jumpToLabel");

	// これは、ScriptEngine の側で、起動時と終了時に呼び出しておく、ということで、いいのですね？
	CardFactory::Init();

	//_gameManager = new GameManager();
	//_gameManager->Init ( GameManager::Mode::FREE );

	_effect_Curtain_Close = NULL; // これはエフェクトマネージャーの中で解放してもらえますよね？

	_currentTextWindowDesignID = "";

	_isRendered = true;

	_hasFocus = false;

	_actorReactionState = "BEFORE_NOTIFYING";
	
	_chapterName = "";
	
	_resignButton = new ResignButton();
	_resignButton->SetActive(false);
}

//=========================================================
// デストラクタ
//=========================================================
ScriptEngine::~ScriptEngine() {

printf ( "ScriptEngine::~ScriptEngine()");

	// グローバルデータを保存
	printf ( "1" );	SaveGlobalData();

	// コンフィグ保存
	printf ( "2" );	SaveConfig();

	printf ( "3" );	delete _textMessageWindow;
	printf ( "4" );	delete _readTextManager;
	printf ( "5" );	delete _BGM;
	printf ( "6" );	delete _soundManager;
	printf ( "7" );	delete _select;
	printf ( "8" );	delete _menu;
	printf ( "9" );	delete _fade;
	printf ( "10" );	delete _globalSaveData;
	printf ( "11" );	delete _fileExplorer;
	printf ( "12" );	delete _endAstralProjectionButton;

	printf ( "13" );	Image_Release();
	printf ( "14" );	Button_Release();
	printf ( "15" );	GameManager_ReleaseAll();
	printf ( "16" );	ClickableImage_Release ();
	printf ( "17" );	TextLabel_Release ();

	printf ( "18" );	delete _currentProcess;

	printf ( "19\n" );	CardFactory::Release();
	printf ( "20\n" );	delete _camera;
	delete _skipIcon;
	delete _skipIcon_inactive;
	//delete _scriptButton;
	//delete _gameManager;
	
	if ( _freeBattleSelector != NULL ) { delete _freeBattleSelector; }
	delete _resignButton;

printf ( "ScriptEngine::~ScriptEngine() done \n");
}

//=========================================================
// 初期設定
//=========================================================
void ScriptEngine::Init ( string scriptDir ) {

	///// プロセスタグのラベルと、そのプロセスが定義されているスクリプトファイルの対応リストを作成。
	MakeProcessIndex ( scriptDir );

	// main プロセスから開始。
	ScriptProcess *mainProcess = ReadProcess ( "main" );

	// あれ？ mainプロセスがないの？
	if ( mainProcess == NULL ) {
		printf ("Error： main プロセスが定義されていません。\n");
		return;
	}

	// 今現在のプロセスリスト、ということ。
	//_processList[ mainProcess->label ] = mainProcess;
	_currentProcess = mainProcess;

	// 状態を変更。（以前は NOT_ACTIVE だったはず）
	_state = ScriptEngine::STATE_CURRENT_UNIT_FINISHED;

	// メインプロセスから開始
	//_currentProcessLabel = mainProcess->label;


	// コンフィグをファイルから読み込む
	LoadConfig();

	// メニューに設定内容を反映させる
	_menu->SetVolumeBGM ( GetVolumeBGM() );
	_menu->SetVolumeSE  ( GetVolumeSE() );
}

//=========================================================
// プロセスタグとスクリプトファイル名の対応表を作成
//=========================================================
void ScriptEngine::MakeProcessIndex ( string scriptDir ) {

	///// 一応、クリアしておきまして、
	_processLabel_ScriptFileName_Table.clear();

	///// スクリプトファイルの一覧を取得
	vector<string> scriptFileNameList;

#if defined(_WIN32)

	Utils_File::GetFileList ( scriptDir, scriptFileNameList );

#elif defined(__linux__)
	
	set<int> visitedDirList;
	Utils_File::GetFileList ( scriptDir, scriptFileNameList, visitedDirList );

#endif


	///// プロセスタグだけ、つまみ食い
	for ( unsigned int i = 0; i < scriptFileNameList.size(); ++i ) {

		///// Processタグが最上位に並んでいるはず。
		vector <NotXML::Tag> tagList = NotXML().Read( scriptFileNameList[i] );

		for ( unsigned int n = 0; n < tagList.size(); ++n ) {

			// プロセスタグ、ですよね？
			if ( tagList[n].name == "Process") {

				// ラベルは何かな？
				string label = NotXML::GetAttribute ( tagList[n], "label" );

				// 同じラベルがすでにあったらダメ！
				if (_processLabel_ScriptFileName_Table.find ( label ) != _processLabel_ScriptFileName_Table.end() ) {
					printf ("Error：同じラベルの Process が定義されています。%s\n", label.c_str());
				}

				// このラベルの Process は、このファイルに書かれてますよー。
				_processLabel_ScriptFileName_Table [ label ] = scriptFileNameList[i];
			}

		}
	}

}

//=========================================================
// ラベルを指定して、Process を読み込む
//=========================================================
ScriptProcess* ScriptEngine::ReadProcess ( string label ) {

	ScriptProcess *result = NULL;

	///// どのファイルを読めばいいのかな？
	string scriptFileName = "";
	if ( _processLabel_ScriptFileName_Table.find ( label ) != _processLabel_ScriptFileName_Table.end() ) {
		scriptFileName = _processLabel_ScriptFileName_Table.find ( label )->second;
	} else {
		printf ("Error：定義されていない Process です。%s\n", label.c_str());
	}

	///// Processタグが最上位に並んでいるはず。該当の Processタグを探しますよ。
	vector <NotXML::Tag> tagList = NotXML().Read( scriptFileName );

	for ( unsigned int i = 0; i < tagList.size(); ++i ) {

		// プロセスタグでー、
		if (tagList[i].name == "Process") {

			// ラベルも、一致してるかな？
			string processLabel = NotXML::GetAttribute ( tagList[i], "label" );

			// あった！ これだよ、これ！
			if ( processLabel == label ) {

				// 解釈しまして、
				result = ReadProcessTag ( tagList[i] );

				// 意気揚々とループを出る。
				break;
			}
		}
	}

	return result;
}

/*
//=========================================================
// スクリプトファイルを読み込む
//=========================================================
void ScriptEngine::ReadScriptFile ( string fileName ) {


	//	このメソッドは provate
	//	なぜなら、起動時にルートプロセスを読み込む以外の読み込みは、内部での読み込み動作だから。

	///// Processタグが最上位に並んでいるはず。
	vector <NotXML::Tag> tagList = NotXML().Read( fileName );

	for ( unsigned int i = 0; i < tagList.size(); ++i ) {

		if (tagList[i].name != "Process") {
			printf ("Warning : Processタグではない。Processタグとして解釈します。%s in %s\n",
				tagList[i].name.c_str(), fileName.c_str() );
		}

		ScriptProcess *process = ReadProcessTag ( tagList[i] );

		map<string, ScriptProcess*>::iterator findIt = _processList.find ( process->label );
		if ( findIt != _processList.end() ) {
			printf ("Warning : 同名のプロセスがすでに存在します。上書きします。%s \n", process->label);

			delete findIt->second;
		}
		_processList [ process->label ] = process;
	}

}
*/

//=========================================================
// プロセスタグを解釈
//=========================================================
ScriptProcess* ScriptEngine::ReadProcessTag ( NotXML::Tag &processTag ) {

	ScriptProcess *result = new ScriptProcess();

	///// ラベルを取得：無記名は  root とする。
	result->label = NotXML::GetAttribute ( processTag, "label", "root");


	///// 子要素は、Unitタグのリストであるはず
	for ( unsigned int i = 0; i < processTag.childList.size(); ++i ) {

		NotXML::Tag tag = processTag.childList[i];

		/*
		if ( tag.name != "Unit" ) {
			printf ("Warning : Unitタグではない。Unitタグとして解釈します。%s\n", tag.name.c_str() );
		}
		*/

		///// Unit を 生成
		ScriptUnit unit;
		//unit.type          = NotXML::GetAttribute ( tag, "type", SCRIPT::UNIT_TYPE::PUT_TEXT); // 処理型が書いてなかったらテキスト表示。
		unit.type          = tag.name;
		unit.label         = NotXML::GetAttribute ( tag, "label", "");
		unit.nextUnitLabel = NotXML::GetAttribute ( tag, "next", "");
		unit.unitTag       = tag;

		///// Unit を Processに追加
		result->_unitList.push_back ( unit );
	}


	return result;
}


//=========================================================
// 更新
//=========================================================
void ScriptEngine::Update ( sf::RenderWindow &window ) {

// ホイール操作検出動作を確認
/*
		int mouseWheel = Application::GetInstance()->_mouseWheel;
		
		if ( mouseWheel > 0 ) {
			printf ("Wheel Move : Up\n");
		}
		if ( mouseWheel < 0 ) {
			printf ("Wheel Move : Down\n");
		}

// 矢印キー操作検出動作を確認
		if ( sf::Keyboard::isKeyPressed ( sf::Keyboard::Up ) ) {
			printf ("Keyboard::Up\n");
		}
		if ( sf::Keyboard::isKeyPressed ( sf::Keyboard::Down ) ) {
			printf ("Keyboard::Down\n");
		}		
*/

	/* 旧・タイトル画面に戻る機能 → セーブデータを利用する方法に変更済
	///// タイトル画面に戻ろうとしている
	if ( SceneScript::BACK_TO_TITLE > 0 ) {

		if ( SceneScript::BACK_TO_TITLE == 1 ) {
			
			_fade->FadeOut(10);
			
			SceneScript::BACK_TO_TITLE = 2;
		}
		
		if ( SceneScript::BACK_TO_TITLE == 2 ) {
			
			if ( _fade->IsStabilized() ) {
				SceneScript::BACK_TO_TITLE = 3;
			}
		}

		_fade->Update(window);
	}
	*/
	
	/*
	// 乱数のテスト
	if ( _hasFocus ) {
		if ( sf::Keyboard::isKeyPressed ( sf::Keyboard::M ) ) {
			
			vector<int> listTest;
			listTest.push_back(0);
			listTest.push_back(1);
			listTest.push_back(2);
			listTest.push_back(3);
			listTest.push_back(4);
			listTest.push_back(5);
			listTest.push_back(6);
			listTest.push_back(7);
			listTest.push_back(8);
			listTest.push_back(9);
			
			UTILS_OTHERS::RandomShuffle(listTest);
			
			for ( unsigned int i = 0; i < listTest.size(); ++i) {
				
				if ( i > 0 ) {
					printf ( "," );
				}
				printf ( "%d", listTest[i] );
				
						
			}
			printf ( "\n" );
		}
	}
	*/
	
	
	// これは描画時にした方がよかったりしますか？ で、描画の方に書いたら「これは更新時にした方がよかったりしますか？」って言いますよね？ 実は両想い。
	_hasFocus = Application::GetInstance()->HasFocus();

	///// クイックセーブ/ロードする？
	if ( _hasFocus ) {

		if ( Application::GetInstance()->_releasedKeyCode == sf::Keyboard::Q ) {
			QuickSave();
		}
		if ( Application::GetInstance()->_releasedKeyCode == sf::Keyboard::L ) {
			QuickLoad();
		}
	}

	/*
	if ( Application::GetInstance()->_releasedKeyCode == sf::Keyboard::P ) {
		SystemMessage("システムメッセージ実験");
	}
	*/
	
	///// Ctrlで「既読スキップフラグ」を切り替える
	if ( Application::GetInstance()->_releasedKeyCode == sf::Keyboard::LControl || Application::GetInstance()->_releasedKeyCode == sf::Keyboard::RControl ) 
	{
		if ( _menu->GetItemValue ( "IsSkip" ) == "on") {
			
			_menu->SetItemValue( "IsSkip", "off" ); 
			
			_isSkippingNow = false;
		
		} else {
		
			_menu->SetItemValue( "IsSkip", "on" ); 
		}
	}
	
	
	///// 上下反転
	if ( _hasFocus ) {
		if ( sf::Keyboard::isKeyPressed ( sf::Keyboard::R ) ) {
			_camera->VerticalReverse(true);
		} else {
			_camera->VerticalReverse(false);
		}
	}
	/*
	if ( Application::GetInstance()->_releasedKeyCode == sf::Keyboard::R ) {
		_camera->VerticalReverse(false);
	}
	*/
	
	///// 変数をコンソールに表示
	if ( _hasFocus ) {
		if ( Application::GetInstance()->_releasedKeyCode == sf::Keyboard::V ) {
			ShowVariablesToConsole();
		}
	}


	///// 描画してもらうのがデフォルト。必要なときだけ Application に描画を休んでもらう
	_isRendered = true;

	///// メニュー表示中？
	if ( _menu->IsActive() ) {

		// 処理を独占いたしますー。
		_menu->Update ( window );

	///// メニューが表示されていない。
	} else {

		///// メニュー表示中はゲーム盤の更新を止める所存。
		GameManager_Update ( window );

		///// 今だ！ スクリプトを進行させるのだ！
		Update_ProgressScript();

		if (_fileExplorer != NULL ) {
			_fileExplorer->Update(window);
		}


		if (_endAstralProjectionButton != NULL) {
			_endAstralProjectionButton->Update(window);
		}
		
		
		// そっかー、テキスト欄は NULL って可能性があるのですねー。
		if ( _textMessageWindow != NULL ) {
			_textMessageWindow->Update ( window );
		}


		// 選択肢は読み返しモードのときには無反応になっていただく
		if (_textMessageWindow == NULL || !_textMessageWindow->IsScenarioLogMode() ) {
			_select ->Update ( window );
		}
		
		if ( _freeBattleSelector != NULL ) {
			_freeBattleSelector->Update ( window );
		}


		// カメラ更新
		_camera->Update();

		
		///// 右クリックを受け付けないかもしれない
		if
		(   ///// 受け付けない条件を列挙。列挙するほどあるかな？
			(_textMessageWindow != NULL && !_textMessageWindow->IsScenarioLogMode() ) // 読み返しビューワー表示中（というか、読み返しモード解除の右クリック と かぶる）
			&&
			!GameManager_IsHoldingCard() // カードの右クリック回転 と かぶる
		)
		{


			///// 右クリックを受け付ける：スクリプトの _state には無関係なのですね。画像の遷移中とか。
			if ( Application::GetInstance()->_mouseState_Right == MOUSE::BUTTON::STATE::RELEASE ) {

				// 一時的に止めておく。
				// ゲーム盤の操作との折り合いを考え次第、復活。
				_menu->SetActive ( true );
			}

		}

		Image_Update ( window );
		Button_Update ( window );
		_BGM->Update ();
		

		// 早送り中はフェードもスキップ
		if  ( IsSkip() ) {
			_fade->Stabilize();
		} else {
			_fade->Update ( window );
		}

		ClickableImage_Update ( window );
		TextLabel_Update(window);
	}


	//_scriptButton->Update(window);
	//if (_scriptButton->IsRelease()) {
	//	printf ( "%s 押された\n", _scriptButton->GetJumpToLabel().c_str() );
	//}


	//_saveData->Update( window );
	
	// あまり深い意味はないかもしれないけれど。
	_skipIcon->Update( window );
	_skipIcon_inactive->Update( window );

	/*
	///// カットインエフェクト実験
	if ( Application::GetInstance()->_mouseState_Middle == MOUSE::BUTTON::STATE::RELEASE ) {

		map <string, string> param;

		Effect_CutIn_COME_AND_GO *effect = new Effect_CutIn_COME_AND_GO ( NULL, 0, _textMessageWindow->GetSameSetuppedInstance(), param );
		//effect->_textMessageWindow = _textMessageWindow->GetSameSetuppedInstance();

		Application::GetInstance()->_effectManager->Add ( effect );

	}
	*/

	// 動作テスト
	//_fileExplorer->Update(window);

	// 動作テスト
	/*
	if ( Application::GetInstance()->_mouseState_Middle == MOUSE::BUTTON::STATE::RELEASE ) {
		_scoreSheetViewer->Add("動作テスト");
		_scoreSheetViewer->MoveToBottom();
	}
	if ( Application::GetInstance()->_mouseState_Right == MOUSE::BUTTON::STATE::RELEASE ) {
		_scoreSheetViewer->Add("動作テスト。");
		_scoreSheetViewer->MoveToBottom();
	}
	*/

	// 動作テストUpdate
	//_scoreSheetViewer->Update ( window );

	//_scenarioLogViewer->Update(window);
	
}

//=========================================================
// スクリプト進行
//=========================================================
void ScriptEngine::Update_ProgressScript() {

	///// 状態：現在の処理が終了するのを待っている（実行中）
	if ( _state == ScriptEngine::STATE_WAIT_UNTIL_UNIT_END ) {

		int currentIndex = _currentProcess->currentUnitIndex;

		if ( currentIndex < 0 ) {

			// これは、ありえない

		} else {

			// ユニットを取得できるとしましょう。
			ScriptUnit &unit = _currentProcess->_unitList[currentIndex];

			// ユニットごとの処理へ
			UnitProcessing (unit);
		}


	///// 状態：現在の処理は終了している（次の処理を開始）
	} else if ( _state == ScriptEngine::STATE_CURRENT_UNIT_FINISHED ) {

		/*

			終わってたら呼び出し元に戻る。
			呼び出し元がなかったら、全部終わったということ。

			次にどのユニットを実行すればいいのかは、
			現在のユニット、が持っている。
		*/

		// 次に実行するユニットのインデックス
		int nextUnitIndex = 0;

		// 現在のプロセスが、現在実行中のユニットのインデックスを持っている。
		//int currentIndex = _processList [ _currentProcessLabel ]->currentUnitIndex;
		int currentIndex = _currentProcess->currentUnitIndex;

		// このプロセスを開始した直後は、こうなる。「現在のユニット」がないから
		if ( currentIndex < 0 ) {

			// インデックス０。ユニットリストの冒頭
			nextUnitIndex = 0;

		// 「現在のユニット」から取得
		} else {

			// ラベルが指定されている？
			string nextUnitLabel = _currentProcess->_unitList[ currentIndex ].nextUnitLabel;
			if ( nextUnitLabel != "" ) {

				// そのラベルのユニットを探す。
				bool isExistSpecifyUnit = false;
				for ( unsigned int i = 0; i < _currentProcess->_unitList.size(); ++i ) {

					if ( _currentProcess->_unitList[i].label == nextUnitLabel ) {
						nextUnitIndex = i;
						isExistSpecifyUnit = true;
						break;
					}
				}
				if (!isExistSpecifyUnit) {
					printf ("Error : 現在のプロセス(%s)に指定のラベルがついたユニット(%s)がありません。\n",
						_currentProcess->label.c_str(), nextUnitLabel.c_str() );
				}


			// ラベル指定なし
			} else {

				// 次のユニット。
				nextUnitIndex = currentIndex + 1;
			}
		}

		///// プロセス終了？
		if ( nextUnitIndex >= _currentProcess->_unitList.size() ) {

			// Applicationに描画を休んでもらう。
			_isRendered = false;

			// プロセスのユニットインデックスを初期化しておく。次回に備えて。
			_currentProcess->currentUnitIndex = -1;

			// 呼び出し元
			ScriptProcess *refererProcess = _currentProcess->referingProcess;

			// 現在のプロセス（サブルーチン）を解放
			//delete _processList [ _currentProcessLabel ];
			//_processList.erase ( _processList.find(_currentProcessLabel) );

			_currentProcess->referingProcess = NULL;  // 呼び出し元を退避しておく。一緒に削除されないように。
			delete _currentProcess;                   // その上で、削除。
			_currentProcess = NULL;

			// 呼び出し元があるなら、そのプロセスに移る
			if ( refererProcess != NULL ) {

				_currentProcess = refererProcess;

			// 呼び出し元がない＝ルートプロセスが終了した、ということ
			} else {

				_state = ScriptEngine::STATE_NOT_ACTIVE;
				_currentProcess = NULL;

				printf ("Information : 全プロセスが終了しました。\n");

				// 一応、元に戻しておく：Updateの冒頭で元に戻しているわけですが、終了なので、もはや元に戻らない。終了なのだから戻らなくていい？ どうでしょう？
				_isRendered = true;
			}


		// プロセス終了じゃない
		} else {

			// 次のユニットへ進む
			_currentProcess->currentUnitIndex = nextUnitIndex;

			// ユニットを取得できるとしましょう。
			ScriptUnit &unit = _currentProcess->_unitList[nextUnitIndex];

			// ユニットごとの処理へ
			UnitStart ( unit );

			// ユニットを開始できた、としましょう。状態を変更：「終了を待機（実行中）」
			// これは、各処理型に依存することとする。
			// 処理の終了を待って欲しい処理なら、そうするし、すぐに終わっていいなら、そのようにする。
			//_state = ScriptEngine::STATE_WAIT_UNTIL_UNIT_END;

			// Applicationに描画を休んでもらう。
			_isRendered = false;
		}


	///// 状態：停止中
	} else if (_state == ScriptEngine::STATE_NOT_ACTIVE) {

		// 処理なし


	///// 状態：現在のユニットを開始処理からやり直す
	} else if (_state == ScriptEngine::STATE_RESUME) {

		// 現在のユニットを取得
		ScriptUnit &unit = _currentProcess->_unitList[_currentProcess->currentUnitIndex];

		// ユニットごとの処理へ
		UnitStart ( unit );

		// Applicationに描画を休んでもらう。
		_isRendered = false;
	}
}


//=========================================================
// ユニット開始処理へ
//=========================================================
void ScriptEngine::UnitStart ( ScriptUnit &unit ) {

	// ユニットごとの処理へ
	if      ( unit.type == SCRIPT::UNIT_TYPE::DEFINE_TEXT_WINDOW ) { UnitStart_DefineTextWindow   ( unit );}
	else if ( unit.type == SCRIPT::UNIT_TYPE::TEXT_WINDOW_Design ) { UnitStart_TextWindowDesign   ( unit );}
	else if ( unit.type == SCRIPT::UNIT_TYPE::PUT_TEXT           ) { UnitStart_Text               ( unit );}
	else if ( unit.type == SCRIPT::UNIT_TYPE::TEXTAlias_Regist   ) { UnitStart_TextAlias_Regist   ( unit );}
	else if ( unit.type == SCRIPT::UNIT_TYPE::PROTAGONIST_Regist ) { UnitStart_Protagonist_Regist ( unit );}
	else if ( unit.type == SCRIPT::UNIT_TYPE::PRINT              ) { UnitStart_Print              ( unit );}
	else if ( unit.type == SCRIPT::UNIT_TYPE::SYSTEM_MESSAGE     ) { UnitStart_SystemMessage      ( unit );}
	else if ( unit.type == SCRIPT::UNIT_TYPE::IMAGE              ) { UnitStart_Image              ( unit );}
	else if ( unit.type == SCRIPT::UNIT_TYPE::IMAGE_Regist       ) { UnitStart_Image_Regist       ( unit );}
	else if ( unit.type == SCRIPT::UNIT_TYPE::IMAGE_Group        ) { UnitStart_ImageGroup         ( unit );}
	else if ( unit.type == SCRIPT::UNIT_TYPE::BGM_Regist         ) { UnitStart_BGM_Regist         ( unit );}
	else if ( unit.type == SCRIPT::UNIT_TYPE::BGM_Change         ) { UnitStart_BGM_Change         ( unit );}
	else if ( unit.type == SCRIPT::UNIT_TYPE::BGM_Config         ) { UnitStart_BGM_Config         ( unit );}
	else if ( unit.type == SCRIPT::UNIT_TYPE::Sound_Regist       ) { UnitStart_Sound_Regist       ( unit );}
	else if ( unit.type == SCRIPT::UNIT_TYPE::Sound              ) { UnitStart_Sound              ( unit );}
	else if ( unit.type == SCRIPT::UNIT_TYPE::TextLabel          ) { UnitStart_TextLabel          ( unit );}
	else if ( unit.type == SCRIPT::UNIT_TYPE::Select             ) { UnitStart_Select             ( unit );}
	else if ( unit.type == SCRIPT::UNIT_TYPE::Variable_Define    ) { UnitStart_Variable_Define    ( unit );}
	else if ( unit.type == SCRIPT::UNIT_TYPE::Variable_Release   ) { UnitStart_Variable_Release   ( unit );}
	else if ( unit.type == SCRIPT::UNIT_TYPE::Variable_Operation ) { UnitStart_Variable_Operation ( unit );}
	else if ( unit.type == SCRIPT::UNIT_TYPE::Variable_Console   ) { UnitStart_Variable_Console   ( unit );}
	else if ( unit.type == SCRIPT::UNIT_TYPE::IF                 ) { UnitStart_IF                 ( unit );}
	else if ( unit.type == SCRIPT::UNIT_TYPE::Switch             ) { UnitStart_Switch             ( unit );}
	else if ( unit.type == SCRIPT::UNIT_TYPE::Gosub              ) { UnitStart_Gosub              ( unit );}
	else if ( unit.type == SCRIPT::UNIT_TYPE::Button             ) { UnitStart_Button             ( unit );}
	else if ( unit.type == SCRIPT::UNIT_TYPE::Game               ) { UnitStart_Game               ( unit );}
	else if ( unit.type == SCRIPT::UNIT_TYPE::Game_Release       ) { UnitStart_Game_Release       ( unit );}
	else if ( unit.type == SCRIPT::UNIT_TYPE::Move               ) { UnitStart_Move               ( unit );}
	else if ( unit.type == SCRIPT::UNIT_TYPE::Put                ) { UnitStart_Put                ( unit );}
	else if ( unit.type == SCRIPT::UNIT_TYPE::Remove             ) { UnitStart_Remove             ( unit );}
	else if ( unit.type == SCRIPT::UNIT_TYPE::RemoveAll          ) { UnitStart_RemoveAll          ( unit );}
	else if ( unit.type == SCRIPT::UNIT_TYPE::WatchEvent         ) { UnitStart_WatchEvent         ( unit );}
	else if ( unit.type == SCRIPT::UNIT_TYPE::WatchGameOperation ) { UnitStart_WatchGameOperation ( unit );}
	else if ( unit.type == SCRIPT::UNIT_TYPE::Jump               ) { UnitStart_Jump               ( unit );}
	else if ( unit.type == SCRIPT::UNIT_TYPE::Wait               ) { UnitStart_Wait               ( unit );}
	else if ( unit.type == SCRIPT::UNIT_TYPE::WaitClick          ) { UnitStart_WaitClick          ( unit );}
	else if ( unit.type == SCRIPT::UNIT_TYPE::Battle             ) { UnitStart_Battle             ( unit );}
	else if ( unit.type == SCRIPT::UNIT_TYPE::AstralProjection   ) { UnitStart_AstralProjection   ( unit );}
	else if ( unit.type == SCRIPT::UNIT_TYPE::Fade               ) { UnitStart_Fade               ( unit );}
	else if ( unit.type == SCRIPT::UNIT_TYPE::Shake              ) { UnitStart_Shake              ( unit );}
	else if ( unit.type == SCRIPT::UNIT_TYPE::Camera             ) { UnitStart_Camera             ( unit );}
	else if ( unit.type == SCRIPT::UNIT_TYPE::Title              ) { UnitStart_Title              ( unit );}
	else if ( unit.type == SCRIPT::UNIT_TYPE::Chapter            ) { UnitStart_Chapter            ( unit );}
	else if ( unit.type == SCRIPT::UNIT_TYPE::Save               ) { UnitStart_Save               ( unit );}
	else if ( unit.type == SCRIPT::UNIT_TYPE::Load               ) { UnitStart_Load               ( unit );}
	else if ( unit.type == SCRIPT::UNIT_TYPE::WriteSaveData      ) { UnitStart_WriteSaveData      ( unit );}
	else if ( unit.type == SCRIPT::UNIT_TYPE::FileSelect         ) { UnitStart_FileSelect         ( unit );}
	else if ( unit.type == SCRIPT::UNIT_TYPE::ClickableImage         ) { UnitStart_ClickableImage ( unit );}
	else if ( unit.type == SCRIPT::UNIT_TYPE::ClearClickableImage    ) { UnitStart_ClearClickableImage ( unit );}
	else if ( unit.type == SCRIPT::UNIT_TYPE::DefineControlTag       ) { UnitStart_DefineControlTag    ( unit );}
	else if ( unit.type == SCRIPT::UNIT_TYPE::WaitForClickableImages ) { UnitStart_WaitForClickableImages ( unit );}
	else if ( unit.type == SCRIPT::UNIT_TYPE::FreeBattleSelector     ) { UnitStart_FreeBattleSelector     ( unit );}
	else if ( unit.type == SCRIPT::UNIT_TYPE::Skipping               ) { UnitStart_Skipping               ( unit );}

}

//=========================================================
// ユニット実行中（終了を待機）
//=========================================================
void ScriptEngine::UnitProcessing ( ScriptUnit &unit ) {

	if      ( unit.type == SCRIPT::UNIT_TYPE::DEFINE_TEXT_WINDOW ) { UnitProcessing_DefineTextWindow   ( unit );}
	else if ( unit.type == SCRIPT::UNIT_TYPE::TEXT_WINDOW_Design ) { UnitProcessing_TextWindowDesign   ( unit );}
	else if ( unit.type == SCRIPT::UNIT_TYPE::PUT_TEXT           ) { UnitProcessing_Text               ( unit );}
	else if ( unit.type == SCRIPT::UNIT_TYPE::TEXTAlias_Regist   ) { UnitProcessing_TextAlias_Regist   ( unit );}
	else if ( unit.type == SCRIPT::UNIT_TYPE::PROTAGONIST_Regist ) { UnitProcessing_Protagonist_Regist ( unit );}
	else if ( unit.type == SCRIPT::UNIT_TYPE::PRINT              ) { UnitProcessing_Print              ( unit );}
	else if ( unit.type == SCRIPT::UNIT_TYPE::SYSTEM_MESSAGE     ) { UnitProcessing_SystemMessage      ( unit );}
	else if ( unit.type == SCRIPT::UNIT_TYPE::IMAGE              ) { UnitProcessing_Image              ( unit );}
	else if ( unit.type == SCRIPT::UNIT_TYPE::IMAGE_Regist       ) { UnitProcessing_Image_Regist       ( unit );}
	else if ( unit.type == SCRIPT::UNIT_TYPE::IMAGE_Group        ) { UnitProcessing_Image_Group        ( unit );}
	else if ( unit.type == SCRIPT::UNIT_TYPE::BGM_Regist         ) { UnitProcessing_BGM_Regist         ( unit );}
	else if ( unit.type == SCRIPT::UNIT_TYPE::BGM_Change         ) { UnitProcessing_BGM_Change         ( unit );}
	else if ( unit.type == SCRIPT::UNIT_TYPE::BGM_Config         ) { UnitProcessing_BGM_Config         ( unit );}
	else if ( unit.type == SCRIPT::UNIT_TYPE::Sound_Regist       ) { UnitProcessing_Sound_Regist       ( unit );}
	else if ( unit.type == SCRIPT::UNIT_TYPE::Sound              ) { UnitProcessing_Sound              ( unit );}
	else if ( unit.type == SCRIPT::UNIT_TYPE::TextLabel          ) { UnitProcessing_TextLabel          ( unit );}
	else if ( unit.type == SCRIPT::UNIT_TYPE::Select             ) { UnitProcessing_Select             ( unit );}
	else if ( unit.type == SCRIPT::UNIT_TYPE::Variable_Define    ) { UnitProcessing_Variable_Define    ( unit );}
	else if ( unit.type == SCRIPT::UNIT_TYPE::Variable_Release   ) { UnitProcessing_Variable_Release   ( unit );}
	else if ( unit.type == SCRIPT::UNIT_TYPE::Variable_Operation ) { UnitProcessing_Variable_Operation ( unit );}
	else if ( unit.type == SCRIPT::UNIT_TYPE::Variable_Console   ) { UnitProcessing_Variable_Console   ( unit );}
	else if ( unit.type == SCRIPT::UNIT_TYPE::IF                 ) { UnitProcessing_IF                 ( unit );}
	else if ( unit.type == SCRIPT::UNIT_TYPE::Switch             ) { UnitProcessing_Switch             ( unit );}
	else if ( unit.type == SCRIPT::UNIT_TYPE::Gosub              ) { UnitProcessing_Gosub              ( unit );}
	else if ( unit.type == SCRIPT::UNIT_TYPE::Button             ) { UnitProcessing_Button             ( unit );}
	else if ( unit.type == SCRIPT::UNIT_TYPE::Game               ) { UnitProcessing_Game               ( unit );}
	else if ( unit.type == SCRIPT::UNIT_TYPE::Game_Release       ) { UnitProcessing_Game_Release       ( unit );}
	else if ( unit.type == SCRIPT::UNIT_TYPE::Move               ) { UnitProcessing_Move               ( unit );}
	else if ( unit.type == SCRIPT::UNIT_TYPE::Put                ) { UnitProcessing_Put                ( unit );}
	else if ( unit.type == SCRIPT::UNIT_TYPE::Remove             ) { UnitProcessing_Remove             ( unit );}
	else if ( unit.type == SCRIPT::UNIT_TYPE::RemoveAll          ) { UnitProcessing_RemoveAll          ( unit );}
	else if ( unit.type == SCRIPT::UNIT_TYPE::WatchEvent         ) { UnitProcessing_WatchEvent         ( unit );}
	else if ( unit.type == SCRIPT::UNIT_TYPE::WatchGameOperation ) { UnitProcessing_WatchGameOperation ( unit );}
	else if ( unit.type == SCRIPT::UNIT_TYPE::Jump               ) { UnitProcessing_Jump               ( unit );}
	else if ( unit.type == SCRIPT::UNIT_TYPE::Wait               ) { UnitProcessing_Wait               ( unit );}
	else if ( unit.type == SCRIPT::UNIT_TYPE::WaitClick          ) { UnitProcessing_WaitClick          ( unit );}
	else if ( unit.type == SCRIPT::UNIT_TYPE::Battle             ) { UnitProcessing_Battle             ( unit );}
	else if ( unit.type == SCRIPT::UNIT_TYPE::AstralProjection   ) { UnitProcessing_AstralProjection   ( unit );}
	else if ( unit.type == SCRIPT::UNIT_TYPE::Fade               ) { UnitProcessing_Fade               ( unit );}
	else if ( unit.type == SCRIPT::UNIT_TYPE::Shake              ) { UnitProcessing_Shake              ( unit );}
	else if ( unit.type == SCRIPT::UNIT_TYPE::Camera             ) { UnitProcessing_Camera             ( unit );}
	else if ( unit.type == SCRIPT::UNIT_TYPE::Title              ) { UnitProcessing_Title              ( unit );}
	else if ( unit.type == SCRIPT::UNIT_TYPE::Chapter            ) { UnitProcessing_Chapter            ( unit );}
	else if ( unit.type == SCRIPT::UNIT_TYPE::Save               ) { UnitProcessing_Save               ( unit );}
	else if ( unit.type == SCRIPT::UNIT_TYPE::Load               ) { UnitProcessing_Load               ( unit );}
	else if ( unit.type == SCRIPT::UNIT_TYPE::WriteSaveData      ) { UnitProcessing_WriteSaveData      ( unit );}
	else if ( unit.type == SCRIPT::UNIT_TYPE::FileSelect         ) { UnitProcessing_FileSelect         ( unit );}
	else if ( unit.type == SCRIPT::UNIT_TYPE::ClickableImage         ) { UnitProcessing_ClickableImage         ( unit );}
	else if ( unit.type == SCRIPT::UNIT_TYPE::ClearClickableImage    ) { UnitProcessing_ClearClickableImage    ( unit );}
	else if ( unit.type == SCRIPT::UNIT_TYPE::DefineControlTag       ) { UnitProcessing_DefineControlTag       ( unit );}
	else if ( unit.type == SCRIPT::UNIT_TYPE::WaitForClickableImages ) { UnitProcessing_WaitForClickableImages ( unit );}
	else if ( unit.type == SCRIPT::UNIT_TYPE::FreeBattleSelector     ) { UnitProcessing_FreeBattleSelector     ( unit );}
	else if ( unit.type == SCRIPT::UNIT_TYPE::Skipping               ) { UnitProcessing_Skipping               ( unit );}
}

//=========================================================
// サブルーチンを呼び出す
//=========================================================
void ScriptEngine::Gosub ( string processLabel ) {

	///// 別ユニットを処理中なので、今は無理ー。全プロセスが終了した後も、無理。
	if (_state != STATE_CURRENT_UNIT_FINISHED) {
		printf ("今は別ルーチンに移動できません。\n");
		return;
	}


	///// プロセスを取得
	ScriptProcess *subProcess = ReadProcess ( processLabel );

	if ( subProcess == NULL ) {
		printf ( "Error : プロセス \"%s\" は定義されていません。\n", processLabel.c_str());
		return;
	}

	///// プロセスリストに入れる
	//_processList[processLabel] = process;

	///// 呼び出し元を設定。
	subProcess->referingProcess = _currentProcess;

	///// 「現在のプロセス」とする。
	_currentProcess = subProcess;
}


//=========================================================
// 描画
//=========================================================
void ScriptEngine::Draw   ( sf::RenderWindow &window ) {

	///// 一旦、Updateをかけてから表示しなおしたい？
	//if ( _isRestDraw ) {
	//	_isRestDraw = false;
	//	return;
	//}

	Image_Draw_Back ( window );

	GameManager_Draw ( window );

	Image_Draw_Object ( window );

	ClickableImage_Draw ( window );

	TextLabel_Draw ( window );
	

	if ( _freeBattleSelector != NULL ) {
		_freeBattleSelector->Draw ( window );
	}


	///// 選択肢クラス：読み返しモードのときには、テキスト欄よりも奥に描画
	if (_textMessageWindow != NULL && _textMessageWindow->IsScenarioLogMode() ) {
		_select->Draw(window);
	}

	// テキスト欄
	if ( _textMessageWindow != NULL ) {
		
		// 非表示のオプション
		if ( !sf::Keyboard::isKeyPressed ( sf::Keyboard::LShift  ) && !sf::Keyboard::isKeyPressed ( sf::Keyboard::RShift ) ) {
			_textMessageWindow->Draw ( window );
		}
	}
	

	// 口出しモードの進行パネル：テキスト欄の後（手前Z）に
	for ( map<string, GameManager*>::iterator it = _gameManagerList.begin(); it != _gameManagerList.end(); ++it ) {

		if (it->second->IsActive()) {
			it->second->Draw_InterfereModePanel(window);
		}
	}


	Button_Draw ( window );

	// このへんで（重ね合わせ具合で、ということ）棋譜を表示
	GameManager_Draw_ScoreSheet ( window );

	// 検討モード終了ボタンは棋譜ビューワーの上に来ないと、、、来ないと？ 大変なことになる！ キャー！
	if (_endAstralProjectionButton != NULL) {
		_endAstralProjectionButton->Draw(window);
	}
	/*
		TODO ただし、読み返しログビューワーを出すと、これまた表示順がいただけないことになる。
		ログビューワーの上に棋譜関連のボタンが出るのですよね。
		なぜかというと、読み返しログビューワーはテキストウィンドウの中にあるから。

		もう、TextMessageWindow を GUI_SFML から ScriptEngine側に移してですね、
		で、IDrawable を持たせて、後回し描画をするとよいのかもしれない。
	*/


	///// 選択肢クラス：読み返しモードじゃないなら、このZ位置に描画
	if (_textMessageWindow == NULL || !_textMessageWindow->IsScenarioLogMode() ) {
		_select->Draw(window);
	}

	///// ファイル選択さんがいらっしゃれば
	if ( _fileExplorer != NULL ) {
		_fileExplorer->Draw(window);
	}
	
	
	///// じゃあこのへんで、投了ボタンを（表示すべきかどうかはこの内部で判定している）オブジェクトも常に存在する仕様
	_resignButton->Draw(window);


	///// SS撮影の要請であります。これは SaveData用
	// この段階で SS を撮れば、メニューを除外できる。
	//if ( Application::GetInstance()->_mouseState_Middle == MOUSE::BUTTON::STATE::RELEASE ) {
	if ( _takeSSforThisSaveData != NULL ) {

		sf::Texture texture;
		texture.create ( window.getSize().x, window.getSize().y );
		texture.update ( window );
		
		sf::Image image = texture.copyToImage();
		
		// サイズを圧縮したい
		sf::Image compressedImage = CompressImage ( image, 160, 120 );
		
		
		//image.saveToFile ( _takeSSforThisSaveData->GetSSFileName() );
		compressedImage.saveToFile ( _takeSSforThisSaveData->GetSSFileName() );
		
		///// deprecated
		//sf::Image ss = window.capture();
		//ss.saveToFile ( _takeSSforThisSaveData->GetSSFileName() );

		// なるほど、画像の更新までお知らせしてくれるのですね。
		_takeSSforThisSaveData->UpdateSS();

		// 任務完了
		_takeSSforThisSaveData = NULL;
	}
	///// ファイル名指定で SSを所望
	if ( _takeSStoThisFile != "" ) {

		sf::Texture texture;
		texture.create ( window.getSize().x, window.getSize().y );
		texture.update ( window );

		sf::Image image = texture.copyToImage();
		image.saveToFile ( _takeSStoThisFile );

		// 任務完了
		_takeSStoThisFile = "";
	}
	
	_fade->Draw ( window );
	
	// 既読スキップフラグのアイコン
	if ( _menu->GetItemValue ( "IsSkip" ) == "on") {
		
		if (_isSkippingNow) {
			_skipIcon->Draw(window);
		} else {
			_skipIcon_inactive->Draw(window);
		}
		
	}

	_menu->Draw ( window );


	/* 旧・タイトル画面に戻る機能に対応するためのもの
		// 平常時
		if ( SceneScript::BACK_TO_TITLE == 0 ) {
			
		// タイトル画面に戻ろうとしているとき（メニューもフェードアウトで覆う）
		} else {
			_menu->Draw ( window );
			_fade->Draw ( window );
		}
	*/


	// 動作テスト
	//_scoreSheetViewer->Draw ( window );
	//_scenarioLogViewer->Draw(window);
	//_fileExplorer->Draw(window);

}


//=========================================================
// SS用。画像圧縮
// より小さいサイズに「圧縮」する前提。汎用性はなし。
//=========================================================
sf::Image ScriptEngine::CompressImage ( sf::Image &srcImage, int newWidth, int newHeight ) {
	
	sf::Image result;
	
	result.create(newWidth, newHeight);
	
	float newWidthF  = (float)newWidth;
	float newHeightF = (float)newHeight;
	
	float srcWidth  = (float)srcImage.getSize().x;
	float srcHeight = (float)srcImage.getSize().y;
	
	// 以降、ゼロ除算の分岐を入れなくて済むように
	if ( srcImage.getSize().x == 0 || srcImage.getSize().y == 0 ) {
		printf ("元画像のサイズがゼロなので処理なしで終了\n");
		return result;
	}
	
	float xRate = newWidthF  / srcWidth;
	float yRate = newHeightF / srcHeight;
	
	printf ( "サイズの割合：xRate : %f, yRate : %f\n", xRate, yRate );
	
	for ( unsigned int y = 0; y < newHeight; ++y ) {
	
		for ( unsigned int x = 0; x < newWidth; ++x ) {
			
			// この (x, y) は 元画像では どの範囲に該当するか？
			
			int minX = (float)x / xRate;
			int maxX = (float)(x + 1) / xRate;
			
			int minY = (float)y   / yRate;
			int maxY = (float)(y + 1) / yRate;
			
			int r = 0;
			int g = 0;
			int b = 0;
			int a = 0;
			int count = 0;
			for ( unsigned int nY = minY; nY < maxY; ++nY ) {
				for ( unsigned int nX = minX; nX < maxX; ++nX ) {
					
					if ( nX >= srcImage.getSize().x ) {
						continue;
					}
					if ( nY >= srcImage.getSize().y ) {
						continue;
					}
					
					++count;
					sf::Color color = srcImage.getPixel(nX, nY);
					r += color.r;
					g += color.g;
					b += color.b;
					a += color.a;
				}
			}
			
			
			sf::Color newColor(0,0,0,0);
			if ( count != 0 ) {
				//printf ( "(%d, %d) count:%d, color(%d,%d,%d,%d)\n", x, y, count, r,g,b,a );
				newColor = sf::Color( r / count, g / count, b / count, a / count );
			}
			
			result.setPixel(x, y, newColor);
			
		}
	}
	
	
	
	return result;
}



//=========================================================
// ユニット開始処理：テキストウィンドウ定義
//=========================================================
void ScriptEngine::UnitStart_DefineTextWindow ( ScriptUnit &unit ) {

	// とりあえず、設定を無視してインスタンス生成：すでにあったら前のを消す？
	/*
		すでにあったら、指定されているプロパティを上書き変更、ということでどうか？
		表示位置を動かしたり、サイズを変更したり。
	*/
	//_textMessageWindow = new TextMessageWindow();
	//_textMessageWindow->SetPosition( sf::Vector2f(10.0f, 400.0f) );

	///// インスタンスがないなら、生成する（あるなら、使いまわす）
	if ( _textMessageWindow == NULL ) {

		_textMessageWindow = new TextMessageWindow ( this );
		_textMessageWindow->SetScenarioLogger (this);

		// TODO システム公式のテキスト欄、とでも言えばよろしいでしょうか。new しているのがこの箇所だけなのは幸い。
		ScriptEngine::textMessageWindowParaTi = _textMessageWindow;
	}


	///// 値を設定
	for ( unsigned int i = 0; i < unit.unitTag.childList.size(); ++i ) {

		string tagName  = unit.unitTag.childList[i].name;
		string tagValue = unit.unitTag.childList[i].value;

		if ( tagName == "width" ) {
			_textMessageWindow->SetWidth ( Utils_Convert::String_to_Int ( tagValue ) );
		}
		else if ( tagName == "height" ) {
			_textMessageWindow->SetHeight ( Utils_Convert::String_to_Int ( tagValue ) );
		}
		else if ( tagName == "position_x" ) {
			_textMessageWindow->SetPositionX ( Utils_Convert::String_to_Int ( tagValue ) );
		}
		else if ( tagName == "position_y" ) {
			_textMessageWindow->SetPositionY ( Utils_Convert::String_to_Int ( tagValue ) );
		}
		else if ( tagName == "offset_x" ) {
			_textMessageWindow->SetOffsetX ( Utils_Convert::String_to_Int ( tagValue ) );
		}
		else if ( tagName == "offset_y" ) {
			_textMessageWindow->SetOffsetY ( Utils_Convert::String_to_Int ( tagValue ) );
		}
		else if ( tagName == "speakerName_displayType" ) {
			_textMessageWindow->SetSpeakerNameDisplayType ( tagValue );
		}
		else if ( tagName == "speakFrom" ) {
			_textMessageWindow->SetSpeakFrom ( tagValue );
		}
		else if ( tagName == "speakerName_offset_x" ) {
			_textMessageWindow->SetSpeakerNameOffsetX ( Utils_Convert::String_to_Int ( tagValue ) );
		}
		else if ( tagName == "speakerName_offset_y" ) {
			_textMessageWindow->SetSpeakerNameOffsetY ( Utils_Convert::String_to_Int ( tagValue ) );
		}
		else if ( tagName == "speakerName_offset_centering" ) {
			_textMessageWindow->SetSpeakerNameOffsetCentering ( Utils_Convert::String_to_Int ( tagValue ) );
		}
		else if ( tagName == "speakerName_backImage" ) {
			_textMessageWindow->SetSpeakerNameBackImage ( ParseImageFileNameAsAlias (tagValue) );
		}
		else if ( tagName == "speakerName_backImage_wide" ) {

			_textMessageWindow->ClearSpeakerNameBackImage_wide();

			NotXML::Tag subTag = unit.unitTag.childList[i];

			for ( unsigned int n = 0; n < subTag.childList.size(); ++n ) {

				if ( subTag.childList[n].name == "fileName" ) {
					_textMessageWindow->SetSpeakerNameBackImage_wide ( ParseImageFileNameAsAlias (subTag.childList[n].value) );
				}
			}
		}
		else if ( tagName == "speakerName_backImage_offset_x" ) {
			_textMessageWindow->SetSpeakerNameBackImageOffsetX ( Utils_Convert::String_to_Int ( tagValue ) );
		}
		else if ( tagName == "speakerName_backImage_offset_y" ) {
			_textMessageWindow->SetSpeakerNameBackImageOffsetY ( Utils_Convert::String_to_Int ( tagValue ) );
		}
		else if ( tagName == "fontSize" ) {
			_textMessageWindow->SetFontSize ( Utils_Convert::String_to_Int ( tagValue ) );
		}
		else if ( tagName == "rowHeight" ) {
			_textMessageWindow->SetRowHeight ( Utils_Convert::String_to_Int ( tagValue ) );
		}
		else if ( tagName == "cursorSpeed" ) {
			_textMessageWindow->SetCursorSpeed ( Utils_Convert::String_to_Int ( tagValue ) );
		}
		else if ( tagName == "backImage" ) {
			_textMessageWindow->SetBackImage ( ParseImageFileNameAsAlias (tagValue) );
		}
		else if ( tagName == "clickWaitCorsor" ) {
			_textMessageWindow->SetClickWaitCorsor ( tagValue );
		}
		else if ( tagName == "frameOutlineThickness" ) {
			_textMessageWindow->SetFrameOutlineThickness ( Utils_Convert::String_to_Int ( tagValue ) );
		}
		else if ( tagName == "frameFillColor" ) {

			_textMessageWindow->SetFrameFillColor ( SFML_Utils::Convert_string_to_color( tagValue ) );
		}
		else if ( tagName == "frameOutlineColor" ) {

			_textMessageWindow->SetFrameOutlineColor ( SFML_Utils::Convert_string_to_color( tagValue ) );
		}
		else if ( tagName == "fontColor" ) {

			_textMessageWindow->SetFontColor ( SFML_Utils::Convert_string_to_color( tagValue ) );

		} else if ( tagName == "HideProtagonistName" ) {
			TextMessageWindow::HideProtagonistName();
		} else if ( tagName == "ShowProtagonistName" ) {
			TextMessageWindow::ShowProtagonistName();
		} else if ( tagName == "Hide" ) {
			_textMessageWindow->SetShown(false);
		} else if ( tagName == "Show" ) {
			_textMessageWindow->SetShown(true);
		}

	}


	///// デザインIDが指定されている？
	string designID = NotXML::GetAttribute ( unit.unitTag, "designID", "" );

	if ( designID != "" ) {

		printf ("テキストウィンドウのデザインをセーブ [%s]\n", designID.c_str());

		///// 今回の設定内容をそのIDで保存し、「現在のデザインID」とする
		SaveTextWindowDesignAs ( designID );
	}



	// 処理は、これで終了
	_state = ScriptEngine::STATE_CURRENT_UNIT_FINISHED;

}

//=========================================================
// ユニット開始処理：テキストウィンドウのデザインを指定
//=========================================================
void ScriptEngine::UnitStart_TextWindowDesign ( ScriptUnit &unit ) {

	string valueFromAttribute = NotXML::GetAttribute ( unit.unitTag, "id", "" );
	string value              = unit.unitTag.value;
	value = ( valueFromAttribute != "" ) ? valueFromAttribute : value;

	printf ("テキストウィンドウのデザインを指定 [%s]\n", value.c_str());

	///// 指定のIDのデザインをロードし、「現在のデザインID」とする
	LoadTextWindowDesign ( value );

}

//=========================================================
// ユニット開始処理：現在スキップ中のフラグを設定
//=========================================================
void ScriptEngine::UnitStart_Skipping ( ScriptUnit &unit ) {
	
	string value = unit.unitTag.value;
	
	bool arg = Utils_Convert::String_to_bool ( value );
	
	_isSkippingNow = arg;
	
}


//=========================================================
// テキストウィンドウのデザインを保存
//=========================================================
void ScriptEngine::SaveTextWindowDesignAs ( string designID ) {

	// 前提
	if (_textMessageWindow == NULL) {
		return;
	}

	GUI_SFML::TextMessageWindow::Design design;

	design = _textMessageWindow->GetDesign();

	_textMessageWindowDesignList[designID] = design;


	_currentTextWindowDesignID = designID;
}

//=========================================================
// テキストウィンドウのデザインをロード
//=========================================================
void ScriptEngine::LoadTextWindowDesign ( string designID ) {

	// 前提
	if (_textMessageWindow == NULL) {
		return;
	}

	//map<string, GUI_SFML::TextMessageWindow::Design> _textMessageWindowDesignList

	if (_textMessageWindowDesignList.find(designID) == _textMessageWindowDesignList.end() ) {
		printf("テキストウィンドウのデザイン情報[%s]が存在しません。\n", designID.c_str());
		return;
	}

	_textMessageWindow->SetDesign (_textMessageWindowDesignList[designID] );

	_currentTextWindowDesignID = designID;
}


//=========================================================
// ユニット開始処理：テキスト表示
//=========================================================
void ScriptEngine::UnitStart_Text ( ScriptUnit &unit ) {

	if ( _textMessageWindow != NULL ) {

		string text = unit.unitTag.value;


		// 非表示状態だったとしても、表示状態にする。
		_textMessageWindow->SetShown(true);

		// 話者名取得
		string speakerName = NotXML::GetAttribute ( unit.unitTag, "speaker", "" );

		// 話者名欄を使わずに行の冒頭に挿入？
		if ( _textMessageWindow->GetSpeakerNameDisplayType() == GUI_SFML::TextMessageWindow::SPEAKER_NAME_DISPLAY_TYPE_IN ) {

			_textMessageWindow->SetSpeakerName (speakerName);

			string speakFrom = _textMessageWindow->GetSpeakFrom();
			string speakerNameReplacement = speakerName + speakFrom;
			text = Utils_String::Replace ( text, speakFrom, speakerNameReplacement );

		// 話者名欄を使う？
		} else {
			_textMessageWindow->SetSpeakerName (speakerName);
		}





		// このユニットは既読？
		//bool isReadFinished = (_readTextManager->GetReadFinishedLength ( _currentProcessLabel, _processList[_currentProcessLabel]->currentUnitIndex ) == ReadTextManager::ALL_FINISHED );
		bool isReadFinished = (_readTextManager->GetReadFinishedLength ( _currentProcess->label, _currentProcess->currentUnitIndex ) == ReadTextManager::ALL_FINISHED );

		// 既読スキップ発動中？
		bool isSkip = IsSkip();

		// 強制スキップ？
		bool isSkipWhatever = sf::Keyboard::isKeyPressed(sf::Keyboard::Return);
		if ( isSkipWhatever ) {
			isReadFinished = true;
		}

		// 色指定ある？
		string color = NotXML::GetAttribute ( unit.unitTag, "color", "" );
		if (color != "") {

			if ( isReadFinished && isSkip ) {
				_textMessageWindow->AddText_asFinishedText ( text, SFML_Utils::Convert_string_to_color ( color ));
			} else {
				_textMessageWindow->AddText ( text, SFML_Utils::Convert_string_to_color ( color ));
			}

		// 色指定なし
		} else {

			if ( isReadFinished && isSkip ) {
				_textMessageWindow->AddText_asFinishedText ( text );
			} else {
				_textMessageWindow->AddText ( text );
			}
		}

		// 処理の終了を待機してもらう
		_state = ScriptEngine::STATE_WAIT_UNTIL_UNIT_END;
	}
}

//=========================================================
// ユニット開始処理：文字列のエイリアスを登録
//=========================================================
void ScriptEngine::UnitStart_TextAlias_Regist ( ScriptUnit &unit ) {
	
	string multiple = NotXML::GetAttribute ( unit.unitTag, "multiple", "" );
	bool isMultiple = Utils_String::isPositiveWord(multiple);
	
	
	vector<NotXML::Tag> tagList;
	vector<NotXML::Tag> tagList_Protagonist_Regist;
	
	if ( isMultiple ) {
		
		for ( unsigned int i = 0; i < unit.unitTag.childList.size(); ++i ) {
			
			NotXML::Tag childTag = unit.unitTag.childList[i];
			
			///// TextAlias_Registタグであるハズ
			if ( childTag.name == "TextAlias_Regist" ) {
				tagList.push_back ( childTag );
			}
			
			///// Protagonist_Regist も扱うことにする
			if ( childTag.name == "Protagonist_Regist" ) {
				tagList_Protagonist_Regist.push_back ( childTag );
			}
		}
		
	} else {
		
		tagList.push_back(unit.unitTag);
		
	}
	
	//printf ( "テキストエイリアス %d件登録\n", tagList.size());
	
	for ( unsigned int i = 0; i < tagList.size(); ++i ) {
		
		NotXML::Tag textAliasRegistTag = tagList[i];		

		///// 書かれているはずのタグから値を取得
		string alias = NotXML::GetAttribute ( textAliasRegistTag, "alias", "" );
		string value = NotXML::GetAttribute ( textAliasRegistTag, "value", "" );

		///// テキスト欄が存在するならば
		/*
			TODO テキストのエイリアスは「テキスト欄」に設定する？
			理論上、「テキスト欄」は複数持てるのではなかったのか？
			たとえばイベントでCGを表示しているときにはテキスト欄を別の形状（画面全体）などに変更したいかもしれない。
			その都度、テキスト欄を定義しなおすのはいいとしても、
			そのたびに、エイリアスの設定が失われてしまう？

			そもそも、今の実装だと、既読のセーブ用データもテキスト欄を作り直すたびに消えてしまうのではないか？
			そういう「引き継ぎが必要なデータ」はテキスト欄を定義しなおすたびに引き継ぐことにする？
			つまり、テキスト欄を定義するときに既存のものがあれば引き継ぐ、と。
		*/

		///// テキスト欄のオブジェクトがあってもなくても登録する。staticとしてある。
		TextMessageWindow::RegistTextAlias ( alias, value );
	
	}
	
	for ( unsigned int i = 0; i < tagList_Protagonist_Regist.size(); ++i ) {
		
		NotXML::Tag protagonistRegistTag = tagList_Protagonist_Regist[i];
		
		///// 書かれているはずのタグから値を取得
		string name = NotXML::GetAttribute ( protagonistRegistTag, "name", "" );
		
		///// テキスト欄のオブジェクトがあってもなくても登録する。staticとしてある。
		TextMessageWindow::RegistProtagonist( name );
		
	}
	

	// 処理はこれで終了
	_state = ScriptEngine::STATE_CURRENT_UNIT_FINISHED;
}
//=========================================================
// ユニット開始処理：主人公名を登録
//=========================================================
void ScriptEngine::UnitStart_Protagonist_Regist ( ScriptUnit &unit ) {

	///// 書かれているはずのタグから値を取得
	string name = NotXML::GetAttribute ( unit.unitTag, "name", "" );

	///// テキスト欄のオブジェクトがあってもなくても登録する。staticとしてある。
	TextMessageWindow::RegistProtagonist( name );

	// 処理はこれで終了
	_state = ScriptEngine::STATE_CURRENT_UNIT_FINISHED;
}
//=========================================================
// ユニット開始処理：コンソールに文字列を表示。つまりデバッグ目的
//=========================================================
void ScriptEngine::UnitStart_Print ( ScriptUnit &unit ) {

	// 何を表示しますか？
	printf ( "%s\n", unit.unitTag.value.c_str());

	// 処理はこれで終了
	_state = ScriptEngine::STATE_CURRENT_UNIT_FINISHED;
}
//=========================================================
// ユニット開始処理：システムメッセージを表示
//=========================================================
void ScriptEngine::UnitStart_SystemMessage ( ScriptUnit &unit ) {

	if ( unit.unitTag.value != "" ) {
		SystemMessage ( unit.unitTag.value );
	}

	// 処理はこれで終了
	_state = ScriptEngine::STATE_CURRENT_UNIT_FINISHED;
}
//=========================================================
// ユニット開始処理：画像ファイル名のエイリアスを登録
//=========================================================
void ScriptEngine::UnitStart_Image_Regist ( ScriptUnit &unit ) {

	string multiple = NotXML::GetAttribute ( unit.unitTag, "multiple", "" );
	bool isMultiple = Utils_String::isPositiveWord(multiple);
	
	///// 例
	/*
	<Image_Regist fileNameAlias="校門前・朝" file="./image/kimagureAfter/BG14a_80.jpg" />
	<Image_Regist fileNameAlias="校門前・夕方" file="./image/kimagureAfter/BG14b_80.jpg" />
	*/
	
	vector<NotXML::Tag> tagList;
	
	///// 複数指定の場合
	if ( isMultiple ) {
		
		for ( unsigned int i = 0; i < unit.unitTag.childList.size(); ++i ) {
			
			// Image_Regist タグであるハズ。
			NotXML::Tag childTag = unit.unitTag.childList[i];
			
			if ( childTag.name != "Image_Regist" ) {
				continue;
			}
			
			tagList.push_back ( childTag );
		}
		
	///// 単一指定の場合
	} else {
		tagList.push_back ( unit.unitTag );
	}
	
	//printf ("画像エイリアス %d件登録\n", tagList.size());
	
	///// リストとして処理
	for ( unsigned int i = 0; i < tagList.size(); ++i ) {
		
		NotXML::Tag imageRegistTag = tagList[i];
		
		///// 書かれているはずの属性から値を取得
		string alias    = NotXML::GetAttribute ( imageRegistTag, "fileNameAlias", "" );
		string fileName = NotXML::GetAttribute ( imageRegistTag, "file", "" );


		///// 同じ名前のものがあったら警告でも出しておいてあげる
		if ( _imageFileNameAliases.find ( alias ) != _imageFileNameAliases.end() ) {

			printf ( " 画像ファイル名のエイリアス %s(%s) が上書きされて %s(%s) になりました。\n",
				alias.c_str(), _imageFileNameAliases[alias].c_str(),
				alias.c_str(), fileName.c_str()
			);
		}

		// 登録
		_imageFileNameAliases [ alias ] = fileName;
	
	}

	// 処理はこれで終了
	_state = ScriptEngine::STATE_CURRENT_UNIT_FINISHED;
}

//=========================================================
// ユニット開始処理：画像
//=========================================================
void ScriptEngine::UnitStart_Image ( ScriptUnit &unit ) {

	///// Zソートする？
	bool isZSortNeeded = false;

	if ( ImageTagSub (unit.unitTag) ) {
		isZSortNeeded = true;
	}


	/*

	///// ラベルをもとに既存の画像を取得。なければ新規に生成。あればそれに対して操作を行う。
	NotXML::Tag labelTag = NotXML::GetTagFromName ( unit.unitTag, "label" );
	string      label    = labelTag.value;


	Image *image = Image_Get ( label );
	if ( image == NULL ) {

		image = new Image();             // 生成して
		image->SetLabel ( label );       // ラベルを貼って
		_imageList.push_back( image );   // リストに追加
		isZSortNeeded = true;            // あとでZソートしてね
	}

	// 既読スキップ発動中？ → 画像切り替えのエフェクトを無効にする
	bool isSkip = IsSkip();

	for ( unsigned int i = 0; i < unit.unitTag.childList.size(); ++i) {

		NotXML::Tag tag = unit.unitTag.childList[i];

		if ( tag.name == "file" ) {

			string fileName = ParseImageFileNameAsAlias (tag.value);
			string transition = NotXML::GetAttribute ( tag, "transition", Image::CHANGE_IMAGE_EFFECT_NONE );

			// スキップ中はトランジションエフェクトなし。
			if (isSkip) {
				transition = Image::CHANGE_IMAGE_EFFECT_NONE;
			}

			image->ChangeImage ( fileName, transition );
		}
		else if ( tag.name == "copyFrom" ) {

			Image *srcImage = Image_Get ( tag.value );
			if ( srcImage != NULL ) {

				string fileName = srcImage->GetCurrentImageFileName();
				string transition = NotXML::GetAttribute ( tag, "transition", Image::CHANGE_IMAGE_EFFECT_NONE );

				// スキップ中はトランジションエフェクトなし。
				if (isSkip) {
					transition = Image::CHANGE_IMAGE_EFFECT_NONE;
				}

				image->ChangeImage ( fileName, transition );
			}
		}

		else if ( tag.name == "wipeOutMask" ) {
			image->SetWipeOutImage ( ParseImageFileNameAsAlias (tag.value) );
		}
		else if ( tag.name == "wipeInMask" ) {
			image->SetWipeInImage ( ParseImageFileNameAsAlias (tag.value) );
		}
		else if ( tag.name == "wipeOutSpeed" ) {
			image->SetWipeOutSpeed ( Utils_Convert::String_to_Int (tag.value) );
		}
		else if ( tag.name == "wipeInSpeed" ) {
			image->SetWipeInSpeed ( Utils_Convert::String_to_Int (tag.value) );
		}
		else if ( tag.name == "fadeOutSpeed" ) {
			image->SetFadeOutSpeed ( Utils_Convert::String_to_Int (tag.value) );
		}
		else if ( tag.name == "fadeInSpeed" ) {
			image->SetFadeInSpeed ( Utils_Convert::String_to_Int (tag.value) );
		}
		else if ( tag.name == "x" ) {
			image->SetPositionX ( ConvertCoordinateX_from_string (tag.value) );
		}
		else if ( tag.name == "y" ) {
			image->SetPositionY ( ConvertCoordinateY_from_string (tag.value) );
		}
		else if ( tag.name == "z" ) {
			image->SetZ ( Utils_Convert::String_to_Int (tag.value) );
			isZSortNeeded = true;
		}
		else if ( tag.name == "moveToX" ) {
			image->MoveToX ( ConvertCoordinateX_from_string (tag.value) );
		}
		else if ( tag.name == "moveToY" ) {
			image->MoveToY ( ConvertCoordinateY_from_string (tag.value) );
		}
		else if ( tag.name == "moveToX_relative" ) {
			image->MoveToX_Relative ( ConvertCoordinateX_from_string (tag.value) );
		}
		else if ( tag.name == "moveToY_relative" ) {
			image->MoveToY_Relative ( ConvertCoordinateY_from_string (tag.value) );
		}
		else if ( tag.name == "mask" ) {
			image->SetMask ( SFML_Utils::Convert_string_to_color (tag.value) );
		}
		//else if ( tag.name == "reverseX" ) {
		//	image->FlipX();
		//}
		//else if ( tag.name == "reverseY" ) {
		//	image->FlipY();
		//}
		else if ( tag.name == "flipX" ) {
			image->FlipX();
		}
		else if ( tag.name == "flipY" ) {
			image->FlipY();
		}
		else if ( tag.name == "Release" ) {

			Image_Release ( label );

			// 解放してしまったのだから、これでおしまいー。
			Image_ZSort();
			_state = ScriptEngine::STATE_CURRENT_UNIT_FINISHED;

			return;
		}
	}
	*/

	///// Zソートする？
	if ( isZSortNeeded ) {
		Image_ZSort();
	}

	// 処理の終了を待機してもらう？
	_state = ScriptEngine::STATE_WAIT_UNTIL_UNIT_END;
}

//=========================================================
// ユニット開始処理：Imageタグ処理部分
// 戻り値：Zソートする必要が生じたらtrue
//=========================================================
bool ScriptEngine::ImageTagSub ( NotXML::Tag &imageTag ) {

	NotXML::Tag labelTag = NotXML::GetTagFromName ( imageTag, "label" );
	string      label    = labelTag.value;

	///// Zソートする？
	bool isZSortNeeded = false;

	Image *image = Image_Get ( label );
	if ( image == NULL ) {

		image = new Image();             // 生成して
		image->SetLabel ( label );       // ラベルを貼って
		_imageList.push_back( image );   // リストに追加
		isZSortNeeded = true;            // あとでZソートしてね
	}

	// 既読スキップ発動中？ → 画像切り替えのエフェクトを無効にする
	bool isSkip = IsSkip();

	for ( unsigned int i = 0; i < imageTag.childList.size(); ++i) {

		NotXML::Tag tag = imageTag.childList[i];

		if ( tag.name == "file" ) {

			string fileName = ParseImageFileNameAsAlias (tag.value);
			string transition = NotXML::GetAttribute ( tag, "transition", Image::CHANGE_IMAGE_EFFECT_NONE );

			// スキップ中はトランジションエフェクトなし。
			if (isSkip) {
				transition = Image::CHANGE_IMAGE_EFFECT_NONE;
			}

			image->ChangeImage ( fileName, transition );
		}
		else if ( tag.name == "copyFrom" ) {

			Image *srcImage = Image_Get ( tag.value );
			if ( srcImage != NULL ) {

				string fileName = srcImage->GetCurrentImageFileName();
				string transition = NotXML::GetAttribute ( tag, "transition", Image::CHANGE_IMAGE_EFFECT_NONE );

				// スキップ中はトランジションエフェクトなし。
				if (isSkip) {
					transition = Image::CHANGE_IMAGE_EFFECT_NONE;
				}

				image->ChangeImage ( fileName, transition );
			}
		}

		else if ( tag.name == "wipeOutMask" ) {
			image->SetWipeOutImage ( ParseImageFileNameAsAlias (tag.value) );
		}
		else if ( tag.name == "wipeInMask" ) {
			image->SetWipeInImage ( ParseImageFileNameAsAlias (tag.value) );
		}
		else if ( tag.name == "wipeOutSpeed" ) {
			image->SetWipeOutSpeed ( Utils_Convert::String_to_Int (tag.value) );
		}
		else if ( tag.name == "wipeInSpeed" ) {
			image->SetWipeInSpeed ( Utils_Convert::String_to_Int (tag.value) );
		}
		else if ( tag.name == "fadeOutSpeed" ) {

			if ( tag.value == "STANDARD" ) { tag.value = "10"; }

			image->SetFadeOutSpeed ( Utils_Convert::String_to_Int (tag.value) );
		}
		else if ( tag.name == "fadeInSpeed" ) {

			if ( tag.value == "STANDARD" ) { tag.value = "10"; }

			image->SetFadeInSpeed ( Utils_Convert::String_to_Int (tag.value) );
		}
		else if ( tag.name == "x" ) {
			image->SetPositionX ( ConvertCoordinateX_from_string (tag.value) );
		}
		else if ( tag.name == "y" ) {

			if ( tag.value == "CHAR_HIGHT_STANDARD" ) { tag.value = "560"; }

			image->SetPositionY ( ConvertCoordinateY_from_string (tag.value) );
		}
		else if ( tag.name == "z" ) {
			image->SetZ ( Utils_Convert::String_to_Int (tag.value) );
			isZSortNeeded = true;
		}
		else if ( tag.name == "moveToX" ) {
			image->MoveToX ( ConvertCoordinateX_from_string (tag.value) );
		}
		else if ( tag.name == "moveToY" ) {

			if ( tag.value == "CHAR_HIGHT_STANDARD" ) { tag.value = "560"; }

			image->MoveToY ( ConvertCoordinateY_from_string (tag.value) );
		}
		else if ( tag.name == "moveToX_relative" ) {
			image->MoveToX_Relative ( ConvertCoordinateX_from_string (tag.value) );
		}
		else if ( tag.name == "moveToY_relative" ) {
			image->MoveToY_Relative ( ConvertCoordinateY_from_string (tag.value) );
		}
		else if ( tag.name == "moveForm" ) {
			image->SetMoveForm( tag.value );
		}
		else if ( tag.name == "spring_softness" ) {
			image->SetMoveForm_SPRING_softness ( Utils_Convert::String_to_Float (tag.value) );
		}
		else if ( tag.name == "linear_increment" ) {
			image->SetMoveForm_LINEAR_increment ( Utils_Convert::String_to_Float (tag.value) );
		}

		else if ( tag.name == "mask" ) {
			image->SetMask ( SFML_Utils::Convert_string_to_color (tag.value) );
		}
		//else if ( tag.name == "reverseX" ) {
		//	image->FlipX();
		//}
		//else if ( tag.name == "reverseY" ) {
		//	image->FlipY();
		//}
		else if ( tag.name == "flipX" ) {
			image->FlipX();
		}
		else if ( tag.name == "flipY" ) {
			image->FlipY();
		}
		else if ( tag.name == "Release" ) {

			Image_Release ( label );

			// 解放してしまったのだから、これでおしまいー。
			Image_ZSort();
			_state = ScriptEngine::STATE_CURRENT_UNIT_FINISHED;

			return isZSortNeeded;
		}
	}

	return isZSortNeeded;

}
//=========================================================
// ユニット開始処理：画像集
//=========================================================
void ScriptEngine::UnitStart_ImageGroup ( ScriptUnit &unit ) {

	///// ラベルをもとに既存の画像を取得。なければ新規に生成。あればそれに対して操作を行う。

	vector<NotXML::Tag> imageTagList = NotXML::GetTagListFromName( unit.unitTag, SCRIPT::UNIT_TYPE::IMAGE );

	bool isZSortNeeded = false;

	for ( unsigned int i = 0 ; i < imageTagList.size(); ++i ) {

		if (ImageTagSub (imageTagList[i])) {
			isZSortNeeded = true;
		}
	}

	///// Zソートする？
	if ( isZSortNeeded ) {
		Image_ZSort();
	}

	// 処理の終了を待機してもらう？
	_state = ScriptEngine::STATE_WAIT_UNTIL_UNIT_END;
}



//=========================================================
// 座標を表す文字列の末尾は '%' かもしれないのでした：X編
//=========================================================
int ScriptEngine::ConvertCoordinateX_from_string ( string coordinate ) {

	int x = 0;

	if ( coordinate == "MIDDLE" ) {
		x = Application::GetInstance()->GetScreenWidth() / 2;

	} else if ( coordinate != "" && coordinate[ coordinate.size() - 1 ] == '%' ) {

		string percentageStr = coordinate.substr ( 0, coordinate.size() - 1 );
		int percentage = std::stoi(percentageStr);
		x = Application::GetInstance()->GetScreenWidth() * percentage / 100;

	} else {
		x = std::stoi ( coordinate );
	}

	return x;
}
//=========================================================
// 座標を表す文字列の末尾は '%' かもしれないのでした：Y編
//=========================================================
int ScriptEngine::ConvertCoordinateY_from_string ( string coordinate ) {

	int y = 0;

	if ( coordinate == "MIDDLE" ) {
		y = Application::GetInstance()->GetScreenHeight() / 2;

	} else if ( coordinate != "" && coordinate[ coordinate.size() - 1 ] == '%' ) {

		string percentageStr = coordinate.substr ( 0, coordinate.size() - 1 );
		int percentage = std::stoi(percentageStr);
		y = Application::GetInstance()->GetScreenHeight() * percentage / 100;

	} else {
		y = std::stoi ( coordinate );
	}

	return y;
}

//=========================================================
// ファイル名をエイリアスとして解釈してみる
//=========================================================
string ScriptEngine::ParseImageFileNameAsAlias ( string maybeAlias ) {

	string result = maybeAlias;

	// エイリアスがあれば、置換する：なければそのまま
	if ( _imageFileNameAliases.find ( maybeAlias ) != _imageFileNameAliases.end() ) {

		result = _imageFileNameAliases [ maybeAlias ];
	}

	return result;
}

//=========================================================
// ユニット開始処理：ボタン
//=========================================================
void ScriptEngine::UnitStart_Button ( ScriptUnit &unit ) {

	///// 名前をもとに既存のボタンを取得。なければ新規に生成。あればそれに対して操作を行う。
	NotXML::Tag nameTag = NotXML::GetTagFromName ( unit.unitTag, "name" );
	string      name    = nameTag.value;

	ScriptButton *button = Button_Get ( name );
	if ( button == NULL ) {

		// 新規作成なのだから、あるはずのもの。
		NotXML::Tag subRutinName = NotXML::GetTagFromName ( unit.unitTag, "jumpToLabel" );
		NotXML::Tag label        = NotXML::GetTagFromName ( unit.unitTag, "label" );

		// 生成して,
		button = new ScriptButton ( label.value, sf::Vector2f(-1, -1), 16 );

		// ラベルを貼って
		button->SetName ( name );

		// リストに追加
		_scriptButtonList [ name ] = button;
	}

	///// 新規じゃなくても、あるはずのもの。
	for ( unsigned int i = 0; i < unit.unitTag.childList.size(); ++i) {

		NotXML::Tag tag = unit.unitTag.childList[i];

		///// 位置
		if ( tag.name == "Position_X" ) {
			button->SetPositionX ( Utils_Convert::String_to_Float( tag.value ) );
		}
		else if ( tag.name == "Position_Y" ) {
			button->SetPositionY ( Utils_Convert::String_to_Float( tag.value ) );
		}

		///// 表示/非表示
		else if ( tag.name == "Visible" ) {
			button->SetVisible ( Utils_Convert::String_to_bool( tag.value ) );
		}
		///// 呼び出すサブルーチンを変えたりもするの？
		else if ( tag.name == "jumpToLabel" ) {
			button->SetJumpToLabel ( tag.value );
		}
	}


	// 処理終了
	_state = ScriptEngine::STATE_CURRENT_UNIT_FINISHED;
}

//=========================================================
// ユニット開始処理：フェード
//=========================================================
void ScriptEngine::UnitStart_Fade ( ScriptUnit &unit ) {

	///// あるはずのもの：操作は何かな？
	string operation = NotXML::GetTagFromName ( unit.unitTag, "Operation" ).value;
	
	
	bool isWaitUntilFinish = true;

	///// プロパティを変更するだけ
	if ( operation == "SetProperty" ) {

		for ( unsigned int i = 0; i < unit.unitTag.childList.size(); ++i) {

			NotXML::Tag tag = unit.unitTag.childList[i];

			///// テクスチャをセット
			if ( tag.name == "Texture" ) {
				_fade->SetFadeTexture ( tag.value );

			///// フェードアウト開始時点の色を指定
			} else if ( tag.name == "Color_FadeOutStart" ) {
				_fade->SetColor_FadeOutStart ( GUI_SFML::SFML_Utils::Convert_string_to_color ( tag.value ) );

			///// フェードイン開始時の色を指定
			} else if ( tag.name == "Color_FadeInStart" ) {
				_fade->SetColor_FadeInStart ( GUI_SFML::SFML_Utils::Convert_string_to_color ( tag.value ) );
			}
		}

		// 処理終了
		_state = ScriptEngine::STATE_CURRENT_UNIT_FINISHED;
		return;


	///// フェードアウト
	} else if ( operation == "OUT" ) {

		///// あるはずのもの
		int speed = 1;

		for ( unsigned int i = 0; i < unit.unitTag.childList.size(); ++i) {

			NotXML::Tag tag = unit.unitTag.childList[i];

			///// 速度
			if ( tag.name == "Speed" ) {
				speed = Utils_Convert::String_to_Int ( tag.value );
				
				isWaitUntilFinish = Utils_Convert::String_to_bool ( NotXML::GetAttribute ( tag, "wait", "on" ) );

			}
		}

		_fade->FadeOut ( speed );

	///// フェードイン
	} else if ( operation == "IN" ) {

		///// あるはずのもの
		int speed = 1;

		for ( unsigned int i = 0; i < unit.unitTag.childList.size(); ++i) {

			NotXML::Tag tag = unit.unitTag.childList[i];

			///// 速度
			if ( tag.name == "Speed" ) {
				speed = Utils_Convert::String_to_Int ( tag.value );

				isWaitUntilFinish = Utils_Convert::String_to_bool ( NotXML::GetAttribute ( tag, "wait", "on" ) );

			}
		}

		_fade->FadeIn ( speed );
	}
	
	
	
	
	if ( isWaitUntilFinish ) {
	
		// 処理するよ
		_state = ScriptEngine::STATE_WAIT_UNTIL_UNIT_END;
	
	} else {
	
		_state = ScriptEngine::STATE_CURRENT_UNIT_FINISHED;
	
	}
}

//=========================================================
// ユニット処理：フェード
//=========================================================
void ScriptEngine::UnitProcessing_Fade ( ScriptUnit &unit ) {

	///// フェードの動きが停止していたら、終了
	if ( _fade->IsStabilized() ) {
		_state = ScriptEngine::STATE_CURRENT_UNIT_FINISHED;
	}
}

//=========================================================
// ユニット開始処理：カメラ
//=========================================================
void ScriptEngine::UnitStart_Camera ( ScriptUnit &unit ) {

	// 最初の子タグだけ有効とする
	vector<NotXML::Tag> tagList = unit.unitTag.childList;

	// 何も書いてない？ それはおかしい！
	if ( tagList.size() <= 0) {
		_state = ScriptEngine::STATE_CURRENT_UNIT_FINISHED;
		return;
	}

	// ご指定は何ですか？
	NotXML::Tag tag = tagList[0];

	if ( tag.name == "Move" ) {

		int x = std::stoi ( NotXML::GetAttribute ( tag, "x", "0" ));
		int y = std::stoi ( NotXML::GetAttribute ( tag, "y", "0" ));

		_camera->Move ( x, y );

		_state = ScriptEngine::STATE_CURRENT_UNIT_FINISHED;

	}
	else if ( tag.name == "MoveTo" ) {

		int x       = std::stoi ( NotXML::GetAttribute ( tag, "x"    , "0" ));
		int y       = std::stoi ( NotXML::GetAttribute ( tag, "y"    , "0" ));
		int speed   = std::stoi ( NotXML::GetAttribute ( tag, "speed", "1" ));
		string wait = NotXML::GetAttribute ( tag, "wait" , "true" );

		_camera->MoveTo( x, y, speed );

		// 終了を待機するかどうか
		if ( Utils_String::isPositiveWord( wait ) && !IsSkip() ) {

			_state = ScriptEngine::STATE_WAIT_UNTIL_UNIT_END;

		} else {

			_state = ScriptEngine::STATE_CURRENT_UNIT_FINISHED;

		}
	}
	else if ( tag.name == "Position" ) {

		int x = std::stoi ( NotXML::GetAttribute ( tag, "x", "0" ));
		int y = std::stoi ( NotXML::GetAttribute ( tag, "y", "0" ));

		_camera->SetPosition( x, y );

		_state = ScriptEngine::STATE_CURRENT_UNIT_FINISHED;
	}
	else if ( tag.name == "Reset" ) {

		_camera->Reset();

		_state = ScriptEngine::STATE_CURRENT_UNIT_FINISHED;
	}


}

//=========================================================
// ユニット進行処理：カメラ
//=========================================================
void ScriptEngine::UnitProcessing_Camera ( ScriptUnit &unit ) {

	if ( _camera->IsActionEnd() ) {
		_state = ScriptEngine::STATE_CURRENT_UNIT_FINISHED;

	}
}

//=========================================================
// ユニット開始処理：画面ゆれ
//=========================================================
void ScriptEngine::UnitStart_Shake ( ScriptUnit &unit ) {

	///// あるはずのもの：操作は何かな？
	string magnitude  = NotXML::GetAttribute ( unit.unitTag, "magnitude", "30" );
	string frameCount = NotXML::GetAttribute ( unit.unitTag, "frameCount", "60" );
	string interval   = NotXML::GetAttribute ( unit.unitTag, "interval", "1" );
	string wait       = NotXML::GetAttribute ( unit.unitTag, "wait", "true" );

	Effect_Shake::Property prop;

	sf::RenderWindow *window = &(Application::GetInstance()->_window);
	prop.window = window;
	prop.magnitude   = std::stoi ( magnitude );
	prop.interval    = std::stoi ( interval );
	prop.frameCount  = std::stoi ( frameCount );
	prop.camera      = _camera;
	prop.textWindow  = _textMessageWindow;

	// 画面ゆれが終了するまで待機する？ メッセージスキップ中のときは待機しない
	if ( Utils_String::isPositiveWord ( wait ) && !IsSkip() ) {

		_isShakeEnd = "false";

		prop.isShakeEnd = &_isShakeEnd;

		Effect_Shake *effect = new Effect_Shake(prop);
		Application::GetInstance()->_effectManager->Add(effect);

		// 待機する
		_state = ScriptEngine::STATE_WAIT_UNTIL_UNIT_END;

	} else {

		prop.isShakeEnd = NULL;

		Effect_Shake *effect = new Effect_Shake(prop);
		Application::GetInstance()->_effectManager->Add(effect);

		// 待機せずに次のタグへ進む
		_state = ScriptEngine::STATE_CURRENT_UNIT_FINISHED;
	}
}

//=========================================================
// ユニット進行処理：画面ゆれ
//=========================================================
void ScriptEngine::UnitProcessing_Shake ( ScriptUnit &unit ) {

	if ( _isShakeEnd == "true" ) {
		_state = ScriptEngine::STATE_CURRENT_UNIT_FINISHED;

	}
}

//=========================================================
// ユニット開始処理：BGM登録
//=========================================================
void ScriptEngine::UnitStart_BGM_Regist ( ScriptUnit &unit ) {
	
	string multiple = NotXML::GetAttribute ( unit.unitTag, "multiple", "" );
	bool isMultiple = Utils_String::isPositiveWord(multiple);
	
	
	vector<NotXML::Tag> tagList;
	
	///// 複数指定の場合
	if ( isMultiple ) {
		
		for ( unsigned int i = 0; i < unit.unitTag.childList.size(); ++i ) {
			
			// BGM_Regist タグであるハズ。
			NotXML::Tag childTag = unit.unitTag.childList[i];
			
			if ( childTag.name != "BGM_Regist" ) {
				continue;
			}
			
			tagList.push_back ( childTag );
		}
		
	///// 単一指定の場合
	} else {
		tagList.push_back ( unit.unitTag );
	}
	
	
	for ( unsigned int i = 0; i < tagList.size(); ++i ) {
		
		NotXML::Tag bgmRegistTag = tagList[i];
		

		// 属性ではなく タグで取得していたときの処理
		///// 書かれているはずのタグから値を取得
		//string fileName = NotXML::GetTagFromName ( unit.unitTag, "File" ).value;
		//string label    = NotXML::GetTagFromName ( unit.unitTag, "Label" ).value;

		///// 書かれているはずの属性から値を取得
		string fileName = NotXML::GetAttribute ( bgmRegistTag, "file", "" );
		string label    = NotXML::GetAttribute ( bgmRegistTag, "bgmLabel", "" );

		// 登録。この時点では、バッファに読み込まれない。
		_BGM->Regist ( fileName, label );
		
	}
	// 処理はこれで終了
	_state = ScriptEngine::STATE_CURRENT_UNIT_FINISHED;
}
//=========================================================
// ユニット開始処理：BGM変更
//=========================================================
void ScriptEngine::UnitStart_BGM_Change ( ScriptUnit &unit ) {

	///// エフェクト指定は何かな？
	string transitionEffect = NotXML::GetAttribute ( unit.unitTag, "transition", "" );

	///// 書かれているはずのタグから値を取得
	string label    = NotXML::GetTagFromName ( unit.unitTag, "Label" ).value;

	// 演奏。
	_BGM->Change ( label, transitionEffect );

	/*
		するってぇと、_BGM->Play() は、結局使わないわけですか？
		そんな気がしてきました。予感どおり。
	*/

	// 処理はこれで終了
	_state = ScriptEngine::STATE_CURRENT_UNIT_FINISHED;
}
//=========================================================
// ユニット開始処理：BGM設定   BGM_Config
//=========================================================
void ScriptEngine::UnitStart_BGM_Config ( ScriptUnit &unit ) {

	///// ピッチ：属性で取得
	string pitchStr = NotXML::GetAttribute( unit.unitTag, "pitch", "" );

	///// ピッチ：子タグがあれば見る
	if ( NotXML::IsExist ( unit.unitTag, "Pitch" ) ) {
		pitchStr = NotXML::GetTagFromName ( unit.unitTag, "Pitch" ).value;
	}

	///// ピッチ、指定されてたら設定する
	if ( pitchStr != "" ) {

		float pitch = std::stof(pitchStr);
		_BGM->SetPitch ( pitch );
	}


	///// ループ：属性で取得
	string loop = NotXML::GetAttribute( unit.unitTag, "loop", "" );

	///// ループ：子タグがあれば見る
	if ( NotXML::IsExist ( unit.unitTag, "Loop" ) ) {
		loop = NotXML::GetTagFromName ( unit.unitTag, "Loop" ).value;
	}

	///// ループ、指定されてたら設定する
	if ( loop != "" ) {

		bool isLoop = Utils_Convert::String_to_bool(loop);
		_BGM->SetLoop( isLoop );
	}

	///// ピッチを設定？
	/*
	if ( NotXML::IsExist ( unit.unitTag, "Pitch" ) ) {

		string pitchStr = NotXML::GetTagFromName ( unit.unitTag, "Pitch" ).value;

		float pitch = std::stof(pitchStr);

		_BGM->SetPitch ( pitch );
	}
	*/

	// 処理はこれで終了
	_state = ScriptEngine::STATE_CURRENT_UNIT_FINISHED;
}


//=========================================================
// ユニット開始処理：効果音ラベルを登録
//=========================================================
void ScriptEngine::UnitStart_Sound_Regist ( ScriptUnit &unit ) {

	string multiple = NotXML::GetAttribute ( unit.unitTag, "multiple", "" );
	bool isMultiple = Utils_String::isPositiveWord(multiple);
	
	
	vector<NotXML::Tag> tagList;
	
	///// 複数指定の場合
	if ( isMultiple ) {
		
		for ( unsigned int i = 0; i < unit.unitTag.childList.size(); ++i ) {
			
			// Sound_Regist タグであるハズ。
			NotXML::Tag childTag = unit.unitTag.childList[i];
			
			if ( childTag.name != "Sound_Regist" ) {
				continue;
			}
			
			tagList.push_back ( childTag );
		}
		
	///// 単一指定の場合
	} else {
		tagList.push_back ( unit.unitTag );
	}
	
	
	for ( unsigned int i = 0; i < tagList.size(); ++i ) {
		
		NotXML::Tag soundRegistTag = tagList[i];

		///// エフェクト指定は何かな？
		string file       = NotXML::GetAttribute ( soundRegistTag, "file", "" );
		string soundLabel = NotXML::GetAttribute ( soundRegistTag, "soundLabel", "" );

		_soundManager->Regist ( soundLabel, file );
	}


	// 処理はこれで終了
	_state = ScriptEngine::STATE_CURRENT_UNIT_FINISHED;
}

//=========================================================
// ユニット開始処理：効果音
//=========================================================
void ScriptEngine::UnitStart_Sound ( ScriptUnit &unit ) {

	///// 書かれているはずのものを取得
	string file       = NotXML::GetAttribute ( unit.unitTag, "file", "" );
	string soundLabel = NotXML::GetAttribute ( unit.unitTag, "soundLabel", "" );
	bool isPreload = Utils_Convert::String_to_bool ( NotXML::GetAttribute ( unit.unitTag, "preload", "false") );

	///// ラベルで指定されている？
	if ( soundLabel != "" ) {

		_soundManager->Play_FromLabel ( soundLabel, isPreload  );
	}
	///// ファイル名で指定されている？
	if ( file != "" ) {

		_soundManager->Play ( file, isPreload  );
	}
	/*
		両方なんて指定しないでー。
	*/


	// 処理はこれで終了
	_state = ScriptEngine::STATE_CURRENT_UNIT_FINISHED;
}

//=========================================================
// ユニット開始処理：選択肢
//=========================================================
void ScriptEngine::UnitStart_Select ( ScriptUnit &unit ) {

	///// Optionタグのリストがあるはずなのです！
	vector<SelectOption*> selectList;

	vector<NotXML::Tag> optionTagList = NotXML::GetTagListFromName ( unit.unitTag, "Option");

	for ( unsigned int i = 0; i < optionTagList.size(); ++i ) {

		NotXML::Tag optionTag = unit.unitTag.childList[i];

		bool isShown = true;

		// 条件つき？
		string condition = NotXML::GetAttribute(optionTag, "onlyShownIf", "");
		if ( condition != "" ) {

			//printf("選択表示条件：%s\n", condition.c_str());

			isShown = false;

			Utils_String::BinaryOperationElements boe = Utils_String::ParseAsBinaryOperation(condition);

			// 右辺が変数の場合
			// ${} で囲まれてたら変数
			if
			(
				boe.right.length() > 3 &&
				boe.right.substr(0, 2) == "${" &&
				boe.right.substr( boe.right.size()-1 , 1) == "}"
			)
			{
				//right = boe.right.substr( 2, boe.right.size()-3 );

				if ( IsInt( boe.left) ) {

					int left  = GetIntVariable(boe.left);
					int right = GetIntVariable(boe.right);

					isShown = Utils_String::ExecuteRelationalExpression ( left, boe.op, right);


				} else if ( IsString( boe.left) ) {

					string left  = GetStringVariable(boe.left);
					string right = GetStringVariable(boe.right);

					isShown = Utils_String::ExecuteRelationalExpression ( left, boe.op, right);

				} else {
					printf("左辺 %s は 数値でも文字列でもない。\n", boe.left.c_str());
				}

			// 右辺がリテラルの場合
			} else {
				//right = boe.right;

				if ( IsInt( boe.left) ) {

					int left  = GetIntVariable(boe.left);
					int right = std::stoi(boe.right);

					isShown = Utils_String::ExecuteRelationalExpression ( left, boe.op, right);


				} else if ( IsString( boe.left) ) {

					string left  = GetStringVariable(boe.left);
					string right = boe.right;

					isShown = Utils_String::ExecuteRelationalExpression ( left, boe.op, right);

				} else {
					printf("左辺 %s は 数値でも文字列でもない。\n", boe.left.c_str());
				}
			}
		}

		if ( isShown ) {
			// Text タグ と To タグ が書かれていると信じてる。
			string text        = NotXML::GetTagFromName ( optionTag, "Text" ).value;
			string jumpToLabel = NotXML::GetTagFromName ( optionTag, "To" ).value;

			TextMessageWindow::ReplaceWithAlias(text);

			// じゃあ、そのつもりで。
			SelectOption *option = new SelectOption ( text, jumpToLabel );

			// Moveタグがあったりもする（ない場合は Tag.nameが空文字）
			NotXML::Tag moveTag = NotXML::GetTagFromName ( optionTag, "Move" );

			//option->SetMoveTag ( moveTag );
			AI_Modules::Move mv = Game_Utils::Move_From_Tag(moveTag);
			option->SetMove ( mv );


			selectList.push_back ( option );
		}
	}

	// ゲーム盤への紐付けをしますか？ 次の一手問題専用っぽい
	string gameBoardId = NotXML::GetAttribute( unit.unitTag, "gameBoardId", "");
	_select->SetGameBoardId ( gameBoardId );


    // 選択肢の位置
	_select->Move ( 0, 0 ); // 位置をクリア
	string optionPosData = NotXML::GetAttribute( unit.unitTag, "optionPosition", "0,0");
    vector<string> optionPosDataParsed = Utils_String::Split ( optionPosData, "," );
    int optionPosX = ( optionPosDataParsed.size() >= 2 ) ?  stoi(optionPosDataParsed[0]) : 0;
    int optionPosY = ( optionPosDataParsed.size() >= 2 ) ?  stoi(optionPosDataParsed[1]) : 0;

	// おねがいします。
	_select->Clear(); // 一応。
	
	//delete _select;
	//_select = new Select();
	
	_select->SetUpSelectOptions( selectList );

    // 位置移動は選択肢が設定された後、ってことですよね。
	_select->Move ( optionPosX, optionPosY );

	// 選択されるまで、待機なのです。
	_state = ScriptEngine::STATE_WAIT_UNTIL_UNIT_END;


	// こういうこともありえるよね！
	if ( selectList.size() == 0 ) {

		_state = ScriptEngine::STATE_CURRENT_UNIT_FINISHED;
	}

}

//=========================================================
// ユニット開始処理：ファイル選択さんをお呼びする
//=========================================================
void ScriptEngine::UnitStart_FileSelect ( ScriptUnit &unit ) {

	///// あるはずの子要素を受けとりまして、
	NotXML::Tag         folderTag     = NotXML::GetTagFromName     ( unit.unitTag, "Folder" );
	vector<NotXML::Tag> filterTagList = NotXML::GetTagListFromName ( unit.unitTag, "Filter" );
	NotXML::Tag         variableTag   = NotXML::GetTagFromName     ( unit.unitTag, "Variable" );
	bool isPreviousPositionToUse = Utils_Convert::String_to_bool ( NotXML::GetAttribute ( NotXML::GetTagFromName(unit.unitTag, "Folder"), "isPreviousPositionToUse", "off" ) );
	

	///// フィルター用の文字列を作成するよ！
	string filter = "";
	for ( NotXML::Tag filterTag : filterTagList ) {

		if ( filter != "" ) { filter += ","; }
		filter += filterTag.value;

	}

	///// 開くのですね？
	string folder = folderTag.value;
	
	if (_fileExplorer != NULL) {
		delete _fileExplorer;
	}
	_fileExplorer = new FileExplorer();
	_fileExplorer->ReadFolder ( isPreviousPositionToUse, folder, filter );


	///// 受け取る変数名を覚えておく
	_fileSelectVariableName = variableTag.value;


	// 選択されるまで、待機なのです。
	_state = ScriptEngine::STATE_WAIT_UNTIL_UNIT_END;

}
//=========================================================
// ユニット開始処理：テキストラベルに関する操作
//=========================================================
void ScriptEngine::UnitStart_TextLabel ( ScriptUnit &unit ) {

	///// まずはお名前を教えてください。
	string name = "";

	if ( NotXML::IsExist( unit.unitTag, "Name" ) ) {
		name = NotXML::GetTagFromName     ( unit.unitTag, "Name" ).value;
	}
	else if ( NotXML::IsExist( unit.unitTag, "Label" ) ) {
		name = NotXML::GetTagFromName     ( unit.unitTag, "Label" ).value;
	}
	if ( name == "" ) {
		// 処理はこれで終了
		_state = ScriptEngine::STATE_CURRENT_UNIT_FINISHED;
		return;
	}

	///// 解放でしたか？
	if ( NotXML::IsExist( unit.unitTag, "Release" ) ) {

		TextLabel_Release ( name );

		// 処理はこれで終了
		_state = ScriptEngine::STATE_CURRENT_UNIT_FINISHED;
		return;
	}


	///// リストから取得。なければ生成して登録
	TextLabel *textLabel = TextLabel_Get ( name );

	if ( textLabel == NULL ) {
		textLabel = new TextLabel(name);
		_textLabelList[name] = textLabel;

		textLabel->SetFont(_font);
	}

	///// 文字列を設定
	if ( NotXML::IsExist( unit.unitTag, "Text" ) ) {

		string text = NotXML::GetTagFromName ( unit.unitTag, "Text" ).value;

		// おおっと、黒魔術か？！：不思議なパワァでエイリアスが置換されるぞ！
		TextMessageWindow::ReplaceWithAlias ( text );

		textLabel->SetText(text);
	}

	///// その他設定あれこれ
	if ( NotXML::IsExist( unit.unitTag, "x" ) ) {

		sf::Vector2f pos = textLabel->GetPosition();
		pos.x = std::stoi(NotXML::GetTagFromName ( unit.unitTag, "x" ).value);
		textLabel->SetPosition(pos);
	}
	if ( NotXML::IsExist( unit.unitTag, "y" ) ) {
		sf::Vector2f pos = textLabel->GetPosition();
		pos.y = std::stoi(NotXML::GetTagFromName ( unit.unitTag, "y" ).value);
		textLabel->SetPosition(pos);
	}
	if ( NotXML::IsExist( unit.unitTag, "Color" ) ) {
		textLabel->SetColor(NotXML::GetTagFromName ( unit.unitTag, "Color" ).value);
	}
	if ( NotXML::IsExist( unit.unitTag, "Size" ) ) {
		textLabel->SetSize( std::stoi ( NotXML::GetTagFromName ( unit.unitTag, "Size" ).value) );
	}

	// 処理はこれで終了
	_state = ScriptEngine::STATE_CURRENT_UNIT_FINISHED;
}


//=========================================================
// ユニット開始処理：変数を宣言
//=========================================================
void ScriptEngine::UnitStart_Variable_Define ( ScriptUnit &unit ) {
	
	string multiple = NotXML::GetAttribute ( unit.unitTag, "multiple", "" );
	bool isMultiple = Utils_String::isPositiveWord(multiple);
	

	vector<NotXML::Tag> tagList;
	
	if ( isMultiple ) {
		
		for ( unsigned int i = 0; i < unit.unitTag.childList.size(); ++i ) {
			
			NotXML::Tag childTag = unit.unitTag.childList[i];
			
			if ( childTag.name != "Variable_Define" ) {
				continue;
			}
			
			tagList.push_back(childTag);
			
		}
		
	} else {
		
		tagList.push_back(unit.unitTag);
		
	}
	
	
	for ( unsigned int i = 0; i < tagList.size(); ++i ) {
		
		NotXML::Tag variableDefineTag = tagList[i];
		//NotXML::Tag variableDefineTag = unit.unitTag;
	


		///// 書かれているはずのものを取得
		string scope = NotXML::GetAttribute ( variableDefineTag, "scope", "" );
		string name  = NotXML::GetAttribute ( variableDefineTag, "name", "" );
		string type  = NotXML::GetAttribute ( variableDefineTag, "type", "" );

		if ( scope != "global" && scope != "local") {
			printf ("Warning : 変数宣言。scope(%s) が不正です。\n", scope.c_str());
			continue;
		}
		if ( type != "string" && type != "int") {
			printf ("Warning : 変数宣言。type(%s) が不正です。\n", type.c_str());
			continue;
		}
		if ( name == "") {
			printf ("Warning : 変数宣言。name が未指定です。\n");
			continue;
		}

		///// 同じ名前の変数があってはいけないのです。
		if
		(
			_variable_Global_Int   .find ( name ) != _variable_Global_Int   .end() ||
			_variable_Global_String.find ( name ) != _variable_Global_String.end() ||
			_variable_Local_Int    .find ( name ) != _variable_Local_Int    .end() ||
			_variable_Local_String .find ( name ) != _variable_Local_String .end()
		)
		{
			printf ( "Information：同じ名前の変数が既に定義されています。(%s)\n", name.c_str() );
			continue;
		}


		///// セーブデータ間で共有する変数
		if ( scope == "global" ) {

			///// 数値
			if ( type == "int" ) {

				_variable_Global_Int[name] = 0;

			///// 文字列
			} else if ( type == "string" ) {

				_variable_Global_String[name] = "";
			}

		///// セーブデータ固有の変数
		} else if ( scope == "local" ) {

			///// 数値
			if ( type == "int" ) {

				_variable_Local_Int[name] = 0;

			///// 文字列
			} else if ( type == "string" ) {

				_variable_Local_String[name] = "";
			}
		}
	}

	// 処理終了
	_state = ScriptEngine::STATE_CURRENT_UNIT_FINISHED;
}

//=========================================================
// ユニット開始処理：変数を解放
//=========================================================
void ScriptEngine::UnitStart_Variable_Release ( ScriptUnit &unit ) {

	///// どの道こうなる、ということで。
	_state = ScriptEngine::STATE_CURRENT_UNIT_FINISHED;

	///// 書かれているはずのものを取得
	string scope = NotXML::GetAttribute ( unit.unitTag, "scope", "" );
	string name  = NotXML::GetAttribute ( unit.unitTag, "name", "" );
	string type  = NotXML::GetAttribute ( unit.unitTag, "type", "" );

	if ( scope != "global" && scope != "local") {
		printf ("Warning : 変数宣言。scope(%s) が不正です。\n", scope.c_str());
		return;
	}
	if ( type != "string" && type != "int") {
		printf ("Warning : 変数宣言。type(%s) が不正です。\n", type.c_str());
		return;
	}
	if ( name == "") {
		printf ("Warning : 変数宣言。name が未指定です。\n");
		return;
	}


	///// セーブデータ間で共有する変数
	if ( scope == "global" ) {

		///// 数値
		if ( type == "int" ) {

			if ( _variable_Global_Int.find ( name ) == _variable_Global_Int.end() ) {
				printf("指定のglobal変数 %s は存在しません。", name.c_str());

			} else {

				_variable_Global_Int.erase ( _variable_Global_Int.find ( name ) );
			}


		///// 文字列
		} else if ( type == "string" ) {

			if ( _variable_Global_String.find ( name ) == _variable_Global_String.end() ) {
				printf("指定のglobal変数 %s は存在しません。", name.c_str());

			} else {

				_variable_Global_String.erase ( _variable_Global_String.find ( name ) );
			}
		}

	///// セーブデータ固有の変数
	} else if ( scope == "local" ) {

		///// 数値
		if ( type == "int" ) {

			if ( _variable_Local_Int.find ( name ) == _variable_Local_Int.end() ) {
				printf("指定のlocal変数 %s は存在しません。", name.c_str());

			} else {

				_variable_Local_Int.erase ( _variable_Local_Int.find ( name ) );
			}

		///// 文字列
		} else if ( type == "string" ) {

			if ( _variable_Local_String.find ( name ) == _variable_Local_String.end() ) {
				printf("指定のlocal変数 %s は存在しません。", name.c_str());

			} else {

				_variable_Local_String.erase ( _variable_Local_String.find ( name ) );
			}
		}
	}

	// 処理終了
	_state = ScriptEngine::STATE_CURRENT_UNIT_FINISHED;
}

//=========================================================
// ユニット開始処理：変数を操作
//=========================================================
void ScriptEngine::UnitStart_Variable_Operation ( ScriptUnit &unit ) {

	///// 書かれているはずのものを取得
	string operatorType = NotXML::GetAttribute ( unit.unitTag, "operator",    "" );
	string left         = NotXML::GetAttribute ( unit.unitTag, "left",        "" );
	string right_v      = NotXML::GetAttribute ( unit.unitTag, "right_v",     "" );
	string right_l      = NotXML::GetAttribute ( unit.unitTag, "right_l",     "" );
	string expression   = NotXML::GetAttribute ( unit.unitTag, "expression",  "" );

	///// エラーチェック？
	/*
		ちゃんと使ってよ。
	*/

	///// 式で指定？
	if ( expression != "" ) {

		Utils_String::BinaryOperationElements boe = Utils_String::ParseAsBinaryOperation(expression);

		left = boe.left;
		operatorType = boe.op;

		// ${} で囲まれてたら変数
		if
		(
			boe.right.length() > 3 &&
			boe.right.substr(0, 2) == "${" &&
			boe.right.substr( boe.right.size()-1 , 1) == "}"
		)
		{

			right_v = boe.right.substr( 2, boe.right.size()-3 );
			right_l = "";

		// そうじゃなければリテラル
		} else {
			right_v = "";
			right_l = boe.right;
		}
	}


	///// 左辺は、数値？
	if ( IsInt ( left ) ) {

		int result     = 0;
		int leftValue  = GetIntVariable ( left );
		int rightValue = 0;

		///// 右辺は変数でご指名でございます。
		if ( right_v != "" ) {

			///// 当然、数値変数をご指名ですよね？
			rightValue = GetIntVariable ( right_v );

		///// 右辺はリテラルでご指定であります。
		} else if ( right_l != "" ) {

			///// 当然、数値になりえるモノを書いてくれてますよね？
			rightValue = Utils_Convert::String_to_Int ( right_l );
		}


		///// おおせのままに加減乗除
		if      ( operatorType == "=" ) { result = rightValue; }
		else if ( operatorType == "+" ) { result = leftValue + rightValue; }
		else if ( operatorType == "-" ) { result = leftValue - rightValue; }
		else if ( operatorType == "*" ) { result = leftValue * rightValue; }
		else if ( operatorType == "/" ) {

			if ( rightValue != 0 ) {
				result = leftValue / rightValue;
			} else {
				printf ("Error : 0 で 割れません。\n" );
			}
		}

		///// 結果を、左辺で指定された変数に代入。エラーなんて出てませんよね？
		SetIntValue ( left, result );


	///// 左辺は、文字列？
	} else if ( IsString ( left ) ) {

		string result     = "";
		string leftValue  = GetStringVariable ( left );
		string rightValue = "";

		///// 右辺は変数でご指名でございます。
		if ( right_v != "" ) {

			///// 当然、文字列変数をご指名ですよね？
			rightValue = GetStringVariable ( right_v );

		///// 右辺はリテラルでご指定であります。
		} else if ( right_l != "" ) {

			rightValue = right_l;
		}

		///// 文字列の演算子は、代入と、プラスだけ。
		if      ( operatorType == "=" ) { result = rightValue; }
		else if ( operatorType == "+" ) { result = leftValue + rightValue; }

		///// 結果を、左辺で指定された変数に代入。エラーなんて出てませんよね？
		SetStringValue ( left, result );
	}

	// 処理終了
	_state = ScriptEngine::STATE_CURRENT_UNIT_FINISHED;
}

//=========================================================
// ユニット開始処理：ウィンドウタイトル
//=========================================================
void ScriptEngine::UnitStart_Title ( ScriptUnit &unit ) {

	// 直接入れるとフルスクリーンから戻したときにタイトルが消えるので、Application を通して文字列を保存している。
	// SFML の ウィンドウクラスになぜか getTitle() がない。setTitle() はあるのに。
	// SFMLのウィンドウに入れると文字化け。UTF-32 への変換は Application 側でやっている。
	Application::GetInstance()->SetWindowTitle( unit.unitTag.value );

	// 処理終了
	_state = ScriptEngine::STATE_CURRENT_UNIT_FINISHED;
}
//=========================================================
// ユニット開始処理：チャプター
//=========================================================
void ScriptEngine::UnitStart_Chapter ( ScriptUnit &unit ) {
	
	_chapterName = unit.unitTag.value;
	
	TextMessageWindow::ReplaceWithAlias(_chapterName);	
	
	printf ( "チャプター設定：%s\n", unit.unitTag.value.c_str() );
	
	// 処理終了
	_state = ScriptEngine::STATE_CURRENT_UNIT_FINISHED;
}

//=========================================================
// ユニット開始処理：セーブウィンドウ開く
//=========================================================
void ScriptEngine::UnitStart_Save ( ScriptUnit &unit ) {

	_menu->OpenSaveWindow();

	// 処理終了
	_state = ScriptEngine::STATE_CURRENT_UNIT_FINISHED;
}
//=========================================================
// ユニット開始処理：ロードウィンドウ開く
//=========================================================
void ScriptEngine::UnitStart_Load ( ScriptUnit &unit ) {

	_menu->OpenLoadWindow();

	// 処理終了
	_state = ScriptEngine::STATE_CURRENT_UNIT_FINISHED;
}

//=========================================================
// ユニット開始処理：この時点でのセーブデータを作成する
//=========================================================
void ScriptEngine::UnitStart_WriteSaveData ( ScriptUnit &unit ) {
	
	
	string fileName = unit.unitTag.value;
	SaveData *saveDataForBackToTitle = new SaveData ( fileName, 0 );

	// SSは作成しない
	SaveTo (saveDataForBackToTitle, false, "");
	delete saveDataForBackToTitle;
		
	printf ( "セーブデータ作成 : %s\n", fileName.c_str() );
	
	

	// 処理終了
	_state = ScriptEngine::STATE_CURRENT_UNIT_FINISHED;
}

//=========================================================
// ユニット開始処理：条件分岐
//=========================================================
void ScriptEngine::UnitStart_IF ( ScriptUnit &unit ) {

	///// 子要素のタグで
	string left         = NotXML::GetTagFromName ( unit.unitTag, "Variable_Left").value;
	string operatorType = NotXML::GetTagFromName ( unit.unitTag, "Operator").value;
	string right_v      = NotXML::GetTagFromName ( unit.unitTag, "Variable_Right_v").value;
	string right_l      = NotXML::GetTagFromName ( unit.unitTag, "Variable_Right_l").value;
	string jumpTo_then  = NotXML::GetTagFromName ( unit.unitTag, "Then").value;
	string jumpTo_else  = NotXML::GetTagFromName ( unit.unitTag, "Else").value;
	string expression   = NotXML::GetTagFromName ( unit.unitTag, "Expression").value;

	///// 属性で
	string left_A         = NotXML::GetAttribute ( unit.unitTag, "Variable_Left");
	string operatorType_A = NotXML::GetAttribute ( unit.unitTag, "Operator");
	string right_v_A      = NotXML::GetAttribute ( unit.unitTag, "Variable_Right_v");
	string right_l_A      = NotXML::GetAttribute ( unit.unitTag, "Variable_Right_l");
	string jumpTo_then_A  = NotXML::GetAttribute ( unit.unitTag, "Then");
	string jumpTo_else_A  = NotXML::GetAttribute ( unit.unitTag, "Else");
	string expression_A   = NotXML::GetAttribute ( unit.unitTag, "expression");

	///// 子要素タグで指定されてなければ、属性値を使う。
	left         = ( left         == "" ) ? left_A         : left;
	operatorType = ( operatorType == "" ) ? operatorType_A : operatorType;
	right_v      = ( right_v      == "" ) ? right_v_A      : right_v;
	right_l      = ( right_l      == "" ) ? right_l_A      : right_l;
	jumpTo_then  = ( jumpTo_then  == "" ) ? jumpTo_then_A  : jumpTo_then;
	jumpTo_else  = ( jumpTo_else  == "" ) ? jumpTo_else_A  : jumpTo_else;
	expression   = ( expression   == "" ) ? expression_A   : expression;


	///// 真偽
	bool result = false;


	///// 式で指定？
	if ( expression != "" ) {

		Utils_String::BinaryOperationElements boe = Utils_String::ParseAsBinaryOperation(expression);

		left = boe.left;
		operatorType = boe.op;

		// ${} で囲まれてたら変数
		if
		(
			boe.right.length() > 3 &&
			boe.right.substr(0, 2) == "${" &&
			boe.right.substr( boe.right.size()-1 , 1) == "}"
		)
		{

			right_v = boe.right.substr( 2, boe.right.size()-3 );
			right_l = "";

		// そうじゃなければリテラル
		} else {
			right_v = "";
			right_l = boe.right;
		}
	}



	///// 左辺は数値？
	if ( IsInt ( left ) ) {

		int leftValue = GetIntVariable ( left );
		int rightValue = 0;

		///// 右辺は変数でご指名でございます。
		if ( right_v != "" ) {

			///// 当然、数値変数をご指名ですよね？
			rightValue = GetIntVariable ( right_v );

		///// 右辺はリテラルでご指定であります。
		} else if ( right_l != "" ) {

			///// 当然、数値になりえるモノを書いてくれてますよね？
			rightValue = Utils_Convert::String_to_Int ( right_l );
		}

		///// 演算子さん、なんでもおっしゃいなさい
		if      ( operatorType == "=="  ) { result = ( leftValue == rightValue );  }
		else if ( operatorType == "lt"  ) { result = ( leftValue <  rightValue );  }
		else if ( operatorType == "le"  ) { result = ( leftValue <= rightValue );  }
		else if ( operatorType == "gt"  ) { result = ( leftValue >  rightValue );  }
		else if ( operatorType == "ge"  ) { result = ( leftValue >= rightValue );  }
		else if ( operatorType == "!="  ) { result = ( leftValue != rightValue );  }

	///// 左辺は文字列？
	} else if ( IsString ( left ) ) {

		string leftValue = GetStringVariable ( left );
		string rightValue = "";

		///// 右辺は変数でご指名でございます。
		if ( right_v != "" ) {

			///// 当然、数値変数をご指名ですよね？
			rightValue = GetStringVariable ( right_v );

		///// 右辺はリテラルでご指定であります。
		} else if ( right_l != "" ) {

			rightValue = right_l;
		}

		///// 文字列の条件演算子は、以下のだけ面倒みますよ。
		if      ( operatorType == "=="  ) { result = ( leftValue == rightValue );  }
		else if ( operatorType == "!="  ) { result = ( leftValue != rightValue );  }

	///// どっちでもない？
	} else {
		// じゃあ、真偽値は false ってことで。別に仕様ではないのですけれども。
		printf ( "変数%s は数値でも文字列でもない。おそらく無定義。\n", left.c_str() );
	}


	///// 真でした。
	if ( result ) {

		unit.nextUnitLabel = jumpTo_then;

	///// 偽でした。
	} else {

		unit.nextUnitLabel = jumpTo_else;
	}


	// 処理終了
	_state = ScriptEngine::STATE_CURRENT_UNIT_FINISHED;

}

//=========================================================
// ユニット開始処理：スイッチ分岐
//=========================================================
void ScriptEngine::UnitStart_Switch ( ScriptUnit &unit ) {

	///// あるはずのものを取得
	string variableName  = NotXML::GetAttribute ( unit.unitTag, "variableName");
	string variableScope = NotXML::GetAttribute ( unit.unitTag, "variableScope", "local");

	///// 対象とされている変数の値を求める
	string value = "";
	int randomIndex = -1; // ランダム指定の場合のインデックス。-1でランダムではないの意味。
	if ( variableScope == "local" ) {

		// ローカルの文字列変数？
		if ( _variable_Local_String.find(variableName) != _variable_Local_String.end()) {
			value = _variable_Local_String.find(variableName)->second;
		}
		// ローカルの数値変数？ 文字列化して取得しますよ。
		else if ( _variable_Local_Int.find(variableName) != _variable_Local_Int.end()) {
			value = Utils_Convert::Int_to_String ( _variable_Local_Int.find(variableName)->second);
		}
	}
	else if ( variableScope == "global" ) {

		// グローバルの文字列変数？
		if ( _variable_Global_String.find(variableName) != _variable_Global_String.end()) {
			value = _variable_Global_String.find(variableName)->second;
		}
		// グローバルの数値変数？ 文字列化して取得しますよ。
		else if ( _variable_Global_Int.find(variableName) != _variable_Global_Int.end()) {
			value = Utils_Convert::Int_to_String ( _variable_Global_Int.find(variableName)->second);
		}
	}
	else if ( variableScope == "random" ) {
		
		// Caseタグの件数を確認。
		int caseTagNum = 0;
		
		for ( unsigned int i = 0; i < unit.unitTag.childList.size(); ++i ) {

			NotXML::Tag subTag = unit.unitTag.childList[i];

			///// Caseタグですよね？
			if ( subTag.name == "Case" ) {
				++caseTagNum;
			}
		}
		
		randomIndex = Utils_Cal::GerRandom ( caseTagNum );
	}


	///// 内部のタグ（Caseタグのはず）でループ
	int caseTagCount = -1;
	for ( unsigned int i = 0; i < unit.unitTag.childList.size(); ++i ) {

		NotXML::Tag subTag = unit.unitTag.childList[i];

		///// Caseタグですよね？
		if ( subTag.name == "Case" ) {
			
			++caseTagCount;
			
			// ランダム指定の場合はこちら
			if ( randomIndex != -1 ) {
				
				if ( caseTagCount == randomIndex ) {
					
					// ジャンプする
					unit.nextUnitLabel = subTag.value;

					// 最初に当てはまったところで終了。
					break;
				}
				
			// こちらは変数指定
			} else {
			
				// 変数が、この値だったら
				if ( value == NotXML::GetAttribute (subTag, "value", "") ) {

					// ジャンプする
					unit.nextUnitLabel = subTag.value;

					// 最初に当てはまったところで終了。
					break;
				}
			}
		}
	} // Caseでのループ閉じ：当てはまるものが１つもなければ、単に次のタグへ進む。


	_state = ScriptEngine::STATE_CURRENT_UNIT_FINISHED;
}

//=========================================================
// ユニット開始処理：サブルーチンを呼び出す
//=========================================================
void ScriptEngine::UnitStart_Gosub ( ScriptUnit &unit ) {

	///// 書かれているはずのものを取得
	string to = NotXML::GetAttribute ( unit.unitTag, "to", "" );

	///// サブルーチンを呼び出す
	Gosub ( to );

	///// 処理終了
	_state = ScriptEngine::STATE_CURRENT_UNIT_FINISHED;
}

//=========================================================
// ユニット開始処理：ゲーム盤を定義
//=========================================================
void ScriptEngine::UnitStart_Game ( ScriptUnit &unit ) {

printf ("ゲーム盤定義処理開始\n");

	///// 最低限、あるはずのもの。
	string name = NotXML::GetTagFromName ( unit.unitTag, "Name").value;

	///// すでにある？ なければ新たに作るよ。
	GameManager *gameManager = GameManager_Get(name);
	if ( gameManager == NULL ) {

		gameManager = new GameManager(name, this);

		// リストへの登録も忘れないでね。
		_gameManagerList [ name ] = gameManager;
	}

	///// 削除する？
	if (NotXML::GetTagFromName ( unit.unitTag, "Release" ).name == "Release" ) {

		GameManager_Release ( name );

		// おしまい。
		_state = ScriptEngine::STATE_CURRENT_UNIT_FINISHED;
		return;
	}


	//gameManager->Init ( GameManager::Mode::NONE );

	for ( unsigned int i = 0; i < unit.unitTag.childList.size(); ++i) {

		NotXML::Tag tag = unit.unitTag.childList[i];

		if ( tag.name == "Active" ) {

			gameManager->SetActive ( Utils_Convert::String_to_bool ( tag.value ) );

		} else if ( tag.name == "BoardSize" ) {

			// 両方とも書いてくれてますよね。
			int width  = Utils_Convert::String_to_Int ( NotXML::GetAttribute ( tag, "width",  "1" ) );
			int height = Utils_Convert::String_to_Int ( NotXML::GetAttribute ( tag, "height", "1" ) );

			gameManager->SetUpBoard ( width, height );

		} else if ( tag.name == "BoardPosition" ) {

			// 両方とも書いてくれてますよね。
			int x = Utils_Convert::String_to_Int ( NotXML::GetAttribute ( tag, "x", Utils_Convert::Int_to_String(APPLICATION::WINDOW_SIZE::WIDTH / 2) ) );
			int y = Utils_Convert::String_to_Int ( NotXML::GetAttribute ( tag, "y", Utils_Convert::Int_to_String(APPLICATION::WINDOW_SIZE::HEIGHT / 2) ) );

			gameManager->SetBoardCenterPosition ( x, y );

		} else if ( tag.name == "ActorPosition" ) {
			/*
			int x = Utils_Convert::String_to_Int ( NotXML::GetAttribute ( tag, "x", Utils_Convert::Int_to_String(APPLICATION::WINDOW_SIZE::WIDTH / 2) ) );
			int y = Utils_Convert::String_to_Int ( NotXML::GetAttribute ( tag, "y", Utils_Convert::Int_to_String(APPLICATION::WINDOW_SIZE::HEIGHT / 2) ) );

			gameManager->SetActorPosition ( x, y );
			*/

		} else if ( tag.name == "BattleTitle" ) {

			gameManager->SetBattleTitle ( tag.value );

		} else if ( tag.name == "Operation" ) {


			for ( unsigned int n = 0; n < tag.childList.size(); ++n ) {

				NotXML::Tag subTag = tag.childList[n];

				if ( subTag.name == "Lock" ) {
					gameManager->SetOperation_Lock ( Utils_Convert::String_to_bool ( subTag.value ) );


				} else if ( subTag.name == "TurnOrder" ) {

					vector<string> turnOrderIdList = Utils_String::Split ( subTag.value, "," );

					gameManager->SetTurnOrder ( turnOrderIdList );

				}
			}

		///// 参加者設定
		} else if ( tag.name == "Participants" ) {

			printf ("参加者設定\n");

			// この時点で既存の参加者データをクリアしておく
			gameManager->clearAllParticipants();


			for ( unsigned int n = 0; n < tag.childList.size(); ++n ) {

				NotXML::Tag subTag = tag.childList[n];

				///// 参加者タグ
				if ( subTag.name == "Participant" ) {

					GameManager::Participant participant;

					participant.id   = NotXML::GetAttribute ( subTag, "id", "" );
					participant.side = NotXML::GetAttribute ( subTag, "side", "" );
					participant.name = NotXML::GetAttribute ( subTag, "name", "" );

					// この段階で参加者登録
					gameManager->addParticipant ( participant );

					for ( unsigned int n2 = 0; n2 < subTag.childList.size(); ++n2 ) {

						NotXML::Tag subsubTag = subTag.childList[n2];

						///// AI設定ファイル
						if ( subsubTag.name == "AI_File" ) {

							gameManager->SetAIFile ( participant.id, subsubTag.value );

						}
						///// Actor設定ファイル
						else if ( subsubTag.name == "Actor_File" ) {

							string positionStr            = NotXML::GetAttribute ( subsubTag, "position", "0,0" );
							vector<string> positionVector = Utils_String::Split ( positionStr, "," );
							int posX = ( positionVector.size() == 2 ) ? Utils_Convert::String_to_Int ( positionVector[0], 0 ) : 0;
							int posY = ( positionVector.size() == 2 ) ? Utils_Convert::String_to_Int ( positionVector[1], 0 ) : 0;

							int startPosX = posX;
							int startPosY = posY;
							string positionFrom = NotXML::GetAttribute ( subsubTag, "positionFrom", "" );

							Image *fromImage = Image_Get ( positionFrom );
							if ( fromImage != NULL ) {
								startPosX = fromImage->GetCurrentPosition().x;
								startPosY = fromImage->GetCurrentPosition().y;
							}


							// セリフ表示用のテキスト欄を用意
							//TextMessageWindow *textWindow = (_textMessageWindow != NULL) ? _textMessageWindow->GetSameSetuppedInstance() : NULL;

							gameManager->SetActorScriptFile ( participant, subsubTag.value, posX, posY, startPosX, startPosY, _textMessageWindow);

						}
						///// 口出しモードすでに候補手からの選択モードが設定されているときには口出しモードは設定しない。
						else if ( subsubTag.name == "InterferenceMode" ) {

							// 制限回数：デフォルトは３回ということで。
							int maxTimes = Utils_Convert::String_to_Int ( NotXML::GetAttribute ( subsubTag, "maxTimes", "3" ) );

							// 選択肢の表示位置
							string selectOptionsPosString = NotXML::GetAttribute ( subsubTag, "selectOptionPosition", "" );

							gameManager->SetInterferenceModeToParticipant ( participant.id, maxTimes, selectOptionsPosString );
						}
						///// 候補手からの選択モード：先に口出しモードが設定されているときには口出しモードを無効とする
						else if ( subsubTag.name == "ChoiceMode" ) {

							// 選択肢の件数：デフォルトは３回ということで。
							int numberOfChoices = Utils_Convert::String_to_Int ( NotXML::GetAttribute ( subsubTag, "numberOfChoices", "3" ) );

							// 選択肢の表示位置
							string selectOptionsPosString = NotXML::GetAttribute ( subsubTag, "selectOptionPosition", "" );
							
							// ２番目以降の選択肢の取得基準： extraChoicesRange="1-9,10-"
							string choicesFrom = NotXML::GetAttribute ( subsubTag, "extraChoicesRange", "" );
							vector<string> choicesFromListStr = Utils_String::Split ( choicesFrom, "," );
							
							vector< pair<int, int> > choicesFromList;
							for ( unsigned int iii = 0; iii < choicesFromListStr.size(); ++iii ) {
								
								vector<string> ana1 = Utils_String::Split ( choicesFromListStr[iii], "-");
								string from = ( ana1.size() >= 1 ) ? ana1[0] : "0";
								string to   = ( ana1.size() >= 2 ) ? ana1[1] : "-1";
								
								from = Utils_String::Trim ( from );
								to   = Utils_String::Trim ( to );
								
								if ( from == "" ) { from = "0"; }
								if ( to   == "" ) { to   = "-1"; }
								
								pair<int, int> p;
								p.first  = Utils_Convert::String_to_Int ( from );
								p.second = Utils_Convert::String_to_Int ( to );
								
								choicesFromList.push_back ( p );
							}
							
							// 確認
							printf ( "ChoiceMode extraChoicesRange = {" );
							for ( unsigned int iii = 0; iii < choicesFromList.size(); ++iii ) {
								
								if ( iii > 0 ) { printf (","); }
								printf ( "%d-%d", choicesFromList[iii].first, choicesFromList[iii].second );
								
							}
							printf ("}\n");
							
							gameManager->SetChoiceModeToParticipant ( participant.id, numberOfChoices, selectOptionsPosString, choicesFromList );

						}
					}

				} // 参加者タグParticipantの for閉じ


			} // Participantsタグのサブタグループの閉じ


			// ここでチーム判定
			gameManager->TeamForming();


		///// AI設定（旧）
		} else if ( false && tag.name == "AI" ) {
/*
			// 一応、どのAIかの指定を受け取るようにしておく
			string AI_Name = NotXML::GetAttribute ( tag, "name", "" );

			for ( unsigned int n = 0; n < tag.childList.size(); ++n ) {

				NotXML::Tag subTag = tag.childList[n];

				///// 思考設定ファイル
				if ( subTag.name == "File" ) {

					gameManager->SetAIFile ( AI_Name, subTag.value );

				///// アクタースクリプトファイル
				} else if ( subTag.name == "Actor_File" ) {

					// セリフ表示用のテキスト欄を用意
					TextMessageWindow *textWindow = (_textMessageWindow != NULL) ? _textMessageWindow->GetSameSetuppedInstance() : NULL;

					// P2側？ P1側？：何も言わなければP2側ということで。
					string playingSide = UTILS_OTHERS::GetValueFromMap ( subTag.attribute, "side", "P2" );

					gameManager->SetActorScriptFile ( AI_Name, subTag.value, playingSide, textWindow );

				}
*/
				/*
				///// 着手エフェクト挙動スクリプトファイル
				} else if ( subTag.name == "Script_MoveEffectDecision" ) {

					// これは、スクリプトエンジン側で使う、けど、
					// この AI に 所属するものだから、やっぱり gameManagerさんにお預け。以下のスクリプトも同様。
					gameManager->SetScriptFile_MoveEffectDecision ( AI_Name, subTag.value );

				///// 感情値変動処理スクリプトファイル
				} else if ( subTag.name == "Script_EmotionProcess" ) {

					gameManager->SetScriptFile_EmotionProcess ( AI_Name, subTag.value );

				///// リアクション処理スクリプトファイル
				} else if ( subTag.name == "Script_Reaction" ) {

					gameManager->SetScriptFile_Reaction ( AI_Name, subTag.value );
				}
				*/
//			}

		///// 棋譜
		} else if ( tag.name == "ScoreSheet" ) {

			NotXML::Tag fileTag     = NotXML::GetTagFromName ( tag, "File" );
			NotXML::Tag variableTag = NotXML::GetTagFromName ( tag, "Variable" );
			NotXML::Tag indexTag    = NotXML::GetTagFromName ( tag, "Index" );

			///// 棋譜ファイルは……
			string fileName = "";

			///// 棋譜ファイルを直接指定？
			if ( fileTag.value != "" ) {

				fileName = fileTag.value;

			///// 棋譜ファイルを変数経由で指定？
			} else if (variableTag.value != "") {

				fileName = GetStringVariable(variableTag.value);
			}

			/*
				ファイル名直書きと変数経由とどちらを優先するか？
				ファイル名直書きを優先する。
				直書きするのって、おそらく、デバッグ目的なのでない？ そうでもないですか？
				タグが重複するシチュエーションとしては、デバッグ、なような気がする。

			*/
			if ( fileName != "" ) {

				///// 棋譜ファイルを読み込む
				gameManager->ReadScoreSheetFile ( fileName );

			}


			///// 棋譜の選択位置を指定？
			// ちなみに、ここは棋譜の読み込み時意外でも呼び出せることに注目だぜ。瞠目だぜ。刮目だぜ。
			if ( indexTag.value != "" ) {

				// ただし、検討モードの最中じゃないと内部で無視されるってことでよいですよね？ うん。よいよ。
				gameManager->SetScoreIndex ( indexTag.value );

			}

			///// 表示/非表示
			if ( NotXML::IsExist ( tag, "Show" ) ) {
				gameManager->ShowScoreSheetViewer();
			}
			if ( NotXML::IsExist ( tag, "Hide" ) ) {
				gameManager->HideScoreSheetViewer();
			}
		}

	}

	///// 処理終了
	_state = ScriptEngine::STATE_CURRENT_UNIT_FINISHED;
}

//=========================================================
// ユニット開始処理：ゲーム盤を解放
//=========================================================
void ScriptEngine::UnitStart_Game_Release ( ScriptUnit &unit ) {

	///// 書かれているはずのものを取得
	string name = NotXML::GetAttribute ( unit.unitTag, "game", "" );

	///// 解放してね。
	GameManager_Release ( name );

	///// 処理終了
	_state = ScriptEngine::STATE_CURRENT_UNIT_FINISHED;
}

//=========================================================
// ユニット開始処理：ゲーム盤に対して、指し手を実行
//=========================================================
void ScriptEngine::UnitStart_Move ( ScriptUnit &unit ) {


	// 待機なのです。
	_state = ScriptEngine::STATE_WAIT_UNTIL_UNIT_END;
	
	
}
//=========================================================
// ユニット実行処理：ゲーム盤に対して、指し手を実行
//=========================================================
void ScriptEngine::UnitProcessing_Move ( ScriptUnit &unit ) { 

	///// 書かれているはずのものを取得
	string name = NotXML::GetAttribute ( unit.unitTag, "game", "" );

	///// ご指名の子がいるかな？
	GameManager *gameManager = GameManager_Get(name);
	if ( gameManager != NULL ) {
        
        // 盤面が「落ち着いて」いるかどうかを判定。
        
        /*
        bool isAllPreviousMoveFinished = true;
        
        ///// 駒の動きが止まっているかどうかを確認
        if ( isAllPreviousMoveFinished ) { // ←確認項目ごとに{}に入れて、「その時点ですでにfalseになっていたらもう確認する必要はない」という意味
        
            vector<Piece*> pieceList = gameManager->GetAllPieces_forManagement();
            for ( unsigned int i = 0; i < pieceList.size(); ++i ) {
                
                Piece *piece = pieceList[i];
                
                if ( !piece->IsMovingStop() ) {
                    isAllPreviousMoveFinished = false;
                    break;
                }
            }
        }

        if ( isAllPreviousMoveFinished ) { // ←確認項目ごとに{}に入れて、「その時点ですでにfalseになっていたらもう確認する必要はない」という意味
            if (gameManager->IsMoveEffectGoing()){
                isAllPreviousMoveFinished = false;
            }
        }
        */
        
        
        //if ( isAllPreviousMoveFinished ) {
        if ( gameManager->IsScriptingMovingFinished() ) {
            
            gameManager->StartScriptingMoving();
            
            gameManager->Move ( unit.unitTag );

            ///// 処理終了
            _state = ScriptEngine::STATE_CURRENT_UNIT_FINISHED;

		} else {
            printf ("終了していない指し手があるので待機（ユニット実行処理：ゲーム盤に対して、指し手を実行）\n", name.c_str());
		}
		
		
    ///// 指定の盤がない？ おかしいですね。とりあえず終了はしておく
	} else {
	
        printf ("指定の盤[%s]がない（ユニット実行処理：ゲーム盤に対して、指し手を実行）\n", name.c_str());

        ///// 処理終了
        _state = ScriptEngine::STATE_CURRENT_UNIT_FINISHED;
	}

}


//=========================================================
// ユニット開始処理：ゲーム盤に対して、駒およびカードを追加
//=========================================================
void ScriptEngine::UnitStart_Put ( ScriptUnit &unit ) {

	///// 書かれているはずのもの
	string name   = NotXML::GetAttribute ( unit.unitTag, "game", "" );

	///// ご指名の子がいるかな？
	GameManager *gameManager = GameManager_Get(name);
	if ( gameManager == NULL ) {

		///// 処理終了
		_state = ScriptEngine::STATE_CURRENT_UNIT_FINISHED;

		return;
	}


	///// 子要素でループするよ！
	for ( unsigned int i = 0; i < unit.unitTag.childList.size(); ++i) {

		NotXML::Tag tag = unit.unitTag.childList[i];

		///// 駒を、どうするのかな？
		if ( tag.name == "Piece" ) {

			///// 要素は何が書かれているかな？
			string color = NotXML::GetAttribute ( tag, "color", "BLACK" ); // 色は、何か書いてあることにしてしまう。
			string box   = NotXML::GetAttribute ( tag, "box",   "" );
			string gridX = NotXML::GetAttribute ( tag, "x", "" );
			string gridY = NotXML::GetAttribute ( tag, "y", "" );

			////// これは、GameManagerさんの側でしてもらう？ カードの向きも。
			///// 色を、文字列から、色値に変換するですよー。
			int pieceColor = Piece::Color::BLACK;
			if ( color == "WHITE" ) {
				pieceColor = Piece::Color::WHITE;
			}


			///// P1の駒台に追加するの？
			if ( box == "P1" || box == "p1" ) {

				gameManager->AddPiece ( color, box );

			///// P2の駒台に追加するの？
			} else if ( box == "P2" || box == "p2" ) {

				gameManager->AddPiece ( color, box );

			///// 盤面に乗せるってことですね？
			} else if ( gridX != "" && gridY != "" ) {

				int x = Utils_Convert::String_to_Int ( gridX );
				int y = Utils_Convert::String_to_Int ( gridY );

				gameManager->AddPiece ( color, x, y );
			}

		}
		///// カードを、どうするのかな？
		else if ( tag.name == "Card" ) {

			string name      = NotXML::GetAttribute ( tag, "name"     , "" );
			string direction = NotXML::GetAttribute ( tag, "direction", "" );
			string box       = NotXML::GetAttribute ( tag, "box"      , "" );
			string gridX     = NotXML::GetAttribute ( tag, "x"    , "" );
			string gridY     = NotXML::GetAttribute ( tag, "y"    , "" );



			///// P1のカード置き場に追加するの？
			if ( box == "P1" || box == "p1" ) {

				gameManager->AddCard ( name, direction, box );

			///// P2のカード置き場に追加するの？
			} else if ( box == "P2" || box == "p2" ) {

				gameManager->AddCard ( name, direction, box );

			///// 盤面に乗せるってことですね？
			} else if ( gridX != "" && gridY != "" ) {

				int x = Utils_Convert::String_to_Int ( gridX );
				int y = Utils_Convert::String_to_Int ( gridY );

				gameManager->AddCard ( name, direction, x, y );
			}
		}
	}

	///// 処理終了
	_state = ScriptEngine::STATE_CURRENT_UNIT_FINISHED;
}

//=========================================================
// ユニット開始処理：ゲーム盤から、駒およびカードを削除
//=========================================================
void ScriptEngine::UnitStart_Remove ( ScriptUnit &unit ) {

	///// 書かれているはずのもの
	string name   = NotXML::GetAttribute ( unit.unitTag, "game", "" );

	///// ご指名の子がいるかな？
	GameManager *gameManager = GameManager_Get(name);
	if ( gameManager == NULL ) {

		///// 処理終了
		_state = ScriptEngine::STATE_CURRENT_UNIT_FINISHED;

		return;
	}

	///// 子要素でループするよ！
	for ( unsigned int i = 0; i < unit.unitTag.childList.size(); ++i) {

		NotXML::Tag tag = unit.unitTag.childList[i];

		///// 駒を、どうするのかな？
		if ( tag.name == "Piece" ) {

			///// 要素は何が書かれているかな？
			string box   = NotXML::GetAttribute ( tag, "box",   "" );
			string gridX = NotXML::GetAttribute ( tag, "gridX", "" );
			string gridY = NotXML::GetAttribute ( tag, "gridY", "" );

			///// P1の駒台から削除するの？
			if ( box == "P1" || box == "p1" ) {

				gameManager->RemovePiece ( box );

			///// P2の駒台から削除するの？
			} else if ( box == "P2" || box == "p2" ) {

				gameManager->RemovePiece ( box );

			///// 盤面から削除ってことですね？
			} else if ( gridX != "" && gridY != "" ) {

				int x = Utils_Convert::String_to_Int ( gridX );
				int y = Utils_Convert::String_to_Int ( gridY );

				gameManager->RemovePiece ( x, y );
			}

		}
		///// カードを、どうするのかな？
		else if ( tag.name == "Piece" ) {

			string name      = NotXML::GetAttribute ( tag, "name"     , "" );
			string box       = NotXML::GetAttribute ( tag, "box"      , "" );
			string gridX     = NotXML::GetAttribute ( tag, "gridX"    , "" );
			string gridY     = NotXML::GetAttribute ( tag, "gridY"    , "" );


			///// P1のカード置き場から削除するの？
			if ( box == "P1" || box == "p1" ) {

				gameManager->RemoveCard ( name, box );

			///// P2のカード置き場から削除するの？
			} else if ( box == "P2" || box == "p2" ) {

				gameManager->RemoveCard ( name, box );

			///// 盤面から削除ってことですね？
			} else if ( gridX != "" && gridY != "" ) {

				int x = Utils_Convert::String_to_Int ( gridX );
				int y = Utils_Convert::String_to_Int ( gridY );

				gameManager->RemoveCard ( x, y );
			}
		}
	}

	///// 処理終了
	_state = ScriptEngine::STATE_CURRENT_UNIT_FINISHED;
}
//=========================================================
// ユニット開始処理：ゲーム盤から、駒およびカードをまとめて削除
//=========================================================
void ScriptEngine::UnitStart_RemoveAll ( ScriptUnit &unit ) {

	///// 書かれているはずのもの
	string name   = NotXML::GetAttribute ( unit.unitTag, "game", "" );

	///// ご指名の子がいるかな？
	GameManager *gameManager = GameManager_Get(name);
	if ( gameManager == NULL ) {

		///// 処理終了
		_state = ScriptEngine::STATE_CURRENT_UNIT_FINISHED;

		return;
	}


	///// 書かれているかもしれないもの
	bool isPieceBoxP1   = Utils_Convert::String_to_bool ( NotXML::GetAttribute ( unit.unitTag, "pieceBoxP1"  , "off" ) );
	bool isPieceBoxP2   = Utils_Convert::String_to_bool ( NotXML::GetAttribute ( unit.unitTag, "pieceBoxP2"  , "off" ) );
	bool isCardBoxP1    = Utils_Convert::String_to_bool ( NotXML::GetAttribute ( unit.unitTag, "cardBoxP1"   , "off" ) );
	bool isCardBoxP2    = Utils_Convert::String_to_bool ( NotXML::GetAttribute ( unit.unitTag, "cardBoxP2"   , "off" ) );
	bool isPieceOnBoard = Utils_Convert::String_to_bool ( NotXML::GetAttribute ( unit.unitTag, "pieceOnBoard", "off" ) );
	bool isCardOnBoard  = Utils_Convert::String_to_bool ( NotXML::GetAttribute ( unit.unitTag, "cardOnBoard" , "off" ) );


	///// 何を削除するって？
	if ( isPieceBoxP1 ) {
		gameManager->RemoveAllPiece_fromBox ( "P1" );
	}
	if ( isPieceBoxP2 ) {
		gameManager->RemoveAllPiece_fromBox ( "P2" );
	}
	if ( isCardBoxP1 ) {
		gameManager->RemoveAllCard_fromBox ( "P1" );
	}
	if ( isCardBoxP2 ) {
		gameManager->RemoveAllCard_fromBox ( "P2" );
	}
	if ( isPieceOnBoard ) {
		gameManager->RemoveAllPiece_fromBoard();
	}
	if ( isCardOnBoard ) {
		gameManager->RemoveAllCard_fromBoard();
	}

	///// 処理終了
	_state = ScriptEngine::STATE_CURRENT_UNIT_FINISHED;
}

//=========================================================
// ユニット開始処理：イベント監視
//=========================================================
void ScriptEngine::UnitStart_WatchEvent ( ScriptUnit &unit ) {

	///// 引き続き？
	bool isContinue = Utils_Convert::String_to_bool ( NotXML::GetAttribute ( unit.unitTag, "continue", "off" ) );
	if ( isContinue ) {

		///// 旧設定内容（が、あると信じて）で、処理開始
		_state = ScriptEngine::STATE_WAIT_UNTIL_UNIT_END;

		return;
	}

	///// というか、イベント監視を開始する時点で、旧データがクリアされる。それまでは状態が残っている。
	_watchEventDataList.clear();

	///// さぁ、いらっしゃい！
	for ( unsigned int i = 0; i < unit.unitTag.childList.size(); ++i ) {

		const NotXML::Tag &tag = unit.unitTag.childList[i];

		// Eventタグですよね？
		if ( tag.name == "Event" ) {

			WatchEventData watchEventData;

			// 何を監視？
			watchEventData.type        = NotXML::GetTagFromName ( tag, "Type" ).value;
			watchEventData.jumpToLabel = NotXML::GetTagFromName ( tag, "Jump" ).value;

			// 回数つき？
			string count = NotXML::GetTagFromName ( tag, "Count" ).value;
			if ( count != "" ) {
				watchEventData.count = Utils_Convert::String_to_Int(count);
			}

			// 指し手を監視
			if ( watchEventData.type == "GameMove" ) {

				// どのゲーム盤ですか？
				watchEventData.gameManagerName = NotXML::GetTagFromName ( tag, "Game" ).value;

				// ゲーム盤、ありますよね？
				GameManager *gameManager = GameManager_Get ( watchEventData.gameManagerName );
				if ( gameManager == NULL ) {
					continue;
				}

				// 現時点での最終着手ですよ。
				watchEventData.move_Before = gameManager->GetLastMove();

				// アクションだけで指定？
				string action = Utils_String::Trim ( NotXML::GetTagFromName ( tag, "Action" ).value );
				if ( action != "" ) {

					if      ( action == "PUT_PIECE"  ) { watchEventData.moveAction = AI_Modules::MOVE::ACTION::PUT_PIECE;  }
					else if ( action == "PUT_CARD"   ) { watchEventData.moveAction = AI_Modules::MOVE::ACTION::PUT_CARD;   }
					else if ( action == "TAKE_CARD"  ) { watchEventData.moveAction = AI_Modules::MOVE::ACTION::TAKE_CARD;  }
					else if ( action == "MOVE_PIECE" ) { watchEventData.moveAction = AI_Modules::MOVE::ACTION::MOVE_PIECE;

						// 駒移動のときだけ、ぶつけるフラグを見るよ。
						NotXML::Tag actionTag = NotXML::GetTagFromName ( tag, "Action" );
						watchEventData.isCollide = NotXML::GetAttribute( actionTag, "isCollide", "" );
					}

				// じゃあ、Move タグがあるんだろうねぇ。
				} else {

					watchEventData.move = Game_Utils::Move_From_Tag ( NotXML::GetTagFromName ( tag, "Move" ) );
				}

				// さぁ、監視するよ！
				_watchEventDataList.push_back ( watchEventData );

			// ボタンを監視
			} else if ( watchEventData.type == "Button" ) {

				// どのボタンですか？
				watchEventData.buttonName = NotXML::GetTagFromName ( tag, "Button" ).value;

				// ボタンがなくても監視するのですね？ ンマー！ いじわる！

				// さぁ、監視するよ！
				_watchEventDataList.push_back ( watchEventData );
			}
		}
	}

	///// 処理開始するよ！ イベントタグがなくてもね！
	_state = ScriptEngine::STATE_WAIT_UNTIL_UNIT_END;
}


//=========================================================
// ユニット開始処理：ゲーム盤の操作を監視
//=========================================================
void ScriptEngine::UnitStart_WatchGameOperation ( ScriptUnit &unit ) {

	///// 引き続き？
	bool isContinue = Utils_Convert::String_to_bool ( NotXML::GetAttribute ( unit.unitTag, "continue", "off" ) );
	if ( isContinue ) {

		///// 旧設定内容（が、あると信じて）で、処理開始
		_state = ScriptEngine::STATE_WAIT_UNTIL_UNIT_END;

		return;
	}

	///// というか、イベント監視を開始する時点で、旧データがクリアされる。それまでは状態が残っている。
	_watchGameOperationDataList.clear();

	///// さぁ、いらっしゃい！
	for ( unsigned int i = 0; i < unit.unitTag.childList.size(); ++i ) {

		const NotXML::Tag &tag = unit.unitTag.childList[i];

		// Eventタグですよね？
		if ( tag.name == "Event" ) {

			WatchGameOperationData watchEventData;

			// イベント発生時のジャンプ先
			watchEventData.jumpToLabel = NotXML::GetTagFromName ( tag, "Jump" ).value;

			// 回数つき？
			string count = NotXML::GetTagFromName ( tag, "Count" ).value;
			if ( count != "" ) {
				watchEventData.count = Utils_Convert::String_to_Int(count);
			}

			// どのゲーム盤ですか？
			watchEventData.gameManagerName = NotXML::GetTagFromName ( tag, "Game" ).value;

			// ゲーム盤、ありますよね？
			GameManager *gameManager = GameManager_Get ( watchEventData.gameManagerName );
			if ( gameManager == NULL ) {
				continue;
			}

			// 現時点での最終着手ですよ。
			watchEventData.move_Before = gameManager->GetLastMove();
			watchEventData.scoreLength_Before = gameManager->GetScoreRecordLength();

			// アクションだけで指定？
			string action = Utils_String::Trim ( NotXML::GetTagFromName ( tag, "Action" ).value );
			if ( action != "" ) {

				if      ( action == "PUT_PIECE"  ) { watchEventData.moveAction = AI_Modules::MOVE::ACTION::PUT_PIECE;  }
				else if ( action == "PUT_CARD"   ) { watchEventData.moveAction = AI_Modules::MOVE::ACTION::PUT_CARD;   }
				else if ( action == "TAKE_CARD"  ) { watchEventData.moveAction = AI_Modules::MOVE::ACTION::TAKE_CARD;  }
				else if ( action == "MOVE_PIECE" ) { watchEventData.moveAction = AI_Modules::MOVE::ACTION::MOVE_PIECE;

					// 駒移動のときだけ、ぶつけるフラグを見るよ。
					NotXML::Tag actionTag = NotXML::GetTagFromName ( tag, "Action" );
					watchEventData.isCollide = NotXML::GetAttribute( actionTag, "isCollide", "" );
				}

			// じゃあ、Move タグがあるんだろうねぇ。
			} else {

				watchEventData.move = Game_Utils::Move_From_Tag ( NotXML::GetTagFromName ( tag, "Move" ) );
			}

			// さぁ、監視するよ！
			_watchGameOperationDataList.push_back ( watchEventData );

		}
	}

	///// 処理開始するよ！ イベントタグがなくてもね！
	_state = ScriptEngine::STATE_WAIT_UNTIL_UNIT_END;
}


//=========================================================
// ユニット開始処理：待機
//=========================================================
void ScriptEngine::UnitStart_Wait ( ScriptUnit &unit ) {

	///// この１フレームは含む？ 野放図にコーディングすると、含まない、になりますよね。
	_waitFrame = Utils_Convert::String_to_Int ( NotXML::GetAttribute ( unit.unitTag, "frame", "0" ) );

	///// 処理開始するよ！
	_state = ScriptEngine::STATE_WAIT_UNTIL_UNIT_END;
}
//=========================================================
// ユニット開始処理：クリック待機
//=========================================================
void ScriptEngine::UnitStart_WaitClick ( ScriptUnit &unit ) {

	///// 処理開始するよ！
	_state = ScriptEngine::STATE_WAIT_UNTIL_UNIT_END;
}
//=========================================================
// ユニット開始処理：変数の内容をコンソールで確認
//=========================================================
void ScriptEngine::UnitStart_Jump ( ScriptUnit &unit ) {

	// unit.nextUnitLabel が設定されてる前提。素通りするだけ。

	// 処理終了
	_state = ScriptEngine::STATE_CURRENT_UNIT_FINISHED;
}

//=========================================================
// ユニット開始処理：変数の内容をコンソールで確認
//=========================================================
void ScriptEngine::UnitStart_Variable_Console ( ScriptUnit &unit ) {
	
	ShowVariablesToConsole();
	
	// 処理終了
	_state = ScriptEngine::STATE_CURRENT_UNIT_FINISHED;
}

//=========================================================
// 変数をコンソールに表示
//=========================================================
void ScriptEngine::ShowVariablesToConsole() {

	printf ("//===== _variable_Global_Int =====\n");
	for (
		map<string, int>::iterator it = _variable_Global_Int.begin();
		it != _variable_Global_Int.end();
		++it
	)
	{
		printf ( "%s = %d\n", it->first.c_str(), it->second );
	}
	printf("\n");

	printf ("//===== _variable_Global_String =====\n");
	for (
		map<string, string>::iterator it = _variable_Global_String.begin();
		it != _variable_Global_String.end();
		++it
	)
	{
		printf ( "%s = %s\n", it->first.c_str(), it->second.c_str() );
	}
	printf("\n");

	printf ("//===== _variable_Local_Int =====\n");
	for (
		map<string, int>::iterator it = _variable_Local_Int.begin();
		it != _variable_Local_Int.end();
		++it
	)
	{
		printf ( "%s = %d\n", it->first.c_str(), it->second );
	}
	printf("\n");

	printf ("//===== _variable_Local_String =====\n");
	for (
		map<string, string>::iterator it = _variable_Local_String.begin();
		it != _variable_Local_String.end();
		++it
	)
	{
		printf ( "%s = %s\n", it->first.c_str(), it->second.c_str() );
	}
	printf("\n");

}

//=========================================================
// ユニット開始処理：制御文字を定義
//=========================================================
void ScriptEngine::UnitStart_DefineControlTag ( ScriptUnit &unit ) {

	///// 書かれているはずのもの
	string type = NotXML::GetAttribute ( unit.unitTag, "type" );
	string tag  = NotXML::GetAttribute ( unit.unitTag, "tag" );


	if ( type == "NEW_LINE" ) {
		GUI_SFML::TextMessageWindow::Set_CONTROL_CHAR_NEW_LINE ( tag );
	}
	else if ( type == "NEW_PAGE" ) {
		GUI_SFML::TextMessageWindow::Set_CONTROL_CHAR_NEW_PAGE ( tag );
	}
	else if ( type == "WAIT_CLICK" ) {
		GUI_SFML::TextMessageWindow::Set_CONTROL_CHAR_WAIT_CLICK ( tag );
	}
	else {

		printf ( "制御文字を定義できず：どの type にも当てはまらない。[%s]\n", type.c_str() );
	}

	///// 処理終了
	_state = ScriptEngine::STATE_CURRENT_UNIT_FINISHED;
}


//=========================================================
// ユニット実行中（終了を待機）：テキストウィンドウ定義
//=========================================================
void ScriptEngine::UnitProcessing_DefineTextWindow ( ScriptUnit &unit ) {

	// 素通りする処理型なので、ここにはこないはず。
	_state = ScriptEngine::STATE_CURRENT_UNIT_FINISHED;
}
//=========================================================
// ユニット実行中（終了を待機）：テキスト表示
//=========================================================
void ScriptEngine::UnitProcessing_Text ( ScriptUnit &unit ) {

	if ( _textMessageWindow != NULL ) {

		// 全文表示完了している？
		if ( _textMessageWindow->IsAllCharShown() ) {

			_state = ScriptEngine::STATE_CURRENT_UNIT_FINISHED;

			// このユニットは既読になりました。
			this->Finished();
		}

	// ここにはこないはず
	} else {
		_state = ScriptEngine::STATE_CURRENT_UNIT_FINISHED;
	}

}
//=========================================================
// ユニット実行中（終了を待機）：画像
//=========================================================
void ScriptEngine::UnitProcessing_Image ( ScriptUnit &unit ) {

	///// ラベルをもとに既存の画像を取得。ここにくるからには、ないはずはない。
	NotXML::Tag labelTag = NotXML::GetTagFromName ( unit.unitTag, "label" );
	string      label    = labelTag.value;

	Image *image = Image_Get ( label );

	// Release されてたら、ないはずもありますね。
	if ( image == NULL ) {
		//printf ("Image NULL\n");
		_state = ScriptEngine::STATE_CURRENT_UNIT_FINISHED;
		return;
	}
	
	// スキップ中は移動処理を早送りして欲しい
	if ( IsSkip() ) {
		//printf ("Image Move Skipping\n");
		image->Skipping();
	}

	// 移動が終了していて、トランジションも終了している？
	if ( image->IsMoveFinished() && image->IsTransitionFinished()) {
		_state = ScriptEngine::STATE_CURRENT_UNIT_FINISHED;
	}
	/*
		↑何を待機するべきなのかは、特定できた方がいい？
	*/
}
//=========================================================
// ユニット実行中（終了を待機）：画像集
//=========================================================
void ScriptEngine::UnitProcessing_Image_Group ( ScriptUnit &unit ) {

	bool isAllDone = true;

	///// ラベルをもとに既存の画像を取得。ここにくるからには、ないはずはない。
	vector<NotXML::Tag> imageTagList = NotXML::GetTagListFromName( unit.unitTag, SCRIPT::UNIT_TYPE::IMAGE);

	for (unsigned int i = 0; i < imageTagList.size(); ++i) {

		NotXML::Tag labelTag = NotXML::GetTagFromName ( imageTagList[i], "label" );
		string      label    = labelTag.value;

		Image *image = Image_Get ( label );

		// Release されてたら、ないはずもありますね。
		if ( image == NULL ) {
			_state = ScriptEngine::STATE_CURRENT_UNIT_FINISHED;
			continue;
		}
		
		if ( IsSkip() ) {
			image->Skipping();
		}

		// 移動が終了していて、トランジションも終了している？
		if ( !image->IsMoveFinished() || !image->IsTransitionFinished()) {
			isAllDone = false;
		}
	}


	if (isAllDone) {
		_state = ScriptEngine::STATE_CURRENT_UNIT_FINISHED;
	}


}
//=========================================================
// ユニット実行中（終了を待機）：選択肢
//=========================================================
void ScriptEngine::UnitProcessing_Select ( ScriptUnit &unit ) {

	// 読み返しモードの最中は選択を受け付けませんよ。
	/*
		Update()はしていないのですが、ここで_select->GetSelectOption();を呼んでいる。
		すると、キーボードでは選べてしまうのでした。
	*/
	if ( _textMessageWindow != NULL && _textMessageWindow->IsScenarioLogMode() ) {
		return;
	}


	// 選択されたかな？
	string selectOption = _select->GetSelectOption();

	if ( selectOption != "" ) {

		WriteScenarioLog ( "\t選択 → " + _select->GetSelectText() + "\n" );

		// 次に実行するユニットのラベルとする。：なんか裏技っぽい？ 日の当たらない世界？
		unit.nextUnitLabel = selectOption;

		// 選択肢が出てないときには、更新処理の負担がないように。
		_select->Clear();

		// 処理終了
		_state = ScriptEngine::STATE_CURRENT_UNIT_FINISHED;
	}

}

//=========================================================
// ユニット実行中（終了を待機）：ファイル選択
//=========================================================
void ScriptEngine::UnitProcessing_FileSelect ( ScriptUnit &unit ) {

	if ( _fileExplorer == NULL ) {

		printf ( "ファイル選択ウィンドウがない？ おかしいですね\n" );

		// 空文字を選択したことにする。後のことは知ーらない
		if ( _fileSelectVariableName != "" ) {
			SetLocalString ( _fileSelectVariableName, "" );
		}
		// 処理終了
		_state = ScriptEngine::STATE_CURRENT_UNIT_FINISHED;

		return;
	}


	// 選択されたかな？
	string selectedFile = _fileExplorer->GetSelectedFileName();

	if ( selectedFile != "" ) {

		printf ( "ファイルを選択したよ！ %s\n", selectedFile.c_str() );

		if ( _fileSelectVariableName != "" ) {
			SetLocalString ( _fileSelectVariableName, selectedFile );

		} else {
			printf ("ファイルを選択したけれど、結果を入れる変数の変数名が指定されていないので何もしません。\n");
		}

		delete _fileExplorer;
		_fileExplorer = NULL;

		// 処理終了
		_state = ScriptEngine::STATE_CURRENT_UNIT_FINISHED;
		
		return;
	}
	
	// 閉じるボタンかな？
	if ( _fileExplorer->IsTerminated() ) {

		printf ( "閉じるボタンでした\n" );

		// 空文字を選択したことにする。後のことは知ーらない
		SetLocalString ( _fileSelectVariableName, "" );
		
		delete _fileExplorer;
		_fileExplorer = NULL;

		// 処理終了
		_state = ScriptEngine::STATE_CURRENT_UNIT_FINISHED;
		
		return;
	}

}




//=========================================================
// ユニット実行中（終了を待機）：イベント監視
//=========================================================
void ScriptEngine::UnitProcessing_WatchEvent ( ScriptUnit &unit ) {

	///// え？ 監視するイベントがない？ おかしな話だけど、処理は終了としておくよ。気が利くね！
	if ( _watchEventDataList.size() == 0 ) {

		// 処理終了
		_state = ScriptEngine::STATE_CURRENT_UNIT_FINISHED;
	}

	///// 最終着手を更新しますか？
	bool isUpdateLastMove = false;
	AI_Modules::Move lastMove;

	///// 監視するイベントのリストでループですよ。
	for ( unsigned int i = 0; i < _watchEventDataList.size(); ++i ) {

		///// ゲームの指し手？
		if ( _watchEventDataList[i].type == "GameMove" ) {

			// 対象のゲーム盤、ありますよね？
			GameManager *gameManager = GameManager_Get ( _watchEventDataList[i].gameManagerName );
			if ( gameManager == NULL ) {
				continue; // あっ！ フリーズ確定だ！ でも、ここに来てるからには、あったはず。
			}

			// 最終着手を見せていただきます。
			lastMove = gameManager->GetLastMove();

			///// 変化があったら、イベント。
			if ( !AI_Modules::AI::MoveEquals ( _watchEventDataList[i].move_Before, lastMove ) ) {

				///// あとで、みんなに対して、更新しておいてね。この場で更新できないのはなぜか？
				isUpdateLastMove = true;

				// アクションだけ？
				if ( _watchEventDataList[i].moveAction != -1 ) {

					// 待ち受けていたアクション？
					if ( lastMove.action == _watchEventDataList[i].moveAction ) {

						bool isReallyOk = false;

						// 駒移動のときは、ぶつけるフラグも見るよ。
						if ( lastMove.action == AI_Modules::MOVE::ACTION::MOVE_PIECE ) {

							// 設定ある？
							if (_watchEventDataList[i].isCollide != "") {

								// じゃあ、bool として見ましょう。
								if ( lastMove.movePieceAndIsCllide == Utils_Convert::String_to_bool ( _watchEventDataList[i].isCollide ) ) {
									isReallyOk = true;
								}

							// 設定ないなら、もうぶつけててもぶつけてなくても、移動は移動、で。
							} else {
								isReallyOk = true;
							}

						} else {
							isReallyOk = true;
						}

						if ( isReallyOk ) {
							_watchEventDataList[i].count -= 1;
						}
					}

				// 指し手自体を見るよ。
				} else {

					// 待ち受けているのと同じ？
					if ( AI_Modules::AI::MoveEquals ( _watchEventDataList[i].move_Before, lastMove ) ) {
						_watchEventDataList[i].count -= 1;
					}
				}
			}


		///// ボタン？
		} else if ( _watchEventDataList[i].type == "Button") {

			// 対象のボタン、ありますよね？
			ScriptButton *button = Button_Get ( _watchEventDataList[i].buttonName );
			if ( button == NULL ) {
				continue; // あっ！ フリーズ確定だ！
			}

			// 押下されてたら、カウントを減らす。
			if ( button->IsRelease() ) {

				_watchEventDataList[i].count -= 1;
			}
		}

		///// カウントゼロで発動
		if ( _watchEventDataList[i].count <= 0 ) {

			// 次に実行するユニットのラベルとする。：なんか裏技っぽい？ 日の当たらない世界？
			unit.nextUnitLabel = _watchEventDataList[i].jumpToLabel;

			// 処理終了
			_state = ScriptEngine::STATE_CURRENT_UNIT_FINISHED;

			// もう、このイベントは起きたので、監視対象からははずしておく
			UTILS_OTHERS::deleteFromVector<WatchEventData> (_watchEventDataList, i);

			// おっと、ループからも出るのですよー。
			break;
		}

	}


	///// 「最終着手」を更新これは、全監視データに対して行う必要があるのですね。
	if ( isUpdateLastMove ) {
		for ( unsigned  int n = 0; n < _watchEventDataList.size(); ++n ) {
			_watchEventDataList[n].move_Before = lastMove;
		}
	}
}

//=========================================================
// ユニット実行中（終了を待機）：ゲーム盤の操作を監視
//=========================================================
void ScriptEngine::UnitProcessing_WatchGameOperation ( ScriptUnit &unit ) {

	///// え？ 監視するイベントがない？ おかしな話だけど、処理は終了としておくよ。気が利くね！
	if ( _watchGameOperationDataList.size() == 0 ) {

		// 処理終了
		_state = ScriptEngine::STATE_CURRENT_UNIT_FINISHED;
	}

	///// 監視対象に含まれているゲーム盤の更新処理（操作受付のみ）を呼び出す
	set<string> gameManagerNameList;
	for ( unsigned int i = 0; i < _watchGameOperationDataList.size(); ++i ) {

		if ( gameManagerNameList.find ( _watchGameOperationDataList[i].gameManagerName ) == gameManagerNameList.end() ) {
			gameManagerNameList.insert ( _watchGameOperationDataList[i].gameManagerName );
		}
	}
	for ( set<string>::iterator it = gameManagerNameList.begin(); it != gameManagerNameList.end(); ++it ) {

		GameManager *obj = GameManager_Get ( *it );
		if ( obj != NULL ) {
			
			obj->StartTutorialEvent();
		
			obj->Update_Operation();
		}
	}


	///// 最終着手を更新しますか？
	bool isUpdateLastMove = false;
	AI_Modules::Move lastMove;
	int              currentTurnCount = 0;

	///// 監視するイベントのリストでループですよ。
	for ( unsigned int i = 0; i < _watchGameOperationDataList.size(); ++i ) {

		// 対象のゲーム盤、ありますよね？
		GameManager *gameManager = GameManager_Get ( _watchGameOperationDataList[i].gameManagerName );
		if ( gameManager == NULL ) {
			continue; // あっ！ フリーズ確定だ！ でも、ここに来てるからには、あったはず。
		}

		// 最終着手を見せていただきます。
		lastMove = gameManager->GetLastMove();
		currentTurnCount = gameManager->GetScoreRecordLength();

		///// 変化があったら、イベント。
		//if ( !AI_Modules::AI::MoveEquals ( _watchGameOperationDataList[i].move_Before, lastMove ) ) {
		if ( _watchGameOperationDataList[i].scoreLength_Before != currentTurnCount ) {

			//printf ("★★★★★★★★★★★★★変化があった\n");

			///// あとで、みんなに対して、更新しておいてね。この場で更新できないのはなぜか？
			isUpdateLastMove = true;

			// アクションだけ？
			if ( _watchGameOperationDataList[i].moveAction != -1 ) {

				// 待ち受けていたアクション？
				if ( lastMove.action == _watchGameOperationDataList[i].moveAction ) {

					bool isReallyOk = false;

					// 駒移動のときは、ぶつけるフラグも見るよ。
					if ( lastMove.action == AI_Modules::MOVE::ACTION::MOVE_PIECE ) {

						// 設定ある？
						if (_watchGameOperationDataList[i].isCollide != "") {

							// じゃあ、bool として見ましょう。
							if ( lastMove.movePieceAndIsCllide == Utils_Convert::String_to_bool ( _watchGameOperationDataList[i].isCollide ) ) {
								isReallyOk = true;
							}

						// 設定ないなら、もうぶつけててもぶつけてなくても、移動は移動、で。
						} else {
							isReallyOk = true;
						}

					} else {
						isReallyOk = true;
					}

					if ( isReallyOk ) {
						_watchGameOperationDataList[i].count -= 1;
					}
				}

			// 指し手自体を見るよ。
			} else {

				// 待ち受けているのと同じ？
				if ( AI_Modules::AI::MoveEquals ( _watchGameOperationDataList[i].move_Before, lastMove ) ) {
					_watchGameOperationDataList[i].count -= 1;
				}
			}
		}


		///// カウントゼロで発動
		if ( _watchGameOperationDataList[i].count <= 0 ) {

			// 次に実行するユニットのラベルとする。：なんか裏技っぽい？ 日の当たらない世界？
			unit.nextUnitLabel = _watchGameOperationDataList[i].jumpToLabel;

			// 処理終了
			_state = ScriptEngine::STATE_CURRENT_UNIT_FINISHED;

			// もう、このイベントは起きたので、監視対象からははずしておく
			UTILS_OTHERS::deleteFromVector<WatchGameOperationData> (_watchGameOperationDataList, i);

			// おっと、ループからも出るのですよー。
			break;
		}

	}


	///// 「最終着手」を更新これは、全監視データに対して行う必要があるのですね。
	if ( isUpdateLastMove ) {
		for ( unsigned  int n = 0; n < _watchGameOperationDataList.size(); ++n ) {
			_watchGameOperationDataList[n].move_Before = lastMove;
			_watchGameOperationDataList[n].scoreLength_Before = currentTurnCount;
		}
	}
}
//=========================================================
// ユニット実行中（終了を待機）：待機
//=========================================================
void ScriptEngine::UnitProcessing_Wait ( ScriptUnit &unit ) {

	///// 早送り中はウェイトなし
	if  ( IsSkip() ) {
		_waitFrame = 0;
	}

	///// ウェイトカウントを減らして、
	if ( _waitFrame-- <= 0) {

		// 処理終了
		_state = ScriptEngine::STATE_CURRENT_UNIT_FINISHED;
	}

}
//=========================================================
// ユニット実行中（終了を待機）：クリック待機
//=========================================================
void ScriptEngine::UnitProcessing_WaitClick ( ScriptUnit &unit ) {

	///// 左クリックした？
	if ( Application::GetInstance()->_mouseState_Left == MOUSE::BUTTON::STATE::RELEASE ) {

		// 処理終了
		_state = ScriptEngine::STATE_CURRENT_UNIT_FINISHED;
	}

}
//=========================================================
// 処理なしユニット群
//=========================================================
void ScriptEngine::UnitProcessing_BGM_Regist   ( ScriptUnit &unit ) { _state = ScriptEngine::STATE_CURRENT_UNIT_FINISHED; }
void ScriptEngine::UnitProcessing_BGM_Change   ( ScriptUnit &unit ) { _state = ScriptEngine::STATE_CURRENT_UNIT_FINISHED; }
void ScriptEngine::UnitProcessing_BGM_Config   ( ScriptUnit &unit ) { _state = ScriptEngine::STATE_CURRENT_UNIT_FINISHED; }
void ScriptEngine::UnitProcessing_Sound_Regist ( ScriptUnit &unit ) { _state = ScriptEngine::STATE_CURRENT_UNIT_FINISHED; }
void ScriptEngine::UnitProcessing_Sound        ( ScriptUnit &unit ) { _state = ScriptEngine::STATE_CURRENT_UNIT_FINISHED; }
void ScriptEngine::UnitProcessing_Print        ( ScriptUnit &unit ) { _state = ScriptEngine::STATE_CURRENT_UNIT_FINISHED; }
void ScriptEngine::UnitProcessing_SystemMessage( ScriptUnit &unit ) { _state = ScriptEngine::STATE_CURRENT_UNIT_FINISHED; }
void ScriptEngine::UnitProcessing_TextLabel          ( ScriptUnit &unit ) { _state = ScriptEngine::STATE_CURRENT_UNIT_FINISHED; }
void ScriptEngine::UnitProcessing_TextAlias_Regist   ( ScriptUnit &unit ) { _state = ScriptEngine::STATE_CURRENT_UNIT_FINISHED; }
void ScriptEngine::UnitProcessing_Protagonist_Regist ( ScriptUnit &unit ) { _state = ScriptEngine::STATE_CURRENT_UNIT_FINISHED; }
void ScriptEngine::UnitProcessing_Image_Regist       ( ScriptUnit &unit ) { _state = ScriptEngine::STATE_CURRENT_UNIT_FINISHED; }
void ScriptEngine::UnitProcessing_Variable_Define    ( ScriptUnit &unit ) { _state = ScriptEngine::STATE_CURRENT_UNIT_FINISHED; }
void ScriptEngine::UnitProcessing_Variable_Release   ( ScriptUnit &unit ) { _state = ScriptEngine::STATE_CURRENT_UNIT_FINISHED; }
void ScriptEngine::UnitProcessing_Variable_Operation ( ScriptUnit &unit ) { _state = ScriptEngine::STATE_CURRENT_UNIT_FINISHED; }
void ScriptEngine::UnitProcessing_Variable_Console   ( ScriptUnit &unit ) { _state = ScriptEngine::STATE_CURRENT_UNIT_FINISHED; }
void ScriptEngine::UnitProcessing_IF                 ( ScriptUnit &unit ) { _state = ScriptEngine::STATE_CURRENT_UNIT_FINISHED; }
void ScriptEngine::UnitProcessing_Switch             ( ScriptUnit &unit ) { _state = ScriptEngine::STATE_CURRENT_UNIT_FINISHED; }
void ScriptEngine::UnitProcessing_Gosub              ( ScriptUnit &unit ) { _state = ScriptEngine::STATE_CURRENT_UNIT_FINISHED; }
void ScriptEngine::UnitProcessing_Button             ( ScriptUnit &unit ) { _state = ScriptEngine::STATE_CURRENT_UNIT_FINISHED; }
void ScriptEngine::UnitProcessing_Game               ( ScriptUnit &unit ) { _state = ScriptEngine::STATE_CURRENT_UNIT_FINISHED; }
void ScriptEngine::UnitProcessing_Game_Release       ( ScriptUnit &unit ) { _state = ScriptEngine::STATE_CURRENT_UNIT_FINISHED; }
void ScriptEngine::UnitProcessing_Put                ( ScriptUnit &unit ) { _state = ScriptEngine::STATE_CURRENT_UNIT_FINISHED; }
void ScriptEngine::UnitProcessing_Remove             ( ScriptUnit &unit ) { _state = ScriptEngine::STATE_CURRENT_UNIT_FINISHED; }
void ScriptEngine::UnitProcessing_RemoveAll          ( ScriptUnit &unit ) { _state = ScriptEngine::STATE_CURRENT_UNIT_FINISHED; }
void ScriptEngine::UnitProcessing_Jump               ( ScriptUnit &unit ) { _state = ScriptEngine::STATE_CURRENT_UNIT_FINISHED; }
void ScriptEngine::UnitProcessing_Title              ( ScriptUnit &unit ) { _state = ScriptEngine::STATE_CURRENT_UNIT_FINISHED; }
void ScriptEngine::UnitProcessing_Chapter            ( ScriptUnit &unit ) { _state = ScriptEngine::STATE_CURRENT_UNIT_FINISHED; }

void ScriptEngine::UnitProcessing_Save               ( ScriptUnit &unit ) { _state = ScriptEngine::STATE_CURRENT_UNIT_FINISHED; }
void ScriptEngine::UnitProcessing_Load               ( ScriptUnit &unit ) { _state = ScriptEngine::STATE_CURRENT_UNIT_FINISHED; }
void ScriptEngine::UnitProcessing_WriteSaveData      ( ScriptUnit &unit ) { _state = ScriptEngine::STATE_CURRENT_UNIT_FINISHED; }
void ScriptEngine::UnitProcessing_ClickableImage     ( ScriptUnit &unit ) { _state = ScriptEngine::STATE_CURRENT_UNIT_FINISHED; }
void ScriptEngine::UnitProcessing_ClearClickableImage( ScriptUnit &unit ) { _state = ScriptEngine::STATE_CURRENT_UNIT_FINISHED; }
void ScriptEngine::UnitProcessing_DefineControlTag   ( ScriptUnit &unit ) { _state = ScriptEngine::STATE_CURRENT_UNIT_FINISHED; }
void ScriptEngine::UnitProcessing_TextWindowDesign   ( ScriptUnit &unit ) { _state = ScriptEngine::STATE_CURRENT_UNIT_FINISHED; }
void ScriptEngine::UnitProcessing_Skipping           ( ScriptUnit &unit ) { _state = ScriptEngine::STATE_CURRENT_UNIT_FINISHED; }
//=========================================================
// イメージリスト関連：更新
//=========================================================
void ScriptEngine::Image_Update ( sf::RenderWindow &window ) {

	for ( unsigned int i = 0; i < _imageList.size(); ++i ) {

		_imageList[i]->Update ( window );
	}

}
//=========================================================
// イメージリスト関連：描画
//=========================================================
void ScriptEngine::Image_Draw_Back   ( sf::RenderWindow &window ) {

	for ( unsigned int i = 0; i < _imageList.size(); ++i ) {

		// Z値 100 に達したら背景終了（100を描画しない）
		if ( _imageList[i]->GetZ() >= 100) {
			break;
		}

		_imageList[i]->Draw ( window );
	}
}
void ScriptEngine::Image_Draw_Object   ( sf::RenderWindow &window ) {

	for ( unsigned int i = 0; i < _imageList.size(); ++i ) {

		// Z値 100 以上のものを描画（100を描画する）
		if ( _imageList[i]->GetZ() >= 100) {
			_imageList[i]->Draw ( window );
		}
	}
}
//=========================================================
// イメージリスト関連：Zソート
//=========================================================
void ScriptEngine::Image_ZSort  () {

	if ( _imageList.size() == 0 ) {
		return;
	}

	bool isSwap = true;
	while ( isSwap ) {

		isSwap = false;

		for ( unsigned int i = 0; i < _imageList.size() - 1; ++i ) {

			if ( _imageList[i]->GetZ() > _imageList[i + 1]->GetZ() ) {

				isSwap = true;

				Image *first  = _imageList[i];
				Image *second = _imageList[i+1];

				_imageList[i]   = second;
				_imageList[i+1] = first;
			}
		}
	}

}
//=========================================================
// イメージリスト関連：ラベルを指定して取得
//=========================================================
Image* ScriptEngine::Image_Get    ( string label ) {

	Image *result = NULL;

	for ( unsigned int i = 0; i < _imageList.size(); ++i ) {

		if ( _imageList[i]->GetLabel() == label ) {
			result = _imageList[i];
			break;
		}
	}

	return result;
}
//=========================================================
// イメージリスト関連：クリア
//=========================================================
void ScriptEngine::Image_Release () {

	for ( unsigned int i = 0; i < _imageList.size(); ++i ) { delete _imageList[i];}
	_imageList.clear();

}
void ScriptEngine::Image_Release ( string label ) {

	for ( vector<Image*>::iterator it = _imageList.begin(); it != _imageList.end(); ++it ) {

		if ( (*it)->GetLabel() == label ) {
			delete (*it);
			_imageList.erase ( it );
			break;
		}

	}
}



//=========================================================
// スクリプトボタン関連：更新
//=========================================================
void ScriptEngine::Button_Update ( sf::RenderWindow &window ) {

	for ( map<string, ScriptButton*>::iterator it = _scriptButtonList.begin(); it != _scriptButtonList.end(); ++it) {

		it->second->Update ( window );

		// 押した！ さぁ、どうします？
		if ( it->second->IsRelease() ) {

			printf ("%s を押しましたよー。\n", it->second->GetJumpToLabel().c_str());

			Gosub ( it->second->GetJumpToLabel() );

			break;
		}
	}


}

//=========================================================
// スクリプトボタン関連：描画
//=========================================================
void ScriptEngine::Button_Draw   ( sf::RenderWindow &window ) {


	for ( map<string, ScriptButton*>::iterator it = _scriptButtonList.begin(); it != _scriptButtonList.end(); ++it) {

		it->second->Draw ( window );

	}
}


//=========================================================
// スクリプトボタン関連：名前を指定して取得
//=========================================================
ScriptButton* ScriptEngine::Button_Get ( string name ) {

	ScriptButton *result = NULL;

	map<string, ScriptButton*>::iterator it = _scriptButtonList.find ( name );

	if ( it != _scriptButtonList.end() ) {
		result = it->second;
	}

	return result;
}


//=========================================================
// スクリプトボタン関連：クリア
//=========================================================
void ScriptEngine::Button_Release () {

	for ( map<string, ScriptButton*>::iterator it = _scriptButtonList.begin(); it != _scriptButtonList.end(); ++it) {

		delete it->second;

	}

	_scriptButtonList.clear();
}


//=========================================================
// テキストラベル関連：更新
//=========================================================
void ScriptEngine::TextLabel_Update    ( sf::RenderWindow &window ) {

	for ( map<string, TextLabel*>::iterator it = _textLabelList.begin(); it != _textLabelList.end(); ++it) {

		it->second->Update ( window );

	}
}

//=========================================================
// テキストラベル関連：描画
//=========================================================
void ScriptEngine::TextLabel_Draw      ( sf::RenderWindow &window ) {

	for ( map<string, TextLabel*>::iterator it = _textLabelList.begin(); it != _textLabelList.end(); ++it) {

		it->second->Draw ( window );

	}
}
//=========================================================
// テキストラベル関連：全て解放
//=========================================================
void ScriptEngine::TextLabel_Release   () {

	for ( map<string, TextLabel*>::iterator it = _textLabelList.begin(); it != _textLabelList.end(); ++it) {

		delete it->second;

	}
	_textLabelList.clear();
}
//=========================================================
// テキストラベル関連：名前を指定して解放
//=========================================================
void ScriptEngine::TextLabel_Release   ( string name ) {

	for ( map<string, TextLabel*>::iterator it = _textLabelList.begin(); it != _textLabelList.end(); ++it ) {

		if ( it->first == name ) {

			TextLabel *obj = it->second;

			delete obj;

			_textLabelList.erase ( it );

			break;
		}

	}
}

//=========================================================
// テキストラベル関連：アクセス
//=========================================================
TextLabel* ScriptEngine::TextLabel_Get ( string name ) {

	TextLabel *result = NULL;

	map<string, TextLabel*>::iterator it = _textLabelList.find ( name );

	if ( it != _textLabelList.end() ) {
		result = it->second;
	}

	return result;
}



//=========================================================
// ゲーム盤関連：クリア
//=========================================================
void ScriptEngine::GameManager_ReleaseAll () {

	while ( _gameManagerList.size() > 0 ) {
		delete _gameManagerList.begin()->second;
		_gameManagerList.erase( _gameManagerList.begin() );
	}
}
///// 名前を指定版
void ScriptEngine::GameManager_Release ( string name ) {

	printf ("ゲーム盤 [%s] を削除します\n", name.c_str());

	map<string, GameManager*>::iterator it = _gameManagerList.find ( name );
	if ( it != _gameManagerList.end() ) {

		delete it->second;
		_gameManagerList.erase ( it );
	} else {
		printf ("ゲーム盤 [%s] がありませんでした\n", name.c_str());
	}

}
//=========================================================
// ゲーム盤関連：更新
//=========================================================
void ScriptEngine::GameManager_Update( sf::RenderWindow &window) {
	
	// 投了ボタン更新は、ここで。
	_resignButton->Update(window);
	
	// 投了確認中ならば、ゲーム盤の更新は止めておく
	if ( _resignButton->IsComfirmWindowDisplaying() ) {
		return;
	}
	
	
	for ( map<string, GameManager*>::iterator it = _gameManagerList.begin(); it != _gameManagerList.end(); ++it ) {

		GameManager *gm = it->second;
		if (gm->IsActive()) {

			gm->Update(window);

			// 目的は棋譜再生モードにて盤面を作り替えた瞬間に表示がブレるのを防ぐこと。
			// UnitProcessing_AstralProjection() の側で見ているのが実質的な処理であって、ここでこれを見るのは形作り。
			// キチンとそろえるなら、gameManager->Update_Battle() のところでも描画お休みフラグを見るべきか。
			if (!gm->IsItOkayToBeRendered()) {
				_isRendered = false;
			}
		}
	}

}
//=========================================================
// ゲーム盤関連：描画
//=========================================================
void ScriptEngine::GameManager_Draw( sf::RenderWindow &window) {

	for ( map<string, GameManager*>::iterator it = _gameManagerList.begin(); it != _gameManagerList.end(); ++it ) {

		if (it->second->IsActive()) {
			it->second->Draw(window);
		}
	}
}
//=========================================================
// ゲーム盤関連：カードをつかんでいる？ 回転操作と右クリックメニューの確執
//=========================================================
bool ScriptEngine::GameManager_IsHoldingCard () {

	bool result = false;

	for ( map<string, GameManager*>::iterator it = _gameManagerList.begin(); it != _gameManagerList.end(); ++it ) {

		if (it->second->IsActive()) {

			if ( it->second->IsHoldingCard() ) {
				result = true;
				break;
			}
		}
	}

	return result;
}

//=========================================================
// 棋譜描画
//=========================================================
void ScriptEngine::GameManager_Draw_ScoreSheet ( sf::RenderWindow &window ) {

	/*
		メッセージウィンドウの下になってしまうので、分けて描画するのです。
	*/

	for ( map<string, GameManager*>::iterator it = _gameManagerList.begin(); it != _gameManagerList.end(); ++it ) {

		if (it->second->IsActive()) {
			it->second->Draw_ScoreSheet(window);
		}
	}
}

//=========================================================
// ゲーム盤関連：取得
//=========================================================
GameManager* ScriptEngine::GameManager_Get ( string name ) {

	GameManager *result = NULL;

	map<string, GameManager*>::iterator it = _gameManagerList.find ( name );
	if ( it != _gameManagerList.end() ) {

		result = it->second;
	}

	return result;
}

//=========================================================
// 変数関連：この名前の変数は、数値ですか？：スコープは問いません
//=========================================================
bool ScriptEngine::IsInt ( string variableName ) {

	bool result = false;

	if ( _variable_Global_Int.find( variableName ) != _variable_Global_Int.end() ) {
		result = true;
	}
	else if ( _variable_Local_Int.find( variableName ) != _variable_Local_Int.end() ) {
		result = true;
	}

	return result;
}

//=========================================================
// 変数関連：この名前の変数は、文字列ですか？：スコープは問いません。
//=========================================================
bool ScriptEngine::IsString ( string variableName ) {

	bool result = false;

	if ( _variable_Global_String.find( variableName ) != _variable_Global_String.end() ) {
		result = true;
	}
	else if ( _variable_Local_String.find( variableName ) != _variable_Local_String.end() ) {
		result = true;
	}

	return result;
}

//=========================================================
// 変数関連：この名前の変数の値（数値）をください。：存在しない場合に、何が返ってくるかは、保証しません。
//=========================================================
int ScriptEngine::GetIntVariable ( string variableName ) {

	int result = 0;

	if ( _variable_Global_Int.find( variableName ) != _variable_Global_Int.end() ) {
		result = _variable_Global_Int.find( variableName )->second;
	}
	else if ( _variable_Local_Int.find( variableName ) != _variable_Local_Int.end() ) {
		result = _variable_Local_Int.find( variableName )->second;
	}

	return result;
}

//=========================================================
// 変数関連：この名前の変数の値（文字列）をください。：存在しない場合に、何が返ってくるかは、知りません。
//=========================================================
string ScriptEngine::GetStringVariable ( string variableName ) {

	string result = "";

	if ( _variable_Global_String.find( variableName ) != _variable_Global_String.end() ) {
		result = _variable_Global_String.find( variableName )->second;
	}
	else if ( _variable_Local_String.find( variableName ) != _variable_Local_String.end() ) {
		result = _variable_Local_String.find( variableName )->second;
	}

	return result;
}

//=========================================================
// 変数関連：この数値を、この名前の変数に代入してください：スコープは問いません。存在しない場合は無視してください。
//=========================================================
void ScriptEngine::SetIntValue ( string variableName, int value ) {

	if ( _variable_Global_Int.find( variableName ) != _variable_Global_Int.end() ) {

		_variable_Global_Int [ variableName ] = value;
		
		SaveGlobalData();

	}
	else if ( _variable_Local_Int.find( variableName ) != _variable_Local_Int.end() ) {

		_variable_Local_Int [ variableName ] = value;
	}
}

//=========================================================
// 変数関連：この文字列を、この名前の変数に代入してください：スコープは問いません。存在しない場合は無視してください。
//=========================================================
void ScriptEngine::SetStringValue ( string variableName, string value ) {

	if ( _variable_Global_String.find( variableName ) != _variable_Global_String.end() ) {

		_variable_Global_String [ variableName ] = value;
		
		SaveGlobalData();

	}
	else if ( _variable_Local_String.find( variableName ) != _variable_Local_String.end() ) {

		_variable_Local_String [ variableName ] = value;
	}
}
//=========================================================
// 変数関連：この文字列を、ローカル変数に代入してください。存在する場合は上書きです。
//=========================================================
void ScriptEngine::SetLocalString ( string variableName, string value ) {

	if ( _variable_Local_String.find( variableName ) != _variable_Local_String.end() ) {

		printf ("ローカル文字列変数が上書きされます  変数名:%s <- 値:%s\n", variableName.c_str(), value.c_str());
	}
	_variable_Local_String [ variableName ] = value;

	printf ("ローカル文字列変数に書き込まれます  変数名:%s <- 値:%s\n", variableName.c_str(), value.c_str());
}


//=========================================================
// セーブデータ用に、スクリプトの進行状況を作成
//=========================================================
ScriptEngine::SaveData_ScriptProgress ScriptEngine::GetSaveData_ScriptProgress() {

	ScriptEngine::SaveData_ScriptProgress result;

	///// プロセスのセーブデータリストを作成
	//for ( map<string, ScriptProcess*>::iterator it = _processList.begin(); it != _processList.end(); ++it ) {
	//
	//	result.processSaveDataList.push_back( ScriptProcess_SaveData ( *it->second ) );
	//}
	ScriptProcess *it = _currentProcess;
	while (it != NULL) {

		// セーブデータを作成して、リストに入れる。
		result.processSaveDataList.push_back( ScriptProcess_SaveData ( *it ) );

		// 呼び出し元に移る：最後がルートプロセスになる、という算段。呼び出し元がNULLになって。
		it = it->referingProcess;
	}

	//// 処理状態
	result.state               = _state;
	//result.currentProcessLabel = _currentProcessLabel;

	return result;
}


//=========================================================
// セーブデータからスクリプトの進行状況を再現
//=========================================================
void ScriptEngine::Load_ScriptProgress ( SaveData_ScriptProgress &saveData_ScriptProgress ) {

	///// 現在のプロセスは全部消してしまう。
	delete _currentProcess;  // これで呼び出し元までたどって全部消えてくれますよね？
	_currentProcess = NULL;

	///// セーブデータから再現して、受け取る
	ScriptProcess *refererProcess = NULL;
	//for ( unsigned int i = 0; i < saveData_ScriptProgress.processSaveDataList.size(); ++i ) {
	for ( int i = saveData_ScriptProgress.processSaveDataList.size() - 1;  i >= 0; --i ) { // 後ろからたどる

		// オブジェクトを作成
		string label = saveData_ScriptProgress.processSaveDataList[i].label;
		ScriptProcess *process = ReadProcess ( label );

		// 実行中だったユニット番号をセット
		process->currentUnitIndex = saveData_ScriptProgress.processSaveDataList[i].currentUnitIndex;

		// 呼び出し元をセット：ループの最初はNULL＝ルートプロセス
		process->referingProcess = refererProcess;

		// 最後の子は、カレントプロセスだった子
		//if ( i == saveData_ScriptProgress.processSaveDataList.size() - 1 ) {
		if ( i == 0) {
			_currentProcess = process;
		}

		// 今の子が、次の子の呼び出し元、ということ。
		refererProcess = process;
	}


	///// 状態を受け取る
	_state = saveData_ScriptProgress.state;

	// クイックセーブというか、試合中の直前でオートセーブしたデータを読み込むときの実験
	//_state = ScriptEngine::STATE_RESUME;

	/*
	///// 現在のプロセスは全部消してしまう。
	for ( map<string, ScriptProcess*>::iterator it = _processList.begin(); it != _processList.end(); ++it ) {

		delete it->second;
	}
	_processList.clear();

	///// セーブデータから再現して、受け取る
	for ( unsigned int i = 0; i < saveData_ScriptProgress.processSaveDataList.size(); ++i ) {

		string label = saveData_ScriptProgress.processSaveDataList[i].label;
		ScriptProcess *process = ReadProcess ( label );

		if ( process == NULL ) {
			printf("Error : プロセス \"%s\" が定義されていません。\n", label.c_str());
			continue;
		}

		// 呼び出し元と、カレントユニットインデックスをセット
		process->referingProcessLabel = saveData_ScriptProgress.processSaveDataList[i].referingProcessLabel;
		process->currentUnitIndex     = saveData_ScriptProgress.processSaveDataList[i].currentUnitIndex;

		// リストに受け取る
		_processList [ process->label ] = process;
	}


	///// 状態と、カレントプロセスラベルを受け取る
	_state               = saveData_ScriptProgress.state;
	_currentProcessLabel = saveData_ScriptProgress.currentProcessLabel;
	*/
}

//=========================================================
// セーブ情報作成：クリック画像監視情報
//=========================================================
string ScriptEngine::Serialize_WatchClickableImageData() {

	string result = "";


	for (unsigned int i = 0 ; i < _watchClickableImageDataList.size(); ++i) {

		if ( i > 0 ) {
			result += "<>";
		}

		WatchClickableImageData elem = _watchClickableImageDataList[i];

		result += elem.cImageName + "@" + elem.jumpToLabel;

	}
	
	//printf ( "セーブ情報作成：クリック画像監視情報 result = %s\n", result.c_str() );

	return result;
}

//=========================================================
// セーブデータから復元：クリック画像監視情報
//=========================================================
void ScriptEngine::DeSerialize_WatchClickableImageData( string str ) {
	
	//printf ( "セーブデータから復元：クリック画像監視情報 str = %s\n", str.c_str() );
	
	_watchClickableImageDataList.clear();
	
	if ( str == "" ) {
		printf ( "クリック画像監視情報なし\n" );
		return;
	}

	vector<string> rows = Utils_String::Split(str, "<>");
	for ( unsigned int i = 0; i < rows.size(); ++i ) {

		vector<string> elems = Utils_String::Split(rows[i], "@");

		if ( elems.size() == 2 ) {

			WatchClickableImageData data;

			data.cImageName  = elems[0];
			data.jumpToLabel = elems[1];

			_watchClickableImageDataList.push_back ( data );

		} else {
			printf("[セーブデータが壊れている？] セーブデータから復元：クリック画像監視情報 [%s] in [str=%s]\n", rows[i].c_str(), str.c_str());
		}

	}

}

//=========================================================
// セーブ情報作成：イベント待ち受け情報
//=========================================================
string ScriptEngine::Serialize_WatchEventData() {

	string result = "";

	//vector<WatchEventData>          _watchEventDataList;
/*
	//=========================================================
	// イベント監視情報
	//=========================================================
	struct WatchEventData {

		string           type;        // GameMove / Button
		AI_Modules::Move move_Before; // イベント監視開始時点での最終着手。
		AI_Modules::Move move;
		int              moveAction;
		string           isCollide; // true. false, 空文字で設定なし、とする。
		string           gameManagerName;
		string           buttonName;

		string jumpToLabel;
		int    count;
	};

*/
	string outerDelim = "<>";
	string innerDelim = "*";
	for ( unsigned int i = 0; i < _watchEventDataList.size(); ++i ) {

		if ( i > 0 ) { result += outerDelim; }

		WatchEventData data = _watchEventDataList[i];

		result += data.type                                   + innerDelim;
		result += SaveData::MoveToString ( data.move_Before ) + innerDelim;
		result += SaveData::MoveToString ( data.move        ) + innerDelim;
		result += std::to_string ( data.moveAction )          + innerDelim;
		result += data.isCollide                              + innerDelim;
		result += data.gameManagerName                        + innerDelim;
		result += data.buttonName                             + innerDelim;
		result += data.jumpToLabel                            + innerDelim;
		result += std::to_string ( data.count )               + innerDelim;
	}

	return result;
}

//=========================================================
// セーブデータから復元：イベント待ち受け情報
//=========================================================
void   ScriptEngine::DeSerialize_WatchEventData( string str ) {

	_watchEventDataList.clear();

	if ( str == "" ) {
		return;
	}

	string outerDelim = "<>";
	string innerDelim = "*";

	vector<string> dataList = Utils_String::Split ( str, outerDelim );

	for ( string data : dataList ) {

		vector<string> aaaaaaaa = Utils_String::Split ( data, innerDelim );

		if ( aaaaaaaa.size() < 9 ) {
			printf ( "セーブデータが壊れています：ScriptEngine::DeSerialize_WatchEventData( string str[%s] )\n", str.c_str() );
			continue;
		}

		WatchEventData watchEventData;
		watchEventData.type            = aaaaaaaa[0];
		watchEventData.move_Before     = SaveData::MoveFromString ( aaaaaaaa[1] );
		watchEventData.move            = SaveData::MoveFromString ( aaaaaaaa[2] );
		watchEventData.moveAction      = std::stoi ( aaaaaaaa[3] );
		watchEventData.isCollide       = aaaaaaaa[4];
		watchEventData.gameManagerName = aaaaaaaa[5];
		watchEventData.buttonName      = aaaaaaaa[6];
		watchEventData.jumpToLabel     = aaaaaaaa[7];
		watchEventData.count           = std::stoi ( aaaaaaaa[8] );

		_watchEventDataList.push_back ( watchEventData );
	}

}

//=========================================================
// セーブ情報作成：盤面操作イベント待ち受け情報
//=========================================================
string ScriptEngine::Serialize_WatchGameOperationData() {

	string result = "";

	//vector<WatchGameOperationData>  _watchGameOperationDataList;

/*
	//=========================================================
	// ゲーム盤の操作イベント監視情報
	//=========================================================
	struct WatchGameOperationData {

		AI_Modules::Move move_Before; // イベント監視開始時点での最終着手。
		int              scoreLength_Before; // これまでの手数
		AI_Modules::Move move;
		int              moveAction;
		string           isCollide; // true. false, 空文字で設定なし、とする。
		string           gameManagerName;

		string jumpToLabel;
		int    count;
	};
*/
	string outerDelim = "<>";
	string innerDelim = "*";
	for ( unsigned int i = 0; i < _watchGameOperationDataList.size(); ++i ) {

		if ( i > 0 ) { result += outerDelim; }

		WatchGameOperationData data = _watchGameOperationDataList[i];

		result += SaveData::MoveToString ( data.move_Before ) + innerDelim;
		result += std::to_string ( data.scoreLength_Before )  + innerDelim;
		result += SaveData::MoveToString ( data.move        ) + innerDelim;
		result += std::to_string ( data.moveAction )          + innerDelim;
		result += data.isCollide                              + innerDelim;
		result += data.gameManagerName                        + innerDelim;
		result += data.jumpToLabel                            + innerDelim;
		result += std::to_string ( data.count )               + innerDelim;
	}


	return result;
}

//=========================================================
// セーブデータから復元：盤面操作イベント待ち受け情報
//=========================================================
void   ScriptEngine::DeSerialize_WatchGameOperationData( string str ) {

	_watchGameOperationDataList.clear();

	if ( str == "" ) {
		return;
	}

	string outerDelim = "<>";
	string innerDelim = "*";

	vector<string> dataList = Utils_String::Split ( str, outerDelim );

	for ( string data : dataList ) {

		vector<string> aaaaaaaa = Utils_String::Split ( data, innerDelim );

		if ( aaaaaaaa.size() < 8 ) {
			printf ( "セーブデータが壊れています：ScriptEngine::DeSerialize_WatchGameOperationData( string str[%s] )\n", str.c_str() );
			continue;
		}

		WatchGameOperationData watchGameOperationData;
		watchGameOperationData.move_Before        = SaveData::MoveFromString ( aaaaaaaa[0] );
		watchGameOperationData.scoreLength_Before = std::stoi ( aaaaaaaa[1] );
		watchGameOperationData.move                = SaveData::MoveFromString ( aaaaaaaa[2] );
		watchGameOperationData.moveAction          = std::stoi ( aaaaaaaa[3] );
		watchGameOperationData.isCollide           = aaaaaaaa[4];
		watchGameOperationData.gameManagerName     = aaaaaaaa[5];
		watchGameOperationData.jumpToLabel         = aaaaaaaa[6];
		watchGameOperationData.count               = std::stoi ( aaaaaaaa[7] );

		_watchGameOperationDataList.push_back ( watchGameOperationData );
	}
}

//=========================================================
// セーブデータ作成：システム部品というかその他というか、入れ忘れというか。
//=========================================================
string ScriptEngine::Serialize_Miscellaneous () {

	/*
		[対象]
		_endAstralProjectionButton

		マウスにくっついているオブジェクト（駒 or カード） → と思ったけれど、これの面倒を見るのは「くっつけた側」とする。

	*/
	string result = "";

	string outerDelim = "<->";
	string innerDelim = "@*@";

	if ( _endAstralProjectionButton != NULL ) {

		ClickableImage::SaveData clickableImage = _endAstralProjectionButton->GetSaveData();

		/*
		struct Property {

			string       imageFile;
			sf::Vector2f size;
			sf::IntRect  rect_Normal;
			sf::IntRect  rect_Over;
			sf::IntRect  rect_Push;

			string name;
		};
		*/

		result += clickableImage.prop.imageFile + innerDelim;
		result += clickableImage.prop.name      + innerDelim;
		result += std::to_string ( (int)clickableImage.prop.size.x ) + innerDelim;
		result += std::to_string ( (int)clickableImage.prop.size.y ) + innerDelim;
		result += GUI_SFML::SFML_Utils::Convert_IntRect_to_string ( clickableImage.prop.rect_Normal ) + innerDelim;
		result += GUI_SFML::SFML_Utils::Convert_IntRect_to_string ( clickableImage.prop.rect_Over   ) + innerDelim;
		result += GUI_SFML::SFML_Utils::Convert_IntRect_to_string ( clickableImage.prop.rect_Push   ) + innerDelim;
		result += std::to_string ( (int)clickableImage.position.x ) + innerDelim;
		result += std::to_string ( (int)clickableImage.position.y );
	}

	///// テキスト欄のデザイン情報（現在使用中のID、と表示/非表示のみ）
	result += outerDelim;
	result += _currentTextWindowDesignID + innerDelim;
	result += ( _textMessageWindow != NULL && _textMessageWindow->IsShown() ) ? "true":"false";


	///// フェードの設定
	result += outerDelim;
	result += _fade->GetFadeTexture() + innerDelim;
	result += GUI_SFML::SFML_Utils::Convert_color_to_string(_fade->GetColor_FadeInStart(), "_" ) + innerDelim;
	result += GUI_SFML::SFML_Utils::Convert_color_to_string(_fade->GetColor_FadeOutStart(), "_" );


	/*
	// マウスにくっついているオブジェクト、は、GameManagerの管轄か
	result += outerDelim;

	if ( Application::GetInstance()->IsMouseClingObjExist() ) {

		string type  = "";
		string value = "";

		if ( Application::GetInstance()->IsMouseClingObj_Piece() ) {

			type = "PIECE";
			Piece *piece = Application::GetInstance()->RefferMouseClingObj();
			value = std::to_string ( piece->GetColor() );
		}
		else if ( Application::GetInstance()->IsMouseClingObj_Card() ) {

			type = "CARD";
			Card *card = Application::GetInstance()->RefferMouseClingObj();
			value = card->GetProperty().cardName;
		}

		result += type + innerDelim;
		result += value;
	}
	*/

	return result;
}

//=========================================================
// ロード：システム部品というかその他というか、入れ忘れというか。
//=========================================================
void ScriptEngine::DeSerialize_Miscellaneous ( string str ) {

	// 検討終了ボタンをクリア
	if ( _endAstralProjectionButton != NULL ) {
		delete _endAstralProjectionButton;
	}
	_endAstralProjectionButton = NULL;

	/*
	// マウスにくっついているものもクリア
	IMouseClingableObject *obj = Application::GetInstance()->DrawMouseClingObj();
	if (obj != NULL) {
		delete obj;
	}
	*/
	string outerDelim = "<->";
	string innerDelim = "@*@";

	vector<string> dataList = Utils_String::Split ( str, outerDelim );

	if ( dataList.size() < 1 ) {
		printf ("セーブデータが壊れていますー：DeSerialize_Miscellaneous ( string str[%s] )\n", str.c_str());
		return;
	}

	string endAstralProjectionButton = dataList[0];
	//string mouseClingObj             = dataList[1];

	if ( endAstralProjectionButton != "" ) {

		vector<string> aaaaaaaaa = Utils_String::Split ( endAstralProjectionButton, innerDelim );

		if ( aaaaaaaaa.size() < 9 ) {
			printf ("セーブデータが壊れています〜：DeSerialize_Miscellaneous ( string str[%s] )\n", str.c_str());
			return;
		}

		ClickableImage::SaveData saveData;

		saveData.prop.imageFile   = aaaaaaaaa[0];
		saveData.prop.name        = aaaaaaaaa[1];
		saveData.prop.size.x      = std::stoi ( aaaaaaaaa[2] );
		saveData.prop.size.y      = std::stoi ( aaaaaaaaa[3] );
		saveData.prop.rect_Normal = GUI_SFML::SFML_Utils::Convert_string_to_IntRect ( aaaaaaaaa[4] );
		saveData.prop.rect_Over   = GUI_SFML::SFML_Utils::Convert_string_to_IntRect ( aaaaaaaaa[5] );
		saveData.prop.rect_Push   = GUI_SFML::SFML_Utils::Convert_string_to_IntRect ( aaaaaaaaa[6] );
		saveData.position.x       = std::stoi ( aaaaaaaaa[7] );
		saveData.position.y       = std::stoi ( aaaaaaaaa[8] );

		ClickableImage::Property prop;
		_endAstralProjectionButton = new ClickableImage(prop);

		_endAstralProjectionButton->Load ( saveData );

	}

	///// テキスト欄のデザイン情報
	if ( dataList.size() >= 2 ) {

		string textWindowDesignInfo = dataList[1];

		//printf ( "[ロード]：テキスト欄のデザイン情報[%s]\n", textWindowDesignInfo.c_str());

		vector<string> aaaaaaaaa = Utils_String::Split ( textWindowDesignInfo, innerDelim );

		if ( aaaaaaaaa.size() < 2 ) {
			printf ("セーブデータが壊れています〜：DeSerialize_Miscellaneous ( string str[%s] )\n", textWindowDesignInfo.c_str());
			return;
		}

		// ID
		LoadTextWindowDesign (aaaaaaaaa[0]);

		// 表示/非表示
		if ( _textMessageWindow != NULL ) {
			_textMessageWindow->SetShown ( Utils_String::isPositiveWord(aaaaaaaaa[1]) );
		}

	} else {
		printf ( "[ロード]：テキスト欄のデザイン情報がない\n" );
	}

	///// フェード設定の情報
	if ( dataList.size() >= 3 ) {

		string fadeConfigInfo = dataList[2];

		vector<string> aaaaaaaaa = Utils_String::Split ( fadeConfigInfo, innerDelim );

		if ( aaaaaaaaa.size() < 3 ) {
			printf ("セーブデータが壊れています〜：DeSerialize_Miscellaneous ( string str[%s] )\n", fadeConfigInfo.c_str());
			return;
		}

		_fade->SetFadeTexture ( aaaaaaaaa[0] );
		_fade->SetColor_FadeInStart  ( GUI_SFML::SFML_Utils::Convert_string_to_color(aaaaaaaaa[1], "_") );
		_fade->SetColor_FadeOutStart ( GUI_SFML::SFML_Utils::Convert_string_to_color(aaaaaaaaa[2], "_") );

	} else {
		printf ( "[ロード]：フェード設定の情報がない\n" );
	}

	// マウスにくっついているオブジェクト、は、GameManagerの管轄か → くっつけた側の所有物、ということですね
	/*
	if ( mouseClingObj != "" ) {

		vector<string> bbbbb = Utils_String::Split ( mouseClingObj, innerDelim );

		if ( bbbbb.size() < 2 ) {
			printf ("セーブデータが壊れています：DeSerialize_Miscellaneous ( string str[%s] )\n", str.c_str());
			return;
		}

		string type  = bbbbb[0];
		string value = bbbbb[1];

		if ( type == "PIECE" ) {

			Piece *piece = new Piece( std::stoi( value ) );
			Application::GetInstance()->AddMouseClingObj(piece);

		}
		else if ( type == "CARD" ) {

			Card *card = CardFactory::GetCard ( value );
			Application::GetInstance()->AddMouseClingObj(card);
		}

	}
	*/


}


//=========================================================
// セーブ
//=========================================================
void ScriptEngine::SaveTo ( SaveData *saveData, bool isTakeSS, string message ) {

	printf("[ScriptEngine::SaveTo] ここは ScriptEngine 側ですよー\n");
	
	saveData->SetChapterName ( _chapterName );

	///// 試合中だったら、試合開始時点のオートセーブデータをコピーして終了
	if ( _battleData.phase != BattleData::PHASE_BATTLE_NOT_STARTED ) {

		string forBattle   = "./data/save/qsave/beforeBattle.sav";
		string forBattleSS = "./data/save/qsave/beforeBattle.png";
		string fileName   = saveData->GetFileName();
		string SSfileName = saveData->GetSSFileName();

		if (!Utils_File::Copy ( forBattle, fileName     ) ) { printf ("むむ！ ScriptEngine::SaveTo  %s, %s\n", forBattle.c_str(), fileName.c_str()); }
		//if (!Utils_File::Copy ( forBattleSS, SSfileName ) ) { printf ("むむ！ ScriptEngine::SaveTo  %s, %s\n", forBattleSS.c_str(), SSfileName.c_str()); }
		
		// SSを圧縮しつつ持ってくる
		sf::Texture texture;
		texture.loadFromFile ( forBattleSS );
		sf::Image image = texture.copyToImage();
		
		sf::Image compressedImage = CompressImage ( image, 160, 120 );
		compressedImage.saveToFile ( SSfileName );
		
		
		saveData->LoadAppearance();
		
		if ( message != "" ) {
			SystemMessage(message);
		}

		SystemMessage ( "試合中のため試合直前のデータで保存しました" );
		
		return;
	}


	saveData->Save_ScriptProgess       ( this );
	saveData->Save_LoclVariable_Int    ( _variable_Local_Int    );
	saveData->Save_LoclVariable_String ( _variable_Local_String );
	saveData->Save_TextMessageWindow   ( _textMessageWindow );
	saveData->Save_BGM                 ( _BGM  );
	saveData->Save_Camera              ( _camera );
	saveData->Save_SoundManager        ( _soundManager  );
	saveData->Save_Image               ( _imageList );
	saveData->Save_Button              ( _scriptButtonList );
	saveData->Save_Select              ( _select );
	saveData->Save_ClickableImage      ( _clickableImageList );
	saveData->Save_TextLabelList       ( _textLabelList );
	saveData->Save_GameManagerList     ( _gameManagerList);
	saveData->Save_WaitClickableImageDataList ( this );
	saveData->Save_WaitEventDataList          ( this );
	saveData->Save_WaitGameOperationDataList  ( this );
	saveData->Save_ScriptEngineMiscellaneous  ( this );

	/*
		ScriptEngine側で持つ部品もお世話する必要あり。
		つまり、
		検討モードの終了ボタン。
		マウスにくっついているオブジェクト。（つまり カード か 駒）

	*/

	if ( isTakeSS ) {
		// この SaveData のためにSSを作成しましょう。Draw() の中でやっている。
		_takeSSforThisSaveData = saveData;
	}

	saveData->Save();

	if ( message != "" ) {
		SystemMessage(message);
	}
}

//=========================================================
// ロード前に必要なクリア処理
//=========================================================
void ScriptEngine::LoadPreparation() {


	// 試合中にロードすると、試合中フラグが立ったままになる。
	// 試合中に試合前のオートセーブのデータをロードすると、BattleStart()の冒頭にてオートセーブのために SaveToを呼ぶと、
	// 「すでに試合中」だから、元のファイルを同じファイルにコピーしようとして失敗して空ファイルになってしまう。
	_battleData = BattleData();
	
	///// 投了ボタン非表示
	_resignButton->SetActive(false);
	
	// エフェクトを全部解放。エフェクトが出ている最中にロードすると、エフェクト側から参照しているボードデータなんかがない、などということになる
	Application::GetInstance()->CleanUpAllEffect();
	
	// チャプター名をクリア。
	_chapterName = "";
}

//=========================================================
// ロード
//=========================================================
void ScriptEngine::LoadFrom ( SaveData *saveData, string message ) {

	LoadPreparation();

	saveData->Load();

	saveData->Load_ScriptProgess       ( this );
	saveData->Load_LoclVariable_Int    ( _variable_Local_Int    );
	saveData->Load_LoclVariable_String ( _variable_Local_String );
	saveData->Load_TextMessageWindow   ( _textMessageWindow );
	saveData->Load_BGM                 ( _BGM );
	saveData->Load_Camera              ( _camera );
	saveData->Load_SoundManager        ( _soundManager );
	saveData->Load_Image               ( _imageList );
	saveData->Load_Button              ( _scriptButtonList );
	saveData->Load_Select              ( _select );
	saveData->Load_ClickableImage      ( _clickableImageList );
	saveData->Load_TextLabelList       ( _textLabelList, _font );
	saveData->Load_GameManagerList     ( _gameManagerList, this );
	saveData->Load_WaitClickableImageDataList ( this );
	saveData->Load_WaitEventDataList          ( this );
	saveData->Load_WaitGameOperationDataList  ( this );
	
	_chapterName = saveData->GetChapterName();
	

	// ロードしたので、メニューは閉じればいいと思う。
	_menu->SetActive ( false );
	
	// ファイル選択ウィンドウは消してしまっていいんですか？
	if  (_fileExplorer != NULL ) {
		delete _fileExplorer;
		_fileExplorer = NULL;
	}
	
	// フリーバトル選択ウィンドウも閉じてしまっていいかな？
	if ( _freeBattleSelector != NULL ) {
		_freeBattleSelector->SetActive(false);
	}

	// フェードアウトしたままロードすると真っ黒になってしまうので
	// ロード時にはフェードを解除しておきたい。
	// というか、案外、こういうことでよいのではないか？ ロードするとフェードイン。
	_fade->FadeIn(10);

	// TODO もしかすると、ここでファイル選択ウィンドウの状態を復元してくれるのかもしれない？
	saveData->Load_ScriptEngineMiscellaneous  ( this );

	// オブジェクトの表示位置が狂っているので、描画は１フレームお休み。
	//_isRestDraw = true;
	_isRendered = false;

	if ( message != "" ) {
		SystemMessage(message);
	}
	
	// 現在スキップ中のフラグはリセット
	_isSkippingNow = false;
}

//=========================================================
// クイックセーブ
//=========================================================
void ScriptEngine::QuickSave() {

	string fileName   = "./data/save/qsave/qsave.sav";
	SaveData *quickSaveData = new SaveData ( fileName, 0 );

	// すぐに delete してしまうので、SSの撮影は別ルートで行う
	SaveTo (quickSaveData, false, "クイックセーブしました");
	delete quickSaveData;

	// SSは不要。ロード時に画像がないとのエラーが出るが支障はない。気になるならダミーの画像ファイルを置いておけばよい。
	//string ssfileName = "./data/save/qsave/qsave.png";
	//_takeSStoThisFile = ssfileName;

}

//=========================================================
// クイックロード
//=========================================================
void ScriptEngine::QuickLoad() {

	string fileName   = "./data/save/qsave/qsave.sav";
	
	
	FILE *f = fopen ( fileName.c_str(), "r" );
	if ( f == NULL ) {
		
		SystemMessage ( "クイックセーブのデータがありません" );
		
		return;
		
	} else {
		
		fclose(f);	
		
		SaveData *quickSaveData = new SaveData ( fileName, 0 );

		LoadFrom ( quickSaveData, "クイックロードしました" );
		
		
	}

}



//=========================================================
// グローバル変数をセーブ
//=========================================================
void ScriptEngine::SaveGlobalData() {

	_globalSaveData->Save_GlobalVariable_Int    ( _variable_Global_Int    );
	_globalSaveData->Save_GlobalVariable_String ( _variable_Global_String );
	_globalSaveData->Save_ReadData              ( _readTextManager        );
	_globalSaveData->Save();
}


//=========================================================
// グローバル変数を、ファイルからロード
//========================================================
void ScriptEngine::LoadGlobalData() {

	_globalSaveData->Load();
	_globalSaveData->Load_GlobalVariable_Int    ( _variable_Global_Int    );
	_globalSaveData->Load_GlobalVariable_String ( _variable_Global_String );
	_globalSaveData->Load_ReadData              ( _readTextManager        );
}


//=========================================================
// コンフィグをセーブ
//=========================================================
void ScriptEngine::SaveConfig() {

	map<string, string> configData;

	configData[ CONFIG::KEY::VolumeBGM ] = std::to_string ( (int)GetVolumeBGM() );
	configData[ CONFIG::KEY::VolumeSE  ] = std::to_string ( (int)GetVolumeSE () );

	Utils_File::SaveToTextFile ( configData, CONFIG::FileName );

}

//=========================================================
// コンフィグを読み込み
//=========================================================
void ScriptEngine::LoadConfig() {

	map<string, string> configData = Utils_File::LoadFromTextFile ( CONFIG::FileName );

	if (configData.find( CONFIG::KEY::VolumeBGM ) != configData.end()) {
		SetVolumeBGM ( std::stoi( configData[ CONFIG::KEY::VolumeBGM ] ) );
	} else {
		SetVolumeBGM ( CONFIG::DEFAULT::VolumeBGM );
	}

	if (configData.find( CONFIG::KEY::VolumeSE ) != configData.end()) {
		SetVolumeSE ( std::stoi( configData[ CONFIG::KEY::VolumeSE] ) );
	} else {
		SetVolumeSE ( CONFIG::DEFAULT::VolumeSE );
	}
}


//=========================================================
// 既読管理への窓口係；今、何文字目まで読み終えてます？
//=========================================================
int ScriptEngine::GetReadFinishedLength () {

	/*
		つまり、今表示中のテキスト（タグ）の、既読文字数を取得、ということ。

		今現在、テキストタグを実行中じゃない場合は？ ゼロでもなんでも適当に。

		ここは、TextMesssageWindow からしか、呼ばれないはず、なのですよね？ うん。
	*/

	int result = 0;

	// つまり、これ。
 	// _processList[_currentProcessLabel]->currentUnitIndex;

	result =
		_readTextManager->GetReadFinishedLength (
			_currentProcess->label,
			_currentProcess->currentUnitIndex
		);


	return result;
}

//=========================================================
// 既読管理への窓口係；全部読み終えた
//=========================================================
void ScriptEngine::Finished () {

	// つまり、これ。
 	// _processList[_currentProcessLabel]->currentUnitIndex;

	_readTextManager->Finished (
		_currentProcess->label,
		_currentProcess->currentUnitIndex
	);
}

//=========================================================
// 既読管理への窓口係：ｎ文字目まで読み終えた
//=========================================================
void ScriptEngine::Finished ( int charLength ) {

	// つまり、これ。
 	// _processList[_currentProcessLabel]->currentUnitIndex;

	_readTextManager->Finished (
		_currentProcess->label,
		_currentProcess->currentUnitIndex,
		charLength
	);
}

//=========================================================
// 既読管理への窓口係；今、既読ならば、スキップするべき？
//=========================================================
bool ScriptEngine::IsSkip() {

	/*
		要は、単に操作方法の問題。
	*/

	bool result = false;

	// とりあえず、Ctrlキーを押している間、としておく。
	// → Ctrlキーはリリースで「オプションの既読スキップフラグ」の切り替えとする
	if
	(
		_hasFocus &&
		(
			//sf::Keyboard::isKeyPressed ( sf::Keyboard::RControl ) ||
			//sf::Keyboard::isKeyPressed ( sf::Keyboard::LControl ) ||
			sf::Keyboard::isKeyPressed ( sf::Keyboard::Return )       // 既読じゃなくても強制スキップ TODO TextMessageWindowの側でも二重にEnterキーを見ている。いいのかな？ 悪くはない。悪くは。
		)
	)
	{
		result = true;

	// Ctrlキーは押していないけれど、メニューで「既読スキップ」をONにしていたら、スキップする
	} else {
		
		if ( _menu->GetItemValue ( "IsSkip" ) == "on" && _isSkippingNow ) {
			result = true;
		}
	}


	return result;
}
//=========================================================
// 設定として、既読をスキップして欲しいということになっているかどうか
//=========================================================
bool ScriptEngine::IsWannaSkip() {
	
	bool result = false;
	
	if ( _menu->GetItemValue ( "IsSkip" ) == "on") {
		result = true;
	}
	
	return result;
}
//=========================================================
// 既読スキップ終了
//=========================================================
void ScriptEngine::SkipEnd() {

	_menu->SetItemValue ( "IsSkip", "off" );

}
//=========================================================
// 現在、既読スキップ実行中
//=========================================================
void ScriptEngine::SkippingNow ( bool arg ) {
	_isSkippingNow = arg;
}



//=========================================================
// 音量設定：BGM
//=========================================================
void  ScriptEngine::SetVolumeBGM ( float value ) {
	_BGM->SetVolume ( value );
}
float ScriptEngine::GetVolumeBGM () {
	return _BGM->GetVolume();
}

//=========================================================
// 音量設定：SE
//=========================================================
void  ScriptEngine::SetVolumeSE ( float value ) {
	_soundManager->SetVolume ( value );
	GUI_SFML::SoundPlayer::SetVolume ( value );
}
float ScriptEngine::GetVolumeSE () {

	// 音を出すオブジェクトが SoundManager と SoundPlayer と２種類あるのが問題
	// ScriptEngine としての音量を取得、というときには、ここでは SoundManager の値を回答することにする。

	return _soundManager->GetVolume();
}


//=========================================================
// 読み返しログ書き込み
//=========================================================
void   ScriptEngine::WriteScenarioLog (string log) {
	
	if ( Utils_String::Trim ( log ) == "" ) {
		return;
	}
	
	//_scenarioLog += log;
	_scenarioLog += log + "\n";
	
	///// ときどきサイズを調整する
	if ( _scenarioLog.size() > 80000 ) {
		
		printf ( "Log Size Adjust : %d\n", _scenarioLog.size() );
		
		string newLog = "";
		
		vector<string> rows = Utils_String::Split ( _scenarioLog, "\n" );
		
		int startIndex = rows.size() - 1000;
		if ( startIndex < 0 ) { startIndex = 0; }
		
		printf ( "startIndex : %d\n", startIndex );
		
		for ( unsigned int i = startIndex; i < rows.size(); ++i ) {
			
			newLog += rows[i] + "\n";
			
		}
		
		_scenarioLog = newLog;
		
		printf ( "newLogSize : %d\n", newLog.size() );
		
	}
	
	
	/*
	std::ofstream ofs;
	ofs.open ( "textLog.txt", std::ios::app );
	ofs << log << std::endl;

	ofs.close();
	*/
}
//=========================================================
// 読み返しログ取得
//=========================================================
string ScriptEngine::ReadScenarioLog () {

	string result = "";

	result = _scenarioLog;

	return result;

}

//=========================================================
// 読み返しログクリア
//=========================================================
void ScriptEngine::ClearScenarioLog() {

	_scenarioLog = "";
	/*
	std::ofstream ofs;
	ofs.open ( "textLog.txt", std::ios::out );
	ofs.close();
	*/

}

//=========================================================
// システムメッセージ
// TODO これは Application側に置くべきか？
//=========================================================
void ScriptEngine::SystemMessage ( string message ) {

	Effect_SystemMessage *effect = new Effect_SystemMessage ( message, TextImageFactory::_font );

	Application::GetInstance()->_effectManager->Add(effect);

}













